// Limiter.cpp: implementation of the Limiter class.
//
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include "AudioSample.h"
#include "AudioSampleManager.h"
#include "DSP/VolumeLimiter.h"
#include "Limiter.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Limiter::Limiter(double threshold = -8.0, double lossIncrement = 0.075, double lossDecrement = -0.00075)
{
	char subFacilityName[100];
	sprintf(subFacilityName, "Limiter:%x", this);
	tracer.SetSubFacilityName(subFacilityName);
	SetTraceLevel();
	volumeLimiter = new VolumeLimiter(threshold, lossIncrement, lossDecrement);
}

Limiter::~Limiter()
{
	if (volumeLimiter)
	{
		delete volumeLimiter;
	}
}

int
Limiter::SetTraceLevel()
{
#ifdef WIN32
	long SystemMask = 0;
	if ((SystemMask = GetRegKeyLong(HKEY_CURRENT_USER, "Software\\Cisco Systems\\MTC\\Tracing", "AllComponents", 0x0)) == 0)
	{
		SystemMask = GetRegKeyLong(HKEY_CURRENT_USER, "Software\\Cisco Systems\\MTC\\Tracing", "Limiter", 0x100000);
	}
#endif
	return 0;
}

int
Limiter::SetParameters(double threshold, double lossIncrement, double lossDecrement)
{
	if (volumeLimiter)
	{
		delete volumeLimiter;
		volumeLimiter = NULL;
	}
	volumeLimiter = new VolumeLimiter(threshold, lossIncrement, lossDecrement);
	return 0;
}

int
Limiter::TransformAudioSamples(std::vector<std::pair<AudioSample *, AudioSource *> > &data, AudioSample **ppAudioSample)
{
	AudioSample *inSample = data[0].first;
	int result = 0;
	*ppAudioSample = inSample;
	inSample->AddRef();
	WAVEFORMATEX sampleFormat;
	inSample->GetFormat(&sampleFormat);
	int numSamples = (inSample->DataSize() * 8) / sampleFormat.wBitsPerSample;
	volumeLimiter->LimitVolume((short *)(inSample->Data()), numSamples);	
	return result;
}
