// PCM2G723Transformer.cpp: implementation of the PCM2G723Transformer class.
//
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include "PCM2G723Transformer.h"
#include "AudioSampleManager.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PCM2G723Transformer::PCM2G723Transformer()
{
	char subFacilityName[100];
	sprintf(subFacilityName, "PCM2G723Transformer:%x", this);
	tracer.SetSubFacilityName(subFacilityName);
	SetTraceLevel();
	SetRate63();
}

PCM2G723Transformer::~PCM2G723Transformer()
{

}

int
PCM2G723Transformer::SetTraceLevel()
{
#ifdef WIN32
	long SystemMask = 0;
	if ((SystemMask = GetRegKeyLong(HKEY_CURRENT_USER, "Software\\Cisco Systems\\MTC\\Tracing", "AllComponents", 0x0)) == 0)
	{
		SystemMask = GetRegKeyLong(HKEY_CURRENT_USER, "Software\\Cisco Systems\\MTC\\Tracing", "PCM2G723Transformer", 0x100000);
	}
#endif
	return 0;
}

int
PCM2G723Transformer::SetRate53()
{
	return g723Codec.SetRate53();
}

int
PCM2G723Transformer::SetRate63()
{
	return g723Codec.SetRate63();
}

int
PCM2G723Transformer::TransformAudioSamples(std::vector<std::pair<AudioSample *, AudioSource *> > &data, AudioSample **ppAudioSample)
{
	AudioSample *inSample = data[0].first;
	AudioSample *outSample;
	int result = 0;
	result = AudioSampleManager::GetInstance()->GetAudioSample(&outSample);

	if (outSample)
	{
		*ppAudioSample = outSample;
		if (g723Codec.GetCodecRate() == G723RATE_53)
		{
                        WAVEFORMATEX wfmx = WaveFormat::GetWaveFormat(WaveFormat_G723_53);
			outSample->SetFormat(wfmx);
		}
		else
		{
                        WAVEFORMATEX wfmx = WaveFormat::GetWaveFormat(WaveFormat_G723_63);
			outSample->SetFormat(wfmx);
		}

		if (inSample->DataSize() <= 0)
		{
			outSample->SetSilenceDuration(inSample->GetSilenceDuration());
		}
		else
		{
			int outputSize = outSample->BufferSize();
			result = g723Codec.Encode(inSample->Data(), inSample->DataSize(), outSample->Data(), &outputSize);
			outSample->SetDataSize(outputSize);
		}
	}
	else
	{
		result = -10;
	}
	return result;
}
