// PCMVolumeTransformer.cpp: implementation of the PCMVolumeTransformer class.
//
//////////////////////////////////////////////////////////////////////

#include <math.h>
#include <stdio.h>

#include "AudioSample.h"
#include "AudioSampleManager.h"
#include "PCMVolumeTransformer.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PCMVolumeTransformer::PCMVolumeTransformer()
{
	char subFacilityName[100];
	sprintf(subFacilityName, "PCMVolumeTransformer:%x", this);
	tracer.SetSubFacilityName(subFacilityName);
	SetTraceLevel();
	volume = 0;
}

PCMVolumeTransformer::~PCMVolumeTransformer()
{

}

int
PCMVolumeTransformer::SetTraceLevel()
{
#ifdef WIN32
	long SystemMask = 0;
	if ((SystemMask = GetRegKeyLong(HKEY_CURRENT_USER, "Software\\Cisco Systems\\MTC\\Tracing", "AllComponents", 0x0)) == 0)
	{
		SystemMask = GetRegKeyLong(HKEY_CURRENT_USER, "Software\\Cisco Systems\\MTC\\Tracing", "PCMVolumeTransformer", 0x100000);
	}
#endif
	return 0;
}

int
PCMVolumeTransformer::SetVolume(int newVolume)
{
	newVolume = min(100, newVolume);
	newVolume = max(0, newVolume);
	volume = (unsigned int)newVolume;
	return 0;
}

int
PCMVolumeTransformer::GetVolume()
{
	return volume;
}

int
PCMVolumeTransformer::TransformAudioSamples(std::vector<std::pair<AudioSample *, AudioSource *> > &data, AudioSample **ppAudioSample)
{
	AudioSample *inSample = data[0].first;
	int result = 0;
	*ppAudioSample = inSample;
	inSample->AddRef();
	int numSamples = inSample->DataSize() / 2;
	short *dataIn = (short *)(inSample->Data());
	for (int i=0; i<numSamples; i++)
	{
		double data = (double)dataIn[i];
		short newValue = (short)(data * ((volume * volume) / 10000.0));
		dataIn[i] = newValue;
	}
	return result;
}
