
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const DigitMap_cxx_Version =
    "$Id: DigitMap.cxx,v 1.18 2001/01/23 22:22:34 cktam Exp $";
#include "MgAgent.hxx"
#include "DigitMap.h"



DigitMap::DigitMap()
{
    int i = 0;

    // Clear the pointers and pattern buffers.
    for (i = 0; i < MAX_PATTERNS; i++)
    {
        myPattern[i] = '\0';
    };

    for (i = 0; i < MAX_PATTERN_ARRAY; i++)
    {
        myPatternArray[i] = '\0';
    };

    // Clear the digit buffer.
    clearBuffer();

    // default case when we are not given a digit map.
    myDigitMap = "[0-7][0-9][0-9][0-9]";
    setDigitMap(myDigitMap);
}



void
DigitMap::clearBuffer()
{
    myDigitState = false;

    for (int i = 0; i < MAX_DIGIT_BUFFER; i++)
    {
        myDigitArry[i] = '\0';
    }
}


void
DigitMap::sendDigits(MgAgent* myAgent,
                     ReqEv* myDtmfReqEv[],
                     string myId)
{
    ReqEv* reqEv = NULL;
    int digitIndex = 0;

    if (myDigitArry[0] != '\0')
    {
        cpLog(LOG_DEBUG, "Sending digits: %s", myDigitArry);

        // Initialize ntfy with the first digit event.

        reqEv = myDtmfReqEv[ digitToEvent(myDigitArry[digitIndex]) ];

        MgNotify ntfy(myId, reqEv->getRequestId(),
                      reqEv->getObservedEvent());

        digitIndex++;

        // After initializing ntfy with the first digit event, use
        // ntfy.addObservedEvents to add more digit events before sending
        // to the call agent.

        while (myDigitArry[digitIndex] != '\0')
        {
            reqEv = myDtmfReqEv [ digitToEvent(myDigitArry[digitIndex]) ];

            ntfy.addObservedEvents(reqEv->getObservedEvent());

            digitIndex++;
        };

        myAgent->send(ntfy);

        // clear the buffer after we send the digits to the call agent.
        clearBuffer();
    }
}



int
DigitMap::matchStr (char *string,
                    char *pattern,
                    regex_t *re)
{
    int status;

    if ((status = regcomp( re, pattern, REG_EXTENDED)) != 0)
        return (status);
    status = regexec( re, string, 0, NULL, 0);
    return (status);
};



int
DigitMap::countDigits(char* pattern)
{
    int i = 0;
    int num = 0;

    while (pattern[i] != '\0')
    {
        if (pattern[i] == '[' )
        {
            while (pattern[i] != ']')
            {
                i ++;
            };
        }
        num++;
        i++;
    }
    return (num);
};



DtmfReqEv
DigitMap::digitToEvent(char digit)
{
    DtmfReqEv event;

    switch (digit)
    {
        case '0':
        {
            event = DtmfReqEv0;
        }
        break;

        case '1':
        {
            event = DtmfReqEv1;
        }
        break; 

        case '2':
        {
            event = DtmfReqEv2;
        }
        break;

        case '3':
        {
            event = DtmfReqEv3;
        }
        break;

        case '4':
        {
            event = DtmfReqEv4;
        }
        break;

        case '5':
        {
            event = DtmfReqEv5;
        }
        break;

        case '6':
        {
            event = DtmfReqEv6;
        }
        break;

        case '7':
        {
            event = DtmfReqEv7;
        }
        break;

        case '8':
        {
            event = DtmfReqEv8;
        }
        break;

        case '9':
        {
            event = DtmfReqEv9;
        }
        break;

        case '*':
        {
            event = DtmfReqEvStar;
        }
        break;

        case '#':
        {
            event = DtmfReqEvHash;
        }
        break;

        case 'T':
        {
            event = DtmfReqEvInterdigitTmr;
        }
        break;

        default:
        {
            cpLog(LOG_ERR, "Unrecognized digit");
            event = DtmfReqEvInvalidEvent;
        }
        break;
    }

    return event;
}




void
DigitMap::setDigitMap(string digitMap)
{
    const char* digitMapPtr;
    int i = 0;
    int z = 0;
    int offset = 0;

    // store the digitMap from the call agent.
    myDigitMap = digitMap;


    // convert from string to char *
    digitMapPtr = myDigitMap.c_str();


    // initialize both the array and the pointers for storing the digit map
    // patterns.
    for (i = 0; i < MAX_PATTERN_ARRAY; i ++)
    {
        myPatternArray[i] = '\0';
    }

    for (i = 0; i < MAX_PATTERNS; i++)
    {
        myPattern[i] = '\0';
    };


    // initialize the 1st pointer to the 1st entry in the array.
    myPattern[0] = &myPatternArray[0];

    for (i = 0; *digitMapPtr != '\0'; digitMapPtr++, i++)
    {
        if (*digitMapPtr != '|')
        {
            switch (*digitMapPtr)
            {
                case 'x':
                {
                    myPatternArray[i + offset] = '[';

                    offset++;
                    myPatternArray[i + offset] = '0';

                    offset++;
                    myPatternArray[i + offset] = '-';

                    offset++;
                    myPatternArray[i + offset] = '9';

                    offset++;
                    myPatternArray[i + offset] = ']';
                }
                break;
                default:
                {
                    myPatternArray[i + offset] = *digitMapPtr;
                }
                break;
            };
        }
        else
        {
            // If we find an '|' then we are looking at a new pattern.
            // Increment to the next pointer to a new position in the
            // pattern array to store the next pattern.
            z++;
            myPattern[z] = &myPatternArray[i + offset + 1];
        }
    }


    // for debugging... print out the stored pattern.

    for (i = 0; i < MAX_PATTERNS; i++)
    {
        if (myPattern[i])
        {
            cout << "\n\n" << i << ":  " << myPattern[i] << endl;
        }
    };
};



string
DigitMap::returnDigitMap()
                {
                    return myDigitMap;
                };



void
DigitMap::handleDigitTimer(MgAgent* myAgent,
                           ReqEv* myDtmfReqEv[],
                           string myId)
{
    int i = 0;

    // Find where we left off.

    while (myDigitArry[i] != '\0')
    {
        i++;
    }


    // TODO  Need to finish and test.   -ctam
    // Store the Timer character to indicate that we got a timeout.
    // myDigitArry[i] = 'T';

    // Okay, we go the timeout so what do we do now??
    // Send what we have so far to the call agent.

    sendDigits(myAgent,
               myDtmfReqEv,
               myId);

};





bool
DigitMap::collect(HardwareEvent DigitEvent,
                  char DigitArry[MAX_DIGIT_BUFFER],
                  bool DigitState)
{
    int i = 0;
    int j = 0;
    int retval;
    regex_t re;
    int num_of_digits = 0;
    int num_check = 0;


    // Clear the array buffer to begin storing the digits.

    if (DigitState == false)
    {
        for (i = 0; i < MAX_DIGIT_BUFFER; i++)
        {
            DigitArry[i] = '\0';
        }
        i = 0;
    }


    // Find the position where we left off

    while (DigitArry[i] != '\0')
    {
        i++;
    };

    // change the digit state to true
    // We are storing the digits until it matches to a pattern.

    DigitState = true;


    // store the digit w.r.t the digit event

    switch (DigitEvent)
    {
        case EventDigit0:
        DigitArry[i] = '0';
        break;
        case EventDigit1:
        DigitArry[i] = '1';
        break;
        case EventDigit2:
        DigitArry[i] = '2';
        break;
        case EventDigit3:
        DigitArry[i] = '3';
        break;
        case EventDigit4:
        DigitArry[i] = '4';
        break;
        case EventDigit5:
        DigitArry[i] = '5';
        break;
        case EventDigit6:
        DigitArry[i] = '6';
        break;
        case EventDigit7:
        DigitArry[i] = '7';
        break;
        case EventDigit8:
        DigitArry[i] = '8';
        break;
        case EventDigit9:
        DigitArry[i] = '9';
        break;
        case EventDigitStar:
        DigitArry[i] = '*';
        break;
        case EventDigitHash:
        DigitArry[i] = '#';
        break;
        default:
        cpLog (LOG_ERR, "collect:  Non digit response, %d", DigitEvent);
        break;
    }


    // num_check is the largest number of digits in all the patterns
    // of the digit map.
    // It is used to determine if there are no more possible matches
    // between the patterns in the digit map and the user input.

    while (myPattern[j])
    {
        if (num_check < countDigits(myPattern[j]))
            num_check = countDigits(myPattern[j]);
        j++;
    };

    // match the digit array with the given patterns from the digit map

    j = 0;


    while (myPattern[j])
    {
        cerr << "Using pattern: " << myPattern[j] << endl;

        num_of_digits = countDigits(myPattern[j]);
        retval = matchStr(DigitArry, myPattern[j], &re);

        if (retval == 0)
        {
            if (num_of_digits == i + 1)
            {
                cpLog (LOG_DEBUG, "Digit Map Match found: ");
                cpLog (LOG_DEBUG, "     Pattern    = %s ", myPattern[j]);
                cpLog (LOG_DEBUG, "     DigitArray = %s", DigitArry);

                DigitState = false;
            }
        }
        else
        {

            if (num_check == i + 1)
            {
                cpLog (LOG_DEBUG, "Cannot match to any patterns");
                return false;
            }
        }

        j++;
    }

    regfree( &re);
    return (DigitState);
};



bool
DigitMap::handledigits(MgAgent* myAgent,
                       ReqEv* myDtmfReqEv[],
                       string myId,
                       HardwareEvent DigitEvent)

{
    // collect will set DigitState to true until it has found a match.
    // After finding a match, collect will change DigitState to false
    //     indicating that it has found a match and is finished collecting
    //     digits.

    myDigitState = collect(DigitEvent, myDigitArry, myDigitState);


    // If DigitState is false, then we must either have a positive match or
    // there is no possible matches left.

    if (myDigitState == false)
    {
        // Make sure that we have digits stored in the array.

        if (myDigitArry[0] != '\0')
        {
            sendDigits(myAgent,
                       myDtmfReqEv,
                       myId);
        }
    }

    return myDigitState;
};
