#ifndef SH3_HARDWARE_H_
#define SH3_HARDWARE_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const SH3Hardware_h_Version =
    "$Id: SH3Hardware.h,v 1.13 2001/01/23 22:22:34 cktam Exp $";

#include "RtpSession.hxx"
#include "Hardware.h"
#include "Gateway.h"

enum SH3DeviceMsgType
{
    SH3NULL,
    SH3DigitType,
};


typedef struct
{
    SH3DeviceMsgType type;
    int channelNum;
    int digit;
}
SH3DeviceMsg;



/**********************************************************************
Class: SH3Hardware
Derived class of Hardware base class
Provides interface for Hitachi's SH3 board using Hitachi's 
 
SH3-DSP Telephony Library (Version 0.3) 
 
and 
 
SH3-DSP G.729A Library (Version 1.0)
 
**********************************************************************/
class SH3Hardware: public Hardware
{
    public:
        /** Creates a hardware object to control a multi-port gateway.
         */
        SH3Hardware ( const char* deviceName, 
                      int channelNum,
                      Fifo < HardwareMsg > * inputQ,
                      Fifo < EndpointMsg > ** outputQ );

        /** Destructor
         */
        ~SH3Hardware (void);

        /** Starts the main processing loop of the hardware
         */
        void*
        hardwareMain (void* parms);

        /** Add device's socket to the fd; returns 0 if successful,
         ** errorcode otherwise 
         */
        int addToFdSet (fd_set* fd);

        /** Process all pending hardware events; returns 0 if successful,
         ** otherwise returns an errorcode.  
         */
        int
        process (fd_set* fd);

        /** Start audio channel on device; returns 0 if successful,
         ** errorcode otherwise 
         */
        int audioStart (const HardwareAudioRequest& request,
                        const int channelNum);

        /** Stops audio channel on device; returns 0 if successful,
         ** errorcode otherwise.
         */
        int
        audioStop (const int channelNum);

        /** Returns the HardwareSignal supported by the hardware which
         ** corresponds to turning on the mgcp encoded signal.
         */
        virtual HardwareSignal
        getSignalOn (MgcpPackageType package, int index) const;

        /** Returns the HardwareSignal supported by the hardware which
         ** corresponds to turning off the mgcp encoded signal.
         */
        virtual HardwareSignal
        getSignalOff (MgcpPackageType package, int index) const;


        void
        deviceMain();

        void
        processDeviceMsg(const SH3DeviceMsg& deviceMsg);

    private:
        /** Trigger the device to send the specified signal; returns 0 if
         ** successful, errorcode otherwise.
         */
        int
        sendSignal (HardwareSignal signal, int channelNum);

        /// provide tone using the hitachi interface
        void
        provideTone (int tone, int channelNum);

        /// converts the dtmfDigit to the corresponding digit event and reports
        /// the event to the endpoint (calls reportEvent).
        int
        handleDigitEvent(int dtmfDigit, int channelNum);

        /// output queue to notify endpoint of occurence of hardware events
        Fifo < EndpointMsg > * endpointQ[GW_ENDPOINT_MAX];

        /// device queue to get messages from the device thread (dtmf events).
        Fifo < SH3DeviceMsg > * deviceQ;


        /// process RTP audio packets
        void processRTP(int channelNum);
        void processInPkt(char* inBuffer, int channelNum);
        void processOutPkt(char* outBuffer, int channelNum);

};

#endif // SH3_HARDWARE_H_
