
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const tftpCfg_cxx_Version =
    "$Id: tftpCfg.cxx,v 1.6 2001/01/23 22:22:34 cktam Exp $";
#ifdef __vxworks

#include "Gateway.h"
#include "tftpCfg.h"

extern int gwNum;
extern int boardNum;
extern char* callAgentHostName;
extern GatewayConfigData configData;



const char* MYHOSTNAME = "thisismyhostname";

static STATUS parseConfigData(char *pBuf, int bufLen)
{
    int i = 0;
    char tag[256], type[256], value[256];
    int cnttag = 0;
    int index = 0;

    sscanf( pBuf, "%s %s %s", tag, type, value);

    boardNum = atoi(value);
    cnttag++;
    if (boardNum == 82)
        gwNum = 1;
    else if (boardNum == 83)
        gwNum = 0;

    while (i < bufLen)
    {
        if (*pBuf == '\r' || *pBuf == '\n')
        {
            sscanf( pBuf, "%s %s %s", tag, type, value);

            if (cnttag >= 9)
                break;

            switch (cnttag)
            {
                case enumBoard:
                break;
                case enumDeviceName:
                configData.deviceName = value;
                break;
                case enumGatewayPort:
                configData.gatewayPort = atoi(value);
                break;
                case enumAgentHostname:
                configData.agentHostname = value;
                break;
                case enumAgentPort:
                configData.agentPort = atoi(value);
                break;
                case enumNumEndpoints:
                configData.numEndpoints = atoi(value);
                break;
                case enumEndpointId:
                configData.endpointId[index] = value;
                break;
                case enumCntsPerEndpt:
                configData.cntsPerEndpt = atoi(value);
                break;
                case enumFirstRtpPort:
                configData.firstRtpPort = atoi(value);
                break;
                default:
                printf("UNKNOWN DATA!!!\n");
                break;
            }
            cnttag++;
        }
        i++;
        pBuf++;
    }

    return OK;
}


static void tftpConfigInit(void)
{
}

static STATUS tftpConfigFile(char *tftpAddr, char *configFileName)
{
    int fd;
    int errFd;           /* for receiving standard error messages from Unix */
    char buf[1024];
    int retVal = OK;
    int num;

    if ( NULL == tftpAddr || NULL == configFileName )
        return ( ERROR );

    printf("hostname = %s\n", tftpAddr);
    printf("filename= %s\n", configFileName);

    if ( ERROR == tftpXfer(tftpAddr, 0, configFileName, "get", "ascii",
                           &fd, &errFd) )
    {
        printf("Error: can not open the file\n");
        return ( ERROR );
    }



    // downloading the data
    while ((num = read (fd, buf, sizeof (buf))) > 0)
    {
        printf ("Parse Config Data here!\n");
        parseConfigData(buf, num);
    }
    close (fd);

    if (read (errFd, buf, sizeof (buf)) > 0)
    {
        printf ("YIKES! An error occurred!:%s\n", buf);
        retVal = ERROR;
    }
    close (errFd);

    return retVal;
}

void tftpCfg(void)
{
    char hostName [] = MYHOSTNAME;
    char fileName[] = "gw.cfg";

    printf("begin tftp......\n");

    tftpConfigInit();
    tftpConfigFile(hostName, fileName);

    printf("end tftp......\n");
}

#endif 


