
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const TestHardware_cxx_Version =
    "$Id: TestHardware.cxx,v 1.8 2000/12/18 23:42:25 bko Exp $";
/* stdlib */
#include <iostream.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include "TestHardware.h"
#include "ReqSig.h"


extern int numOfBadStates;

/** The following array provides mappings from MGCP encoded signals to
 ** the corresponding TestHardware signal.
 */
HardwareSignal TestHardwareGenericSigOn[GenericReqSigMaxSignal] =
    {
        SignalNULL,  //GenericSigPatternDetected,
        SignalLocalRingbackStart,  //GenericSigRingbackTone,
        SignalNULL,  //GenericSigRingbackOnConnection,
        SignalNULL,  //GenericSigConfirmTone,
        SignalNULL,  //GenericSigNetworkCongestionTone,
        SignalNULL,  //GenericSigInterceptTone,
        SignalNULL,  //GenericSigPreemptionTone,
    };

HardwareSignal TestHardwareGenericSigOff[GenericReqSigMaxSignal] =
    {
        SignalNULL,  //GenericSigPatternDetected,
        SignalLocalRingbackStop,  //GenericSigRingbackTone,
        SignalNULL,  //GenericSigRingbackOnConnection,
        SignalNULL,  //GenericSigConfirmTone,
        SignalNULL,  //GenericSigNetworkCongestionTone,
        SignalNULL,  //GenericSigInterceptTone,
        SignalNULL,  //GenericSigPreemptionTone,
    };

HardwareSignal TestHardwareLineSigOn[LineReqSigMaxSignal] =
    {
        SignalNULL,  // LineSigAdsiDisplay,
        SignalNULL,  // LineSigVisualMsg,
        SignalNULL,  // LineSigAnswerTone,
        SignalBusyStart,  // LineSigBusyTone,
        SignalNULL,  // LineSigCallerId,
        SignalNULL,  // LineSigCallWaitTone,
        SignalDialToneStart,  // LineSigDialTone,
        SignalNULL,  // LineSigMsgWaitInd,
        SignalNULL,  // LineSigNetworkBusy,
        SignalRingStart,  // LineSigRinging,
        SignalNULL,  // LineSigDsnctRing0,
        SignalNULL,  // LineSigDsnctRing1,
        SignalNULL,  // LineSigDsnctRing2,
        SignalNULL,  // LineSigDsnctRing3,
        SignalNULL,  // LineSigDsnctRing4,
        SignalNULL,  // LineSigDsnctRing5,
        SignalNULL,  // LineSigDsnctRing6,
        SignalNULL,  // LineSigDsnctRing7,
        SignalNULL,  // LineSigRingSplash,
        SignalNULL,  // LineSigPromptTone,
        SignalNULL,  // LineSigErrorTone,
        SignalNULL,  // LineSigStutterTone,
        SignalNULL,  // LineSigAlertTone,
        SignalNULL,  // LineSigRecordWarnTone,
        SignalNULL,  // LineSigCallCardSrvTone,
        SignalNULL,  // LineSigOffhookWarnTone,
        SignalNULL,  // LineSigDsnctTonePattern,
    };

HardwareSignal TestHardwareLineSigOff[LineReqSigMaxSignal] =
    {
        SignalNULL,  // LineSigAdsiDisplay,
        SignalNULL,  // LineSigVisualMsg,
        SignalNULL,  // LineSigAnswerTone,
        SignalBusyStop,  // LineSigBusyTone,
        SignalNULL,  // LineSigCallerId,
        SignalNULL,  // LineSigCallWaitTone,
        SignalDialToneStop,  // LineSigDialTone,
        SignalNULL,  // LineSigMsgWaitInd,
        SignalNULL,  // LineSigNetworkBusy,
        SignalRingStop,  // LineSigRinging,
        SignalNULL,  // LineSigDsnctRing0,
        SignalNULL,  // LineSigDsnctRing1,
        SignalNULL,  // LineSigDsnctRing2,
        SignalNULL,  // LineSigDsnctRing3,
        SignalNULL,  // LineSigDsnctRing4,
        SignalNULL,  // LineSigDsnctRing5,
        SignalNULL,  // LineSigDsnctRing6,
        SignalNULL,  // LineSigDsnctRing7,
        SignalNULL,  // LineSigRingSplash,
        SignalNULL,  // LineSigPromptTone,
        SignalNULL,  // LineSigErrorTone,
        SignalNULL,  // LineSigStutterTone,
        SignalNULL,  // LineSigAlertTone,
        SignalNULL,  // LineSigRecordWarnTone,
        SignalNULL,  // LineSigCallCardSrvTone,
        SignalNULL,  // LineSigOffhookWarnTone,
        SignalNULL,  // LineSigDsnctTonePattern,
    };

HardwareSignal TestHardwareDtmfSigOn[DtmfReqSigMaxSignal] =
    {
        SignalNULL,  //DtmfSig0,
        SignalNULL,  //DtmfSig1,
        SignalNULL,  //DtmfSig2,
        SignalNULL,  //DtmfSig3,
        SignalNULL,  //DtmfSig4,
        SignalNULL,  //DtmfSig5,
        SignalNULL,  //DtmfSig6,
        SignalNULL,  //DtmfSig7,
        SignalNULL,  //DtmfSig8,
        SignalNULL,  //DtmfSig9,
        SignalNULL,  //DtmfSigHash,
        SignalNULL,  //DtmfSigStar,
        SignalNULL,  //DtmfSigA,
        SignalNULL,  //DtmfSigB,
        SignalNULL,  //DtmfSigC,
        SignalNULL,  //DtmfSigD
    };

HardwareSignal TestHardwareDtmfSigOff[DtmfReqSigMaxSignal] =
    {
        SignalNULL,  //DtmfSig0,
        SignalNULL,  //DtmfSig1,
        SignalNULL,  //DtmfSig2,
        SignalNULL,  //DtmfSig3,
        SignalNULL,  //DtmfSig4,
        SignalNULL,  //DtmfSig5,
        SignalNULL,  //DtmfSig6,
        SignalNULL,  //DtmfSig7,
        SignalNULL,  //DtmfSig8,
        SignalNULL,  //DtmfSig9,
        SignalNULL,  //DtmfSigHash,
        SignalNULL,  //DtmfSigStar,
        SignalNULL,  //DtmfSigA,
        SignalNULL,  //DtmfSigB,
        SignalNULL,  //DtmfSigC,
        SignalNULL,  //DtmfSigD
    };


TestHardware::TestHardware (char* deviceName,
                            int endpointIndex,
                            Fifo < HardwareMsg > * inputQ,
                            Fifo < EndpointMsg > * outputQ,
                            string runmode,
                            string dialDigits,
                            int callDuration,
                            int callDelay,
                            long int answerDelay)
{
    myFD = 0;

    myEndpointIndex = endpointIndex;
    myEndpointQ[0] = outputQ;
    myQ = inputQ;

    myTestCallState = TestNull;

    // *******************************************************************
    BAD_STATE = false;
    testMarker = false;
    for (int i = 0; i < HWTimeoutMaxTimer; i++)
    {
        myTestTimerId[i] = (FifoEventId) NULL;
    }

    myDialDigits = dialDigits.c_str();
    digitPosition = 0;

    // delay constants
    myCallDuration = callDuration;   // in msec
    myDigitDelay = 100;            // in msec
    myCallDelay = callDelay;      // in msec
    myAnswerDelay = answerDelay;


    // Set the run mode to be either calling or receiving
    if (runmode == "CallingGW")
    {
        cpLog (LOG_DEBUG, "Run Mode:  CallingGW");
        myTestType = CallingGW;
    }
    else
    {
        if (runmode == "ReceiveGW")
        {
            cpLog (LOG_DEBUG, "Run Mode:  ReceiveGW");
            myTestType = ReceiveGW;
        }
    }

    // *******************************************************************

    // read and write from standard in and standard out
    //    system("stty -icanon");

}  // end TestHardware::TestHardware()



TestHardware::~TestHardware (void)
{
    // stop reading and writing from standard in and standard out
    //   system("stty icanon");

} // end TestHardware::~TestHardware()



void*
TestHardware::hardwareMain (void* parms)
{
    // process forever on behalf of null hardware
    fd_set readfds;
    struct timeval tv;
    bool operational = true;
    int maxFd = 32;
    int retval = 0;

    int dummy = 0;

    while (operational)
    {
        // reset file descriptor
        FD_ZERO(&readfds);
        addToFdSet(&readfds);
        //if (maxFd <= gateway.myAgent->getFd())
        //maxFd =  gateway.myAgent->getFd() + 1;

        // block on select but timeout needed to service myQ
        tv.tv_sec = 0;
        tv.tv_usec = 300;

        if ((retval = select(maxFd, &readfds, NULL, NULL, &tv)) < 0 && dummy == 1)
        {
            cpLog(LOG_ERR, "Select returned with an error - ignore");
        }
        else // if (retval > 0)
        {

            // hardware input received
            if (process(&readfds) < 0)
            {
                cpLog(LOG_ERR, "Hardware failed - shutting down");
                operational = false;
            }

        }

        // process msgs from endpoints (if any)
        while (myQ->size() > 0)
        {
            processEndpointMsg(myQ->getNext());
        }
    } // end while


    return NULL;

} // end TestHardware::hardwareMain()



// ***********************************************************************
//  process
//
//
// ***********************************************************************

int
TestHardware::process(fd_set* fd)
{
    HardwareEvent event = EventNULL;
    TestCallState callState = TestInCall;
    /*
        if (testMarker == false && myTestType == CallingGW)
        {  
            cpLog (LOG_DEBUG, "starting timer for the first time" ); 
            stopTimer(HWTimeoutStartCall);
            startTimer(HWTimeoutStartCall, (5000 + (300 * myEndpointIndex)));
            testMarker = true;
        }
    */

    callState = getTestCallState();

    switch (callState)
    {
        case TestInCall:
        {
            // Do nothing.

        }

        break;

        case TestBadState:
        {
            /*
                        // Should not be here!  But since we are, go ahead and hang up.
                        // Hope that this is will solve the problem and pretend life is
                        // good. 
             
                        cpLog(LOG_ERR, "***  epIndex %d  BAD STATE!  hu, 10+ sec, make call  ***", 
                              myEndpointIndex);
             
                        stopTimer(HWTimeoutEndCall);
                        startTimer(HWTimeoutEndCall, 10);
                        BAD_STATE = true;
            //            numOfBadStates ++;
                        myTestCallState = TestInCall;
            */
        }
        break;

        case TestOffhook:
        {
            // offhook
            event = EventOffhook;
            hookStateOffhook = true;

            cpLog (LOG_DEBUG, "*****   TestOffhook    *****");

            if (myTestType == CallingGW)
            {
                cpLog(LOG_DEBUG, "Endpoint: %d,   Making a call!", myEndpointIndex);
                //                stopTimer(HWTimeoutDialDigits);
                //                startTimer(HWTimeoutDialDigits, myDigitDelay);

                myTestCallState = TestInCall;
            }
            else
            {
                myTestCallState = TestNull;
            }

        }
        break;

        case TestOnhook:
        {
            // onhook
            event = EventOnhook;
            hookStateOffhook = false;

            cpLog (LOG_DEBUG, "*****   TestOnhook    *****");
            /*
                        if (myTestType == CallingGW)
                        {
                            cpLog(LOG_DEBUG, "Endoint: %d,   Hanging up!",  myEndpointIndex);
                            if (BAD_STATE == true)
                            {
                                // Make life good again 
                                BAD_STATE = false;
             
                                // start the calls again.  Hope that someone likes us.
                                // If not, then we will be in a bad state again.
                                cpLog(LOG_DEBUG, "*********    BAD STATE!     BAD STATE!    *********");
                                cpLog(LOG_DEBUG, "********* Trying to make life good again! *********");
                                cpLog(LOG_DEBUG, "********* Try to make a call in 10 sec    *********");
                                cpLog(LOG_DEBUG, "*********    BAD STATE!     BAD STATE!    *********");
             
                                stopTimer(HWTimeoutStartCall);
                                startTimer(HWTimeoutStartCall, 10000+(200* myEndpointIndex));
                            };
             
                            myTestCallState = TestInCall;
                        }else
                        {
                            myTestCallState = TestNull;
                        };
            */
        }
        break;

        case TestSendDigits:
        {
            /*
                        if (hookStateOffhook)
                        {
                            event = EventDigitTest;
                            myTestCallState = TestInCall;
             
                            // start a second timer in case we don't get a reply
                            stopTimer(HWTimeoutBadState);
                            startTimer(HWTimeoutBadState, 10000);
                        }
                        else
                        {
                            myTestCallState = TestSendDigits;
                            cpLog(LOG_ERR, "Not offhook!  Cannot Send Digits.");
                        }
            */
        }
        break;

        case TestFlashhook:
        {
            // flashhook
            if (hookStateOffhook)
            {
                //event = EventFlashhook;
            }

            else
            {
                cpLog(LOG_ERR, "Not offhook - flashhook ignored");
            }
        }
        break;

        case TestDigit1:
        {
            event = EventDigit1;
            myTestCallState = TestInCall;
        }
        break;

        case TestDigit2:
        {
            event = EventDigit2;
            myTestCallState = TestInCall;

        }
        break;

        case TestDigit3:
        {
            event = EventDigit3;
            myTestCallState = TestInCall;
        }
        break;

        case TestDigit4:
        {
            event = EventDigit4;
            myTestCallState = TestInCall;
        }
        break;

        case TestDigit5:
        {
            event = EventDigit5;
            myTestCallState = TestInCall;
        }
        break;

        case TestDigit6:
        {
            event = EventDigit6;
            myTestCallState = TestInCall;
        }
        break;

        case TestDigit7:
        {
            event = EventDigit7;
            myTestCallState = TestInCall;
        }
        break;

        case TestDigit8:
        {
            event = EventDigit8;
            myTestCallState = TestInCall;
        }
        break;

        case TestDigit9:
        {
            event = EventDigit9;
            myTestCallState = TestInCall;
        }
        break;

        case TestDigit0:
        {
            event = EventDigit0;
            myTestCallState = TestInCall;
        }
        break;

        case TestEndDialing:
        {
            myTestCallState = TestInCall;

        }
        break;

        case TestNull:
        {
            // ignore
        }

        break;

        default:
        {
            // no events
            cpLog(LOG_ERR, "Unknown input = %d", callState);
            break;
        }
    } // end switch

    if (event != EventNULL)
    {
        reportEvent(myEndpointQ[0], event);
    }

    return 0;
}


int
TestHardware::addToFdSet (fd_set* fd)
{
    FD_SET(myFD, fd);
    return 0;
} // end TestHardware::addToFdSet()


int
TestHardware::sendSignal (HardwareSignal signal)
{
    int retval = 0;

    switch (signal)
    {
        case SignalDialToneStart:
        {
            cpLog(LOG_DEBUG, "#####   Start dialtone   #####\n");
        }
        break;

        case SignalDialToneStop:
        {
            cpLog(LOG_DEBUG, "#####   Stop dialtone   #####\n");
        }
        break;

        case SignalRingStart:
        {
            cpLog(LOG_DEBUG, "#####   Start ringing   #####\n");

            if (myTestType == ReceiveGW)
            {
                // Answer the call
                startTimer(HWTimeoutAnswerCall, myAnswerDelay);
            };

        }
        break;

        case SignalRingStop:
        {
            cpLog(LOG_DEBUG, "#####   Stop ringing   #####\n");
        }
        break;

        case SignalRemoteRingbackStart:
        {
            cpLog(LOG_DEBUG, "#####   Start remote ringback   #####\n");
        }
        break;

        case SignalRemoteRingbackStop:
        {
            cpLog(LOG_DEBUG, "#####   Stop remote ringback   #####\n");
        }
        break;

        case SignalLocalRingbackStart:
        {
            cpLog(LOG_DEBUG, "#####   Start local ringback    #####\n");
        }
        break;

        case SignalLocalRingbackStop:
        {
            cpLog(LOG_DEBUG, "#####   Stop local ringback   #####\n");
        }
        break;

        case SignalBusyStart:
        {
            cpLog(LOG_DEBUG, "#####   Start busy start   #####\n");
        }
        break;

        case SignalBusyStop:
        {
            cpLog(LOG_DEBUG, "#####   Stop busy stop   #####\n");
        }
        break;

        default:
        {
            // TBD, add this line back in once more signals are supported
            //cpLog(LOG_ERR, "Unsupported hardware signal = %d", signal);
            retval = -1;
        }
        break;
    } // end switch

    return retval;

} // end TestHardware::sendSignal()


int
TestHardware::audioStart (const HardwareAudioRequest& request)
{
    cpLog(LOG_DEBUG, "### Establishing audio ###");

    if (myTestType == CallingGW)
    {
        /*
                stopTimer(HWTimeoutBadState);
                stopTimer(HWTimeoutEndCall);
                startTimer(HWTimeoutEndCall, myCallDuration);
        */
    }

    cpLog(LOG_DEBUG, "Listening on port: %d ", request.localPort);
    cpLog(LOG_DEBUG, "Sending to host: %s ", request.remoteHost);
    cpLog(LOG_DEBUG, "Sending to port: %d", request.remotePort);
    return 0;
} // end TestHardware::audioStart()


int
TestHardware::audioStop (void)
{
    cpLog(LOG_DEBUG, "### Stopping audio ###");
    /*
        if (myTestType == ReceiveGW)
        {
            // Hang up right away.
            startTimer(HWTimeoutEndCall, 1);
        }else if (myTestType == CallingGW)
        {
    //        stopTimer(HWTimeoutStartCall);
    //        startTimer(HWTimeoutStartCall, myCallDelay);
        }
    */ 
    return 0;
} // end TestHardware::audioStart()


HardwareSignal
TestHardware::getSignalOn (MgcpPackageType package, int index) const
{
    if (index < 0)
    {
        return SignalNULL;
    }

    switch (package)
    {
        case MgcpGenericMediaPackageType:
        {
            if (index < GenericReqSigMaxSignal)
            {
                return TestHardwareGenericSigOn[index];
            }
        }
        break;

        case MgcpLinePackageType:
        {
            if (index < LineReqSigMaxSignal)
            {
                return TestHardwareLineSigOn[index];
            }
        }
        break;

        case MgcpDTMFPackageType:
        {
            if (index < DtmfReqSigMaxSignal)
            {
                return TestHardwareDtmfSigOn[index];
            }
        }
        break;

        case MgcpMFPackageType:
        case MgcpTrunkPackageType:
        case MgcpHandsetPackageType:
        case MgcpRTPPackageType:
        case MgcpNetworkAccessServerPackageType:
        case MgcpAnnouncementPackageType:
        case MgcpScriptPackageType:
        case MgcpXVoiceMailPackageType:
        default:
        {
            cpLog(LOG_ERR, "Unknown package = %d", package);
            return SignalNULL;
        }
        break;
    } // end switch

    return SignalNULL;

} // end TestHardware::getSignalOn()


HardwareSignal
TestHardware::getSignalOff (MgcpPackageType package, int index) const
{
    if (index < 0)
    {
        return SignalNULL;
    }

    switch (package)
    {
        case MgcpGenericMediaPackageType:
        {
            if (index < GenericReqSigMaxSignal)
            {
                return TestHardwareGenericSigOff[index];
            }
        }
        break;

        case MgcpLinePackageType:
        {
            if (index < LineReqSigMaxSignal)
            {
                return TestHardwareLineSigOff[index];
            }
        }
        break;

        case MgcpDTMFPackageType:
        {
            if (index < DtmfReqSigMaxSignal)
            {
                return TestHardwareDtmfSigOff[index];
            }
        }
        break;

        case MgcpMFPackageType:
        case MgcpTrunkPackageType:
        case MgcpHandsetPackageType:
        case MgcpRTPPackageType:
        case MgcpNetworkAccessServerPackageType:
        case MgcpAnnouncementPackageType:
        case MgcpScriptPackageType:
        case MgcpXVoiceMailPackageType:
        default:
        {
            cpLog(LOG_ERR, "Unknown package = %d", package);
            return SignalNULL;

        }
        break;
    } // end switch

    return SignalNULL;

} // end TestHardware::getSignalOff()
