
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const TrunkHardware_cxx_Version =
    "$Id: TrunkHardware.cxx,v 1.10 2000/12/18 23:42:25 bko Exp $";
/* stdlib */
#include <iostream.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include "TrunkHardware.h"
#include "ReqSig.h"

/** The following array provides mappings from MGCP encoded signals to
 ** the corresponding TrunkHardware signal.
 */
HardwareSignal TrunkHardwareGenericSigOn[GenericReqSigMaxSignal] =
    {
        SignalNULL,  //GenericSigPatternDetected,
        SignalLocalRingbackStart,  //GenericSigRingbackTone,
        SignalNULL,  //GenericSigRingbackOnConnection,
        SignalNULL,  //GenericSigConfirmTone,
        SignalNULL,  //GenericSigNetworkCongestionTone,
        SignalNULL,  //GenericSigInterceptTone,
        SignalNULL,  //GenericSigPreemptionTone,
    };

HardwareSignal TrunkHardwareGenericSigOff[GenericReqSigMaxSignal] =
    {
        SignalNULL,  //GenericSigPatternDetected,
        SignalLocalRingbackStop,  //GenericSigRingbackTone,
        SignalNULL,  //GenericSigRingbackOnConnection,
        SignalNULL,  //GenericSigConfirmTone,
        SignalNULL,  //GenericSigNetworkCongestionTone,
        SignalNULL,  //GenericSigInterceptTone,
        SignalNULL,  //GenericSigPreemptionTone,
    };

HardwareSignal TrunkHardwareLineSigOn[LineReqSigMaxSignal] =
    {
        SignalNULL,  // LineSigAdsiDisplay,
        SignalNULL,  // LineSigVisualMsg,
        SignalNULL,  // LineSigAnswerTone,
        SignalBusyStart,  // LineSigBusyTone,
        SignalNULL,  // LineSigCallerId,
        SignalNULL,  // LineSigCallWaitTone,
        SignalDialToneStart,  // LineSigDialTone,
        SignalNULL,  // LineSigMsgWaitInd,
        SignalNULL,  // LineSigNetworkBusy,
        SignalRingStart,  // LineSigRinging,
        SignalNULL,  // LineSigDsnctRing0,
        SignalNULL,  // LineSigDsnctRing1,
        SignalNULL,  // LineSigDsnctRing2,
        SignalNULL,  // LineSigDsnctRing3,
        SignalNULL,  // LineSigDsnctRing4,
        SignalNULL,  // LineSigDsnctRing5,
        SignalNULL,  // LineSigDsnctRing6,
        SignalNULL,  // LineSigDsnctRing7,
        SignalNULL,  // LineSigRingSplash,
        SignalNULL,  // LineSigPromptTone,
        SignalNULL,  // LineSigErrorTone,
        SignalNULL,  // LineSigStutterTone,
        SignalNULL,  // LineSigAlertTone,
        SignalNULL,  // LineSigRecordWarnTone,
        SignalNULL,  // LineSigCallCardSrvTone,
        SignalNULL,  // LineSigOffhookWarnTone,
        SignalNULL,  // LineSigDsnctTonePattern,
    };

HardwareSignal TrunkHardwareLineSigOff[LineReqSigMaxSignal] =
    {
        SignalNULL,  // LineSigAdsiDisplay,
        SignalNULL,  // LineSigVisualMsg,
        SignalNULL,  // LineSigAnswerTone,
        SignalBusyStop,  // LineSigBusyTone,
        SignalNULL,  // LineSigCallerId,
        SignalNULL,  // LineSigCallWaitTone,
        SignalDialToneStop,  // LineSigDialTone,
        SignalNULL,  // LineSigMsgWaitInd,
        SignalNULL,  // LineSigNetworkBusy,
        SignalRingStop,  // LineSigRinging,
        SignalNULL,  // LineSigDsnctRing0,
        SignalNULL,  // LineSigDsnctRing1,
        SignalNULL,  // LineSigDsnctRing2,
        SignalNULL,  // LineSigDsnctRing3,
        SignalNULL,  // LineSigDsnctRing4,
        SignalNULL,  // LineSigDsnctRing5,
        SignalNULL,  // LineSigDsnctRing6,
        SignalNULL,  // LineSigDsnctRing7,
        SignalNULL,  // LineSigRingSplash,
        SignalNULL,  // LineSigPromptTone,
        SignalNULL,  // LineSigErrorTone,
        SignalNULL,  // LineSigStutterTone,
        SignalNULL,  // LineSigAlertTone,
        SignalNULL,  // LineSigRecordWarnTone,
        SignalNULL,  // LineSigCallCardSrvTone,
        SignalNULL,  // LineSigOffhookWarnTone,
        SignalNULL,  // LineSigDsnctTonePattern,
    };

HardwareSignal TrunkHardwareDtmfSigOn[DtmfReqSigMaxSignal] =
    {
        SignalNULL,  //DtmfSig0,
        SignalNULL,  //DtmfSig1,
        SignalNULL,  //DtmfSig2,
        SignalNULL,  //DtmfSig3,
        SignalNULL,  //DtmfSig4,
        SignalNULL,  //DtmfSig5,
        SignalNULL,  //DtmfSig6,
        SignalNULL,  //DtmfSig7,
        SignalNULL,  //DtmfSig8,
        SignalNULL,  //DtmfSig9,
        SignalNULL,  //DtmfSigHash,
        SignalNULL,  //DtmfSigStar,
        SignalNULL,  //DtmfSigA,
        SignalNULL,  //DtmfSigB,
        SignalNULL,  //DtmfSigC,
        SignalNULL,  //DtmfSigD
    };

HardwareSignal TrunkHardwareDtmfSigOff[DtmfReqSigMaxSignal] =
    {
        SignalNULL,  //DtmfSig0,
        SignalNULL,  //DtmfSig1,
        SignalNULL,  //DtmfSig2,
        SignalNULL,  //DtmfSig3,
        SignalNULL,  //DtmfSig4,
        SignalNULL,  //DtmfSig5,
        SignalNULL,  //DtmfSig6,
        SignalNULL,  //DtmfSig7,
        SignalNULL,  //DtmfSig8,
        SignalNULL,  //DtmfSig9,
        SignalNULL,  //DtmfSigHash,
        SignalNULL,  //DtmfSigStar,
        SignalNULL,  //DtmfSigA,
        SignalNULL,  //DtmfSigB,
        SignalNULL,  //DtmfSigC,
        SignalNULL,  //DtmfSigD
    };


TrunkHardware::TrunkHardware ( char* deviceName, int numEndpoints,
                               Fifo < HardwareMsg > * inputQ,
                               Fifo < EndpointMsg > * outputQ[] )
{
    myFD = 0;
    myNumEndpoints = numEndpoints;
    for (int i = 0; i < myNumEndpoints; i++)
    {
        myEndpointQ[i] = outputQ[i];
    }
    myQ = inputQ;

    // read and write from standard in and standard out
    system("stty -icanon");

}  // end TrunkHardware::TrunkHardware()


TrunkHardware::~TrunkHardware (void)
{
    // stop reading and writing from standard in and standard out
    system("stty icanon");

} // end TrunkHardware::~TrunkHardware()


void*
TrunkHardware::hardwareMain (void* parms)
{
    // process forever on behalf of null hardware
    fd_set readfds;
    struct timeval tv;
    bool operational = true;
    int maxFd = 32;
    int retval = 0;

    while (operational)
    {
        // reset file descriptor
        FD_ZERO(&readfds);
        addToFdSet(&readfds);
        //if (maxFd <= gateway.myAgent->getFd())
        //maxFd =  gateway.myAgent->getFd() + 1;

        // block on select but timeout needed to service myQ
        tv.tv_sec = 0;
        tv.tv_usec = 300;

        if ((retval = select(maxFd, &readfds, NULL, NULL, &tv)) < 0)
        {
            cpLog(LOG_ERR, "Select returned with an error - ignore");
        }
        else if (retval > 0)
        {
            // hardware input received
            if (process(&readfds) < 0)
            {
                cpLog(LOG_ERR, "Hardware failed - shutting down");
                operational = false;
            }
        }

        // process msgs from endpoints (if any)
        while (myQ->size() > 0)
        {
            processEndpointMsg(myQ->getNext());
        }
    } // end while

    return NULL;

} // end TrunkHardware::hardwareMain()


int
TrunkHardware::process (fd_set* fd)
{
    HardwareEvent event = EventNULL;

    char keystroke;
    cin >> keystroke;
    switch (keystroke)
    {
        case 'a':
        {
            // offhook
            event = EventOffhook;
            hookStateOffhook = true;
        }
        break;

        case 'z':
        {
            // onhook
            event = EventOnhook;
            hookStateOffhook = false;
        }
        break;

        case 'f':
        {
            // flashhook
            if (hookStateOffhook)
            {
                event = EventFlashhook;
            }
            else
            {
                cpLog(LOG_ERR, "Not offhook - flashhook ignored");
            }
        }
        break;

        case '1':
        {
            event = EventDigit1;
        }
        break;

        case '2':
        {
            event = EventDigit2;
        }
        break;

        case '3':
        {
            event = EventDigit3;
        }
        break;

        case '4':
        {
            event = EventDigit4;
        }
        break;

        case '5':
        {
            event = EventDigit5;
        }
        break;

        case '6':
        {
            event = EventDigit6;
        }
        break;

        case '7':
        {
            event = EventDigit7;
        }
        break;

        case '8':
        {
            event = EventDigit8;
        }
        break;

        case '9':
        {
            event = EventDigit9;
        }
        break;

        case '0':
        {
            event = EventDigit0;
        }
        break;

        default:
        {
            // no events
            cpLog(LOG_ERR, "Unknown hardware input = %d", keystroke);
            break;
        }
    } // end switch

    if (event != EventNULL)
    {
        // TBD, need to determine appropriate endpoint
        reportEvent(myEndpointQ[0], event);
    }

    return 0;
} // end TrunkHardware::process()


int
TrunkHardware::addToFdSet (fd_set* fd)
{
    FD_SET(myFD, fd);
    return 0;
} // end TrunkHardware::addToFdSet()


int
TrunkHardware::sendSignal (HardwareSignal signal)
{
    int retval = 0;

    switch (signal)
    {
        case SignalDialToneStart:
        {
            cpLog(LOG_DEBUG, "###   Start dialtone   ###");
        }
        break;

        case SignalDialToneStop:
        {
            cpLog(LOG_DEBUG, "###   Stop dialtone   ###");
        }
        break;

        case SignalRingStart:
        {
            cpLog(LOG_DEBUG, "###   Start ringing   ###");
        }
        break;

        case SignalRingStop:
        {
            cpLog(LOG_DEBUG, "###   Stop ringing   ###");
        }
        break;

        case SignalRemoteRingbackStart:
        {
            cpLog(LOG_DEBUG, "###   Start remote ringback   ###");
        }
        break;

        case SignalRemoteRingbackStop:
        {
            cpLog(LOG_DEBUG, "###   Stop remote ringback   ###");
        }
        break;

        case SignalLocalRingbackStart:
        {
            cpLog(LOG_DEBUG, "###   Start local ringback   ###");
        }
        break;

        case SignalLocalRingbackStop:
        {
            cpLog(LOG_DEBUG, "###   Stop local ringback   ###");
        }
        break;

        case SignalBusyStart:
        {
            cpLog(LOG_DEBUG, "###   Start busy start   ###");
        }
        break;

        case SignalBusyStop:
        {
            cpLog(LOG_DEBUG, "###   Stop busy stop   ###");
        }
        break;

        default:
        {
            // TBD, add this line back in once more signals are supported
            //cpLog(LOG_ERR, "Unsupported hardware signal = %d", signal);
            retval = -1;
        }
        break;
    } // end switch

    return retval;

} // end TrunkHardware::sendSignal()


int
TrunkHardware::audioStart (const HardwareAudioRequest& request)
{
    cpLog(LOG_DEBUG, "### Establishing audio ###");
    cpLog(LOG_DEBUG, "Listening on port: %d ", request.localPort);
    cpLog(LOG_DEBUG, "Sending to host: %s ", request.remoteHost);
    cpLog(LOG_DEBUG, "Sending to port: %d", request.remotePort);
    return 0;
} // end TrunkHardware::audioStart()


int
TrunkHardware::audioStop (void)
{
    cpLog(LOG_DEBUG, "### Stopping audio ###");
    return 0;
} // end TrunkHardware::audioStart()


HardwareSignal
TrunkHardware::getSignalOn (MgcpPackageType package, int index) const
{
    if (index < 0)
    {
        return SignalNULL;
    }

    switch (package)
    {
        case MgcpGenericMediaPackageType:
        {
            if (index < GenericReqSigMaxSignal)
            {
                return TrunkHardwareGenericSigOn[index];
            }
        }
        break;

        case MgcpLinePackageType:
        {
            if (index < LineReqSigMaxSignal)
            {
                return TrunkHardwareLineSigOn[index];
            }
        }
        break;

        case MgcpDTMFPackageType:
        {
            if (index < DtmfReqSigMaxSignal)
            {
                return TrunkHardwareDtmfSigOn[index];
            }
        }
        break;

        case MgcpMFPackageType:
        case MgcpTrunkPackageType:
        case MgcpHandsetPackageType:
        case MgcpRTPPackageType:
        case MgcpNetworkAccessServerPackageType:
        case MgcpAnnouncementPackageType:
        case MgcpScriptPackageType:
        case MgcpXVoiceMailPackageType:
        default:
        {
            cpLog(LOG_ERR, "Unknown package = %d", package);
            return SignalNULL;
        }
        break;
    } // end switch

    return SignalNULL;

} // end TrunkHardware::getSignalOn()


HardwareSignal
TrunkHardware::getSignalOff (MgcpPackageType package, int index) const
{
    if (index < 0)
    {
        return SignalNULL;
    }

    switch (package)
    {
        case MgcpGenericMediaPackageType:
        {
            if (index < GenericReqSigMaxSignal)
            {
                return TrunkHardwareGenericSigOff[index];
            }
        }
        break;

        case MgcpLinePackageType:
        {
            if (index < LineReqSigMaxSignal)
            {
                return TrunkHardwareLineSigOff[index];
            }
        }
        break;

        case MgcpDTMFPackageType:
        {
            if (index < DtmfReqSigMaxSignal)
            {
                return TrunkHardwareDtmfSigOff[index];
            }
        }
        break;

        case MgcpMFPackageType:
        case MgcpTrunkPackageType:
        case MgcpHandsetPackageType:
        case MgcpRTPPackageType:
        case MgcpNetworkAccessServerPackageType:
        case MgcpAnnouncementPackageType:
        case MgcpScriptPackageType:
        case MgcpXVoiceMailPackageType:
        default:
        {
            cpLog(LOG_ERR, "Unknown package = %d", package);
            return SignalNULL;

        }
        break;
    } // end switch

    return SignalNULL;

} // end TrunkHardware::getSignalOff()
