#ifndef TRUNK_HARDWARE_H_
#define TRUNK_HARDWARE_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TrunkHardware_h_Version =
    "$Id: TrunkHardware.h,v 1.8 2000/12/18 23:42:25 bko Exp $";

#include "Hardware.h"
#include "Gateway.h"

/**********************************************************************
Class: TrunkHardware
Derived class of Hardware base class
Used for testing a trunk gateway hardware
**********************************************************************/
class TrunkHardware: public Hardware
{
    public:
        /** Creates a hardware object to control a trunk gateway.
         */
        TrunkHardware ( char* deviceName, int numEndpoints,
                        Fifo < HardwareMsg > * inputQ,
                        Fifo < EndpointMsg > ** outputQ );

        /** Destructor
         */
        ~TrunkHardware (void);

        /** Starts the main processing loop of the hardware
         */
        void*
        hardwareMain (void* parms);

        /** Add device's socket to the fd; returns 0 if successful,
         ** errorcode otherwise 
         */
        int addToFdSet (fd_set* fd);

        /** Process all pending hardware events; returns 0 if successful,
         ** otherwise returns an errorcode.  
         */
        int
        process (fd_set* fd);

        /** Start audio channel on device; returns 0 if successful,
         ** errorcode otherwise 
         */
        int audioStart (const HardwareAudioRequest& request);

        /** Stops audio channel on device; returns 0 if successful,
         ** errorcode otherwise.
         */
        int
        audioStop (void);

        /** Returns the HardwareSignal supported by the hardware which
         ** corresponds to turning on the mgcp encoded signal.
         */
        HardwareSignal
        getSignalOn (MgcpPackageType package, int index) const;

        /** Returns the HardwareSignal supported by the hardware which
         ** corresponds to turning off the mgcp encoded signal.
         */
        HardwareSignal
        getSignalOff (MgcpPackageType package, int index) const;

    private:
        /** Trigger the device to send the specified signal; returns 0 if
         ** successful, errorcode otherwise.
         */
        int
        sendSignal (HardwareSignal signal);

        /** Number of endpoints supported by hardware.
         */
        int myNumEndpoints;

        /** Output queues to notify endpoints of occurence of hardware
         ** events.
         */
        Fifo < EndpointMsg > * myEndpointQ[GW_ENDPOINT_MAX];
};

#endif // TRUNK_HARDWARE_H_
