#ifndef COMMON_H_
#define COMMON_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const common_h_Version =
    "$Id: common.h,v 1.20 2001/01/18 02:45:31 liuhong Exp $";

#include <pthread.h>
#include "Fifo.h"
//#include "sdp.hxx"
#include "cpLog.h"
#include "MgCommand.hxx"
#include "MgcpResponseCode.hxx"

/** Abbreviated types for all cmds that may be retrieved from the
 ** mgcp stack. Note that any cmds sent into the stack requires
 ** the allocation of real memory space so smart prts are not
 ** used.
 */
typedef Sptr < MgCommand > CMD;
typedef Sptr < MgEndpointConfiguration > EPCF;
typedef Sptr < MgCreateConnection > CRCX;
typedef Sptr < MgModifyConnection > MDCX;
typedef Sptr < MgDeleteConnectionAgent > DLCX;
typedef Sptr < MgNotificationRequest > RQNT;
typedef Sptr < MgAuditEndpoint > AUEP;
typedef Sptr < MgAuditConnection > AUCX;


// events detected by hardware and reported to control software
enum HardwareEvent
{
    EventNULL,
    EventOffhook,
    EventOnhook,
    EventFlashhook,
    EventDigit0,
    EventDigit1,
    EventDigit2,
    EventDigit3,
    EventDigit4,
    EventDigit5,
    EventDigit6,
    EventDigit7,
    EventDigit8,
    EventDigit9,
    EventDigitStar,
    EventDigitHash,
    EventAudioFailed,
    EventAudioClosed,
};

// types of messages which may be sent to the Hardware
enum HardwareMsgType
{
    HardwareSignalType,
    HardwareAudioType,
    HardwareStartNewCall,
    HardwareTimeoutType,    // Used for stress testing
};

// signals provided by the hardware
enum HardwareSignal
{
    SignalNULL,
    SignalDialToneStart,
    SignalDialToneStop,
    SignalRingStart,
    SignalRingStop,
    SignalRemoteRingbackStart,
    SignalRemoteRingbackStop,
    SignalLocalRingbackStart,
    SignalLocalRingbackStop,
    SignalBusyStart,
    SignalBusyStop,
    SignalDigit0,
    SignalDigit1,
    SignalDigit2,
    SignalDigit3,
    SignalDigit4,
    SignalDigit5,
    SignalDigit6,
    SignalDigit7,
    SignalDigit8,
    SignalDigit9,
    SignalDigitStar,
    SignalDigitHash,
    SignalDigitA,
    SignalDigitB,
    SignalDigitC,
    SignalDigitD
};

// audio requests provided by the hardware
enum HardwareAudioRequestType
{
    AudioStart,
    AudioStop,
    //AudioSwap,
    //AudioModify,
};

struct HardwareAudioRequest
{
    HardwareAudioRequestType type;
    int localPort;
    int remotePort;
    const char* remoteHost;
    const char* mode;
    bool echoCancellation;
};

// ************************************************************************
enum TestType
{
    NotInTest,
    CallingGW,
    ReceiveGW,
};
// ************************************************************************

enum HardwareTimeout
{
    HWTimeoutEndCall,
    HWTimeoutStartCall,
    HWTimeoutDialDigits,
    HWTimeoutAnswerCall,
    HWTimeoutBadState,
    GW_FAKE_RSIP,
    HWTimeoutMaxTimer,
};


typedef struct
{
    HardwareMsgType type;
    int endpointIndex;
    union
    {
        HardwareSignal signal;
        struct HardwareAudioRequest request;
        HardwareTimeout timeout;
    };
}
HardwareMsg;


// types of messages which may be sent to an Endpoint
enum EndpointMsgType
{
    EndpointAgentCommandType,
    EndpointHardwareEventType,
    EndpointTimeoutType
};

typedef struct EndpointAgentCommandStruct
{
    MgCommandType type;
    CMD cmd;
}
EndpointAgentCommand;

enum EndpointTimeout
{
    TimeoutInterdigit = 0,
    TimeoutStartCall,
    TimeoutEndCall,
    TimeoutAnswerCall,
    TimeoutRecover,
    TimeoutBadState,
    TimeoutStuckState,
    TimeoutFlashhook,
    // other timers
    // :
    TimeoutMaxTimer,
    TimeoutInvalidTimer
};

typedef struct
{
    EndpointMsgType type;
    int endpointIndex;
    //  union
    //  {

    // warning: EndpointAgentCommand should not be a pointer -- if
    // it is, then the fifo class will not copy the command, but
    // the copy in the fifo will merely point to another
    // EndpointAgentCommand structure, e.g.

    // ---------------
    // | EndpointMsg |----
    // ---------------   |
    //                   --> ---------------
    //                       | EndptAgtCmd |
    //                       ---------------

    // when copied, becomes

    //
    // ---------------
    // |   copy of   |
    // | EndpointMsg |----------
    // ---------------         |
    //                         |
    // ---------------         |
    // | EndpointMsg |----     |
    // ---------------   |     v
    //                   ----> ---------------
    //                         | EndptAgtCmd |
    //                         ---------------

    // this was made worse by the fact that EndptAgtCmd was static
    // so then when you pass in another, it becomes

    EndpointAgentCommand command;
    HardwareEvent event;
    EndpointTimeout timeout;
    //  };
}

EndpointMsg;


#endif // COMMON_H_
