/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpHeader_cxx_Version =
    "$Id: HttpHeader.cxx,v 1.6 2001/11/05 18:07:36 icahoon Exp $";


#include "HttpHeader.hxx"
#include "Verify.hxx"
#include <cassert>


using Vocal::Http::Header;
using Vocal::Http::Exception;
using Vocal::Http::Type;
using Vocal::Http::Token;
using Vocal::Http::EXTENSION;
using Vocal::Http::SP;

size_t              Header::myTotal = 0;


// General headers
//
const Type  Header::CacheControl        = Header::myTotal++;
const Type  Header::Connection          = Header::myTotal++;
const Type  Header::Date                = Header::myTotal++;
const Type  Header::Pragma              = Header::myTotal++;
const Type  Header::Trailer             = Header::myTotal++;
const Type  Header::TransferEncoding    = Header::myTotal++;
const Type  Header::Upgrade             = Header::myTotal++;
const Type  Header::Via                 = Header::myTotal++;
const Type  Header::Warning             = Header::myTotal++;


// Request headers
//
const Type  Header::Accept              = Header::myTotal++;
const Type  Header::AcceptCharset       = Header::myTotal++;
const Type  Header::AcceptEncoding      = Header::myTotal++;
const Type  Header::AcceptLanguage      = Header::myTotal++;
const Type  Header::Authorization       = Header::myTotal++;
const Type  Header::Expect              = Header::myTotal++;
const Type  Header::From                = Header::myTotal++;
const Type  Header::Host                = Header::myTotal++;
const Type  Header::IfMatch             = Header::myTotal++;
const Type  Header::IfModifiedSince     = Header::myTotal++;
const Type  Header::IfNoneMatch         = Header::myTotal++;
const Type  Header::IfRange             = Header::myTotal++;
const Type  Header::IfUnmodifiedSince   = Header::myTotal++;
const Type  Header::MaxForwards         = Header::myTotal++;
const Type  Header::ProxyAuthorizatiation = Header::myTotal++;
const Type  Header::Range               = Header::myTotal++;
const Type  Header::Referer             = Header::myTotal++;
const Type  Header::TE                  = Header::myTotal++;
const Type  Header::UserAgent           = Header::myTotal++;


// Response headers
//
const Type  Header::AcceptRanges        = Header::myTotal++;
const Type  Header::Age                 = Header::myTotal++;
const Type  Header::ETag                = Header::myTotal++;
const Type  Header::Location            = Header::myTotal++;
const Type  Header::ProxyAuthentication = Header::myTotal++;
const Type  Header::RetryAfter          = Header::myTotal++;
const Type  Header::Server              = Header::myTotal++;
const Type  Header::Vary                = Header::myTotal++;
const Type  Header::WWWAuthenticate     = Header::myTotal++;


// Entity headers
//
const Type  Header::Allow               = Header::myTotal++;
const Type  Header::ContentEncoding     = Header::myTotal++;
const Type  Header::ContentLanguage     = Header::myTotal++;
const Type  Header::ContentLength       = Header::myTotal++;
const Type  Header::ContentLocation     = Header::myTotal++;
const Type  Header::ContentMD5          = Header::myTotal++;
const Type  Header::ContentRange        = Header::myTotal++;
const Type  Header::ContentType         = Header::myTotal++;
const Type  Header::Expires             = Header::myTotal++;
const Type  Header::LastModified        = Header::myTotal++;


// General headers
//
const Token     Header::TokenCacheControl("CacheControl");
const Token     Header::TokenConnection("Connection");
const Token     Header::TokenDate("Date");
const Token     Header::TokenPragma("Pragma");
const Token     Header::TokenTrailer("Trailer");
const Token     Header::TokenTransferEncoding("TransferEncoding");
const Token     Header::TokenUpgrade("Upgrade");
const Token     Header::TokenVia("Via");
const Token     Header::TokenWarning("Warning");


// Request headers
//
const Token      Header::TokenAccept("Accept");
const Token      Header::TokenAcceptCharset("Accept-Charset");
const Token      Header::TokenAcceptEncoding("Accept-Encoding");
const Token      Header::TokenAcceptLanguage("Accept-Language");
const Token      Header::TokenAuthorization("Authorization");
const Token      Header::TokenExpect("Expect");
const Token      Header::TokenFrom("From");
const Token      Header::TokenHost("Host");
const Token      Header::TokenIfMatch("If-Match");
const Token      Header::TokenIfModifiedSince("If-Modified-Since");
const Token      Header::TokenIfNoneMatch("If-None-Match");
const Token      Header::TokenIfRange("If-Range");
const Token      Header::TokenIfUnmodifiedSince("If-Unmodified-Since");
const Token      Header::TokenMaxForwards("Max-Forwards");
const Token      Header::TokenProxyAuthorizatiation("Proxy-Authorizatiation");
const Token      Header::TokenRange("Range");
const Token      Header::TokenReferer("Referer");
const Token      Header::TokenTE("TE");
const Token      Header::TokenUserAgent("User-Agent");


// Response headers
//
const Token      Header::TokenAcceptRanges("Accept-Ranges");
const Token      Header::TokenAge("Age");
const Token      Header::TokenETag("ETag");
const Token      Header::TokenLocation("Location");
const Token      Header::TokenProxyAuthentication("Proxy-Authentication");
const Token      Header::TokenRetryAfter("Retry-After");
const Token      Header::TokenServer("Server");
const Token      Header::TokenVary("Vary");
const Token      Header::TokenWWWAuthenticate("WWW-Authenticate");


// Entity headers
//
const Token      Header::TokenAllow("Allow");
const Token      Header::TokenContentEncoding("Content-Encoding");
const Token      Header::TokenContentLanguage("Content-Language");
const Token      Header::TokenContentLength("Content-Length");
const Token      Header::TokenContentLocation("Content-Location");
const Token      Header::TokenContentMD5("Content-MD5");
const Token      Header::TokenContentRange("Content-Range");
const Token      Header::TokenContentType("Content-Type");
const Token      Header::TokenExpires("Expires");
const Token      Header::TokenLastModified("Last-Modified");


Header::Table       Header::myTable;
Header::DecodeTable Header::myDecodeTable;


Header::Header(Type pType)
{
    init();
    
    type(pType);
    
    test_covered("0");
}


Header::Header(const Token & pToken)
    :   myType(EXTENSION)
{
    init();
    
    if ( pToken.size() )
    {
        type(pToken);
    }
    
    test_covered("1");
}


Header::Header(Type pType, const Value & pValue)
{
    init();
    
    type(pType);
    value(pValue);
    
    test_covered("2");
}


Header::Header(const Token & pToken, const Value & pValue)
    :   myType(EXTENSION)
{
    init();
    
    if ( pToken.size() )
    {
        type(pToken);
        value(pValue);
    }

    test_covered("3");
}


Header::Header(const Header & src)
    :   myType(src.myType),
        myToken(src.myToken),
        myValue(src.myValue)
{
    test_covered("4");
}


Header::~Header()
{
    test_covered("5");
}


const Header &  
Header::operator=(const Header & src)
{
    if ( this != &src )
    {
        myType = src.myType;
        myToken = src.myToken;
        myValue = src.myValue;
        
        test_covered("6");
    }
    
    test_covered("7");
    
    return ( *this );
}


bool            
Header::operator==(const Header & src) const
{
    if ( this == &src )
    {
        test_covered("8");
        
        return ( true );
    }

    if ( isExtension() && src.isExtension() )
    {
        test_covered("9");
        
        return ( myToken == src.myToken && myValue == src.myValue );
    }
    
    test_covered("10");
    
    return ( myType == src.myType && myValue == src.myValue );
}


bool            
Header::operator<(const Header & src) const
{
    if ( this == &src )
    {
        test_covered("11");
        
        return ( false );
    }
    
    if ( isExtension() && src.isExtension() )
    {
        test_covered("12");
        
        return ( myToken != src.myToken 
                    ? myToken < src.myToken 
                    : myValue < src.myValue );
    }
    
    test_covered("13");
    
    return ( myType != src.myType 
                ? myType < src.myType 
                : myValue < src.myValue );
}


Type       
Header::type() const
{
    test_covered("14");
    
    return ( myType );
}


void
Header::type(Type pType)
{
    myToken.erase();
    myValue.erase();
    
    Table::const_iterator i = myTable.find(pType);
    if ( i == myTable.end() )
    {
        pType = EXTENSION;
        
        test_covered("15");
    }
    myType = pType;

    test_covered("16");
}


void
Header::type(const Token & pToken)
throw ( Vocal::Http::Exception )
{
    myToken.erase();
    myValue.erase();
    
    DecodeTable::const_iterator i = myDecodeTable.find(pToken);
    if ( i != myDecodeTable.end() )
    {
        myType = i->second;
        
        test_covered("17");   
        
        return;
    }

    if ( pToken.size() == 0 )
    {
        test_covered("18");
        
        throw ( Vocal::Http::Exception(Exception::BadHeader) );
    }
    
    // This is an extension
    //
    myType = EXTENSION;
    
    // Probably need to do better constraint checking at some point.
    //
    myToken = pToken;

    test_covered("19");
}


const Header::Value &  
Header::value() const
{
    test_covered("20");
    
    return ( myValue );
}


void
Header::value(const Value & pValue)
{
    myValue = pValue;
    
    test_covered("21");
}


void
Header::continueValue(const Value & pValue)
{
    if ( pValue.size() )
    {
        myValue += SP;
        myValue += pValue;
    
        test_covered("22");
    }
}


const Token &  
Header::token() const
{
    if ( isExtension() )
    {
        test_covered("23");
        
        return ( myToken );
    }
    test_covered("24");
    
    return ( *myTable[myType] );
}


const Token &  
Header::name() const
{
    test_covered("25");
    
    return ( token() );
}


size_t   
Header::totalTypes()
{
    test_covered("26");
    
    return ( myTotal );
}


bool    
Header::isGeneral() const
{
    test_covered("27");
    
    return ( myType >= CacheControl && myType <= Warning );
}


bool    
Header::isRequest() const
{
    test_covered("28");
    
    return ( myType >= Accept && myType <= UserAgent );
}


bool    
Header::isResponse() const
{
    test_covered("29");
    
    return ( myType >= AcceptRanges && myType <= WWWAuthenticate );
}


bool    
Header::isEntity() const
{
    test_covered("30");
    
    return ( myType >= Allow && myType <= LastModified );
}


bool    
Header::isExtension() const
{
    test_covered("31");
    
    return ( myType == EXTENSION );
}
        

ostream &       
Header::writeTo(ostream & out) const
{
    test_covered("32");
    
    return ( out << name() << ": " << value() );
}


void
Header::init()
{
    if ( myTable.size() == 0 )
    {
        initTable();
        initDecodeTable();
        
        test_covered("33");
    }
}


void    
Header::initTable()
{
    myTable[CacheControl]           = &TokenCacheControl;
    myTable[Connection]             = &TokenConnection;
    myTable[Date]                   = &TokenDate;
    myTable[Pragma]                 = &TokenPragma;
    myTable[Trailer]                = &TokenTrailer;
    myTable[TransferEncoding]       = &TokenTransferEncoding;
    myTable[Upgrade]                = &TokenUpgrade;
    myTable[Via]                    = &TokenVia;
    myTable[Warning]                = &TokenWarning;
    myTable[Accept]                 = &TokenAccept;
    myTable[AcceptCharset]          = &TokenAcceptCharset;
    myTable[AcceptEncoding]         = &TokenAcceptEncoding;
    myTable[AcceptLanguage]         = &TokenAcceptLanguage;
    myTable[Authorization]          = &TokenAuthorization;
    myTable[Expect]                 = &TokenExpect;
    myTable[From]                   = &TokenFrom;
    myTable[Host]                   = &TokenHost;
    myTable[IfMatch]                = &TokenIfMatch;
    myTable[IfModifiedSince]        = &TokenIfModifiedSince;
    myTable[IfNoneMatch]            = &TokenIfNoneMatch;
    myTable[IfRange]                = &TokenIfRange;
    myTable[IfUnmodifiedSince]      = &TokenIfUnmodifiedSince;
    myTable[MaxForwards]            = &TokenMaxForwards;
    myTable[ProxyAuthorizatiation]  = &TokenProxyAuthorizatiation;
    myTable[Range]                  = &TokenRange;
    myTable[Referer]                = &TokenReferer;
    myTable[TE]                     = &TokenTE;
    myTable[UserAgent]              = &TokenUserAgent;
    myTable[AcceptRanges]           = &TokenAcceptRanges;
    myTable[Age]                    = &TokenAge;
    myTable[ETag]                   = &TokenETag;
    myTable[Location]               = &TokenLocation;
    myTable[ProxyAuthentication]    = &TokenProxyAuthentication;
    myTable[RetryAfter]             = &TokenRetryAfter;
    myTable[Server]                 = &TokenServer;
    myTable[Vary]                   = &TokenVary;
    myTable[WWWAuthenticate]        = &TokenWWWAuthenticate;
    myTable[Allow]                  = &TokenAllow;
    myTable[ContentEncoding]        = &TokenContentEncoding;
    myTable[ContentLanguage]        = &TokenContentLanguage;
    myTable[ContentLength]          = &TokenContentLength;
    myTable[ContentLocation]        = &TokenContentLocation;
    myTable[ContentMD5]             = &TokenContentMD5;
    myTable[ContentRange]           = &TokenContentRange;
    myTable[ContentType]            = &TokenContentType;
    myTable[Expires]                = &TokenExpires;
    myTable[LastModified]           = &TokenLastModified;

    assert( totalTypes() == myTable.size() );

    test_covered("34");
}


void    
Header::initDecodeTable()
{
    myDecodeTable[TokenCacheControl]            = CacheControl;
    myDecodeTable[TokenConnection]              = Connection;
    myDecodeTable[TokenDate]                    = Date;
    myDecodeTable[TokenPragma]                  = Pragma;
    myDecodeTable[TokenTrailer]                 = Trailer;
    myDecodeTable[TokenTransferEncoding]        = TransferEncoding;
    myDecodeTable[TokenUpgrade]                 = Upgrade;
    myDecodeTable[TokenVia]                     = Via;
    myDecodeTable[TokenWarning]                 = Warning;
    myDecodeTable[TokenAccept]                  = Accept;
    myDecodeTable[TokenAcceptCharset]           = AcceptCharset;
    myDecodeTable[TokenAcceptEncoding]          = AcceptEncoding;
    myDecodeTable[TokenAcceptLanguage]          = AcceptLanguage;
    myDecodeTable[TokenAuthorization]           = Authorization;
    myDecodeTable[TokenExpect]                  = Expect;
    myDecodeTable[TokenFrom]                    = From;
    myDecodeTable[TokenHost]                    = Host;
    myDecodeTable[TokenIfMatch]                 = IfMatch;
    myDecodeTable[TokenIfModifiedSince]         = IfModifiedSince;
    myDecodeTable[TokenIfNoneMatch]             = IfNoneMatch;
    myDecodeTable[TokenIfRange]                 = IfRange;
    myDecodeTable[TokenIfUnmodifiedSince]       = IfUnmodifiedSince;
    myDecodeTable[TokenMaxForwards]             = MaxForwards;
    myDecodeTable[TokenProxyAuthorizatiation]   = ProxyAuthorizatiation;
    myDecodeTable[TokenRange]                   = Range;
    myDecodeTable[TokenReferer]                 = Referer;
    myDecodeTable[TokenTE]                      = TE;
    myDecodeTable[TokenUserAgent]               = UserAgent;
    myDecodeTable[TokenAcceptRanges]            = AcceptRanges;
    myDecodeTable[TokenAge]                     = Age;
    myDecodeTable[TokenETag]                    = ETag;
    myDecodeTable[TokenLocation]                = Location;
    myDecodeTable[TokenProxyAuthentication]     = ProxyAuthentication;
    myDecodeTable[TokenRetryAfter]              = RetryAfter;
    myDecodeTable[TokenServer]                  = Server;
    myDecodeTable[TokenVary]                    = Vary;
    myDecodeTable[TokenWWWAuthenticate]         = WWWAuthenticate;
    myDecodeTable[TokenAllow]                   = Allow;
    myDecodeTable[TokenContentEncoding]         = ContentEncoding;
    myDecodeTable[TokenContentLanguage]         = ContentLanguage;
    myDecodeTable[TokenContentLength]           = ContentLength;
    myDecodeTable[TokenContentLocation]         = ContentLocation;
    myDecodeTable[TokenContentMD5]              = ContentMD5;
    myDecodeTable[TokenContentRange]            = ContentRange;
    myDecodeTable[TokenContentType]             = ContentType;
    myDecodeTable[TokenExpires]                 = Expires;
    myDecodeTable[TokenLastModified]            = LastModified;

    assert( totalTypes() == myDecodeTable.size() );

    test_covered("35");
}
