#if !defined(VOCAL_HTTP_MESSAGE_HXX)
#define VOCAL_HTTP_MESSAGE_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpMessage_hxx_Version =
    "$Id: HttpMessage.hxx,v 1.5 2001/04/24 17:32:49 icahoon Exp $";


#include "HttpMethod.hxx"
#include "HttpUrl.hxx"
#include "HttpVersion.hxx"
#include "HttpStatus.hxx"
#include "HttpHeader.hxx"
#include "HttpException.hxx"
#include "Writer.hxx"
#include <list>
#include <string>


/** Infrastructure common to VOCAL.<br><br>
 */
namespace Vocal 
{


/** Http Protocol Libraries<br><br>
 */
namespace Http
{


using std::list;
using std::string;


/** Implementation of RFC 2616, Section 4-1.<br><br>
 */
class Message : public Vocal::IO::Writer
{
    public:


        /** Message type
         */
        enum    Type
        {
            Unknown,
            Request,
            Response
        };
        
        
        /** Default constructor
         */
        Message();


        /** Request constructor
         */
        Message(
            const Method        &   method,
            const Url           &   url,
            const list<Header>  &   headers,
            const string        &   body
        );


        /** Response constructor
         */
        Message(
            const Status        &   status,
            const list<Header>  &   headers,
            const string        &   body
        );


        /** Copy constructor
         */
        Message(const Message &);
        
        
        /** Virtual destructor
         */        
        virtual ~Message();


        /** Assignment operator
         */
        const Message & operator=(const Message &);
        
        
        /** Equality operator
         */
        bool operator==(const Message &);
        
        
        /** Less than relational operator
         */
        bool operator<(const Message &);
        

        /** Message type accessor
         */
        Type    type() const;


        /** Message type mutator
         */
        void    type(Type);


        /** Method accessor
         */
        const Method &  method() const;
        
        
        /** Method mutator
         */
        void    method(const Method &);
        
        
        /** Url accessor
         */
        const Url &     url() const;
        
        
        /** Url mutator
         */
        void    url(const Url &);
        
        
        /** Version accessor
         */
        const Version & version() const;
        
        
        /** Version mutator
         */
        void    version(const Version &);


        /** Status accessor
         */
        const Status &  status() const;
        
        
        /** Status mutator
         */
        void    status(const Status &);
        
        
        /** Header accessor
         */
        const list<Header> &    headers() const;
        
        
        /** Header mutator
         */
        void    headers(const list<Header> &);
        
        
        /** Body accessor
         */
        const string &      body() const;
        

        /** Body accessor
         */
        void    body(const string &);
        

        /** Write the message to a stream
         */
        ostream &   writeTo(ostream &) const;                
        

        /** Encode the message
         */
        const string &      encode() throw ( Vocal::Http::Exception );


        /** Decode the message. Returns the number of bytes read from
         *  the string.
         */
        size_t  decode(const string &) throw ( Vocal::Http::Exception );


    private:

        bool    decodeStatusLine(Token &, Token &, size_t &) 
                    throw ( Vocal::Http::Exception );
                    
        void    decodeRequestLine(Token &, Token &, size_t &) 
                    throw ( Vocal::Http::Exception );

        size_t  decodeHeaders(Token &, Token &, size_t &)
                    throw ( Vocal::Http::Exception );

        void    decodeMessageBody(size_t, Token &, size_t &)
                    throw (  Vocal::Http::Exception );

                                        
        Type            myType;
        Method          myMethod;
        Url             myUrl;
        Version         myVersion;
        Status          myStatus;
        list<Header>    myHeaders;
        string          myBody;

        string          myEncoded;
        bool            myDirty;
};


} // namespace Http
} // namespace Vocal


#endif // !defined(VOCAL_HTTP_MESSAGE_HXX)
