#if !defined(VOCAL_HTTP_METHOD_HXX)
#define VOCAL_HTTP_METHOD_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpMethod_hxx_Version =
    "$Id: HttpMethod.hxx,v 1.8 2002/03/19 20:17:22 jason Exp $";


#include "Http.hxx"
#include "HttpException.hxx"
#include "Writer.hxx"
#include <map>


/** Infrastructure common to VOCAL.<br><br>
 */
namespace Vocal 
{


/** Http Protocol Libraries<br><br>
 */
namespace Http
{


/** Implementation of RFC 2616, Section 5-1.<br><br>
 */
class Method : public Vocal::IO::Writer
{
    public:

        ///
      enum 
      {
         OPTIONS,
         GET,
         HEAD,
         POST,
         PUT,
         DELETE,
         TRACE,
         CONNECT,
         END
      };

        
        static const Token  TokenOPTIONS;
        static const Token  TokenGET;
        static const Token  TokenHEAD;
        static const Token  TokenPOST;
        static const Token  TokenPUT;
        static const Token  TokenDELETE;
        static const Token  TokenTRACE;
        static const Token  TokenCONNECT;
                
        /** Create the method given the type
         */
        Method(Type = Http::EXTENSION);


        /** Create the method given the token
         */
        Method(const Token &);
        

        /** Copy constructor
         */
        Method(const Method &);

    
        /** Virtual destructor
         */
        virtual ~Method();


        /** Assignment operator
         */
        const Method &  operator=(const Method &);
        
        
        /** Equality operator
         */
        bool    operator==(const Method &) const;


        /** Less than relational operator
         */
        bool    operator<(const Method &) const;


        /** Type accessor
         */
        Type    type() const;    


        /** Type mutator
         */
        void    type(Type);    


        /** Set the type given the token 
         */
        void    type(const Token &) throw ( Vocal::Http::Exception );
        
        
        /** Returns true if the status code is an extended code.
         */
        bool    isExtension() const;
        

        /** Provide the token for this method
         */
        const Token &   token() const;


        /** The total number of method types
         */
        static size_t   totalTypes();
        

        /** Write the method to a stream
         */
        ostream &       writeTo(ostream &) const;


    protected:


        void    init();
        

        /** Create the header type to token table
         */
        void    initTable();

        typedef map<Type, const Token *>   Table;
        
        static Table        myTable;


        /** Create the token to header type table 
         */
        void    initDecodeTable();

        typedef map<Token, Type>           DecodeTable;

        static DecodeTable  myDecodeTable;


        static size_t       myTotal; 
        
        Type                myType;
        Token               myToken;
};


} // namespace Http
} // namespace Vocal


#endif // !defined(VOCAL_HTTP_METHOD_HXX)
