/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpMethodTest_cxx_Version = 
    "$Id: HttpExceptionTest.cxx,v 1.4 2002/07/11 06:01:57 icahoon Exp $";


#include "Application.hxx"
#include "HttpException.hxx"
#include "Verify.hxx"


using namespace Vocal;
using Vocal::Http::Exception;


class TestApplication : public Application
{
    public:
        TestApplication() : myBytesRemaining(32) {}
        virtual ~TestApplication() {}
        
    	ReturnCode  run();
        
    private:
        
        void    boom(Exception::Error) throw ( Vocal::Http::Exception );
        
        size_t  myBytesRemaining;
};


class TestApplicationCreator : public ApplicationCreator
{
    Application * create () { return ( new TestApplication ); }
};


int main(int argc, char ** argv, char ** arge)
{
    TestApplicationCreator creator;
    Application::creator(&creator);
    ReturnCode count = Application::main(argc, argv, arge);

    return ( test_return_code(count) );
}


ReturnCode TestApplication::run()
{
    for ( Exception::Error i = 0; i < Http::Exception::total(); i++ )
    {   
        try 
        {
            boom(i);
        }
        catch ( Http::Exception & e )
        {
            cout << e.getName() << ": " << e.message() << endl;

            test_verify( e.error() == Exception::IncompleteInBody
                         ? e.bytesRemaining() == myBytesRemaining
                         : e.bytesRemaining() == 0 );
            test_verify( e.getName() == string("Vocal::Http::Exception") );
            test_verify( i == e.error() );
        } 
    }
    
    return ( Exception::total() * 3 );
}


void    
TestApplication::boom(Exception::Error err) throw ( Vocal::Http::Exception )
{
    Exception   e(err);
    e.bytesRemaining(myBytesRemaining);
    
    throw e;
}    
