/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpHeaderTest_cxx_Version = 
    "$Id: HttpHeaderTest.cxx,v 1.6 2002/07/11 06:01:57 icahoon Exp $";


#include "Application.hxx"
#include "HttpHeader.hxx"
#include "Verify.hxx"


using namespace Vocal;


class TestApplication : public Application
{
    public:
        TestApplication() {}
        virtual ~TestApplication() {}
        
    	ReturnCode  	run();
};


class TestApplicationCreator : public ApplicationCreator
{
    Application * create() { return ( new TestApplication ); }
};


int main(int argc, char ** argv, char ** arge)
{
    TestApplicationCreator creator;
    Application::creator(&creator);
    ReturnCode  count = Application::main(argc, argv, arge);
    return ( test_return_code(count) );
}


ReturnCode TestApplication::run()
{
    int test_count = 0;
    
    for ( Http::Type i = 0; i < Http::Header::totalTypes(); i++ )
    {
        Http::Header    h = Http::Header(i, "Test Value");
        
        cout << h << endl;
        
        test_verify( h.isExtension() == false );    ++test_count;
    }
    cout << endl;

    // Default ctor test
    //
    Http::Header    a(Http::Header::Accept);
    
    test_verify( a == a );                                  ++test_count;
    test_verify( !(a < a) );                                ++test_count;
    test_verify( a.type() == Http::Header::Accept );        ++test_count;
    test_verify( a.value() == string("") );                 ++test_count;
    test_verify( a.name() == Http::Header::TokenAccept );   ++test_count;
    test_verify( a.isGeneral() == false );                  ++test_count;
    test_verify( a.isRequest() == true );                   ++test_count;
    test_verify( a.isResponse() == false );                 ++test_count;
    test_verify( a.isEntity() == false );                   ++test_count;
    test_verify( a.isExtension() == false );                ++test_count;

    a = a;        

    test_verify( a == a );                                  ++test_count;
    test_verify( !(a < a) );                                ++test_count;
    test_verify( a.type() == Http::Header::Accept );        ++test_count;
    test_verify( a.value() == string("") );                 ++test_count;
    test_verify( a.name() == Http::Header::TokenAccept );   ++test_count;
    test_verify( a.isGeneral() == false );                  ++test_count;
    test_verify( a.isRequest() == true );                   ++test_count;
    test_verify( a.isResponse() == false );                 ++test_count;
    test_verify( a.isEntity() == false );                   ++test_count;
    test_verify( a.isExtension() == false );                ++test_count;

    // Token ctor
    //
    Http::Header    b(Http::Header::TokenVia);

    test_verify( !(b == a) );                               ++test_count;
    test_verify( b < a );                                   ++test_count;
    test_verify( b.type() == Http::Header::Via );           ++test_count;
    test_verify( b.value() == string("") );                 ++test_count;
    test_verify( b.name() == Http::Header::TokenVia );      ++test_count;
    test_verify( b.isGeneral() == true );                   ++test_count;
    test_verify( b.isRequest() == false );                  ++test_count;
    test_verify( b.isResponse() == false );                 ++test_count;
    test_verify( b.isEntity() == false );                   ++test_count;
    test_verify( b.isExtension() == false );                ++test_count;

    // Type / value ctor
    //
    Http::Header::Value     expiresValue("300");

    Http::Header    c(Http::Header::Expires, expiresValue);

    test_verify( !(c == a) );                               ++test_count;
    test_verify( b < c );                                   ++test_count;
    test_verify( c.type() == Http::Header::Expires );       ++test_count;
    test_verify( c.value() == expiresValue );               ++test_count;
    test_verify( c.name() == Http::Header::TokenExpires );  ++test_count;
    test_verify( c.isGeneral() == false );                  ++test_count;
    test_verify( c.isRequest() == false );                  ++test_count;
    test_verify( c.isResponse() == false );                 ++test_count;
    test_verify( c.isEntity() == true );                    ++test_count;
    test_verify( c.isExtension() == false );                ++test_count;


    // Token / value ctor
    //
    Http::Header::Value     locationValue("here");

    Http::Header    d(Http::Header::TokenLocation, locationValue);

    test_verify( !(d == a) );                               ++test_count;
    test_verify( d < c );                                   ++test_count;
    test_verify( d.type() == Http::Header::Location );      ++test_count;
    test_verify( d.value() == locationValue );              ++test_count;
    test_verify( d.name() == Http::Header::TokenLocation ); ++test_count;
    test_verify( d.isGeneral() == false );                  ++test_count;
    test_verify( d.isRequest() == false );                  ++test_count;
    test_verify( d.isResponse() == true );                  ++test_count;
    test_verify( d.isEntity() == false );                   ++test_count;
    test_verify( d.isExtension() == false );                ++test_count;


    // Copy ctor
    //
    Http::Header    e(d);

    test_verify( d == e );                                  ++test_count;
    test_verify( !(d == c) );                               ++test_count;
    test_verify( e < c );                                   ++test_count;
    test_verify( e.type() == Http::Header::Location );      ++test_count;
    test_verify( e.value() == locationValue );              ++test_count;
    test_verify( e.name() == Http::Header::TokenLocation ); ++test_count;
    test_verify( e.isGeneral() == false );                  ++test_count;
    test_verify( e.isRequest() == false );                  ++test_count;
    test_verify( e.isResponse() == true );                  ++test_count;
    test_verify( e.isEntity() == false );                   ++test_count;
    test_verify( e.isExtension() == false );                ++test_count;


    // Assignment operator
    //
    e = c;

    test_verify( !(d == e) );                               ++test_count;
    test_verify( e == c );                                  ++test_count;
    test_verify( d < e );                                   ++test_count;
    test_verify( e.type() == Http::Header::Expires );       ++test_count;
    test_verify( e.value() == expiresValue );               ++test_count;
    test_verify( e.name() == Http::Header::TokenExpires );  ++test_count;
    test_verify( e.isGeneral() == false );                  ++test_count;
    test_verify( e.isRequest() == false );                  ++test_count;
    test_verify( e.isResponse() == false );                 ++test_count;
    test_verify( e.isEntity() == true );                    ++test_count;
    test_verify( e.isExtension() == false );                ++test_count;
    

    // Check extensions
    //
    Http::Token     foobar("Foobar"), feh("Feh");

    Http::Header::Value foobarValue("This is the foobar header"),
                        fehValue("This is the feh header");
                        
    Http::Header    f(foobar, foobarValue),
                    g(feh, fehValue);
                    
    test_verify( f.isExtension() == true );                 ++test_count;
    test_verify( g.isExtension() == true );                 ++test_count;
    test_verify( f.name() == foobar );                      ++test_count;
    test_verify( g.name() == feh );                         ++test_count;
    test_verify( f.value() == foobarValue );                ++test_count;
    test_verify( g.value() == fehValue );                   ++test_count;
    test_verify( g < f );                                   ++test_count;
    test_verify( !(g == f) );                               ++test_count;


    // Check bogus type
    //
    f.type(734);
    
    test_verify( f.isExtension() == true );                 ++test_count;
    test_verify( f.name() == string("") );                  ++test_count;
    test_verify( f.value() == string("") );                 ++test_count;


    // Change name
    //
    f.type(Http::Header::Via);
    f.value("129.168.4.23");
    
    test_verify( f.isExtension() == false );                ++test_count;
    test_verify( f.name() == Http::Header::TokenVia );      ++test_count;
    test_verify( f.value() == string("129.168.4.23") );     ++test_count;

    f.value("1.2.3.4");
    
    test_verify( f.isExtension() == false );                ++test_count;
    test_verify( f.name() == Http::Header::TokenVia );      ++test_count;
    test_verify( f.value() == string("1.2.3.4") );          ++test_count;

    f.type("Server");
    
    test_verify( f.isExtension() == false );                ++test_count;
    test_verify( f.name() == "Server" );                    ++test_count;
    test_verify( f.value() == "" );                         ++test_count;
    
    f.type("Argh");
    f.value("Matey");

    test_verify( f.isExtension() == true );                 ++test_count;
    test_verify( f.name() == "Argh" );                      ++test_count;
    test_verify( f.value() == "Matey" );                    ++test_count;
    
    // Test creating an invalid header
    //
    Http::Header    z;
    try
    {
        z.type(string(""));
        test_verify(false);     ++test_count;
    }
    catch ( Http::Exception & e )
    {
        test_verify( e.error() == Http::Exception::BadHeader ); ++test_count;
    }
        
    return ( test_count );
}
