/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpVersionTest_cxx_Version = 
    "$Id: HttpVersionTest.cxx,v 1.5 2002/07/11 06:01:57 icahoon Exp $";


#include "Application.hxx"
#include "HttpVersion.hxx"
#include "Verify.hxx"


using namespace Vocal;


class TestApplication : public Application
{
    public:
        TestApplication() {}
        virtual ~TestApplication() {}
        
    	ReturnCode  	run();
};


class TestApplicationCreator : public ApplicationCreator
{
    Application * create() { return ( new TestApplication ); }
};


int main(int argc, char ** argv, char ** arge)
{
    TestApplicationCreator creator;
    Application::creator(&creator);
    ReturnCode count = Application::main(argc, argv, arge);
    
    return ( test_return_code(count) );
}


ReturnCode TestApplication::run()
{
    Http::Version   v11, v10(1,0), v623;
    
    v623.majorNumber(6);
    v623.minorNumber(23);
    
    test_verify( v11.majorNumber() == 1 && v11.minorNumber() == 1 );
    test_verify( v10.majorNumber() == 1 && v10.minorNumber() == 0 );
    test_verify( v623.majorNumber() == 6 && v623.minorNumber() == 23 );
    
    cout << Http::Version() << endl
         << Http::Version(1,0) << endl
         << Http::Version(6,23) << endl;

    Http::Version v2(v11);
    
    test_verify( v2 == v11 );
    test_verify( v2.majorNumber() == v11.majorNumber() );
    test_verify( v2.minorNumber() == v11.minorNumber() );
    
    v2 = v623;

    test_verify( v2 == v623 );
    test_verify( v2.majorNumber() == v623.majorNumber() );
    test_verify( v2.minorNumber() == v623.minorNumber() );
    
    v2 = v2;

    test_verify( v2 == v623 );
    test_verify( v2.majorNumber() == v623.majorNumber() );
    test_verify( v2.minorNumber() == v623.minorNumber() );

    test_verify( v10 < v623 );
    test_verify( !(v623 == v10 ) );
    test_verify( v623 == v623 );
    test_verify( !( v11 < v10 ) );    
    
    Http::Version   v3;
    string          mjr("3"), mnr("15");
    
    v3.majorNumber(mjr);
    v3.minorNumber(mnr);
    
    test_verify( v3.majorNumber() == 3 && v3.minorNumber() == 15 );

    return ( 17 );
}
