
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "LGParam.hxx"
#include <iostream>

LGParam::UserType LGParam::User = LGParam::NONE;      /// default user type
LGParam::Mode LGParam::Function = LGParam::ORIGINATING;/// default running mode
LGParam::TransportType LGParam::Transport = LGParam::UDP;/// default
static char * function[] = { "ORIGINATING",
			     "TERMINATING",
			     "REGISTRATION"};
int LGParam::TotalCalls     = -1;                    /// default infinite calls
int LGParam::CPS            = 10;                    /// default for 10cps
int LGParam::MaxRetry       = 8;                     /// default for 10cps
int LGParam::CallHoldTime   = 0;                     /// defualt call duration
int LGParam::RingTime       = 0;                     /// defualt ring duration
int LGParam::CleanUpTime    = 0;                     /// clean up asap
int LGParam::InviteTimeOut  = 32000;                 /// defualt time out 32sec
int LGParam::RequestTimeOut = 32000;                 /// defualt time out 32sec
int LGParam::BurstRate      = 0;                     /// default Burst ALL
int LGParam::UDPPort        = 5060;                  /// defualt SIP port
int LGParam::RTPPort        = 0;                     /// defualt RTP port
bool LGParam::SDPOff        = false;                 /// defualt include SDP
bool LGParam::RPort         = false;                 /// defualt don't put
bool LGParam::Random        = false;                 /// defualt don't put
char * LGParam::NextHopHost = 0;                     /// next hop
char * LGParam::NextHopPort = "5060";                ///  - ~~ -
//char * LGParam::URIHost = 0;                         /// req uri host
//char * LGParam::URIPort = "5060";                    ///  - ~~ - port
char * LGParam::FromHost    = 0;                     /// (from theSystem?)
char * LGParam::FromUrl     = 0;                     /// (default fromhost)
int    LGParam::FromUrlPort = 0;                     /// (default udpport)
char * LGParam::ToHost      = 0;
char * LGParam::ToPort      = "5060";
char * LGParam::SessionName = "ElGee Test Session";
char * LGParam::Secret      = 0;
Data * LGParam::MsgData     = 0;
char * LGParam::ContactHost = 0;
char * LGParam::ContactPort = 0;
char * LGParam::LeaseTime   = "3600";                /// default lease 3600 sec
/// default lease refresh mode
LGParam::RefreshType LGParam::Refresh = LGParam::OVERWRITE;


void
LGParam::print(ostream& os)
{
    os << function[LGParam::Function] << " mode configuration:"<< endl;
    if(LGParam::Secret)
    os << "\tSecret          = " << LGParam::Secret <<endl;
    os << "\tCPS             = " << LGParam::CPS <<endl;
    os << "\tMax Retry Count = " << LGParam::MaxRetry <<endl;
    os << "\tCall Burst Rate = ";
    if(LGParam::BurstRate)
	os << LGParam::BurstRate;
    else
	os << "ALL";
    os << " at a time" << endl;
    os << "\tREQUEST_TIMEOUT = " << (LGParam::RequestTimeOut/1000) <<endl;
    os << "\tNEXT_HOP        = " << LGParam::NextHopHost << ":"
       << LGParam::NextHopPort <<endl;
    os << "\tSOURCE_ADDRESS  = " << LGParam::FromHost << ":"
       << LGParam::UDPPort << endl;
    if(LGParam::FromUrl != LGParam::FromHost)
	    os << "\tFROM URL        = " << LGParam::FromUrl << ":"
	       << LGParam::FromUrlPort << endl;
    os << "\tTO_ADDRESS      = " << LGParam::ToHost << ":"
       << LGParam::ToPort <<endl;

    if(LGParam::Function != LGParam::TERMINATING)
	os << "\tTransport       = " <<
	    (LGParam::Transport == LGParam::UDP ? "UDP" : "TCP")
	   << endl;
    if(LGParam::User != LGParam::NONE)
	os << "\tUSER TYPE       = " <<
	    (LGParam::User == LGParam::IP ? "IP" :
	     (LGParam::User == LGParam::E164 ? "E164":"PHONE"))
	   << endl;
    switch(LGParam::Function)
    {
	case LGParam::ORIGINATING:
	    os << "\tRTP Port        = " << LGParam::RTPPort <<endl;
	    os << "\tCALL_HOLD       = " << LGParam::CallHoldTime/1000 <<endl;
	    os << "\tINVITE_TIMEOUT  = " << (LGParam::InviteTimeOut/1000)
	       <<endl;
	    os << "\tSDP_MEDIA       = " << (LGParam::SDPOff ? "OFF": "ON")
	       << endl;
	    os << "\tRPORT           = " << (LGParam::RPort ? "ON": "OFF")
	       << endl;
	    os << "\tRandom Mode     = " << (LGParam::Random ? "ON": "OFF")
	       << endl;
	    if(LGParam::MsgData)
	    {
            
		string s = LGParam::MsgData->logData();
		os << "\tSample Message:\n" << s  <<endl<<endl;
	    }
	    else
		os << "\tNO Sample Msg" <<endl;
	    break;

	case LGParam::TERMINATING:
	    os << "\tRING_TIME       = " << LGParam::RingTime/1000 <<endl;
	    os << "\tHOLD_TIME       = ";
	    if (LGParam::CallHoldTime)
	    {
	       	os << (LGParam::CallHoldTime/1000);
	    }
	    else
		os << "Infinit";
	    os << endl;
	    break;

	case LGParam::REGISTRATION:
	    os << "\tCONTACT HOST    = " << LGParam::ContactHost << endl;
	    os << "\tCONTACT PORT    = " << LGParam::ContactPort << endl;
	    os << "\tRefresh Mode    = " <<
		(LGParam::Refresh == LGParam::OVERWRITE ?
		 "OVERWRITE" : "REMOVE")
	       << endl;
	    os << "\tLEASE DURATION  = " << LGParam::LeaseTime << endl;
	    break;
    }
}
