
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "SipProxyEvent.hxx"
#include "TimerEvent.hxx"
#include "LGOperator.hxx"
#include "LGState.hxx"
#include "CallInfo.hxx"
#include "LGParam.hxx"
#include "OpChallenged.hxx"
#include "LGInfo.hxx"
#include "InviteMsg.hxx"
#include "RegisterMsg.hxx"
#include "Data.hxx"
#include "ByeMsg.hxx"
#include "SipEvent.hxx"
#include "TimerEvent.hxx"
#include "SipTransceiver.hxx"
#include "LGStateMcData.hxx"
#include "SipBasic.hxx"
#include "SipDigest.hxx"
#include "SipCHAP.hxx"
#include "SipProxyAuthorization.hxx"
#include "SipAuthorization.hxx"
#include "SipWwwAuthenticate.hxx"
#include "SipProxyAuthenticate.hxx"
#include "SipCallLeg.hxx"
#include "CallContainer.hxx"

LGState *
OpChallenged::process(const Sptr<SipProxyEvent>& event)
{
    cpLog(LOG_DEBUG_STACK,"Processing a challenge");
    Sptr<CallInfo> callInfo = event->getCallInfo();
    Sptr<LGStateMcData> lgData;
    Sptr<StatusMsg> response;
    Sptr<SipEvent> sipEvent;
    sipEvent.dynamicCast(event);
    response.dynamicCast(sipEvent->getSipMsg());
    if(lgData.dynamicCast(callInfo->getCallStateData())==0)
    {
	assert(0);
    }

    Sptr<SipCommand> request;
    if(LGParam::Function == LGParam::ORIGINATING)
    {
	Sptr<InviteMsg> tmpInvite;
	tmpInvite.dynamicCast(callInfo->getSipCommand());
	cpLog(LOG_DEBUG_STACK,"constructing the new invite");
	request = new InviteMsg(*tmpInvite);
    }
    else
    {
	Sptr<RegisterMsg> tmpRegister;
	tmpRegister.dynamicCast(callInfo->getSipCommand());
	cpLog(LOG_DEBUG_STACK,"constructing the new register");
	request = new RegisterMsg(*tmpRegister);
    }

/////set the authorization headers//////
    Data scheme, nonce, realm, domain, username, qop, algo, id, opaque;
    Data nc, cnonce;
    SipProxyAuthorization * authProxy = 0;
    SipAuthorization * auth = 0;
    if(response->getStatusLine().getStatusCode() == 407)
    {
	/// Proxy-Authorization...
	SipProxyAuthenticate challenge = response->getProxyAuthenticate();
	getAuthDetails(challenge,scheme,nonce,realm,domain,username,qop,
		       algo,id,opaque);
	authProxy = new SipProxyAuthorization(request->getProxyAuthorization());

	/// get old values and increment...
	if(qop.length())
	{
	    nc = authProxy->getTokenValue("nc");
	    char tmp[9], dBuf[256];
	    sprintf(tmp,"%08lx",(strtol(nc.getData(dBuf, 256),NULL,16)+1));
	    tmp[8]=0;
	    nc = tmp;
	    sprintf(tmp,"%08lx",random());
	    tmp[8]=0;
	    cnonce = tmp;
	}
	authProxy->setAuthScheme(scheme);
	authProxy->setTokenDetails(REALM, realm);
	cpLog(LOG_DEBUG_STACK, "Challenged from Proxy (Proxy-Authenticate)");
    }
    else if(response->getStatusLine().getStatusCode() == 401)
    {
	/// WWW-Authorization...
	SipWwwAuthenticate challenge = response->getWwwAuthenticate();
	getAuthDetails(challenge,scheme,nonce,realm,domain,username,qop,
		       algo,id,opaque);
	auth = new SipAuthorization(request->getAuthorization());

	/// get old values and increment...
	if(qop.length())
	{
	    nc = authProxy->getTokenValue("nc");
	    char tmp[9], dBuf[256];
	    sprintf(tmp,"%08lx",(strtol(nc.getData(dBuf, 256),NULL,16)+1));
	    tmp[8]=0;
	    nc = tmp;
	    sprintf(tmp,"%08lx",random());
	    tmp[8]=0;
	    cnonce = tmp;
	}
	auth->setAuthScheme(scheme);
	auth->setTokenDetails(REALM, realm);
	cpLog(LOG_DEBUG_STACK, "Challenged from Client (WWW-Authenticate)");
    }
    else
	assert(0);

    Data authResponse, uri, method;
    if(!username.length()) // i.e. if !CHAP ...
	username = request->getFrom().getUser();
    if(scheme.length())
    {
	if (scheme == AUTH_BASIC)
	{
	    /// do the basic authentication
	    if(authProxy)
		authProxy->setBasicCookie(
		    SipBasic().formSIPBasic(username,lgData->secret));
	    else auth->setBasicCookie(
		    SipBasic().formSIPBasic(username,lgData->secret));
	}
	else if (scheme == AUTH_DIGEST)
	{
	    /// do the digest authentication
	    SipRequestLine reqLine(request->getRequestLine());
	    method = reqLine.getMethod();
	    uri = reqLine.encode();
	    Data tmp = uri;
	    tmp.match("\r\n",&uri,false);
	    authResponse = SipDigest().form_SIPdigest(nonce, username,
						  lgData->secret, method,
						  uri, realm, qop, cnonce,
						  algo, nc);
	    if(authProxy)
	    {
		authProxy->setAuthScheme(scheme);
		authProxy->setTokenDetails(REALM, realm);
		authProxy->setTokenDetails("username",username);
		authProxy->setTokenDetails("nonce", nonce);
		authProxy->setTokenDetails("uri",uri);
		if(algo.length())
		    authProxy->setTokenDetails("algorithm",algo);
		if(qop.length())
		{
		    authProxy->setTokenDetails("qop",qop);
		    authProxy->setTokenDetails("cnonce",cnonce);
		    authProxy->setTokenDetails("nc",nc);
		}
		if(opaque.length())
		    authProxy->setTokenDetails("opaque",opaque);
		authProxy->setTokenDetails("response",authResponse);
	    }
	    else
	    {
		auth->setAuthScheme(scheme);
		auth->setTokenDetails(REALM, realm);
		auth->setTokenDetails("username",username);
		auth->setTokenDetails("nonce", nonce);
		auth->setTokenDetails("uri",uri);
		if(algo.length())
		    auth->setTokenDetails("algorithm",algo);
		if(qop.length())
		{
		    auth->setTokenDetails("qop",qop);
		    auth->setTokenDetails("cnonce",cnonce);
		    auth->setTokenDetails("nc",nc);
		}
		if(opaque.length())
		    auth->setTokenDetails("opaque",opaque);
		auth->setTokenDetails("response",authResponse);
	    }
	}
	else if (scheme == AUTH_CHAP)
	{
	    /// do the CHAP authentication
	    authResponse = SipCHAP::form_CHAP(id, lgData->secret, nonce);
	    if(authProxy)
	    {
		authProxy->setAuthScheme(scheme);
		authProxy->setTokenDetails("username",username);
		authProxy->setTokenDetails("nonce",nonce);
		authProxy->setTokenDetails("response",authResponse);
		if(id.length())
		    authProxy->setTokenDetails("id",id);
	    }
	    else
	    {
		auth->setAuthScheme(scheme);
		auth->setTokenDetails("username",username);
		auth->setTokenDetails("id",id);
		auth->setTokenDetails("nonce",nonce);
		auth->setTokenDetails("response",authResponse);
	    }

	}
	else if (scheme == AUTH_PGP)
	{
	    /// do the PGP authentication
	    /// ????
	    assert(0);
	}
	else
	    assert(0);
    }
    else
    {
	cpLog(LOG_ERR,"Invalid Authentication Response!!!");
	assert(0);
    }
    if(authProxy)
    {
	request->setProxyAuthorization(*authProxy);
	cpLog(LOG_DEBUG_STACK,"Completed challenge response [%s]",authProxy->encode()
	      .logData());
	delete authProxy;
    }
    else
    {
	request->setAuthorization(*auth);
	cpLog(LOG_DEBUG_STACK,"Completed challenge response [%s]",auth->encode()
	      .logData());
	delete auth;
    }

//////////////increase CSeq//////////////
    SipCSeq cseq = request->getCSeq();

    if(LGParam::Function == LGParam::REGISTRATION)
    {
	cpLog(LOG_DEBUG,"updating callinfo for new cSeq");
	Sptr<SipCallLeg> callLeg = callInfo->getSipCallLeg();
	callLeg->setCSeq(cseq);
	cpLog(LOG_DEBUG,"removing callleg entry from call container");
	Sptr<CallContainer> callContainer = event->getCallContainer();
	callContainer->removeCall(callLeg);
	cpLog(LOG_DEBUG,"done removing call leg entry");

//	static Sptr<TimerEvent> bogus = new TimerEvent();
//	callInfo->setTimer(bogus);
	callLeg = new SipCallLeg(request->computeCallLeg());
	cseq.incrCSeq();
	callLeg->setCSeq(cseq);
	callInfo = callContainer->addCall(callLeg,callInfo);
    }
    else
	cseq.incrCSeq();

    request->setCSeq(cseq);
    callInfo->setSipCommand(request);
    if(LGParam::Function == LGParam::ORIGINATING)
    {
	if(LGParam::InviteTimeOut)
	    callInfo->setTimer(this->setTimer(event, LGParam::InviteTimeOut));
    }
    else
	if(LGParam::RequestTimeOut)
	    callInfo->setTimer(this->setTimer(event, LGParam::RequestTimeOut));
    Sptr<SipTransceiver> sipStack = sipEvent->getSipStack();
    sipStack->sendAsync(request, LGParam::NextHopHost, LGParam::NextHopPort);
    cpLog(LOG_DEBUG_STACK,"done processing the authentication challenge");
    return nextState;
}

