
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "InviteMsg.hxx"
#include "RegisterMsg.hxx"
#include "SampleMsg.hxx"
#include "LGParam.hxx"
#include <string>
#include "SipMsg.hxx"
#include "Data.hxx"
#include "SipUrl.hxx"
#include "SipUser.hxx"
#include "SipTo.hxx"
#include "SipContact.hxx"
#include "SipRequestLine.hxx"
#include "SipFrom.hxx"
#include "SipSdp.hxx"
#include "cpLog.h"
#include <stdlib.h>
#include "SipSubject.hxx"
#include "SipUrl.hxx"

//// initialization...

SipCommand * SampleMsg::rawMsg = 0;
//UserGroup * SampleMsg::users = 0;
Mutex      SampleMsg::lock;

using Vocal::SDP::SdpSession;
using Vocal::SDP::SdpConnection;
using Vocal::SDP::SdpMedia;
using Vocal::SDP::MediaAttributes;
using Vocal::SDP::ValueAttribute;
using Vocal::SDP::SdpRtpMapAttribute;

using namespace Vocal;

static class SeedRandom
{
public:
    SeedRandom()
	{
	    srandom(time(NULL));
	}
} seed;


//// sdp data for dumb rtp rfc:pg137
/******************************************************************\
static Data sdpData =
"v=0
o=- 99513809 99513809 IN IP4 128.107.140.140
s=VOVIDA Session
c=IN IP4 128.107.140.140
t=962892393 0
m=audio 0 RTP/AVP 0
";
////a=rtpmap:0 PCMU/8000
\******************************************************************/

SampleMsg::MsgContainer*
SampleMsg::getNextMsg()
{
    if(!rawMsg)
    {
	lock.lock();
	if(!rawMsg)
	{
	    setRaw();
	}
	lock.unlock();
    }
    return setNext();
}

SampleMsg::MsgContainer*
SampleMsg::setNext()
{
    /// constructed new with To, From, contact, CallId and ReqUri
    /// to optimize(??) lazy!
    SipCommand * newMsg = 0;
    if(LGParam::Function == LGParam::ORIGINATING)
	newMsg = new InviteMsg(*dynamic_cast<InviteMsg*>(rawMsg));
    else
	newMsg = new RegisterMsg(*dynamic_cast<RegisterMsg*>(rawMsg));

    PairInfo * nextPair = LGUserGroup::instance()->nextPair();
    Data userType("phone");
    if(LGParam::User == LGParam::IP)
	    userType = "ip";
    else if(LGParam::User == LGParam::E164)
    {
	    char *tmp = nextPair->to;
	    nextPair->to = new char[strlen(tmp)+2];
	    nextPair->to[0]= '+';
	    strcpy(nextPair->to+1,tmp);
	    delete[] tmp;
    }

    Sptr<BaseUrl> reqUrl = newMsg->getRequestLine().getUrl();
    if(LGParam::Function == LGParam::ORIGINATING)
	    if(LGParam::User == LGParam::NONE)
		    reqUrl->setUser(new SipUser(nextPair->to));
	    else
		    reqUrl->setUserValue(nextPair->to, userType);
    else
	reqUrl->setUser(0);

    newMsg->getMutableRequestLine().setUrl(reqUrl);

    SipFrom from = newMsg->getFrom();
    from.setUser(nextPair->from);

    union {
	    unsigned char chars[4];
	    long ints;
    } tempTag;
    tempTag.ints = random();
    Data fromTag;
    fromTag = convertToHex(tempTag.chars, 4);
    from.setTag(fromTag);

    newMsg->setFrom(from);

    SipTo to = newMsg->getTo();
//    to.setUser(nextPair->to);
    reqUrl = to.getUrl();
    if(LGParam::User == LGParam::NONE)
	    reqUrl->setUser(new SipUser(nextPair->to));
    else
	    reqUrl->setUserValue(nextPair->to, userType);
    to.setUrl(reqUrl);
    newMsg->setTo(to);

/******moved back to setraw, 'coz theres nothing user specific here******
    SipContact contact;//newInvite->getContact();
    contact.setUrl(from.getUrl());
    newInvite->setContact(contact);
*************************************************************************/
    SipContact contact = newMsg->getContact();
    reqUrl = contact.getUrl();
    if(LGParam::Function == LGParam::ORIGINATING)
    {
	SipCallId callId;
	newMsg->setCallId( callId );

	/// this is to set user in contact
	reqUrl->setUser(new SipUser(nextPair->from));
    }
    else
    {
	/// this is to keep cseq's in order
	Data cseqval = nextPair->cseqCount;
/////// instead this has been added to callid below ///////
//	newMsg->setCSeq(SipCSeq(SIP_REGISTER, cseqval));
///////////////////////////////////////////////////////////

	//// this to maintain the same callid for each user's registration
	static Data callid = Data(random());
	newMsg->setCallId (callid+nextPair->to+"."+cseqval);

	/// this is to set user in contact
	if(LGParam::User == LGParam::NONE)
	    reqUrl->setUser(new SipUser(nextPair->to));
	else
	    reqUrl->setUserValue(nextPair->to, userType);
    }
    contact.setUrl(reqUrl);
    newMsg->removeContact();
    newMsg->setContact(contact);

    SampleMsg::MsgContainer* msg = new SampleMsg::MsgContainer();
    msg->nextMsg = newMsg;
    msg->secret = nextPair->secret;
    delete nextPair;
    return msg;
}


void
SampleMsg::setRaw()
{
    SipCommand * msg = 0;
    SipVia via;
    SipFrom sipfrom;
    SipTo sipto;
    if(LGParam::Function == LGParam::ORIGINATING)
	if(LGParam::MsgData)
	{
	    msg = new InviteMsg(*LGParam::MsgData);
	    via = msg->getVia(0);
	    sipfrom = msg->getFrom();
	    sipto = msg->getTo();
	}
	else msg = new InviteMsg();
    else
	msg = new RegisterMsg();

    /////    SipVia via = msg->getVia(0);
    via.setHost(LGParam::FromHost);
    via.setPort(LGParam::UDPPort);
    via.setRPort(LGParam::RPort);
    if(LGParam::Transport == LGParam::TCP)
	via.setTransport("TCP");
    msg->flushViaList();
    msg->setVia(via);
    
    //create the From header
    //////    SipFrom sipfrom = msg->getFrom();
    //getUserName, and getDisplayName are extern functions,
    //which contains the user's details. Set from command line?
//    Data user = theSystem.getUserName();
//    sipfrom.setUser(user);
//    Data dispName = theSystem.getDisplayName();
//    sipfrom.setDisplayName(dispName);
//    Data port = LGParam::UDPPort;
//    sipfrom.setHost( Data(theSystem.gethostAddress()) );
    sipfrom.setDisplayName("LG Sender");
    sipfrom.setHost(LGParam::FromUrl);
    sipfrom.setPort(LGParam::FromUrlPort);
//    sipfrom.setHost(LGParam::FromHost);
//    sipfrom.setPort(LGParam::UDPPort);
    msg->setFrom(sipfrom);

    //create the RequestLine.
    if(!LGParam::NextHopHost)
	assert(0);
    Sptr<BaseUrl> reqUrl;
    if(!LGParam::MsgData)
    {
///	reqUrl = new SipUrl(Data("sip:")+LGParam::URIHost+
///			    ":"+LGParam::URIPort);
	reqUrl = new SipUrl(Data("sip:")+LGParam::ToHost+
			    ":"+LGParam::ToPort);
        SipRequestLine reqLine;
	if(LGParam::Function == LGParam::ORIGINATING)
	    reqLine = SipRequestLine(SIP_INVITE, reqUrl);
	else
	    reqLine = SipRequestLine(SIP_REGISTER, reqUrl);
	switch(LGParam::Transport)
	{
	case LGParam::TCP :
	    reqLine.setTransportParam("TCP");
	    break;
	case LGParam::UDP :
	    reqLine.setTransportParam("UDP");
	    break;
	default:
	    break;
	}
        msg->setRequestLine(reqLine);
    }
    else
    {
        SipRequestLine& reqLine = msg->getMutableRequestLine();
///	reqLine.setHost(LGParam::URIHost);
///	reqLine.setPort(LGParam::URIPort);
	reqLine.setHost(LGParam::ToHost);
	reqLine.setPort(LGParam::ToPort);
    }

    //create the To header
//    reqUrl = new SipUrl(Data("sip:")+LGParam::ToHost+
//				      ":"+LGParam::ToPort);
    ///////    SipTo sipto = msg->getTo();
    sipto.setDisplayName("LG Receiver");
    sipto.setHost(LGParam::ToHost);
    sipto.setPortData(LGParam::ToPort);
    msg->setTo(sipto);

    //create the callId header
    SipCallId callId;
    msg->setCallId(callId);

    //create the CSeq header
    Data cseqval = "1";
    if(LGParam::Function == LGParam::ORIGINATING)
	msg->setCSeq(SipCSeq(SIP_INVITE, cseqval));
    else
	msg->setCSeq(SipCSeq(SIP_REGISTER, cseqval));

    //create the subject.
    SipSubject subject("VovidaMSG");
    msg->setSubject(subject);

/*********We don't set it here, do it while****************\
    //set Contact.  This is MUST in Msg.
    SipContact contact;
    contact.setUrl(sipfrom.getUrl());
    contact.setDisplayName(sipfrom.getDisplayName());
    msg->setContact(contact);
\**************setting the next msg.************************/
//    msg->flushContact();
    msg->removeContact();
    Sptr<SipUrl> contUrl = new SipUrl(Data("sip:")+LGParam::ContactHost+
				      ":"+LGParam::ContactPort);
    switch(LGParam::Transport)
    {
    case LGParam::TCP :
	contUrl->setTransportParam("tcp");
	break;
    case LGParam::UDP :
	contUrl->setTransportParam("udp");
	break;
    default:
	break;
    }
    SipContact contact;
    contact.setUrl(contUrl);
    if(LGParam::Function == LGParam::ORIGINATING)
    {
/////	contact.setUrl(sipfrom.getUrl());
//    msg->flushContentEncodingList();
//    msg->flushContentData();
	if(!LGParam::MsgData || LGParam::SDPOff)
	{
	    while(msg->getNumContentData())
		msg->removeContentData();
	}
	if(!LGParam::SDPOff && !msg->getNumContentData())
	{
	    SipContentType type(Data("application"), Data("SDP") );
	    
	    //create the sdp.
	    SdpSession sdpDesc;
	    
	    SdpConnection connection;
	    connection.setUnicast(LGParam::FromHost);
	    sdpDesc.setConnection(connection);
	    
	    SdpMedia* media = new SdpMedia;
	    media->setPort(LGParam::RTPPort); //// don't want any rtp!!!
	    
// 	    MediaAttributes * mediaAttrib = new MediaAttributes();
// 	    string attrStr("rtpmap:0 PCMU/8000");
// 	    mediaAttrib->setAttribute(attrStr);
// 	    media->setMediaAttributes(mediaAttrib);
	    
	    sdpDesc.addMedia(media);
	    srandom((unsigned int)time(NULL));
	    int verStr = random();
	    srandom((unsigned int)time(NULL));
	    int sessionStr = random();
	    sdpDesc.setVersion(verStr);
	    sdpDesc.setSessionId(sessionStr);
	    SipSdp sdp(sdpDesc);
	    sdp.setSessionName(LGParam::SessionName);
	    sdp.setAddress(LGParam::FromHost);
	    msg->setContentData(&sdp);
	}
    }
    else
    {
//////	contact.setUrl(new SipUrl(Data("sip:")+LGParam::ContactHost+
//////				  ":"+LGParam::ContactPort));
	contact.setExpires(SipExpires(LGParam::LeaseTime));

	SipExpires sipexpires;
	sipexpires.setDelta(Data(LGParam::LeaseTime));
	msg->setExpires(sipexpires);
    }
    msg->setContact(contact);
    
    /// to optimize(??) lazy!
    if(LGParam::Function == LGParam::ORIGINATING)
	rawMsg = new InviteMsg(msg->encode());
    else
	rawMsg = new RegisterMsg(msg->encode());

    cpLog(LOG_INFO,"gonna do\n%s\n",msg->encode().logData());
    delete msg;
    rawMsg->getRequestLine().getUrl();
    rawMsg->getFrom().getUser();
    rawMsg->getTo().getUser();
}
