
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const lz_cxx
= "$Id: lz.cxx,v 1.6 2002/08/01 22:40:37 bhadoria Exp $";

#include "ElGee.hxx"
#include "LGBuilder.hxx"
#include "LGInfo.hxx"
#include "LGParam.hxx"
#include "LGCommandLine.hxx"
#include <iostream>
#include <stdio.h>
#include "cpLog.h"
#include "changes.hxx"

int main(int c, char **v)
{
    char ver[256], * tmp = ver;
//    const char * curr = lz_cxx;
    const char * curr = lz_change_log;
    for(;*curr && *curr!= ',';curr++);
    for(int i=0; *(curr) && *(++curr) != '$' && i<4;)
    {
	if(*curr == ' ') i++;
        *(tmp++) = i<4 ? *curr : '\0';
    }
    *tmp = 0;
    
    cerr << "ElGee - The Loadgen! (" << ver << ")\n" << endl;
    if(!LGCommandLine::parse(c,v))
	return -1;

    LGBuilder builder;

    ElGee lz(&builder);
    
    lz.run();
    char comm;
    cout << v[0] << ">" << flush;
    int totCalls = LGParam::TotalCalls;
    while((comm=getchar())!='q')
    {
	while(getchar()!='\n'); /// eat up the extra \n
	switch (comm)
	{
	    case 's':
		LGInfo::print(cout);
		break;
	    case 'c':
		LGParam::print(cout);
		break;
	    case 'z':
		if(LGParam::TotalCalls)
		{
		    cout << "suspended new calls" << endl;
		    LGParam::TotalCalls = 0;
		}
		else
		{
		    cout << "resumed new calls" << endl;
		    LGParam::TotalCalls = totCalls;
		}
		break;
	    default:
		cout << "unknown command \"" << comm <<"\"" << endl;
		cout << "usage:" << endl;
		cout << "   s : to print status" << endl;
		cout << "   c : to print configuration" << endl;
		cout << "   z : suspend/resume new calls" << endl;
	}
	cout << v[0] << ">" << flush;
    }
    lz.shutdown();
//    return 0;
    lz.join();

    return 0;
}
