
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const SipUdp_impl_cxx_Version = 
"$Id: SipUdp_impl.cxx,v 1.1 2001/10/18 17:13:28 sprajpat Exp $";


#include <cassert>
#include <sys/time.h>
#include <string>
#include <stdlib.h>
#include "Sptr.hxx"
#include "SipUdp_impl.hxx"
#include "Fifo.h"
#include "LockHelper.hxx"
#include "SipMsg.hxx"
#include "StatusMsg.hxx"
#include "SipCommand.hxx"
/// this is a function to print messages
string printMsgBrief(Sptr<SipMsg> msg); 
 /*  retransmitOff from TranceiverSymbols.hxx */
bool SipUdp_impl::Udpretransmitoff = retransmitOff;
 /* retransmitTimeInitial from TranceiverSymbols.hxx */  
int SipUdp_impl::Udpretransmitimeinitial = retransmitTimeInitial; 
 /* retransmitTimeMax from TranceiverSymbols.hxx */
int SipUdp_impl::Udpretransmittimemax = retransmitTimeMax; 

SipUdp_impl::SipUdp_impl(Fifo< Sptr<SipMsg> >* fifo, int port /*default argument*/ )
    :
    randomLosePercent(0),
    udpStack( NULL, port ),
    recFifo(fifo),
    sendFifo(),
#ifdef USE_BIND
    shutdown(false),
    enablesrv(false)
#else
     shutdown(false)
#endif
{
    sendThread.spawn(sendThreadWrapper, this);
    receiveThread.spawn(rcvThreadWrapper, this);
    
}   

SipUdp_impl::~SipUdp_impl() 
{      
    shutdownMutex.lock();
    shutdown = true;
    shutdownMutex.unlock();
//insert shutdown message in the send fifo;;
    sendFifo.add(0);
    sendThread.join();
    receiveThread.join();
    fifoEventMapMutex.lock();

    FifoEventMap::iterator si;
    si  = fifoEventMap.begin( );
    
    while ( si != fifoEventMap.end() )
    {
        fifoEventMap.erase(si);
	si  = fifoEventMap.begin( );
    }

    si  = fifoStatusEventMap.begin( );
   
    while ( si != fifoStatusEventMap.end() )
    {
        fifoStatusEventMap.erase(si);
	si  = fifoStatusEventMap.begin( );
    }

    fifoEventMapMutex.unlock();
}

void*
SipUdp_impl::receiveMain()
{      
    fd_set netFD;
    NetworkAddress sender;
    while (1)
    {
        FD_ZERO (&netFD);
        int maxfd = udpStack.getSocketFD();
        int len = 0;
        int newlen = 0;	
        struct timeval timeout;
        timeout.tv_sec = 0;
        timeout.tv_usec = 10000;
        FD_SET(maxfd,&netFD);

        int selret = select(maxfd+1, &netFD, NULL, NULL, &timeout);
        if ( selret > 0) 
        {
            static const string::size_type MTU = 3600;
            char buf[MTU+1];
            buf[0] = '\0';
            bool readfd = FD_ISSET(udpStack.getSocketFD(), &netFD);
            
            if (readfd)
            { 
                len = udpStack.receiveFrom( buf,MTU , &sender );
                buf[len] = '\0';
            }
            
            newlen = strlen(buf);
            
            if (( newlen == 0) || ( newlen < 0 ))
            {
                return 0;
            }
            else if(recFifo->size() <= 100 )
            {
                assert( newlen > 0);
                buf[newlen] = '\0';

                
                Data data(buf, newlen);

                cpLog( LOG_DEBUG_STACK, "Received UDP Message \n%s" , 
		       data.getData());
                
                Sptr<SipMsg> sipMsg = SipMsg::decode(data);

                if(sipMsg != 0)
                {
                    sipMsg->setReceivedIPName( sender.getIpName() );

		    cpLog( LOG_DEBUG_STACK, "received message in stack: %s", 
			   printMsgBrief(sipMsg).c_str());

		    sipMsg->setReceivedIPName(sender.getIpName());
		    sipMsg->setReceivedIPPort(sender.getPort());

		    recFifo->add(sipMsg);
		    cpLog( LOG_DEBUG_STACK, "done adding to stack");

                }
		else
		{
		    // this message failed to decode
		    cpLog(LOG_DEBUG, 
			  "received message did not decode\n%s" , 
			  data.getData());
		}
                //get the fifo id and get the transaction id 
                //insert it into the map
            }
            else
            {
                cpLog(LOG_DEBUG,  "***Ignoring received msg **:%d" , recFifo->size());
            }
        }
        if ( selret < 0) 
        {
            if ( errno != EBADF && errno != EINTR )
            {
        	cpLog ( LOG_ALERT, 
			"select () failed, reason: %s", strerror(errno));
                assert(0);
            }
       }
       if ( selret == 0) 
       {
	    shutdownMutex.lock();
	    bool shouldShutdown = shutdown;
	    shutdownMutex.unlock();
	    if(shouldShutdown)
	    {
		return 0;
	    }
       }
    }
    return 0;
}

  
void
SipUdp_impl::cancel(const SipTransactionId& id, bool isStatus) 
{
    LockHelper lLock(fifoEventMapMutex);


    if(!isStatus)
    {
	// trying to cancel a request

	//lookup map, get fifo id, corresponding to transactionId.
	FifoEventMap::iterator i = fifoEventMap.find(id);
	
	if (i != fifoEventMap.end())
	{
	    FifoEventId eventId = i->second;
	    
	    //call cancel on this eventId.
	    sendFifo.cancel(eventId);
	    
	    //call erase on the transactionId in the fifo Event Map. 
	    fifoEventMap.erase(i);
	}
	else
	{
	    cpLog(LOG_DEBUG_STACK, "failed to cancel msg (id not found)");
	}
    }
    else
    {
	// this is a status msg we're trying to cancel
	// operate on the fifoStatusEventMap instead 

	FifoEventMap::iterator i = fifoStatusEventMap.find(id);
	
	if (i != fifoStatusEventMap.end())
	{
	    FifoEventId eventId = i->second;
	    
	    //call cancel on this eventId.
	    sendFifo.cancel(eventId);
	    
	    //call erase on the transactionId in the fifo Event Map. 
	    fifoStatusEventMap.erase(i);
	}
	else
	{
	    cpLog(LOG_DEBUG_STACK, "failed to cancel msg (id not found)");
	}
    }

      
}
    
bool 
SipUdp_impl::retransmission(Sptr<SipMsg> sipMessage) 
{
    if (Udpretransmitoff)
    {
    	return false;
    }
    bool retransmit = false;
    
    //retransmit only if status>=200 for INVITE, and for all requests.
    Sptr<StatusMsg> statusMsg;
    statusMsg.dynamicCast( sipMessage );
    
    
    Sptr<SipCommand> sipCommand;
    sipCommand.dynamicCast( sipMessage );
 
	if ( statusMsg != 0) 
	{
	    Data method = statusMsg->getCSeq().getMethod();
	    int statusCode = statusMsg->getStatusLine().getStatusCode();
	    if ( ( method == INVITE_METHOD) &&
		 ( statusCode >= 200)
		)
	    {
		retransmit = true;
		
	    }
	}
	else if ( ( sipCommand != 0) &&
		  ( sipCommand->getType() != SIP_ACK )
            )
	{
	        retransmit = true;
	    
	}
    return retransmit;
}


void*
SipUdp_impl::sendThreadWrapper(void *p) 
{
    return static_cast<SipUdp_impl*>(p)->sendMain();
}

    
int
SipUdp_impl::udpSend(Sptr<SipMsg> sipMessage, Data host, int port)
{
    int ret_status = 0;
    if ( port == 0 )
    {
    	port = SIP_PORT;
    	cpLog( LOG_DEBUG_STACK,"Changed port to %d",port);
    }
    NetworkAddress dest(host.getData(), port);
    
    cpLog( LOG_DEBUG, "Network Address host=%s:%d", dest.getIpName().c_str(),dest.getPort());
    
    
    Data data = sipMessage->encode();
    
    // define USE_VFILTER to use the new VFilter code, which allows
    // you to use perl to do translation.
    
    // read the comments in VFilter.hxx for usage information
    
#ifdef USE_VFILTER
    
    string dataString = data;
    string newData;
    newData = VFilter::outFilter(dataString);
    data = newData;
    
#endif

    cpLog(LOG_DEBUG_STACK, "Sending UDP message to %s:%d \n%s", 
	  host.getData(),port,data.getData());
    
    // send it 
    try
    {
    	int lngth = data.length(); 
    	ret_status = udpStack.transmitTo(data.getDataBuf(),lngth,&dest);
    }
    catch (UdpStackExceptionPortsInUse& udpExcep)
    {
    	cpLog(LOG_DEBUG_STACK,"All ports in use");
    }
    return ret_status;
} 
  
  
void*
SipUdp_impl::rcvThreadWrapper(void *p) 
{
    return  static_cast<SipUdp_impl*>(p)->receiveMain();
}       
    
#ifdef USE_BIND
void* 
SipUdp_impl::sendMain( )
{   
    unsigned fifoMaxSize = 100u;
    while(1) 
    {
	int ret_status = 0;
    	Sptr<RetransmitContents> retransmitt =  sendFifo.getNext();

	shutdownMutex.lock();
	bool shouldShutdown = shutdown;
	shutdownMutex.unlock();

	if(shouldShutdown)
	{
	    return 0;
	}

        if (retransmitt != 0)
    	{
    	    Sptr<SipMsg> sipMessage = retransmitt->getMsg();
    	    int count = retransmitt->getCount();
	    
            int timeToGo = retransmitt->getTimeToGo();
    	    if (sipMessage != 0)   
    	    {
		bool resendMsg = true;

		SipTransactionId id(*sipMessage);
    
                if (count)
		{
		    cpLog(LOG_DEBUG_STACK, "count = %d", count);

		    //get host, and port, and send it off.
		    Data host;
		    int port;
		    
		    //starting the loop for the dnscounts
		
		    Sptr<DnsResolver> dnsResolver = sipMessage->getDnsResolver();
		    int dnsCount = dnsResolver->getNumDnsRecords();
		    cpLog(LOG_DEBUG_STACK, "Dns Count ise %d", 
			  dnsCount);
		    
        	    bool condition = false;
		    int numCount = 0;
		    while ( (numCount < dnsCount  ) && ( condition == false) ) 
		    {
			// get the record containing the destination address
			Sptr<DnsRecord>* dnsRecord = dnsResolver->getRecord(dnsCount - 1 );
			Sptr<SipCommand> sCommand;
			sCommand.dynamicCast(sipMessage);
			Data address = (*dnsRecord)->getAddress();
                        (*dnsRecord)->printRecord();
			
			// setting the maddr of the SipUrl in the RequestLine
			Sptr<SipUrl> sipurl;
			sipurl.dynamicCast(sCommand->getRequestLine().getUrl());
			sipurl->setMaddrParam(address);
			SipRequestLine requestLine = sCommand->getRequestLine();
			requestLine.setUrl(sipurl);
			cpLog(LOG_DEBUG_STACK, "RequestLine of dns resolver is %s", 
			      requestLine.encode().getData());
			sCommand->setRequestLine(requestLine);
			
			
			getHostPort(sCommand, &host, &port);
			if(!randomLosePercent || 
			   ((random() % 100) >= randomLosePercent))
			{
			    //
			    ret_status = udpSend(sCommand , host, port);
			    if ( ret_status == 0) 
			    {
				condition= true;
			    }
			    else
			    if ( ret_status != 0)
			    {
				if ( ret_status == 1)
				{   
				//send 403 for ECONNREFUSED.
				    if  (sCommand != 0) 
				    {
					Sptr<StatusMsg> statusMsg 
					    = new StatusMsg((*sCommand), 403);
					recFifo->add(statusMsg);
					cpLog(LOG_INFO, "ECONNREFUSED! ");
					condition = true;
				    }
				    
				}
				
				else if ( ret_status == 2)
				{
				    Sptr<SipCommand> commandMsg;
				    commandMsg.dynamicCast(sipMessage);
				    
				//send 408 for EHOSTDOWN.
				    if  (commandMsg != 0) 
				    {
					if ( numCount == ( dnsCount -1))
					{
					    Sptr<StatusMsg> statusMsg 
						= new StatusMsg((*commandMsg), 408);
					    Sptr <SipMsg> sipMsg;
					    sipMsg.dynamicCast(statusMsg);
					    recFifo->add(statusMsg);
					    cpLog(LOG_INFO, "EHOSTDOWN! ");
					    condition = true;
					}
				    }
				}
				else if (ret_status == 3)
				{
				    Sptr<SipCommand> commandMsg;
				    commandMsg.dynamicCast(sipMessage);
				    
				//send 404 for EHOSTUNREACH.
				    if  (commandMsg != 0) 
				    {
					if ( numCount == (dnsCount-1 ))
					{
					    Sptr<StatusMsg> statusMsg 
						= new StatusMsg((*commandMsg), 404);
					    Sptr <SipMsg> sipMsg;
					    sipMsg.dynamicCast(statusMsg);
					    recFifo->add(statusMsg);
					    cpLog(LOG_INFO, "EHOSTUNREACH! ");
					    condition = true; 
					}
				    }
				}
			    }
			    
			}
			else
			{
			    cpLog(LOG_INFO, "XXX random lose: message not sent!");
			}
			
			numCount++;
		    } //while
                    //remove the FifoEvent Id from TransMap.
			fifoEventMapMutex.lock();

		    // need to decide if this is a status msg or not

		    if(sipMessage->getType() != SIP_STATUS)
		    {
			// not a status msg
			FifoEventMap::iterator i = fifoEventMap.find(id);
			if (i != fifoEventMap.end())
			{
			    //erase
			    fifoEventMap.erase(i);
			}
			else
			{
			    // if the event is NOT in the event map, do
			    // not send it, since that means this event
			    // has already been cancelled
			    resendMsg = false;
			}
		    }
		    else
		    {
			// is a status msg
			FifoEventMap::iterator i = fifoStatusEventMap.find(id);
			if (i != fifoStatusEventMap.end())
			{
			    //erase
			    fifoStatusEventMap.erase(i);
			}
			else
			{
			    // if the event is NOT in the event map, do
			    // not send it, since that means this event
			    // has already been cancelled
			    resendMsg = false;
			}
		    }
    
                    //update retransDetails

		    if(resendMsg)
		    {
                        int nCount = 0;
			if ( ret_status != 0)
			{
			    retransmitt->setCount(0);
			}
		
			    //decrement and set it back.
                            nCount = count - 1;
			    retransmitt->setCount(count - 1);

                            if(nCount == 0) 
                            {
                                fifoEventMapMutex.unlock();
                                continue;
                            }

			    //increment the time to send out.
			    if (timeToGo == 0)
			    {
				retransmitt->setTimeToGo(
				    retransmitTimeInitial);
			    }
			    else
			    {
				if(timeToGo < retransmitTimeMax)
				{
				    retransmitt->setTimeToGo(timeToGo * 2);
				}
			    }
			    
			    //add into Fifo.
			    FifoEventId eventid = 
				sendFifo.addDelayMs( retransmitt, 
						      retransmitt->getTimeToGo());
			    
			    //add into map.
			    if(sipMessage->getType() != SIP_STATUS)
			    {
				fifoEventMap[id] = eventid;
			    }
			    else
			    {
				fifoStatusEventMap[id] = eventid;
			    }

			    if(fifoEventMap.size() > fifoMaxSize)
			    {
				cpLog(LOG_WARNING, 
				      "fifoEventMap.size() > %d", fifoMaxSize);
				fifoMaxSize += 100;
			    }

			    if(fifoStatusEventMap.size() > fifoMaxSize)
			    {
				cpLog(LOG_WARNING, 
				      "fifoStatusEventMap.size() > %d", 
				      fifoMaxSize);
				fifoMaxSize += 100;
			    }
			
		    }
		    
		    fifoEventMapMutex.unlock();

		}
		else //if count == 0
		{
		    cpLog(LOG_DEBUG_STACK, "count is 0 for msg", count);
		    //take it out of map here too.
		    fifoEventMapMutex.lock();


		    if(sipMessage->getType() != SIP_STATUS)
		    {
			FifoEventMap::iterator i = fifoEventMap.find(id);
			if (i != fifoEventMap.end())
			{
			    fifoEventMap.erase(i);
			}
#if 0
			else
			{
			    cpLog(LOG_ALERT, 
				  "failed to erase from Map (id not found)");
			}
#endif
		    }
		    else
		    {
			FifoEventMap::iterator i = fifoStatusEventMap.find(id);
			if (i != fifoStatusEventMap.end())
			{
			    fifoStatusEventMap.erase(i);
			}
#if 1
			else
			{
			    cpLog(LOG_ALERT, 
				  "failed to erase from Map (id not found)");
			}
#endif
		    }


		    fifoEventMapMutex.unlock();

                    if (retransmission(sipMessage))
		    {
			Sptr<SipCommand> commandMsg;
			commandMsg.dynamicCast(sipMessage);
			
			//send 408 if a command, other than an ACK.
			if ( (commandMsg != 0) &&
			     (commandMsg->getType() == SIP_ACK) 
			    )
			{
			    Sptr<StatusMsg> statusMsg 
				= new StatusMsg((*commandMsg), 408);
			    Sptr <SipMsg> sipMsg;
			    sipMsg.dynamicCast(statusMsg);
			    recFifo->add(sipMsg);
			    // recFifo->add(statusMsg);
			}
		    }
		}
	    }//if sipMsg
	}//if retransmitt.
    }//end while
}
		    
#else
    
    
void* 
SipUdp_impl::sendMain( )
{   
    unsigned fifoMaxSize = 100u;
    while(1) 
    {
	int ret_status = 0;
    	Sptr<RetransmitContents> retransmitt =  sendFifo.getNext();

	shutdownMutex.lock();
	bool shouldShutdown = shutdown;
	shutdownMutex.unlock();

	if(shouldShutdown)
	{
	    return 0;
	}

        if (retransmitt != 0)
    	{
    	    Sptr<SipMsg> sipMessage = retransmitt->getMsg();
    	    int count = retransmitt->getCount();
	    
            int timeToGo = retransmitt->getTimeToGo();
    	    if (sipMessage != 0)   
    	    {
		bool resendMsg = true;

		SipTransactionId id(*sipMessage);
    
                if (count)
		{
		    cpLog(LOG_DEBUG_STACK, "count = %d", count);

		    //get host, and port, and send it off.
		    Data host;
		    int port;
		    
		    getHostPort(sipMessage, &host, &port);
		    if(!randomLosePercent || 
		       ((random() % 100) >= randomLosePercent))







		    {
			//
			ret_status = udpSend(sipMessage, host, port);
			if ( ret_status != 0)
			{
			    if ( ret_status == 1)




















			    {
				Sptr<SipCommand> commandMsg;
				commandMsg.dynamicCast(sipMessage);
				
				//send 403 for ECONNREFUSED.
				if  (commandMsg != 0) 
				{
				    Sptr<StatusMsg> statusMsg 
					= new StatusMsg((*commandMsg), 403);
				     Sptr <SipMsg> sipMsg;
				     sipMsg.dynamicCast(statusMsg);
				     recFifo->add(sipMsg);
				     //recFifo->add(statusMsg);
				    cpLog(LOG_INFO, "ECONNREFUSED! ");
				}
				
			    }
			
			    else if ( ret_status == 2)
			    {
				Sptr<SipCommand> commandMsg;
				commandMsg.dynamicCast(sipMessage);











				
				//send 408 for EHOSTDOWN.
				if  (commandMsg != 0) 
				{
				    Sptr<StatusMsg> statusMsg 
					= new StatusMsg((*commandMsg), 408);
				     Sptr <SipMsg> sipMsg;
				     sipMsg.dynamicCast(statusMsg);
				     recFifo->add(sipMsg);
				     //recFifo->add(statusMsg);
				    cpLog(LOG_INFO, "EHOSTDOWN! ");










				}
			    }
			    else if (ret_status == 3)
			    {
				Sptr<SipCommand> commandMsg;
				commandMsg.dynamicCast(sipMessage);
				
				//send 404 for EHOSTUNREACH.
				if  (commandMsg != 0) 
				{
				    Sptr<StatusMsg> statusMsg 
					= new StatusMsg((*commandMsg), 404);
				     Sptr <SipMsg> sipMsg;
				     sipMsg.dynamicCast(statusMsg);
				     recFifo->add(sipMsg);
				    //recFifo->add(statusMsg);
				    cpLog(LOG_INFO, "EHOSTUNREACH! ");
					  









				}
			    }
			}
			
			}
			else
			{
			    cpLog(LOG_INFO, "XXX random lose: message not sent!");
			}
		    


                    //remove the FifoEvent Id from TransMap.
                    fifoEventMapMutex.lock();

		    // need to decide if this is a status msg or not

		    if(sipMessage->getType() != SIP_STATUS)
		    {
			// not a status msg
			FifoEventMap::iterator i = fifoEventMap.find(id);
			if (i != fifoEventMap.end())
			{
			    //erase
			    fifoEventMap.erase(i);
			}
			else
			{
			    // if the event is NOT in the event map, do
			    // not send it, since that means this event
			    // has already been cancelled
			    resendMsg = false;
			}
		    }
		    else
		    {
			// is a status msg
			FifoEventMap::iterator i = fifoStatusEventMap.find(id);
			if (i != fifoStatusEventMap.end())
			{
			    //erase
			    fifoStatusEventMap.erase(i);
			}
			else
			{
			    // if the event is NOT in the event map, do
			    // not send it, since that means this event
			    // has already been cancelled
			    resendMsg = false;
			}
		    }
    
                    //update retransDetails

		    if(resendMsg)
		    {
                        int nCount = 0;
			if ( ret_status != 0)
			{
			    retransmitt->setCount(0);
			}
		
			    //decrement and set it back.
                            nCount = count - 1;
			    retransmitt->setCount(count - 1);

                            if(nCount == 0) 
                            {
                                fifoEventMapMutex.unlock();
                                continue;
                            }

			    //increment the time to send out.
			    if (timeToGo == 0)
			    {
				retransmitt->setTimeToGo(
				    retransmitTimeInitial);
			    }
			    else
			    {
				if(timeToGo < retransmitTimeMax)
				{
				    retransmitt->setTimeToGo(timeToGo * 2);
				}
			    }
			    
			    //add into Fifo.
			    FifoEventId eventid = 
				sendFifo.addDelayMs( retransmitt, 
						      retransmitt->getTimeToGo());
			    
			    //add into map.
			    if(sipMessage->getType() != SIP_STATUS)
			    {
				fifoEventMap[id] = eventid;
			    }
			    else
			    {
				fifoStatusEventMap[id] = eventid;
			    }

			    if(fifoEventMap.size() > fifoMaxSize)
			    {
				cpLog(LOG_WARNING, 
				      "fifoEventMap.size() > %d", fifoMaxSize);
				fifoMaxSize += 100;
			    }

			    if(fifoStatusEventMap.size() > fifoMaxSize)
			    {
				cpLog(LOG_WARNING, 
				      "fifoStatusEventMap.size() > %d", 
				      fifoMaxSize);
				fifoMaxSize += 100;
			    }
			
		    }
		    
		    fifoEventMapMutex.unlock();

		}
		else //if count == 0
		{
		    cpLog(LOG_DEBUG_STACK, "count is 0 for msg", count);
		    //take it out of map here too.
		    fifoEventMapMutex.lock();


		    if(sipMessage->getType() != SIP_STATUS)
		    {
			FifoEventMap::iterator i = fifoEventMap.find(id);
			if (i != fifoEventMap.end())
			{
			    fifoEventMap.erase(i);
			}
#if 0
			else
			{
			    cpLog(LOG_ALERT, 
				  "failed to erase from Map (id not found)");
			}
#endif
		    }
		    else
		    {
			FifoEventMap::iterator i = fifoStatusEventMap.find(id);
			if (i != fifoStatusEventMap.end())
			{
			    fifoStatusEventMap.erase(i);
			}
#if 1
			else
			{
			    cpLog(LOG_ALERT, 
				  "failed to erase from Map (id not found)");
			}
#endif
		    }


		    fifoEventMapMutex.unlock();

                    if (retransmission(sipMessage))
		    {
			Sptr<SipCommand> commandMsg;
			commandMsg.dynamicCast(sipMessage);
			
			//send 408 if a command, other than an ACK.
			if ( (commandMsg != 0) &&
			     (commandMsg->getType() == SIP_ACK) 
			    )
			{
			    Sptr<StatusMsg> statusMsg 
				= new StatusMsg((*commandMsg), 408);
			    Sptr <SipMsg> sipMsg;
			    sipMsg.dynamicCast(statusMsg);
			    recFifo->add(sipMsg);
			    // recFifo->add(statusMsg);
			}
		    }
		}
	    }//if sipMsg
	}//if retransmitt.
    }//end while
}
    
    
    
    
#endif
    
    
void 
SipUdp_impl::getHostPort(Sptr<SipMsg> sipMessage, Data* host, int* port)
{       
    if ( sipMessage->getType() == SIP_STATUS  )
    {
    	assert( sipMessage->getNumVia() > 0 );
	
    	if ( sipMessage->getNumVia() > 0 )
    	{
    	    SipVia via = sipMessage->getVia( 0 );
	    // check for viaReceived parameter in the via header
	    if ( via.isViaReceived() == true)
	    {
		string netadd = via.getReceivedhost();
		NetworkAddress anetaddress(netadd);	
		*host = anetaddress.getIpName();
                *port = via.getReceivedport();
		//*port = sipMessage->getReceivedIPPort();
	    } 
	    else
	    {
		*host = via.getHost();
		*port = via.getPort();
		cpLog( LOG_DEBUG_STACK,
		       "got host from Via of %s",host->getData());
		cpLog( LOG_DEBUG,"got port from Via of %d",*port);
	    }
	}
    }
    else 
    {
        Sptr<SipCommand> command;
        command.dynamicCast(sipMessage);
        if ( command != 0)
        {
            Sptr <BaseUrl> dest;
            dest = command->getRequestLine().getUrl();
            
            SipRequestLine reqLine = command->getRequestLine();
            reqLine.setUrl(dest);
            command->setRequestLine(reqLine);
            
	    if (dest.getPtr() != 0)
	    {
		if (dest->getType() == SIP_URL)
		{
		    Sptr<SipUrl> sipUrl;
		    Data hostport = (sipUrl.dynamicCast(dest))->getMaddrParam();
		    if ( hostport )
		    {
			//need to clarify this whether the port should be a default port because the maddr in SipUrl can only be a host
			    *host = hostport;
			    *port = Data(DEFAULT_SIP_PORT);
		
			
		    }
		    else
		    {
			Sptr <SipUrl> sipUrl;
			*host = (sipUrl.dynamicCast(dest))->getHost();
			*port = (sipUrl.dynamicCast(dest))->getPort();
		    }
		}
	    }
            
            cpLog( LOG_DEBUG, "got host from start line: %s:%d", 
		   host->getData(), *port);
        }
        else
        {
            assert(0);
        }
    }//if sipCommand
}

#ifdef USE_BIND
/* Sets the map and the retransmitt contents */
void* 
SipUdp_impl::send(Sptr<SipMsg> msg)
{ 

    SipTransactionId id(*msg);
    Sptr<RetransmitContents> retransDetails = new RetransmitContents;
    Sptr<SipUrl> sipurl;
    // if the message is a siprequest then set the dns resolver object
    if ( msg->getType() != SIP_STATUS)
    {
	Sptr<SipCommand> sipCommand;
	sipCommand.dynamicCast(msg);

	// instantiate the dnsresolver object and set it in the sipmessage. 
	// change it by passing the information from the transceiver to use the srv or the A option . For testing this is hardcoded for now. 
    
	Sptr<DnsResolver> dnsResolve = new DnsResolver( (sipurl.dynamicCast ( sipCommand->getRequestLine().getUrl())->getHost().getData()), ns_t_srv,_SIP , _UDP);
        cpLog(LOG_DEBUG_STACK, 
	      "DNS RESOLVER OBJECT IS  " 
	    );
	sipCommand->setDnsResolver(dnsResolve);
	retransDetails->setMsg(sipCommand);
    }
    else
    {
	retransDetails->setMsg(msg);
    }
     retransDetails->setTimeToGo(0);
    
    //if this has to be retransmitted.
    if (retransmission(msg))
    {

        cpLog(LOG_DEBUG_STACK, 
	      "$$$$$$$$$$$$$$$Msg is for retransmission: %s", 
	      msg->encode().getData());
        retransDetails->setCount( sipmaxRetrans+ 1);
    }
    else
    {
	//got to change it to 1
	retransDetails->setCount(1);

    }
    
    fifoEventMapMutex.lock();

    //send immediately.
    FifoEventId eventid = sendFifo.addDelayMs( retransDetails , 0);
    
    if(msg->getType() != SIP_STATUS)
    {
	fifoEventMap[id] = eventid;
    }
    else
    {
	fifoStatusEventMap[id] = eventid;
    }


    fifoEventMapMutex.unlock();
    
    return 0;
}
#else
//ifdef USE_BIND
/* Sets the map and the retransmitt contents */
void* 
SipUdp_impl::send(Sptr<SipMsg> msg)
{ 
    
    SipTransactionId id(*msg);
    Sptr<RetransmitContents> retransDetails = new RetransmitContents;







    retransDetails->setMsg(msg);
    retransDetails->setTimeToGo(0);













    
    //if this has to be retransmitted.
    if (retransmission(msg))
    {

        cpLog(LOG_DEBUG_STACK, 
	      "$$$$$$$$$$$$$$$Msg is for retransmission: %s", 
	      msg->encode().getData());
        retransDetails->setCount( sipmaxRetrans+ 1);
    }
    else
    {
	//got to change it to 1
	retransDetails->setCount(1);

    }
    
    fifoEventMapMutex.lock();

    //send immediately.
    FifoEventId eventid = sendFifo.addDelayMs( retransDetails , 0);
    
    if(msg->getType() != SIP_STATUS)
    {
	fifoEventMap[id] = eventid;
    }
    else
    {
	fifoStatusEventMap[id] = eventid;
    }


    fifoEventMapMutex.unlock();
    
    return 0;
}
    
     
    
#endif
    
void
SipUdp_impl::reTransOn()
{
    Udpretransmitoff = false;
}
    


void
SipUdp_impl::reTransOff()
{
    Udpretransmitoff = true;
}


void
SipUdp_impl::setRetransTime(int initial, int max)
{
    Udpretransmitimeinitial = initial;
    Udpretransmittimemax = max;
}


void
SipUdp_impl::setRandomLosePercent(int percent)
{
    if((percent >=0) && (percent <= 100))
    {
	randomLosePercent = percent;
    }
}


void
SipUdp_impl::printSize()
{
    LockHelper lock(fifoEventMapMutex);

    cpLog(LOG_INFO, 
	  "fifoEventMap: %d fifoStatusEventMap: recFifo: %d, sendFifo: %d", 
	  fifoEventMap.size(),
	  fifoStatusEventMap.size(),
	  recFifo->size(),
	  sendFifo.size());
}

#ifdef USE_BIND
void SipUdp_impl::enableSrv()
{
    enablesrv = true;
}
#endif
//End of File
