
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const OSPServerData_cxx_Version =
    "$Id: OSPServerData.cxx,v 1.8 2000/12/18 23:42:51 bko Exp $";
#include "VIoException.hxx"
#include "OSPServerData.hxx"
#include "PolicyCfg.hxx"


///
OSPServerData::OSPServerData(const string& path, const string& fileName)
{
    VFile vf( path, fileName);
    parse( vf );
}

///
OSPServerData& OSPServerData::operator=(const OSPServerData& other)
{
    if (this != &other)
    {
        _numServicePoints = other._numServicePoints;
        _servicePoint = other._servicePoint;    // copy the service point urls
        _localValidation = other._localValidation;
        _sslLifeTime = other._sslLifeTime;
        _httpMaxConnections = other._httpMaxConnections;
        _httpPersistence = other._httpPersistence;
        _httpRetryDelay = other._httpRetryDelay;
        _httpRetryLimit = other._httpRetryLimit;
        _httpTimeout = other._httpTimeout;

        _customExt = other._customExt;
        _customerID = other._customerID;
        _deviceID = other._deviceID;

        _caEnrollmentServer = other._caEnrollmentServer;
        _caEnrollmentPort = other._caEnrollmentPort;

        _auditURL = other._auditURL;
    }
    return (*this);
}

///
ostream& operator<<(ostream& os, const OSPServerData& data)
{
    os << "OSP Server Data:- # Service Points:" << data.getNumServicePoints() << endl;
    int i = 1;
    for ( list < string > ::const_iterator itr = data._servicePoint.begin();
            itr != data._servicePoint.end(); itr++ )
    {
        os << "Service Point " << i << ":" << (*itr).c_str() << endl;
        i++;
    }
    os << "Local Validation:" << data.getLocalValidation() << endl;
    os << "SSL LifeTime:" << data.getSslLifeTime() << endl;
    os << "HTTP Max Connections:" << data.getHttpMaxConnections() << endl;
    os << "HTTP Persistence:" << data.getHttpPersistence() << endl;
    os << "HTTP Retry Delay: " << data.getHttpRetryDelay() << endl;
    os << "HTTP Retry Limit:" << data.getHttpRetryLimit() << endl;
    os << "HTTP Timeout: " << data.getHttpTimeout() << endl;
    os << "Customer Extension:" << (int) data.getCustomExt() << endl;
    os << "Customer ID:" << data.getCustomerID() << endl;
    os << "Device ID:" << data.getDeviceID() << endl;
    os << "CA Enrollment Server:" << data.getCaEnrollmentServer() << endl;
    os << "CA Enrollment Port:" << data.getCaEnrollmentPort() << endl;
    os << "Audit URL:" << data.getAuditURL() << endl;

    return os;
}

///
void OSPServerData::parse( VXmlParser& parser )
{
    bool servicePointChanged = false;
    list < string > currServicePoint = _servicePoint;
    // extract info needed by the OSP Provider:

    _numServicePoints = atoi(
                            parser.getContent("OSPServer", "numServicePoints").c_str());

    xmlNodePtr root = parser.findNode(NULL, "OSPServer");
    if (root != NULL)
    {
        parser.getContent(root, "servicePointURL", "", _servicePoint);
        if (_servicePoint.size() != currServicePoint.size())
        {
            servicePointChanged = true;
        }
        else
        {
            list < string > ::iterator itr = _servicePoint.begin();
            for (list < string > ::iterator itr2 = currServicePoint.begin();
                    itr2 != currServicePoint.end(); itr2++)
            {
                if ((*itr) != (*itr2))
                {
                    servicePointChanged = true;
                    break;
                }
                itr++;
            }
        }
    }
    else
    {
        cpLog( LOG_ALERT, "parse() cannot findNode() OSPServer");
    }
    _localValidation = atoi(
                           parser.getContent("OSPServer", "localValidation").c_str());
    _sslLifeTime = atoi(
                       parser.getContent("OSPServer", "SSLLifetime").c_str()) / 1000;
    _httpMaxConnections = atoi(
                              parser.getContent("OSPServer", "httpMaxConnections").c_str());
    _httpPersistence = atoi(
                           parser.getContent("OSPServer", "httpPersistence").c_str()) / 1000;
    _httpRetryDelay = atoi(
                          parser.getContent("OSPServer", "httpRetryDelay").c_str()) / 1000;
    _httpRetryLimit = atoi(
                          parser.getContent("OSPServer", "httpRetryLimit").c_str());
    _httpTimeout = atoi(
                       parser.getContent("OSPServer", "httpTimeout").c_str());

    _customExt = (CUSTOM_EXTENSION) atoi( parser.getContent("OSPServer", "OSPExtension").c_str());
    if (_customExt == TRANSNEXUS)    // TransNexus specific data
    {
        _customerID = parser.getContent("OSPServer", "customerID");
        _deviceID = parser.getContent("OSPServer", "deviceID");
    }

    _auditURL = parser.getContent("OSPServer", "auditURL");

    ///If any of the service points changed, we do not flag the the policy cfg changed
    ///For those case the Policy Server should be restarted
    if (!servicePointChanged)
    {
        PolicyCfg::Instance().changed(!servicePointChanged);
    }
}

// end of file
