
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const OSPTime_cxx_Version =
    "$Id: OSPTime.cxx,v 1.14 2001/11/05 18:07:37 icahoon Exp $";
#include <assert.h>
#include "cpLog.h"
#include "OSPTime.hxx"

const int OSPTime::_DaysAtMonth[] =
    {
        0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
    };

const int OSPTime::BASE_YEAR = 1900;

const int OSPTime::BASE_DATE_IN_DAYS = 719528;

const string OSPTime::OSP_TS = "YYYY-MM-DDThh:mm:ssZ";

const string OSPTime::OSP_TS_FORMAT = "%Y-%m-%dT%H:%M:%SZ";



///
// get current time
time_t OSPTime::getCurrentTime( void )
{
    return ( time(0) );
}

///
//  Convert OSP formatted string to calendar time (in UTC)
time_t OSPTime::timeStringToTime_t( const char * ts )
{
    assert( ts );
    time_t tm = 0;
    //  Format of an OSP disply time stamp: YYYY-MM-DDThh:mm:ssZ
    //                          ts[] index: 01234567890123456789
    int year, month, day, hour, minute, second;
    char str[ 8 ];
    memset( str, 0, sizeof(str));
    strncpy( str, ts, 4 );
    year = atoi( str );
    strncpy( str, &ts[5], 2 );
    str[2] = '\0';
    month = atoi( str );
    strncpy( str, &ts[8], 2 );
    str[2] = '\0';
    day = atoi( str );
    strncpy( str, &ts[11], 2 );
    str[2] = '\0';
    hour = atoi( str );
    strncpy( str, &ts[14], 2 );
    str[2] = '\0';
    minute = atoi( str );
    strncpy( str, &ts[17], 2 );
    str[2] = '\0';
    second = atoi( str );

    cpLog(LOG_DEBUG, "mon=%d, day=%d, y=%d, hh=%d, mm=%d, ss=%d",
          month, day, year, hour, minute, second );
    tm = timeToGMT( month, day, year, hour, minute, second );
    return ( tm );
}   // OSPTime::timeStringToTime_t()

/// Input: (broken time)
//  Month (1-12!), Day of month, Year, hh, mm, ss
time_t OSPTime::timeToGMT(
    int month, int day, int year, int hour, int minute, int second )
{
    cpLog(LOG_DEBUG, "timeToGMT: mon=%d, day=%d, y=%d, hh=%d, mm=%d, ss=%d",
          month, day, year, hour, minute, second );
    time_t days = 0;   // day of the year
    // figure out the n-th day of the year:
    days = (year * 365) + (year / 4)-(year / 100) + (year / 400) + _DaysAtMonth[month - 1] + day;
    if ((month < 3) && OSPTime::isLeapYear( year ))   // Jan. or Feb.?
    {
        days--;
    }
    days -= BASE_DATE_IN_DAYS;    // subtract out the base date: Jan. 1, 1970
    // return time since epoch
    return ( (days * 86400) + (hour * 3600) + (minute * 60) + second);

}

///
int OSPTime::time_T2GMTString( time_t time, string& fmtStr )
{
    struct tm * t;
    int errCode = 0;
    int numChars = 0;
    int sz = OSPTime::OSP_TS.length() + 1;
    char str[32];
    memset( str, 0, sizeof(str) );

    t = gmtime( &time );    // convert UTC time to struct tm (in static data area)

    if ( (numChars = strftime( str, sz, OSP_TS_FORMAT.c_str(), t ))
            == (int) sz - 1 )
    {
        fmtStr.assign( str, sz );
        cpLog(LOG_DEBUG, "time_T2GMTString() - t=%lu, fmStr=|%s|",
              time, fmtStr.c_str());
    }
    else
    {
        errCode = -1;
        cpLog(LOG_DEBUG,
              "time_T2GMTString() - strftime() error, t=%lu, nc=%d, str=%s",
              time, numChars, str);
    }
    return ( errCode );
}

// end of file
