#ifndef  PDPServerData_hxx
#define  PDPServerData_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const PDPServerData_hxx_Version =
    "$Id: PDPServerData.hxx,v 1.9 2000/12/18 23:42:51 bko Exp $";



#include <string>

#include "VXmlParser.hxx"
#include "VFile.hxx"
#include "cpLog.h"

class PDPServerData
{
    public:
        ///
        PDPServerData(void)
        {
            init();
        }

        ///
        PDPServerData(const VFile& xmlFile)
        {
            init();
            parse( xmlFile );
        }

        ///
        PDPServerData( VXmlParser& parser )
        {
            init();
            parse( parser );
        }

        ///
        void init( void )
        {
            _host = "uninitialized";
            _maxThreads = 8;
            ///Fixed port for COPS
            _port = 3288;
            _keepAliveTimer = _acctTimer = 0;
        }

        ///
        PDPServerData(const string& path, const string& fileName);

        ///
        PDPServerData& operator=(const PDPServerData& other)
        {
            if (this != &other)
            {
                _host = other._host;
                _port = other._port;
                _maxThreads = other._maxThreads;
                _keepAliveTimer = other._keepAliveTimer;
                _acctTimer = other._acctTimer;
            }
            return (*this);
        }

        ///
        friend ostream& operator<<(ostream& os, const PDPServerData& data);

        ///
        const string& getHost(void ) const
        {
            return _host;
        };
        ///
        int getPort(void ) const
        {
            return _port;
        };
        ///
        int getMaxThreads(void ) const
        {
            return _maxThreads;
        };
        ///
        int getKeepAliveTimer( void ) const
        {
            return _keepAliveTimer;
        }
        ///
        int getAcctTimer( void ) const
        {
            return _acctTimer;
        }

        ///
        void parse(VXmlParser& parser);
        ///
        void parse(const VFile& vf)
        {
            VXmlParser parser( vf );
            parse( parser );
        }

    private:
        string _host;
        int _port;                        // default = 3288
        int _maxThreads;                  // in ms
        int _keepAliveTimer;              // in ms
        int _acctTimer;                   // in ms
}
;

///
inline ostream& operator<<(ostream& os, const PDPServerData& data)
{
    os << "PDPServer:- Host:" << data.getHost() << endl;
    os << "Port:" << data.getPort() << endl;
    os << "maxThreads:" << data.getMaxThreads() << endl;
    os << "keepAliveTimer:" << data.getKeepAliveTimer() << endl;
    os << "accTimer:" << data.getAcctTimer() << endl;
    return os;
}

#endif
