#ifndef COPSClientAcceptMsg_hxx
#define COPSClientAcceptMsg_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSClientAcceptMsg_hxx_Version =
    "$Id: COPSClientAcceptMsg.hxx,v 1.9 2000/12/18 23:42:53 bko Exp $";

#include "COPSMsg.hxx"

#include "COPSHeader.hxx"
#include "COPSKATimer.hxx"
#include "COPSAcctTimer.hxx"
#include "COPSIntegrity.hxx"

///COPSClientAcceptMsg

class COPSClientAcceptMsg : public COPSMsg
{
    public:
        ///Constructor
        COPSClientAcceptMsg();

        ///
        virtual ~COPSClientAcceptMsg();


        /** Checks the sanity of COPS message and throw an
          * COPSBadDataException when data is bad.
          */
        void checkSanity() const throw (COPSBadDataException&);

        ///Add message header
        COPSClientAcceptMsg& operator<<(const COPSHeader& obj);

        ///Add Timer object to the message
        COPSClientAcceptMsg& operator<<(const COPSTimer& obj);

        ///Add Integrity objects
        COPSClientAcceptMsg& operator<<(const COPSIntegrity& obj);

        ///Dumps the whole object to a output stream
        void dump(ostream& os) const;

        ///
        string className() const
        {
            return "COPSClientAcceptMsg";
        };

        ///
        const Sptr < COPSKATimer > & getKATimer() const
        {
            return _kaTimer;
        };

        ///returns true if has a account timer object
        bool hasAcctTimer() const
        {
            return (_acctTimer != 0);
        };

        ///Should check hasAcctTimer() before calling
        const Sptr < COPSAcctTimer > & getAcctTimer() const
        {
            return (_acctTimer);
        }

        ///Returns true if has a Integrity object
        bool hasIntegrity() const
        {
            return (_integrity.getPtr() != 0);
        };

        ///Should check hasIntegrity() before calling
        const Sptr < COPSIntegrity > & getIntegrity() const
        {
            return (_integrity);
        }

        ///Writes data to a given socket id
        void writeData(int id) const throw(VNetworkException&);

    private:
        friend class COPSMsgParser;
        ///Create object from data
        COPSClientAcceptMsg(const char* data);

        ///
        void parse(const char* data) throw (COPSBadDataException&);

        ///
        void parse(const COPSHeader&hdr, const char* data)
        throw (COPSBadDataException&);

        ///Set the message length, base on the set of objects it contains
        void setMsgLength();

        /* COPSHeader coming from base class */
        Sptr < COPSKATimer > _kaTimer;
        Sptr < COPSAcctTimer > _acctTimer;
        Sptr < COPSIntegrity > _integrity;
};

#endif
