
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSError_cxx_Version =
    "$Id: COPSError.cxx,v 1.9 2001/05/13 07:26:01 icahoon Exp $";

#include "VocalCommon.hxx" 
#include <netinet/in.h>
#include "COPSError.hxx"
#include "COPSBadDataException.hxx"


static const char* G_errmsgArray[] = {
   "Unknown.",
   "Bad handle.",
   "Invalid handle reference.",
   "Bad message format (Malformed message).",
   "Unable to process.",
   "Mandatory client-specific info missing.",
   "Unsupported client-type",
   "Mandatory COPS object missing.",
   "Client failure.",
   "Communication failure.",
   "Unknown.",
   "Shutting down.",
   "Redirect to preferred server.",
   "Unknown COPS object",
   "Authentication failure.",
   "Authentication required.",
};

COPSError::COPSError(u_int16_t errCode, u_int16_t subCode)
{
    _dataRep._errCode = errCode;
    if(_dataRep._errCode < 1 && _dataRep._errCode > COPS_ERR_MAX)
    {
        cpLog(LOG_ALERT, "Bad error code");
    }
    _dataRep._errSubCode = subCode;
    _objHdr.setCNum(COPS_ERROR);
    _objHdr.setCType(1);
    _objHdr.setDataLength(4);
}

///
COPSError::COPSError(const char* dataPtr)
{
    _objHdr.parse(dataPtr);
    _objHdr.checkDataLength();
    u_int16_t eCode;
    u_int16_t eSubCode;
    memcpy(&eCode, dataPtr + 4, sizeof(u_int16_t));
    memcpy(&eSubCode, dataPtr + 6, sizeof(u_int16_t));
    _dataRep._errCode = ntohs(eCode);

    _dataRep._errSubCode = ntohs(eSubCode);
    if(_dataRep._errCode < 1 && _dataRep._errCode > COPS_ERR_MAX)
    {
        cpLog(LOG_ALERT, "Bad error code");
        throw COPSBadDataException("Bad error code",
                                   __FILE__, __LINE__, 0);
    }
    _objHdr.setDataLength(sizeof(u_int32_t)); 
}

COPSError::COPSError(const COPSError& other)
{
    _objHdr = other._objHdr;
    _dataRep._errCode = other._dataRep._errCode;
    _dataRep._errSubCode = other._dataRep._errSubCode;
}

string
COPSError::getDescription()
{
    string errStr1;
    string errStr2;

    ///Get the details from the error code
    errStr1 = G_errmsgArray[_dataRep._errCode];
    //TODO - defind error sub-codes
    errStr2 = "";
    return (errStr1 + ":", errStr2);
}
void
COPSError::dump(ostream& os) const
{
    os << _objHdr << endl;
    os.width(25);
    os << "Error Code: " << _dataRep._errCode << endl;
    os.width(25);
    os << "Error Sub Code: " << _dataRep._errSubCode << endl;
}

COPSError&
COPSError::operator=(const COPSError& other)
{
    if (this != &other)
    {
        _objHdr = other._objHdr;
        _dataRep._errCode = other._dataRep._errCode;
        _dataRep._errSubCode = other._dataRep._errSubCode;
    }
    return (*this);
}

void
COPSError::writeData(int id) const throw(VNetworkException&)
{
    cpLog(LOG_DEBUG, "COPSError::writeData");
    _objHdr.writeData(id);
    u_int16_t eCode = htons(_dataRep._errCode);
    u_int16_t eSubCode = htons(_dataRep._errSubCode);
    COPSUtil::writeData(id, &eCode, 2);
    COPSUtil::writeData(id, &eSubCode, 2);
}
