#ifndef COPSHeader_hxx
#define COPSHeader_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSHeader_hxx_Version =
    "$Id: COPSHeader.hxx,v 1.10 2001/05/13 07:26:01 icahoon Exp $";
#include "VocalCommon.hxx"
#include <cassert>
#include <iostream>
#include <netinet/in.h>
#include "global.h"
#include "VNetworkException.hxx"

typedef enum {
    COPS_OP_REQ = 1,
    COPS_OP_DEC,
    COPS_OP_RPT,
    COPS_OP_DRQ,
    COPS_OP_SSQ,
    COPS_OP_OPN,
    COPS_OP_CAT,
    COPS_OP_CC,
    COPS_OP_KA,
    COPS_OP_SSC
} COPSOp_Code;

typedef struct _hTag
{
    ///High order nibble contains version, low order nibble contains flags
    u_int8_t _versionNflg;
    u_int8_t _opCode;
    u_int16_t _cType;
    u_int32_t _msgLength;
}
HeaderData;

///COPSHeader

class COPSHeader
{
    public:
        ///
        COPSHeader();

        ///
        COPSHeader(u_int8_t opCode, u_int16_t clientType = 0);

        ///
        COPSHeader(const char* buf);

        ///
        COPSHeader(const COPSHeader& other);

        //Accessors
        bool isARequest() const;
        ///
        bool isADecision() const;
        ///
        bool isAReport() const;
        ///
        bool isADeleteReq() const;
        ///
        bool isASyncStateReq() const;
        ///
        bool isAClientOpen() const;
        ///
        bool isAClientAccept() const;
        ///
        bool isAClientClose() const;
        ///
        bool isAKeepAlive() const;
        ///
        bool isASyncComplete() const;

        ///
        u_int32_t getMsgLength()const ;

        ///
        u_int32_t getHdrLength()
        {
            return (sizeof(u_int32_t) * 2);
        }

        ///
        u_int8_t getOpCode() const
        {
            return _dataRep._opCode;
        };

        //Mutators
        //Set the solicitation flag
        void setFlag(u_int8_t flg);

        ///Set the client type
        void setClientType(u_int16_t cType)
        {
            _dataRep._cType = cType;
        };

        ///Set the message length
        void setMsgLength(u_int32_t len);

        ///
        u_int16_t getClientType() const
        {
            return (_dataRep._cType);
        };

        ///
        bool isCOPSHeader() const
        {
            return true;
        };

        ///
        friend ostream& operator<<(ostream& os, const COPSHeader&);

        ///Writes object to given network socket in network byte order
        void writeData(int id) const throw(VNetworkException&);

        ///
        HeaderData& getDataRep()
        {
            return _dataRep;
        };

    private:
        HeaderData _dataRep;
};

inline ostream& operator<<(ostream& os, const COPSHeader& obj)
{
    os << "**MSG HEADER**" << endl;
    os.width(25);
    os << "Version: " << static_cast < int > (obj._dataRep._versionNflg >> 4) << endl;
    os.width(25);
    os << "Flags: " << static_cast < int > (obj._dataRep._versionNflg & 0x01) << endl;
    os.width(25);
    os << "OpCode: " << static_cast < int > (obj._dataRep._opCode) << endl;
    os.width(25);
    os << "Client-type: " << static_cast < int > (obj._dataRep._cType) << endl;
    os.width(25);
    os << "Message-length(bytes): " << obj._dataRep._msgLength << endl;
    return os;
}

#endif
