
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSIpv6Address_cxx_Version =
    "$Id: COPSIpv6Address.cxx,v 1.9 2001/05/13 07:26:01 icahoon Exp $";

#include "VocalCommon.hxx"
#include <cerrno>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>

#include "COPSIpv6Address.hxx"
#include "cpLog.h"

COPSIpv6Address&
COPSIpv6Address::operator=(const COPSIpv6Address& other)
{
    if (this != &other)
    {
        memcpy(&_addr , &(other._addr), 16);
    }
    return (*this);
}

void
COPSIpv6Address::setIpAddress(const string& hostName)
{
#ifdef __sparc
    cpLog(LOG_ALERT, "IPv6 address format not supported");
    assert(0);
#else
    struct hostent *pxHostEnt = gethostbyname2(hostName.c_str(), AF_INET6);
    assert(pxHostEnt);
    memcpy((void *)&(_addr), (void *)pxHostEnt->h_addr, pxHostEnt->h_length);
#endif
}

string
COPSIpv6Address::getIpName() const
{
    string ipName;

#ifdef __sparc
    cpLog(LOG_ALERT, "IPv6 address format not supported");
    assert(0);
#else
    char buf[INET6_ADDRSTRLEN];
    const char *c = 0;
    c = inet_ntop(AF_INET6, _addr, buf, INET6_ADDRSTRLEN);
    if (!c)
    {
        cpLog(LOG_ALERT, "Failed to convert to IPv6 address format, reason:%s", strerror(errno));
    }
    else
    {
        ipName = c;
    }
#endif 
    return ipName;
}

void
COPSIpv6Address::parse(const char* dataPtr)
{
    memcpy(&_addr, dataPtr, 16);
}
