
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSMsgParser_cxx_Version =
    "$Id: COPSMsgParser.cxx,v 1.9 2001/04/22 05:36:37 icahoon Exp $";

// On FreeBSD netinet/in.h requires sys/types.h to be included
// before it. We'll bring it in through VocalCommon.hxx.
//
#include "VocalCommon.hxx"
#include <cassert>

#include "COPSMsgParser.hxx"


Sptr < COPSMsg >
COPSMsgParser::parse(const char* data) throw (COPSBadDataException&)
{
    COPSHeader hdr;
    memcpy(&hdr, data, 8);
    return (parse(hdr, data + 8));
}

Sptr < COPSMsg >
COPSMsgParser::parse(const COPSHeader& hdr, const char* data) throw (COPSBadDataException&)
{
    COPSMsg* copsMsg = 0;
    COPSOp_Code cCode = static_cast < COPSOp_Code > (hdr.getOpCode());
    switch (cCode)
    {
        case COPS_OP_REQ:
        {
            cpLog(LOG_DEBUG, "Creating REQ msg");
            copsMsg = new COPSReqMsg();
            copsMsg->parse(hdr, data);
        }
        break;
        case COPS_OP_DEC:
        {
            cpLog(LOG_DEBUG, "Creating DEC msg");
            copsMsg = new COPSDecisionMsg();
            copsMsg->parse(hdr, data);
        }
        break;
        case COPS_OP_RPT:
        {
            cpLog(LOG_DEBUG, "Creating RPT msg");
            copsMsg = new COPSReportMsg();
            copsMsg->parse(hdr, data);
        }
        break;
        case COPS_OP_DRQ:
        {
            cpLog(LOG_DEBUG, "Creating DRQ msg");
            copsMsg = new COPSDeleteMsg();
            copsMsg->parse(hdr, data);
        }
        break;
        case COPS_OP_OPN:
        {
            cpLog(LOG_DEBUG, "Creating Client-Open msg");
            copsMsg = new COPSClientOpenMsg;
            copsMsg->parse(hdr, data);
        }
        break;
        case COPS_OP_CAT:
        {
            cpLog(LOG_DEBUG, "Creating Client-Accept msg");
            copsMsg = new COPSClientAcceptMsg;
            copsMsg->parse(hdr, data);
        }
        break;
        case COPS_OP_CC:
        {
            cpLog(LOG_DEBUG, "Creating Client-Close msg");
            copsMsg = new COPSClientCloseMsg;
            copsMsg->parse(hdr, data);
        }
        break;
        case COPS_OP_KA:
        {
            cpLog(LOG_DEBUG, "Creating KA msg");
            copsMsg = new COPSKAMsg();
            copsMsg->parse(hdr, data);
        }
        break;
        case COPS_OP_SSQ:
        case COPS_OP_SSC:
        {
            cpLog(LOG_DEBUG, (cCode == COPS_OP_SSC) ? "Creating Sync-State Complete msg" : "Creating Sync-State Request msg");
            copsMsg = new COPSSyncStateMsg();
            copsMsg->parse(hdr, data);
        }
        break;
        default:
        cpLog(LOG_DEBUG, "Unknown message type");
        break;
    }
    return copsMsg;
}
