#ifndef COPSObjBase_hxx
#define COPSObjBase_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSObjBase_hxx_Version =
    "$Id: COPSObjBase.hxx,v 1.12 2001/05/13 07:26:01 icahoon Exp $";
#include <cassert>
#include <iostream>
#include <string>
#include "global.h"
#include "vtypes.h"
#include "VNetworkException.hxx"
#include "COPSUtil.hxx"
#include "cpLog.h"
#include "COPSData.hxx"

///COPSObjBase
class COPSObjBase
{
    public:
        ///Add padding in the data, if the data does not fall on 32-bit boundary
        static COPSData getPadding(int len)
        {
            char* padBuf = new char[len];
            memset(padBuf, '0', len);
            COPSData d(padBuf, len);
            delete [] padBuf;
            return d;
        }

        ///
        virtual void dump(ostream& os) const
        { }

        ///Writes data to a given network socket
        virtual void writeData(int id) const throw (VNetworkException&)
        { }

        ///
        virtual u_int16_t getDataLength() const = 0;

        ///
        virtual string className()const
        {
            return "COPSObjBase";
        }

        ///
        friend ostream& operator<<(ostream& os, const COPSObjBase&);

        ///
        virtual bool isCOPSHeader() const
        {
            return false;
        }
        
        ///
        virtual bool isClientHandle() const
        {
            return false;
        }
        
        ///
        virtual bool isContext() const
        {
            return false;
        }
        
        ///
        virtual bool isInterface() const
        {
            return false;
        }
        
        ///
        virtual bool isDecision() const
        {
            return false;
        }
        
        ///
        virtual bool isLocalDecision() const
        {
            return false;
        }
        
        ///
        virtual bool isReport() const
        {
            return false;
        }
        
        ///
        virtual bool isError() const
        {
            return false;
        }
        
        ///
        virtual bool isTimer() const
        {
            return false;
        }
        
        ///
        virtual bool isPepId() const
        {
            return false;
        }
        
        ///
        virtual bool isReason() const
        {
            return false;
        }
        
        ///
        virtual bool isPdpAddress() const
        {
            return false;
        }
        
        ///
        virtual bool isClientSI() const
        {
            return false;
        }
        
        ///
        virtual bool isMessageIntegrity() const
        {
            return false;
        }
        
    protected:
        ///
        COPSObjBase()
        { }

        ///
        virtual ~COPSObjBase()
        { }
};

inline ostream& operator<<(ostream& os, const COPSObjBase& obj)
{
    obj.dump(os);
    return os;
}

#endif
