#ifndef COPSPrClassError_hxx
#define COPSPrClassError_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSPrClassError_hxx_Version =
    "$Id: COPSPrClassError.hxx,v 1.4 2000/12/18 23:42:55 bko Exp $";

#include "COPSPrError.hxx"
#include "COPSObjHeader.hxx"
#include "COPSData.hxx"


///COPSPrClassError

typedef enum {
    C_spaceExhausted = 1,
    C_instanceInvalid = 2,
    C_attrValueInvalid = 3,
    C_attrValueSupLimited = 4,
    C_attrEnumSupLimited = 5,
    C_attrMaxLengthExceeded = 6,
    C_attrRefUnknown = 7,
    C_notifyOnly = 8,
    C_unknownPrc = 9,
    C_tooFewAttrs = 10,
    C_invalidAttrType = 11,
    C_deletedInRef = 12,
    C_specificError = 13,
    C_errmax = 14
} CPERR_code;


static const char* CerrTable[C_errmax] =
    {
        "Reserved",
        "No more instances may currently be installed in the given class",
        "Invalid class instance",
        "Invalid attribute value",
        "The value for attribute not currently supported by the device",
        "The enumeration for attribute not currently supported by the device",
        "Attribute length exceeds device limitations",
        "Unknown attribute reference",
        "Only supported for use by request or report",
        "Class not supported by PEP",
        "Too few attributes",
        "Invalid attribute type",
        "Reference to deleted instance",
        "PRC specific error, check subcode for more details"
    };

class COPSPrClassError : public COPSPrError
{
    public:
        ///
        COPSPrClassError(u_int16_t eCode, u_int16_t eSubCode) :
        COPSPrError(eCode, eSubCode)
        {
            assert(eCode < C_errmax);
            _sNum = COPSPrObjBase::PR_CPERR;
            _sType = COPSPrObjBase::PR_BER;
        }

        ///
        COPSPrClassError& operator=(const COPSPrClassError& other)
        {
            if (this != &other)
            {
                COPSPrError::operator=(other);
            }
            return *this;
        }

        ///
        string className() const
        {
            return "COPSPrClassError";
        };

        ///
        bool isPRCClassError() const
        {
            return true;
        }

        ///
        string strError() const
        {
            return CerrTable[_errCode];
        };

    private:
        friend class COPSPrObjParser;
        friend class COPSReportMsg;

        ///Parse the data and create a PrClassError object
        COPSPrClassError(const char* dataPtr) : COPSPrError(dataPtr)
        { }
        ;
};

#endif
