
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSRoleCombination_cxx_Version =
    "$Id: COPSRoleCombination.cxx,v 1.5 2001/05/13 07:26:02 icahoon Exp $";


#include "VocalCommon.hxx"
#include <netinet/in.h>

#include "COPSRoleCombination.hxx"
#include "COPSBadDataException.hxx"
#include "COPSUtil.hxx"
#include "cpLog.h"

COPSRoleCombination::COPSRoleCombination(COPSp_type pType, const COPSData& comb)
    : COPSRsvpObjBase(pType)
{
    _data = comb;
    _objHdr.setDataLength(_data.length() + 4);
}

///
COPSRoleCombination::COPSRoleCombination(const char* dataPtr)
    : COPSRsvpObjBase(dataPtr)
{
    _data = COPSData(dataPtr + 4, _objHdr.getDataLength() - 4);
    if (_data.length() % 4)
    {
        int padLen = 4 - _data.length() % 4;
        _padding = getPadding(padLen);
    }
}

COPSRoleCombination::COPSRoleCombination(const COPSRoleCombination& other)
{
    copyObj(other);
}

COPSRoleCombination&
COPSRoleCombination::operator=(const COPSRoleCombination& other)
{
    if (this != &other)
    {
        copyObj(other);
    }
    return (*this);
}

void
COPSRoleCombination::copyObj(const COPSRoleCombination& other)
{
    _objHdr = other._objHdr;
    _data = other._data;
    _padding = other._padding;
}

void
COPSRoleCombination::writeData(int id) const throw(VNetworkException&)
{
    cpLog(LOG_DEBUG, "COPSRoleCombination::writeData");
    _objHdr.writeData(id);
    COPSUtil::writeData(id, _data.getData(), _data.length());
    if (_padding.length())
    {
        COPSUtil::writeData(id, _padding.getData(), _padding.length());
    }
}
