#ifndef COPSTimer_hxx
#define COPSTimer_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSTimer_hxx_Version =
    "$Id: COPSTimer.hxx,v 1.6 2000/12/18 23:42:55 bko Exp $";

#include <sys/types.h>

#include "COPSObjBase.hxx"
#include "COPSObjHeader.hxx"

///COPSTimer

class COPSTimer : public COPSObjBase
{
    public:
        ///Returns size in number of octects, including header
        u_int16_t getDataLength() const;

        ///
        void dump(ostream& os)const;

        ///
        u_int16_t getTimerVal()
        {
            return _dataRep._timerValue;
        };

        ///
        string className() const
        {
            return "COPSTimer";
        };

        ///
        bool isTimer() const
        {
            return true;
        };
        ///
        virtual bool isKATimer() const
        {
            return false;
        };
        ///
        virtual bool isAcctTimer() const
        {
            return false;
        };

        ///Write data to given socket in Network byte order
        void writeData(int id) const throw(VNetworkException&);

    protected:
        ///
        COPSTimer(u_int16_t timeVal);

        ///Receive data that is in netwrok byte order and fill in the obj.
        COPSTimer(const char* dataPtr);

        ///
        COPSTimer(const COPSTimer& other);

        ///
        COPSTimer& operator=(const COPSTimer& other);

        COPSObjHeader _objHdr;
        struct
        {
            u_int16_t _reserved;
            u_int16_t _timerValue;
        }
        _dataRep;
};

#endif
