#ifndef OSPAUTHINFO_HXX
#define OSPAUTHINFO_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const OSPAuthInfo_hxx_Version =
    "$Id: OSPAuthInfo.hxx,v 1.10 2001/01/26 20:55:24 cvs Exp $";

#include "global.h"
#include <sys/types.h>
#include <string>
#include "OSPDefines.hxx"


/// Information class needed for authorization request
class OSPAuthInfo
{
    public:
        /// constructor
        OSPAuthInfo( void );
        /// destructor
        virtual ~OSPAuthInfo();

        /// get caller info
        const char * getSource( void ) const;

	/// set caller info w/ a C-style string
        //int setSource( const char * source);

	/// set caller info w/ a string obj
        int setSource( const string& source );

	/// get caller device info
        const char * getSourceDevice(void) const
        {
            return (_ospAuthInfo._sourceDevice.c_str());
        }

	/// set caller device info
        void setSourceDevice( const char * s )
        {
            assert( s );
            _ospAuthInfo._sourceDevice = s;
        }

	/// get caller's number
        const char * getCallingNum(void) const
        {
            return (_ospAuthInfo._callingNum.c_str());
        }

	/// set caller's number
        void setCallingNum( const char * cn )
        {
            assert( cn );
            _ospAuthInfo._callingNum = cn;
        }

	/// get callee number
        const char * getCalledNum(void) const
        {
            return (_ospAuthInfo._calledNum.c_str());
        }

	/// set callee number
        void setCalledNum( const char * cn )
        {
            assert( cn );
            _ospAuthInfo._calledNum = cn;
        }

	/// get user info
        const char * getUser(void) const
        {
            return (_ospAuthInfo._user.c_str());
        }

	/// set user info
        void setUser( const char * u )
        {
            assert( u );
            _ospAuthInfo._user = u;
        }

	/// get number of call ids
        const unsigned int getNumOfCallIds( void ) const
        {
            return (1);
        }

	/// set number of call ids
        void setNumOfCallIds( unsigned int nc )
        {
            // _ospAuthInfo._numCallIds = nc;
            _ospAuthInfo._numCallIds = 1;
        }

        /// manage one CallId for now:
        OSPCallIdList * getCallIds( void ) const
        {
            return ( _ospAuthInfo._callIds);
        }

	/** set one call id at a time
	 *  For current release, only one callid is maintained, so,
	 *  idx is always ignored; len is 1 + length of the *val string.
	 */
        int setCallId( int idx, int len, const char * val );

        /**
	 *  Get preferred destination info.
         *  _preferDests stores a list of char strings containing preferred
         *  destinations for a call.  The list is terminated by an empty
         *  string  or a NULL pointer.  The list may be empty if the application
         *  has no preferred destinations.
         *
         *  For now, we do not use this field.
	 */
        const char ** getPreferDests(void ) const
        {
            // bad practice: but can't use neither
            // static|dynamic|reinterpret casting.
            // return( reinterpret_cast<const char**>(_ospAuthInfo._preferDests)); }
            return ( (const char **)(_ospAuthInfo._preferDests));
        }

        /// set preferred destination info
        void setPreferDests( const char ** pd )
        {
            // if appl. wishes no preferred destination, pd can be empty
            //_ospAuthInfo._preferDests = pd;
            _ospAuthInfo._preferDests = 0;
        }

        /// get number of destination
        unsigned int getNumDest( void ) const
        {
            return ( _ospAuthInfo._numDest);
        }
        /// set number of destination
        void setNumDest( unsigned int nd )
        {
            _ospAuthInfo._numDest = nd;
        }


        /// debug only
        void dump(void );

    private:
        /// not used: equal operator
        bool operator==( const OSPAuthInfo& rhs );
        /// not used: copy constructor
        OSPAuthInfo( const OSPAuthInfo& rhs );
        /// not used: assigned operator
        OSPAuthInfo& operator=( const OSPAuthInfo& rhs );

        ///
        // Future - handle multiple callIds and preferred destinations
        //       new container classes
  
        /// structure containing info for composing an auth request
        struct
        {
	    /// caller: "[192.82.1.101]:112" or "www.carrier.com"
            string _source;        
	    /// name of calling device, optional, OK to be ""
            string _sourceDevice; 
	    /// e.g. "14083831000"
            string _callingNum;  
	    /// e.g. "14087673676"
            string _calledNum;   
	    /// end user name (e.g. calling card/PIN #) "" OK
            string _user;      
	    /// _callIds array size == 1 for now
            unsigned int _numCallIds;  

	    // for future expansion:  OSPCallIdList * _callIds[];
	    //                        char * _preferDests[];
	      
	    /// Vocal Softswitch only provides one call id
            OSPCallIdList * _callIds; 
	    /// preferred destination(s) == "" for now
            char ** _preferDests;      // "gateway1.carrier.com" or
                                       // [172.16.1.2]:112" OK to be optional
            /** input: max. number of dest. appl. wants to consider
             *  output: actual # of destinations authorized
	     */
            unsigned int _numDest;   // input: max. number of dest. appl. wants to
        } _ospAuthInfo;

};


// end of File

#endif
