
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const OSPProvider_cxx_Version =
    "$Id: OSPProvider.cxx,v 1.8 2001/01/20 01:33:06 wbreu Exp $";
#include "cpLog.h"
#include "ospinit.h"
#include "OSPProvider.hxx"
#include "OSPErrStrings.hxx"


///
// need to call setData() to make the obj well-formed
// e.g. OSPProivder *ospProvider = new OSPProvider();
//      OSPProviderInfo  data;
//      data.numOfServicePoints = ....;
//      data.....
//      ospProvider->connect( data );
OSPProvider::OSPProvider ( void ) : _providerReady(false),
        _providerHandle(0)
{
    cpLog( LOG_DEBUG, "OSPProvider ctor invoked");
    // application needs to have called  OSPPInit();
    // now, call connect() to establish connect w/ OSP Server
}

// invoke OSPInit() and OSPPProviderNew() osplib api

///
// call to OSPPProviderDelete() osplib api
OSPProvider::~OSPProvider()
{
    int errCode = 0;

    cpLog( LOG_DEBUG, "OSPProvider dtor called");
    if (_providerHandle != 0)
    {
        errCode = OSPPProviderDelete( _providerHandle, 0 );
        if (errCode != 0)
        {
            cpLog( LOG_ERR, "~OSPProvider error=%d",
                   OSPErrStrings::Instance().dumpErr(errCode));
        }
        else
            cpLog( LOG_DEBUG, "~OSPProvider handle=%d deleted",
                   _providerHandle );
    }
}

///
// invoke OSPPProviderNew() in osplib:
int OSPProvider::connect( OSPProviderInfo& info )
{
    cpLog( LOG_DEBUG, "OSPProvider::connect() called");
    return 0;
}  // OSPProvider::connect()

int OSPProvider::connect( unsigned int numOfServicePoints,
                          const char * servicePoints[],
                          const char * auditURL,
                          const OSPTPRIVATEKEY * localPrivateKey,
                          unsigned char * localCertificate,
                          unsigned int numOfAuthorityCertificates,
                          const void * authorityCertificates[],
                          unsigned int localValidation,
                          unsigned int sslLifetime,
                          unsigned int httpMaxConnections,
                          unsigned int httpPersistence,
                          unsigned int httpRetryDelay,
                          unsigned int httpRetryLimit,
                          unsigned int httpTimeout,
                          const char* customerId,
                          const char* deviceId )
{
    // TODO: parameter validity checks

    if (_providerReady == true)
    {
        cpLog(LOG_DEBUG, "OSPProvider object is already connected!");
        return -1;
    }
    int errCode = OSPPProviderNew( numOfServicePoints,
                                   servicePoints,
                                   auditURL, localPrivateKey, localCertificate,
                                   numOfAuthorityCertificates, authorityCertificates,
                                   localValidation, sslLifetime, httpMaxConnections,
                                   httpPersistence, httpRetryDelay, httpRetryLimit,
                                   httpTimeout, customerId, deviceId, &_providerHandle );

    if (errCode != 0)
    {
        cpLog(LOG_ERR, "::connect(...) error=%d", errCode);
    }
    else
    {
        cpLog(LOG_DEBUG, "::connect() OK");
        _providerReady = true;
    }

    return errCode;
}   // OSPProvider::connect(....)


// Debug only:
void OSPProvider::print( void )
{
    ;
}

///
// access functions to get & set configuration of a Provider
// object.  The set methods do not interrupt any active
// communication exchanges, but subsequent exchanges will
// use the new values.
//
int OSPProvider::getNumberOfAuthorityCertificates(

    unsigned int *numOfAuthorityCertificates )
{
    if (numOfAuthorityCertificates != 0)
    {
        return ( OSPPProviderGetNumberOfAuthorityCertificates(
                     _providerHandle, numOfAuthorityCertificates ));
    }
    else
        return -1;
}

///
// Descriptions: getAuthorityCeritficates() -
//    get current certificate authority public keys entrusted by
//    OSPProvider. These keys are returned in the form of X.509
//    certificates array, authorityCertificates.
//
// Input:
//    sizeOfCertificate - the max size of any individual certificate
//        If any cert. exceeds that size limit, no certificates are
//        returned and an error results
//    *numOfAuthorityCertificates - max no. of certs to return
//    * authorityCertificatesPtr - ptr to an array. Each element points
//        to an address to a buffer for acert. authority public key
//
// Output:
//    *numOfAuthorityCertificates - actual number of certs returned
int OSPProvider::getAuthorityCeritficates(
    unsigned int sizeOfCertificate,
    unsigned int *numOfAuthorityCertificates,
    void * authorityCertificatesPtr[] )
{

    return ( OSPPProviderGetAuthorityCertificates(
                 _providerHandle, sizeOfCertificate,
                 numOfAuthorityCertificates,
                 authorityCertificatesPtr) );
}

int OSPProvider::setAuthorityCeritficates(
    unsigned int numOfAuthorityCertificates,
    const void * authorityCertificatesPtr[] )
{
    if (authorityCertificatesPtr)
        return ( OSPPProviderSetAuthorityCertificates( _providerHandle,
                 numOfAuthorityCertificates, authorityCertificatesPtr) );
    else
        return -1;
}

// OSPPProviderGet/SetHTTPMaxConnections()
int OSPProvider::getHttpMaxConnections( unsigned int *httpMaxConnectons)
{
    if ( httpMaxConnectons )
        return ( OSPPProviderGetHTTPMaxConnections( _providerHandle,
                 httpMaxConnectons ) );
    else
        return -1;
}

int OSPProvider::setHttpMaxConnections( unsigned int httpMaxConnectons)

{
    return ( OSPPProviderSetHTTPMaxConnections( _providerHandle,
             httpMaxConnectons ) );
}

// OSPPProviderGet/SetHTTPPersistence()
int OSPProvider::getHttpPersistence( unsigned int *httpPersistence )
{
    if (httpPersistence)
        return ( OSPPProviderGetHTTPPersistence( _providerHandle,
                 httpPersistence ) );
    else
        return -1;
}

int OSPProvider::setHttpPersistence( unsigned int httpPersistence )
{
    return ( OSPPProviderSetHTTPPersistence( _providerHandle,
             httpPersistence ) );

}


// OSPPProviderGet/SetHTTPRetryDelay()
int OSPProvider::getHttpRetryDelay( unsigned int *httpRetryDelay )
{
    if (httpRetryDelay)
        return ( OSPPProviderGetHTTPRetryDelay( _providerHandle,
                                                httpRetryDelay ) );
    else
        return -1;
}

int OSPProvider::setHttpRetryDelay( unsigned int httpRetryDelay )
{
    return ( OSPPProviderSetHTTPRetryDelay( _providerHandle,
                                            httpRetryDelay ) );
}


// OSPPProviderGet/SetHTTPRetryLimit()
int OSPProvider::getHttpRetryLimit( unsigned int *httpRetryLimit )
{
    if (httpRetryLimit)
        return ( OSPPProviderGetHTTPRetryLimit( _providerHandle,
                                                httpRetryLimit ) );
    else
        return -1;
}

int OSPProvider::setHttpRetryLimit( unsigned int httyRetryLimit )
{
    return ( OSPPProviderSetHTTPRetryLimit( _providerHandle,
                                            httyRetryLimit ) );
}


// OSPPProviderGet/SetHTTPTimeout()
int OSPProvider::getHttpTimeout( unsigned int *httpTimeout )
{
    if (httpTimeout)
        return ( OSPPProviderGetHTTPTimeout( _providerHandle,
                                             httpTimeout ) );
    else
        return -1;
}

int OSPProvider::setHttpTimeout( unsigned int httpTimeout )
{
    return ( OSPPProviderSetHTTPTimeout( _providerHandle,
                                         httpTimeout ) );
}


// OSPPProviderGet/SetLocalKeys()
int OSPProvider::getLocalKeys( OSPTPRIVATEKEY *localPrivateKey,
                               unsigned int sizeOfCertificate,
                               unsigned char * localCertificate )
{
    if (localPrivateKey && localCertificate)
        return ( OSPPProviderGetLocalKeys( _providerHandle,
                                           localPrivateKey, sizeOfCertificate, localCertificate ) );
    else
        return -1;
}

int OSPProvider::setLocalKeys( const OSPTPRIVATEKEY *localPrivateKey,
                               const unsigned char * localCertificate )

{
    if ( localPrivateKey && localCertificate)
        return ( OSPPProviderSetLocalKeys( _providerHandle,
                                           localPrivateKey, localCertificate ));
    else
        return -1;
}

// OSPPProviderGet/SetLocalValidation
int OSPProvider::getLocalValidation( unsigned int *localValidation )
{
    if ( localValidation )
        return ( OSPPProviderGetLocalValidation( _providerHandle,
                 localValidation ) );
    else
        return -1;
}

int OSPProvider::setLocalValidation( unsigned int localValidation )

{
    return ( OSPPProviderSetLocalValidation( _providerHandle,
             localValidation ) );
}

// OSPPProviderGet/SetServicePoints()
int OSPProvider::getNumOfServicePoints( unsigned int * numOfServicePoints )
{
    if (numOfServicePoints)
        return ( OSPPProviderGetNumberOfServicePoints( _providerHandle,
                 numOfServicePoints ) );
    else
        return -1;
}

int OSPProvider::getServicePoints( unsigned int numOfServicePoints,
                                   unsigned int sizeOfServicePoint,
                                   char * servicePoints[] )
{
    if (servicePoints)
        return ( OSPPProviderGetServicePoints( _providerHandle,
                                               numOfServicePoints, sizeOfServicePoint, servicePoints ) );
    else
        return -1;
}

int OSPProvider::setServicePoints( unsigned int numOfServicePoints,
                                   const char* servicePoints[] )
{
    if (servicePoints)
        return ( OSPPProviderSetServicePoints( _providerHandle,
                                               numOfServicePoints, servicePoints ) );
    else
        return -1;
}


// OSPPProviderGet/SetSSLLifetime
int OSPProvider::getSslLifetime( unsigned int *sslLifetime )
{
    if (sslLifetime)
        return ( OSPPProviderGetSSLLifetime( _providerHandle,
                                             sslLifetime ) );
    else
        return -1;
}

int OSPProvider::setSslLifetime( unsigned int sslLifetime )
{
    return ( OSPPProviderSetSSLLifetime( _providerHandle, sslLifetime ));
}

// End of File
