
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const OSPValInfo_cxx_Version =
    "$Id$";
#include "cpLog.h"
#include "OSPValInfo.hxx"

OSPValInfo::OSPValInfo()
{
    cpLog(LOG_DEBUG, "OSPValInfo ctor invoked");
    _ospValInfo._source = "SourceUnknown";
    _ospValInfo._destination = "DestUnknown";

    // leave _sourceDevice and _destDevice to be empty

    _ospValInfo._authFlag =  0;     // default to unAuthorized
    _ospValInfo._timeLimit = 0;     // no time limit exists
}


OSPValInfo::~OSPValInfo()
{
    cpLog(LOG_DEBUG, "OSPValInfo dtor invoked ");
}

const char * OSPValInfo::getSource( void )
{
    return ( _ospValInfo._source.c_str() );
}

int OSPValInfo::setSource( const string& source )
{
    if (source.length() > 0)
    {
        _ospValInfo._source = source;
        return 0;
    }
    else return -1;
}

const char * OSPValInfo::getDestination( void )
{
    return ( _ospValInfo._destination.c_str() );
}

int OSPValInfo::setDestination( const string& dest )
{
    if (dest.length() > 0)
    {
        _ospValInfo._destination = dest;
        return 0;
    }
    else return -1;
}


void OSPValInfo::dump( void )
{
    cpLog( LOG_DEBUG, "source=%s, dest=%s\n"
           "calling#=%s called#=%s\n"
           "callId=%s authFlag=%d timeLimit=%d\n"
           "tokensize=%d",
           _ospValInfo._source.c_str(), _ospValInfo._destination.c_str(),
           _ospValInfo._callingNum.c_str(), _ospValInfo._calledNum.c_str(),
           _ospValInfo._callId.c_str(), _ospValInfo._authFlag,
           _ospValInfo._timeLimit, _ospValInfo._token.length() );
}

// end of file
