
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSProcessor_cxx_Version =
    "$Id: COPSProcessor.cxx,v 1.14 2002/05/15 01:26:25 sprajpat Exp $";
#include "COPSReqMsg.hxx"
#include "COPSReportMsg.hxx"
#include "COPSProcessor.hxx"
#include "COPSDecisionMsg.hxx"
#include "Cops_def.hxx"
#include "PdpAgent.hxx"
#include "cpLog.h"
#include "AgentFactory.hxx"
#include "COPSAgent.hxx"
#include "VMissingDataException.hxx"
#ifdef HAS_OSP
#include "PolicyTranslator.hxx"
#include "PolicyTransaction.hxx"
#include "OSPException.hxx"
#endif
void
COPSProcessor::process(const COPSMsg& msg)
{
    if (msg.getHeader().isARequest())
    {
        cpLog(LOG_DEBUG, "Processing ReqMessage");
        const COPSReqMsg* reqMsg = dynamic_cast < const COPSReqMsg* > (&msg);
        const COPSContext& cntxt = reqMsg->getContext();
        const COPSHeader& header = reqMsg->getHeader();
		//u_int16_t reqType = cntxt.getRequestType();
		u_int16_t cType   = header.getClientType();
#ifndef HAS_RSVPLIB
		if (cType == C_RSVP)
		{
           cpLog(LOG_ALERT,
				"RSVP library not supported for request opcode:  ", 
				static_cast < int > (msg.getHeader().getOpCode()));
		   return;
		}
#endif
        Sptr < COPSAgent > agent =
            AgentFactory::createAgent(cntxt, cType, _connId);
        cpLog(LOG_ALERT,
              "createAgent called, clientType=%d msgType=%d, connId=%d ",
              header.getClientType(), cntxt.getMessageType(), _connId);
        agent->processRequest(*reqMsg);
    }
    else if (msg.getHeader().isAReport())
    {
        cpLog(LOG_DEBUG, "Processing Report");
        const COPSReportMsg* repMsg = dynamic_cast < const COPSReportMsg* > (&msg);
        assert(repMsg);
        const COPSHandle& handle = repMsg->getClientHandle();
        const COPSHeader& header = repMsg->getHeader();
        try
        {
            const COPSContext& cntxt = PdpAgent::instance().findDecisionContext(handle.getId());
            //u_int16_t reqType = cntxt.getRequestType();
			u_int16_t cType   = header.getClientType();
#ifndef HAS_RSVPLIB
			if (cType == C_RSVP)
			{
			   cpLog(LOG_ALERT,
		       	"RSVP library not supported for report opcode:  ",
			    static_cast < int > (msg.getHeader().getOpCode()));
			   return;
			}
#endif
            Sptr < COPSAgent > agent =
                AgentFactory::createAgent(cntxt, cType, _connId);
            agent->processReport(*repMsg);
        }
        catch (VMissingDataException& e)
        {
            cpLog(LOG_ALERT, "Failed to get decision context for report for handle:%s",
                  handle.getId().str().c_str());
        }
        catch (VNetworkException& e)
        {
            cpLog(LOG_ALERT, "Failed to send the report to connection %d, reason:%s",
                  _connId, e.getDescription().c_str());
        }
    }
    else
    {
        cpLog(LOG_ALERT, "Unsupported request type:%d", static_cast < int > (msg.getHeader().getOpCode()));
    }

}
