#ifndef PolRequest_hxx
#define PolRequest_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const PolRequest_hxx_Version =
    "$Id: PolRequest.hxx,v 1.21 2002/05/15 01:26:26 sprajpat Exp $";

#include "Sptr.hxx"
#include "COPSReqMsg.hxx"
#include "VNetworkException.hxx"
#include "Connection.hxx"
#include "COPSReportType.hxx"
#include "ReqState.hxx"
#include "Cops_def.hxx"
#include "PolicyTransaction.hxx"


class SipMsg;

/**
    PolRequest represent a wrapper for COPS request state, this hiding
    all the details of COPS as well as maintains a request state.The object
    act as proxy to the PEP request.PEP makes makes a COPS request by
    creating a PolRequest object and continues.Once a COPS response is received PEP
    is notified. 
*/
class PolRequest
{
    public:
	///
        enum PolReqStatus {
            STATUS_OK = 0,
            STATUS_NO_RESPONSE,
            STATUS_FAILED,
            STATUS_TIMEDOUT
        };

        ///
        string className()
        {
            return "PolRequest";
        }

        ///Destructor
        ~PolRequest()
        {}


        ///
        const COPSData& getClientHandle() const
        {
            return _clientHandle;
        }

        ///
        PolReqStatus getStatus() const;

        ///
        string getToken() const;

        ///
        const string& getCallId() const
        {
            return _callId;
        }

        ///
        void implStart();
        ///
        void implFinish();

        ///
        void implAbort();

        ///
        const COPSContext& getContext() const;

        ///Return true of the context for the policy request is incoming
        bool isIncoming() const;

    private:
        ///To request authorization
        PolRequest(const string& calledNumber,
                   const string& callingNumber,
                   const string& callId,
                   COPS_msg_type mType);

        ///To request token validation
        PolRequest(const string& calledNumber,
                   const string& callingNumber,
                   const string& callId,
                   const COPSData& token,
                   COPS_msg_type mType);


        ///To enable/disable QoS
        PolRequest::PolRequest(const string& callId,
                               COPS_msg_type mType);

        ///
        void auth();

        ///
        void validate();

        ///
        void enableQoS(const string& sourceIp, int sourcePort,
                       const string& destIp, int destPort);

        ///
        void disableQoS();

        ///
        void checkIfTimedOut();

        ///
        bool toBeDeleted() const;

        ///
        bool hasFailed() const;

        ///
        void setPolicyTransaction(PolicyTransaction* polT);
        ///
        void sendCOPSMsg(Connection& conn) throw (VNetworkException&);

        ///
        void sendUsage(Connection& conn) throw (VNetworkException&);

        ///
        void createUsageReport(COPSReportType::RepType rType, bool noUsage = false);

        ///
        void timeOut();

        ///
        void fail();
        ///
        void received();
        ///
        void reSend();
        ///
        void finish();
        ///
        void remove();
        ///
        void setState(ReqState* newState)
        {
            _state = newState;
        };

        ///
        void createNewHandle();

        ///
        void init();

        ///
        void resendUsage();

        ///
        Sptr < ReqState > _state;
	///
        COPSData _clientHandle;
	///
        string _callId;
	///
        Sptr < COPSMsg > _copsMsg;
	///
        Sptr < PolicyTransaction > _polTransaction;

	///
        u_int32_t _startTime;
	///
        u_int32_t _endTime;
	///
        PolReqStatus _status;

	///
        int _usageRetry;
	///
        int _sendRetry;
       
	///
        friend class PepAgent;

	// Only the ReqState* objects can update the PolRequest's _status
	///
        friend class ReqState;
        ///
        friend class ReqStateTimeout;
	///
        friend class ReqStateFail;
	///
        friend class ReqStateReceived;
	///
        friend class ReqStateImplStart;
	///
        friend class ReqStateSent;
	///
        friend class ReqStatePending;
};

#endif
