
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const policyServer_cxx_Version =
    "$Id: policyServer.cxx,v 1.12 2002/06/03 21:31:43 sprajpat Exp $";
#include "PdpAgent.hxx"
#include "VException.hxx"
#include "CommandLine.hxx"
#include "PolicyCfg.hxx"
#include "Daemon.hxx"

/** Main driver to launch policyServer:
 *  Usage: policyServer pserver:port altPserver:altport xxxx
 *  It instantiates 2 singleton objects - 
 *  PolicyCfg to get configuration data from Provisioning Server and
 *  PdpAgent  to launch PDP (Policy Decision Server)
 */
int
main(int argc, const char** argv)
{
    try
    {
        Sptr < CommandLine > cIn = CommandLine::instance(argc, argv,
                                   "Usage: policyServer pserver:port altPserver:altport xxxx");
        string pHost = cIn->getString("pshostname");
        int pPort = cIn->getInt("psremoteport");
        cpLog(LOG_DEBUG, "ProvisionServer:%s", pHost.c_str());
        cpLog(LOG_DEBUG, "ProvisionServer port:%d", pPort);

        string altHost = cIn->getString("psbackuphostname");
        int altPort = cIn->getInt("psbackupremoteport");
        cpLog(LOG_DEBUG, "backup ProvisionServer:%s", altHost.c_str());
        cpLog(LOG_DEBUG, "backup ProvisionServer port:%d", altPort);

        if (!CommandLine::instance()->getInt("NO_DAEMON"))
        {
            cpLog(LOG_DEBUG, "Running as daemon");
            Daemon();
        }

#ifdef HAS_OSP
        PolicyCfg::Instance(pHost.c_str(), pPort, altHost.c_str(), altPort);
#endif
        PdpAgent::instance().run();

    }
    catch (VException& e)
    {
        cpLog(LOG_ALERT, "Appplication exiting, reason: %s", e.getDescription().c_str());
    }
    catch (...)
    {
        cpLog(LOG_ALERT, "Appplication exiting, reason:Unknown");
    }
}
