// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#include "Field.hxx"

using namespace Epoisses;

Assignment<int>::Assignment(const Field<int> &arg,
                            const Field<int> &value)
   : BaseAssignment(arg, value)
{}

Assignment<int>::Assignment(const Field<int> &arg,
                            int value)
   : BaseAssignment(arg, Constant(value))
{}

// Implement -= .dlb. generalize to operator eventally
Assignment<int>::Assignment(const Field<int> &arg,
                            int value, 
                            bool dummy)
   : BaseAssignment(arg, Constant(arg.getName(), value))
{}

void
Assignment<int>::set(int v)
{
   BaseAssignment::set(Constant(v));
}

void
Assignment<int>::set(const Field<int> &v)
{
   // replace the value field
   BaseAssignment::set(v);
}

//------------------------------------------------------------

Assignment<bool>::Assignment(const Field<bool> &arg,
                             const Field<bool> &value)
   : BaseAssignment(arg, value)
{}

Assignment<bool>::Assignment(const Field<bool> &arg,
                             bool value)
   : BaseAssignment(arg, (Constant(value)))
{}

void
Assignment<bool>::set(bool v)
{
   BaseAssignment::set(Constant(v));
}

void
Assignment<bool>::set(const Field<bool> &v)
{
   // replace the value field
   BaseAssignment::set(v);
}

//------------------------------------------------------------

Assignment<double>::Assignment(const Field<double> &arg,
                               const Field<double> &value)
   : BaseAssignment(arg, value)
{}

Assignment<double>::Assignment(const Field<double> &arg,
                               double value)
   : BaseAssignment(arg, Constant(value))
{}

// give a new value to the assignment
void
Assignment<double>::set(double v)
{
   BaseAssignment::set(Constant(v));
}

void
Assignment<double>::set(const Field<double> &v)
{
   BaseAssignment::set(v);
}

//------------------------------------------------------------

Assignment<string>::Assignment(const Field<string> &arg,
                               const Field<string> &value)
   : BaseAssignment(arg, value)
{}

Assignment<string>::Assignment(const Field<string> &arg,
                               const string& value)
   : BaseAssignment(arg, Constant(value))
{}

void
Assignment<string>::set(const string &v)
{
   BaseAssignment::set(Constant(v));
}

void
Assignment<string>::set(const Field<string> &v)
{
   BaseAssignment::set(v);
}

//------------------------------------------------------------
const vector<string> Assignment< vector<string> >::empty;

Assignment< vector<string> >::Assignment(const Field< vector<string> > &arg,
                               const Field< vector<string> > &value)
   : BaseAssignment(arg, value)
{}

Assignment< vector<string> >::Assignment(const Field< vector<string> > &arg,
                               const  vector<string> & value)
   : BaseAssignment(arg, Constant(value))
{}

void
Assignment< vector<string> >::set(const  vector<string>  &v)
{
   BaseAssignment::set(Constant(v));
}

void
Assignment< vector<string> >::set(const Field< vector<string> > &v)
{
   BaseAssignment::set(v);
}

