// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#ifndef Assignment_hxx
#define Assignment_hxx

#include "BaseAssignment.hxx"

namespace Epoisses {

// forward declare the Field classes
template <typename T> class Field;

//************************************************************
// Note: these classes may not have additional data members
//************************************************************

// Not much I can do to make these easier to define.  The difference between a
// reference and value Assigment is about half the contents of the class.

class Assignment<int> : public BaseAssignment
{
   public:
      Assignment(const Field<int> &arg,
                 const Field<int> &value);

      Assignment(const Field<int> &arg,
                 int value = 0);

      // Implement -= .dlb. generalize to operator eventally
      Assignment(const Field<int> &arg,
                 int value, bool dummy);

      // give a new value to the assignment
      void set(int v);
      
      /// give a new value to the assignment
      void set(const Field<int> &v);
};

class Assignment<bool> : public BaseAssignment
{
   public:
      Assignment(const Field<bool> &arg,
                 const Field<bool> &value);

      Assignment(const Field<bool> &arg,
                 bool value = false);

      // Copy constructor
      Assignment(const Assignment &rhs)
         : BaseAssignment(rhs)
      {}

      // give a new value to the assignment
      void set(bool v);
      
      /// give a new value to the assignment
      void set(const Field<bool> &v);

      virtual ~Assignment() {}
};

class Assignment<double> : public BaseAssignment
{
   public:
      Assignment(const Field<double> &arg,
                 const Field<double> &value);

      Assignment(const Field<double> &arg,
                 double value = 0.0);

      // Copy constructor
      Assignment(const Assignment &rhs)
         : BaseAssignment(rhs)
      {}

      // give a new value to the assignment
      void set(double v);

      /// give a new value to the assignment
      void set(const Field<double> &v);
      
      virtual ~Assignment() {}
};

class Assignment<string> : public BaseAssignment
{
   public:
      Assignment(const Field<string> &arg,
                 const Field<string> &value);

      Assignment(const Field<string> &arg,
                 const string& value = "");

      // Copy constructor
      Assignment(const Assignment &rhs)
         : BaseAssignment(rhs)
      {}

      /// give a new value to the assignment
      void set(const string &v);
      
      /// give a new value to the assignment
      void set(const Field<string> &v);

      virtual ~Assignment() {}
};

class Assignment< vector<string> > : public BaseAssignment
{
   public:
      Assignment(const Field< vector<string> > &arg,
                 const Field< vector<string> > &value);

      Assignment(const Field< vector<string> > &arg,
                 const vector<string> & value = empty);

      // Copy constructor
      Assignment(const Assignment &rhs)
         : BaseAssignment(rhs)
      {}

      /// give a new value to the assignment
      void set(const  vector<string>  &v);
      
      /// give a new value to the assignment
      void set(const Field< vector<string> > &v);

      virtual ~Assignment() {}
   private:
      static const vector<string> empty;
};

} // namespace Epoisses

#endif // Assignment_hxx
