// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#ifndef BaseAssignment_hxx
#define BaseAssignment_hxx

#include "BaseIAssignment.hxx"

namespace Epoisses {

class FilterableInterface;

// common type for Assignments
class BaseAssignment : public PSDBInterface
{
   public:
      BaseAssignment(const BaseField &arg,
                     const BaseField &value)
         : assignment(new BaseIAssignment(arg, value))
      {}
      
      BaseAssignment(const BaseAssignment &rhs)
         : assignment(rhs.assignment)
      {}

      virtual ~BaseAssignment() 
      {}

      BaseAssignment&
      operator=(const BaseAssignment& rhs)
      {
         if (this != &rhs)
         {
            assignment = rhs.assignment;
         }
         return *this;
      }

      const string
      getFieldName() const
      {
         return static_cast<FieldInterface&>(*(assignment->field)).getName();
      }
      
      const string
      getFieldValue() const
      {
         return static_cast<FieldInterface&>(*(assignment->value)).getName();
      }

      // derived Assignments vary?
      virtual void stringify(ostream &s) const
      {
         assignment->stringify(s);
      }

      // don't allow a standard comma operator
      void operator,(const FilterableInterface&)
      {
      }
      
   protected:

      /// reset the assigned value
      void set(const BaseField &value)
      {
         assignment->set(value);
      }

      Sptr<BaseIAssignment> assignment;
};

/**
   Template for external (user visible) Assignment representations.  Each class
   of this template wraps an instance of the BaseIAssignment class in a smart
   pointer.

   The point is that on construction of one of these instances, a
   single shared instance of the BaseIAssignment is heap allocated.
   This wrapper is passed by value, which is cheap.

   Further, the indirection allows bind to write to heap memory.

   The generated classes have nice names like Assignment<int>.
*/   
template <typename T>
class Assignment
{
   private:
      Assignment()
      {
         // don't allow implicit instantiation
         //BOOST_STATIC_ASSERT("you may use only the pre-defined classes"[0]);
      }
};

/**
   Allow arrays of heterogeneous Assignments. */
class Assignment<void> : public BaseAssignment
{
public:
      // Copy constructor
      Assignment(const BaseAssignment &rhs)
         : BaseAssignment(rhs)
      {}
   private:
      Assignment<void>();
};

} // namespace Epoisses

#endif // BaseAssignment_hxx
