// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#ifndef DBTYPES_H
#define DBTYPES_H

#include <map>
#include <string>
#include <vector>

#include "global.h"
#include "StringHash.hxx"

/// Supported Data types
enum Datatype
{
    STRINGTYPE,
    INTEGERTYPE,
    FLOATTYPE,
    BOOLTYPE,
    TEXTBLOBTYPE,
    BINBLOBTYPE,
    TIMESTAMPTYPE,
    DATETYPE,
    TIMETYPE,
};

Datatype stringToType(const string& typeString);

/// Supported Order Types
enum Ordertype
{
    ASCENDING,
    DESCENDING,
    UNDEFINED
};

struct DBInfo
{
  // "MYSQL" - "LDAP" - ????
  string dbBackend;

  string database;
  string host;
  int port;
  string user;
  string password;
  string dn;
};


/// When using DBCreate() or DBDrop() to create or dop domains, 
/// set the ExistenceCheck argument to one of the following:
static const bool IGNOREEXISTENCE = false;
///
static const bool CHECKEXISTENCE = true;

/// Maximum length of a Database query string
//static const unsigned short MAXQUERYLENGTH = 512;

static const unsigned short MAXQUERYLENGTH = 32768;  
// 2^15 = 32768 (a short is 16 bits)

/// Type used to describe field datatype
typedef vector<Datatype> FieldListDatatype;

/// Type used to describe a single record
typedef vector<string> ResultRecordtype;

/// Type used to describe a group of records
typedef vector<ResultRecordtype> ResultTabletype;

/// Type used to describe a group of fields
typedef vector< string > FieldListtype;

/// Type used to describe a group of Field-Value pairs
typedef map< string, string > FieldValuetype;



#endif // DBTYPES_H
