// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#include "FieldList.hxx"
#include "BaseField.hxx"

using namespace Epoisses;

void
IFieldList:: stringify(ostream &str) const
{
   // mapcar equivalent?
   for(vector < Sptr <BaseIField> >::const_iterator it = contents.begin();
       it != contents.end();
       it++)
   {
      (*it)->stringify(str);
      str << ", "; // or something
   }
}

//------------------------------------------------------------
const FieldList FieldList::Empty;

// non-templatized method that does the work given the size
void
FieldList::fromArray(const Field<void> arr[], int sz)
{
   // add each of the Fields in the array
   for (int i = 0; i < sz; i++)
   {
      add(arr[i]);
   }
}

FieldList::FieldList(const BaseColumnField &field)
   : fieldList(new IFieldList(field.internalField))
{}

FieldList::FieldList()
   : fieldList(new IFieldList())
{}

FieldList
FieldList::add(const BaseColumnField &field)
{
   fieldList->add(field.internalField);
   return *this;
}

FieldList
FieldList::addAll(const FieldList &field)
{
   fieldList->addAll(*field.fieldList);
   return *this;
}

FieldList
FieldList::operator,(const BaseColumnField& field)
{
   add(field);
   return *this;
}

void
FieldList::stringify(ostream &str) const
{
   fieldList->stringify(str);
}

// static
FieldList
FieldList::fromList(const FieldListtype &lt)
{
   FieldList l;
   return l;
}

//------------------------------------------------------------
// Global operator

namespace Epoisses {

// create a field list from two fields
FieldList
operator,(const BaseColumnField &field1,
          const BaseColumnField &field2)
{
   FieldList fieldList(field1);
   fieldList.add(field2);
   return fieldList;
}

// prepend a field to a field list
FieldList
operator,(const BaseColumnField &field,
          const FieldList &fieldList)
{
   FieldList fieldListC(fieldList);
   fieldListC.add(field);
   return fieldListC;
}
 
} // namespace Epoisses
