/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const PS_Common_cxx_Version = "$Id: PSCommon.cxx,v 1.1 2002/09/30 04:55:12 jason Exp $";

#include <ace/SOCK_Stream.h>
#include <boost/lexical_cast.hpp>

#include "PSCommon.hxx"
#include "Logger.hxx"
#include "VNetworkException.hxx"

using namespace Epoisses;

#define VOCAL_SUBSYSTEM Vocal::Subsystem::PROV

///
const string PSCommon::OPEN_BRACKET("<");
///
const string PSCommon::END_ELEMENT_OPEN_BRACKET("</");
///
const string PSCommon::CLOSE_BRACKET(">");


/* Common HTTP Method tags reserved for PS Protocol
 */
///
const string PSCommon::GET_RECORDS_IN_DOMAIN_STR("GetRecordsInDomain");
///
const string PSCommon::GET_FIELDS_IN_DOMAIN_STR("GetFieldsInDomain");
///
const string PSCommon::SET_RECORDS_IN_DOMAIN_STR("SetRecordsInDomain");
///
const string PSCommon::ADD_RECORD_IN_DOMAIN_STR("AddRecordInDomain");
///
const string PSCommon::DELETE_RECORDS_IN_DOMAIN_STR("DeleteRecordsInDomain");
///
const string PSCommon::SESSION_OPEN_STR("SessionOpen");
///
const string PSCommon::SESSION_CLOSE_STR("SessionClose");
///
const string PSCommon::SESSION_SEARCH_DB_STR("SessionSearchDB");
///
const string PSCommon::SESSION_GET_DB_RESULT_STR("SessionGetDBResult");
///
const string PSCommon::SESSION_SORT_DB_RESULT_STR("SessionSortDBResult");
///
const string PSCommon::SESSION_SET_DB_RESULT_STR("SessionSetDBResult");
///
const string PSCommon::SESSION_DELETE_ALL_STR("SessionDeleteAll");
///
const string PSCommon::GET_XML_DESC_STR("GetXMLDescription");
///
const string PSCommon::LIST_XML_CAT_STR("ListXMLCategories");
///
const string PSCommon::LIST_XML_DESC_STR("ListXMLDescriptions");
///
const string PSCommon::REGISTER_DOMAIN_FOR_UPDATE_STR("RegisterDomainForUpdate");
///
const string PSCommon::REGISTER_RECORD_FOR_UPDATE_STR("RegisterRecordForUpdate");
///
const string PSCommon::UPDATE_DOMAIN_STR("UpdateDomain");
///
const string PSCommon::UPDATE_RECORD_STR("UpdateRecord");
///
const string PSCommon::UN_REGISTER_DOMAIN_FOR_UPDATE_STR("UnRegisterDomainForUpdate");
///
const string PSCommon::UN_REGISTER_RECORD_FOR_UPDATE_STR("UnRegisterRecordForUpdate");

/// Foreign Update strings
const string PSCommon::REGISTER_DOMAIN_FOR_FOREIGN_UPDATE_STR("RegisterDomainForForeignUpdate");
const string PSCommon::FOREIGN_UPDATE_DOMAIN_STR("ForeignUpdateDomain");
const string PSCommon::UN_REGISTER_DOMAIN_FOR_FOREIGN_UPDATE_STR("UnRegisterDomainForForeignUpdate");


/* Common XML tags reserved for PS Protocol
 */

/// GET/PUT/POST request reserved keywords:
const string PSCommon::ARGUMENTS_STR("arguments");
const string PSCommon::ARGUMENTS_START("<arguments>");
const string PSCommon::ARGUMENTS_END("</arguments>");
///
const string PSCommon::DOMAIN_STR("domain");
const string PSCommon::DOMAIN_START("<domain>");
const string PSCommon::DOMAIN_END("</domain>");
///
const string PSCommon::FOREIGN_DOMAIN_STR("foreigndomain");
const string PSCommon::FOREIGN_DOMAIN_START("<foreigndomain>");
const string PSCommon::FOREIGN_DOMAIN_END("</foreigndomain>");
///
const string PSCommon::FOREIGN_FIELD_STR("foreignfield");
const string PSCommon::FOREIGN_FIELD_START("<foreignfield>");
const string PSCommon::FOREIGN_FIELD_END("</foreignfield>");
///
const string PSCommon::FILTER_STR("filter");
const string PSCommon::FILTER_START("<filter>");
const string PSCommon::FILTER_END("</filter>");
///
const string PSCommon::FIELDLIST_STR("fieldlist");
const string PSCommon::FIELDLIST_START("<fieldlist>");
const string PSCommon::FIELDLIST_END("</fieldlist>");
///
const string PSCommon::ORDER_BY_STR("orderby");
const string PSCommon::ORDER_BY_START("<orderby>");
const string PSCommon::ORDER_BY_END("</orderby>");
///
const string PSCommon::LISTITEM_STR("listitem");
const string PSCommon::LISTITEM_START("<listitem>");
const string PSCommon::LISTITEM_END("</listitem>");

/* PUT request and GET response reserved k/w for signifying a 
 * list of k/v pairs:
 */
const string PSCommon::CHANGE_TYPE_STR("changetype");
const string PSCommon::CHANGE_TYPE_START("<changetype>");
const string PSCommon::CHANGE_TYPE_END("</changetype>");

const string PSCommon::FIELDVALUE_STR("fieldvalue");
const string PSCommon::FIELDVALUE_START("<fieldvalue>");
const string PSCommon::FIELDVALUE_END("</fieldvalue>");

/// GET response reserved keywords:
const string PSCommon::RESPONSE_STR("response");
const string PSCommon::RESPONSE_START("<response>");
const string PSCommon::RESPONSE_END("</response>");
///
const string PSCommon::RESULTCOUNT_STR("resultcount");
const string PSCommon::RESULTCOUNT_START("<resultcount>");
const string PSCommon::RESULTCOUNT_END("</resultcount>");
///
const string PSCommon::MATCH_STR("match");
const string PSCommon::MATCH_START("<match>");
const string PSCommon::MATCH_END("</match>");

/// Session-specific reserved keywords:
const string PSCommon::EXPIRETIME_STR("expiretime");    // for session open
const string PSCommon::EXPIRETIME_START("<expiretime>");    // for session open
const string PSCommon::EXPIRETIME_END("</expiretime>");    // for session open
///
const string PSCommon::TOKEN_STR("token");
const string PSCommon::TOKEN_START("<token>");
const string PSCommon::TOKEN_END("</token>");
///
const string PSCommon::START_STR("start");         // start index
const string PSCommon::START_START("<start>");         // start index
const string PSCommon::START_END("</start>");         // start index
///
const string PSCommon::END_STR("end");             // end index
const string PSCommon::END_START("<end>");             // end index
const string PSCommon::END_END("</end>");             // end index

/// sort related reserved keywords:
const string PSCommon::SORTTYPE_STR("sorttype");
const string PSCommon::SORTTYPE_START("<sorttype>");
const string PSCommon::SORTTYPE_END("</sorttype>");
///
const string PSCommon::SORTFIELD_STR("sortfield");
const string PSCommon::SORTFIELD_START("<sortfield>");
const string PSCommon::SORTFIELD_END("</sortfield>");
///
const string PSCommon::SORT_ASCEND_STR("ascend");         
///
const string PSCommon::SORT_DESCEND_STR("descend");

/// error reserved keyword:
const string PSCommon::ERROR_STR("error");
const string PSCommon::ERROR_START("<error>");
const string PSCommon::ERROR_END("</error>");

/// xml desc reserved keyword:
const string PSCommon::XMLCATEGORY_STR("xmlcategory");
const string PSCommon::XMLCATEGORY_START("<xmlcategory>");
const string PSCommon::XMLCATEGORY_END("</xmlcategory>");

const string PSCommon::XMLDESC_STR("xmldescription");
const string PSCommon::XMLDESC_START("<xmldescription>");
const string PSCommon::XMLDESC_END("</xmldescription>");

/// Registration stuff
const string PSCommon::PRIMARYKEY_STR("primarykey");
const string PSCommon::PRIMARYKEY_START("<primarykey>");
const string PSCommon::PRIMARYKEY_END("</primarykey>");

const string PSCommon::REGISTER_HOSTPORT_STR("register_hostport");
const string PSCommon::REGISTER_HOSTPORT_START("<register_hostport>");
const string PSCommon::REGISTER_HOSTPORT_END("</register_hostport>");

const string PSCommon::UPDATE_UID_STR("update_uid");
const string PSCommon::UPDATE_UID_START("<update_uid>");
const string PSCommon::UPDATE_UID_END("</update_uid>");
///

const string PSCommon::ChangeTypes[] = {"ADD", "DELETE", "SET", "!bad change type!"};

void PSCommon::xmlAddTag( const string& tag,  string& target ) {
    target += tag + "\r\n";
};

void PSCommon::xmlAddTagValue( const string& startTag, const string& val,
                       const string& endTag,   string& target ) {
    target += startTag + val + endTag + "\r\n";
}

string
PSCommon::readHttpMessage(ACE_SOCK_Stream& strm)
{
   string header;
   header.reserve(120);
   
   // read a character at a time
   char charBuf[2];
   size_t nRead = 0;
   int cr=0;
   // read until just after second CR/LF
   for (int i = 0; i < 2; i++)
   {
      while ((cr = strm.recv_n(charBuf, 1)) > 0)
      {
         charBuf[1] = 0;
         header += charBuf;
         if (charBuf[0] == 10)
         {
            break; // CR
         }
      }

      if (cr == -1) // connection closed / error condition
      {
         DebugLog (<< "connection to pserver error: "  << strerror(errno) );
         strm.close();
         throw VNetworkException("Connection closed", __FILE__, __LINE__);
      }
      if (cr == 0) // end of file
      {
         DebugLog (<< "connection to pserver terminated by server" );
         strm.close();
         throw VNetworkException("Header ends early?", __FILE__, __LINE__);
      }
   }

   static const string contentLength("Content-Length: ");
   // extract content length
   const string::size_type cp = header.find(contentLength);
   if (cp == string::npos)
   {
      DebugLog (<< "Bad http message, content-length missing: <" << header << ">");
      throw VNetworkException("Content-Length missing", __FILE__, __LINE__);
   }

   const int cl = boost::lexical_cast<int>(header.substr(cp + contentLength.size()));
   //DebugLog(<< "sendRequest length=" << cl << " header=<" << header << ">");

   // know how big the string will be
   string message(header, 0, header.size() + cl+3);
   // read the rest of the message
   char rdbuf[cl+3];
   if ((cr = strm.recv_n((void*)rdbuf, cl+2, 0, &nRead)) > 0)
   {
      rdbuf[nRead] = 0;
      message += rdbuf;
   }
   else
   {
      if (cr == -1) // connection closed / error condition
      {
         DebugLog (<< "connection to pserver error: "  << strerror(errno) );
         strm.close();
         throw VNetworkException("Connection closed", __FILE__, __LINE__);
      }
      if (cr == 0) // end of file
      {
         DebugLog (<< "connection to pserver terminated by server: ");
         strm.close();
         throw VNetworkException("Message ends early?", __FILE__, __LINE__);
      }
   }
   //DebugLog(<< "sendRequest response=<" << message << ">");
   return message;
}
