/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const PSReqXmlContentHandlersrCxxVersion = "$Id: PSReqXmlContentHandlers.cxx,v 1.1 2002/09/30 04:55:12 jason Exp $";


// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <stdio.h>
#include <xercesc/util/XMLUniDefs.hpp>
#include <boost/lexical_cast.hpp>

#define VOCAL_SUBSYSTEM Vocal::Subsystem::PROV

#include "Logger.hxx"
#include "PSCommon.hxx"
#include "XmlContent.hxx"
#include "PSReqXmlContentHandlers.hxx"
#include "VNetworkException.hxx"

#include "Domain.hxx"
#include "BaseField.hxx"
#include "FieldList.hxx"
#include "AssignmentList.hxx"
#include "Filter.hxx"
#include "Field.hxx" // for Constant/BaseField comparison

using namespace Epoisses;

// ---------------------------------------------------------------------------
//  PSReqXmlContentHandlers: Constructors
// ---------------------------------------------------------------------------

PSReqXmlContentHandlers::PSReqXmlContentHandlers(Sptr<PSDBRequest> target)
   : myCharacters("")
{
   myParsedContent = target;    // store parsed xml content here
}

// ---------------------------------------------------------------------------
//  PSReqXmlContentHandlers: Destructor
// ---------------------------------------------------------------------------
PSReqXmlContentHandlers::~PSReqXmlContentHandlers()
{
}


// ---------------------------------------------------------------------------
//  PSReqXmlContentHandlers: Overrides of the SAX ErrorHandler interface
// ---------------------------------------------------------------------------
void PSReqXmlContentHandlers::error(const SAXParseException& e)
{
   DebugLog (<< "SAX Handler throws error. " 
             << " file: " << StrX(e.getSystemId())
             << " line: " << e.getLineNumber()
             << " char: " << e.getColumnNumber()
             << " : " << StrX(e.getMessage()) );

   throw VNetworkException(StrX(e.getMessage()).localForm(), __FILE__, __LINE__);
}

void PSReqXmlContentHandlers::fatalError(const SAXParseException& e)
{
   DebugLog (<< "SAX Handler throws fatal error. " 
             << " file: " << StrX(e.getSystemId())
             << " line: " << e.getLineNumber()
             << " char: " << e.getColumnNumber()
             << " : " << StrX(e.getMessage()) );

   throw VNetworkException(StrX(e.getMessage()).localForm(), __FILE__, __LINE__);
}

void PSReqXmlContentHandlers::warning(const SAXParseException& e)
{
   DebugLog (<< "SAX Handler throws warning. " 
             << " file: " << StrX(e.getSystemId())
             << " line: " << e.getLineNumber()
             << " char: " << e.getColumnNumber()
             << " : " << StrX(e.getMessage()) );

   throw VNetworkException(StrX(e.getMessage()).localForm(), __FILE__, __LINE__);
}


// ---------------------------------------------------------------------------
//  PSReqXmlContentHandlers: Overrides of the SAX DTDHandler interface
// ---------------------------------------------------------------------------
void PSReqXmlContentHandlers::unparsedEntityDecl(const     XMLCh* const name
                                                 , const   XMLCh* const publicId
                                                 , const   XMLCh* const systemId
                                                 , const   XMLCh* const notationName)
{
   // Not using DTD presently
}


void PSReqXmlContentHandlers::notationDecl(const   XMLCh* const name
                                           , const XMLCh* const publicId
                                           , const XMLCh* const systemId)
{
   // Not using DTD presently
}


// ---------------------------------------------------------------------------
//  PSReqXmlContentHandlers: Overrides of the SAX DocumentHandler interface
// ---------------------------------------------------------------------------
void PSReqXmlContentHandlers::characters(const     XMLCh* const    chars,
                                         const   unsigned int    length)
{
   string& fn = myFields.top();   // get current field name
   string  val= StrX( chars ).localForm();
   myCharacters = val;

   //DebugLog( << "PSReqXmlContentHandlers::characters=" << val);
    
   if (myParsedContent->blobFlag == true)
   {
      myParsedContent->setXmlBlob(val);
      return;
   }

   if (fn == PSCommon::ARGUMENTS_STR)
   {
      myParsedContent->setArguments(val);
   }
   else if ((fn == PSCommon::FIELDVALUE_STR) ||
            (fn == PSCommon::FIELDLIST_STR))
   {
      myParsedContent->setKeywordFlag(fn, val);
      myParsedContent->tagLevel = 1;
   }
   else if (fn == PSCommon::DOMAIN_STR)
   {
      myParsedContent->setDomain( val );
      //DebugLog( << "domain found: " << val);
   }
   else if (fn == PSCommon::FILTER_STR)
   {
      myParsedContent->setFilter( val );
      //DebugLog( <<  "filter rules = " << val);
   }
   else if (fn == PSCommon::TOKEN_STR)
   {
      myParsedContent->setSessionID( val );
      //DebugLog( <<  "session token id = " << val);
   }
   else if (fn == PSCommon::LISTITEM_STR)
   {
      myParsedContent->addFieldList(val); // add to end of the fieldlist
      //DebugLog( <<  "List item = " << val);
   }
   else if (fn == PSCommon::ORDER_BY_STR)
   {
      myParsedContent->setOrderBy(val); // add to end of the fieldlist
      //DebugLog( <<  "Order by = " << val);
   }
   else if (fn == PSCommon::XMLCATEGORY_STR)
   {
      myParsedContent->setXmlCategory( val );
      //DebugLog( <<  "Xml category found: " << val);
   }
   else if (fn == PSCommon::XMLDESC_STR)
   {
      myParsedContent->setXmlDescName( val );
      //DebugLog( <<  "Xml description name found: " << val);
   }
   else if (fn == PSCommon::PRIMARYKEY_STR)
   {
      myParsedContent->addFieldList( val );
      //DebugLog( <<  "primary key = " << val);
   }    
   else if (fn == PSCommon::REGISTER_HOSTPORT_STR)
   {
      myParsedContent->setRegisterHostport(val);
      //DebugLog (<<  "HostPort = " << val);
   }
   else if (fn == PSCommon::CHANGE_TYPE_STR)
   {
      myParsedContent->setChangeType(val);
      //DebugLog (<<  "change type = " << val);
   }
   else if (fn == PSCommon::UPDATE_UID_STR)
   {
      myParsedContent->setFunctorUId(val);
      //DebugLog (<<  "Functor UId = " << val);
   }
   else if (fn == PSCommon::FOREIGN_DOMAIN_STR)
   {
      myParsedContent->setForeignDomain(val);
      //DebugLog (<<  "Foreign domain = " << val);
   }
   else if (fn == PSCommon::FOREIGN_FIELD_STR)
   {
      myParsedContent->setForeignField(val);
      //DebugLog (<<  "Foreign field = " << val);
   }
   else // anything else, stuff it into myParsedContent's list of kv pairs
   {
      myParsedContent->setFieldValue(fn, val);
      //DebugLog( <<  "added " << fn << "/": << val << " to table");
   }
    
   //DebugLog(<< "*** end characters ");
}


void PSReqXmlContentHandlers::endDocument()
{
   //DebugLog( <<  "endDocument() reached" );
   assert(myFields.empty());
}


void PSReqXmlContentHandlers::endElement(const XMLCh* const uri, 
                                         const XMLCh* const localname, 
                                         const XMLCh* const qname)
{
   string fn = StrX( qname ).localForm();
   //DebugLog( <<  "***** endElement qname = " << fn);
   myFields.pop();      

   myParsedContent->tagLevel--;

   if (myParsedContent->blobFlag == true) 
   {
      if (myParsedContent->tagLevel == 1)
      {
         myParsedContent->setFieldValue(fn, myParsedContent->getXmlBlob());
         //DebugLog(<<  "Final add fieldname [" << fn << "] with blob [" << myParsedContent->getXmlBlob() << "] into table");

         myParsedContent->clearXmlBlob();
         //myParsedContent->blobState = false;
         //myParsedContent->tagLevel = 0;
         //myParsedContent->xmlBlob = "";
         return;
      }
      //DebugLog (<<  "Add end of element into xmlBlob: "<< fn);
      string temp = PSCommon::END_ELEMENT_OPEN_BRACKET + fn + PSCommon::CLOSE_BRACKET;
      myParsedContent->setXmlBlob(temp);
      //DebugLog (<<  "Xml Blob = " << myParsedContent->getXmlBlob());
   }
   else if (myCharacters.empty())
   {
      //DebugLog(<<  "added field [" << fn << "] (with empty value) to table");
      myParsedContent->setFieldValue(fn, "");
   }

}


void PSReqXmlContentHandlers::ignorableWhitespace( const   XMLCh* const chars
                                                   ,const  unsigned int length)
{
   //DebugLog( <<  "*********  ignorableWhitespace() called " );
}



void PSReqXmlContentHandlers::startDocument()
{
   //DebugLog( <<   "startDocument() ************ ");
}


void PSReqXmlContentHandlers::startElement(const   XMLCh* const    uri,
					   const   XMLCh* const    localname,
					   const   XMLCh* const    qname,
					   const   Attributes&		attributes)
{
   string fn = StrX(qname).localForm();  // field name
   //DebugLog( <<  " *** startElement qname : " << fn);
   myFields.push( fn );   // remember it on the stack
   myCharacters = "";

   if (myParsedContent->tagLevel)
   {
      myParsedContent->tagLevel++;
      //DebugLog (<<  "BLOB Counter = " << myParsedContent->blobCounter);
      if (myParsedContent->tagLevel > 2)
      {
         myParsedContent->blobFlag = true;
      }
   }

   if (myParsedContent->blobFlag == true)
   {
      //DebugLog (<<  "Add start of element into xmlBlob: " << fn.c_str());
      string temp = PSCommon::OPEN_BRACKET + fn + PSCommon::CLOSE_BRACKET;
      myParsedContent->setXmlBlob(temp);
      //DebugLog (<<  "Xml Blob = " << myParsedContent->getXmlBlob());
   }

}


// *****************************************************************
// Single Transaction XML Content Builder
// *****************************************************************

string PSReqXmlContentHandlers::buildGetXmlDescriptionReq(
   const string& xmlCategory, 
   const string& xmlDescription)
   throw( VException& ) 
{
   //DebugLog(<<  "buildGetXMLDescriptionReq() category "  << xmlCategory << "," << xmlDescription);

   /* build the XML content
    */ 

   string content;

   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
   PSCommon::xmlAddTagValue( PSCommon::XMLCATEGORY_START, xmlCategory, 
                             PSCommon::XMLCATEGORY_END, content );
   PSCommon::xmlAddTagValue( PSCommon::XMLDESC_START, xmlDescription, 
                             PSCommon::XMLDESC_END, content );
   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    
   //DebugLog(<<  "GetXmlDescription xml body: " << content);
   return content;
}

string PSReqXmlContentHandlers::buildListXmlCatagoriesReq()
   throw( VException& ) 
{
   //DebugLog(<<  "buildListXMLCatagoriesReq()");
    
   /* build the XML content
    */ 

   string body;
   // this message has an empty body
    
   //DebugLog(<<  "ListXmlCatagories");
   return body;
}

string PSReqXmlContentHandlers::buildListXmlDescriptionsReq(
   const string& xmlCategory) 
   throw( VException& ) 
{
   //DebugLog(<<  "buildListXMLDescriptionsReq() category " << xmlCategory);

   /* build the XML content
    */ 

   string content;

   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
   PSCommon::xmlAddTagValue( PSCommon::XMLCATEGORY_START, xmlCategory, 
                             PSCommon::XMLCATEGORY_END, content );
   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    
   //DebugLog(<<  "ListXmlDescriptions xml body: " << content);
   return content;
}

string PSReqXmlContentHandlers::buildGetRecordsInDomainReq(
   const Domain& domain, 
   const Filter& filter,
   const FieldList& fieldList)
   throw( VException& ) 
{
   //DebugLog(<<  "buildGetRecordsInDomainReq() domain " << domain << ", rules " << filter.toString());

   /* build the XML content
    */ 

   string content;

   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
   PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain.getName(), 
                             PSCommon::DOMAIN_END, content );
   if (filter != Filter::None)
   {
      PSCommon::xmlAddTagValue( PSCommon::FILTER_START, filter.toString(),
                                PSCommon::FILTER_END, content );
   }
   if (!fieldList.empty())
   {
      PSCommon::xmlAddTag( PSCommon::FIELDLIST_START, content );

      for (FieldList::Contents::const_iterator i = fieldList.begin();
           i != fieldList.end(); i++)
      {
         PSCommon::xmlAddTagValue( PSCommon::LISTITEM_START,
                                   (*i)->getName(), PSCommon::LISTITEM_END, content );
      } 
      PSCommon::xmlAddTag( PSCommon::FIELDLIST_END, content );
   }
   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    
   //DebugLog(<<  "GetRecordsInDomain xml body: "<< content);
   return content;
}

string 
PSReqXmlContentHandlers::buildGetRecordsInDomainReq(const Domain& domain,
                                                    const Filter& filter,
                                                    const FieldList& fieldList,
                                                    const Field<void>& orderBy)
   throw( VException& )
{
   //DebugLog(<<  "buildGetRecordsInDomainReq() domain " << domain.getName() << ", rules " << filter.toString());

   /* build the XML content
    */ 

   string content;

   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
   PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain.getName(), 
                             PSCommon::DOMAIN_END, content );
   if (filter != Filter::None)
   {
      PSCommon::xmlAddTagValue( PSCommon::FILTER_START, filter.toString(),
                                PSCommon::FILTER_END, content );
   }
   PSCommon::xmlAddTagValue( PSCommon::ORDER_BY_START, orderBy.toString(), 
                             PSCommon::ORDER_BY_END, content );
   if (!fieldList.empty())
   {
      PSCommon::xmlAddTag( PSCommon::FIELDLIST_START, content );

      for (FieldList::Contents::const_iterator i = fieldList.begin();
           i != fieldList.end(); i++)
      {
         PSCommon::xmlAddTagValue( PSCommon::LISTITEM_START,
                                   (*i)->getName(), PSCommon::LISTITEM_END, content );
      } 
      PSCommon::xmlAddTag( PSCommon::FIELDLIST_END, content );
	
   }
   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    
   //DebugLog(<<  "GetRecordsInDomain xml body: " << content);
   return content;
}

///
string PSReqXmlContentHandlers::buildGetFieldsInDomainReq( 
   const Domain& domain )
   throw( VException& ) 
{
   //DebugLog(<<  "buildGetFieldsInDomainReq() domain " << domain.getName());

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
   PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain.getName(),
                             PSCommon::DOMAIN_END, content );
   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );

   //DebugLog(<<  "GetFieldsInDomain xml body: " << content);
   return content;
}

/// update existing records with keyword/value pairs provided
string 
PSReqXmlContentHandlers::buildSetRecordsInDomainReq( 
   const Domain& domain, 
   const Filter& filter,
   const AssignmentList& assignments )
   throw (VException& )
{
   //DebugLog(<<  "buildSetRecordsInDomainReq() domain " << domain.getName());

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
   PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain.getName(),
                             PSCommon::DOMAIN_END, content );
   if (filter != Filter::None)
   {
      PSCommon::xmlAddTagValue( PSCommon::FILTER_START, filter.toString(),
                                PSCommon::FILTER_END, content );
   }
   if (!assignments.empty())
   {
      string startField, endField;
	
      PSCommon::xmlAddTag( PSCommon::FIELDVALUE_START, content );
      for ( AssignmentList::const_iterator itr = 
               assignments.begin();
            itr != assignments.end(); itr++ )
      {
         startField = PSCommon::OPEN_BRACKET + itr->getFieldName() +
            PSCommon::CLOSE_BRACKET;
         endField = PSCommon::END_ELEMENT_OPEN_BRACKET +
            itr->getFieldName() + 
            PSCommon::CLOSE_BRACKET;
         PSCommon::xmlAddTagValue( startField, itr->getFieldValue(),
                                   endField, content);
      }
      PSCommon::xmlAddTag( PSCommon::FIELDVALUE_END, content );
	
   }
   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );

   //DebugLog(<<  "SetRecordsInDomain xml body: " << content);
   return content;
}

/// insert a new record
string 
PSReqXmlContentHandlers::buildAddRecordInDomainReq( const Domain& domain, 
                                                    const AssignmentList& assignments)
   throw( VException& ) 
{
   //DebugLog(<<  "AddRecordsInDomain reuses SetRecordsInDomainReq()");

   return ( buildSetRecordsInDomainReq( domain, Filter::None, assignments ) );
}

string
PSReqXmlContentHandlers::buildDeleteRecordsInDomainReq( const Domain& domain,
                                                        const Filter& filter )
   throw( VException& ) 
{
   //DebugLog(<<  "buildDeleteRecordsInDomainReq, domain " << domain.toString() << ", filter " <<filter.toString());
   /* build the XML content
    */

   string content;

   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
   PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain.getName(),
                             PSCommon::DOMAIN_END, content );
   if (filter != Filter::None)
   {
      PSCommon::xmlAddTagValue( PSCommon::FILTER_START, filter.toString(),
                                PSCommon::FILTER_END, content );
   }
   // if not filter rules, then delete the entire table
   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    
   //DebugLog(<<  "DeleteRecordsInDomain xml body: " << content);
   return content;
}


// *****************************************************************
// Session related XML Content Builder
// *****************************************************************

string 
PSReqXmlContentHandlers::buildSessionOpenReq( int timeInSeconds ) 
   throw(VException&)
{ 
   //DebugLog(<<  "buildSessionOpenReq() time:  " << timeInSeconds << " seconds");

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
   PSCommon::xmlAddTagValue( PSCommon::EXPIRETIME_START, boost::lexical_cast<string>(timeInSeconds).c_str(),
                             PSCommon::EXPIRETIME_END, content );
   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );

   //DebugLog(<<  "SessionOpen xml body: " << content);
   return content;
}
         

///
string
PSReqXmlContentHandlers::buildSessionCloseReq( const string& sessionID ) throw( VException& )
{ 
   //DebugLog(<<  "buildSessionCloseReq() token:  "<< sessionID);

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
   PSCommon::xmlAddTagValue( PSCommon::TOKEN_START, sessionID,
                             PSCommon::TOKEN_END, content );
   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );

   //DebugLog(<<  "SessionOpen xml body: " << content);
    
   return content; 
}

///
string 
PSReqXmlContentHandlers::buildSessionSearchDBReq( 
   const string& sessionID, 
   const Domain& domain, 
   const Filter& filter,
   const FieldList& fieldList,  
   PSCommon::PSSortType sType,
   const BaseField& sortField ) throw(VException&)
{

   /*
   DebugLog(<<  "buildSessionSearchDBReq() token:  "  << sessionID
            << " domain=" << domain.getName()
            << " filter=" << filter.toString()
            << " sortType=" << int(sType)
            << " sortField=" << sortField.toString());
   */

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
   PSCommon::xmlAddTagValue( PSCommon::TOKEN_START, sessionID,
                             PSCommon::TOKEN_END, content );
   PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain.getName(),
                             PSCommon::DOMAIN_END, content );
   if (filter != Filter::None)
   {
      PSCommon::xmlAddTagValue( PSCommon::FILTER_START, filter.toString(),
                                PSCommon::FILTER_END, content );
   }
   if (fieldList.empty())
   {
      PSCommon::xmlAddTag( PSCommon::FIELDLIST_START, content );
      for (FieldList::Contents::const_iterator i = fieldList.begin();
           i != fieldList.end(); i++)
      {
         PSCommon::xmlAddTagValue( PSCommon::LISTITEM_START,
                                   (*i)->getName(), PSCommon::LISTITEM_END, content );
      }
      PSCommon::xmlAddTag( PSCommon::FIELDLIST_END, content );
   }
    
   if (sType == PSCommon::SORT_ASCENDING)
   {
      PSCommon::xmlAddTagValue( PSCommon::SORTTYPE_START,
                                PSCommon::SORT_ASCEND_STR, PSCommon::SORTTYPE_END, content );
   }
   else if (sType == PSCommon::SORT_DESCENDING)
   {
      PSCommon::xmlAddTagValue( PSCommon::SORTTYPE_START,
                                PSCommon::SORT_DESCEND_STR, PSCommon::SORTTYPE_END, content );
   }
   else
   {
      //DebugLog (<<  "No sorting type selected.  Default = \"ascending\"");
      PSCommon::xmlAddTagValue( PSCommon::SORTTYPE_START,
                                PSCommon::SORT_ASCEND_STR, PSCommon::SORTTYPE_END, content );
   }


   if (sortField != BaseField::None)
   {
      PSCommon::xmlAddTagValue( PSCommon::SORTFIELD_START,
                                sortField.getName(), PSCommon::SORTFIELD_END, content );
   }
        
   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    
   //DebugLog(<<  "SessionSearchDB xml body: " << content);

   return content;
}

///
string
PSReqXmlContentHandlers::buildSessionGetDBResultReq( 
   const string& sessionID, int startIdx,
   int endIdx, const FieldList& fieldList)
   throw (VException&)
{
   /*
   DebugLog(<< "buildSessionGetDBResultReq() token: " << sessionID
            << "startIdx=" << startIdx << "endIdx=" << endIdx);
   */

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
   PSCommon::xmlAddTagValue( PSCommon::TOKEN_START, sessionID,
                             PSCommon::TOKEN_END, content );
   PSCommon::xmlAddTagValue( PSCommon::START_START, boost::lexical_cast<string>(startIdx).c_str(),
                             PSCommon::START_END, content );
   PSCommon::xmlAddTagValue( PSCommon::END_START, boost::lexical_cast<string>(endIdx).c_str(),
                             PSCommon::END_END, content );
    
   if (fieldList.empty())
   {
      PSCommon::xmlAddTag( PSCommon::FIELDLIST_START, content );
      for (FieldList::Contents::const_iterator i = fieldList.begin();
           i != fieldList.end(); i++)
      {
         PSCommon::xmlAddTagValue( PSCommon::LISTITEM_START,
                                   (*i)->getName(), PSCommon::LISTITEM_END, content );
      }
      PSCommon::xmlAddTag( PSCommon::FIELDLIST_END, content );
   }
    
   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );

   //DebugLog(<<  "SessionGetDBResult xml body: " << content);

   return content;

}

///
string
PSReqXmlContentHandlers::buildSessionSetDBResultReq( 
   const string& sessionID, 
   const map< string, string>& fieldValPairs)
   throw (VException&)
{
   //DebugLog(<<  "buildSessionSetDBReq() sessionID " << sessionID);

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
   PSCommon::xmlAddTagValue( PSCommon::TOKEN_START, sessionID,
                             PSCommon::TOKEN_END, content );
    
   if (fieldValPairs.empty())
   {
      string startField, endField;
	
      PSCommon::xmlAddTag( PSCommon::FIELDVALUE_START, content );
      for ( map< string, string >::const_iterator itr =
               fieldValPairs.begin();
            itr != fieldValPairs.end(); itr++ )
      {
         startField = PSCommon::OPEN_BRACKET + itr->first +
            PSCommon::CLOSE_BRACKET;
         endField = PSCommon::END_ELEMENT_OPEN_BRACKET +
            itr->first +
            PSCommon::CLOSE_BRACKET;
         PSCommon::xmlAddTagValue( startField, itr->second,
                                   endField, content);
      }
      PSCommon::xmlAddTag( PSCommon::FIELDVALUE_END, content );
	
   }
   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );

   //DebugLog(<<  "SessionSetDBResult xml body: " << content);
   return content;

}

///
string
PSReqXmlContentHandlers::buildSessionSortDBResultReq( 
   const string& sessionID, 
   PSCommon::PSSortType sType,
   const BaseField& sortField)
   throw (VException&) 
{
   /*
   DebugLog(<<  "buildSessionSortDBResultReq() token: " << sessionID
            << " sortType=" << int(sType)
            << " sortField=" << sortField.getName());
   */

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
   PSCommon::xmlAddTagValue( PSCommon::TOKEN_START, sessionID,
                             PSCommon::TOKEN_END, content );
   if (sType == PSCommon::SORT_ASCENDING)
   {
      PSCommon::xmlAddTagValue( PSCommon::SORTTYPE_START,
                                PSCommon::SORT_ASCEND_STR, PSCommon::SORTTYPE_END, content );
   }
   else if (sType == PSCommon::SORT_DESCENDING)
   {
      PSCommon::xmlAddTagValue( PSCommon::SORTTYPE_START,
                                PSCommon::SORT_DESCEND_STR, PSCommon::SORTTYPE_END, content );
   }
   if (sortField != BaseField::None)
   {
      PSCommon::xmlAddTagValue( PSCommon::SORTFIELD_START,
                                sortField.getName(), PSCommon::SORTFIELD_END, content );
   }
    
   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    
   //DebugLog(<<  "SessionSetDBResult xml body: " << content);

   return content;

}

///
string
PSReqXmlContentHandlers::buildSessionDeleteAllReq( const string& sessionID ) 
   throw (VException&) 
{
   //DebugLog(<<  "buildSessionDeleteAllReq() token: " << sessionID);

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
   PSCommon::xmlAddTagValue( PSCommon::TOKEN_START, sessionID,
                             PSCommon::TOKEN_END, content );
   PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );

   //DebugLog(<<  "SessionDeleteAll xml body: " << content);

   return content;
}

// *****************************************************************
// Registration Content Builder
// *****************************************************************
///
string
PSReqXmlContentHandlers::buildRegisterForUpdateReq (const string& hostport,
						    const Domain& domain,
						    const KeySet& keySet,
						    const string &uid)
   throw (VException&)
{
   //DebugLog (<<  "buildRegisterForUpdate() hostport = " << hostport << ", domain = " << domain.toString());

   /* build the XML content
    */

   string content;

   PSCommon::xmlAddTag (PSCommon::ARGUMENTS_START, content);
   PSCommon::xmlAddTagValue(PSCommon::REGISTER_HOSTPORT_START, hostport,
                            PSCommon::REGISTER_HOSTPORT_END, content);
   PSCommon::xmlAddTagValue(PSCommon::DOMAIN_START, domain.getName(),
                            PSCommon::DOMAIN_END, content);
   PSCommon::xmlAddTagValue(PSCommon::UPDATE_UID_START, uid,
                            PSCommon::UPDATE_UID_END, content);

   for (vector<string>::const_iterator keyIt = keySet._keys.begin();
        keyIt != keySet._keys.end(); keyIt++)
   {
      PSCommon::xmlAddTagValue(PSCommon::PRIMARYKEY_START, *keyIt,
                               PSCommon::PRIMARYKEY_END, content);
   }

   PSCommon::xmlAddTag (PSCommon::ARGUMENTS_END, content);

   //DebugLog (<<  "buildRegisterForUpdateReq xml body: " << content);

   return content;
}

///
string 
PSReqXmlContentHandlers::buildUpdateReq (const string& domain,
					 const FieldListtype& keySet,
                                         PSCommon::ChangeType changeType)
   throw (VException&)
{
   //DebugLog (<<  "buildUpdateReq, domain = " << domain);

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTag(PSCommon::RESPONSE_START, content);
   PSCommon::xmlAddTagValue(PSCommon::DOMAIN_START, domain,
                            PSCommon::DOMAIN_END, content);

   assert(changeType < PSCommon::MAX_CHANGE_TYPE);
   PSCommon::xmlAddTagValue(PSCommon::CHANGE_TYPE_START, PSCommon::ChangeTypes[changeType],
                            PSCommon::CHANGE_TYPE_END, content);

   PSCommon::xmlAddTag(PSCommon::MATCH_START, content);
    
   for (FieldListtype::const_iterator keyIt = keySet.begin();
        keyIt != keySet.end(); keyIt++)
   {
      PSCommon::xmlAddTagValue(PSCommon::PRIMARYKEY_START, *keyIt,
                               PSCommon::PRIMARYKEY_END, content);
   }
   PSCommon::xmlAddTag(PSCommon::MATCH_END, content);
   PSCommon::xmlAddTag(PSCommon::RESPONSE_END, content);

   //DebugLog (<<  "buildUpdateReq - xml body = " << content);

   return content;
}

///
string
PSReqXmlContentHandlers::buildUnRegisterForUpdateReq (const string &hostport,
                                                      const Domain &domain,
						      const string &uid)
   throw (VException&)
{
   //DebugLog (<<  "buildUnRegisterForUpdate() hostport = " << hostport << ", domain = " << domain.getName() << ", uid = " << uid);
   /* build the XML content
    */

   string content;

   PSCommon::xmlAddTag (PSCommon::ARGUMENTS_START, content);
   PSCommon::xmlAddTagValue(PSCommon::REGISTER_HOSTPORT_START, hostport,
                            PSCommon::REGISTER_HOSTPORT_END, content);
   PSCommon::xmlAddTagValue(PSCommon::DOMAIN_START, domain.getName(),
                            PSCommon::DOMAIN_END, content);
   PSCommon::xmlAddTagValue(PSCommon::UPDATE_UID_START, uid,
                            PSCommon::UPDATE_UID_END, content);
   PSCommon::xmlAddTag (PSCommon::ARGUMENTS_END, content);

   //DebugLog (<<  "buildUnRegisterForUpdateReq xml body: " << content);

   return content;
}

string
PSReqXmlContentHandlers::buildRegisterForForeignUpdateReq
(const string& hostport,
 const Domain& foreignDomain,
 const string& foreignKeyField,
 const Domain& primaryDomain,
 const string &uid) throw (VException&)
{
   /*
   DebugLog (<<  "buildRegisterForUpdate() hostport = " << hostport 
             << ", foreign domain = " << foreignDomain.getName() 
             << ", primary domain = " << primaryDomain.getName());
   */

   /* build the XML content
    */

   string content;

   PSCommon::xmlAddTag (PSCommon::ARGUMENTS_START, content);
   PSCommon::xmlAddTagValue(PSCommon::REGISTER_HOSTPORT_START, hostport,
                            PSCommon::REGISTER_HOSTPORT_END, content);
   PSCommon::xmlAddTagValue(PSCommon::FOREIGN_DOMAIN_START, foreignDomain.getName(),
                            PSCommon::FOREIGN_DOMAIN_END, content);
   PSCommon::xmlAddTagValue(PSCommon::FOREIGN_FIELD_START, foreignKeyField,
                            PSCommon::FOREIGN_FIELD_END, content);
   PSCommon::xmlAddTagValue(PSCommon::DOMAIN_START, primaryDomain.getName(),
                            PSCommon::DOMAIN_END, content);
   PSCommon::xmlAddTagValue(PSCommon::UPDATE_UID_START, uid,
                            PSCommon::UPDATE_UID_END, content);
   PSCommon::xmlAddTag (PSCommon::ARGUMENTS_END, content);

   //DebugLog (<<  "buildRegisterForUpdateReq xml body: "<< content);

   return content;
}

string
PSReqXmlContentHandlers::buildForeignUpdateReq
(const string& uid,
 const FieldListtype& keySet,
 PSCommon::ChangeType changeType) throw (VException&)
{
   //DebugLog (<<  "buildForeignUpdateReq");

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTag(PSCommon::RESPONSE_START, content);
   PSCommon::xmlAddTagValue(PSCommon::UPDATE_UID_START, uid,
                            PSCommon::UPDATE_UID_END, content);

   assert(changeType < PSCommon::MAX_CHANGE_TYPE);
   PSCommon::xmlAddTagValue(PSCommon::CHANGE_TYPE_START, PSCommon::ChangeTypes[changeType],
                            PSCommon::CHANGE_TYPE_END, content);

   PSCommon::xmlAddTag(PSCommon::MATCH_START, content);
    
   for (FieldListtype::const_iterator keyIt = keySet.begin();
        keyIt != keySet.end(); keyIt++)
   {
      PSCommon::xmlAddTagValue(PSCommon::PRIMARYKEY_START, *keyIt,
                               PSCommon::PRIMARYKEY_END, content);
   }
   PSCommon::xmlAddTag(PSCommon::MATCH_END, content);
   PSCommon::xmlAddTag(PSCommon::RESPONSE_END, content);

   //DebugLog (<<  "buildForeignUpdateReq - xml body = " << content);

   return content;
}

string 
PSReqXmlContentHandlers::buildUnRegisterForForeignUpdateReq
(const string& hostport,
 const Domain& foreignDomain,
 const string &uid) throw (VException&)
{
   //DebugLog (<<  "buildUnRegisterForForeignUpdateReq() hostport = " << hostport << " foreign domain = " << foreignDomain.getName() << ", uid = "<< uid);

   /* build the XML content
    */

   string content;

   PSCommon::xmlAddTag (PSCommon::ARGUMENTS_START, content);
   PSCommon::xmlAddTagValue(PSCommon::REGISTER_HOSTPORT_START, hostport,
                            PSCommon::REGISTER_HOSTPORT_END, content);
   PSCommon::xmlAddTagValue(PSCommon::FOREIGN_DOMAIN_START, foreignDomain.getName(),
                            PSCommon::FOREIGN_DOMAIN_END, content);
   PSCommon::xmlAddTagValue(PSCommon::UPDATE_UID_START, uid,
                            PSCommon::UPDATE_UID_END, content);
   PSCommon::xmlAddTag (PSCommon::ARGUMENTS_END, content);

   //DebugLog (<<  "buildUnRegisterForForeignUpdateReq xml body: " << content);

   return content;
}

