/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const PSRspXmlContentHandlersCxxVersion = "$Id: PSRspXmlContentHandlers.cxx,v 1.1 2002/09/30 04:55:12 jason Exp $";

// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <stdio.h>

#include <xercesc/util/XMLUniDefs.hpp>

#include "PSCommon.hxx"
#include "XmlContent.hxx"
#include "PSRspXmlContentHandlers.hxx"
#include "VNetworkException.hxx"
#include "Logger.hxx"

#define VOCAL_SUBSYSTEM Vocal::Subsystem::PROV

using namespace Epoisses;

// ---------------------------------------------------------------------------
//  PSRspXmlContentHandlers: Constructors
// ---------------------------------------------------------------------------
PSRspXmlContentHandlers::PSRspXmlContentHandlers(Sptr<PSDBResult> target)
   : myCurMatch(-1), myCharacters("")
{
   //DebugLog(<<  "PSRspXmlContentHandlers constructor invoked ");
   myParsedContent = target;

}

// ---------------------------------------------------------------------------
//  PSRspXmlContentHandlers: Destructor
// ---------------------------------------------------------------------------
PSRspXmlContentHandlers::~PSRspXmlContentHandlers()
{
   //DebugLog(<<  "PSRspXmlContentHandlers destructor invoked ");
}


// ---------------------------------------------------------------------------
//  PSRspXmlContentHandlers: Overrides of the SAX ErrorHandler interface
// ---------------------------------------------------------------------------
void PSRspXmlContentHandlers::error(const SAXParseException& e)
{
   DebugLog(<< "SAX Handler throws Error. " 
            << StrX(e.getSystemId()).localForm() << ":" << e.getLineNumber() << ":" << e.getColumnNumber()
            << " " << StrX(e.getMessage()).localForm());

   string err = "Catching XML Exception: ";
   err += StrX(e.getMessage()).localForm();

   throw VNetworkException(err, __FILE__, __LINE__);
}

void PSRspXmlContentHandlers::fatalError(const SAXParseException& e)
{
   DebugLog(<< "SAX Handler throws Fatal Error. " 
            << StrX(e.getSystemId()).localForm() << ":" << e.getLineNumber() << ":" << e.getColumnNumber()
            << " " << StrX(e.getMessage()).localForm());

   string err = "Catching XML Exception: ";
   err += StrX(e.getMessage()).localForm();

   throw VNetworkException(err, __FILE__, __LINE__);
}

void PSRspXmlContentHandlers::warning(const SAXParseException& e)
{
   DebugLog(<< "SAX Handler throws Warning. " 
            << StrX(e.getSystemId()).localForm() << ":" << e.getLineNumber() << ":" << e.getColumnNumber()
            << " " << StrX(e.getMessage()).localForm());

   string err = "Catching XML Exception: ";
   err += StrX(e.getMessage()).localForm();

   throw VNetworkException(err, __FILE__, __LINE__);
}


// ---------------------------------------------------------------------------
//  PSRspXmlContentHandlers: Overrides of the SAX DTDHandler interface
// ---------------------------------------------------------------------------
void PSRspXmlContentHandlers::unparsedEntityDecl(const     XMLCh* const name
                                                 , const   XMLCh* const publicId
                                                 , const   XMLCh* const systemId
                                                 , const   XMLCh* const notationName)
{
   // Not using DTD presently
}


void PSRspXmlContentHandlers::notationDecl(const   XMLCh* const name
                                           , const XMLCh* const publicId
                                           , const XMLCh* const systemId)
{
   // Not using DTD presently
}


// ---------------------------------------------------------------------------
//  PSRspXmlContentHandlers: Overrides of the SAX DocumentHandler interface
// ---------------------------------------------------------------------------
void PSRspXmlContentHandlers::characters(const     XMLCh* const    chars
                                         , const   unsigned int    length)
{
   string& fn = myFields.top();   // get field name
   string  val= StrX( chars ).localForm();
   myCharacters = val;

   //DebugLog(<<  "  ****PSRspXmlContentHandlers::characters = " << val << " len = " << length);

   if (myParsedContent->blobFlag == true)
   {
      //DebugLog (<<  "Add the characters into xmlBlob");
      myParsedContent->setXmlBlob(val);
      //DebugLog (<<  "Xml Blob = " << myParsedContent->getXmlBlob());
      return;
   }

   if ((fn == PSCommon::RESPONSE_STR) ||
       (fn == PSCommon::MATCH_STR))
   {
      ; // do nothing
   }
   else if ((fn == PSCommon::FIELDLIST_STR) ||
            (fn == PSCommon::FIELDVALUE_STR))
   {
      myParsedContent->tagLevel = 1;        
   }
   else if (fn == PSCommon::DOMAIN_STR)
   {
      myParsedContent->setDomain( val );
   }
   else if (fn == PSCommon::RESULTCOUNT_STR)
   {
      int num = atoi(val.c_str()); 
      myParsedContent->setNumMatches( num );
      myParsedContent->reserveNumTables( num );
      //DebugLog( <<  "number of matches found " <<  num );
   }
   else if (fn == PSCommon::ERROR_STR)
   {
      //DebugLog( <<  "error: " <<  val);
      myParsedContent->setError( val );
   }
   else if (fn == PSCommon::TOKEN_STR)
   {
      myParsedContent->setSessionID(val);
      //DebugLog( <<  "sessionID = " <<  val);
   }				      
   else if (fn == PSCommon::LISTITEM_STR)
   {
      myParsedContent->addFieldList(val);
      //DebugLog( <<  "field = " <<  val);
   }
   else if (fn == PSCommon::XMLCATEGORY_STR)
   {
      myParsedContent->setXmlCategory(val);
      //DebugLog( <<  "xmlcategory = " <<  val);
   }
   else if (fn == PSCommon::XMLDESC_STR)
   {
      myParsedContent->setXmlDescription(val);
      //DebugLog( <<  "xmldescription = " <<  val);
   }
   else if (fn == PSCommon::PRIMARYKEY_STR)
   {
      myParsedContent->addFieldList(val);
      //DebugLog( <<  "primary key = " <<  val);
   }
   else // anything else, stuff it into myParsedContent's table
   {
      // rely on myCurMatch as the index
      myParsedContent->setFieldValue(myCurMatch, fn, val);
      //DebugLog( <<  "Table: " << myCurMatch << ", add " << fn << "/" << val);
   }
   //DebugLog(<<  "**** end characters ");
}


void PSRspXmlContentHandlers::endDocument()
{
   //DebugLog( <<  "endDocument() reached" );
   // used to say, myFields.empty() should not be false
}


void PSRspXmlContentHandlers::endElement(const XMLCh* const uri, 
                                         const XMLCh* const localname, 
                                         const XMLCh* const qname)
{
   string fn = StrX( qname ).localForm();
   // do no use name spaces, ignore localname
   //	fFormatter << qname << chCloseAngle;
   //DebugLog( <<  "***** endElement qname = " <<  fn );
   myFields.pop();
   //DebugLog( <<  " end endElement ***** ");

   myParsedContent->tagLevel--;

   if (myParsedContent->blobFlag == true) 
   {
      if (myParsedContent->tagLevel == 1)
      {
         myParsedContent->setFieldValue(myCurMatch, fn, myParsedContent->getXmlBlob());
         //DebugLog(<<  "Final add fieldname [" << fn << "] with blob [" << myParsedContent->getXmlBlob() << "] into table");

         // clear out the xml blob
         myParsedContent->clearXmlBlob();
         //myParsedContent->blobFlag = false;
         //myParsedContent->tagLevel = 0;
         //myParsedContent->xmlBlob = "";
         return;
      }

      //DebugLog (<<  "Add end of element into xml blob: " <<  fn);
      string temp = PSCommon::END_ELEMENT_OPEN_BRACKET + fn + PSCommon::CLOSE_BRACKET;
      myParsedContent->setXmlBlob(temp);
      //DebugLog (<<  "Xml Blobg = " <<  myParsedContent->getXmlBlob());
   }
   else if (myCharacters.empty())
   {
      //DebugLog(<<  "added field [" << fn << "] (with empty value) to table");
      myParsedContent->setFieldValue(myCurMatch, fn, "");
   }

}


void PSRspXmlContentHandlers::ignorableWhitespace( const   XMLCh* const chars
                                                   ,const  unsigned int length)
{
   cout << "*********  ignorableWhitespace() called " << endl;
   // fFormatter.formatBuf(chars, length, XMLFormatter::NoEscapes);
}



void PSRspXmlContentHandlers::startDocument()
{
   //DebugLog( <<   "startDocument() ************ ");

}


void PSRspXmlContentHandlers::startElement(const   XMLCh* const    uri,
					   const   XMLCh* const    localname,
					   const   XMLCh* const    qname,
					   const   Attributes& attributes)
{
   string fn = StrX(qname).localForm();  // field name
   //DebugLog( <<  " *** startElement qname : " <<  fn );

   if (fn == PSCommon::MATCH_STR.c_str())
   {
      myCurMatch++;
      //DebugLog( <<  "matching record, idx = " <<  myCurMatch );
   }
   myFields.push( fn );
   myCharacters = "";

   if (myParsedContent->tagLevel)
   {
      myParsedContent->tagLevel++;
      //DebugLog (<<  "BLOB Counter = " <<  myParsedContent->tagLevel);
      if (myParsedContent->tagLevel > 2)
      {
         myParsedContent->blobFlag = true;
      }
   }

   if (myParsedContent->blobFlag == true)
   {
      //DebugLog (<<  "Add start of element into blobString: " <<  fn);
      string temp = PSCommon::OPEN_BRACKET + fn + PSCommon::CLOSE_BRACKET;
      myParsedContent->setXmlBlob(temp);
      //DebugLog (<<  "Xml Blob = " <<  myParsedContent->getXmlBlob());
   }

}



// *****************************************************************
// Single Transaction Response XML Content Builder
// *****************************************************************

string PSRspXmlContentHandlers::buildGetXmlDescriptionRsp(
   const string& payload) 
   throw ( VException& ) 
{
   //DebugLog(<<  "buildGetXmlDescription() payload " <<   payload);

   /* build the XML content
    */ 

   string content;

   PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
   content += payload;
   PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );

   //DebugLog(<<  "GetXmlDescription response xml body: " <<  content);
   return content;

}

string PSRspXmlContentHandlers::buildListXmlCategoriesRsp(
   const StringList& resList) 
   throw ( VException& ) 
{
   //DebugLog(<<  "buildListXmlCategories()");

   /* build the XML content
    */ 

   string content;
   char buf[256];
    
   PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
   PSCommon::xmlAddTag( PSCommon::RESULTCOUNT_START, content );
   sprintf(buf, "%d", resList.size());
   content += string(buf);
   PSCommon::xmlAddTag( PSCommon::RESULTCOUNT_END, content );
    
   StringList::const_iterator i = resList.begin();
   while ( i != resList.end() )
   {
      string s = *i;
      i++;

      PSCommon::xmlAddTag( PSCommon::XMLCATEGORY_START, content );
      content += s;
      PSCommon::xmlAddTag( PSCommon::XMLCATEGORY_END, content );
   }

   PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );

   //DebugLog(<<  "ListXmlCategories response xml body: " <<  content);
   return content;

}


string PSRspXmlContentHandlers::buildListXmlDescriptionsRsp(
   const StringList& resList) 
   throw ( VException& ) 
{
   //DebugLog(<<  "buildListXmlDescriptions()");

   /* build the XML content
    */ 

   string content;
   char buf[256];
    
   PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
   PSCommon::xmlAddTag( PSCommon::RESULTCOUNT_START, content );
   sprintf(buf, "%d", resList.size());
   content += string(buf);
   PSCommon::xmlAddTag( PSCommon::RESULTCOUNT_END, content );
    
   StringList::const_iterator i = resList.begin();
   while ( i != resList.end() )
   {
      string s = *i;
      i++;

      PSCommon::xmlAddTag( PSCommon::XMLDESC_START, content );
      content += s;
      PSCommon::xmlAddTag( PSCommon::XMLDESC_END, content );
   }

   PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );

   //DebugLog(<<  "ListXmlDescriptions response xml body: " <<  content);
   return content;

}

///
string PSRspXmlContentHandlers::buildResultSetRsp(
   const string& domain, 
   const ResultTabletype & resultTable,
   const FieldListtype & fieldList )
   throw ( VException& ) 
{
   //DebugLog(<<  "buildResultSetRsp() domain " <<  domain);

   /* build the XML content
    */ 

   string content;

   PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
   PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain, 
                             PSCommon::DOMAIN_END, content );
   int numRecords = resultTable.size();
   string  startTag, endTag;
   char buf[10];
   sprintf( buf, "%d", numRecords );
   PSCommon::xmlAddTagValue( PSCommon::RESULTCOUNT_START,
                             buf, PSCommon::RESULTCOUNT_END, content );
   vector<string> record;

   for (int i = 0; i < numRecords; i++)
   {
      record = resultTable[i];
      PSCommon::xmlAddTag( PSCommon::MATCH_START, content );
      PSCommon::xmlAddTag( PSCommon::FIELDVALUE_START, content );
	
      for (unsigned short j = 0; j < fieldList.size(); j++)
      {
         startTag = PSCommon::OPEN_BRACKET + fieldList[j] +
            PSCommon::CLOSE_BRACKET;
         endTag = PSCommon::END_ELEMENT_OPEN_BRACKET + fieldList[j] +
            PSCommon::CLOSE_BRACKET;
         PSCommon::xmlAddTagValue( startTag, record[j], endTag, content);
      }

      PSCommon::xmlAddTag( PSCommon::FIELDVALUE_END, content );
      PSCommon::xmlAddTag( PSCommon::MATCH_END, content );
   } 
   PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );

   //DebugLog(<<  "BuildResultSet response xml body: " <<  content);
   return content;

}


string PSRspXmlContentHandlers::buildResultCountRsp(
   const string& domain,
   const string& resultCount )
   throw ( VException& )
{
   //DebugLog(<<  "buildResultCountRsp() domain " <<  domain);

   /* build the XML content
    */

   string content;

   PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
   PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain,
                             PSCommon::DOMAIN_END, content );

   PSCommon::xmlAddTagValue( PSCommon::RESULTCOUNT_START,
                             resultCount, 
                             PSCommon::RESULTCOUNT_END, 
                             content );


   PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );

   //DebugLog(<<  "BuildResultCount response xml body: " <<  content);
   return content;

}



///
string PSRspXmlContentHandlers::buildGetFieldsRsp( 
   const string& domain,
   const ResultTabletype & resultTable )
   throw( VException& ) 
{
   //DebugLog(<<  "buildGetFieldsRsp() domain " << domain);

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
   PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain, 
                             PSCommon::DOMAIN_END, content );
    
   int num = resultTable.size();
   char buf[10];
   sprintf( buf, "%d", num );
    
   PSCommon::xmlAddTagValue( PSCommon::RESULTCOUNT_START,
                             buf, PSCommon::RESULTCOUNT_END, content );

   PSCommon::xmlAddTag( PSCommon::MATCH_START, content );
   // build list of field names:
   PSCommon::xmlAddTag( PSCommon::FIELDLIST_START, content );
   for (unsigned short i = 0; i < num; i++)
   {
      PSCommon::xmlAddTagValue( PSCommon::LISTITEM_START,
                                resultTable[i][0],
                                PSCommon::LISTITEM_END, 
                                content );
   }
   PSCommon::xmlAddTag( PSCommon::FIELDLIST_END, content );
   PSCommon::xmlAddTag( PSCommon::MATCH_END, content );

   PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );

   //DebugLog(<<  "GetFields response xml body: " <<  content);
   return content;
}


/// update existing records with keyword/value pairs provided
string 
PSRspXmlContentHandlers::buildSetRecordsRsp(
   const string& domain, 
   const string& resultCount )
   throw (VException& )
{
   //DebugLog(<<  "buildSetRecordsRsp() domain " << domain << ", count = " << resultCount);

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
   PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain,
                             PSCommon::DOMAIN_END, content );
    
   PSCommon::xmlAddTagValue( PSCommon::RESULTCOUNT_START,
                             resultCount, PSCommon::RESULTCOUNT_END, content );
    
   PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );

   //DebugLog(<<  "SetRecords response xml body: " <<  content);
   return content;

}

// *****************************************************************
// Session related XML Content Builder
// *****************************************************************

string 
PSRspXmlContentHandlers::buildSessionOpenRsp( const string& token ) 
   throw(VException&)
{ 
   //DebugLog(<<  "buildSessionOpenRsp() token:  " <<  token );

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
   PSCommon::xmlAddTagValue( PSCommon::TOKEN_START, token,
                             PSCommon::TOKEN_END, content );
   PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );

   //DebugLog(<<  "SessionOpen xml body: " <<  content);
   return content;
}
         

///
string
PSRspXmlContentHandlers::buildErrorRsp( const string& errorText )
   throw (VException&) 
{
   //DebugLog(<<  "buildErrorRsp() reason:  "<< errorText);

   /* build the XML content
    */
   string content;

   PSCommon::xmlAddTagValue( PSCommon::ERROR_START, errorText,
                             PSCommon::ERROR_END, content );

   //DebugLog(<<  "ErrorRsp xml body: " <<  content);

   return content;
}

