// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
#ifndef CacheManager_hxx
#define CacheManager_hxx


#include <list>

#include "RWMutex.hxx"
#include "SingletonHolder.hxx"

class ProvisioningLossTest;

namespace Epoisses
{

class CacheBase;

/// Communicates the pserver status to all caches, 
/// keeps status of pserver status
class CacheManager
{
   public:
      CacheManager();

      /// any client may call this method
      void serviceUnavailable();

      // don't think this needs a lock
      bool isServiceAvailable() const;

      /// shutdown all caches
      void shutdown();

   private:
      friend class ReconnectProvisioning;
      friend class ::ProvisioningLossTest;
      void serviceAvailable();

      friend class CacheBase;
      void addCache(CacheBase* cacheBase);
      void removeCache(CacheBase* cacheBase);

      volatile bool _isServiceAvailable;
      Vocal::Threads::RWMutex _mutex;
      list<CacheBase*> _caches;
};

/// Provides a pointer type to store caches and interface for dealing with
/// change in pserver status.
class CacheBase
{
      friend class CacheManager;
   protected:
      CacheBase()
      {
         SingletonHolder<CacheManager>::Instance().addCache(this);
      }

      virtual ~CacheBase()
      {
         SingletonHolder<CacheManager>::Instance().removeCache(this);
      }
      
   protected:
      /// called when the pserver comes back after disappearing
      virtual void serviceAvailable() = 0;
      virtual void shutdown() = 0;
      
};

}

#endif // CacheManager_hxx
