// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
#ifndef ForeignUpdateFunctor_hxx
#define ForeignUpdateFunctor_hxx

#include "UpdateFunctorBase.hxx"

namespace Epoisses {

/**
   This updator maps updates from one domain (foreignDomain) to another
   domain. The foreign domain must have an identified foreign key to the primary
   domain.

   For example, consider the feature instances that belong a user. The feature
   domain will have a foreign key to the user domain. This style of updator
   allows the application to respond to a change in the feature set as a change
   to the user.

   Insertion/deletions to foreign domain cause the registered updator to fire.

   Subtype to customize; override operator(). 

   See ProvisioningInterface::registerForForiegnUpdates. */
class ForeignUpdateFunctor : public UpdateFunctorBase
{
   public:
      ForeignUpdateFunctor();
      
      /// Override to respond to updates on foreign domain that impact domain.
      virtual void operator()(const KeySet& keys) = 0;
      
   private:
      ForeignUpdateFunctor(const ForeignUpdateFunctor& rhs);
      ForeignUpdateFunctor& operator=(const ForeignUpdateFunctor&);
};

} // namespace Epoisses

#endif // ForeignUpdateFunctor_hxx
