/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const PSIFCreator_cxx_Version = "$Id: PSIFCreator.cxx,v 1.1 2002/09/30 04:55:14 jason Exp $";

#include "VNetworkException.hxx"
#include "VBadDataException.hxx"
#include "PSIFCreator.hxx"
#include "Logger.hxx"
#include "PSHttpXMLInterface.hxx"

#define VOCAL_SUBSYSTEM Vocal::Subsystem::PROVC

using namespace Epoisses;

/** PSIFCreator is a concrete class which provides a default
 *  implementation for the factory mehtod.
 *  Usage:
 *
 *  try {
 *      ProvisioningInterface& pif = PSIFCreator::getPSIFInstance(
 *           "host1",6005,"host2",6005,PSIFTYPE::HTTPXML);
 *      long sid = pif.sessionOpen( 3600 );
 *      .....
 *  }
 *  catch (VException& e)
 *  ....
 */
ProvisioningInterface& PSIFCreator::getPSIFInstance( const char* primHost,
                                                     const int primPort,
                                                     const char* altHost,
                                                     const int altPort,  
                                                     PSCommon::PSIFType type)
   throw (VException&)
{
   if (ProvisioningInterface::myInstance != 0)
   {
      return (*ProvisioningInterface::myInstance);
   }

   if (primHost == 0 || primPort == 0 || altHost == 0 || altPort == 0)
   {
      InfoLog (<< "PServer hostname and port not provided");
      VNetworkException exception("pserver hostname and port not provided", __FILE__, __LINE__);
      assert(false);
      throw exception;
   }

   switch (type)
   {
      case PSCommon::HTTPXML:
      {
         return  PSHttpXMLInterface::getInstance(primHost, primPort);
      }
      break;

      default:
      {
         InfoLog (<< "Unsupported PSIFType");
         throw VBadDataException("Unsupported PSIFType", __FILE__, __LINE__, type);
      }
      break;
   }
}


