// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
#ifndef TimeGuard_hxx
#define TimeGuard_hxx

#include "TimeFunc.hxx"
#include "Logger.hxx"
#include <sstream>

// !dlb! get rid of TimeFunc/FunctionTimer

namespace Epoisses {

#undef VOCAL_SUBSYSTEM
#define VOCAL_SUBSYSTEM Vocal::Subsystem::TEST

// default output policy
class StandardErrorOutput
{
   public:
      void output(const string& header,
                  const string& message,
                  const FunctionTimer& timer)
      {
         cerr << header << message << " took " << const_cast<FunctionTimer&>(timer) << " usecs" << endl;
      }
};

// may switch to debug
class CpLogDebugOutput
{
   public:
      void output(const string& header,
                  const string& message,
                  const FunctionTimer& timer)
      {
         stringstream s;
         s << const_cast<FunctionTimer&>(timer);
         DebugLog (<< header << " " << message << " took " << s.str() << " usecs");
      }
};

// Times a block.
template <class OP = StandardErrorOutput>
class TimeGuard
{
   public:
      TimeGuard(const string& message)
         : _message(message)
      {
         timer.start();
      }

      ~TimeGuard()
      {
         timer.end();
         OP op;
         op.output("TIME: ", _message, timer);
      }

   private:
      FunctionTimer timer;
      string _message;
};
#undef VOCAL_SUBSYSTEM

} // namespace Epoisses

#endif // TimeGuard_hxx
