// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#ifndef LockedStreamMap_hxx
#define LockedStreamMap_hxx

#include <ext/hash_map>
#include <ace/SOCK_Stream.h>
#include <ace/SOCK_Connector.h>

#include "StringHash.hxx"
#include "RWMutex.hxx"
#include "Mutex.hxx"

namespace Epoisses
{

class LockedStreamMap
{
   public:
      LockedStreamMap();

      ~LockedStreamMap();

   private:
      class LockedStream
      {
            friend class LockedStreamMap;
         public:
            LockedStream(const string& hostport);

         private:
            ~LockedStream();

         public:
            void write(const string& data);
            
         private:
            Vocal::Threads::Mutex _mutex;
            ACE_SOCK_Stream _server;
            ACE_SOCK_Connector _connector;
            
            bool _disconnected;
      };

   public:
      void add(const string& hostPort);
      void remove(const string& hostPort);
      void write(const string& hostPort, const string& message);
      
   private:
      hash_map<string, LockedStream*> _map;
      Vocal::Threads::RWMutex _mutex;
};
 
}

#endif
