// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#ifndef ProvisionClientHandler_hxx
#define ProvisionClientHandler_hxx

#include <string>

#include "ClientHandler.hxx"
#include "HttpMessage.hxx"
#include "PSProtocolService.hxx"

namespace Epoisses {

using namespace Vocal;

class ProvisionServer;
class PSDBRequest;

class ProvisionClientHandler : public Epoisses::ClientHandler<ProvisionClientHandler>
{
   public:

      ProvisionClientHandler();

      string className() { return "ProvisionClientHandler"; }

   protected:
      ~ProvisionClientHandler();

   private:
       ///  Basically a big set of if's to decide how to handle an incoming
      int process();

      // Message handling
      void handleGetRequest( const Http::Message &httpmsg ); 
      void handlePostRequest( const Http::Message &httpmsg );
      void handlePutRequest( const Http::Message &httpmsg); 
      void handleDeleteRequest( const Http::Message &httpmsg ); 

      // Get Requests 
      void handleGetRecordsInDomain( const string& httpBody);
      void handleGetFieldsInDomain( const string& httpBody);
      void handleSessionOpen( const string& httpBody);
      void handleSessionGetDBResult( const string& httpBody);
    
      // Post Requests 
      void handleSessionClose( const string& httpBody);
      void handleSessionSearchDB( const string& httpBody);
      void handleSessionSortDBResult( const string& httpBody);
      void handleRegisterDomainForUpdate( const string& httpBody);
      void handleUnRegisterDomainForUpdate( const string& httpBody);
      void handleRegisterDomainForForeignUpdate( const string& httpBody);
      void handleUnRegisterDomainForForeignUpdate( const string& httpBody);

      // Put Requests 
      void handleSetRecordsInDomain( const string& httpBody);
      void handleAddRecordInDomain( const string& httpBody);
      void handleSessionSetDBResult( const string& httpBody);

      // Delete Requests 
      void handleDeleteRecordsInDomain( const string& httpBody);
      void handleSessionDeleteAll( const string& httpBody);

      /// Check the register list for updates.
      void updateCheck(DBResultset* resultset,
                       PSCommon::ChangeType changeType);

      class FieldResult
      {
         public:
            FieldResult(const string& fieldName,
                        auto_ptr<DBResultset> result)
               : _fieldName(fieldName),
                 _result(result)
               {}
            
            string _fieldName;
            mutable auto_ptr<DBResultset> _result;

            // should not be necessary to declare?
            FieldResult(const FieldResult& rhs)
               : _fieldName(rhs._fieldName),
                 _result(rhs._result)
            {}
            FieldResult&
            operator=(const FieldResult& rhs)
            {
               _fieldName = rhs._fieldName;
               _result = rhs._result;
               return *this;
            }
      };

      /// Check the register list for foreign updates.
      void foreignUpdateCheck(const list<FieldResult>& resultset,
                              PSCommon::ChangeType changeType);

      /// Make a 400 and send on the provided connection
      void sendHttpFail(const string& retData);

      /// Make a 200 and send on the provided connection
      void sendHttpOk(const string& retData);

      /// Determine the id and leaf-domain of affected rows (called before updates)
      DBResultset* affectedRows(const string& domain,const string& filter);

      /// Determine the id and leaf-domain of affected foreign rows
      list<FieldResult> affectedForeignRows(const string& domain, const string& filter);
};

} // namespace Epoisses


#endif // ProvisionClientHandler_hxx
