static const char* const ProvisionServer_cxx_Version = "$Id: ProvisionServer.cxx,v 1.1 2002/09/30 04:55:16 jason Exp $";

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



#include "Logger.hxx"
#include "VIoException.hxx"
#include "ProvisionServer.hxx"
#include "ProvisionClientHandler.hxx"
#include "PSConfiguration.hxx"
#include "ClientAcceptor.hxx"

#include <ace/TP_Reactor.h>

//User defined
#define VOCAL_SUBSYSTEM Vocal::Subsystem::PROVS

using namespace Epoisses;

int
ProvisionServer::Port = 6005;

ProvisionServerImpl::ProvisionServerImpl() 
   : _pServerPort(ProvisionServer::Port)
{
   mySessionManager.run();
}

#if 1
void
ProvisionServerImpl::run()
{
   DebugLog (<< "Listening on port " << _pServerPort);
   ACE_Reactor reactor;
   Epoisses::ClientAcceptor<ProvisionClientHandler> acceptor;
   
  if (acceptor.open (ACE_INET_Addr (_pServerPort), &reactor) == -1)
  {
     InfoLog (<< "Failed to listen on port " << _pServerPort);
     assert(0);
  }

  //Loop through for Incoming messages
  InfoLog(<< "Pserver successfully started -- Running");
  while (true)
  {
     reactor.handle_events();
  }
}
#else
void
ProvisionServerImpl::run()
{
   DebugLog (<< "Listening on port " << _pServerPort);

   auto_ptr<ACE_TP_Reactor> holder(new ACE_TP_Reactor);
   auto_ptr<ACE_Reactor> reactor (new ACE_Reactor(holder.get()));
   ACE_Reactor::instance(reactor.get());
   ClientAcceptor <ProvisionClientHandler> acceptor;
   
   if (acceptor.open (ACE_INET_Addr (_pServerPort), reactor.get()) == -1)
   {
      InfoLog (<< "Failed to listen on port " << _pServerPort);
      assert(0);
   }
   
   ACE_Thread_Manager::instance()->spawn_n(20,
                                           ProvisionServerImpl::eventLoop,
                                           ACE_static_cast(void*,reactor.get()));
   //ACE_Thread_Manager::instance()->spawn(ProvisionServerImpl::controller,
   //ACE_static_cast(void*, reactor.get()));
   
   InfoLog(<< "Pserver successfully started -- Running");
   ACE_Thread_Manager::instance()->wait();
}

void*
ProvisionServerImpl::eventLoop(void* arg)
{
   ACE_Reactor* reactor = ACE_static_cast(ACE_Reactor*, arg);
   reactor->owner(ACE_OS::thr_self());
   return (void*)(reactor->run_reactor_event_loop());
}



#endif
