/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "VDbException.hxx"
#include "PGSQLConnection.hxx"
#include "ConnectionManager.hxx"
#include "PSConfiguration.hxx"
#include "DBInfo.h"
#include "Logger.hxx"
#include "StringUtil.hxx"
//#include "FastDBConnection.hxx"

#define VOCAL_SUBSYSTEM Vocal::Subsystem::PROVS

using namespace Epoisses;

list<DBConnection*> ConnectionManagerImpl::ConnectionCache;
Vocal::Threads::Mutex ConnectionManagerImpl::CacheMutex;
VSemaphore ConnectionManagerImpl::CacheSemaphore(ConnectionCount);

ConnectionManager::Connection::Connection(const string& domain)   
{
   
   if (StringUtil::toLower(domain) == "##registrationbindings")
   {
//      dbconn = FastDBConnection::instance();
   }
   else
   {
      dbconn = ConnectionManager::Instance().checkoutConnection();
   }
}


ConnectionManager::Connection::~Connection()
{
   // if (dbconn != FastDBConnection::instance())
   {
      ConnectionManager::Instance().checkInConnection(dbconn);
   }
}

ConnectionManagerImpl::ConnectionManagerImpl()
{
   // create a static connection pool 
   if (ConnectionCache.empty())
   {
      DBInfo dbInfo = PSConfiguration::instance()->getDBInfo();
      // construct a pool of pgsql connections
      for (unsigned int i = 0; i < ConnectionCount; i++)
      {
         ConnectionCache.push_back(new PGSQLConnection(dbInfo.database,
                                                       dbInfo.host,
                                                       dbInfo.user, 
                                                       dbInfo.password));
      }
   }
}

ConnectionManagerImpl::~ConnectionManagerImpl()
{ 
   Vocal::Threads::Lock lock(CacheMutex);
   for (list<DBConnection*>::const_iterator i = ConnectionCache.begin();
        i != ConnectionCache.end(); i++)
   {
      delete *i;
   }
}

DBConnection* 
ConnectionManagerImpl::checkoutConnection()
{
   CacheSemaphore.Wait();
   Vocal::Threads::Lock lock(CacheMutex);
   DBConnection *connection = ConnectionCache.back();
   ConnectionCache.pop_back();
   return connection;
}

void
ConnectionManagerImpl::checkInConnection(DBConnection* conn)
{
   Vocal::Threads::Lock lock(CacheMutex);
   assert(dynamic_cast<PGSQLConnection*>(conn));
   ConnectionCache.push_back(conn);
   assert(ConnectionCache.size() <= ConnectionCount);
   CacheSemaphore.Post();
}

