#ifndef FASTDBRESULTSET_HXX
#define FASTDBRESULTSET_HXX

// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//


#include "DBResultset.hxx"

namespace Epoisses {
   
/// FastDBResultset class- encapsulate database query results
class FastDBResultset : public DBResultset
{
   public:
      FastDBResultset(const string& searchfilter, const FieldListtype& fieldlist, ResultTabletype* values);
      
      /// Destructor
      virtual ~FastDBResultset();

      /// Return all records from a query
      virtual ResultTabletype GetRecords(void) const;

      /// Return records between offset1 and offset2
      virtual ResultTabletype GetRecords(unsigned short offset1, 
                                         unsigned short offset2) const;

      /// Return the record at [offset]
      virtual ResultRecordtype operator[](unsigned short offset)
         const;

      /// Return the record at offset
      virtual ResultRecordtype GetRecord(unsigned short offset)
         const;

      /// Return field list from query
      virtual FieldListtype GetFields(void) const;

      /// Return number of records in query
      virtual unsigned int GetRecordCount(void) const;

      
      /// Return search filter from query
      virtual const string& GetSearchFilter(void) const;
      
   private:
      const string _searchFilter;

      /// STL vector of strings representing fieldlist
      FieldListtype _fieldList;
      ResultTabletype* _values;
      /// Number of records in resultset

};

} // namespace Epoisses

#endif
