/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



#include "ParsedDef.hxx"
#include <algo.h>
#include <stdio.h>
#include "Inserter.hxx"

using namespace Epoisses;

const DifferenceSet ParsedDef::compareOld( const ParsedDef &oldDef ) {

    DifferenceSet returnVal;
    FieldListtype::const_iterator result;


    // If the primary keys don't agree, don't even bother checking anything
    // else, since we have to wipe the old version of the database
    if ((myParent == "") && (myPrimaryKey != oldDef.getPrimaryKey())) 
    {
        returnVal.setChangedPrimary();
        return returnVal;
    }
        
    cerr << "OldDef: " << Inserter(oldDef.myFieldList) << endl;
    cerr << "NewDef: " << Inserter(myFieldList) << endl;

    
    // Walk the new list and see if the entries are in the old list. If they
    // are not, then these are new entries -- add them to the newList
    // or if the types have changed on existing entries add them to the list
    for (FieldListtype::iterator walker = myFieldList.begin(); walker != myFieldList.end();
         walker++) {

        result = find((oldDef.myFieldList).begin(),
                      (oldDef.myFieldList).end(),
                      *walker);

        if (result == oldDef.myFieldList.end())
        {           
           returnVal.addNewField(*walker);
        }
        else
        {
           FieldListDatatype::const_iterator myType = myFieldTypes.begin();
           FieldListDatatype::const_iterator oldType = oldDef.myFieldTypes.begin();

           advance(myType, distance(myFieldList.begin(), walker));
           advance(oldType, distance(oldDef.myFieldList.begin(), result));

           if (*oldType != *myType)
           {
              if (*result == myPrimaryKey)
              {
                 returnVal.setChangedPrimary();
                 return returnVal;
              }
              returnVal.addNewField(*walker);
              returnVal.addDelField(*walker);
           }
        }
    }

    // Next walk the old one and look for any that don't seem to be in the
    // new list. We will add these to the delete list.

    for (FieldListtype::const_iterator walker = oldDef.myFieldList.begin();
         walker != oldDef.myFieldList.end();
         walker++) {
        
        result = find(myFieldList.begin(), myFieldList.end(),
                      *walker);
        
        if (result == myFieldList.end())
        {           
           returnVal.addDelField(*walker);
        }
    }

    return returnVal;

}



void ParsedDef::addField(string newField )
    throw (VBadDataException &) {

    transform(newField.begin(), newField.end(), newField.begin(), ::tolower);
   
    FieldListtype::iterator result;
   
    result = find(myFieldList.begin(), myFieldList.end(), newField);
    
    if (result != myFieldList.end()) {
        char buf[256];
        sprintf (buf, "Duplicate field name %s encountered!",
                 newField.c_str());
        throw VBadDataException( buf, __FILE__, __LINE__, 0 );
    }

    myFieldList.push_back(newField); 
}

void ParsedDef::addFieldType(const Datatype newType) 
    throw (VBadDataException &)
{
  if (myFieldList.size() != (myFieldTypes.size()+1))
    {
      throw VBadDataException("Number of fields does not \
          match number of types", __FILE__, __LINE__, 0);
    }

  myFieldTypes.push_back(newType);
}
