// ********************************************************************
//  Database creation tool
// ********************************************************************

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include <xercesc/sax2/XMLReaderFactory.hpp>
#include <xercesc/sax2/Attributes.hpp>
#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/DefaultHandler.hpp>
#include <xercesc/util/PlatformUtils.hpp>


#include "DBConnection.hxx"
#include "DBResultset.hxx"
#include "ConnectionManager.hxx"
#include "VDbException.hxx"
#include "PSConfiguration.hxx"
#include "PSCommandLine.hxx"
#include "ParsedDef.hxx"
#include "DBInfo.h"
#include "Logger.hxx"
#include "Log.hxx"


#define VOCAL_SUBSYSTEM Vocal::Subsystem::PROVS

using namespace std;
using namespace Epoisses;
using namespace Vocal;

/// Handler class for the XML definition
class MySAX2Handler : public DefaultHandler {
   private:
      bool foundTable;


   public:
    
      MySAX2Handler() {}

      void startElement(
         const   XMLCh* const    uri,
         const   XMLCh* const    localname,
         const   XMLCh* const    qname,
         const   Attributes&     attrs
         );

      void endElement (const XMLCh *const uri,
                       const XMLCh *const localname,
                       const XMLCh *const qname);

      void fatalError(const SAXParseException&);

      ParsedDef myDef;
      ParsedDef getMyDef() { return myDef; }
    
};



void MySAX2Handler::endElement (const XMLCh *const uri,
                                const XMLCh *const localname,
                                const XMLCh *const qname)
{
   // .dlb. type set in startElement
   return;
   
   string element = XMLString::transcode(qname);
 
   if (strcmp(element.c_str(), "DBEntry") == 0)
   {
      if (!foundTable)
      {
         //if the table type was not found within this data definition
         //make this column to be of the default type
         myDef.addFieldType(STRINGTYPE);
      }
      else
      {
         //if the tabletype was found within this data definition, make 
         //make this column to be of the text blob type
         myDef.addFieldType(TEXTBLOBTYPE);
      }
      foundTable = false; //reset
   }    
}


/** Called when a new element is seen. We only care if it is DataDef (begining
    of a new table) or DBEntry (a column in that table) */
void MySAX2Handler::startElement(const   XMLCh* const    uri,
                                 const   XMLCh* const    localname,
                                 const   XMLCh* const    qname,
                                 const   Attributes&     attrs)
{

   string element = XMLString::transcode(qname);
   const XMLCh *key;
   const XMLCh *value;
    
   if (element == "DataDefn") 
   {
      key = XMLString::transcode("name");
      value = attrs.getValue(key);
      if (value != NULL)
      {
         myDef.setDomain(XMLString::transcode(value));
      } 
      else
      {
         cerr << "Attribute \"name\" not found for element DataDefn" << endl;
         assert(false);
      }

      key = XMLString::transcode("key");
      value = attrs.getValue(key);
      if (value != NULL)
      {
         myDef.setPrimaryKey(XMLString::transcode(value));

         // optional parent attribute not allowed
         key = XMLString::transcode("parent");
         value = attrs.getValue(key);
         if (value != NULL)
         {
            cerr << "May not have both \"key\" and \"parent\" for element DataDefn" << endl;
            assert(false);
         }
      }
      else
      {
         // optional parent attribute
         key = XMLString::transcode("parent");
         value = attrs.getValue(key);
         if (value != NULL)
         {
            myDef.setParent(XMLString::transcode(value));
         } 
         else
         {
            cerr << "Must have either \"key\" or \"parent\" for element DataDefn" << endl;
            assert(false);
         }
      }
   }

   if (element == "DBEntry") 
   {
      key = XMLString::transcode("name");
      value = attrs.getValue(key);
      if (value != NULL)
      {
         myDef.addField(XMLString::transcode(value));
      } 
      else
      {
         cerr << "Attribute \"name\" not found for element DBEntry" << endl;
         assert(false);
      }
        
      // look for type
      key = XMLString::transcode("type");
      value = attrs.getValue(key);
      if (value != NULL)
      {
         myDef.addFieldType(stringToType(XMLString::transcode(value)));
      }
      else
      {
         myDef.addFieldType(STRINGTYPE);
      }
   }
   else if (element == "TableType")
   {
      myDef.addFieldType(TEXTBLOBTYPE);
   }
}




/// Handle a fatal error while parsing the description file.   
void MySAX2Handler::fatalError(const SAXParseException& exception)
{
   cerr << "Fatal Error: " << XMLString::transcode(exception.getMessage())
        << " at line: " << exception.getLineNumber()
        << endl;
   exit(0);
}


class FirstFieldTheSame
{
   public:
      FirstFieldTheSame(const ResultRecordtype & rec)
         : _rec(rec)
      {}
      bool operator()(const ResultRecordtype & o)
      {
         return o[0] == _rec[0];
      }
   private:
      const ResultRecordtype & _rec;
};
      

// Main driver program
int
main(int argc, char **argv )
{
   PSCommandLine::instance( (const int )argc, (const char**) argv );
   Log::initialize(Log::SYSLOG, Log::toLevel(PSConfiguration::instance()->getDebugLevel()), argv[0]);
   PSConfiguration::instance()->show();
   PSConfiguration::instance()->parseConfigFile();
   
   // Initialize the parser, read in the file and parse it
   try 
   {
      XMLPlatformUtils::Initialize();
   }
   catch (const XMLException& toCatch) {
      cerr << "Error during XML initialization!" << endl
           << "  " << toCatch.getMessage() << endl
           << " ******** CREATION FAILED! ********" << endl;
      return -1;
   }
   
   string xmlFile = PSConfiguration::instance()->popLeftOver();
   if (xmlFile == "")
   {
      cerr << "No xml file specified." << endl;
      exit(-1);
   }

   cerr << endl << "Processing file: " << xmlFile << endl;
   
   SAX2XMLReader* parser = XMLReaderFactory::createXMLReader();
    
   MySAX2Handler *myHandler = new MySAX2Handler();
   ContentHandler* contentHandler = myHandler;
   ErrorHandler* errHandler = (ErrorHandler*) contentHandler;
   parser->setContentHandler(contentHandler);
   parser->setErrorHandler(errHandler);

   try 
   {
      parser->parse(xmlFile.c_str());
   }
   catch (const XMLException& toCatch) 
   {
      cerr << "File not found: '" << xmlFile << endl
           << "  Exception message is: " << toCatch.getMessage() << endl;
      return -1;
   }

   // get the data from the parsed file
   ParsedDef result = myHandler->getMyDef();
   string myDomain = result.getDomain();
   string myParent = result.getParent();

   try
   {
      ConnectionManager::Connection dbconn(myDomain);

      // See if this table already exists in the database
      bool exists = dbconn.DBVerify(myDomain);

      if (exists) 
      {
         //DebugLog(<<"!!Table already exists!!");
      
         string existingTableKey;
         ResultTabletype existingTableDesc;
        
         try 
         {
            existingTableDesc = dbconn.DBDescribe(myDomain);
            existingTableKey = dbconn.DBPrimaryKey(myDomain);
         }
         catch (VDbException &e) 
         {
            cerr << "Error! Domain exists but unable to get description" << endl
                 << " ******** CREATION FAILED! ********" << endl;
         }
         bool isBase = (myParent == "");
         ResultTabletype parentDesc;
         if (!isBase)
         {
            parentDesc = dbconn.DBDescribe(dbconn.DBParent(myDomain));
         }
            
         ParsedDef existingDesc;

         existingDesc.setDomain(myDomain);
         existingDesc.setParent(myParent);
         existingDesc.setPrimaryKey(existingTableKey);

         ResultTabletype::const_iterator resultIterator;

         if (isBase)
         {
            for (resultIterator = existingTableDesc.begin();
                 resultIterator != existingTableDesc.end(); resultIterator++)
            {
               existingDesc.addField((*resultIterator)[0]);
               existingDesc.addFieldType(stringToType((*resultIterator)[1]));         
            }
         }
         else
         {
            for (resultIterator = existingTableDesc.begin();
                 resultIterator != existingTableDesc.end(); resultIterator++)
            {
               if (find_if(parentDesc.begin(), parentDesc.end(), FirstFieldTheSame(*resultIterator)) == parentDesc.end())
               {
                  existingDesc.addField((*resultIterator)[0]);
                  existingDesc.addFieldType(stringToType((*resultIterator)[1]));         
               }
            }
         }

         DifferenceSet diffs = result.compareOld(existingDesc);
        
         FieldListtype newFields = diffs.getNewFields();
         FieldListtype delFields = diffs.getDelFields();
        
         FieldListtype::iterator itr;

         bool changedParent = !( (myParent == dbconn.DBParent(myDomain)) ||
                                 (myParent == "" && (dbconn.DBParent(myDomain) == myDomain)) );

         if (changedParent || (isBase && diffs.changedPrimary()))
         {
            try 
            {
               dbconn.DBDrop(myDomain, false);
            }
            catch (VDbException &e) {
               cerr << "Error! Unable to remove domain "
                    << myDomain << "from database" << endl;
               return -1; 
            }

            try
            {
               if (myParent == "")
               {
                  dbconn.DBCreate(myDomain, 
                                  result.getFieldList(), 
                                  result.getFieldTypes(), 
                                  result.getPrimaryKey(), 
                                  IGNOREEXISTENCE);
               }
               else
               {
                  dbconn.DBCreate(myDomain, 
                                  myParent,
                                  result.getFieldList(), 
                                  result.getFieldTypes(), 
                                  IGNOREEXISTENCE);
               }
            }
            catch (VDbException &e) {
               cerr << "Error! Unable to create domain in database." << endl
                    << e.getDescription() << endl
                    << "  Perhaps the description was incorrect or you" << endl
                    << " don't have permission to create?\n" << endl
                    << " ******** CREATION FAILED! ********" << endl;
               return -1;
            }

            cout << "Succeeded in altering domain -- data dropped" << endl;
            return 0;
         }
        
         // check for fields that no longer exist in the xml description, remove
         // from database
         for (itr = delFields.begin(); itr != delFields.end(); itr++)
         {
            try
            {
               dbconn.DBAlter(myDomain, *itr);
            }
            catch (VDbException &e) {
               cerr << "Error! Unable to remove field " << *itr
                    << " from database." << endl
                    << "  Perhaps the description was incorrect or you"
                    << " don't have permission to delete?\n" << endl
                    << " ******** CREATION FAILED! ********" << endl;
               return -1;
            }
         }

         // check for new fields. In all cases, these are just added
         for (itr = newFields.begin(); itr != newFields.end(); itr++) 
         {
            try 
            {
            
               FieldListDatatype::const_iterator typeItr = result.getFieldTypes().begin();            
               advance(typeItr,
                       distance(result.getFieldList().begin(),
                                find(result.getFieldList().begin(), result.getFieldList().end(), *itr)));            

            

               dbconn.DBAlter(myDomain, *itr, *typeItr);
            }
            catch (VDbException &e) {
               cerr << "Error! Unable to add field " << *itr
                    << " to database." << endl
                    << "  Perhaps the description was incorrect or you"
                    << " don't have permission to add?\n" << endl
                    << " ******** CREATION FAILED! ********" << endl;
               return -1;
            }
         }
      }
      // if the domain doesn't exist, we add it
      else
      {
         try 
         {
            if (myParent == "")
            {
               dbconn.DBCreate(myDomain, 
                               result.getFieldList(), 
                               result.getFieldTypes(), 
                               result.getPrimaryKey(), 
                               IGNOREEXISTENCE);
            }
            else
            {
               dbconn.DBCreate(myDomain, 
                               myParent,
                               result.getFieldList(), 
                               result.getFieldTypes(), 
                               IGNOREEXISTENCE);
            }
         }
         catch (VDbException &e) {
            cerr << "Error! Unable to create domain in database." << endl
                 << e.getDescription() << endl
                 << "  Perhaps the description was incorrect or you don't" 
                 << "  have permission to create?\n" << endl
                 << " ******** CREATION FAILED! ********" << endl;
            return -1;
         }
      }
   }
   catch (VDbException& e)
   {
      InfoLog (<< "Database exception: " << e);
      cerr << "Database exception: " << e << endl;
   }
   catch (VException& e)
   {
      InfoLog (<< "exception: " << e);
      cerr << "exception: " << e << endl;
   }
   catch (...)
   {
      InfoLog (<< "Unknown exception: ");
   }
   
   return 0;
}
