#include <unistd.h>
#include <strstream>
#include <boost/lexical_cast.hpp>
#include "cppunit/TestCase.h"
#include "cppunit/TestCaller.h"
#include "cppunit/TestSuite.h"
#include "cppunit/TextTestRunner.h"
#include "cppunit/TextTestResult.h"
#include "VDbException.hxx"
#include "Logger.hxx"
#include "ConnectionManager.hxx"
#include "VThread.hxx"

#define VOCAL_SUBSYSTEM Vocal::Subsystem::TEST


using namespace CppUnit;
using namespace Epoisses;
using namespace Vocal;

/// Base for customized test case classes.
class ResultStringTestCase : public TestCase
{
   public:
      virtual ~ResultStringTestCase()
      {}
      
      virtual void setUp()
      {
         resultString.clear();
      }

      virtual void tearDown()
      {
         resultString.freeze(0);
      }

   protected:
      strstream resultString;
      
};


class CM1 : public ResultStringTestCase
{
   public:

      void testOne()
      {
         // grab a connection and release it a few times
         DebugLog(<< "testOne");
         {
            ConnectionManager::Connection conn;
         }
         {
            ConnectionManager::Connection conn;
         }
         {
            ConnectionManager::Connection conn;
         }
         {
            ConnectionManager::Connection conn;
         }
         {
            ConnectionManager::Connection conn;
         }
         {
            ConnectionManager::Connection conn;
         }
         {
            ConnectionManager::Connection conn;
         }
      }

      void testSeveral()
      {
         // grab several connections
         DebugLog(<< "testSeveral");
         ConnectionManager::Connection conn1;
         ConnectionManager::Connection conn2;
         ConnectionManager::Connection conn3;
      }

      // have more threads potentially wanting a connection than there
      // are connections
      void testConcurrent()
      {
         DebugLog(<< "testConcurrent");
         
         my_thread1.spawn(grabConnections, 0);
         my_thread2.spawn(grabConnections, 0);
         my_thread3.spawn(grabConnections, 0);
         my_thread4.spawn(grabConnections, 0);
         my_thread5.spawn(grabConnections, 0);
         my_thread6.spawn(grabConnections, 0);
         my_thread7.spawn(grabConnections, 0);
         my_thread8.spawn(grabConnections, 0);
         my_thread9.spawn(grabConnections, 0);
         my_thread10.spawn(grabConnections, 0);
         my_thread11.spawn(grabConnections, 0);
         my_thread12.spawn(grabConnections, 0);
         
         my_thread1.join();
         my_thread2.join();
         my_thread3.join();
         my_thread4.join();
         my_thread5.join();
         my_thread5.join();
         my_thread6.join();
         my_thread7.join();
         my_thread8.join();
         my_thread9.join();
         my_thread10.join();
         my_thread11.join();
         my_thread12.join();
      }

      static void* grabConnections(void*)
      {
         for (int i = 0; i < 100; i++)
         {
            // grab a connection and hold it for a while
            ConnectionManager::Connection conn;
            usleep(random()%3000);
         }
         return 0;
      }

   private:
      VThread my_thread1;
      VThread my_thread2;
      VThread my_thread3;
      VThread my_thread4;
      VThread my_thread5;
      VThread my_thread6;
      VThread my_thread7;
      VThread my_thread8;
      VThread my_thread9;
      VThread my_thread10;
      VThread my_thread11;
      VThread my_thread12;
};

#define cmTEST(_method) suite->addTest(new TestCaller<CM1>(#_method, &CM1::_method))

int
main(int argc, char** argv)
{
   Log::initialize(Log::COUT, Log::DEBUG, "testConnectionManager");

   // intialize the singleton
   ConnectionManager::Instance();

   TestSuite* suite = new TestSuite;
   
   TextTestResult result;

   cmTEST(testOne);
   cmTEST(testSeveral);
   cmTEST(testConcurrent);

   TextTestRunner runner;
   runner.addTest(suite);
   runner.run();
   return 0;
}
