static const char* const pserver_cxx_Version = "$Id: pserver.cxx,v 1.1 2002/09/30 04:55:16 jason Exp $";

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



#include <boost/lexical_cast.hpp>
#include <signal.h>

#include "ConnectionManager.hxx"
#include "Daemon.hxx"
#include "Logger.hxx"
#include "PSCommandLine.hxx"
#include "PSProtocolService.hxx"
#include "ProvisionServer.hxx"
#include "VEnvVar.hxx"
#include "VException.hxx"
#include "cpLog.h"
#include "TimeAccumulate.hxx"

#define VOCAL_SUBSYSTEM Vocal::Subsystem::PROVS

using namespace Epoisses;

void
signalHandler(int signo)
{
   cerr << "Caught Ctrl-C, exiting gracefully" << endl;
   TimeAccumulate::dump();
   abort();
}

int main(int argc, char** argv)
{
   if ( signal( SIGPIPE, SIG_IGN) == SIG_ERR)
   {
      cerr << "Couldn't install signal handler for SIGPIPE" << endl;
      exit(-1);
   }

   if ( signal( SIGINT, signalHandler ) == SIG_ERR )
   {
      cerr << "Couldn't install signal handler for SIGINT" << endl;
      exit( -1 );
   }

   // Read command line options.  If the config file option is selected,
   // then the parameters given in the config file will be used.

   PSCommandLine::instance( (const int )argc, (const char**) argv );
   Log::initialize(Log::SYSLOG, Log::toLevel(PSConfiguration::instance()->getDebugLevel()), argv[0]);
   PSConfiguration::instance()->show();
   if ( PSConfiguration::instance()->getDaemonMode() )
   {
      Daemon();
   } 

   // reference singletons
   ProvisionServer::Port = boost::lexical_cast<int>(PSConfiguration::instance()->getLocalPort());
   ConnectionManager::Instance();
   PSProtocolServiceHolder::Instance();
   
   while (1)
   {
      try
      {
         ProvisionServer::Instance().run();
      }
      catch ( VException& e )
      {
         ErrLog (<< "Fatal exception. Ignoring");
         DebugLog (<< "caught: " << e);
      }
      catch ( ... )
      {
         ErrLog (<< "Fatal unknown exception. Ignoring");
      }
      ErrLog (<< "Sleeping 5 seconds");
      sleep (5);
   }
}

