// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

static const char* const CommandLineParser_cxx_Version =
"$Id: CommandLineParser.cxx,v 1.1 2002/09/30 04:55:17 jason Exp $";

#include <boost/lexical_cast.hpp>
#include <getopt.h>
#include <cstdio>

#include "RandomHex.hxx"
#include "CommandLineParser.hxx"
#include "Logger.hxx"

#define VOCAL_SUBSYSTEM Vocal::Subsystem::PROVS

using namespace Vocal;
using namespace Epoisses;

CommandLineParser::CommandLineParser(int argc, char* argv[]) : 
   _threads(10),
   _daemon(false),
   _port(5060),
   _level(Log::INFO),
   _type(Log::SYSLOG),
   _pserver("localhost"),
   _proxy("localhost")
{
   int c = 0;
    
   while( ( c = getopt( argc, const_cast<char**>( argv ),
                        "dv:t:p:l:s:x:" ) ) != EOF )
   {
      switch( c )
      {
         case 'd':
            _daemon = true;
            break;

         case 'v':
            // loglevel
            _level = Log::toLevel(string(optarg));
            break;

         case 't':
            // num of threads
            _threads = boost::lexical_cast<int,char*>(optarg);
            break;

         case 'p':
            // localport
            _port = boost::lexical_cast<int,char*>(optarg);
            break;
            
         case 'l':
         {
            string arg(optarg);
            _type = ((arg == "cout" || arg == "COUT") ? Log::COUT : Log::SYSLOG);
            break;
         }
         case 's':
            _pserver = string(optarg);
            break;
         case 'x':
            _proxy = string(optarg);
            break;
            
         case '?':
            cerr << "Unknown arg" << endl;
            cerr << "Usage:  " << argv[0] << " [OPTION]..." << endl;
            cerr << endl;
            cerr << "  Where [OPTION] is one of: " << endl;
            cerr << "    -v <log_level>                      "
                 << "    set loglevel" << endl;
            cerr << "    -p <local port>                     "
                 << "    set port to listen for incoming\n" 
                 << "      requests" << endl;
            cerr << "    -d                                  " 
                 << "    disable daemon mode (run in \n" 
                 << "      foreground)" << endl;
            cerr << "    -t <number of threads (25)>         " 
                 << "    number of request proccess threads." << endl;
            cerr << "    -l <cout|syslog>         " << endl;
            cerr << "    -s <pserver-host>        " << endl;
            cerr << "    -x <proxy-host:port>     " << endl;
            cerr << endl;
            exit( -1);

            break;

      };
   }

   if ( ( argc - optind ) > 0 )
   {
      InfoLog( << "Unknown option: " << argv[ optind ] );
      exit( -1 );
   }

   RandomHex::initialize();
}

