package org.vovida.prov.psLib;

/// Fields can take on values 
public abstract class Field extends BaseField 
{ 
      static class NotBoundException extends RuntimeException 
      { 
      } 

      Field(String name) 
      { 
         super(name); 
         this.isBound = false; 
      } 
             
      protected boolean isBound() 
      { 
         return isBound; 
      } 

      /// Called by DBDresult.Iterator 
      void setBound() 
      { 
         isBound = true; 
      } 
             
      /// Called by DBDresult.Iterator 
      void clearBound() 
      { 
         isBound = false; 
      } 

      /// Called by DBDresult.Iterator 
      abstract void bind(String v); 
               
      private boolean isBound; 
} 

class BaseField implements ProvisioningInterfaceElement
{ 
      BaseField(String name) 
      { 
         if (name == null || name == "")
         {
            throw new IllegalArgumentException("Field names must be non-empty");
         }

         this.name = name.toLowerCase(); 
      } 

      protected BaseField(String name, boolean preserve) 
      { 
         if (name == null)
         {
            throw new IllegalArgumentException("Constant values must be non-null");
         }

         this.name = name; 
      } 
             
      public void stringify(StringBuffer s)
      {
         s.append(name);
      } 

      static public String encodeString(String v)
      {
         // !dlb! deal with embedded single quotes
         return "'" + v + "'";
      }

      static public String decodeString(String v)
      {
         // !dlb! deal with embedded single quotes
         return v;
      }

      // already quoted
      static public String encodeEmbeddedString(String v)
      {
         // !dlb! deal with embedded single quotes, commas
         return v;
      }

      // already dequoted
      static public String decodeEmbeddedString(String v)
      {
         // !dlb! deal with embedded single quotes, commas
         return v;
      }

      String name;
} 

class Constant extends BaseField 
{ 
      Constant(boolean v) 
      { 
         super(v ? "true" : "false", true);
      } 

      Constant(int v) 
      { 
         super(Integer.toString(v), true); 
      } 

      Constant(double v) 
      { 
         super(Double.toString(v), true); 
      } 

      Constant(String v) 
      { 
         super(encodeString(v), true); 
      } 

      Constant(java.util.List v) 
      { 
         super(listToString(v), true); 
      } 
             
      Constant(Object[] v) 
      { 
         super(arrayToString(v), true); 
      } 

      Constant(boolean[] v) 
      { 
         super(booleanArrayToString(v), true);
      }

      Constant(int[] v) 
      { 
         super(intArrayToString(v), true);
      }

      Constant(double[] v) 
      { 
         super(doubleArrayToString(v), true);
      }

      static String listToString(java.util.List v) 
      { 
         StringBuffer s = new StringBuffer();

         s.append("'"); 
         java.util.Iterator i = v.iterator(); 
         boolean first = true; 
         while (i.hasNext()) 
         { 
            if (!first) { 
               s.append(","); 
            } 
            first = false; 
            s.append(encodeEmbeddedString(i.next().toString())); 
         } 
         s.append("'"); 
         return s.toString(); 
      } 

      static String arrayToString(Object[] v) 
      { 
         StringBuffer s = new StringBuffer();
               
         s.append("'");               
         for (int i = 0; i < v.length; i++) { 
	    if (i != 0) { 
               s.append(","); 
	    } 
	    s.append(encodeEmbeddedString(v[i].toString())); 
         } 
         s.append("'"); 

         return s.toString(); 
      } 

      static String booleanArrayToString(boolean[] v) 
      { 
         StringBuffer s = new StringBuffer();
               
         s.append("'");               
         for (int i = 0; i < v.length; i++) { 
	    if (i != 0) { 
               s.append(","); 
	    } 
	    s.append(v[i]); 
         } 
         s.append("'"); 

         return s.toString(); 
      } 

      static String intArrayToString(int[] v) 
      { 
         StringBuffer s = new StringBuffer();
               
         s.append("'");               
         for (int i = 0; i < v.length; i++) { 
	    if (i != 0) { 
               s.append(","); 
	    } 
	    s.append(v[i]); 
         } 
         s.append("'"); 

         return s.toString(); 
      } 

      static String doubleArrayToString(double[] v) 
      { 
         StringBuffer s = new StringBuffer();
               
         s.append("'");               
         for (int i = 0; i < v.length; i++) { 
	    if (i != 0) { 
               s.append(","); 
	    } 
	    s.append(v[i]); 
         } 
         s.append("'"); 

         return s.toString(); 
      } 
}
