package org.vovida.prov.psLib;

public interface Filter extends ProvisioningInterfaceElement
{ 
      public static class NoFilter implements Filter
      {
            public void stringify(StringBuffer s)
            {}

            public String toString()
            {
               return "";
            }
      }

      public static final NoFilter None = new NoFilter();
} 

class Eq implements Filter
{
      Eq(BaseField f1,
         BaseField f2)
      {
         this.f1 = f1;
         this.f2 = f2;
      }

      public void stringify(StringBuffer s)
      {
         s.append("(");
         f1.stringify(s);
         s.append(" = ");
         f2.stringify(s);
         s.append(")");
      }

      public String toString()
      {
         StringBuffer s = new StringBuffer();
         stringify(s);
         return s.toString();
      }

      private BaseField f1;
      private BaseField f2;
}

class Ne implements Filter
{
      Ne(BaseField f1,
         BaseField f2)
      {
         this.f1 = f1;
         this.f2 = f2;
      }

      public void stringify(StringBuffer s)
      {
         s.append("(");
         f1.stringify(s);
         s.append(" != ");
         f2.stringify(s);
         s.append(")");
      }

      public String toString()
      {
         StringBuffer s = new StringBuffer();
         stringify(s);
         return s.toString();
      }

      private BaseField f1;
      private BaseField f2;
}

class Lt implements Filter
{
      Lt(BaseField f1,
         BaseField f2)
      {
         this.f1 = f1;
         this.f2 = f2;
      }

      public void stringify(StringBuffer s)
      {
         s.append("(");
         f1.stringify(s);
         s.append(" &lt; ");
         f2.stringify(s);
         s.append(")");
      }

      public String toString()
      {
         StringBuffer s = new StringBuffer();
         stringify(s);
         return s.toString();
      }

      private BaseField f1;
      private BaseField f2;
}

class Gt implements Filter
{
      Gt(BaseField f1,
         BaseField f2)
      {
         this.f1 = f1;
         this.f2 = f2;
      }

      public void stringify(StringBuffer s)
      {
         s.append("(");
         f1.stringify(s);
         s.append(" &gt; ");
         f2.stringify(s);
         s.append(")");
      }

      public String toString()
      {
         StringBuffer s = new StringBuffer();
         stringify(s);
         return s.toString();
      }

      private BaseField f1;
      private BaseField f2;
}

class Le implements Filter
{
      Le(BaseField f1,
         BaseField f2)
      {
         this.f1 = f1;
         this.f2 = f2;
      }

      public void stringify(StringBuffer s)
      {
         s.append("(");
         f1.stringify(s);
         s.append(" &lt;= ");
         f2.stringify(s);
         s.append(")");
      }

      public String toString()
      {
         StringBuffer s = new StringBuffer();
         stringify(s);
         return s.toString();
      }

      private BaseField f1;
      private BaseField f2;
}

class Ge implements Filter
{
      Ge(BaseField f1,
         BaseField f2)
      {
         this.f1 = f1;
         this.f2 = f2;
      }

      public void stringify(StringBuffer s)
      {
         s.append("(");
         f1.stringify(s);
         s.append(" &gt;= ");
         f2.stringify(s);
         s.append(")");
      }

      public String toString()
      {
         StringBuffer s = new StringBuffer();
         stringify(s);
         return s.toString();
      }

      private BaseField f1;
      private BaseField f2;
}

class Like implements Filter
{
      Like(BaseField f1,
           BaseField f2)
      {
         this.f1 = f1;
         this.f2 = f2;
      }

      public void stringify(StringBuffer s)
      {
         s.append("(");
         f1.stringify(s);
         s.append(" LIKE ");
         f2.stringify(s);
         s.append(")");
      }

      public String toString()
      {
         StringBuffer s = new StringBuffer();
         stringify(s);
         return s.toString();
      }

      private BaseField f1;
      private BaseField f2;
}

/// Allow Constant(boolean) as filter element
class Identity implements Filter
{
      Identity(boolean v)
      {
         this.f1 = new Constant(v);
      }

      public void stringify(StringBuffer s)
      {
         s.append("(");
         f1.stringify(s);
         s.append(")");
      }

      public String toString()
      {
         StringBuffer s = new StringBuffer();
         stringify(s);
         return s.toString();
      }

      private BaseField f1;
}

