<?php

// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

require_once("ProvisioningInterface.php");

class User extends Domain
{
  // 1. all initialized member vars are persistent
  // 2. persistent member vars MUST be initialized to their database type
  // 3. first initialized member var is the primary key (if parent is Domain)

  var $userId = "string";
  var $expired = true;
  var $credits = 0;
  var $count = 0;
  var $aliases = array();

  function User($domain = "TestUsers")
  {
    $this->Domain($domain);
  }
}

$user = &new User();

print "test domain->toString()\n";
print $user->toString();
exit();

assert($user->toString() == "User[userId=string, expired=1, credits=0, count=0, aliases='']");

$prov = &new ProvisioningInterface("ottawa.gloo.net", 6005);

print "test remove all\n";
$prov->deleteRecordsInDomain($user,
                             NoFilter());

print "test getRecordsInDomain, empty result\n";
$res = $prov->getRecordsInDomain($user, 
                                 like(userId, '%@home.com'),
                                 array(userId, count, expired, aliases));
assert($res->toString() == "Result[
]
");

print "test addRecordInDomain\n";
$prov->addRecordInDomain($user,
                         array(userId => 'sip:belle@home.com',
                               count => 17,
                               expired => false,
                               credits => 1000));

print "test addRecordInDomain, array\n";
$prov->deleteRecordsInDomain($user,
                             eq(userId, 'sip:speedy@home.com'));

$prov->addRecordInDomain($user,
                         array(userId => 'sip:speedy@home.com',
                               count => 21,
                               aliases => array(speed, pf, cow, klunk),
                               expired => false,
                               credits => 1000));

print "test getRecordsInDomain, LIKE, array\n";
$res = $prov->getRecordsInDomain($user, 
                                 like(userId, '%@home.com'),
                                 array(userId, count, expired, aliases));
assert($res->toString() == "Result[
 {userid => sip:belle@home.com, count => 17, expired => f, aliases => }
 {userid => sip:speedy@home.com, count => 21, expired => f, aliases => speed,pf,cow,klunk}
]
");

print "test BindIterator\n";

$out = "";
$iterator = $res->bindIterator($user);
while ($iterator->hasNext()) {
   $iterator->next();
   $out .= $user->toString()."\n";
}
assert($out == "User[userId=sip:belle@home.com, expired=, credits=0, count=17, aliases='']
User[userId=sip:speedy@home.com, expired=, credits=0, count=21, aliases='speed,pf,cow,klunk']
");


print "test getRecordsInDomain\n";
$res = $prov->getRecordsInDomain($user, 
                                 gt(count, 17),
                                 array(userId, count, expired));
assert($res->toString() == "Result[
 {userid => sip:speedy@home.com, count => 21, expired => f}
]
");

print "test getRecordsInDomain\n";
$res = $prov->getRecordsInDomain($user, 
                                 lt(count, 18),
                                 array(userId, count, expired));
assert($res->toString() == "Result[
 {userid => sip:belle@home.com, count => 17, expired => f}
]
");

print "test setRecordsInDomain\n";
$prov->setRecordsInDomain($user,
                          lt(count, 18),
                          array(count => 25,
                                aliases => array('trouble', 'cg')));
$res = $prov->getRecordsInDomain($user,
                                 eq(count, 25),
                                 array(userId, count, expired));
assert($res->toString() == "Result[
 {userid => sip:belle@home.com, count => 25, expired => f}
]
");

print "test setRecordsInDomain, field reference\n";
$prov->setRecordsInDomain($user,
                          cpOr(le(count, 0), expired),
                          array(count => $user->DB->credits,
                                expired => false));

print "test deleteRecordInDomain\n";
$prov->deleteRecordsInDomain($user,
                             cpOr(le(count, 0),
                                  expired));

// test duplicate primary key value rejection
class SpecUser extends User
{
  function SpecUser($domain = "TestSpecUsers")
  {
    $this->User($domain);
  }
}

print "test duplicate primary key detection\n";
$prov->deleteRecordsInDomain($user,
                             NoFilter());

$prov->addRecordInDomain($user,
                         array(userId => 'sip:belle@home.com',
                               count => 17,
                               expired => false,
                               credits => 1000));

$specUser = new SpecUser();
$prov->addRecordInDomain($specUser,
                         array(userId => 'sip:belle@home.com',
                               count => 17,
                               expired => false,
                               credits => 1000));
?>
