<?php
// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

// define is-a if not defined
if (!function_exists('is_a')) {
  function is_a(&$instance, $anc_class_name)
  {
    return is_a_helper(get_class($instance), strtolower($anc_class_name));
  }

  function is_a_helper(&$class_name, &$anc_class_name) 
  {
    if (!$class_name) {
      return false;
    }

    if ($class_name == $anc_class_name) {
      return true;
    }

    return is_a_helper(get_parent_class($class_name), 
                       $anc_class_name);
  }
}

if (!function_exists('array_key_exists')) {
  function array_key_exists($key, &$arr)
  {
    foreach ($arr as $k => $v) {
      if ($k == $key) {
        return true;
      } 
    }
    return false;
  }
}

define("ElementDelimeter", ","); // .dlb. comma may not be the best choice 

// Cache the name and initial value of persistent vars and do translations.
class Field
{
  var $name;
  var $phpName;
  var $init; // set from initial value
    
  function Field($name, $init)
  {
    $this->phpName = $name;
    $this->name = strtolower($name);
    $this->init = $init;
  }

  // translate to DB type
  function &dbValue(&$value)
  {
    return $this->castToDBType($this->init, $value);
  }
  
  // static
  function castToDBType(&$castTypeValue, &$value)
  {
    if (is_null($value)) {
      trigger_error("Null value for castToDBType", E_USER_ERROR);
      return;
    }

    // fields are not touched -- better be SQL compatible types
    if (is_a($value, 'Field')) {
      return $value->name;
    }

    // if it is an object, call encode on it
    if (is_object($value)) {
      return $this->encode($value);
    }
     
    // use PHP conversions, you get what you deserve
    if (is_int($castTypeValue)) {
      return (int)$value;
    }
    if (is_bool($castTypeValue)) {
      if ((bool)$value) {
        return 'true';
      } else {
        return 'false';
      }
    }
    if (is_float($castTypeValue)) {
      return (float)$value;
    }
    if (is_string($castTypeValue)) {
      return "'".htmlspecialchars((string)$value)."'"; // strings are encoded and quoted
    }    
    if (is_array($castTypeValue)) {
      $a = (array)$value;
      return $this->arrayToString($a);
    }    
  }

  // translate from DB type
  function &phpValue(&$value)
  {
    return $this->castToPhpType($this->init, $value);
  }

  function &castToPhpType(&$castTypeValue, &$value)
  {
    // use PHP conversions, you get what you deserve
    if (is_int($castTypeValue)) {
      return (int)$value;
    }
    if (is_bool($castTypeValue)) {
      // not clear what to do here -- 't' is true for DB, but everything but "",
      // 0, 0.0, and FALSE are true for PHP
      if ($value == 'f') {
        return FALSE;
      } else {
        return TRUE;
      }
    }
    if (is_float($castTypeValue)) {
      return (float)$value;
    }
    if (is_string($castTypeValue)) {
       $s = (string)$value; // !dlb! unencode html
      return $s;
    }    
    if (is_array($castTypeValue)) {
      $s = (string)$value; // !dlb! unencode html
      return $this->stringToArray($s);
    }    
    // if the expected type is an object, call decode on the prototype (i.e. var init)
    if (is_object($castTypeValue)) {
      return $castTypeValue->decode($value);
    }
  }      

  // convert from CSV to array
  function &stringToArray(&$s)
  {
    return explode(ElementDelimeter, $s);
  }

  // convert from array to CVS
  function &arrayToString(&$a)
  {
    $first = TRUE;
      
    $s = "'";
    foreach ($a as $k => $v) {
      if (is_string($k)) {
        trigger_error("String key neglected -- only value stored", E_WARNING);
      }
      
      if ($first) {
        $s = $s.$v;
        $first = false;
      } else {
        $s = $s.ElementDelimeter.$v;
      }
    }
    return $s."'";
  }
}

// All filters are composed of Operators
class Operator
{
  var $name;

  function Operator($name)
  {
    $this->name = $name;
  }

  function stringify(&$domain, &$x)
  {
    return Field::castToDBType($x, $x); // return as passed type for SQL
  }

  // output leaf fields as booleans
  function stringifyB(&$domain, $x)
  {
    if (is_a($x, 'Operator')) {
      return $x->toString($domain);
    } else if (is_a($x, 'Field')) {
      return ''.$x.' == true';
    } else if (is_string($x)) {
      return $x;               // treat string literal as Field
    } else  return !!$x;
  }
}

class UnaryOperator extends Operator
{
  var $arg1;

  function UnaryOperator($name, &$arg1)
  {
    $this->Operator($name);
    $this->arg1 = $arg1;
  }

  // takes a single boolean arg
  function &toString(&$domain)
  {
    return $this->name.'('.$this->stringifyB($domain, $this->arg1).')';
  }
}

class BinaryOperator extends Operator
{
  var $arg1;
  var $arg2;

  function BinaryOperator($name, &$arg1, &$arg2)
  {
    $this->Operator($name);
    $this->arg1 = $arg1;
    $this->arg2 = $arg2;
  }

  // Allow the client to use literal strings to represent Fields by name.
  // Field to Field operations are likely rare and are permitted only with
  // explicit Fields
  //
  // string Op value ==> Field(string)->name Op Field(string)->dbValue(value)
  // field Op value ==> field->name Op field->dbValue(value)
  // field1 Op field2 ==> field1->name Op field2->name
  // value Op field ==> illegal
  // string1 Op string2 ==> Field(string1)->name Op Field(string1)->dbValue(string2)
  // value Op value ==> NOT PERMITTED
  function &toString(&$domain)
  {
    if (is_a($this->arg1, 'Field')) {
      $s = '(';
      $s .= $this->arg1->name;
      $s .= ' '.$this->name;
      // convert to Field's type
      $s .= ' '.$this->arg1->dbValue($this->arg2);
      $s .= ')';
      return $s;
    }
    
    if (is_string($this->arg1)) {
      // treat argument 1 as Field name
      $fieldName = $this->arg1;
      $s = '(';
      $s .= strtolower($fieldName);
      $s .= ' '.$this->name;
      $field = $domain->DB->$fieldName;
      if (!$field) {
        trigger_error("Domain ".$domain->domainName." has no field '$fieldName'",
                      E_USER_ERROR);
      }
      $s .= ' '.$field->dbValue($this->arg2);
      $s .= ')';
      return $s;
    }

    if (is_a($this->arg2, 'Field')) {    
      $s = '(';
      // convert to Field's type
      $s .= ' '.$this->arg2->dbValue($this->arg1);
      $s .= ' '.$this->name;
      $s .= $this->arg2->name;
      $s .= ')';
      return $s;
    }
    
    trigger_error("Operators MUST include a Field", E_USER_ERROR);
  }
}

class NaryOperator extends Operator
{
  var $args;

  function NaryOperator($name, &$args)
  {
    $this->Operator($name);
    $this->args = $args;
  }

  function &toString(&$domain)
  {
    $s = '(';
    if (count($this->args) == 1) {
      $s .= $this->args[0];
    } else {
      for ($i = 0; $i < count($this->args); $i++) {
        if ($i != 0) {
          $s .= ' '.$this->name.' ';
        }
        // treat string arg as Field name
        $s .= $this->stringifyB($domain, $this->args[$i]);
      }
    }
    $s .= ')';
    return $s;
  }
}

class NoOp extends Operator
{
  function NoOp()
  {
     $this->Operator("");
  }

  function &toString(&$domain)
  {
    $s = "";
    return $s;
  }
}

function &NoFilter()
{
   return new NoOp();
}

// global names for operators on Filters
function &cpAnd()
{
  $a =  func_get_args();
  return new NaryOperator("AND", $a);
}

function &cpOr()
{
  $a = func_get_args();
  return new NaryOperator("OR", $a);
}

function &Not($arg)
{
  return new UnaryOperator("NOT", $arg);
}

function &Eq($arg1, $arg2)
{
  return new BinaryOperator("=", $arg1, $arg2);
}

function &Ne($arg1, $arg2)
{
  return new BinaryOperator("!=", $arg1, $arg2);
}

function &Gt($arg1, $arg2)
{
  return new BinaryOperator("&gt;", $arg1, $arg2);
}

function &Lt($arg1, $arg2)
{
  return new BinaryOperator("&lt;", $arg1, $arg2);
}

function &Ge($arg1, $arg2)
{
  return new BinaryOperator("&gt;=", $arg1, $arg2);
}

function &Le($arg1, $arg2)
{
  return new BinaryOperator("&lt;=", $arg1, $arg2);
}

function Like($arg1, $arg2)
{
  return new BinaryOperator("LIKE", $arg1, $arg2);
}

// each instance is customized for a particular domain class
class DomainDescriptor
{
  var $domainFields = array();
}

// shared DomainDescriptors instance of same Domain
$CPROV_DBs = array();
global $CPROV_DBs;
$DependentDomains = array();
global $DependentDomains;

// Base class for all domain classes
// Each derived domain specifies initialized class vars for each column.
// The class var name must be the column name.
// The class var's initial value type must match the column type.
// The first initialized class var of a domain that directly derives from Domain
//   is the primary key field.
class Domain
{
  var $domainName;
  var $printName;
  // descriptor for this instance (not generally seen by end-user)
  var $DB;

  function Domain($name)
  {
    $this->printName = $name;
    $this->domainName = strtolower($name);
    $this->DB = $this->generateDescriptor();
  }

  // return the name of the root Domain
  function& getRoot()
  {
    // traverse up the parent classes to just before Domain
    $c = get_class($this);
    if ($c == 'domain') {
      trigger_error("Called getRoot on a direct instance of Domain!",
                    E_USER_ERROR);
    }
    $p = get_parent_class($c);
    while ($p != 'domain') {
      $c = $p;
      $p = get_parent_class($c);
    }
    return $c;
  }

  // return the primary key field -- first field in root domain
  function& getPrimaryKey()
  {
    return $this->DB->domainFields[0]->phpName;
  }

  function& getAll()
  {
     return array_keys($this->getClassVars());
  }
        
  /// return the declared vars with root vars first
  function &getClassVars()
  {
     // retrieve all of the vars
     $vars = get_class_vars(get_class($this));

     // retrieve the base vars
     $root = $this->getRoot();
     $base = get_class_vars($root);
   
     // move the base vars to the front of the array of vars
     // this maintains the primary key, sigh
     // $d = array_diff($vars, $base);
     return array_merge($base, $vars);
  }

  // if it doesn't already exist,
  // create a DB reflection instance customized to the derived Domain
  function &generateDescriptor()
  {
    global $CPROV_DBs;

    if (!array_key_exists(get_class($this), $CPROV_DBs)) {
      //print "Domain->generateDB\n";
      $db = &new DomainDescriptor();

      // cache the persistent fields
      $cvars = $this->getClassVars();
      $cvarsKeys = array_keys($cvars);
      // remove transients
      $trans = $this->getTransients();
      array_unique($trans);
      $pvars = array_diff($cvarsKeys, $trans);

      // make a Field from the member name and initial value (type info)
      foreach ($pvars as $fieldName) {
        $field = &new Field($fieldName, $cvars[$fieldName]);
        array_push($db->domainFields, $field);
        // set the field into the descriptor for direct access
        //print "setting ".$this->domainName."->DB->$".$fieldName."\n";
        $db->$fieldName = &$field;
      }
      $CPROV_DBs[get_class($this)] = $db;
    }
    
    return $CPROV_DBs[get_class($this)];
  }

  // Fields that are not persisted.
  // Overriders should look like:
  // $trans = new array("trans1", "trans2", ...");
  // return array_merge($trans, <parent>::getTransients());
  // (alternatively, do an is-a walk and union the sets in the
  //  base code? not clear that can be done in PHP -- all functions are virtual?)
  function &getTransients()
  {
    return array("DB", "domainName", "domainKey", "domainForeignKey");
  }

  // clear all non-transient values
  function unsetAll()
  {
    $cvars = &$this->DB->domainFields;
    foreach ($cvars as $k => $cvar) {
      $this->$cvar = NULL;
    }
  }

  // generic dump
  function &toString()
  {
    $s = $this->printName;
    $s .= "[\n";
    foreach ($this->DB->domainFields as $cvar) {
      $s .= " ".$cvar->phpName."=";
      $key = $cvar->phpName;
      $s .= $cvar->dbValue($this->$key);
      $s .= ",\n";
    }
    $s .= "]\n";

    return $s;
  }

  function registerDependent(&$domain)
  {
     global $DependentDomains;

     $k = $this->getRoot();
     // create an array entry if needed
     if (!array_key_exists($k, $DependentDomains)) {
        $DependentDomains[$k] = array($domain);
     }
     else {
        // add to the existing entry
        $DependentDomains[$k][] = $domain;
     }
  }

  // must override if the derived Domain registers as a dependent
  function dependentRemove(&$domain, &$id)
  {
     trigger_error($this->domainName."->dependentRemove not overridden",
                   E_USER_ERROR);
  }
}

class Result
{
  var $values = array(); // array of array of values indexed by fields

  function Result(&$xmlParseTree)
  {
    // each row is top ="children"=> 2..
    $matches = $xmlParseTree['children'];
    array_shift($matches); // "domain"
    array_shift($matches); // "resultcount"

    foreach ($matches as $row) {
      //print "row is ";
      //var_dump($row['children'][0]['children']);
      
      $a = array();
      foreach ($row['children'][0]['children'] as $rowEl) {
        //print "rowEl is ";
        //var_dump($rowEl);
        $a[$rowEl['tag']] = $rowEl['value'];
      }
      $this->values[] = $a;
    }
  }

  function count()
  {
    return count($this->values);
  }

  function &bindIterator(&$domain) 
  {
    return new BindIterator($domain, $this);
  }

  function &toString()
  {

    $s = "Result[\n";
    foreach($this->values as $row) {
      $s .= " {";
      $first = true;
      foreach ($row as $key => $value) {
        if (!$first) {
          $s .= ", ";
        }
        $first = false;
        $s .= "$key => $value";
      }
      $s .= "}\n";
    }
    $s .= "]\n";
    return $s;
  }
}

// sets the members of the passed class object
// against each of the result's rows in turn
class BindIterator
{
  var $count;
  var $domain;
  var $result;

  function BindIterator(&$domain, &$result)
  {
    $this->domain = &$domain;
    $this->result = &$result;
    $this->count = 0;

    // clear all values
    foreach ($this->domain->DB->domainFields as $field) {
       $name = $field->phpName;
       // intepret according to Field type
       $this->domain->$name = NULL;
    }
  }

  function hasNext()
  {
    return $this->count < count($this->result->values);
  }
    
  function next()
  {
    if (!$this->hasNext()) {
      trigger_error("Advanced past end of iterator", E_USER_ERROR);
    }

    // set the values of domain by the fields
    $vals = $this->result->values[$this->count];
    
    foreach ($vals as $cvar => $value) {
      // field names from DB are lowercase, have to find the Field
      foreach ($this->domain->DB->domainFields as $field) {
        if ($field->name == $cvar) {
          $name = $field->phpName;
          // intepret according to Field type
          $this->domain->$name = $field->phpValue($value);
          break;
        }
      }
    }
    $this->count++;
  }
}

?>
