<?php
// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

require_once("cprov.php");

// Examples
class User extends Domain
{
  // 1. all initialized member vars are persistent
  // 2. persistent member vars MUST be initialized to their database type

  var $userId = "initial user id";
  var $expired = true;
  var $credits = 0;
  var $count = 0;

  function User()
  {
    $this->Domain("User");
  }
}

print "test derived domain\n";
$user = &new User();
//var_dump($user);

print "test eq operator\n";
$op = eq("userId", "sip:bill@work.com");
assert($op->toString($user) == "(userid = 'sip:bill@work.com')");

print "test and operator\n";
$op = cpAnd(eq("userId", "sip:bill@work.com"),
            gt("count", 14));
assert($op->toString($user) == "((userid = 'sip:bill@work.com') AND (count &gt; 14))");

print "test or operator\n";
$op = cpOr(lt("count", 17),
           "expired");
assert($op->toString($user) == "((count &lt; 17) OR expired)");

print "test not operator\n";
$op = Not(expired);
assert($op->toString($user) == "NOT(expired)");

print "test not operator, composed\n";
$op = Not(lt("count", 17));
assert($op->toString($user) == "NOT((count &lt; 17))");

print "test operator composition, field reference\n";
$op = cpOr(cpAnd(Not(expired),
                 gt(count, 100)),
           cpAnd(le(count, 57),
                 ge(credits, $user->DB->count)));
assert($op->toString($user) == "((NOT(expired) AND (count &gt; 100)) OR ((count &lt;= 57) AND (credits &gt;= count)))");

print "test value to field conversion\n";
$op = lt("userId", 17);
assert($op->toString($user) == "(userid &lt; '17')");

print "test value to field comparison\n";
$op = lt(count, $user->DB->credits);
assert($op->toString($user) == "(count &lt; credits)");

print "test dbValue conversions\n";
// types: Field, boolean, integer, float, string, array
$b = TRUE;
$i = 10;
$f = 11.11;
$s = 'string';
$a = array('a','b','c','d');

$field = &new Field("field", "value");
$booleanField = &new Field("booleanField", TRUE);
$integerField = &new Field("integerField", 11);
$floatField = &new Field("floatField", 11.11);
$stringField = &new Field("stringField", "string");
$arrayField = &new Field("arrayField", array());

assert($booleanField->dbValue($field) == 'field');
assert($booleanField->dbValue($b) == 'true');
assert($booleanField->dbValue($i) == 'true');
assert($booleanField->dbValue($f) == 'true');
assert($booleanField->dbValue($s) == 'true');
assert($booleanField->dbValue($a) == 'true');

assert($integerField->dbValue($field) == 'field');
assert($integerField->dbValue($b) == 1);
assert($integerField->dbValue($i) == $i);
assert($integerField->dbValue($f) == 11);
assert($integerField->dbValue($s) == 0);
assert($integerField->dbValue($a) == 1); // ?dlb? really? why?

assert($floatField->dbValue($field) == 'field');
assert($floatField->dbValue($b) == 1.0);
assert($floatField->dbValue($i) == 10.0);
assert($floatField->dbValue($f) == $f);
assert($floatField->dbValue($s) == 0.0);
assert($floatField->dbValue($a) == 1.0);

assert($stringField->dbValue($field) == 'field');
assert($stringField->dbValue($b) == "'1'");
assert($stringField->dbValue($i) == "'10'");
assert($stringField->dbValue($f) == "'11.11'");
assert($stringField->dbValue($s) == "'".$s."'");
assert($stringField->dbValue($a) == "'Array'");

assert($arrayField->dbValue($field) == 'field');
assert($arrayField->dbValue($b) == "'1'");
assert($arrayField->dbValue($i) == "'10'");
assert($arrayField->dbValue($f) == "'11.11'");
assert($arrayField->dbValue($s) == "'".$s."'");
assert($arrayField->dbValue($a) == "'a,b,c,d'");

// test all combinations of field->phpValue
$b = 't';
$i = '10';
$f = '11.11';
$s = 'string';
$a = 'a,b,c,d';

assert($booleanField->phpValue($b) == TRUE);
assert($booleanField->phpValue($i) == TRUE);
assert($booleanField->phpValue($f) == TRUE);
assert($booleanField->phpValue($s) == TRUE);
assert($booleanField->phpValue($a) == TRUE);
$b = 'f';
assert($booleanField->phpValue($b) == FALSE);

assert($integerField->phpValue($b) == 0);
assert($integerField->phpValue($i) == 10);
assert($integerField->phpValue($f) == 11);
assert($integerField->phpValue($s) == 0);
assert($integerField->phpValue($a) == 0);

assert($floatField->phpValue($b) == 0); // zero whether 't' or 'f', sadly
assert($floatField->phpValue($i) == 10.0);
assert($floatField->phpValue($f) == 11.11);
assert($floatField->phpValue($s) == 0.0);
assert($floatField->phpValue($a) == 0.0);

assert($stringField->phpValue($b) == 'f');
assert($stringField->phpValue($i) == '10');
assert($stringField->phpValue($f) == '11.11');
assert($stringField->phpValue($s) == $s);
assert($stringField->phpValue($a) == $a);

$arr = $arrayField->phpValue($b);
assert(count($arr) == 1);
assert($arr[0] == 'f');

$arr = $arrayField->phpValue($i);
assert(count($arr) == 1);
assert($arr[0] == '10');

$arr = $arrayField->phpValue($f);
assert(count($arr) == 1);
assert($arr[0] == '11.11');

$arr = $arrayField->phpValue($s);
assert(count($arr) == 1);
assert($arr[0] == $s);

$arr = $arrayField->phpValue($a);
assert(count($arr) == 4);
assert($arr[0] == 'a');
assert($arr[1] == 'b');
assert($arr[2] == 'c');
assert($arr[3] == 'd');

print "test HTML encoding\n";
$htmlString = '<R&B>';
assert($stringField->dbValue($htmlString) == "'&lt;R&amp;B&gt;'");

print "test bad field reference (fatal error should follow)\n";
$op = lt("fooby", 17);
print $op->toString($user);

?>
