#ifndef _APPLTABLE_H
#define _APPLTABLE_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const ApplTable_hxx_Version =
    "$Id: ApplTable.hxx,v 1.14 2002/09/06 09:00:42 bko Exp $";

#include "global.h"
#include <string>
#include <vector>
#include "SnmpCommon.h"
#include <sys/time.h>
#include <unistd.h>
#include "VMutex.h"

// there is a duplicate of operStatusT in the TrapAgent.hxx
// file.  It is there for use in c files.

class ApplInfo
{
    public :
        typedef enum {
            up = 1,
            down,
            halted,
            congested,
            restarting,
            quiescing
    } operStatusT;

        int index;
        string name;
        int stackPort;
        enum ServerType type;
        int applPort;
        operStatusT status;
        struct timeval startTime;
        struct timeval lastUpdateTime;

        ApplInfo()
        {}
        ;
        ~ApplInfo()
        {}


        bool operator==(const ApplInfo& x)
        {
            return index == x.index;
        }

};

/**
 * this class keeps the information on applications (application table) that 
 * is needed to implement the networkApplicationMib
 */

class ApplTable
{
    public :
        ///
        ApplTable();
        ///
        ~ApplTable();
        ///
        /// not yet implemented
        voReturnStatus deleteAppl(enum ServerType type, int port, string name );
        ///
        /// not yet implimented
        voReturnStatus deleteAppl(int index);
        /**
         * this function will process register messages and update the application table
         * this table is includes the application and stack names, snmpd comm port of stack, 
         * snmpd comm port of application, time of startup
         * @param type the type of server (rs/ms...)
         * @param port the port the application/stack talks to the snmpd (via agentApi)
         * @param name the name of the application or application running above the stack
         * returns voReturnStatus
         */
        voReturnStatus addOrUpdateAppl(enum ServerType type, int port, string name );

        /**
         * this function gets the information on a given application given the index in the NetworkApplicationTable
         * @param index index of the application based on the NetworkApplicationTable mib
         * @returns structure describing the application
         */
        ApplInfo getAppl(int index);

        /**
         * this function gets the information on a given application given the server information
         * @param type type of server (ms/rs......)
         * @param port port the server or stack communicates to the snmpd via AgentApi
         * @param name name of the application
         * @returns structure describing the application
         */
        ApplInfo getAppl(enum ServerType type, int port, string name );

        /**
         * this function returns the port the stack (sip/mgcp...) communicates to
         * the snmpd via agentapi.  
         * @param index index of the application based on the NetworkApplicationTable mib
         * @returns the port
         */
        int getStackPort(int index);
        /**
         * this function returns the port the application (fs/rs...) communicates to
         * the snmpd via agentapi.  
         * @param index index of the application based on the NetworkApplicationTable mib
         * @returns the port
         */
        int getApplPort(int index);
        ///
        int getNumApps()
        {
            return theTable.size();
        }

        /**
         * This function searchs the application table for the index to the next application 
         * within the appl table.
         * @param index index we will return the following index of
         * @param type type of server to look for.  default = unknown (ie. all)
         * @return index  or 0 if there is none -1 on error
         */
        int getNextIndex(int index, enum ServerType type );

    private :
        typedef vector < ApplInfo > Table;
        typedef vector < ApplInfo > ::iterator TableIter;
        typedef vector < ApplInfo > ::reverse_iterator RevTableIter;

        typedef enum {
            portClash,
            nameClash,
            partialExists,
            completeExists,
            newOne,
            err
    } RecordStatus;

        mutable Table theTable;
        TableIter apI;
        RevTableIter apRI;

        /**
         * checkes to see if the type is an application or stack
         * @param type server type
         * @returns bool false if application true if stack
         */
        bool isStackType(enum ServerType type);


        /**
         * takes as input the information from a register message and compaires it to the
         * application table and returns a status based on the result.  the result types are
         * portClash, port already exists with another name. error condition. 
         *       probably something came up, then went down, then another came up and grabbed the port
         * nameClash, name already exists with another port. error condition. The same application has probable
         *       come up, gone down, then come up on another port.
         * partialExists, eather the port or application has registered, but not both
         * completeExists, both the port and application has registered
         * newOne, new record.
         * err, an internal error has occured
         *
         * @param type type of server (ms/rs......)
         * @param port port the server or stack communicates to the snmpd via AgentApi
         * @param name name of the application
         * @returns the record status
         */
        ApplTable::RecordStatus applClash(enum ServerType type, int port, string name );
        VMutex tableLock;

};

#endif /*  _APPLTABLE_H  */
