
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLInterpreter_cxx_Version =
    "$Id: CPLInterpreter.cxx,v 1.32 2002/12/12 01:19:37 chok Exp $";

#include "CPLInterpreter.hxx"
#include <iostream.h>
#include "cpLog.h"
#include <string>
#include "CallProcessingLanguage.hxx"
#include "CPLOpStart.hxx"
#include "CPLOpFwdCall.hxx"
#include "CPLOpProxy.hxx"
#include "CPLOpRedirect.hxx"
#include "CPLOpVmRedirect.hxx"
#include "CPLOpLocation.hxx"
#include "CPLOpRemoveLocation.hxx"
#include "Operator.hxx"
#include "State.hxx"
#include "CPLOpNull.hxx"
#include "CPLOpEnd.hxx"
#include "CPLOpNonAnswer.hxx"
#include "CPLOpAddressSwitch.hxx"
#include "CPLOpAddress.hxx"
#include "CPLOpReject.hxx"
#include "CPLOpStringSwitch.hxx"
#include "CPLOpString.hxx"
#include "CPLOpTimeSwitch.hxx"
#include "CPLOpTime.hxx"
#include "CPLOpTimeZone.hxx"
#include "CPLOpStandard.hxx"
#include "CPLOpDaylight.hxx"
#include "CPLOpLookup.hxx"
#include "CPLOpAckLookup.hxx"
#include "CPLOpPriority.hxx"
#include "CPLOpPrioritySwitch.hxx"
#include "CPLOpMail.hxx"
#include "MailThread.hxx"
#include "MailThreadMgr.hxx"
#include "CPLOpLog.hxx"
#include "CPLOpDfltAction.hxx"
#include "CPLOpFork.hxx"
#include "CPLOpTransition.hxx"
#include "CPLOpBusy.hxx"
#include "CPLOpNoAnswer.hxx"
#include "CPLOpFailure.hxx"

using namespace Vocal;



/*********SINGLETON IMPLEMENTATION***********/
CPLInterpreter* CPLInterpreter::myInstance = NULL;
/********************************************/


///
CPLInterpreter::CPLInterpreter()
{
    myCplBuildStatus = CPLBuildStatus::Instance();
    myStartState = new CPLStartStateHolder;
}  //  CPLInterpreter


///
CPLInterpreter*
CPLInterpreter::Instance()
{
    if ( myInstance == 0 )
    {
        myInstance = new CPLInterpreter;
    }  //if

    return myInstance;

} // Instance


///
Sptr < CPLStartStateHolder >
CPLInterpreter::createFeatureFromCPL( xmlNodePtr rootPtr )
{
    if ( traverse( rootPtr ) == false)
        return 0;

    Sptr < CPLStartStateHolder > retValue = new CPLStartStateHolder;
    *retValue = *myStartState;

    //Free-up member variables to avoid the Smart Pointers
    //from staying attached when the objects become unused.
    myStartState->clearAll();
    myEndState = 0;
    myEndOp = 0;

    //If no one used general proxyOp & State, then we must remove the
    //cyclical dependencies so the Smart Pointers can
    //delete the objects.
    if ( (*myProxyOp.getCount() == static_cast<int>(2) ) && (*myProxyState.getCount() == static_cast<int>(2) ) )
       myProxyState->clearAllOperators();
    myProxyOp = 0;
    myProxyState = 0;

    return retValue;
} // process


///
int
CPLInterpreter::nodeToI(const char* str,
                        const char* strTable[],
                        const int tableSize)
{
    int i;
    for (i = 0; i < tableSize; i++)
    {
        if (strcmp (str, strTable [i]) == 0)
        {
            break;
        }
    }
    return i;
}  //nodeToI


///
bool
CPLInterpreter::nodeStart( int nodeFound, xmlNodePtr ptr )
{
    int returnValue = true;

    switch (nodeFound)
    {
        case CPLNodeCpl:
        returnValue = processCpl( ptr );
        break;
        case CPLNodeLog:
        returnValue = processLog( ptr );
        break;
        case CPLNodeSub:
        returnValue = processSub( ptr );
        break;
        case CPLNodeBusy:
        returnValue = processBusy( ptr );
        break;
        case CPLNodeMail:
        returnValue = processMail( ptr );
        break;
        case CPLNodeTime:
        returnValue = processTime( ptr );
        break;
        case CPLNodeProxy:
        returnValue = processProxy( ptr );
        break;
        case CPLNodeLookup:
        returnValue = processLookup( ptr );
        break;
        case CPLNodeReject:
        returnValue = processReject( ptr );
        break;
        case CPLNodeString:
        returnValue = processString( ptr );
        break;
        case CPLNodeAddress:
        returnValue = processAddress( ptr );
        break;
        case CPLNodeDefault:
        returnValue = processDefault( ptr );
        break;
        case CPLNodeFailure:
        returnValue = processFailure( ptr );
        break;
        case CPLNodeSuccess:
        returnValue = processSuccess( ptr );
        break;
        case CPLNodeIncoming:
        returnValue = processIncoming( ptr );
        break;
        case CPLNodeLocation:
        returnValue = processLocation( ptr );
        break;
        case CPLNodeNoanswer:
        returnValue = processNoAnswer( ptr );
        break;
        case CPLNodeNotfound:
        returnValue = processNotFound( ptr );
        break;
        case CPLNodeOutgoing:
        returnValue = processOutgoing( ptr );
        break;
        case CPLNodeDaylight:
        returnValue = processDaylight( ptr );
        break;
        case CPLNodePriority:
        returnValue = processPriority( ptr );
        break;
        case CPLNodeRedirect:
        returnValue = processRedirect( ptr );
        break;
        case CPLNodeVmRedirect:
        returnValue = processVmRedirect( ptr );
        break;
        case CPLNodeStandard:
        returnValue = processStandard( ptr );
        break;
        case CPLNodeTimezone:
        returnValue = processTimeZone( ptr );
        break;
        case CPLNodeOtherwise:
        returnValue = processOtherwise( ptr );
        break;
        case CPLNodeSubaction:
        returnValue = processSubAction( ptr );
        break;
        case CPLNodeNotpresent:
        returnValue = processNotPresent( ptr );
        break;
        case CPLNodeRedirection:
        returnValue = processRedirection( ptr );
        break;
        case CPLNodeTimeswitch :
        returnValue = processTimeSwitch( ptr );
        break;
        case CPLNodeStringswitch :
        returnValue = processStringSwitch( ptr );
        break;
        case CPLNodeComments:
        //Unnecessary to process comments
        return true;
        break;
        case CPLNodeAddressswitch :
        returnValue = processAddressSwitch( ptr );
        break;
        case CPLNodePriorityswitch :
        returnValue = processPrioritySwitch( ptr );
        break;
        case CPLNodeRemoveLocation:
        returnValue = processRemoveLocation( ptr );
        break;
        default:
        cpLog( LOG_ERR, "Invalid CPL tag or Processing Instruction <%s>.",
               (char*)ptr->name);
        returnValue = false;
        break;
    } /*switch*/


    return returnValue;

}  //buildStateMachine


///
bool
CPLInterpreter::nodeExit( int nodeFound, xmlNodePtr ptr )
{
    return ( myCplBuildStatus->nodeExit( nodeFound ) );
} // nodeExit


///
void
CPLInterpreter::processNodeAttributes( Sptr < CPLOperator > node,
                                       xmlNodePtr ptr,
                                       const char* attrTable[],
                                       const unsigned int tableSize )
{
    xmlChar* attrValue;

    for (unsigned int i = 0; i < tableSize; i++)
    {
        attrValue = xmlGetProp( ptr, (const xmlChar*) attrTable[i] );
        if ( attrValue != NULL)
            node->setAttributes( (const char*) attrTable[i],
                                 (const char*) attrValue );
    } /*for*/

}  //processNodeAttributes



///
bool
CPLInterpreter::traverse( xmlNodePtr ptr )
{
    if (ptr == NULL) return true;

    int nodeFound;
    nodeFound = nodeToI( (const char*)ptr->name, CPLNodeStr,
                         CPLNodeEndofNodes );

    if ( nodeStart( nodeFound, ptr ) == false)
        return false;

    if ( traverse( ptr->children ) == false )
        return false;
    if ( traverse( ptr->next ) == false )
        return false;

    if (nodeExit( nodeFound, ptr ) == false)
        return false;

    return true;
}  // traverse


///
bool
CPLInterpreter::searchNodes( xmlNodePtr ptr, const char* nodeName )
{
    while ( ptr != 0 )
    {
        if (strcmp( (const char*)ptr->name, nodeName ) == 0 )
            return true;
        ptr = ptr->next;
    } /*while*/

    return false;
}  //searchNodes


///
bool
CPLInterpreter::processAddressSwitch( xmlNodePtr ptr )
{
    //Create new operator for state machine
    Sptr < CPLOpAddressSwitch > addrSwitchNode = new CPLOpAddressSwitch;
    if (addrSwitchNode == 0) return false;

    //Find attributes
    processNodeAttributes( addrSwitchNode, ptr,
                           AddrSwitchAttrTable,
                           AddrSwitchAttrTableSize );


    //Add these default operators to the CPLOpAddressSwitch operator.
    //CPLOpAddressSwitch operator must add its address operators ahead
    //of these 2 operators.  A simple push_back cannot be used.
    addrSwitchNode->addOperator( myProxyOp );
    addrSwitchNode->addOperator( myEndOp );

    //Add Address-Switch operator to current state machine
    return (myCplBuildStatus->nodeEntry( addrSwitchNode, CPLNodeAddressswitch ));

}  //processAddressSwitch


///
bool
CPLInterpreter::processAddress( xmlNodePtr ptr )
{
    //Create new operator for state machine
    Sptr < CPLOpAddress > addressNode = new CPLOpAddress;
    if (addressNode == 0) return false;

    //Find attributes
    processNodeAttributes( addressNode, ptr,
                           AddressAttrTable,
                           AddressAttrTableSize );

    //Add Address-Switch operator to current state machine
    bool result = myCplBuildStatus->nodeEntry( addressNode, CPLNodeAddress );
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;

}  //processAddress


///
bool
CPLInterpreter::processCpl( xmlNodePtr ptr )
{
    //Generate other states and operators that will be needed later.
    myEndState = new CPLState( "End" );
    myProxyState = new CPLState( "CPLStateProxy" );

    myProxyOp = new CPLOpProxy;
    myProxyOp->setState( myProxyState );

    myEndOp = new CPLOpEnd();
    myEndOp->setState( myEndState );

    myProxyState->addOperator( myProxyOp );
    myProxyState->addOperator( myEndOp );

    return true;
}  // CPLInterpreter::processCpl


///
bool
CPLInterpreter::processIncoming( xmlNodePtr ptr )
{
    Sptr < CPLOpStart > startOp = new CPLOpStart;
    if ( startOp == 0 )
    {
        cpLog( LOG_ERR, "Failed to allocate CPLOpStart." );
        return false;
    }

    Sptr < CPLState > firstState = new CPLState("CPLStateStart");
    myStartState->setIncoming( firstState );

    if ( firstState == 0 )
    {
        cpLog( LOG_ERR, "Failed to allocate CPLStateStart." );
        return false;
    }

    firstState->addOperator( startOp );

    if (myCplBuildStatus->nodeEntry( startOp, CPLNodeIncoming ) == false )
    {
        firstState = 0;
        myStartState->setIncoming( 0 );
        return false;
    }

    if (ptr->children == NULL)
        return processDefaultAction( ptr );
    else
        return true;

}  //processIncoming


///
bool
CPLInterpreter::processOutgoing( xmlNodePtr ptr )
{
    Sptr < CPLOpStart > startOp = new CPLOpStart;
    if ( startOp == 0 )
    {
        cpLog( LOG_ERR, "Failed to allocate CPLOpStart." );
        return false;
    }

    Sptr < CPLState > firstState = new CPLState("CPLStateStart");
    myStartState->setOutgoing( firstState );
    if ( firstState == 0 )
    {
        cpLog( LOG_ERR, "Failed to allocate CPLStateStart." );
        return false;
    }

    firstState->addOperator( startOp );

    if (myCplBuildStatus->nodeEntry( startOp, CPLNodeOutgoing ) == false )
    {
        firstState = 0;
        myStartState->setOutgoing( 0 );
        return false;
    }

    if (ptr->children == NULL)
        return processDefaultAction( ptr );
    else
        return true;

}  //processOutgoing


///
bool
CPLInterpreter::processSubAction( xmlNodePtr ptr)
{
    xmlChar* attrValue;

    attrValue = xmlGetProp( ptr, (const xmlChar*)"id");
    if (attrValue == 0)
        return false;

    //This operator is used to hold the SubAction ID for
    //CPLBuildStatus to use.  This operator is never
    //included in the state machine because when a <sub> is encountered,
    //the operator pointed to by CPLOpNull is added to the state
    //machine (CPLOpNull is skipped). 
    Sptr < CPLOperator > tempOperator = new CPLOpNull( (char*)attrValue );

    bool result;
    result = myCplBuildStatus->nodeEntry( tempOperator, CPLNodeSubaction );

    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;

} //processSubAction


///
bool
CPLInterpreter::processSub( xmlNodePtr ptr )
{
    xmlChar* attrValue;

    attrValue = xmlGetProp( ptr, (const xmlChar*)"ref");
    if (attrValue == 0)
        return false;

    string id( (char*) attrValue );

    Sptr < CPLOperator > subActionOp;

    subActionOp = myCplBuildStatus->getSubAction( id );
    if (subActionOp == 0)
        return false;

    return (myCplBuildStatus->nodeEntry( subActionOp, CPLNodeSub ) );

}  // CPLInterpreter::processSub


///
bool
CPLInterpreter::processLog( xmlNodePtr ptr )
{
    //Create new operator for state machine
    Sptr < CPLOpLog > aNode = new CPLOpLog;
    if (aNode == 0) return false;

    //Find attributes
    processNodeAttributes( aNode, ptr,
                           LogAttrTable,
                           LogAttrTableSize );

    //Add log operator to current state machine
    bool result = myCplBuildStatus->nodeEntry( aNode, CPLNodeLog );

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;

}  // CPLInterpreter::processLog


///
bool
CPLInterpreter::processTime( xmlNodePtr ptr )
{
    //Create new operator for state machine
    Sptr < CPLOpTime > timeNode = new CPLOpTime;

    if (timeNode == 0) return false;

    //Find attributes
    processNodeAttributes( timeNode, ptr,
                           TimeAttrTable,
                           TimeAttrTableSize );

    //Add String-Switch operator to current state machine
    bool result = myCplBuildStatus->nodeEntry( timeNode, CPLNodeTime );

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;


} // CPLInterpreter::processTime


///
bool
CPLInterpreter::processProxy( xmlNodePtr ptr )
{
    //There are 2 different state machines that need to be created
    //dependent on whether 'ordering' has been set to 'first-only' or
    //to 'sequential/parallel'.
    xmlChar* orderingValue;

    orderingValue = xmlGetProp( ptr, (const xmlChar*)"ordering" );
    if (orderingValue == 0)
        return false;

    if ( strcmp( (const char*)orderingValue, "first-only" ) == 0 )
        return processProxyFirstOnly( ptr );
    else
        return processProxyFork( ptr );

} //processProxy

///
bool
CPLInterpreter::processProxyFirstOnly( xmlNodePtr ptr )
{
    cpLog( LOG_DEBUG, "Add OpFwdCall" );
    //Create operator to send the call to the next hop
    //Create state to wait for a response from the call attempt
    Sptr < CPLOpFwdCall > proxyNode = new CPLOpFwdCall();
    if ( proxyNode == 0 )
        return false;

    processNodeAttributes( proxyNode, ptr,
                           ProxyAttributesTable,
                           ProxyAttributesTableSize );

    //It is unnecessary to set a timer if the <noanswer> output
    //is not used.
    if ( (searchNodes( ptr->children, "noanswer" ) == false) &&
         (searchNodes( ptr->children, "default" ) == false) )
    {
        proxyNode->resetTimeout();
    } /*if*/

    //Add an error operator so CPLOpFwdCall can recover if proxy
    //cannot be done.
    proxyNode->setErrorHandlingOp( myEndOp );

    //If <proxy> was defined with outputs (busy,failure,noanswer), then
    //an additional operator must be added to process these outputs.
    if (ptr->children != NULL)
    {
        Sptr < CPLState > waitState2 = new CPLState("CPLStateAwaitAnswer");
        if ( waitState2 == 0 )
            return false;
        proxyNode->setState( waitState2 );

        //Append OpFwdCall & StateAwaitAnswer
        if (myCplBuildStatus->nodeEntry( proxyNode, CPLNodeUndefined ) == false )
            return false;

        Sptr < CPLOpNonAnswer > processNonAnswer = new CPLOpNonAnswer;

        //Add default operators if Busy | NoAnswer | Failure aren't
        //defined.
        processNonAnswer->addOperator( myProxyOp );
        processNonAnswer->addOperator( myEndOp );

        //Add the default handling operators to the state objects.
        //ProxyOp must be added before EndOp, so it can execute first.
        Sptr < CPLOpProxy > proxyOperator = new CPLOpProxy;
        proxyOperator->setState( waitState2 );
        waitState2->addOperator( proxyOperator );
        waitState2->addOperator( myEndOp );

        //CPLNodeProxy forces the CPLBuildStatus to keep track of
        //the CPLOpNonAnswer operator because it still needs output operators.
        return (myCplBuildStatus->nodeEntry( processNonAnswer, CPLNodeProxy));
    } /*if*/
    else
    {
        proxyNode->setState( myProxyState );
        return (myCplBuildStatus->nodeEntry( proxyNode, CPLNodeProxy));
    } /*else*/

} //processProxyFirstOnly


///
bool
CPLInterpreter::processProxyFork( xmlNodePtr ptr )
{

    // creat operator to make transition to the new state
    Sptr < CPLOpTransition > anOp = new CPLOpTransition;
    if (anOp == 0)
        return false;

    // creat a new state, to wait while proxying is being done...
    Sptr < CPLState > aState = new CPLState("CPLStateProxying");
    if (aState == 0)
        return false;

    // set this as the next state of transition operator...
    anOp->setState(aState);

    // Create the proxyTag operator
    Sptr < CPLOpFork > proxyTag = new CPLOpFork();
    if ( proxyTag == 0 )
        return false;

    // process the attributes of proxy operator...
    processNodeAttributes( proxyTag, ptr,
                           ProxyAttributesTable,
                           ProxyAttributesTableSize );

    // Creat an actual proxy...
    Sptr < CPLOpProxy > aProxy = new CPLOpProxy;

    // set the proxy state as the next state of proxy...
    aProxy->setState(aState);

    // add the proxy to proxyTag...
    proxyTag->setNextOperator(aProxy, CPLNodeUndefined);

    // add the proxy tag as the first operator of proxy state...
    aState->addOperator(proxyTag);

    // set endOp as the last operator of this new state...
    aState->addOperator(myEndOp);

    // make an entry of transition operator in the builder...
    if (myCplBuildStatus->nodeEntry(anOp, CPLNodeUndefined) == false)
        return false;

    // make entry of the proxy tag in the builder...
    if (myCplBuildStatus->nodeEntry(proxyTag, CPLNodeProxy) == false)
        return false;

    return true;
} //processProxyFork


///
bool
CPLInterpreter::processLookup( xmlNodePtr ptr )
{
    cpLog( LOG_DEBUG, "processLookup" );

    //Create the node that holds the success, failure, and notfound
    //operators.
    Sptr < CPLOpAckLookup > ackLookupNode = new CPLOpAckLookup();
    if ( ackLookupNode == 0 )
        return false;

    processNodeAttributes( ackLookupNode,
                           ptr,
                           LookupAttributesTable,
                           LookupAttrTableSize );

    //Create the node that initiates the lookup.
    Sptr < CPLOpLookup > lookupNode = new CPLOpLookup;
    if ( lookupNode == 0 )
        return false;

    processNodeAttributes( lookupNode,
                           ptr,
                           LookupAttributesTable,
                           LookupAttrTableSize );

    //Create the state that waits for a response after sending the
    //lookup.
    Sptr < CPLState > waitState = new CPLState("CPLStateAwaitLookup");
    if ( waitState == 0 )
        return false;

    lookupNode->setState( waitState );

    //Add the default handling operators to the state object.
    //ProxyOp must be added before EndOp, so it can execute first.
    Sptr < CPLOpProxy > proxyOperator = new CPLOpProxy;
    proxyOperator->setState( waitState );
    waitState->addOperator( myEndOp );
    waitState->addOperator( proxyOperator );

    //Append lookupNode and waitState to state machine
    if (myCplBuildStatus->nodeEntry( lookupNode, CPLNodeUndefined ) == false )
        return false;

    //Append ackLookupNode to state machine
    return ( myCplBuildStatus->nodeEntry( ackLookupNode, CPLNodeLookup ) );

}  //processLookup


///
bool
CPLInterpreter::processMail( xmlNodePtr ptr )
{
    //Create new operator for state machine
    Sptr < CPLOpMail > aNode = new CPLOpMail;
    if (aNode == 0) return false;

    //start the mail thread...
    MailThreadMgr::thread();

    //Find attributes
    processNodeAttributes( aNode, ptr,
                           MailAttrTable,
                           MailAttrTableSize );

    //Add mail operator to current state machine
    bool result = myCplBuildStatus->nodeEntry( aNode, CPLNodeMail );

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;


} //processMail


///
bool
CPLInterpreter::processString( xmlNodePtr ptr )
{
    //Create new operator for state machine
    Sptr < CPLOpString > stringNode = new CPLOpString;
    if (stringNode == 0) return false;

    //Find attributes
    processNodeAttributes( stringNode, ptr,
                           StringAttrTable,
                           StringAttrTableSize );

    //Add String operator to current state machine
    bool result = myCplBuildStatus->nodeEntry( stringNode, CPLNodeString );

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;


}  //processString


///
bool
CPLInterpreter::processPriority( xmlNodePtr ptr )
{
    //Create new operator for state machine
    Sptr < CPLOpPriority > aNode = new CPLOpPriority;
    if (aNode == 0) return false;

    //Find attributes
    processNodeAttributes( aNode, ptr,
                           PriorityAttrTable,
                           PriorityAttrTableSize );

    //Add Priority operator to current state machine
    bool result = myCplBuildStatus->nodeEntry( aNode, CPLNodePriority );

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;


}  //processPriority


///
bool
CPLInterpreter::processDaylight( xmlNodePtr ptr )
{
    //Create new operator for state machine
    Sptr < CPLOpDaylight > aNode = new CPLOpDaylight;
    if (aNode == 0) return false;

    //Find attributes
    processNodeAttributes( aNode, ptr,
                           DaylightAttrTable,
                           DaylightAttrTableSize );

    //Add daylight operator to current state machine
    return (myCplBuildStatus->nodeEntry( aNode, CPLNodeDaylight ));

}  //processDaylight


///
bool
CPLInterpreter::processStandard( xmlNodePtr ptr )
{
    //Create new operator for state machine
    Sptr < CPLOpStandard > aNode = new CPLOpStandard;
    if (aNode == 0) return false;

    //Find attributes
    processNodeAttributes( aNode, ptr,
                           StandardAttrTable,
                           StandardAttrTableSize );

    //Add Standard operator to current state machine
    return (myCplBuildStatus->nodeEntry( aNode, CPLNodeStandard ));

}  //processStandard


///
bool
CPLInterpreter::processBusy( xmlNodePtr ptr )
{
    //There are 2 different state machines that need to be created
    //dependent on whether the parent Node, proxy, has 'ordering' set to
    //'first-only' or  'sequential/parallel'.
    xmlChar *orderingValue;
    orderingValue = xmlGetProp( ptr->parent, (const xmlChar*)"ordering" );
    if (orderingValue == 0)
    {
        cpLog(LOG_ERR, "Could not find 'ordering' in <%s>.",
              (const char*) ptr->parent->name );
        return false;
    } /*if*/

    bool result;

    if ( strcmp( (const char*)orderingValue, "first-only" ) == 0 )
    {
        result = myCplBuildStatus->nodeEntry( 0, CPLNodeBusy);
    }  /*if*/
    else
    {
        //Create new operator for state machine
        Sptr < CPLOpBusy > aNode = new CPLOpBusy;
        if (aNode == 0) return false;

        //Add Standard operator to current state machine
        result = myCplBuildStatus->nodeEntry( aNode, CPLNodeBusyExt );

    } /*else*/

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;


}  //CPLInterpreter::processBusy


///
bool
CPLInterpreter::processFailure( xmlNodePtr ptr )
{
    //There are 2 different state machines that need to be created
    //dependent on whether the parent Node, proxy, has 'ordering' set to
    //'first-only' or  'sequential/parallel'.
    xmlChar *orderingValue;
    orderingValue = xmlGetProp( ptr->parent, (const xmlChar*)"ordering" );
    if (orderingValue == 0)
    {
        cpLog(LOG_ERR, "Could not find 'ordering' in <%s>.",
              (const char*) ptr->parent->name );
        return false;
    } /*if*/


    bool result;

    if ( strcmp( (const char*)orderingValue, "first-only" ) == 0 )
    {
        result = myCplBuildStatus->nodeEntry( 0, CPLNodeFailure );
    } /*if*/
    else
    {
        //Create new operator for state machine
        Sptr < CPLOpFailure > aNode = new CPLOpFailure;
        if (aNode == 0) return false;

        //Add Standard operator to current state machine
        result = myCplBuildStatus->nodeEntry( aNode, CPLNodeFailureExt );
    } /*else*/

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;


}  // processFailure


///
bool
CPLInterpreter::processSuccess( xmlNodePtr ptr )
{
    bool result = myCplBuildStatus->nodeEntry( 0, CPLNodeSuccess);

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;

} //processSuccess


///
bool
CPLInterpreter::processDefault( xmlNodePtr ptr )
{
    //There are 2 different state machines that need to be created
    //dependent on whether the parent Node, proxy, has 'ordering' set to
    //'first-only' or  'sequential/parallel'.
    xmlChar *orderingValue;
    orderingValue = xmlGetProp( ptr->parent, (const xmlChar*)"ordering" );
    if (orderingValue == 0)
    {
        cpLog(LOG_ERR, "Could not find 'ordering' in <%s>.",
              (const char*) ptr->parent->name );
        return false;
    } /*if*/

    bool result;

    if ( strcmp( (const char*)orderingValue, "first-only" ) == 0 )
    {
        result = myCplBuildStatus->nodeEntry( 0, CPLNodeDefault);
    }  /*if*/
#if 0   //UNCOMMENT WHEN FORKING IS READY 
    else
    {
        //Create new operator for state machine
        Sptr < CPLOpProxyDefault > aNode = new CPLOpProxyDefault;
        if (aNode == 0) return false;

        //Add Standard operator to current state machine
        result = cplBuildStatus->nodeEntry( aNode, CPLNodeDefaultExt );

    } /*else*/
#endif 

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;
} //processDefault


///
bool
CPLInterpreter::processRedirection( xmlNodePtr ptr )
{
    //There are 2 different state machines that need to be created
    //dependent on whether the parent Node, proxy, has 'ordering' set to
    //'first-only' or  'sequential/parallel'.
    xmlChar *orderingValue;
    orderingValue = xmlGetProp( ptr->parent, (const xmlChar*)"ordering" );
    if (orderingValue == 0)
    {
        cpLog(LOG_ERR, "Could not find 'ordering' in <%s>.",
              (const char*) ptr->parent->name );
        return false;
    } /*if*/

    bool result;

    if ( strcmp( (const char*)orderingValue, "first-only" ) == 0 )
    {
        result = myCplBuildStatus->nodeEntry( 0, CPLNodeRedirection);
    }  /*if*/
#if 0     //UNCOMMENT WHEN FORKING IS READY 
    else
    {
        //Create new operator for state machine
        Sptr < CPLOpProxyRedirection > aNode = new CPLOpProxyRedirection;
        if (aNode == 0) return false;

        //Add Standard operator to current state machine
        result = cplBuildStatus->nodeEntry( aNode, CPLNodeRedirectionExt );

    } /*else*/
#endif 

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;

} //processRedirection


///
bool
CPLInterpreter::processLocation( xmlNodePtr ptr )
{
    //Create the Location class
    Sptr < CPLOpLocation > locationNode = new CPLOpLocation;
    if ( locationNode == 0 )
        return false;

    processNodeAttributes( locationNode, ptr,
                           LocationAttributesTable,
                           LocationAttributesTableSize );

    //Add Location operator to existing state machine
    bool result = myCplBuildStatus->nodeEntry( locationNode, CPLNodeLocation );

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;

}  // processLocation


///
bool
CPLInterpreter::processNoAnswer( xmlNodePtr ptr )
{
    //There are 2 different state machines that need to be created
    //dependent on whether the parent Node, proxy, has 'ordering' set to
    //'first-only' or  'sequential/parallel'.
    xmlChar *orderingValue;
    orderingValue = xmlGetProp( ptr->parent, (const xmlChar*)"ordering" );
    if (orderingValue == 0)
    {
        cpLog(LOG_ERR, "Could not find 'ordering' in <%s>.",
              (const char*) ptr->parent->name );
        return false;
    } /*if*/

    bool result;
    if ( strcmp( (const char*)orderingValue, "first-only" ) == 0 )
    {
        result = myCplBuildStatus->nodeEntry( 0, CPLNodeNoanswer );
    } /*if*/
    else
    {
        //Create new operator for state machine
        Sptr < CPLOpNoAnswer > aNode = new CPLOpNoAnswer;
        if (aNode == 0) return false;

        //Add Standard operator to current state machine
        result = myCplBuildStatus->nodeEntry( aNode, CPLNodeNoanswerExt );
    } /*else*/

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;

}  //processNoAnswer


///
bool
CPLInterpreter::processNotFound( xmlNodePtr ptr )
{
    bool result = myCplBuildStatus->nodeEntry( 0, CPLNodeNotfound);

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;

}  //processNotFound


///
bool
CPLInterpreter::processNotPresent( xmlNodePtr ptr )
{
    bool result = myCplBuildStatus->nodeEntry( 0, CPLNodeNotpresent);

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;


}  //processNotPresent


///
bool
CPLInterpreter::processRedirect( xmlNodePtr ptr )
{
    //Create new operator for state machine
    Sptr < CPLOpRedirect > redirectNode = new CPLOpRedirect();
    if (redirectNode == 0) return false;

    //Note: Redirect has no attributes to process

    //Set next state to the 'End' state
    redirectNode->setState( myEndState );

    //Add Redirect operator to current state machine
    return (myCplBuildStatus->nodeEntry( redirectNode, CPLNodeRedirect ));

} //processRedirect


///
bool
CPLInterpreter::processVmRedirect( xmlNodePtr ptr )
{
    // UrlCollector is not used as the script does not contain the
    // URLs to redirect the call to as the voiceMail ports available
    // can not be known in advance

    //Create new operator for state machine
    Sptr < CPLOpVmRedirect > Node = new CPLOpVmRedirect( );
    if (Node == 0) return false;

    //Note: VmRedirect has no attributes to process

    //Set next state to the 'Proxy' state
    Node->setState( myProxyState );

    //Add Redirect operator to current state machine
    return (myCplBuildStatus->nodeEntry( Node, CPLNodeVmRedirect ));

} //processVmRedirect


///
bool
CPLInterpreter::processReject( xmlNodePtr ptr )
{
    //Create new object for the state machine
    Sptr < CPLOpReject > rejectNode = new CPLOpReject;

    rejectNode->setState( myEndState );

    processNodeAttributes( rejectNode, ptr,
                           RejectAttrTable,
                           RejectAttrTableSize);

    //Add the Reject operator to current state machine
    return (myCplBuildStatus->nodeEntry( rejectNode, CPLNodeReject ));
}  // processReject


///
bool
CPLInterpreter::processOtherwise( xmlNodePtr ptr )
{
    //Find Otherwise's parent node
    assert( ptr->parent != NULL);
    xmlChar* parentName = (xmlChar*)((ptr->parent)->name);

    bool result;

    if ( strcmp((const char*)parentName, "address-switch" ) == 0)
    {
        Sptr < CPLOpAddress > addrNode = new CPLOpAddress;
        if (addrNode == 0) return false;

        //Add Address-Switch operator to current state machine
        result = myCplBuildStatus->nodeEntry( addrNode, CPLNodeAddress );

    } /* if address-switch */
    else if ( strcmp((const char*)parentName, "string-switch" ) == 0)
    {
        Sptr < CPLOpString > stringNode = new CPLOpString;
        if (stringNode == 0) return false;

        //Add String-Switch operator to current state machine
        result = myCplBuildStatus->nodeEntry( stringNode, CPLNodeString );
    }  /* if string-switch */
    else if ( strcmp((const char*)parentName, "time-switch" ) == 0)
    {
        Sptr < CPLOpTime > timeNode = new CPLOpTime;
        if (timeNode == 0) return false;

        result = myCplBuildStatus->nodeEntry( timeNode, CPLNodeTime);
    }  /* if time-switch */
    else if ( strcmp((const char*)parentName, "priority-switch" ) == 0)
    {
        Sptr < CPLOpPriority > priorityNode = new CPLOpPriority;
        if (priorityNode == 0) return false;

        result = myCplBuildStatus->nodeEntry( priorityNode, CPLNodePriority );

    }  /* if priority-switch */
    else
    {
        //Unexpected parent Node
        cpLog( LOG_ERR, "Unexpected parent: '%s'", (const char*) parentName );
        return false;
    } /*else*/

    //Add default action if no node follows
    if ( (result != false) && (ptr->children == NULL) )
        return processDefaultAction( ptr );
    else
        return result;

}  //processOtherwise


///
bool
CPLInterpreter::processTimeSwitch( xmlNodePtr ptr )
{
    //Create new operator for state machine
    Sptr < CPLOpTimeSwitch > timeSwitchNode = new CPLOpTimeSwitch;
    if (timeSwitchNode == 0) return false;

    //Not using processNodeAttribute because timezone is tricky.
    xmlChar* attrValue = xmlGetProp(ptr, (xmlChar *)"timezone");
    if (attrValue == 0 || strcmp((char *)attrValue, "local") == 0)
    {
        timeSwitchNode->timeZone(new CPLOpTimeZoneLocal);
    }
    else if (strcmp((char *)attrValue, "utc") == 0)
    {
        timeSwitchNode->timeZone(new CPLOpTimeZoneUTC);
    }
    else
    {
        Sptr < CPLOpTimeZone > timeZone;
        timeZone.dynamicCast(myCplBuildStatus->getTimeZone(string((char *)attrValue)));
        if (timeZone == 0)
        {
            cpLog(LOG_DEBUG, "ERR: illegal timezone \"%s\"...", (char *)attrValue);
            timeZone = new CPLOpTimeZoneLocal;
        }
        timeSwitchNode->timeZone(timeZone);
    }

    //Add these default operators to the CPLOpTimeSwitch operator.
    timeSwitchNode->addOperator( myProxyOp );
    timeSwitchNode->addOperator( myEndOp );

    //Add Time-Switch operator to current state machine
    return (myCplBuildStatus->nodeEntry( timeSwitchNode, CPLNodeTimeswitch ));

}  // processTimeSwitch


///
bool
CPLInterpreter::processStringSwitch( xmlNodePtr ptr )
{
    //Create new operator for state machine
    Sptr < CPLOpStringSwitch > stringSwitchNode = new CPLOpStringSwitch;
    if (stringSwitchNode == 0) return false;

    //Find attributes
    processNodeAttributes( stringSwitchNode, ptr,
                           StringSwitchAttrTable,
                           StringSwitchAttrTableSize );


    //Add these default operators to the CPLOpStringSwitch operator.
    //CPLOpStringSwitch operator must add its "string" operators ahead
    //of these 2 operators.  A simple push_back cannot be used.
    stringSwitchNode->addOperator( myProxyOp );
    stringSwitchNode->addOperator( myEndOp );

    //Add String-Switch operator to current state machine
    return (myCplBuildStatus->nodeEntry(stringSwitchNode, CPLNodeStringswitch ));
} //processStringSwitch


///
bool
CPLInterpreter::processPrioritySwitch( xmlNodePtr ptr )
{
    //Create new operator for state machine
    Sptr < CPLOpPrioritySwitch > prioritySwitchNode = new CPLOpPrioritySwitch;
    if (prioritySwitchNode == 0) return false;

    //Find attributes
    processNodeAttributes( prioritySwitchNode, ptr,
                           PrioritySwitchAttrTable,
                           PrioritySwitchAttrTableSize );


    //Add these default operators to the CPLOpPrioritySwitch operator.
    prioritySwitchNode->addOperator( myProxyOp );
    prioritySwitchNode->addOperator( myEndOp );

    //Add String-Switch operator to current state machine
    return (myCplBuildStatus->nodeEntry(prioritySwitchNode, CPLNodePriorityswitch ));
} //processPrioritySwitch


///
bool
CPLInterpreter::processTimeZone( xmlNodePtr ptr )
{
    //Create new operator for state machine
    Sptr < CPLOpTimeZone > aNode = new CPLOpTimeZone;
    if (aNode == 0) return false;

    //Find attributes
    processNodeAttributes( aNode, ptr,
                           TimeZoneAttrTable,
                           TimeZoneAttrTableSize );

    //Add TimeZone operator to current state machine
    return (myCplBuildStatus->nodeEntry(aNode, CPLNodeTimezone ));
} //processTimeZone


///
bool
CPLInterpreter::processDefaultAction( xmlNodePtr ptr )
{
    cpLog( LOG_DEBUG, "processDefaultAction()" );

    //Find out if this script is an incoming or outgoing script.
    xmlNodePtr tempPtr;
    tempPtr = ptr;

    bool outgoingFound = false;
    while (tempPtr != 0 )
    {
        assert( tempPtr->name != 0 );

        if (strcmp( (const char*)tempPtr->name, "incoming" ) == 0)
            break;

        if (strcmp( (const char*)tempPtr->name, "outgoing" ) == 0)
        {
            outgoingFound = true;
            break;
        } /*if*/

        if ( (strcmp( (const char*)tempPtr->name, "subaction") == 0) ||
                (strcmp((const char*) tempPtr->name, "timezone") == 0) )
            tempPtr = tempPtr->next;
        else
            tempPtr = tempPtr->parent;
    } /* while */

    //If this is an outgoing script, build a different default action
    //state machine than if this was an incoming script.
    if (outgoingFound == true)
        return processDefaultOutgoing( ptr );
    else
        return processDefaultIncoming( ptr );

}  //processDefaultAction

///
bool
CPLInterpreter::processDefaultIncoming( xmlNodePtr ptr )
{
    //Add the default action for an INCOMING feature that will
    //either do the Reject operator or the Redirect operator.
    Sptr < CPLOpDfltAction > defaultNode = new CPLOpDfltAction;
    if ( defaultNode == 0 )
    {
        cpLog( LOG_ERR, "Failed to allocate." );
        return false;
    } /*if*/

    Sptr < CPLOpReject > rejectNode = new CPLOpReject;
    if ( rejectNode == 0 )
    {
        cpLog( LOG_ERR, "Failed to allocate." );
        return false;
    } /*if*/

    //Manually set attributes since a CPL tag was not used.
    rejectNode->setAttributes("reason", "Default Action: No locations defined.");
    rejectNode->setAttributes("status", "notfound" );

    rejectNode->setState( myEndState );
    defaultNode->setRejectOperator( rejectNode );

    Sptr < CPLOpRedirect > redirectNode = new CPLOpRedirect;
    if ( redirectNode == 0 )
    {
        cpLog( LOG_ERR, "Failed to allocate." );
        return false;
    } /*if*/

    redirectNode->setState( myEndState );
    defaultNode->setRedirectOperator( redirectNode );

    return (myCplBuildStatus->nodeEntry(defaultNode, CPLBuildStatus::SimpleOp));
}  //processDefaultIncoming


///
bool
CPLInterpreter::processDefaultOutgoing( xmlNodePtr ptr )
{
    //Create state machine manually to contact redirect server
    //to look-up Request's Url.  Then proxy to that URL.

    //Create Lookup
    Sptr < CPLOpAckLookup > ackLookupNode = new CPLOpAckLookup();
    if ( ackLookupNode == 0 )
        return false;
    ackLookupNode->setAttributes( "clear", "no" );

    Sptr < CPLOpLookup > lookupNode = new CPLOpLookup;
    if ( lookupNode == 0 )
        return false;

    Sptr < CPLState > waitState = new CPLState("CPLStateAwaitLookup");
    if ( waitState == 0 )
        return false;

    lookupNode->setState( waitState );
    lookupNode->setAttributes( "timeout", "2" );
    lookupNode->setAttributes( "source", "registration" );

    Sptr < CPLOpProxy > proxyOperator = new CPLOpProxy;
    proxyOperator->setState( waitState );
    waitState->addOperator( myEndOp );
    waitState->addOperator( proxyOperator );

    bool result;
    result = myCplBuildStatus->nodeEntry( lookupNode, CPLNodeUndefined );
    if (result == false) return false;

    result = myCplBuildStatus->nodeEntry( ackLookupNode, CPLNodeLookup );
    if (result == false) return false;

    //Tell CPLBuildStatus we are about to add to the 'success' output.
    result = myCplBuildStatus->nodeEntry( 0, CPLNodeSuccess );
    if (result == false) return false;

    //Add proxy operator
    Sptr < CPLOpFwdCall > proxyNode = new CPLOpFwdCall;
    if ( proxyNode == 0 )
    {
        cpLog( LOG_DEBUG, "Failed to allocate." );
        return false;
    } /*if*/

    proxyNode->resetTimeout();
    proxyNode->setErrorHandlingOp( myEndOp );

    proxyNode->setState( myProxyState );
    proxyNode->setAttributes( "timeout", "0" );
    proxyNode->setAttributes( "ordering", "first-only" );
    result = myCplBuildStatus->nodeEntry( proxyNode, CPLNodeProxy );
    if ( result == false) return false;

    if ( myCplBuildStatus->nodeExit( CPLNodeProxy ) == false ) return false;

    //Tell CPLBuildStatus we are about to add to the 'notfound' output.
    result = myCplBuildStatus->nodeEntry( 0, CPLNodeNotfound );
    if (result == false) return false;

    //Add reject operator
    Sptr < CPLOpReject > rejectNode = new CPLOpReject;
    if ( rejectNode == 0 )
    {
        cpLog( LOG_DEBUG, "Failed to allocate." );
        return false;
    } /*if*/

    rejectNode->setState( myEndState );
    rejectNode->setAttributes( "status", "reject" );
    rejectNode->setAttributes( "reason", "Default action: Lookup failed." );

    result = myCplBuildStatus->nodeEntry( rejectNode, CPLNodeReject );

    result = myCplBuildStatus->nodeExit( CPLNodeLookup );

    return result;
} //processDefaultOutgoing

bool
CPLInterpreter::processRemoveLocation( xmlNodePtr ptr )
{
    //Create the Location class
    Sptr < CPLOpRemoveLocation > removeLocationNode = new CPLOpRemoveLocation;
    if ( removeLocationNode == 0 )
        return false;

    processNodeAttributes( removeLocationNode, ptr,
                           RemoveLocationAttributesTable,
                           RemoveLocationAttributesTableSize );

    //Add Location operator to existing state machine
    return (myCplBuildStatus->nodeEntry( removeLocationNode,
                                       CPLNodeRemoveLocation ));

}  // processRemoveLocation

//End of file

