#ifndef CPLOpAckLookup_Hxx
#define CPLOpAckLookup_Hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CPLOpAckLookupVersion =
    "$Id$";




#include "CPLOperator.hxx"
#include "StatusMsg.hxx"
#include "SipEvent.hxx"

namespace Vocal
{


/**
 *  Waits and processes messages from Redirect Server after contacting
 *  it in CPLOpLookup. 
 */
class CPLOpAckLookup : public CPLOperator
{
    public:
        /**@name Initialization & Deallocation Functions */
        //@{
        ///Constructor
        CPLOpAckLookup();
        ///Destructor
        ~CPLOpAckLookup();
        //@}

        /**@name State Machine Building Functions */
        //@{
        ///Append an operator to this class
        bool setNextOperator( Sptr < Operator > op, int prevNode );
        ///Initialize the Attributes member variables
        bool setAttributes( const char* attrName, const char* attrValue);
        //@}

        /**@name State Machine Execution Functions */
        //@{
        ///
        const Sptr < State > process( Sptr < SipProxyEvent > event);
        ///
        void clearTimer( Sptr < SipProxyEvent > event );
        ///
        Sptr < State > doOutput( Sptr < SipProxyEvent > event, int lookupResult );
        ///
        int processRedirectionMsg(Sptr < SipEvent > event, Sptr < StatusMsg > statusMsg);
        ///Acknowledge for responses message to redirect server
        void ackAll( Sptr < SipEvent > event, Sptr < StatusMsg > statusMsg );
        /// Obtain the contact list from the contact header
        Sptr<BaseUrl> getRequestUrlFromContact( Sptr < StatusMsg > statusMsg ) const;
        //@}

        /**@name Debugging Functions */
        //@{
        ///Debug success,failure,not found operators
        void doCpLog() const;
        ///cpLogs values of member variables
        void logData() const;
        ///Returns the name of the class
        const char* const name() const;
        //@}

    private:
        /**@name Lookup outputs */
        //@{
        ///
        Sptr < Operator > mySuccessOp;
        ///
        Sptr < Operator > myFailureOp;
        ///
        Sptr < Operator > myNotFoundOp;
        //@}

        ///CPL Attribute
        bool myClearLocations;

        /**@name Constants */
        //@{
        ///Lookup_Success = 0, Lookup_NotFound = 1, Lookup_Failure = 2
        enum lookupResults { Lookup_Success = 0, Lookup_NotFound, Lookup_Failure,
                             Lookup_Undefined = -1 };
        ///
        typedef lookupResults LookupResults;
        //@}
};
 
} // namespace Vocal

#endif

