
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpAddressSwitch_cxx_Version =
    "$Id: CPLOpAddressSwitch.cxx,v 1.16 2002/11/09 02:05:37 derekm Exp $";
#include "CPLOpAddressSwitch.hxx"
#include "CPLOpAddress.hxx"

using namespace Vocal;

///Constructor
CPLOpAddressSwitch::CPLOpAddressSwitch():
    myField( CPLOpAddressSwitch::NotPresentField ),
    mySubfield( CPLOpAddressSwitch::EntireAddress_SubField ),
    myNotPresentOp( NULL )
{
    //Set-up default values
#ifdef MEM_TEST
    cpLog ( LOG_DEBUG, "new CPLOpAddressSwitch[%x]", this );
#endif
}  //c'tor


///Destructor
CPLOpAddressSwitch::~CPLOpAddressSwitch()
{
#ifdef MEM_TEST
    cpLog ( LOG_DEBUG, "~CPLOpAddressSwitch[%x]", this );
#endif
}  //d'tor


///Append an operator to this class
bool
CPLOpAddressSwitch::setNextOperator( Sptr < Operator > op, int prevNode )
{
    if ( prevNode == CPLNodeNotpresent )
    {
        myNotPresentOp = op;
        return true;
    } /*if*/

    //Assumption:  CPLOpProxy & CPLOpEnd are in the list already.
    //Address operators must be added in front of the CPLOpProxy.
    myOpIterator it = myOperators.begin();
    myOpIterator endOfList = myOperators.end();

    while ( it != endOfList )
    {
        if ( strcmp( (*it)->name(), "CPLOpProxy" ) == 0 )
        {
            myOperators.insert( it, op );
            return true;
        } /*if*/
        ++it;
    } //while

    //If we reach here, then CPLOpProxy was not encountered.
    assert( 0 );
    return true;
}  //setNextOperator


///Initialize the Attributes member variables
bool
CPLOpAddressSwitch::setAttributes( const char* attrName, const char* attrValue)
{
    //Save 'field'
    if ( strcmp( attrName, "field" ) == 0 )
    {
        if ( strcmp( attrValue, "origin" ) == 0 )
            myField = CPLOpAddressSwitch::FromField;
        else if ( strcmp( attrValue, "destination" ) == 0 )
            myField = CPLOpAddressSwitch::RequestURLField;
        else if ( strcmp( attrValue, "original-destination" ) == 0 )
            myField = CPLOpAddressSwitch::ToField;
        else  //Unknown value
        {
            if ( attrValue != 0 )
                cpLog( LOG_DEBUG, "Unknown '%s'.", attrValue );
            assert( 0 );
        } /*else*/
        return true;
    } /*if*/

    //Save 'subfield'
    if ( strcmp( attrName, "subfield" ) == 0 )
    {
        if ( strcmp( attrValue, "user" ) == 0 )
            mySubfield = CPLOpAddressSwitch::User_SubField;
        else if ( strcmp( attrValue, "host" ) == 0 )
            mySubfield = CPLOpAddressSwitch::Host_SubField;
        else if ( strcmp( attrValue, "display" ) == 0 )
            mySubfield = CPLOpAddressSwitch::Display_SubField;
        else if ( strcmp( attrValue, "port" ) == 0 )
            mySubfield = CPLOpAddressSwitch::Port_SubField;
        else if ( strcmp( attrValue, "tel" ) == 0 )
            mySubfield = CPLOpAddressSwitch::Tel_SubField;
        else if ( strcmp( attrValue, "address-type" ) == 0 )
            mySubfield = CPLOpAddressSwitch::AddressType_SubField;
        else if ( strcmp( attrValue, "npa-area-code" ) == 0 )
            mySubfield = CPLOpAddressSwitch::AreaCode_SubField;
        else if ( strcmp( attrValue, "npa-prefix" ) == 0 )
            mySubfield = CPLOpAddressSwitch::Prefix_SubField;
        else if ( strcmp( attrValue, "npa-line-number" ) == 0 )
            mySubfield = CPLOpAddressSwitch::LineNumber_SubField;
        else //Unknown value
        {
            if ( attrValue != 0 )
                cpLog( LOG_DEBUG, "Unknown '%s'.", attrValue );
            assert( 0 );
        } /*else*/
        return true;
    } /* if */

    assert( 0 );
    return false;
}  //setAttributes


///
const Sptr < State >
CPLOpAddressSwitch::process( Sptr < SipProxyEvent > event)
{
    Sptr < State > returnValue;
    Data eventField;

    cpLog(LOG_DEBUG, "%s::process()", name() );

    //Try to get destination or origin address from the SIP message
    //If field isn't found, do the not-present operator
    if ( getUrl( event, eventField ) != true )
    {
        if ( myNotPresentOp != 0 )
            return ( myNotPresentOp->process( event ) );
        else  //Put a dummy value here & hope it gets to the Otherwise op
            eventField = "*";
    } /*if*/

    cpLog(LOG_DEBUG, "Switch on subfield(%s)", eventField.logData() );

    //Iterate through all the operators.
    myOpIterator it = myOperators.begin();
    myOpIterator endOfList = myOperators.end();

    while ( it != endOfList )
    {
        //Different handling of different operators
        //The CPLOpAddress operators need to have its compare()
        //called first before we process it.
        if ( strcmp( (*it)->name(), "CPLOpAddress" ) == 0 )
        {
            CPLOpAddress* addrOp = (CPLOpAddress*) it->getPtr();
            if ( addrOp->compare( mySubfield, eventField ) == true )
            {
                cpLog(LOG_DEBUG, "Compare() result = true");
                returnValue = (*it)->process( event );
                if ( returnValue != 0 )
                    return returnValue;
            } /*if*/
            else
                cpLog(LOG_DEBUG, "Compare() result = false");
        } /*if*/
        else
        {
            cpLog(LOG_DEBUG, "Executing non-<address>" );
            returnValue = (*it)->process( event );
            if ( returnValue != 0 )
                return returnValue;
        } /*else*/
        ++it;
    } /*while*/

    return 0;
}  //process


///
bool
CPLOpAddressSwitch::getUrl( Sptr < SipProxyEvent > event, 
                            Data& addressData ) const
{
    Sptr<BaseUrl> baseUrl; 
    Sptr<SipUrl> sipUrl;

    //Get the field
    switch ( myField )
    {
        case ToField:
        {
            baseUrl = getSipMsg(event)->getTo().getUrl();
        } /*case*/
        break;
        case FromField:
        {
            baseUrl = getSipMsg(event)->getFrom().getUrl();
        } /*case*/
        break;
        case RequestURLField:
        {
            baseUrl = getSipCommand( event )->getRequestLine().getUrl();
        } /*case*/
        break;
        default:
        return false;
    } /*switch*/

    sipUrl.dynamicCast( baseUrl );

    cpLog(LOG_DEBUG, "Field(%s)", sipUrl->getNameAddr().logData() );

    //Get the subfield
    switch ( mySubfield )
    {
        case EntireAddress_SubField:
        {
            addressData = sipUrl->getNameAddr();
        } /*case*/
        break;
        case AddressType_SubField:
        {
            //Hardcoding the signalling protocol
            addressData = "sip";
        } /*case*/
        break;
        case User_SubField:
        {
            addressData = sipUrl->getUserValue();
        } /*case*/
        break;
        case Host_SubField:
        {
            addressData = sipUrl->getHost();
        } /*case*/
        break;
        case Port_SubField:
        {
            addressData = sipUrl->getPort();
        } /*case*/
        break;
        case Tel_SubField:
        {
            return getTelephoneNumber( baseUrl, addressData );
        } /* case */
        break;
        case Display_SubField:
        {
            if ( myField == CPLOpAddressSwitch::ToField )
                addressData = getSipMsg(event)->getTo().getDisplayName();
            else if (myField == CPLOpAddressSwitch::FromField )
                addressData = getSipMsg(event)->getFrom().getDisplayName();
            else  //No Display name in 'requestUrl'
                return false;
        } /*case*/
        break;
        case AreaCode_SubField:
        {
            if ( getTelephoneNumber( baseUrl, addressData ) == true )
                return getNPAAreaCode( addressData );
        } /*case*/
        break;
        case Prefix_SubField:
        {
            if ( getTelephoneNumber( baseUrl, addressData ) == true )
                return getNPAPrefix( addressData );
        } /*case*/
        break;
        case LineNumber_SubField:
        {
            if ( getTelephoneNumber( baseUrl, addressData ) == true )
                return getNPALineNumber( addressData );
        } /*case*/
        break;
        default:
        {
            return false;
        } /*default*/
    } /*switch*/

    return true;
}  //getUrl


///
bool
CPLOpAddressSwitch::getTelephoneNumber( const Sptr<BaseUrl> baseUrl, 
                                        Data& phoneNum ) const
{
    //If the signalling type is "tel:" then the Tel information
    //is located at the User area
    if ( baseUrl->getType() == TEL_URL )
    {
        phoneNum = baseUrl->getUserValue();
        return true;
    } /*if*/
    else if ( baseUrl->getType() != SIP_URL )
    {
        cpLog( LOG_ERR, "Cannot handle Url Type (%d)", baseUrl->getType() ); 
        return false;
    }  /*else*/
     
    Sptr<SipUrl> sipUrl;
    sipUrl.dynamicCast( baseUrl );
    
    //Tel subfield is idential to User subfield only
    //if parameter ?user=phone is defined.
    LocalScopeAllocator lo;
    string urlParam(sipUrl->getUrlParam().getData(lo));
    cpLog(LOG_DEBUG, urlParam.c_str() );

    if ( urlParam.find( "user=phone" ) != string::npos )
    {
        phoneNum = sipUrl->getUserValue();
        return true; 
    } /*if*/
    else
    {
        return false;
    } /*else*/

}  //getTelephoneNumber


///
bool
CPLOpAddressSwitch::getNPAAreaCode( Data& url ) const
{
    //Assuming phone numbers in the North America format, retrieve
    //the area code
           
    if ( url.length() != 11 )
    {
        cpLog( LOG_DEBUG, "Expecting 11.  Got '%s'", url.logData() );
        return false;
    } /*if*/

    if ( url[0] != '1' )
    {
        cpLog( LOG_DEBUG, "Expecting 1+Nxx+Nxx+xxxx.  Got '%s'",
               url.logData() );
        return false;
    } /*if*/

    char tempUrl[4];

    //Verify the area code is composed of digits
    if ( (url[1] < '0') || (url[1] > '9') )
        return false;
    else
        tempUrl[0] = url[1];

    if ( (url[2] < '0') || (url[2] > '9') )
        return false;
    else
        tempUrl[1] = url[2];

    if ( (url[3] < '0') || (url[3] > '9') )
        return false;
    else
        tempUrl[2] = url[3];

    tempUrl[3] = 0;
    url = tempUrl;
    return true;
}  //getNPAAreaCode


///
bool
CPLOpAddressSwitch::getNPAPrefix( Data& url ) const
{
    //Assuming phone numbers in the North America format, retrieve
    //the prefix.
           
    int urlLength = url.length();

    if ( ( urlLength != 11 ) && ( urlLength != 8 ) )
    {
        cpLog( LOG_DEBUG, "Wrong size: Got '%s'", url.logData() );
        return false;
    } /*if*/

    if ( urlLength == 11 ) //Verify number fits NPA format
    {
        if ( url[0] != '1' )
        {
            cpLog( LOG_DEBUG, "Expecting 1+Nxx+Nxx+xxxx.  Got '%s'",
                   url.logData() );
            return false;
        } /*if*/
    } /*if*/

    char tempUrl[4];
    int offset;   //Offset from first char into prefix
    if ( urlLength == 11)
        offset = 4;
    else
        offset = 1;

    //Verify the prefix (aka central office code)
    //is composed of digits
    if ( (url[offset] < '0') || (url[offset] > '9') )
        return false;
    else
        tempUrl[0] = url[offset];

    if ( (url[offset + 1] < '0') || (url[offset + 1] > '9') )
        return false;
    else
        tempUrl[1] = url[offset + 1];

    if ( (url[offset + 2] < '0') || (url[offset + 2] > '9') )
        return false;
    else
        tempUrl[2] = url[offset + 2];

    tempUrl[3] = 0;
    url = tempUrl;

    return true;
}  //getNPAPrefix


///
bool
CPLOpAddressSwitch::getNPALineNumber( Data& url ) const
{
    //Assuming phone numbers in the North America format, retrieve
    //the line number.
           
    int urlLength = url.length();

    if ( ( urlLength != 11 ) && ( urlLength != 8) )
    {
        cpLog( LOG_DEBUG, "Wrong size: Got '%s'", url.logData() );
        return false;
    } /*if*/

    if ( urlLength == 11 )  //Check to verify NPA format
    {
        if ( url[0] != '1' )
        {
            cpLog( LOG_DEBUG, "Expecting 1+Nxx+Nxx+xxxx.  Got '%s'",
                   url.logData() );
            return false;
        } /*if*/
    } /*if*/

    char tempUrl[5];
    int offset;   //Offset from first char into prefix
    if ( urlLength == 11)
        offset = 7;
    else
        offset = 4;


    //Verify the line number is composed of digits
    if ( (url[offset] < '0') || (url[offset] > '9') )
        return false;
    else
        tempUrl[0] = url[offset];

    if ( (url[offset + 1] < '0') || (url[offset + 1] > '9') )
        return false;
    else
        tempUrl[1] = url[offset + 1];

    if ( (url[offset + 2] < '0') || (url[offset + 2] > '9') )
        return false;
    else
        tempUrl[2] = url[offset + 2];

    if ( (url[offset + 3] < '0') || (url[offset + 3] > '9') )
        return false;
    else
        tempUrl[3] = url[offset + 3];

    tempUrl[4] = 0;
    url = tempUrl;

    return true;
}  //getNPALineNumber


///Outputs the values of the member variables
void
CPLOpAddressSwitch::logData() const
{
    char fieldBuffer[4];   //Holds 3 chars only
    char subfieldBuffer[4];  //Holds 3 chars only

    switch ( myField )
    {
        case CPLOpAddressSwitch::ToField:
        {
            strcpy( fieldBuffer, "O-D" );
        } /*case*/
        break;
        case CPLOpAddressSwitch::FromField:
        {
            strcpy( fieldBuffer, "Ori" );
        } /*case*/
        break;
        case CPLOpAddressSwitch::RequestURLField:
        {
            strcpy( fieldBuffer, "Des" );
        } /*case*/
        break;
        default:
        {
            strcpy( fieldBuffer, "???" );
        } /*default*/
    } /*switch*/


    switch ( mySubfield )
    {
        case CPLOpAddressSwitch::EntireAddress_SubField:
        {
            strcpy( subfieldBuffer, "All" );
        }  /*case*/
        break;
        case CPLOpAddressSwitch::AddressType_SubField:
        {
            strcpy( subfieldBuffer, "Adr" );
        } /*case*/
        break;
        case CPLOpAddressSwitch::User_SubField:
        {
            strcpy( subfieldBuffer, "Usr" );
        } /*case*/
        break;
        case CPLOpAddressSwitch::Host_SubField:
        {
            strcpy( subfieldBuffer, "Hst" );
        } /*case*/
        break;
        case CPLOpAddressSwitch::Port_SubField:
        {
            strcpy( subfieldBuffer, "Por" );
        } /*case*/
        break;
        case CPLOpAddressSwitch::Tel_SubField:
        {
            strcpy( subfieldBuffer, "Tel" );
        } /*case*/
        break;
        case CPLOpAddressSwitch::Display_SubField:
        {
            strcpy( subfieldBuffer, "Dsp" );
        } /*case*/
        break;
        case CPLOpAddressSwitch::AreaCode_SubField:
        {
            strcpy( subfieldBuffer, "A C" );
        } /*case*/
        break;
        case CPLOpAddressSwitch::Prefix_SubField:
        {
            strcpy( subfieldBuffer, "Pre" );
        } /*case*/
        break;
        case CPLOpAddressSwitch::LineNumber_SubField:
        {
            strcpy( subfieldBuffer, "L #" );
        } /*case*/
        break;
        default:
        {
            strcpy( subfieldBuffer, "???" );
        } /*default*/
    } /*switch*/

    cpLog(LOG_DEBUG, "%s[%x]-FIELD(%s) SUBFIELD(%s) NOT-PRESENT OP(%x) #_ADDR(%d)",
          name(), this,
          fieldBuffer, subfieldBuffer,
          myNotPresentOp.getPtr() , myOperators.size() );

}  //logData


///Returns the name of the class
const char* const
CPLOpAddressSwitch::name() const
{
    return "CPLOpAddressSwitch";
}  //name


///
void
CPLOpAddressSwitch::doCpLog() const
{
    logData();

    if ( myNotPresentOp != 0 )
    {
        cpLog( LOG_DEBUG, "%s's not-present Op, %s, follows:",
               name(), myNotPresentOp->name() );
        myNotPresentOp->doCpLog();
    } /*if*/

    myOpConstIterator it = myOperators.begin();
    myOpConstIterator endOfList = myOperators.end();

    while ( it != endOfList )
    {
        cpLog( LOG_DEBUG, "%s[%x]'s %s[%x]",
               name(), this, (*it)->name(), it->getPtr() );
        if ( strcmp( (*it)->name(), "CPLOpProxy" ) != 0 )
        {
            (*it)->doCpLog();
        } /*if*/
        ++it;
    } /*while*/
}  //doCpLog


//End of file

