/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const CPLOpFork_cxx_Version =
    "$Id: CPLOpFork.cxx,v 1.25 2002/11/09 02:05:37 derekm Exp $";

#include "CPLOpFork.hxx"
#include "SipMsg.hxx"
#include "StatusMsg.hxx"
#include "SipCommand.hxx"
#include "InviteMsg.hxx"
#include "CancelMsg.hxx"
#include "ByeMsg.hxx" 
#include "AckMsg.hxx"
#include "CallInfo.hxx"
#include "SipEvent.hxx"
#include "SipProxyEvent.hxx"
#include "TimerEvent.hxx"
#include "Data.hxx"
#include "SystemInfo.hxx"
#include "FsConfigFile.hxx"
#include "SipVia.hxx"
#include "SipRecordRoute.hxx"

using namespace Vocal;

///Constructor
CPLOpFork::CPLOpFork()
{
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "new CPLOpFork[%x]", this );
#endif
    ordering = parallel;  // default
    myRecurse = false;  // default
    myTimeOutVal = defaultTimeout;  // default
}  //c'tor


///Destructor
CPLOpFork::~CPLOpFork()
{
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "~CPLOpFork[%x]", this );
#endif
}  //d'tor


///Append an operator to this class
bool
CPLOpFork::setNextOperator( Sptr < Operator > op, int prevNode )
{
    if (strcmp(op->name(), "CPLOpNoAnswer") == 0)
        myNoAnswerOp = op;
    else if (strcmp(op->name(), "CPLOpBusy") == 0)
        myBusyOp = op;
    else if (strcmp(op->name(), "CPLOpFailure") == 0)
        myFailureOp = op;
    else if (strcmp(op->name(), "CPLOpProxy") == 0)
        myProxyOp = op;
    else return false;

    return true;
}  //setNextOperator


///Initialize the Attributes member variables
bool
CPLOpFork::setAttributes( const char* attrName, const char* attrValue)
{
    if (strcmp(attrName, "ordering") == 0)
    {
        if (strcmp(attrValue, "parallel") == 0)
            ordering = parallel;
        else if (strcmp(attrValue, "sequential") == 0)
            ordering = sequential;
        else if (strcmp(attrValue, "first-only") == 0)
            ordering = firstOnly;
        else return false;
    }
    else if (strcmp(attrName, "recurse") == 0)
    {
        if (strcmp(attrValue, "true") == 0)
            myRecurse = true;
        else if (strcmp(attrValue, "false") == 0)
            myRecurse = false;
        else return false;
    }
    else if (strcmp(attrName, "timeout") == 0)
    {
        myTimeOutVal = atoi(attrValue);
    }
    else return false;

    return true;
}  //setAttributes


///
const Sptr < State >
CPLOpFork::process( Sptr < SipProxyEvent > event)
{
    //  most probably a SIP event
    Sptr < SipEvent > ptrSipEvent;
    ptrSipEvent.dynamicCast(event);
    if (ptrSipEvent != 0)
    {
        return processMsg(ptrSipEvent);
    }

    //  else MUST be a timer event
    Sptr < TimerEvent > ptrTimerEvent;
    ptrTimerEvent.dynamicCast(event);
    if (ptrTimerEvent != 0)
    {
        return noAnswer(ptrTimerEvent);
    }

    //  else something terribly wrong here!!!
    cpLog(LOG_ERR, "ERROR - got an event neither SIP nor Timer !!!");
    return wait(event);  //waiting because couldn't abort 
}  //process


///Outputs the values of the member variables
void
CPLOpFork::logData() const
{
    string log = "proxy ordering[";
    switch (ordering)
    {
        case parallel:
        {
            log += "parallel";
        }
        break;
        case sequential:
        {
            log += "sequential";
        }
        break;
        case firstOnly:
        {
            log += "first-only";
        }
        default:
        ;  //Not expecting to do anything else
    } /*switch*/

    cpLog(LOG_DEBUG, "%s] time-out[%d] myRecurse[%s]", log.c_str(),
          myTimeOutVal, (myRecurse == true ) ? "true" : "false");
}  //logData


//Make a detailed log, and call operators 
void 
CPLOpFork::doCpLog() const
{
    logData();

    if (myNoAnswerOp != 0)
        myNoAnswerOp->doCpLog();
    else cpLog(LOG_DEBUG, "No Answer o/p not specified");

    if (myBusyOp != 0)
        myBusyOp->doCpLog();
    else cpLog(LOG_DEBUG, "Busy o/p not specified");

    if (myFailureOp != 0)
        myFailureOp->doCpLog();
    else cpLog(LOG_DEBUG, "Failure o/p not specified");
}


///Returns the name of the class

const char* const
CPLOpFork::name() const
{
    return "CPLOpFork";
}  //name


const Sptr < State > 
CPLOpFork::noAnswer(Sptr < TimerEvent > timerEvent)
{
    cpLog(LOG_DEBUG, "taking the NO ANSWER action");

    //  if ``No-Answer' is defined,
    if (myNoAnswerOp != 0)
    {
        //Cancel all the outstanding requests downstream
        cancel(timerEvent);

        //Cleanup
        cleanUp(timerEvent);

        //Set the proxy status
        getStateMachineData(timerEvent)->setProxyStatus(
            FsStateMachnData::NoAnswer);

        //Take the output
        return myNoAnswerOp->process(Sptr < SipProxyEvent > (timerEvent));
    }

    //If we reach here, then <no-answer> was not defined in
    //the CPL script.  Continue to let phone ring 
    //while waiting for other response messages.
       
    return wait(timerEvent);
}



const Sptr < State > 
CPLOpFork::busy(Sptr < SipEvent > sipEvent)
{
    cpLog(LOG_DEBUG, "taking the BUSY action");

    //  reset the timer
    stopTimer(sipEvent);

    //  set the proxy status
    getStateMachineData(sipEvent)->setProxyStatus(FsStateMachnData::Busy);

    //  if busy output is defined by user, just do it
    if (myBusyOp != 0)
    {
        //  clean up
        cleanUp(sipEvent);

        //  take the output
        return myBusyOp->process(Sptr < SipProxyEvent > (sipEvent));
    }

    //  else,
    //Send the busy response upstream
    sipEvent->getSipStack()->sendReply(
        *(getStateMachineData(sipEvent)->getBestResponse()));

    //Then terminate
    return end(sipEvent);
}



const Sptr < State > 
CPLOpFork::failure(Sptr < SipEvent > sipEvent)
{
    cpLog(LOG_DEBUG, "taking the FAILURE action");

    //  reset the timer
    stopTimer(sipEvent);

    //  set the proxy status
    getStateMachineData(sipEvent)->setProxyStatus(FsStateMachnData::Failure);

    //  if failure output is defined by user, just do it
    if (myFailureOp != 0)
    {
        //  clean up
        cleanUp(sipEvent);

        //  take the output
        return myFailureOp->process(Sptr < SipProxyEvent > (sipEvent));
    }

    //  else,
    //  send the failure response upstream
    sipEvent->getSipStack()->sendReply(
        *(getStateMachineData(sipEvent)->getBestResponse()));

    //  then terminate
    return end(sipEvent);
}


const Sptr < State > 
CPLOpFork::success(Sptr < SipEvent > sipEvent)
{
    cpLog(LOG_DEBUG, "taking the SUCCESS action");

    //  reset the timer
    stopTimer(sipEvent);

    //  cancel all the outstanding requests
    cancel(sipEvent);

    //  set the proxy status
    getStateMachineData(sipEvent)->setProxyStatus(FsStateMachnData::Answered);

    //  send the success response upstream.
    sipEvent->getSipStack()->sendReply(
        *(getStateMachineData(sipEvent)->getBestResponse()));

    //  then wait for the duration of the call
    return wait(sipEvent);

}

const Sptr < State > 
CPLOpFork::redirect(Sptr < SipEvent > sipEvent)
{
    cpLog(LOG_DEBUG, "taking the REDIRECT action");

    //  reset the timer
    stopTimer(sipEvent);

    //  set the proxy status
    getStateMachineData(sipEvent)->setProxyStatus(
        FsStateMachnData::Redirected);

    //  send the redirection response upstream.
    sipEvent->getSipStack()->sendReply(
        *(getStateMachineData(sipEvent)->getBestResponse()));

    //  then terminate
    return end(sipEvent);
}


//  NOTE: this is called if the caller sends a CANCEL/BYE before
//        the call is set-up.
const Sptr < State > 
CPLOpFork::abort(Sptr < SipEvent > sipEvent)
{
    cpLog(LOG_DEBUG, "taking the ABORT action");

    //  reset the timer
    stopTimer(sipEvent);

    //  send the 200 response upstream
    Sptr < CancelMsg > cancelMsg;
    cancelMsg.dynamicCast(sipEvent->getSipMsg());
    StatusMsg response(*cancelMsg, 200);
    sipEvent->getSipStack()->sendReply(response);

    //  send the CANCEL to all outstanding requests
    cancel(sipEvent);

    //  set the proxy status
    getStateMachineData(sipEvent)->setProxyStatus(FsStateMachnData::NoProxy);

    //  then terminate
    return end(sipEvent);
}

const Sptr < State > 
CPLOpFork::wait(Sptr < SipProxyEvent > anEvent)
{
    cpLog(LOG_DEBUG, "waiting");

    //  now retrieve the current state from callinfo and return
    return anEvent->getCallInfo()->getState();
}


// clean up the call info state for this call
void 
CPLOpFork::cleanUp(Sptr < SipProxyEvent > anEvent)
{
    cpLog(LOG_DEBUG, "cleaning up the call state ");
    getStateMachineData(anEvent)->setInRequest(0);
    getStateMachineData(anEvent)->setBestResponse(0);
    Sptr < vector < Sptr < InviteMsg > > > outRequests =
        getStateMachineData(anEvent)->getOutRequests();
    if ( (outRequests != 0) && (outRequests->size() != 0) )
    {
        cpLog(LOG_ERR, "ERROR: there are %d outstanding requests for cleanup!",
              outRequests->size());
        cancel(anEvent);
    }
    getStateMachineData(anEvent)->setOutRequests(0);
    getStateMachineData(anEvent)->setLocationBkup(0);
}

// proxy the intermediate messages
const Sptr < State > 
CPLOpFork::proxy(Sptr < SipEvent > sipEvent)
{
    Sptr < State > state;
    if (myProxyOp != 0)
    {
        cpLog(LOG_DEBUG, "proxying");
        // get the return state, after proxy
        state = myProxyOp->process((Sptr < SipProxyEvent > )sipEvent);
    }
    else
    {
        cpLog(LOG_ERR, "ERROR: proxy o/p not specified");
        state = 0;
    }

    //  if its end of call then clean up
    if (state == 0)
        cleanUp(sipEvent);

    return state;
}


// terminate the call
const Sptr < State > 
CPLOpFork::end(Sptr < SipProxyEvent > anEvent)
{
    cpLog(LOG_DEBUG, "terminating");

    // clean up the call state data
    cleanUp(anEvent);

    // return 0, curr state will call its next op, viz. opEnd
    return 0;
}


//  cancel all the outstanding ``forked'' requests
void 
CPLOpFork::cancel(Sptr < SipProxyEvent > anEvent)
{
    cpLog(LOG_DEBUG, "canceling all the outstanding INVITEs");

    Sptr < vector < Sptr < InviteMsg > > > outRequests =
        getStateMachineData(anEvent)->getOutRequests();

    //Cancel all the request
    CPLOpFork::OutIterator end = outRequests->end();
    CPLOpFork::OutIterator curr = outRequests->begin();
    while (curr != end)
    {
        CancelMsg msg = CancelMsg(**curr);

        sendAsync(anEvent, msg);  // using CPLOperator's method
        curr++;
    }

    //  remove all the requests from list
#if 0
    outRequests->clear();
#endif
}


// fork INVITEs to the urls, and add them to outRequests
void 
CPLOpFork::fork(Sptr < SipEvent > sipEvent, CPLUrlData& urls)
{
    Sptr <SipUrl> sipUrl = new SipUrl;
    Sptr < InviteMsg > inRequest =
        getStateMachineData(sipEvent)->getInRequest();

    cpLog(LOG_DEBUG, "forking %d branches", urls.size());

    while (urls.top(*sipUrl) == true)
    {
	Sptr<SipUrl> originalUrl;

	originalUrl.dynamicCast(inRequest->getRequestLine().getUrl());

	// if this is not a SIP URL, complain
	if(originalUrl == 0)
	{
	    // xxx this is poor practice -- should handle non-SIP URLs
	    cpLog(LOG_ERR, "not a SIP URL -- aborting");
	    return;
	}

        // construct a new INVITE as follows

        //copy the ``original'' incoming request
        Sptr < InviteMsg > inviteMsg = new InviteMsg(*inRequest);

        //Change the SipURL to a BaseUrl
        Sptr <BaseUrl> baseUrl = sipUrl;

        //change the request URI (sec 12.3.1: RFC 2543)
        SipRequestLine requestLine(SIP_INVITE, baseUrl);
        inviteMsg->setRequestLine(requestLine);

        //insert the VIA 
        SipVia via;
        OpAddVia(via, cmdLineData.sipPort);

        //insert the incremented BRANCH tag

	Data branch = inviteMsg->computeBranch2() +
	    getStateMachineData(sipEvent)->getNextBranch();

        via.setBranch( branch );  
        inviteMsg->setVia(via, 0);


        //insert the RECORD-ROUTE
        SipRecordRoute route;
        OpAddRecordRoute(route, *originalUrl, cmdLineData.sipPort);
        inviteMsg->setRecordRoute(route, 0);

        //add cc-diversion ONLY if sending to a different USER.
        if ( requestLine.getUrl()->getUserValue() != inviteMsg->getTo().getUser() )
        {
            addCallControlDiversion(sipEvent, inviteMsg, baseUrl );
        } /* if */
        else
        {
            cpLog( LOG_DEBUG, "Skipping cc-diversion for '%s'.",
                   inviteMsg->getTo().getUser().logData() );
        } /*else*/

        // send this new request
        sipEvent->getSipStack()->sendAsync(*inviteMsg);

        // add this request to outRequests
        getStateMachineData(sipEvent)->getOutRequests()->push_back(inviteMsg);

        cpLog(LOG_DEBUG, "forked URI:\"%s@%s:%s\", branch[%s]\n%s",
              sipUrl->getUserValue().logData(), 
              sipUrl->getHost().logData(),
              sipUrl->getPort().logData(), 
              branch.logData(),
              inviteMsg->encode().logData());
    }
}


//  do a via check on the respone, to find if it is acceptable
//  (sec 12.3.2: RFC 2543)
bool 
CPLOpFork::viaCheck(Sptr < StatusMsg > response)
{

    //  make a VIA for this host
    SipVia myVia;
    OpAddVia(myVia, cmdLineData.sipPort);

    //  get the topmost VIA of this response
    SipVia via = response->getVia(0);

    //  compare the host name of this via
    if (myVia.getHost() != via.getHost())
        return false;

    return true;
}


// detect loop in the incoming request
// (sec 12.3.1: RFC 2543)
bool 
CPLOpFork::isLoop(Sptr < SipCommand > aCommand)
{
    //  make a VIA for this host
    SipVia myVia;
    OpAddVia(myVia, cmdLineData.sipPort);

    //  for each VIA in this command do
    for (int i = aCommand->getNumVia(); i--; )
    {
        //      get the i'th VIA of this command
        SipVia via = aCommand->getVia(i);

        //      compare the host name of this via
        if (myVia.getHost() == via.getHost() &&
                myVia.getPort() == via.getPort())
            return true;
    }
    return false;
} /*isLoop*/


int 
CPLOpFork::ack(Sptr < SipEvent > sipEvent, Sptr < StatusMsg > response)
{
    Sptr < vector < Sptr < InviteMsg > > > outRequests =
        getStateMachineData(sipEvent)->getOutRequests();

    //  find the ref of matching invite in outRequest list
    CPLOpFork::OutIterator curr = outRequests->begin();
    CPLOpFork::OutIterator end = outRequests->end();

    cpLog(LOG_DEBUG, "Size of outRequests: %d", outRequests->size() );

    while (curr != end)
    {
        if (match(response, *curr) == true)
            break;
        curr++;
    }

    //  if ref NOT found MUST be proxied upstream
    //  also if branch did not match, we'll proxy (safe!)
    //  (section 12.3.6: RFC 2543)
    if (curr == end)
    {
        return 0;
    }

    //  check the status code
    int status = response->getStatusLine().getStatusCode();

    //  if < 200 then simply return
    if (status < 200)
        return status;

    //  if 300+ then ACK immediately
    if (status >= 300)
    {
        AckMsg ack(*response);
        SipRequestLine ackReqLine(SIP_ACK,
                                  (*curr)->getRequestLine().getUrl());
        ack.setRequestLine(ackReqLine);
        sipEvent->getSipStack()->sendAsync(ack);
    }

    //  if ref found, delete it
    outRequests->erase(curr);

    return status;
}


void 
CPLOpFork::stopTimer(Sptr < SipProxyEvent > anEvent)
{
    cancelTimer(anEvent);
}


// does a parallel forking
void 
CPLOpFork::doParallel(Sptr < SipEvent > sipEvent,
                           Sptr < CPLUrlData > locationSet)
{
    if (locationSet->empty() == false)
        fork(sipEvent, *locationSet);

    cpLog(LOG_DEBUG, "after PARALLEL forking #%d outstanding requests",
          getStateMachineData(sipEvent)->getOutRequests()->size());
}

// does a sequential forking
void 
CPLOpFork::doSequential(Sptr < SipEvent > sipEvent,
                             Sptr < CPLUrlData > locationSet)
{
    CPLUrlData urls;
    SipUrl *url = new SipUrl;
    if (locationSet->top(*url) == true)
    {
        Sptr<BaseUrl> baseUrl = url;
        urls.push(baseUrl);
        fork(sipEvent, urls);
    }

    cpLog(LOG_DEBUG, "after SEQUENTIAL forking #%d outstanding requests",
          getStateMachineData(sipEvent)->getOutRequests()->size());
}


// does a recursion for 3xx responses
void 
CPLOpFork::doRecursion(Sptr < CPLUrlData > locationSet,
                            Sptr < StatusMsg > response)
{
    for (int i = response->getNumContact()-1; i >= 0; i-- )
        locationSet->push(response->getContact(i).getUrl());
    cpLog(LOG_DEBUG, "doRecursion added %d more urls",
          response->getNumContact());
}


const Sptr < State > 
CPLOpFork::processResponse(Sptr < SipEvent > sipEvent,
        Sptr < StatusMsg > response)
{
    Sptr < FsStateMachnData > mcData = getStateMachineData(sipEvent);

    // check if the response is acceptable
    cpLog(LOG_DEBUG, "going for VIA check");
    if (viaCheck(response) == false)
    {
        cpLog(LOG_DEBUG, "VIA check fail, dropping response");

        return wait(sipEvent);
    }

    // its acceptable, so ACK if necessary and take action
    cpLog(LOG_DEBUG, "going for status check");
    int status = ack(sipEvent, response);

    // take action based on status code
    cpLog(LOG_DEBUG, "Got status code [%d]", status);

    if( status <= 100 && status > 0 )
    {
        cpLog( LOG_DEBUG, "No action" );
        return wait( sipEvent );
    }

    // check if it must be proxied upstream
    if (status < 200 || status == 0)
    {
        cpLog(LOG_DEBUG, "response[%d] proxied upstream",
              response->getStatusLine().getStatusCode());
        return proxy(sipEvent);
    }

    //This response is one of the expected responses.  Continue
    //processing it. 

    response->removeVia(0);

    // check if its the best response so far
    if (mcData->getBestResponse() == 0)
    {
        mcData->setBestResponse(response);
    }
// TODO: clean up after testing
#if 0
    else if (status < mcData->getBestResponse()->getStatusLine().getStatusCode())
    {
        // for 4xx messages preserve the BUSY response
        if ( ( (status / 100) != 4 ) ||
             (mcData->getBestResponse()->getStatusLine().getStatusCode() !=486))
        {
            mcData->setBestResponse(response);
        }
    }
#else
    else
    {
        int bestStatus = mcData->getBestResponse()->getStatusLine().getStatusCode();
        if( status < bestStatus )
        {
            if( ( (status/100) != 4 ) || ( bestStatus != 486 ) )
            {
                mcData->setBestResponse(response);
            }
        }
        else
        {
            if( bestStatus == 200 )
            {
                cpLog( LOG_DEBUG, "Have seen 200, no Action" );
                return wait( sipEvent );
            }
            status = bestStatus;
        }
    }
#endif


    // now take best action based on best response and ordering
// TODO: clean up after testing
#if 0
    status = mcData->getBestResponse()->getStatusLine().getStatusCode();
#endif
    if ( ( (status / 100) != 2) && 
         (mcData->getOutRequests()->size() > 0) )
    {
        return wait(sipEvent);
    } /*if*/

    switch ( (status / 100) *100)
    {
        case 200:
        {
            // take the success output
            return success(sipEvent);
        }
        break;

        case 300:
        {
            // if ordering was first only then we are done
            if (ordering == firstOnly)
                return redirect(sipEvent);

            // if recurse flag is defined then recurse
            if (myRecurse == true)
            {
                doRecursion(mcData->getLocations(), response);
            }
    
            // if ordering is parallel then do a parallel
            if (ordering == parallel)
                doParallel(sipEvent, mcData->getLocations());
            // else do sequential
            else doSequential(sipEvent, mcData->getLocations());

            // wait if there is any outstanding INVITE
            if (mcData->getOutRequests()->size() > 0)
            {
                return wait(sipEvent);
            }
            // else redirect
            return redirect(sipEvent);
        } /*case*/
        break;

        case 400:
        case 500:
        case 600:
        {
            // if ordering was first only then we are done
            if (ordering == firstOnly)
            {
                if (status == 486)
                {
                    return busy(sipEvent);
                }
                //  or failure
                else return failure(sipEvent);
            }

            // if ordering is parallel then do a parallel
            if (ordering == parallel)
                doParallel(sipEvent, mcData->getLocations());
            // else do sequential
            else doSequential(sipEvent, mcData->getLocations());

            // wait if there is any outstanding INVITE
            if (mcData->getOutRequests()->size() > 0)
            {
                return wait(sipEvent);
            }
            // else check whether busy
            if (status == 486)
            {
                return busy(sipEvent);
            }
            //  or failure
            else return failure(sipEvent);
        } /*case*/
        break;

        default:
        cpLog(LOG_ERR, "unknown \"best response\" : [%d]", status);
        return wait(sipEvent);
    }

    cpLog(LOG_ERR, "ERROR: shouldn't have reached here !!!");
    return proxy(sipEvent);  // hoping for the best:)
}



const Sptr < State > 
CPLOpFork::processRequest(Sptr < SipEvent > sipEvent,
        Sptr < SipCommand > command)
{
    cpLog(LOG_DEBUG, "processing a request");
    Sptr < FsStateMachnData > mcData = getStateMachineData(sipEvent);

    //      detect loop
    if (isLoop(command) == true)
    {
        cpLog(LOG_DEBUG, "LOOP detected, dropping request");

        /// Request will be automatically dropped by the SIP stack
        /// after a specific period of time 

        return wait(sipEvent);
    }

    //      check if its an INVITE
    Sptr < InviteMsg > invite;
    invite.dynamicCast(command);
    if (invite != 0)
    {
        // check if its a re-invite
        // (retransmissions are taken care by sip stack!)
        if (mcData->getInRequest() != 0)
        {
            // cancel all the outstanding requests
            cancel(sipEvent);

            // refill the location set
            *(mcData->getLocations()) = *(mcData->getLocationBkup());

            //  reset the best response
            mcData->setBestResponse(0);

            // stop any previous timer
            stopTimer(sipEvent);

            cpLog(LOG_DEBUG, "incoming request is a re-INVITE");
        }
        else
        {
            cpLog(LOG_DEBUG, "incoming request is a new INVITE");

            // check if location set is empty
            if (mcData->getLocations()->size() == 0)
            {
                StatusMsg response(*invite, 404);
                sipEvent->getSipStack()->sendReply(response);
                return end(sipEvent);
            }

            // make a backup of location set
            Sptr < CPLUrlData > locationBkup = new CPLUrlData;
            *locationBkup = *(mcData->getLocations());
            mcData->setLocationBkup(locationBkup);

            // Create a new outRequests set in call info
            mcData->setOutRequests(new vector < Sptr < InviteMsg > > );
        }

        //  copy this ref of new INVITE
        mcData->setInRequest(invite);

//TODO: delete after testing
#if 0
        // respond with 100
        StatusMsg response(*invite, 100);
        sipEvent->getSipStack()->sendReply(response);
#endif

        // if ordering is parallel then do a parallel
        if (ordering == parallel)
            doParallel(sipEvent, mcData->getLocations());
        // else do sequential
        else doSequential(sipEvent, mcData->getLocations());

        // start the timer (if NO ANSWER o/p is defined)
        if (myNoAnswerOp != 0)
            setTimer(sipEvent, myTimeOutVal);

        // and then wait for responses
        return wait(sipEvent);
    }
    //      else check if its a CANCEL 
    else
    {
        Sptr < CancelMsg > cancel;
        cancel.dynamicCast(command);
        if (cancel != 0)
        {
            // check if it belongs to the inRequest
            Sptr < InviteMsg > invite;
            invite.dynamicCast(sipEvent->getPendingCommand(command));
            if (matchInvites(invite, mcData->getInRequest()) == true)
            {
                // belongs to us, abort !!!
                cpLog(LOG_DEBUG, "recieved a CANCEL request, aborting");
                return abort(sipEvent);
            }
        }

        //Check if this BYE message needs to be treated like
        //a CANCEL (e.g. Pingtel's requirements)
        Sptr < ByeMsg > byeMsg;
        byeMsg.dynamicCast( command );
        if ( byeMsg != 0 )
        {
            // check if it belongs to the inRequest
            Sptr < InviteMsg > invite;
            invite.dynamicCast(sipEvent->getPendingCommand(command));
            if (matchInvites(invite, mcData->getInRequest()) == true)
            {
                // belongs to us, abort !!!
                cpLog(LOG_DEBUG, "Abort: Received a BYE/CANCEL request");
                return abort(sipEvent);
            }
            // Hang up
        }  /* if byeMsg != 0 */
        else
        {   //else proxy the request.
            return proxy(sipEvent);
        }
    }
    cpLog(LOG_ERR, "ERROR: should'nt have reached here !!!");
    return proxy(sipEvent);  // hoping for the best:)
}


// compare two INVITEs for equality
// (sec 12.3.4: RFC 2543)

bool 
CPLOpFork::matchInvites(Sptr < InviteMsg > msg1, Sptr < InviteMsg > msg2)
{
    if( msg1 != 0 && msg2 != 0 )
    {
        if ( ( msg1->getTo() == msg2->getTo()) && 
             ( msg1->getFrom() == msg2->getFrom()) && 
             ( msg1->getCallId() == msg2->getCallId() ) && 
             ( msg1->getCSeq() == msg2->getCSeq()) 
           ) 
        {
           return true;
        }
    }
    return false;
}



// match the a response with an Invite
// (sec 12.3.6: RFC 2543)
bool 
CPLOpFork::match(Sptr < StatusMsg > response, Sptr < InviteMsg > invite)
{
    if( response != 0 && invite != 0 )
    {
        if ( response->getTo().getUrl()->areEqual( invite->getTo().getUrl() ) &&       
             (response->getFrom() == invite->getFrom()) &&        
             (response->getCallId() == invite->getCallId()) && 
             (response->getCSeq() == invite->getCSeq()) && 
             (response->getVia(0).getBranch() == invite->getVia(0).getBranch())
           )
        {
            return true;
        }
//TODO: for debug only, delete it later
#if 1
        else
        {
            if ( !response->getTo().getUrl()->areEqual( invite->getTo().getUrl() ) )
                cpLog( LOG_DEBUG, "getTo:\n\n%s\n\n%s\n",
                                  (*response->getTo().getUrl()).encode().logData(),
                                  (*invite->getTo().getUrl()).encode().logData() );
            if( !(response->getFrom() == invite->getFrom()) )
                cpLog( LOG_DEBUG, "getFrom:\n\n%s\n\n%s\n",
                                  response->getFrom().encode().logData(),
                                  invite->getFrom().encode().logData() );
            if( !(response->getCallId() == invite->getCallId()) )
                cpLog( LOG_DEBUG, "getCallId:\n\n%s\n\n%s\n",
                                  response->getCallId().encode().logData(),
                                  invite->getCallId().encode().logData() );
            if( !(response->getCSeq() == invite->getCSeq()) )
                cpLog( LOG_DEBUG, "getCSeq:\n\n%s\n\n%s\n",
                                  response->getCSeq().encode().logData(),
                                  invite->getCSeq().encode().logData() );
            if( !(response->getVia(0).getBranch() == invite->getVia(0).getBranch()) )
                cpLog( LOG_DEBUG, "getVia:\n\n%s\n\n%s\n",
                                  response->getVia().getBranch().logData(),
                                  invite->getVia().getBranch().logData() );
        }
    }
    else
        cpLog( LOG_DEBUG, "%s is 0", (response == 0) ? "response":"invite");
#else
    }
#endif
    return false;
}



// get the original INVITE in the call leg of this response
Sptr < InviteMsg > 
CPLOpFork::getOriginal(Sptr < SipEvent > sipEvent,
        Sptr < StatusMsg > response) const
{
    Sptr < InviteMsg > invite;

    invite = getInvite(sipEvent);
    if (invite == 0)
    {
        return 0;
    }
    InviteMsg inviteCopy( *invite );

    //  get the self's network address
    string hostString( theSystem.gethostAddress() );
    NetworkAddress FSAddress;
    FSAddress.setHostName( hostString );
    FSAddress.setPort( cmdLineData.sipPort );

    int numRecordRoute;
    numRecordRoute = inviteCopy.getNumRecordRoute();

    invite->flushrecordrouteList();

    Sptr<SipUrl> sipUrl;

    for (int idx = (numRecordRoute - 1); idx >= 0; idx-- )
    {
        SipRecordRoute recordRte = inviteCopy.getRecordRoute(idx);

        //Get Host & Port from Record Route and save to Network Address
        int rrPortNum;

        sipUrl.dynamicCast( recordRte.getUrl() );

        rrPortNum = sipUrl->getPort().convertInt();

        LocalScopeAllocator lo;
        string rrHost( sipUrl->getHost().getData(lo) );
        NetworkAddress rrAddr( rrHost, rrPortNum );

        if (rrAddr == FSAddress )
        {
#if 0 //Change to 1 if needed for debugging
            cpLog( LOG_DEBUG, "Record Route Match: %s:%d",
                   rrAddr.getIpName().c_str(), rrAddr.getPort() );
#endif
            break;
        }
        else
        {
#if 0 //Change to 1 if needed for debugging
            cpLog( LOG_DEBUG, "No Record Route Match:  %s:%d vs %s:%d",
                   rrAddr.getIpName().c_str(), rrAddr.getPort(),
                   FSAddress.getIpName().c_str(), FSAddress.getPort() );
#endif
            invite->setRecordRoute( recordRte , -1 );
        }
    } /*for*/

    //  now also remove the TOP via
    invite->removeVia(0);


    return invite;
}


// process this sip message and take the best action
const Sptr < State > 
CPLOpFork::processMsg(Sptr < SipEvent > aSipEvent)
{
    Sptr < FsStateMachnData > mcData = getStateMachineData(aSipEvent);

    //Check if the event is a response to a request we've made.  If
    //not, an error has occurred;  Get the originalInvites and
    //retransmit the request messages. 
    Sptr < StatusMsg > response;
    response.dynamicCast(aSipEvent->getSipMsg());
    if (response != 0)
    {
        cpLog(LOG_DEBUG, "processing a response");

        // o.k. its a response, but check if we missed it's INVITE
        // (possible 'coz lookup etc. may ``eat'' original INVITE)
        if (mcData->getInRequest() == 0)
        {
            cpLog(LOG_DEBUG,
                  "INVITEless response, looking for \"original\" request");

            Sptr < InviteMsg > invite = getOriginal(aSipEvent, response);
            if (invite == 0)
            {
                cpLog(LOG_DEBUG, "unknown response, proxying");
                return proxy(aSipEvent);
            }

            cpLog(LOG_DEBUG, "found original INVITE, working");
            return processRequest(aSipEvent, invite);
        }
        else return processResponse(aSipEvent, response);
    }
    //  else its a request
    else
    {
        Sptr < SipCommand > command;
        command.dynamicCast(aSipEvent->getSipMsg());
        if (command == 0)
        {
            cpLog(LOG_ERR,
                  "Neither StatusMsg nor SipCommand.");
            return wait(aSipEvent);
        }
        return processRequest(aSipEvent, command);
    }
    cpLog(LOG_ERR, "Shouldn't have reached here!");
    return proxy(aSipEvent);  
}


//End of file

