#ifndef CPLOpLocation_HXX
#define CPLOpLocation_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CPLOpLocationVersion =
    "$Id: CPLOpLocation.hxx,v 1.16 2002/11/09 02:05:37 derekm Exp $";




#include "CPLOperator.hxx"
#include <string>

namespace Vocal
{


bool validateUrl( string& url );

/**
  *  Stores a new URL in CPLUrlData
  */
class CPLOpLocation: public CPLOperator
{
    public:
        /// defualt constructor
        CPLOpLocation();

        /// destructor
        ~CPLOpLocation();

        /**@name CPLOperator implementations */
        //@{
        ///
        const Sptr < State > process( const Sptr < SipProxyEvent > event );

        ///
        bool setAttributes( const char* attrName, const char* attrValue );

        ///
        bool setNextOperator( Sptr < Operator > op, int prevNode );

        ///
        const char* const name() const;

    protected:
        ///
        void logData() const;
        //@}


    public:
        /**
         * method to retrieve location of last caller
         * - used in call return feature
         *@param event reference to the current event, containg the CallInfo
         */
        void retrieveLastCaller(Sptr < SipProxyEvent > event );

    protected:
        /// stores the new location from CPL script
        string myURL;

        /// stores user option from CPL script
        bool myClearLocationSet;
};
 
} // namespace Vocal

#endif

