#ifndef _CPL_OP_LOG__HXX
#define _CPL_OP_LOG__HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CPLOpLog_hxx_Version =
    "$Id$";





#include "CPLOperator.hxx"
#include "CPLTimeRange.hxx"
#include "CallProcessingLanguage.hxx"
#include "Operator.hxx"
#include "State.hxx"
#include "Sptr.hxx"
#include "Data.hxx"
#include "ProvisionInterface.hxx"
namespace Vocal
{


/**
 * Implements Log operator for the Call Processing Language
 * Note:  Also adds proprietary attribute value, "lastcaller" to
 *        implement Call Return.      
 */

class CPLOpLog : public CPLOperator
{
    public:
        /// default constructor
        CPLOpLog();

        /// destructor
        ~CPLOpLog();

        /**@name Cpl operator implementation*/
        //@{
        ///
        /** Save CPL tag attributes
          *@param type Attribute's name
          *@param value Value of the attribute 
          *@return True if successful setting attributes */
        bool setAttributes(const char* type, const char* value);

        /**Execute the operator for the state machine
          *@param anEvent Event from the Sip stack 
          *@return Pointer to next state or 0 if the operator did not need to
          *process the event. 
         **/
        const Sptr < State > process(const Sptr <SipProxyEvent> anEvent );

        /**Add special operators
           @param anOperator Pointer to the operator to add
           @param nodeId ID of the last CPL tag.
           @return true if function did not have any errors setting next
                   operator. 
        **/
        bool setNextOperator(Sptr <Operator> anOperator, 
                             int nodeId = CPLNodeEndofNodes);

        /**Return the class name.
         * @return Pointer to character string */
        const char* const name() const;

        /**
         * Save the From/To/DisplayName information for Call Return
         * @param anEvent A SIP event (e.g. INVITE) 
         **/
        void updateLastCaller(Sptr < SipProxyEvent > anEvent);

    private:
        ///Display information regarding the class' member variables
        void logData(void) const;
        //@}


    private:
        /**
         * method to log the information on persistent store
         *@param event reference to the event that caused this o/p
         */
        void log(const Sptr < SipProxyEvent > event);

    private:
        /// name of the log file
        string myName;
          
        /// comments to be placed on log
        string myComment;

        /// reference to the singleton Provisioning Interface instance
        ProvisionInterface *myPif ;
};
 
} // namespace Vocal

#endif

