#ifndef _CPL_OP_MAIL__HXX
#define _CPL_OP_MAIL__HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CPLOpMail_hxx_Version =
    "$Id$";





#include "CPLOperator.hxx"
#include "CPLTimeRange.hxx"
#include "CallProcessingLanguage.hxx"
#include "Operator.hxx"
#include "State.hxx"
#include "Sptr.hxx"
#include "Data.hxx"

namespace Vocal
{


/**
 * Implements the <mail> tag.
 * Mail works by creating a thread that waits to send 
 * messages.  A different thread is used so any problems
 * in sending won't affect the Feature Server's
 * main Event processing worker-thread.
 */

class CPLOpMail : public CPLOperator
{
    public:
        /// default constructors
        CPLOpMail();

        /// destructor
        ~CPLOpMail();

        /**@name CPL operator implementation*/
        //@{
        ///
        bool setAttributes(const char*, const char*);
        ///
        const Sptr < State > process(const Sptr < SipProxyEvent > );
        ///
        bool setNextOperator(Sptr < Operator > , int = CPLNodeEndofNodes);
        ///
        const char* const name() const;
    private:
        ///
        void logData(void) const;
        //@}

    private:
        /**
         * method to construct the message to be mailed
         *@param event reference to the event that caused this o/p
         *@return string containing the message
         */
        string getMsg(const Sptr < SipProxyEvent > & event) const;

    private:
        /// stores the Mail-To URL from CPL script
        string myUrl;
};
 
} // namespace Vocal

#endif

