#ifndef CPLOpNonAnswer_HXX
#define CPLOpNonAnswer_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CPLOpNonAnswerVersion =
    "$Id: CPLOpNonAnswer.hxx,v 1.18 2002/11/09 02:05:37 derekm Exp $";




#include "CPLOperator.hxx"
#include <string>
#include "CancelMsg.hxx"
#include "SipEvent.hxx"
#include "TimerEvent.hxx"

namespace Vocal
{

/**
  *  Operator to handle responses from a call attempt.
  */
class CPLOpNonAnswer: public CPLOperator
{
    public:
        /// default constructor
        CPLOpNonAnswer();

        /// destructor
        ~CPLOpNonAnswer();

        /**
         * method to process a TimerEvent
         *@param event const reference to the timer event
         *@return next state or 0
         */
        Sptr < State > processTimerEvent( const Sptr < SipProxyEvent > event );

        /**
         * method to process a Sip Event
         *@param event const reference to the Sip event
         *@return next state or 0
         */
        Sptr < State > processSipEvent( const Sptr < SipProxyEvent > event );
        /**
         * method to acknowledge a response based on original INVITE
         *@param event reference to the event containing reference to Sip messages
         */
        void ackBasedOnInvite( Sptr < SipProxyEvent > event );

        /**@name CPLOperator implementation methods*/
        //@{
        ///
        const Sptr < State > process( const Sptr < SipProxyEvent > event );
        ///
        bool setAttributes( const char* attrName, const char* attrValue );
        ///
        bool setNextOperator( Sptr < Operator > op, int prevNode );
        ///
        void doCpLog() const;
        ///
        const char* const name() const;

    protected:
        ///
        void logData() const;
        //@}

        /**@name CPLOpNonAnswer specific typedefs*/
        //@{
        /**  _Response = { DestinationBusy,
                       DestinationNoAnswer, 
                       DestinationFailure,
                       DestinationRedirection,
                       DestinationMisc      }
         **/
        enum _Response{ DestinationBusy,
                        DestinationNoAnswer,
                        DestinationFailure,
                        DestinationRedirection,
                        DestinationMisc };

        /**  @see _Response  */
        typedef enum _Response Destination_Response;
        //@}

        /// reference to the BUSY operator
        Sptr < Operator > myBusyReceived;

        /// reference to the No-Answer operator
        Sptr < Operator > myNoAnswerReceived;

        /// reference to the FAILURE operator
        Sptr < Operator > myFailedReceived;

        /// reference to the REDIRECTION operator
        Sptr < Operator > my3xxReceived;
          
        /// reference to the DEFAULT operator
        Sptr < Operator > myUndefinedOutput;
          
    private:
        /**
         * test for time-out events
         *@param event reference to the current event
         *@return boolean test result
         */
        bool isTimeout( Sptr < SipProxyEvent > event );

        /**
         * method to handle the responses for proxied requests
         *@param callResponse response type
         *@param event reference to the current event
         *@return reference to next state or 0
         */
        Sptr < State > handleResponse( Destination_Response callResponse,
                                       Sptr < SipProxyEvent > event );

};
 
} // namespace Vocal

#endif

