
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpPriority_cxx_Version =
    "$Id$";
#include "CPLOpPriority.hxx"

using namespace Vocal;

CPLOpPriority::CPLOpPriority():
    myField( CPLOpPriority::FieldInvalid )
{
}


CPLOpPriority::~CPLOpPriority()
{
    //no-op!!!
}



bool CPLOpPriority::setAttributes(const char* type, const char* value)
{
    myStr = value;

    if ( strcmp( type, "less" ) == 0 )
        myField = CPLOpPriority::FieldLess;
    else if ( strcmp( type, "greater" ) == 0 )
        myField = CPLOpPriority::FieldGreater;
    else if ( strcmp( type, "equal" ) == 0 )
        myField = CPLOpPriority::FieldEqual;
    else  //Unknown attribute name
        assert ( 0 );

    return true;
}


const Sptr < State > CPLOpPriority::process(const Sptr < SipProxyEvent > anEvent)
{
    return CPLOperator::process(anEvent);
}


bool CPLOpPriority::setNextOperator(Sptr < Operator > anOperator, int nodeId /*def value*/)
{
    addOperator(anOperator);
    return true;
}


const char* const 
CPLOpPriority::name() const
{
    return "CPLOpPriority";
}


void CPLOpPriority::logData(void) const
{
    switch (myField)
    {
        case CPLOpPriority::FieldLess:
        cpLog(LOG_DEBUG, "CPLOpPriority-LESS (%s)", myStr.c_str());
        break;

        case CPLOpPriority::FieldGreater:
        cpLog(LOG_DEBUG, "CPLOpPriority-GREATER (%s)", myStr.c_str());
        break;

        case CPLOpPriority::FieldEqual:
        cpLog(LOG_DEBUG, "CPLOpPriority-EQUAL (%s)", myStr.c_str());
        break;

        default:
        //attribute value not set...
        cpLog(LOG_DEBUG, "%s[%x]-FIELD (OTHERWISE)",
              name(), this);
    }
}


bool CPLOpPriority::compare(Data& data)
{
    //Check if this is the <default> case.  The
    //<default> case does not need to do any compares. 
    if ( myField == FieldInvalid )
        return true;      

    string priority[] = {"non-urgent", "normal", "urgent", "emergency"};

    /**
     * if no priority header is specified in a message, the priority is
     * considered to be normal (sec4.4:draft-ietf-iptel-01)...
     */
    int pNumber = 1, myPriority = 1, i;

    /**
     * if an unknown priority is given, the priority is considered to be
     * equivalent to normal  O N L Y  for cases of "less" and "greater",
     * for "equal" it is compared ``literally''...
     */
    LocalScopeAllocator lo;
    for (i = 0; i < 4; i++)
    {
        if (priority[i] == data.getData(lo)) // finding out request priority
        {
            pNumber = i;
            break;
        }
    } /*for*/

    for (i = 0; i < 4; i++)
    {
        if (priority[i] == myStr) // finding the priority to compare
        {
            myPriority = i;
            break;
        }
    } /*for*/


    switch (myField)
    {
        case CPLOpPriority::FieldLess:
        {
            return (pNumber < myPriority ? true : false);
        } /*case*/
        break;

        case CPLOpPriority::FieldGreater:
        {
            return (pNumber > myPriority ? true : false);
        } /*case*/
        break;

        case CPLOpPriority::FieldEqual:         // literal comparison
        {
            return (myStr == data.getData(lo) ? true : false);
        } /*case*/
        break;

        default:
        {
            return false;
        } /*default*/
    }
    return false;
}


