/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const CPLOpProxy_cxx_Version =
    "$Id: CPLOpProxy.cxx,v 1.56.2.2 2003/03/13 18:11:36 chok Exp $";

#include "CPLOpProxy.hxx"
#include <iostream.h>
#include "FsUsers.hxx"
#include "SipContact.hxx"
#include "CallInfo.hxx"
#include "SipFrom.hxx"
#include "SipTo.hxx"
#include "RegisterMsg.hxx"
#include "ReferMsg.hxx"
#include "SubscribeMsg.hxx"
#include "NotifyMsg.hxx"
#include "MessageMsg.hxx"
#include "UnknownExtensionMsg.hxx"
#include "CallContainer.hxx"
#include "FsConfigFile.hxx"
#include "SipRecordRoute.hxx"

using namespace Vocal;


///
CPLOpProxy::CPLOpProxy()
{
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "CPLOpProxy[%x]", this );
#endif
} // CPLOpProxy


///
CPLOpProxy::~CPLOpProxy()
{
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "~CPLOpProxy[%x]", this );
#endif
} // ~CPLOpProxy


///
bool
CPLOpProxy::setAttributes( const char* attrName, const char* attrValue )
{
    //This is not a CPL tag.  It should never have attributes.
    return false;
}  //setAttributes


///
bool
CPLOpProxy::setNextOperator( Sptr < Operator > op, int prevNode )
{
    //This is not a CPL tag.  This function should never be called.
    return false;
}  //setNextOperator


///
const char* const
CPLOpProxy::name() const
{
    return "CPLOpProxy";
} //name


///
void
CPLOpProxy::logData() const
{
    if (myNextState != 0)
        cpLog( LOG_DEBUG, "CPLOpProxy - NEXTSTATE(%s)", myNextState->name() );
    else
        cpLog( LOG_DEBUG, "CPLOpProxy" );

}  //logData


///
const Sptr < State >
CPLOpProxy::process( const Sptr < SipProxyEvent > event )
{
    cpLog(LOG_DEBUG, "process() " );

    // Proxying
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );

    Sptr < State > returnState;

    // Make sure it is a SIP message event
    if ( sipEvent != 0 )
    {
        // Get message from event
        Sptr < SipMsg > msg = sipEvent->getSipMsg();
        assert ( msg != 0 );

        Data prefix =
            getStateMachineData(event)->getPrefixCIDBlk();
#if 0  //Change to 1 if needed for debugging
        cpLog(LOG_DEBUG, "The saved prefix is: %s", prefix.logData());
#endif
        if (prefix.length() > 0)
        {
            restoreOrRemovePrefix(sipEvent, prefix);
        }

        SipFrom savedFrom = getStateMachineData(event)->getOrigFrom();

        Sptr<SipUrl> savedFromsUrl; 
        savedFromsUrl.dynamicCast( savedFrom.getUrl() );

        if ( ( savedFromsUrl != 0 ) && 
             ( savedFromsUrl->getUserValue().length() > 0) )
        {
            restoreOrRemoveCallerId(sipEvent);  // for status msg, restore caller id
            // for other msgs, remove caller id
        }

        returnState = processMsg(sipEvent, msg);
    }
    else
    {
        cpLog( LOG_DEBUG, "Not a SIP message event" );
        cpLog( LOG_DEBUG, "No action in CPLOpProxy" );
    }


    //Remove the pointer, so sipEvent can be deallocated.
    sipEvent = 0;

    return returnState;
}    // CPLOpProxy::process



///
Sptr < State >
CPLOpProxy::processMsg(Sptr < SipEvent > sipEvent, Sptr < SipMsg > msg)
{
    bool endCall = false;

    switch( msg->getType() )
    {
        case SIP_INVITE:
        {
            cpLog( LOG_DEBUG, "SIP INVITE method" );
            Sptr < InviteMsg > inviteMsg;
            inviteMsg.dynamicCast( msg );
            assert( inviteMsg != 0 );
            processInvite( sipEvent, inviteMsg );
        }
        break;
        case SIP_ACK:
        {
            cpLog( LOG_DEBUG, "SIP ACK method" );
            Sptr < AckMsg > ackMsg;
            ackMsg.dynamicCast( msg );
            assert( ackMsg != 0 );
            processAck( sipEvent, ackMsg );
        }
        break;
        case SIP_OPTIONS:
        {
            cpLog( LOG_DEBUG, "SIP OPTIONS method" );
            Sptr < OptionsMsg > optionsMsg;
            optionsMsg.dynamicCast( msg );
            assert( optionsMsg != 0 );
            OptionsMsg newOptionsMsg( *optionsMsg,
                                      OptionsMsg::optionsMsgForward );
            proxyRequest( sipEvent, newOptionsMsg );
        }
        break;
        case SIP_BYE:
        {
            cpLog( LOG_DEBUG, "SIP BYE method" );
            Sptr < ByeMsg > byeMsg;
            byeMsg.dynamicCast( msg );
            assert( byeMsg != 0 );
            // TODO: May not need this anymore when Msft fix messenger
            if ( byeMsg->getNumRoute() == 0 )
            {
                processByeAsCancel( sipEvent, byeMsg );
                endCall = true;
            }
            else
            {
                ByeMsg newByeMsg( *byeMsg , ByeMsg::byeMsgForward );
                proxyRequest( sipEvent, newByeMsg );
            }
        }
        break;
        case SIP_CANCEL:
        {
            cpLog( LOG_DEBUG, "SIP CANCEL method" );
            Sptr < CancelMsg > cancelMsg;
            cancelMsg.dynamicCast( msg );
            assert( cancelMsg != 0 );
            processCancel( sipEvent, cancelMsg );
        }
        break;
        case SIP_REGISTER:
        {
            cpLog( LOG_DEBUG, "SIP REGISTER method" );
            Sptr < RegisterMsg > registerMsg;
            registerMsg.dynamicCast( msg );
            assert( registerMsg != 0 );
            RegisterMsg newRegisterMsg( *registerMsg,
                                        RegisterMsg::registerMsgForward );
            proxyRequest( sipEvent, newRegisterMsg );
        }
        break;
        case SIP_INFO:
        {
            cpLog( LOG_DEBUG, "SIP extension INFO method" );
            Sptr < InfoMsg > infoMsg;
            infoMsg.dynamicCast( msg );
            assert( infoMsg != 0 );
            InfoMsg newInfoMsg( *infoMsg, InfoMsg::infoMsgForward );
            proxyRequest( sipEvent, newInfoMsg );
        }
        break;
        case SIP_REFER:
        {
            cpLog( LOG_DEBUG, "SIP extension REFER method" );
            Sptr < ReferMsg > referMsg;
            referMsg.dynamicCast( msg );
            assert( referMsg != 0 );
            ReferMsg newReferMsg( *referMsg, ReferMsg::referMsgForward );
            proxyRequest( sipEvent, newReferMsg );
        }
        break;
        case SIP_STATUS:
        {
            Sptr< StatusMsg > statusMsg;
            statusMsg.dynamicCast( msg );
            assert( statusMsg != 0 );
            cpLog( LOG_DEBUG,
                   "SIP response %d",
                   statusMsg->getStatusLine().getStatusCode() );
            processStatus( sipEvent, statusMsg );
            if( statusMsg->getStatusLine().getStatusCode() >= 300 )
            {
                endCall = true;
            }
            if ( statusMsg->getStatusLine().getStatusCode() == 200 )
            {
                Data method = statusMsg->getCSeq().getMethod();
                if ( method == "BYE")
                {
                    endCall = true;
                }
            }
        }
        break;
        case SIP_SUBSCRIBE:
        {
            cpLog( LOG_DEBUG, "SIP extension SUBSCRIBE method" );
            Sptr < SubscribeMsg > subscribeMsg;
            subscribeMsg.dynamicCast( msg );
            assert( subscribeMsg != 0 );
            SubscribeMsg newSubscribeMsg( *subscribeMsg,
                                          SubscribeMsg::subscribeMsgForward );
            proxyRequest( sipEvent, newSubscribeMsg );
        }
        break;
        case SIP_NOTIFY:
        {
            cpLog( LOG_DEBUG, "SIP extension NOTIFY method" );
            Sptr < NotifyMsg > notifyMsg;
            notifyMsg.dynamicCast( msg );
            assert( notifyMsg != 0 );
            NotifyMsg newNotifyMsg( *notifyMsg, NotifyMsg::notifyMsgForward );
            proxyRequest( sipEvent, newNotifyMsg );
        }
        break;
        case SIP_MESSAGE:
        {
            cpLog( LOG_DEBUG, "SIP extension MESSAGE method" );
            Sptr < MessageMsg > messageMsg;
            messageMsg.dynamicCast( msg );
            assert( messageMsg != 0 );
            MessageMsg newMessageMsg( *messageMsg, MessageMsg::messageMsgForward );
            proxyRequest( sipEvent, newMessageMsg );
        }
        break;
//        case SIP_UNKNOWNEXTN:
        case SIP_UNKNOWN:
        {
            cpLog( LOG_DEBUG, "Unknown SIP extension method" );
            Sptr < UnknownExtensionMsg > unknownExtensionMsg;
            unknownExtensionMsg.dynamicCast( msg );
            assert( unknownExtensionMsg != 0 );
            UnknownExtensionMsg newUnknownExtensionMsg(
                *unknownExtensionMsg,
                UnknownExtensionMsg::unknownExtensionMsgForward );
            proxyRequest( sipEvent, newUnknownExtensionMsg );
        }
        break;
        default:
        {
            cpLog( LOG_WARNING, "Invalid SIP message type %d", msg->getType() );
        }
    }

    Sptr < State > returnState;

    // End of call (BYE or CANCEL or non-1xx or non-2xx)
    if ( endCall )
    {
        cpLog( LOG_DEBUG, "End of call" );
        //Going to the End State is now a functionality of
        //ErrorOp
        returnState = 0;
    }
    else
    {
        returnState = myNextState;
    }
    return returnState;
}    // CPLOpProxy::processMsg


///
void
CPLOpProxy::processInvite(Sptr < SipEvent > sipEvent, Sptr < InviteMsg > inviteMsg )
{
    cpLog( LOG_DEBUG, "processInvite " );

    // Re-invite: try to find the associated INVITE

    int numRoute = inviteMsg->getNumRoute();
    if ( numRoute == 0 )
    {
        // The Re-Invite is receive before any final status (e.g. 200) message
        // is received (e.g. blind transfer)
        cpLog( LOG_INFO , "Route is empty: forward Re-Invite based on saved Invite");

        // get pending command to which this cancel applies
        // if none exist, return 481;
        // if one or more exists, forward Invite for each pending command
        // Note, only worries about first pending command for now
        // TODO: if forking Invite, need to forward the Re-Invite to
        // all of them

        // Get the originally sent INVITE message
        Sptr < SipCommand > savedSentInvite;
        savedSentInvite = sipEvent->getPendingCommand( inviteMsg );
        if ( savedSentInvite == 0 )
        {
            // error, no corresponding INVITE, return without proxying the Invite
            // reply with 481, Transaction does not exist
            cpLog( LOG_ERR, "No matching request for Re-Invite" );
            StatusMsg statusMsg( *inviteMsg, 481 );
            restoreCallerIdForReply(sipEvent, statusMsg);
            sipEvent->getSipStack()->sendReply( statusMsg );
        }
        else
        {
            // Return 100 to source
            StatusMsg statusMsg( *inviteMsg, 100 );
            restoreCallerIdForReply(sipEvent, statusMsg);
            sipEvent->getSipStack()->sendReply( statusMsg );

            // set RequestUri to that of pending command
            cpLog( LOG_DEBUG, "set Request URI to matched command" );
            SipRequestLine requestLine( SIP_INVITE,
                                        savedSentInvite->getRequestLine().getUrl() );
            inviteMsg->setRequestLine( requestLine );

            // Add Via header (same branch as 1st INVITE)
            SipVia via;
            OpAddVia( via, cmdLineData.sipPort );
            via.setBranch( inviteMsg->computeBranch2() + 
			   getStateMachineData(sipEvent)->getNextBranch());
            inviteMsg->setVia( via, 0);

            // Add Record-Route header
            SipRecordRoute route;
            Sptr<BaseUrl> url = inviteMsg->getRequestLine().getUrl();
            Sptr<SipUrl> sipUrl;
            sipUrl.dynamicCast( url );

            OpAddRecordRoute( route, *sipUrl, cmdLineData.sipPort);
            inviteMsg->setRecordRoute( route, 0 );
            // send to next hop
            sipEvent->getSipStack()->sendAsync( *inviteMsg );
        }
    } // if ( numRoute == 0 )
    else
    {
        // There is a Route which means that a 200 has been received
        // before the Re-Invite: just route based on the Route
        cpLog( LOG_DEBUG, "CPLOpProxy: forward Re-Invite message based on Route" );

        // Return 100 to source
        StatusMsg statusMsg( *inviteMsg, 100 );
        restoreCallerIdForReply(sipEvent, statusMsg);
        sipEvent->getSipStack()->sendReply( statusMsg );

        // Forward message based on Route
        InviteMsg newInviteMsg( *inviteMsg, InviteMsg::inviteMsgForward );

        // Add Via header
        SipVia via;
        OpAddVia( via, cmdLineData.sipPort );
        ;
        via.setBranch(newInviteMsg.computeBranch2() + 
		      getStateMachineData( sipEvent )->getNextBranch() );
        newInviteMsg.setVia(via, 0);

        // Add Record-Route header
        SipRecordRoute route;
        Sptr<BaseUrl> url = newInviteMsg.getRequestLine().getUrl();
        Sptr<SipUrl> sipUrl;
        sipUrl.dynamicCast( url );

        OpAddRecordRoute( route, *sipUrl, cmdLineData.sipPort);
        newInviteMsg.setRecordRoute( route, 0 );

        // Forward message to destination
        sipEvent->getSipStack()->sendAsync( newInviteMsg );
    }

}    // CPLOpProxy::processInvite


///
void
CPLOpProxy::processStatus(Sptr < SipEvent > sipEvent, Sptr < StatusMsg > statusMsg )
{
    // Never forward 100
    // Don't forward  200 for CANCEL
    int status = statusMsg->getStatusLine().getStatusCode();

    cpLog( LOG_DEBUG, "processStatus(%d)", status );

    if ( status == 100 )
    {
        return ;
    }
    if ( status >= 300 )
    {
        // Acknowledge the status message to request URI of matching request
        ackBasedOnInvite(sipEvent, statusMsg);
    }
    else
    {
        Data method = statusMsg->getCSeq().getMethod();
        if ( method == "CANCEL" )
        {
           return ;
        }
    }

    // Remove own Via header
    statusMsg->removeVia( 0 );
    // Forward status message based on Via

    sipEvent->getSipStack()->sendReply( *statusMsg );
}    // CPLOpProxy::processStatus


///
void
CPLOpProxy::processAck(Sptr < SipEvent > sipEvent, Sptr < AckMsg > ackMsg )
{
    cpLog( LOG_DEBUG, "processAck" );

    // check the number of Route, if the number is 0, the
    // ACK is generate by 4xx, don't forward it
    // TODO: There got to be a better way to check this.
    int numRoute = ackMsg->getNumRoute();
    if ( numRoute == 0 )
    {
        cpLog( LOG_INFO , "Route is empty, not for INVITE, don't forward");
        return ;
    }
    // forward ACK based on route
    Sptr < AckMsg > newAckMsg = new AckMsg( *ackMsg , AckMsg::ackMsgForward );

    // Add Via header: need to get the branch that was set in the via in
    // the original INVITE

    SipVia via;
    OpAddVia( via, cmdLineData.sipPort );
    via.setBranch(     newAckMsg->computeBranch2() + 
		       getStateMachineData(sipEvent)->getBranch());
    newAckMsg->setVia(via, 0);

    // Forward message to destination
    sipEvent->getSipStack()->sendAsync( *newAckMsg );
}    // CPLOpProxy::processAck


///
void
CPLOpProxy::processCancel(Sptr < SipEvent > sipEvent, Sptr < CancelMsg > cancelMsg )
{
    cpLog( LOG_DEBUG, "processCancel" );

    Sptr < SipCommand > commandToCancel = sipEvent->getPendingCommand( cancelMsg );
    if ( commandToCancel == 0 )
    {
        // reply with 481, Transaction does not exist
        cpLog( LOG_ERR, "No matching request for cancel" );
        StatusMsg statusMsg( *cancelMsg, 481 );
        restoreCallerIdForReply(sipEvent, statusMsg);
        sipEvent->getSipStack()->sendReply( statusMsg );
    }
    else
    {
        cpLog( LOG_DEBUG, "200 response to CANCEL" );
        // reply with 200, and forward Cancel to receiver of commandToCancel
        StatusMsg statusMsg( *cancelMsg, 200 );
        restoreCallerIdForReply(sipEvent, statusMsg);
        sipEvent->getSipStack()->sendReply( statusMsg );

        cpLog( LOG_DEBUG, "Proxy CANCEL down stream" );
        // set RequestUri to that of pending command
        SipRequestLine requestLine( SIP_CANCEL,
                                    commandToCancel->getRequestLine().getUrl() );
        cancelMsg->setRequestLine( requestLine );

        // flush Via list and add same via used in pending command
        cancelMsg->flushViaList();
        SipVia via( commandToCancel->getVia( 0 ) );

        via.setBranch( cancelMsg->computeBranch2() + 
		       getStateMachineData( sipEvent )->getBranch() );
        cancelMsg->setVia( via, 0 );

        // send to next hop
        sipEvent->getSipStack()->sendAsync( *cancelMsg );
    }
}    // CPLOpProxy::processCancel


///
void
CPLOpProxy::processByeAsCancel(Sptr < SipEvent > sipEvent, Sptr < ByeMsg > byeMsg )
{
    cpLog( LOG_DEBUG, "processByeAsCancel" );

    Sptr < SipCommand > commandToCancel = findPendingCommand(sipEvent, byeMsg );
    if ( commandToCancel == 0 )
    {
        // reply with 481, Transaction does not exist
        cpLog( LOG_ERR, "No matching request for cancel" );
        StatusMsg statusMsg( *byeMsg, 481 );
        restoreCallerIdForReply(sipEvent, statusMsg);
        sipEvent->getSipStack()->sendReply( statusMsg );
    }
    else
    {
        // reply with 200, and forward Bye to receiver of commandToCancel
        StatusMsg statusMsg( *byeMsg, 200 );
        restoreCallerIdForReply(sipEvent, statusMsg);
        sipEvent->getSipStack()->sendReply( statusMsg );

        // set RequestUri to that of pending command
        SipRequestLine requestLine( SIP_BYE,
                                    commandToCancel->getRequestLine().getUrl() );
        byeMsg->setRequestLine( requestLine );

        // flush Via list and add same via used in pending command
        byeMsg->flushViaList();
        SipVia via( commandToCancel->getVia( 0 ) );
        via.setBranch( byeMsg->computeBranch2() + 
		       getStateMachineData( sipEvent )->getBranch() );
        byeMsg->setVia( via, 0 );

        // send to next hop
        sipEvent->getSipStack()->sendAsync( *byeMsg );
    }
}    // CPLOpProxy::processByeAsCancel


void
CPLOpProxy::proxyRequest( Sptr < SipEvent > sipEvent, SipCommand& request )
{
    cpLog( LOG_DEBUG, "proxy request" );

    // Add Via header
    SipVia via;
    OpAddVia( via, cmdLineData.sipPort ); 
    via.setBranch( request.computeBranch2() + 
		   getStateMachineData( sipEvent )->getNextBranch() );
    request.setVia( via, 0 );

    // Forward message to destination
    sipEvent->getSipStack()->sendAsync( request );
}


void
CPLOpProxy::ackBasedOnInvite(Sptr < SipEvent > sipEvent, Sptr < StatusMsg > statusMsg )
{
    // retrieve INVITE associated with sipEvent
    Sptr < InviteMsg > savedInvite = sipEvent->getInvite();
    if ( savedInvite == 0 )
    {
        // error, no corresponding INVITE, return without sending ACK
        cpLog( LOG_ERR, "No INVITE associated with STATUS" );
        return ;
    }

    // send ACK to receiver of corresponding INVITE by setting Request URI
    // of ACK to that of the INVITE
    AckMsg ack( *statusMsg );
    SipRequestLine ackRequestLine( SIP_ACK,
                                   savedInvite->getRequestLine().getUrl() );
    ack.setRequestLine( ackRequestLine );

    SipFrom savedFrom = getStateMachineData( sipEvent )->getOrigFrom();
    SipFrom from = statusMsg->getFrom();
    if( (from.getUser() == savedFrom.getUser()) &&
        (from.getHost() == savedFrom.getHost()) &&
        (from.getPort() == savedFrom.getPort()) )
    {
        // Hide Caller Id is active, undo earlier restore
        Sptr<BaseUrl> baseUrl = from.getUrl();
        Sptr<SipUrl> fromUrl;
        fromUrl.dynamicCast( baseUrl );
        fromUrl->setUserValue( Data( "-" ) );
        fromUrl->setUserParam( Data( "" ) );
        from.setUrl( fromUrl );
        from.setDisplayName( Data( "Anonymous" ) );
        ack.setFrom( from );
    }
    sipEvent->getSipStack()->sendAsync( ack );
}


void
CPLOpProxy::restoreOrRemoveCallerId(Sptr < SipEvent > sipEvent)
{
    cpLog(LOG_DEBUG, "CPLOpProxy::restoreOrRemoveCallerId() ********");

    SipFrom from = sipEvent->getSipMsg()->getFrom();
    Sptr<BaseUrl> baseUrl = from.getUrl();
    Sptr<SipUrl> fromUrl;
    fromUrl.dynamicCast( baseUrl );

    SipTo to = sipEvent->getSipMsg()->getTo();
    Sptr<BaseUrl> toUrl = to.getUrl();
    Data toName = to.getDisplayName();

    SipFrom savedFrom = getStateMachineData(sipEvent)->getOrigFrom();

    baseUrl = savedFrom.getUrl();
    Sptr<SipUrl> savedFromUrl;
    savedFromUrl.dynamicCast( baseUrl ); 

    Data savedName = savedFrom.getDisplayName();

    if ((fromUrl->getUserValue() == Data("-")) &&
            (from.getHost() == savedFrom.getHost()) &&
            (from.getPort() == savedFrom.getPort()))
    {
        Sptr < StatusMsg > statusMsg;
        statusMsg.dynamicCast( sipEvent->getSipMsg() );
        if ( statusMsg != 0 )
        {
            Data method = statusMsg->getCSeq().getMethod();
            if ((statusMsg->getStatusLine().getStatusCode() >= 300) ||
                    ( (method == "BYE") &&
                      (statusMsg->getStatusLine().getStatusCode() == 200) ) )
            {
                sipEvent->getCallContainer()->removeCall(
                    sipEvent->getSipCallLeg());
            }
        }
        cpLog(LOG_DEBUG, "Old From: user[%s] displayname[%s]",
                         fromUrl->getUserValue().logData(),  
                         from.getDisplayName().logData());

        sipEvent->getSipMsg()->setFrom(savedFrom);
        sipEvent->setSipMsg(sipEvent->getSipMsg());  //reset CallLeg
        sipEvent->setCallLeg();

        cpLog(LOG_DEBUG, "New From: user[%s] displayname[%s]",
        sipEvent->getSipMsg()->getFrom().getUrl()->getUserValue().logData(),  
        sipEvent->getSipMsg()->getFrom().getDisplayName().logData());

    } // if (fromUrl.getUser() == Data("-") )

    else
    {
        Sptr<SipUrl> sipToUrl;
        sipToUrl.dynamicCast( toUrl );

        // if callee send Bye first
        if ((sipToUrl->getUserValue() == Data("-")) &&
                (to.getHost() == savedFrom.getHost()) &&
                (to.getPort() == savedFrom.getPort()))
        {
            cpLog(LOG_DEBUG, "Old To: user[%s] displayname[%s]",
                             toUrl->getUserValue().logData(),  
                             to.getDisplayName().logData());

            Sptr<SipUrl> newToUrl;
            SipTo newTo;

            newToUrl = savedFromUrl;
            newTo.setUrl(newToUrl);
            newTo.setDisplayName(savedName);
            sipEvent->getSipMsg()->setTo(newTo);
            sipEvent->setSipMsg(sipEvent->getSipMsg());  //reset CallLeg
            sipEvent->setCallLeg();

            cpLog(LOG_DEBUG, "New To: user[%s] displayname[%s]",
            sipEvent->getSipMsg()->getTo().getUrl()->getUserValue().logData(),
            sipEvent->getSipMsg()->getTo().getDisplayName().logData());
        } //if ((strcmp(toUrl.getUser() == Data("-")) 

        else
        {
            // for Ack, Bye, Cancel etc. msgs
            if ( (from.getUser() == savedFrom.getUser()) && 
                 (from.getHost() == savedFrom.getHost()) &&
                 (from.getPort() == savedFrom.getPort())
                )
            {
                Sptr < CancelMsg > cancelMsg;
                cancelMsg.dynamicCast( sipEvent->getSipMsg() );
                if ( cancelMsg != 0 )
                {
                    sipEvent->getCallContainer()->removeCall(
                        sipEvent->getSipCallLeg());
                }

                cpLog(LOG_DEBUG, "Old From: user[%s] displayname[%s]",
                                 fromUrl->getUserValue().logData(),  
                                 from.getDisplayName().logData());

                fromUrl->setUserValue(Data("-"));
                fromUrl->setUserParam(Data(""));
                from.setUrl(fromUrl);
                from.setDisplayName(Data("Anonymous"));
                sipEvent->getSipMsg()->setFrom(from);
                sipEvent->setSipMsg(sipEvent->getSipMsg());  //reset CallLeg
                sipEvent->setCallLeg();

                cpLog(LOG_DEBUG, "New From: user[%s] displayname[%s]",
                                 fromUrl->getUserValue().logData(),  
                                 from.getDisplayName().logData());
            } //if ((from.getUser() == savedFrom.getUser()) &&
            else
            {
                // for the 200 from caller to callee after Bye from callee
                if ( (to.getUser() == savedFrom.getUser()) && 
                     (to.getHost() == savedFrom.getHost()) &&
                     (to.getPort() == savedFrom.getPort())
                    )
                {
                    Sptr < StatusMsg > statusMsg;
                    statusMsg.dynamicCast( sipEvent->getSipMsg() );
                    if ( statusMsg != 0 )
                    {
                        Data method = statusMsg->getCSeq().getMethod();
                        if ( (method == "BYE") &&
                            (statusMsg->getStatusLine().getStatusCode() == 200) )
                        {
                            sipEvent->getCallContainer()->removeCall(
                                sipEvent->getSipCallLeg());
                        }
                    }

                    cpLog(LOG_DEBUG, "Old To: user[%s] displayname[%s]",
                                     toUrl->getUserValue().logData(),  
                                     to.getDisplayName().logData());

                    toUrl->setUserValue(Data("-"));
                    Sptr<SipUrl> sipToUrl;
                    sipToUrl.dynamicCast( toUrl );

                    sipToUrl->setUserParam(Data(""));
                    to.setUrl(toUrl);
                    to.setDisplayName(Data("Anonymous"));
                    sipEvent->getSipMsg()->setTo(to);
                    sipEvent->setSipMsg(sipEvent->getSipMsg());  //reset CallLeg
                    sipEvent->setCallLeg();

                    cpLog(LOG_DEBUG, "New To: user[%s] displayname[%s]",
                                     toUrl->getUserValue().logData(),  
                                     to.getDisplayName().logData());
                } //if ((to.getHost() == savedFrom.getHost()) &&
                else
                {
                    cpLog( LOG_ERR, "Why didn't it pass above cases?");
                    assert(0); 
                } //else of if ((to.getUser() == savedFrom.getUser()) &&
            } //else of if ((from.getUser() == savedFrom.getUser()) &&
        } //else of if ((toUrl.getUser() == Data("-")) 
    } // else of if (fromUrl.getUser() == Data("-"))
} //restoreOrRemoveCallerId


void
CPLOpProxy::restoreOrRemovePrefix(Sptr < SipEvent > sipEvent, Data prefix )
{
    cpLog(LOG_DEBUG, "restoreOrRemovePrefix()");

    SipFrom from = sipEvent->getSipMsg()->getFrom();
    Sptr<BaseUrl> fromUrl = from.getUrl();
    cpLog(LOG_DEBUG, "the from user is %s",
          fromUrl->getUserValue().logData());

    SipTo to = sipEvent->getSipMsg()->getTo();
    Sptr<BaseUrl> toUrl = to.getUrl();
    cpLog(LOG_DEBUG, "the to user is %s",
          toUrl->getUserValue().logData());

    SipTo savedTo = getStateMachineData(sipEvent)->getPrefixTo();
    Sptr<BaseUrl> savedToUrl = savedTo.getUrl();

    cpLog(LOG_DEBUG, "the saved to user is %s",
          savedToUrl->getUserValue().logData());

    Sptr < StatusMsg > statusMsg;
    LocalScopeAllocator lo;
    statusMsg.dynamicCast( sipEvent->getSipMsg() );
    if ( statusMsg != 0 )
    {
        Data method = statusMsg->getCSeq().getMethod();
        if ((statusMsg->getStatusLine().getStatusCode() >= 300) ||
                ( (method == "BYE") &&
                  (statusMsg->getStatusLine().getStatusCode() == 200) ) )
        {
            sipEvent->getCallContainer()->removeCall(sipEvent->getSipCallLeg());
        }

        if ((to.getHost() == savedTo.getHost()) &&
                (to.getPort() == savedTo.getPort()))
        {
            // restore prefix in To
            to.setUrl(savedToUrl);
            Data tempToDispName = prefix + to.getDisplayName() ;
            cpLog(LOG_DEBUG, "The tempToDispName is: %s",
                  tempToDispName.logData());
            to.setDisplayName(tempToDispName);
            sipEvent->getSipMsg()->setTo(to);
            sipEvent->setSipMsg(sipEvent->getSipMsg());  //reset CallLeg
            sipEvent->setCallLeg();
        }

        else
        {
            if ((from.getHost() == savedTo.getHost()) &&
                    (from.getPort() == savedTo.getPort()))
            {
                // remove prefix in From
                Data xPrefix;
                Data tempFromPhoneNum = fromUrl->getUserValue();
                tempFromPhoneNum.match(prefix.getData(lo), &xPrefix, true, "");
                fromUrl->setUserValue(tempFromPhoneNum);
                from.setUrl(fromUrl);

                Data dispName = from.getDisplayName();
                dispName.match(prefix.getData(lo), &xPrefix, true, "");
                from.setDisplayName(dispName);

                sipEvent->getSipMsg()->setFrom(from);
                sipEvent->setSipMsg(sipEvent->getSipMsg());  //reset CallLeg
                sipEvent->setCallLeg();
                cpLog(LOG_DEBUG, "The modified from Url is: %s",
                      sipEvent->getSipMsg()->getFrom().getUrl()->encode().logData());
            }
            else
            {
                cpLog(LOG_DEBUG,
                      "No restoring/removing prefix performed for status msg.");
            }
        }
    }
    else
    {
        Sptr < CancelMsg > cancelMsg;
        cancelMsg.dynamicCast( sipEvent->getSipMsg() );
        if ( cancelMsg != 0 )
        {
            sipEvent->getCallContainer()->removeCall(sipEvent->getSipCallLeg());
        }

        if ((to.getHost() == savedTo.getHost()) &&
                (to.getPort() == savedTo.getPort()))
        {
            // remove prefix in To and RequestLine
            Data yPrefix;
            Data tempToPhoneNum = toUrl->getUserValue();
            tempToPhoneNum.match(prefix.getData(lo), &yPrefix, true, "");
            toUrl->setUserValue(tempToPhoneNum);
            to.setUrl(toUrl);

            Data dispNameTo = to.getDisplayName();
            dispNameTo.match(prefix.getData(lo), &yPrefix, true, "");
            to.setDisplayName(dispNameTo);

            sipEvent->getSipMsg()->setTo(to);
            sipEvent->setSipMsg(sipEvent->getSipMsg());  //reset CallLeg
            sipEvent->setCallLeg();
            cpLog(LOG_DEBUG, "The modified to Url is: %s",
                  sipEvent->getSipMsg()->getTo().getUrl()->encode().logData());

            Sptr < SipCommand > command;
            command.dynamicCast( sipEvent->getSipMsg() );
            if (command != 0)
            {
                SipRequestLine reqLine = command->getRequestLine();
                Sptr<BaseUrl> reqUrl = reqLine.getUrl();
                Data zPrefix;
                Data tempReqUser = reqUrl->getUserValue();
                tempReqUser.match(prefix.getData(lo), &zPrefix, true, "");
                reqUrl->setUserValue(tempReqUser);
                reqLine.setUrl(reqUrl);
                command->setRequestLine(reqLine);
                cpLog(LOG_DEBUG, "The modified RequestLine Url is:%s",
                      command->getRequestLine().getUrl()->encode().logData());
            }
        }
        else
        {
            if ((from.getHost() == savedTo.getHost()) &&
                    (from.getPort() == savedTo.getPort()))
            {
                // restore prefix in From
                from.setUrl(savedToUrl);
                Data tempFromDispName = prefix + from.getDisplayName() ;
                cpLog(LOG_DEBUG, "The tempFromDispName is: %s",
                      tempFromDispName.logData());
                from.setDisplayName(tempFromDispName);
                sipEvent->getSipMsg()->setFrom(from);
                sipEvent->setSipMsg(sipEvent->getSipMsg());  //reset CallLeg
                sipEvent->setCallLeg();
            }

            else
            {
                cpLog(LOG_DEBUG,
                      "No restoring/removing prefix performed for request.");
            }
        }
    }
}

void
CPLOpProxy::restoreCallerIdForReply(Sptr < SipEvent > sipEvent, 
                                    StatusMsg& statusMsg )
{
    cpLog(LOG_DEBUG, "CPLOpProxy::restoreCallerIdForReply()");

    SipFrom from = statusMsg.getFrom();
    Sptr<BaseUrl> fromUrl = from.getUrl();

    cpLog(LOG_DEBUG, "FROM-user(%s) & displayName(%s)",
                     fromUrl->getUserValue().logData(),  
                     from.getDisplayName().logData() );

    SipFrom savedFrom = getStateMachineData(sipEvent)->getOrigFrom();
    Sptr<BaseUrl> savedFromUrl = savedFrom.getUrl();

    if (savedFromUrl == 0)
    {
        return; 
    } /*if*/

    cpLog(LOG_DEBUG, "the saved user is %s",
          savedFromUrl->getUserValue().logData());
    cpLog(LOG_DEBUG, "the saved displayname is %s",
          savedFrom.getDisplayName().logData());
    
    if ((fromUrl->getUserValue() == Data("-")) &&
            (from.getHost() == savedFrom.getHost()) &&
            (from.getPort() == savedFrom.getPort()))
    {
        cpLog(LOG_DEBUG, "restoring From using savedFrom");
        statusMsg.setFrom(savedFrom);
    }
}


const Sptr < SipCommand >
CPLOpProxy::findPendingCommand(Sptr < SipEvent > sipEvent, Sptr < SipCommand > sipCommand ) const
{
    // allocate vector to store previous msgs with same call leg
    SipEvent::CallLegVector callLegMsgs;
    callLegMsgs = getStack(sipEvent)->getCallLegMsgs( sipCommand );

    // create iterator to search for matching command
    vector < Sptr < SipMsg > > ::iterator iter;
    iter = callLegMsgs.begin();

    // search through previous msgs, and return matching command if found;
    // otherwise return 0
    cpLog( LOG_INFO, "Check each matching command" );
    Sptr < SipMsg > callLegMsg;
    Sptr < SipCommand > pendingCommand;

    while ( iter != callLegMsgs.end() )
    {
        cpLog( LOG_INFO, "Check this command" );

        if ( *iter == 0 )
        {
            cpLog( LOG_ERR, "This command is empty" );
        }

        else if ( pendingCommand.dynamicCast( *iter ) == 0 )
        {
            cpLog( LOG_ERR, "This command is not a SIP command" );
        }

        else
        {
            // first Via of Command must match second Via of pending command
            if ( pendingCommand->getNumVia() >= 2 )
            {
                cpLog(LOG_DEBUG, "Comparing ((branch = %s)) ",
                      pendingCommand->getVia(0).encode().logData());
                int pendingCmdBranch;
                pendingCmdBranch = pendingCommand->getVia(0).getBranch().convertInt();
                if ( pendingCmdBranch ==
                        getStateMachineData(sipEvent)->getBranch().convertInt() )
                {
                    cpLog( LOG_DEBUG, "matching command found" );
                    return pendingCommand;
                }
                cpLog( LOG_DEBUG, "found command, but Via's don't match" );
            }
        }

        ++iter;
    }

    // no match found, return 0
    cpLog( LOG_ERR, "no matching command" );
    return 0;
}

//End of File

