/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const CPLOpRedirect_cxx_Version =
    "$Id: CPLOpRedirect.cxx,v 1.37 2002/11/12 20:25:22 veer Exp $";

#include <iostream.h>
#include <CallInfo.hxx>
#include "CPLOpRedirect.hxx"
#include "SipContact.hxx"
#include "SipVia.hxx"
#include "CPLUrlData.hxx"
#include "SystemInfo.hxx"
#include "FsConfigFile.hxx"

using namespace Vocal;

///
CPLOpRedirect::CPLOpRedirect()
{
#ifdef MEM_TEST
    cpLog (LOG_DEBUG, "CPLOpRedirect[%x]", this );
#endif
} // CPLOpRedirect


///
CPLOpRedirect::~CPLOpRedirect()
{
#ifdef MEM_TEST
    cpLog (LOG_DEBUG, "~CPLOpRedirect[%x]", this );
#endif
} // ~CPLOpRedirect


///
const Sptr < State >
CPLOpRedirect::process( const Sptr < SipProxyEvent > event )
{
    Sptr < StatusMsg > statMsg;
    Sptr < State > returnState = myNextState;

    cpLog( LOG_DEBUG, "process()" );

    // Determine if Event is a SIP event or a timeout has occurred
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent != 0 )
    {
        // Get the the last sent/received INVITE message
        Sptr < InviteMsg > inviteMsg;
        inviteMsg = sipEvent->getInvite();

        if ( inviteMsg != 0 )
        {
            statMsg = buildRedirectMsg( event, inviteMsg );
            if ( statMsg != 0 )
            {
                //Look to see if the top VIA is the feature
                //server's address.  If so, remove it.  
                SipVia via = statMsg->getVia( 0 );
                Data hostAddress( theSystem.gethostAddress() );
                if ( (via.getHost() == hostAddress ) &&
                     (via.getPort() == cmdLineData.sipPort) )
                {
                    statMsg->removeVia(0);
                }

                sipEvent->getSipStack()->sendReply( *statMsg );
            }  /*if (statMsg != 0) */
        } /*if*/
        else
        {
            cpLog( LOG_ERR, "FS is expecting only INVITE messages" );
            returnState = 0;
        } /*else*/
    } /*if (sipEvent != 0) */
    else
    {
        // Get message from event
        Sptr < TimerEvent > timerEvent;
        timerEvent.dynamicCast( event );

        if ( timerEvent != 0 )
        {
            Sptr < SipCommand > sipCommand;
            Sptr < InviteMsg > inviteMsg;

            inviteMsg.dynamicCast(timerEvent->getCallInfo()->getSipCommand());
            statMsg = buildRedirectMsg( event, inviteMsg );
              
            // Remove the FS from the via list as it is present in the
            // INVITE that has been saved in the timer Event
            statMsg->removeVia(0);
            if ( statMsg != 0 )
                timerEvent->getSipStack()->sendReply( *statMsg );
        }
        else
        {
            cpLog( LOG_DEBUG, "Event received is not a Timer event" );
            returnState = 0;
        }
    } /*else*/

    return returnState;
}  //process


///
const char* const
CPLOpRedirect::name() const
{
    return "CPLOpRedirect";
} //name


///
void
CPLOpRedirect::logData() const
{
    cpLog( LOG_DEBUG, "%s", name() );

}  //logData


///
bool
CPLOpRedirect::setAttributes( const char* attrName, const char* attrValue)
{
    //<redirect> has no attributes
    return true;
}  //setID


///
bool
CPLOpRedirect::setNextOperator( Sptr < Operator > op, int )
{
    addOperator( op );
    return true;
}  //setNextOperator


///
Sptr < StatusMsg >
CPLOpRedirect::buildRedirectMsg( Sptr < SipProxyEvent > event,
                                 Sptr < InviteMsg > inviteMsg )
{
    Sptr < CPLUrlData > urlCollector;
    urlCollector = getStateMachineData(event)->getLocations();

    if ( urlCollector->size() > 0 )
    {
        //Find redirect URL
        Sptr<SipUrl> contactUrl = new SipUrl; 
        Data contactDispName;
        urlCollector->getURL( *contactUrl, 0 );
        urlCollector->getDispName( contactDispName, 0 );

        ///To create the contact URL's host and port, we need the
        ///endpoint's Marshal's hostname and port.         
        if ( inviteMsg->getTo().getPort() > 0 )
        {
            ///We assume the Marshal URL information is 
            ///in the "To" field
            contactUrl->setHost(inviteMsg->getTo().getHost());
            contactUrl->setPort( Data(inviteMsg->getTo().getPort()) );
            cpLog(LOG_DEBUG, "Building contactURL using TO." ); 
        } /*if*/
        else
        {
            ///We assume the Marshal URL information is
            ///in the second VIA of the VIA list.
            int totalVias = inviteMsg->getNumVia();
            totalVias -= 2; 

            contactUrl->setHost(inviteMsg->getVia(totalVias).getHost());
            contactUrl->setPort( Data(inviteMsg->getVia(totalVias).getPort()));
            
            cpLog(LOG_DEBUG, "Building contactURL using VIA." ); 
        } /*else*/

        cpLog( LOG_DEBUG, "Redirecting to '%s<%s@%s:%s;user=%s>'",
               contactDispName.logData(),
               contactUrl->getUserValue().logData(),
               contactUrl->getHost().logData(),
               contactUrl->getPort().logData(),
               contactUrl->getUserParam().logData() );

        //Add the redirect URL to the contact list
        SipContact sipcontact;
        Sptr<BaseUrl> baseUrl = contactUrl;
        sipcontact.setUrl( baseUrl );
        sipcontact.setDisplayName( contactDispName );

        //Create a new 302 message.
        Sptr < StatusMsg > statusMsg = new StatusMsg( *inviteMsg, 302 );
        statusMsg->setContact( sipcontact );

        //Add the Call Control Diversion header ONLY if sending to
        //a different USER. 
        if (inviteMsg->getTo().getUser() !=sipcontact.getUrl()->getUserValue() )
        {
            addCallControlDiversion( event, 
                                     statusMsg, 
                                     inviteMsg->getRequestLine().getUrl() );
        } /* if */
        else
        {
            cpLog( LOG_DEBUG, "Skipping adding cc-diversion for '%s'.",
                   inviteMsg->getTo().getUser().logData() );
        } /*else*/

        return statusMsg;
    }
    else
    {
        Sptr < StatusMsg > statusMsg = new StatusMsg( *inviteMsg, 404 );
        cpLog( LOG_DEBUG, "Couldn't find contact, return 404");
        return statusMsg;
    }

}    // CPLOpRedirect::buildRedirectMsg


//End of File

