
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpRemoveLocation_cxx_Version =
    "$Id: CPLOpRemoveLocation.cxx,v 1.20.2.1 2003/01/31 01:02:33 chok Exp $";
#include "CPLOpRemoveLocation.hxx"
#include "SipUrl.hxx"
#include "SipEvent.hxx"
#include "SipProxyEvent.hxx"
#include "CallInfo.hxx"
#include "CallContainer.hxx"

using namespace Vocal;


///
CPLOpRemoveLocation::CPLOpRemoveLocation()
{
    //Unnecessary to initialize member variables
    //since the "string" type defaults to an empty
    //string. 
       
#ifdef MEM_TEST
    cpLog(LOG_DEBUG, "CPLOpRemoveLocation[%x]", this );
#endif
}  //CPLOpRemoveLocation


///
CPLOpRemoveLocation::~CPLOpRemoveLocation()
{
#ifdef MEM_TEST
    cpLog(LOG_DEBUG, "~CPLOpRemoveLocation[%x]", this );
#endif
}  //~CPLOpRemoveLocation


///
const Sptr < State >
CPLOpRemoveLocation::process( const Sptr < SipProxyEvent > event )
{
    if ((myLocation == "complete_calleridblock") ||
            (myLocation == "selective_calleridblock"))
    {
        cpLog( LOG_DEBUG, "Block Caller ID" );
        callerIdBlock(event);
    }
    else
    {
        // other tag not implemented
        cpLog(LOG_ERR, "Remove Location not implemented. ");
    }
    return CPLOperator::process( event );
}  //process


///
const char* const
CPLOpRemoveLocation::name() const
{
    return "CPLOpRemoveLocation";
} //name


///
bool
CPLOpRemoveLocation::setNextOperator( Sptr < Operator > op, int )
{
    addOperator( op );
    return true;
} //setNextOperator


///
bool
CPLOpRemoveLocation::setAttributes( const char* attrName, const char* attrValue)
{
    if ( strcmp( attrName, "location" ) == 0)
    {
        if ( attrValue != 0 )
            myLocation = attrValue;
    }
    else if ( strcmp( attrName, "param" ) == 0)
    {
        if ( attrValue != 0 )
            myParam = attrValue;
    }
    else if ( strcmp( attrName, "value" ) == 0)
    {
        if ( attrValue != 0 )
            myValue = attrValue;
    }

    return true;
}  //setAttributes


///
void
CPLOpRemoveLocation::logData() const
{
    cpLog( LOG_DEBUG, "Removing location(%s)", myLocation.c_str() );

}  //logData


///
void
CPLOpRemoveLocation::callerIdBlock(Sptr < SipProxyEvent > event )
{
    Sptr < FsStateMachnData > fsmData = getStateMachineData(event);

    bool unblock = fsmData->getNoCIDBlk();
    if (!unblock)
    {
        cpLog(LOG_DEBUG, "callerIdBlock()");

        //Get the message's From field 
        Sptr < SipMsg > sipMsg = getSipMsg( event );
        assert (sipMsg != 0 );
        SipFrom from = sipMsg->getFrom();

        //Is there any data in the From field's user?
        if ( from.getUrl()->getUserValue().length() <= 0 )
        {
            cpLog( LOG_ERR, "Size can't be less than 1." ); 
            assert (0);
        } // if (size <= 0) 

        //Make a copy, so original fromUrl won't get modified.
        SipFrom tempFrom = from;

        //Save original From's USER
        fsmData->setOrigFrom( tempFrom );

        cpLog( LOG_DEBUG, "Saved Info - user[%s], displayname[%s]",
               fsmData->getOrigFrom().getUrl()->getUserValue().logData(),
               fsmData->getOrigFrom().getDisplayName().logData() ); 

        //Change the From field's URL to anonymous 
        Sptr<BaseUrl> fromUrl = from.getUrl(); 
        if ( fromUrl->setUserValue(Data("-")) == false )
            cpLog( LOG_ERR, "Failed to setUserValue. " ); 

        //Erase the User Param Data 
        Sptr<SipUrl> sipFromUrl;
        sipFromUrl.dynamicCast( fromUrl );
        sipFromUrl->setUserParam(Data(""));

        //Change the DisplayName 
        from.setUrl(fromUrl);
        from.setDisplayName(Data("Anonymous"));

        //Modify Invite 
        Sptr < InviteMsg > inviteMsg;
        inviteMsg = getInvite( event );
        assert (inviteMsg != 0);
        inviteMsg->setFrom(from);

        // Reset Call Leg 
        sipMsg->setFrom( from );

        cpLog(LOG_DEBUG, "Changed caller - user[%s], displayname[%s]",
                         sipMsg->getFrom().getUrl()->getUserValue().logData(), 
                         sipMsg->getFrom().getDisplayName().logData());

        // add a new entry in CallContainer
        Sptr < CallInfo > call = event->getCallInfo();
        assert (call != 0 );
        Sptr < CallContainer > callContainer = event->getCallContainer();
        assert( callContainer != 0 ); 

        Sptr < SipCallLeg > callLeg = new SipCallLeg( sipMsg->computeCallLeg() );
           
        callContainer->addCall( callLeg, call );
    } // if (!unblock) 
}  //callerIdBlock

//End of File

