
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpStringSwitch_cxx_Version =
    "$Id$";
#include "CPLOpStringSwitch.hxx"
#include "CPLOpString.hxx"
#include "SipSubject.hxx"
#include "SipOrganization.hxx"
#include "SipUserAgent.hxx"

using namespace Vocal;

///
CPLOpStringSwitch::CPLOpStringSwitch():
    myField( CPLOpStringSwitch::FieldInvalid )
{
}  //Constructor


///
CPLOpStringSwitch::~CPLOpStringSwitch()
{
    //no-op!!!
}  //Destructor


///
bool
CPLOpStringSwitch::setAttributes(const char* type, const char* value)
{
    //check attribute type
    if (strcmp(type, "field") == 0)
    {   //check attribute value
        if (strcmp(value, "subject") == 0)
            myField = CPLOpStringSwitch::FieldSubject;
        else if (strcmp(value, "organization") == 0)
            myField = CPLOpStringSwitch::FieldOrganization;
        else if (strcmp(value, "user-agent") == 0)
            myField = CPLOpStringSwitch::FieldUserAgent;
        else
        {
            cpLog( LOG_ERR, "Unknown attribute: \"%s\".", value ); 
            return false; 
        }
        return true;
    }

    //illegal attribute type
    assert(0);

    return false;  
}  //setAttributes


///

const Sptr < State >
CPLOpStringSwitch::process(const Sptr < SipProxyEvent > anEvent)
{
    Sptr < State > retVal;
    Data str;

    cpLog(LOG_DEBUG, ":process()");

    //Look for string in the SIP message 
    if (getStr(anEvent, str) != true)
    {
        //If string is not found, do the <not-present> output.
        if ( myNotPresentOp != 0)
        {
            return (myNotPresentOp->process(anEvent));
        } /*if*/
        else
        {
            /** If string is not found in the SIP message
              * and a <not-present> output was not defined in
              * the CPL script, just use any garbage string. 
              * The garbage string will not succeed in
              *the <string> comparisons regardless. 
             **/
            str = "@#$%!";
        } /*else*/
    }

    cpLog(LOG_DEBUG, " Switch on \"%s\"", str.logData());

    CPLOperator::myOpIterator curr = myOperators.begin();
    CPLOperator::myOpIterator end = myOperators.end();

    //assuming same as CPLOpAddressSwitch that default
    //operators are already in operator list
    while (curr != end)
    {
        if (strcmp((*curr)->name(), "CPLOpString") == 0)
        {   //curr is "String" operator
            if (((CPLOpString *)curr->getPtr())->compare(str) == true)
            {
                cpLog(LOG_DEBUG, " Compare result = true");
                retVal = (*curr)->process(anEvent);
                if (retVal != 0)
                    return retVal;
            }
            else
                cpLog(LOG_DEBUG, " Compare result = false");
        }
        else
        {
            cpLog(LOG_DEBUG, " Executing non-<string>");
            retVal = (*curr)->process(anEvent);
            if (retVal != 0)
                return retVal;
        }
        curr++;
    }

    //no change in state
    return 0;
}  //process


///
bool
CPLOpStringSwitch::setNextOperator(Sptr < Operator > anOperator, int nodeId /*def value*/)
{
    if (nodeId == CPLNodeNotpresent)
    {
        myNotPresentOp = anOperator;
        return true;
    }

    CPLOperator::myOpIterator curr = myOperators.begin();
    CPLOperator::myOpIterator end = myOperators.end();

    //assuming same as CPLOpAddressSwitch that default
    //operators are already in operator list
    while (curr != end)
    {
        if (strcmp((*curr)->name(), "CPLOpProxy") == 0)
        {
            myOperators.insert(curr, anOperator);
            return true;
        }
        curr++;
    }

    //default operator not found
    assert(0);

    return false;
}


///
const char* const
CPLOpStringSwitch::name() const
{
    return "CPLOpStringSwitch";
}  //name


///
void
CPLOpStringSwitch::doCpLog(void) const
{
    logData();

    if (myNotPresentOp != 0)
    {
        cpLog(LOG_DEBUG, "not-present op, %s, follows:",
              myNotPresentOp->name());
        myNotPresentOp->doCpLog();
    }

    CPLOperator::myOpConstIterator curr = myOperators.begin();
    CPLOperator::myOpConstIterator end = myOperators.end();

    while (curr != end)
    {
        cpLog(LOG_DEBUG, "%s[%x]",
              (*curr)->name(), curr->getPtr());
        if (strcmp((*curr)->name(), "CPLOpProxy") != 0)
        {
            (*curr)->doCpLog();
        }
        curr++;
    }
}  //doCpLog


///
void
CPLOpStringSwitch::logData(void) const
{
    switch (myField)
    {
        case CPLOpStringSwitch::FieldSubject:
        {
            cpLog(LOG_DEBUG, 
                  "[%x]-FIELD(%s) NOT-PRESENT OP(%x) # of operators (%d)",
                  name(), 
                  this, 
                  "Sub",
                  myNotPresentOp.getPtr(), 
                  myOperators.size());
        } /*case*/
        break;

        case CPLOpStringSwitch::FieldOrganization:
        {
            cpLog(LOG_DEBUG, 
                  "%s[%x]-FIELD(%s) NOT-PRESENT OP(%x) # of operators (%d)",
                  name(), 
                  this, 
                  "Org", 
                  myNotPresentOp.getPtr(), 
                  myOperators.size());
        } /*case*/
        break;

        case CPLOpStringSwitch::FieldUserAgent:
        {
            cpLog(LOG_DEBUG, 
                  "%s[%x]-FIELD(%s) NOT-PRESENT OP(%x) # of operators (%d)",
                  name(), 
                  this, 
                  "U-A", 
                  myNotPresentOp.getPtr(), 
                  myOperators.size());
        } /*case*/
        break;

        default:
        {
            //attribute value not set
            cpLog(LOG_DEBUG, 
                  "%s[%x]-FIELD(%s) NOT-PRESENT OP(%x) # of operators (%d)",
                  name(), 
                  this, 
                  "???", 
                  myNotPresentOp.getPtr(), 
                  myOperators.size());
        } /*default*/
    } /*switch*/
}  //logData


///
bool
CPLOpStringSwitch::getStr(Sptr < SipProxyEvent > anEvent, Data& data) const
{
    switch (myField)
    {
        case CPLOpStringSwitch::FieldSubject:
        {
            data = getSipCommand(anEvent)->getSubject().get();
            return true;
        } /*case*/
        break; 

        case CPLOpStringSwitch::FieldOrganization:
        {
            data = getSipCommand(anEvent)->getOrganization().get();
            return true;
        } /*case*/
        break; 

        case CPLOpStringSwitch::FieldUserAgent:
        {
            data = getSipCommand(anEvent)->getUserAgent().encode();
            return true;
        } /*case*/
        break; 

        default:
        break;
    } /*switch*/
    return false;
} //getStr


//End of file


