
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpTimeSwitch_cxx_Version =
    "$Id$";
#include "CPLOpTimeSwitch.hxx"
#include "CPLOpTime.hxx"

using namespace Vocal;

///
CPLOpTimeSwitch::CPLOpTimeSwitch()
{
    myTimeZoneOp = 0;
}  //Constructor


///
CPLOpTimeSwitch::~CPLOpTimeSwitch()
{
    myTimeZoneOp = 0;
}  //Destructor


///
bool
CPLOpTimeSwitch::setAttributes(const char* type, const char* value)
{
    //No attributes here.  It should be given the
    //timezone instance directly.
      
    return true;
}  //setAttributes


///
const Sptr < State >
CPLOpTimeSwitch::process(const Sptr < SipProxyEvent > anEvent)
{
    Sptr < State > retVal;
    MyTime myTime;

    cpLog(LOG_DEBUG, ":process()");

    myTime = myTimeZoneOp->getTime();

    cpLog(LOG_DEBUG, " Switch time \"%s\"...", myTime.toString());

    CPLOperator::myOpIterator curr = myOperators.begin();
    CPLOperator::myOpIterator end = myOperators.end();

    //assuming same as CPLOpAddressSwitch that default
    //operators are already in operator list...
    while (curr != end)
    {
        if (strcmp((*curr)->name(), "CPLOpTime") == 0)
        {   //curr is "Time" operator...
            if (((CPLOpTime *)curr->getPtr())->test(myTime) == true)
            {
                cpLog(LOG_DEBUG, " Compare result = true");
                retVal = (*curr)->process(anEvent);
                if (retVal != 0)
                    return retVal;
            }
            else
                cpLog(LOG_DEBUG, " Compare result = false");
        }
        else
        {
            cpLog(LOG_DEBUG, " Executing non <time>");
            retVal = (*curr)->process(anEvent);
            if (retVal != 0)
                return retVal;
        }
        curr++;
    }

    //no change in state...
    return 0;
}  //process


///
bool
CPLOpTimeSwitch::setNextOperator(Sptr < Operator > anOperator, int nodeId /*def value*/)
{
    CPLOperator::myOpIterator curr = myOperators.begin();
    CPLOperator::myOpIterator end = myOperators.end();

    //assuming same as CPLOpAddressSwitch that default
    //operators are already in operator list...
    while (curr != end)
    {
        if (strcmp((*curr)->name(), "CPLOpProxy") == 0)
        {   //curr is either "otherwise" or "default" operator...
            myOperators.insert(curr, anOperator);
            return true;
        }
        curr++;
    }

    //default operator not found...
    assert(0);

    return false;
}  //setNextOperator


///
const char* const
CPLOpTimeSwitch::name() const
{
    return "CPLOpTimeSwitch";
}  //name


///
void
CPLOpTimeSwitch::doCpLog(void) const
{
    logData();

    CPLOperator::myOpConstIterator curr = myOperators.begin();
    CPLOperator::myOpConstIterator end = myOperators.end();

    while (curr != end)
    {
        cpLog(LOG_DEBUG, "output %s[%x] follows...",
              (*curr)->name(), curr->getPtr());
        (*curr)->doCpLog();
        curr++;
    }
}  //doCpLog


///
void
CPLOpTimeSwitch::logData(void) const
{
    cpLog(LOG_DEBUG, "[%x]: using following timezone ...",
          name(), this);
    myTimeZoneOp->doCpLog();
}  //logData


///
CPLOpTimeSwitch&
CPLOpTimeSwitch::timeZone(Sptr < CPLOpTimeZone > aTZ)
{
    myTimeZoneOp = aTZ;
    return *this;
}  //timeZone


//End of file


