
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const FsFeature_cxx_Version =
    "$Id: FsFeature.cxx,v 1.3 2002/11/09 02:05:38 derekm Exp $";
#include "FsFeature.hxx"
#include "SipEvent.hxx"
#include "TimerEvent.hxx"
#include "CallInfo.hxx"

using namespace Vocal;

///
FsFeature::FsFeature()
{
    myCplScriptName = "FSFeature"; 
}    // FsFeature::FsFeature


///

FsFeature::~FsFeature()
{}    // FsFeature::~FsFeature


///

void
FsFeature::setFeatureScript( const string& name )
{
    myCplScriptName = name;
}  //setFeatureScript;


///
string
FsFeature::getFeatureScript() const
{
    return myCplScriptName;
}

///
const char* const
FsFeature::name() const
{
    if ( myCplScriptName.size() > 0 )
        return myCplScriptName.c_str();
    else
        return "FsFeature"; 
}    // FsFeature::name


/// Handles the event passed by FsBuilder: passes the event to
/// State::process for further processing and scans through all the states
/// for this feature.
/// If the state returned (nextstate) is the "end" state then the feature is
/// done with this call and the data associated with the call is cleared.
void
FsFeature::process( const Sptr < SipProxyEvent > event )
{
    assert ( event != 0 );

    StateIter iter;
    Sptr < State > nextState;

    Sptr < State > currentState;
    currentState = event->getCallInfo()->getState();

    // If an event has been received earlier for this feature then the
    // state machine is initialized and there is a currentState
    if ( currentState != 0 )
    {
        //cpLog (LOG_DEBUG, "currentState=%s", currentState->name() ); DBG ONLY

        nextState = currentState->process( event );

        //nextState is zero if a state with 0 operators (End state)
        //is executed.  This is an error.    
        if (nextState == 0 ) 
        {
            cpLog( LOG_ERR, "End state executed by stray timer?" ); 
            return;
        } /*if*/

        cleanupCall( event, nextState->name() ); 
    }
    else
    {
        // first event received for this feature
        for ( iter = myStates.begin(); iter != myStates.end(); iter++ )
        {
            //cpLog(LOG_DEBUG, "start state = %s", (*iter)->name() );//DBG ONLY

            nextState = (* iter)->process(event );
            assert ( nextState != 0 );

            cleanupCall( event, nextState->name() ); 

        } /*for*/
    } /*else*/

    return ;
}  //process


///
bool
FsFeature::cleanupCall( const Sptr<SipProxyEvent> event, const char* nextState )
{
    if ( strcmp( nextState, "End" ) == 0 )
    {
        cpLog( LOG_DEBUG, "Tear down all states call." );
        Sptr < SipEvent > sipEvent;
        sipEvent.dynamicCast( event );

        if ( sipEvent != 0 )
            sipEvent->removeCall();
        else
        {
            Sptr < TimerEvent > timerEvent;
            timerEvent.dynamicCast( event );
            if ( timerEvent != 0 )
                timerEvent->removeCall();
            else assert( 0 );
        } /*else*/

        return true; 
    } /*if*/
    else
    {
        return false;
    } /*else*/

} //cleanupCall

//End of file
