#ifndef FSSTATEMACHNDATA_HXX
#define FSSTATEMACHNDATA_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const FsStateMachnDataVersion =
    "$Id: FsStateMachnData.hxx,v 1.2 2002/07/11 21:47:01 veer Exp $";



#include "StateMachnData.hxx"
#include "CPLUrlData.hxx"
#include "InviteMsg.hxx"
#include "StatusMsg.hxx"
#include <vector>

namespace Vocal
{

/**
 *  Data needed by the State Machine that is active only during the length 
 *  of a call.  
 */
class FsStateMachnData: public StateMachnData
{
    public:
        /**@name Class Maintenance */
        //@{
        //Constructor
        FsStateMachnData();

        //Copy Constructor
        FsStateMachnData( const FsStateMachnData& rhs );

        //Destructor
        ~FsStateMachnData();

        //Assignment operator
        FsStateMachnData& operator=( const FsStateMachnData& rhs );

        ///Returns name of class
        const char* name() const;
        //@}

        /**@name Locations Set */
        //@{
        /** Retrieve pointer to locations added by CPL (lookup,location)
          @return Smart pointer to locations set
         */
        Sptr < CPLUrlData > getLocations() const;
        //@}

        /**@name Branch numbers */
        //@{
        /** Get current Branch Number
          @return current branch number  
         */
        Data getBranch() const;

        /** Set current Branch Number
          @param branch New branch number 
         */
        //void setBranch( const int branch );

        /** Increments & saves branch number
          @return the incremented current branch number
         */
        Data getNextBranch();
        //@}

        /**@name Proxy State */
        //@{

        /** Assign value to proxyStatus
          @param proxyStatus New proxy state 
          @see proxyStatus
          */
        void setProxyStatus( const int proxyStatus );

        /** Retrieve proxyStatus value
          @return Current proxy state 
          @see proxyStatus
          */
        int getProxyStatus() const;

        ///Proxy state constants
        enum _proxyState { NoProxy = 0, Proxied, Redirected, NoAnswer, Busy,
                           Failure, Answered, Received3XX, Unknown };

        ///
        typedef _proxyState proxyState;
        //@}

        /**@name Forking Information */
        //@{
        /** Retrieve value to locationBkup
          @return Smart pointer to locationBkup 
          @see locationBkup
          */
        Sptr < CPLUrlData > getLocationBkup() const;

        /** Assign a value to locationBkup member variable
          @param locationBkup The new value to assign
          @see locationBkup 
         **/
        void setLocationBkup( Sptr < CPLUrlData > locationBkup );

        /** Retrieve value to vector of forked requests
          @return Smart pointer to vector of invite messages
          */
        Sptr < vector < Sptr < InviteMsg > > > getOutRequests() const;

        /** Assign value to outRequests
          @param outRequests Old Invite messages
          */
        void setOutRequests( Sptr < vector < Sptr < InviteMsg > > > outRequests );

        /** Retrieve value to incoming Invite request
          @return Smart pointer to incoming request
          */
        Sptr < InviteMsg > getInRequest() const;

        /** Assign value to InRequest
          @param inRequests Incoming invite message 
          */
        void setInRequest( Sptr < InviteMsg > inRequest );

        /** Retrieve best response message
          @return Smart pointer to a Status message 
          @see bestResponse
          */
        Sptr < StatusMsg > getBestResponse() const;

        /** Assign value to bestResponse
          @param bestResponse Status message 
          @see bestResponse
          */
        void setBestResponse( Sptr < StatusMsg > bestResponse );
        //@}

        /**@name Access methods */
        //@{
        ///
        void setOrigFrom( const SipFrom from) ;
        ///
        const SipFrom getOrigFrom() const;
        ///
        void setPrefixCIDBlk( const Data prefix) ;
        ///
        const Data getPrefixCIDBlk() const;
        ///
        void setPrefixTo( const SipTo toWtihPrefix) ;
        ///
        const SipTo getPrefixTo() const;
        ///
        void setNoCIDBlk( const bool noCidBlk) ;
        ///
        bool getNoCIDBlk() const;
        ///
        void setFeatureCode( const string featureCode) ;
        ///
        const string getFeatureCode() const;
        //@}
    private:
        /// Pointer to the container of the location set
        Sptr < CPLUrlData > myUrlCollector;
          
        /// proxy state - used to set the ccDiversion reason
        int myProxyStatus;

        /** branch number - used to keep track of the branch used in the
            previous message and increment it before sending the new message
         **/
        int myBranchNum;
           
        /** During forking, copies of the entries in urlCollector 
          * are stored here.  This is done because as call processing 
          * continues, entries will be deleted from the urlCollector 
          * as each one is used.  In case the original locations are needed, 
          * locationBkup has all the original information. 
          * @see urlCollector
         **/
        Sptr < CPLUrlData > myLocationBkup;
           
        ///Copy of forked requests
        Sptr < vector < Sptr < InviteMsg > > > myOutRequests;
           
        ///Copy of the "original" request (incoming)
        Sptr < InviteMsg > myInRequest;

        /** After a proxy action has completed, there is a "best"
          * response to the call attempt.
         **/
        Sptr < StatusMsg > myBestResponse;

        /** for caller id blocking feature, From field will be modified
          * this is the place to save the orig From
          * this Variable is also acting as a flag to see if caller id blocking
          * is invoked or not. if its size is zero, meaning not.
         **/
        SipFrom myOrigFrom;

        /** The Caller ID Blocking feature using such as *67, *82 prefixes.
          * FS need to remove them for sending out sip request and add them back
          * for responses. The prefix is store in this data member.
          * This data is also an indicator of if there is prefix dialing
         */
        Data myPrefixCIDBlk;
           
        /// The orig To with prefix is saved in this data member
        SipTo myPrefixTo;
             
        /// The flag to indicate not to block caller id
        bool myNoCIDBlk;

        /// The feature code is an indication if there is special feature
        string myFeatureCode;

};

 
}

#endif
