
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const FsUsers_cxx_Version =
    "$Id: FsUsers.cxx,v 1.2 2002/12/12 01:19:37 chok Exp $";
#include "FsUsers.hxx"
#include "ProvisionInterface.hxx"

using namespace Vocal;

///
FsUsers* FsUsers::myInstance = 0;


///
FsUsers*
FsUsers::instance()
{
    if ( myInstance == 0 )
    {
        myInstance = new FsUsers;
        assert( myInstance );
    }
    return myInstance;
}    // FsUsers::instance


///
FsUsers::FsUsers()
{}    // FsUsers::FsUsers


///

FsUsers::~FsUsers()
{
    // erase all entries in map
    myUserMap.erase( myUserMap.begin(), myUserMap.end() ); 

}    // FsUsers::~FsUsers


///

void
FsUsers::addUserData( const FsUserId& user, Sptr < FsUserData > data )
{
    myUserMap.insert( pair < FsUserId, Sptr < FsUserData > > ( user, data ) );
}    // FsUsers::addUserData


///
void
FsUsers::deleteUserData( const FsUserId& user )
{
    UserMapType::iterator iter = myUserMap.find( user );

    if ( iter != myUserMap.end() )
    {
        myUserMap.erase( iter );
    }
    else
    {
        cpLog( LOG_WARNING, "Cannot delete unknown user %s", user.c_str() );
    }
}    // FsUsers::deleteUserData


///
Sptr < FsUserData >
FsUsers::getUserData( FsUserId& user )
{
    Sptr < FsUserData > data = 0;

    UserMapType::iterator iter = myUserMap.find( user );

    if ( iter != myUserMap.end() )
    {
        data = iter->second;
    }
    else
    {
        // Maybe this user is an alias: as FS only uses master, get the
        // master from provisioning 
        cpLog( LOG_DEBUG, "Unknown user '%s'; Try to find the master.", 
                          user.c_str() );

        string userProv = user;

        // convert user to master
        try
        {
            userProv = ProvisionInterface::instance().getMasterUser( userProv );
            cpLog(LOG_DEBUG, "Converting %s to master %s",
                  user.c_str(), userProv.c_str());
        }
        catch( VException &e )
        {
            cpLog(LOG_DEBUG, "Unknown master '%s'.", userProv.c_str());
        }

        // Try to get the user data again from the map
        UserMapType::iterator iter = myUserMap.find( userProv );
        if ( iter != myUserMap.end() )
        {
           data = iter->second;
        }
        else
	    {
          //cpLog( LOG_DEBUG, "Unknown user %s", user.c_str() );
	    }       

    }

    return data;
}    // FsUsers::getUserData


///
void
FsUsers::showUsers()
{
    cpLog( LOG_INFO, "<<< Configured users begin >>>" );

    UserMapType::iterator iter = myUserMap.begin();
    while ( iter != myUserMap.end() )
    {
        string featureInfo;

        if ( iter->second->getUserData( featureInfo ) == false )
            cpLog( LOG_INFO, "User %s: No features.", iter->first.c_str() );
        else
            cpLog( LOG_INFO, "User %s: %s", iter->first.c_str(),
                   featureInfo.c_str() );

        iter++;
    }
    cpLog( LOG_INFO, "<<<  Configured users end  >>>" );
}    // FsUsers::showUsers


//End of File
