#ifndef _MYTIME_HXX
#define _MYTIME_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const MyTime_hxx_Version =
    "$Id$";






#include <time.h>
#include "global.h"

namespace Vocal
{


typedef time_t TimeField;

/** A wrapper class for "struct tm" to extend it with
  *  operators...
  */
class MyTime : public tm
{
    public:
        /**@name Initialization and Deallocation Functions */
        //@{
        //Constructor 
        MyTime();
        //Constructor
        MyTime(const tm&);
        //Constructor 
        MyTime(const time_t&);
        //@}

        //Returns the time as a string
        char* toString();

        /**@name Access functions */
        //@{
        /** Retrieve Year
          *@return time_t is returned with information on year
          */
        TimeField getYear() const;

        /** Retrieve Month
          *@return time_t is returned with information on month
          */
        TimeField getMonth() const;

        /** Retrieve Date
          *@return time_t is returned with information on date
          */
        TimeField getDate() const;

        /** Retrieve WeekDay
          *@return time_t is returned with information on Weekday
          */
        TimeField getWeekDay() const;

        /** Retrieve Time of Day
          *@return time_t is returned with information on time of day
          */
        TimeField getTimeOfDay() const;
        //@}

        ///Is Daylight Savings being used?
        bool isDayLight() const;

        /**@name Store functions */
        //@{
        /** Assign year to member variable
          * @param aVal Year
          *@return current (this) class
          */
        MyTime& setYear(const TimeField aVal);

        /** Assign month to member variable
          * @param aVal month
          *@return current (this) class
          */
        MyTime& setMonth(const TimeField aVal);

        /** Assign date to member variable
          * @param aVal Date
          *@return current (this) class
          */
        MyTime& setDate(const TimeField aVal);

        /** Assign Weekday to member variable
          * @param aVal Weekday
          *@return current (this) class
          */
        MyTime& setWeekDay(const TimeField aVal);

        /** Assign Time of Day to member variable
          * @param aVal Time of Day
          *@return current (this) class
          */
        MyTime& setTimeOfDay(const TimeField aVal);

        /** Assign Daylight flag to member variable
          * @param aVal Daylight flag
          *@return current (this) class
          */
        MyTime& setDayLight(const bool aBool);
        //@}

        /**@name Overridden Operators */
        //@{
        /** Assign time structure values to member variables
          * @param tm Structure that holds the components of a calendar time
          * @return Pointer to this class
          */
        MyTime& operator=(const tm& t);

        /** Assign time structure values to member variables
          * @param time_t Arithmetic type capable of representing time 
          * @return Pointer to this class
          */
        MyTime& operator=(const time_t& t);  // assuming local time!!!

        /** Compare current class with tm structure to determine equality
          * @param tm Structure that holds the components of a calendar time
          * @return True if all time values are equal
          */
        bool operator== (const tm& t);

        /** Is current class' time values greater than tm structure's values?
          * @param tm Structure that holds the components of a calendar time
          * @return True if all class' time is greater than tm's time.
          */
        bool operator> (const tm& t);

        /** Is current class' time values less than tm structure's values?
          * @param tm Structure that holds the components of a calendar time
          * @return True if all class' time is less than tm's time.
          */
        bool operator< (const tm& t);

        /** Is current class' time values greater than equal to
         *tm structure's values?
          * @param tm Structure that holds the components of a calendar time
          * @return True if all class' time is greater than or equal to 
          * tm's time.
          */
        bool operator>= (const tm& t);

        /** Is current class' time values less than/equal to tm structure's values?
          * @param tm Structure that holds the components of a calendar time
          * @return True if all class' time is less than/equal to tm's time.
          */
        bool operator<= (const tm& t);

        /** Compare current class with tm structure to determine inequality
          * @param tm Structure that holds the components of a calendar time
          * @return True if all class' time is unequal to tm's time.
          */
        bool operator!= (const tm& t);
        //@}
};
 
} // namespace Vocal

#endif

