/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.telephony.callcontrol;

import javax.telephony.*;
import javax.telephony.callcontrol.*;
import com.vovida.telephony.*;
import com.vovida.telephony.callcontrol.events.*;

/************************************************************/
/* VCCallControlTerminalConnection is the Vocal Client     */
/* CallControlTerminalConnection Class                      */
/* which is part of the JTapi implementation.               */
/* It implements the CallControl TerminalConnection         */
/* interface from the JTapi specification.                  */
/* This class provides functionality such as Hold, Unhold   */
/* Join and Leave                                           */
/************************************************************/

/**
 * Implementation of the Jtapi CallControlTerminalConnection Interface
 *
 */
public class VCCallControlTerminalConnection extends VCTerminalConnection
    implements CallControlTerminalConnection
{
    int callControlState;

    /**
     * Constructor
     */
    public VCCallControlTerminalConnection(Terminal term, Connection conn)
    {
	super(term, conn);
	try
	{
	    setCallControlState(CallControlTerminalConnection.IDLE);
	}
	catch (InvalidStateException e)
	{
	    e.printStackTrace();
	}
    }

/************************************************************/
/* JTapi Implementation                                     */
/************************************************************/

    // return the CallControl state of the terminal connection
    /**
     * Refer To JTapi CallControlTerminalConnection Interface
     */
    public int getCallControlState()
    {
	return callControlState;
    }


    // this terminalconnection is put on hold
    // by setting the CallControl state from TALKING to HELD
    /**
     * Refer To JTapi CallControlTerminalConnection Interface
     */
    public void hold() throws InvalidStateException,
	MethodNotSupportedException, PrivilegeViolationException,
	    ResourceUnavailableException
    {
	isProviderOK();
	if (this.getCallControlState() != CallControlTerminalConnection.TALKING)
	{
	    throw (new InvalidStateException(this,
			    InvalidStateException.TERMINAL_CONNECTION_OBJECT,
			    this.getCallControlState()));
	}

   	// pre conditions met
	// send a request to the server

	// perform hold operation
	setCallControlState(CallControlTerminalConnection.HELD);
    }


    // this terminalconnection is taken off hold
    // by setting the CallControl state from HELD to TALKING
    /**
     * Refer To JTapi CallControlTerminalConnection Interface
     */
    public void unhold() throws InvalidStateException,
	MethodNotSupportedException, PrivilegeViolationException,
	    ResourceUnavailableException
    {
	isProviderOK();
	if (this.getCallControlState() != CallControlTerminalConnection.HELD)
	{
	    throw (new InvalidStateException(this,
                            InvalidStateException.TERMINAL_CONNECTION_OBJECT,
                            this.getCallControlState()));
	}

        // pre conditions met
        // send a request to the server

        // perform unhold operation
	setCallControlState(CallControlTerminalConnection.TALKING);
    }


    // this terminalconnection is joined to the call
    // by setting the CallControl state from BRIDGED to TALKING
    /**
     * Refer To JTapi CallControlTerminalConnection Interface
     */
    public void join() throws InvalidStateException,
	MethodNotSupportedException, PrivilegeViolationException,
	    ResourceUnavailableException
    {
	isProviderOK();
	if (this.getCallControlState() != CallControlTerminalConnection.BRIDGED)
    	{
            throw (new InvalidStateException(this,
                            InvalidStateException.TERMINAL_CONNECTION_OBJECT,
                            this.getCallControlState()));
        }

        // pre conditions met
        // send a request to the server

	// perform join operation
	setCallControlState(CallControlTerminalConnection.TALKING);
    }


    // this terminalconnection is bridged
    // by setting the CallControl state from TALKING or HELD to BRIDGED
    // however if this terminalconnection is the last active terminalconnection
    // for this address then the terminal connection is Dropped
    // and the associated connection is disconnected
    /**
     * Refer To JTapi CallControlTerminalConnection Interface
     */
    public void leave() throws InvalidStateException,
	MethodNotSupportedException, PrivilegeViolationException,
	    ResourceUnavailableException
    {
	isProviderOK();
	if (this.getCallControlState() != CallControlTerminalConnection.TALKING)
	{
            throw (new InvalidStateException(this,
                            InvalidStateException.TERMINAL_CONNECTION_OBJECT,
                            this.getCallControlState()));
        }

        // pre conditions met
        // send a request to the server
	    
	Connection c = this.getConnection();
	TerminalConnection[] tc = connection.getTerminalConnections();
	boolean otherActiveCalls = false;
	// check if there are other active termconns on this address
	// if so, then the termconn is bridged, if not the connection is dropped
	for (int i=0; i < tc.length; i++)
	{
	    if (tc[i] != this)
	    {
		int ctrlState = ((VCCallControlTerminalConnection)tc[i]).getCallControlState();
		if ((ctrlState == CallControlTerminalConnection.TALKING) || (ctrlState == CallControlTerminalConnection.HELD))
		{
		    otherActiveCalls = true;
	 	}
	    }
	}
	// perform leave operation
	if (otherActiveCalls == true)
	{
	    setCallControlState(CallControlTerminalConnection.BRIDGED);
	}
	else
	{
	    for (int i=0; i < tc.length; i++)
	    {
		((VCCallControlTerminalConnection)tc[i]).setCallControlState
				(CallControlTerminalConnection.DROPPED);
	    }
	    ((VCCallControlConnection)c).setCallControlState
				(CallControlConnection.DISCONNECTED);
	}
    }


    // the answer fn is overridden from the superclass so that
    // the CallControl state can be set accordingly.
    /**
     * Refer To JTapi CallControlTerminalConnection Interface
     */
    public void answer() throws PrivilegeViolationException,
        ResourceUnavailableException, MethodNotSupportedException,
        InvalidStateException
    {
	super.answer();
	setCallControlState(CallControlTerminalConnection.TALKING);
	VCCallControlConnection conn = (VCCallControlConnection)getConnection();
	conn.setCallControlState(CallControlConnection.ESTABLISHED);
    }

/**************************************************************/
/* Additional Functionality                                   */
/**************************************************************/

    // set the CallControl state of the CallControlTerminalConnection.
    // if the state has already reached the newState then
    // then there is no need to set it or send an event so return.
    // the previous state is checked to ensure valid state transition.
    // NOTE: the corresponding TerminalConnection state is set.

    /**
     * Set the CallControl state of the CallControlTerminalConnection
     * If the state has already reached the newState then
     * there is no need to set it or send an event so return
     * The previous state is checked to ensure valid state transition
     */
    public void setCallControlState(int newState) throws InvalidStateException
    {
	System.out.println("CallControl TermConn Set State");
        // INCLUDE SOME EVT HANDLING
	if (newState != CallControlTerminalConnection.IDLE)
	{
	    if (callControlState == newState)
	    {
		return;
	    }
	}
	int state = callControlState;
        switch (newState)
        {
            case CallControlTerminalConnection.IDLE :
                {
		    // set corresponding core state
		    setState(TerminalConnection.IDLE);
                }
                break;
            case CallControlTerminalConnection.RINGING :
                {
		    if (state == CallControlTerminalConnection.IDLE)
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                            new VCCallCtlTermConnRingingEv(this));
			setState(TerminalConnection.RINGING);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
			    InvalidStateException.TERMINAL_CONNECTION_OBJECT,
				state));
		    }
                }
                break;
	    case CallControlTerminalConnection.TALKING :
		{
		    if ((state == CallControlTerminalConnection.IDLE) ||
			(state == CallControlTerminalConnection.RINGING) ||
			(state == CallControlTerminalConnection.BRIDGED) ||
			(state == CallControlTerminalConnection.HELD) ||
			(state == CallControlTerminalConnection.UNKNOWN))
		    {
		        callEvtNotifier.notifyObserversOfEvent(
			    new VCCallCtlTermConnRingingEv(this));
			setState(TerminalConnection.ACTIVE);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                            InvalidStateException.TERMINAL_CONNECTION_OBJECT,
                                state));
		    }
		}	
		break;
            case CallControlTerminalConnection.HELD :
                {
		    if ((state == CallControlTerminalConnection.TALKING) ||
			(state == CallControlTerminalConnection.RINGING) ||
			(state == CallControlTerminalConnection.IDLE) ||
			(state == CallControlTerminalConnection.UNKNOWN))
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                            new VCCallCtlTermConnHeldEv(this));
			setState(TerminalConnection.ACTIVE);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                            InvalidStateException.TERMINAL_CONNECTION_OBJECT,
                                state));
                    }
                }
                break;
            case CallControlTerminalConnection.BRIDGED :
                {
		    if ((state == CallControlTerminalConnection.TALKING) ||
                        (state == CallControlTerminalConnection.RINGING) ||
                        (state == CallControlTerminalConnection.IDLE) ||
                        (state == CallControlTerminalConnection.UNKNOWN) ||
			(state == CallControlTerminalConnection.HELD))
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                            new VCCallCtlTermConnBridgedEv(this));
			setState(TerminalConnection.PASSIVE);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                            InvalidStateException.TERMINAL_CONNECTION_OBJECT,
                                state));
                    }
                }
                break;
            case CallControlTerminalConnection.INUSE :
                {
		    if ((state == CallControlTerminalConnection.TALKING) ||
                        (state == CallControlTerminalConnection.RINGING) ||
                        (state == CallControlTerminalConnection.IDLE) ||
                        (state == CallControlTerminalConnection.UNKNOWN) ||
                        (state == CallControlTerminalConnection.HELD))
                    {
                        callEvtNotifier.notifyObserversOfEvent(
                            new VCCallCtlTermConnInUseEv(this));
			setState(TerminalConnection.PASSIVE);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                            InvalidStateException.TERMINAL_CONNECTION_OBJECT,
                                state));
                    }
                }
                break;
            case CallControlTerminalConnection.DROPPED :
                {
                    callEvtNotifier.notifyObserversOfEvent(
                        new VCCallCtlTermConnDroppedEv(this));
		    if (getConnection().getTerminalConnections().length == 1)
		    {
			Connection conn = getConnection();
			((VCCallControlConnection)conn).setCallControlState(
					CallControlConnection.DISCONNECTED);
		    }
		    setState(TerminalConnection.DROPPED);
                }
                break;
            case CallControlTerminalConnection.UNKNOWN :
                {
		    if (state != CallControlTerminalConnection.DROPPED)
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                            new VCCallCtlTermConnUnknownEv(this));
			setState(TerminalConnection.UNKNOWN);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                            InvalidStateException.TERMINAL_CONNECTION_OBJECT,
                                state));
                    }
                }
                break;
	    default :
		{
		    //throw InvalidStateException
		}
	}
	callControlState = newState;
    }



}


