/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.telephony.privatedata;

import javax.telephony.*;
import javax.telephony.privatedata.*;
import com.vovida.telephony.*;
import java.util.*;


/*********************************************************/
/* VCPrivateData is the Vocal Client PrivateData Class  */
/* which is part                                         */
/* of the JTapi implementation.                          */
/*********************************************************/

/**
 * Implementation of the Jtapi PrivateData Interface
 * VCPrivateData is the Vocal Client PrivateData Class
 * implementation It is being used for Registration 
 * of the client with the server and also to check
 * registration
 */
public class VCPrivateData implements PrivateData, VCMsgTypes
{
    VCProvider prov;

/************************************************************/
/* JTapi Implementation                                     */
/************************************************************/

    /**
     * Refer To JTapi PrivateData Interface
     */
    public void setPrivateData(Object data)
    {
	prov = (VCProvider)data;
    }

    /**
     * Refer To JTapi PrivateData Interface
     */
    public Object getPrivateData()
    {
	return null;
    }


    // Receives an Object (in this case a Vector) which contains
    // the parameters required to perform the specific operation

    /**
     * Refer To JTapi PrivateData Interface
     */
    public Object sendPrivateData(Object data)
    {
	Vector v = (Vector)data;
	if (v.size() > 0)
	{
	    Integer i = (Integer)v.elementAt(0);
	    int reqType = i.intValue();
	    switch (reqType)
	    {
		case REGISTER :
		    {
			// send the register request to the server
			// if the request is performed successfully
			// then the address is obtained from the provider
			// and returned to the app.
			VCProvider.clearProviderDomain();
			String name, password, caller;
			if (v.size() > 3)
			{
			    name = (String)v.elementAt(1);
			    password = (String)v.elementAt(2);
			    caller = (String)v.elementAt(3);
			    if (VCRequestUtil.register(name, password, caller))
			    {
				Address addr = null;
				VCProvider.setProviderDomain(caller, caller);
				try
				{
			 	    addr = prov.getAddress(caller);
				}
				catch (Exception e)
				{
				    e.printStackTrace();
                    return null;
				}
				return addr;
			    }
			    else
			    {
				return null;
			    }
			}
		    }
		    break;
		case CHKREGISTER :
		    {
			// send the chkRegister msg to the server 
			// if it is successful then get the address
			// from the provider and return it to the app.
            // This is based on the fact that the provider has
            // only one address in  this implementation
			if (VCRequestUtil.chkRegister())
			{
			    Address[] addrs = null;
			    try
			    {
			        addrs = prov.getAddresses();
			    }
			    catch (Exception e)
			    {
				    e.printStackTrace();
                    return null;
			    }
			    if (addrs != null)
			    {
				if (addrs.length == 1)
				{
				    return addrs[0];
				}
			    }
			}
			return null;
		    }
		default :
		    {
		    }
		    break;
	    }
	}
	    
	
	return null;
    }

}


