/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.vocalpad;

import javax.telephony.*;
import javax.telephony.events.*;
import com.vovida.telephony.*;
import com.vovida.telephony.events.*;


// Class VCProviderObserver handles basic telephony events for
// the Provider. The associated app is updated accordingly.

/**
 * Class VCProviderObserver handles basic telephony events for
 * the Provider
 */
public class VCProviderObserver implements ProviderObserver
{
    private Japp app;

    /**
     * Constructor
     */
    public VCProviderObserver()
    {
	super();
    }

    /**
     * Refer To JTapi ProviderObserver Interface
     */
    public void providerChangedEvent(ProvEv[] eventList)
    {
	for (int i=0; i < eventList.length; i++)
	{
	    try
	    {
		if (eventList[i].getID() == ProvInServiceEv.ID)
		{
		    System.out.println("Provider is IN_SERVICE");
		}
		else if (eventList[i].getID() == ProvOutOfServiceEv.ID)
		{
		    System.out.println("Provider is OUT OF SERVICE");
		}
		else if (eventList[i].getID() == ProvShutdownEv.ID)
		{
		    System.out.println("Provider is SHUTDOWN");
		}
		else if (eventList[i].getID() == ProvObservationEndedEv.ID)
		{
		    System.out.println("Provider OBSERVATION ENDED");
		}
		else
		{
		    System.out.println("Provider UNKNOWN EVENT");
		}
	    }
	    catch (Exception e)
	    {
		e.printStackTrace();
	    }

	}
    }

    /**
     * Set the application object
     */
    public void setApp(Japp app)
    {
        this.app = app;
    }

    /**
     * Get the application object
     */
    public Japp getApp()
    {
	return this.app;
    }

}

