
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpCalleeOffhook_cxx_Version =
    "$Id$";
#include "JSOpCalleeOffhook.hxx"
#include "SipEvent.hxx"
#include "SipVia.hxx"
#include "StatusMsg.hxx"
#include "JSCall.hxx"
#include "SipSdp.hxx"
#include "SipContact.hxx"
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"



///
JSOpCalleeOffhook::JSOpCalleeOffhook()
{}


///

JSOpCalleeOffhook::~JSOpCalleeOffhook()
{}


///

const char* const
JSOpCalleeOffhook::name() const
{
    return "JSOpCalleeOffhook";
}


///
const Sptr < State >
JSOpCalleeOffhook::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive 200, only handle 200 for INVITE here
    // 2. Check if received as JSD or JSN
    // 3. If JSD, then just forward the message to previous hop
    // 4. If JSN, then create new 200 and send it to caller

    cpLog( LOG_DEBUG, "JSOpCalleeOffhook operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < StatusMsg > status;
    status.dynamicCast( sipMsg );
    if ( status == 0 )
    {
        return 0;
    }

    if ( status->getStatusLine().getStatusCode() != 200 )
    {
        return 0;
    }

    // Check if this is for TRANSFER or INVITE or BYE
    // INVITE   	is handled in JSOpCalleeOffhook (here)
    // TRANSFER & BYE   is handled in JSOpExitControl or JSOpTransferOk
    Data method = sipMsg->getCSeq().getMethod();

    if ( method != "INVITE" )
    {
        return 0;
    }

    Sptr < JSCall > call;
    call.dynamicCast(event->getCallInfo());
    assert( call != 0 );

    JSProvider* provider = call->getProvider();

    // Save the 200 to call object, will be used later for ACK
    call->setTo200( *status );

    int viaPort = sipMsg->getVia(0).getPort();
    cpLog(LOG_DEBUG, "Message received on via port :%d", viaPort );

    // need the udpStack, dest Network Address and request
    // to send a notification to client
    Sptr < UdpStack > clientUdp = provider->getClientUdpStack();

    JSRequest* req = call->getRequest();
    NetworkAddress dest;
    string callId;
    if ( viaPort == provider->getConfig()->getJSDSipPort().convertInt() )
    {
        dest = call->getReceiver();
        callId = req->getRecvClientCallID();
        if (dest.getPort() == -1)
        {
            cpLog( LOG_DEBUG, "JSOpCalleeAlerted port not set - Callee NJS??");
        }
    }
    else if ( viaPort == provider->getConfig()->getJSNSipPort().convertInt() )
    {
        dest = call->getSender();
        callId = req->getClientCallID();
    }
    if (dest.getPort() == -1)
    {
        cpLog( LOG_DEBUG, "JSOpCalleeAlerted port not set - no notifications" );
    }
    else
    {
        JSNotification notification(clientUdp, dest);
        notification.setType(TERMCONNACTIVE);
        notification.setCall(callId);
        notification.setAddr(req->getCallerAddr());
        notification.send();

        // send notification for Callee
        notification.setType(TERMCONNACTIVE);
        notification.setCall(callId);
        notification.setAddr(req->getCalleeAddr());
        notification.send();
    }

    if ( viaPort == provider->getConfig()->getJSDSipPort().convertInt() )	// JSD
    {
        // Forward the message to previous hop
        status->removeVia( 0 );
        cpLog(LOG_DEBUG, "JSOpCalleeOffhook: replay 200 to preivous hop:");
        cpLog(LOG_DEBUG, "***************************************");
        cpLog(LOG_DEBUG, "%s", status->encode().logData());
        cpLog(LOG_DEBUG, "***************************************");
        sipEvent->getSipStack()->sendReply( *status );
    }
    else if ( viaPort == provider->getConfig()->getJSNSipPort().convertInt() ) // JSN
    {
        Sptr < InviteMsg > invite = call->getFromInvite();
        StatusMsg newstatus( *invite, 200/* OK */ );

        // Add contact to the new 200
        Sptr <SipUrl> myUrl = new SipUrl;
        myUrl->setUserValue( provider->getConfig()->getUserName() );
        myUrl->setHost( Data( theSystem.gethostAddress() ) );
        myUrl->setPort( provider->getConfig()->getJSNSipPort() );

        SipContact me;
        me.setUrl( myUrl );
        newstatus.setNumContact( 0 );    // Clear
        newstatus.setContact( me );

        // First get the SDP from 200 received then add it to the new 200
        Sptr<SipSdp> sdp;
	sdp.dynamicCast ( sipMsg->getContentData(0) );
        if (sdp == 0)
        {
            // No remote SDP in 200, this does not affect JS's functionaility
            // It is a violation of SIP protocol, terminate the call by
            // first ACK then BYE.
            cpLog( LOG_ERR,
                   "No remote SDP or invalid SDP in 200, terminate call" );
            //AckMsg ack( *msg );
            //sipEvent->getSipStack()->sendAsync( ack );
            //ByeMsg bye( *msg );
            //sipEvent->getSipStack()->sendAsync( bye );
            // Change state to unknown to terminate this call
            Sptr < JSStateMachine > stateMachine;
            stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
            assert( stateMachine != 0 );

            return stateMachine->findState( "JSStateUnknown" );
        }

        newstatus.flushContentData();
        // newstatus.flushContentType();
        newstatus.setContentData( &(*sdp) );

        cpLog(LOG_DEBUG, "JSOpCalleeOffhook: new 200 to caller:");
        cpLog(LOG_DEBUG, "***************************************");
        cpLog(LOG_DEBUG, "%s", newstatus.encode().logData());
        cpLog(LOG_DEBUG, "***************************************");
        sipEvent->getSipStack()->sendReply( newstatus );
    }
    else
    {
        cpLog(LOG_ERR, "JSOpCalleeOffhook: unknow via port: %d", viaPort );
        // Change state to unknown to terminate this call
        Sptr < JSStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        return stateMachine->findState( "JSStateUnknown" );
    }
    return 0;
}
