
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpTransfer_cxx_Version =
    "$Id$";
#include "JSOpTransfer.hxx"
#include "SipEvent.hxx"
#include "StatusMsg.hxx"
#include "InviteMsg.hxx"
#include "AckMsg.hxx"
#include "ByeMsg.hxx"
#include "TransferMsg.hxx"
#include "SipSdp.hxx"
#include "SdpConnection.hxx"
#include "JSCall.hxx"


///
JSOpTransfer::JSOpTransfer()
{}


///

JSOpTransfer::~JSOpTransfer()
{}


///

const char* const
JSOpTransfer::name() const
{
    return "JSOpTransfer";
}


///
const Sptr < State >
JSOpTransfer::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive 200 for INVITE(hold) from caller
    // 2. Send ACK back to caller
    // 3. Send TRANSFER(callee) to calller

    cpLog( LOG_DEBUG, "JSOpCallerReady operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < StatusMsg > msg;
    msg.dynamicCast( sipMsg );
    if ( msg == 0 )
    {
        return 0;
    }

    if ( msg->getStatusLine().getStatusCode() != 200 )
    {
        return 0;
    }

    Sptr < InviteMsg > invite = sipEvent->getInvite();
    if ( invite == 0 )
    {
        cpLog( LOG_ERR, "No INVITE associated with 200 STATUS" );
        AckMsg ack( *msg );
        sipEvent->getSipStack()->sendAsync( ack );
        ByeMsg bye( *msg );
        sipEvent->getSipStack()->sendAsync( bye );
        // Change state to unknown to terminate this call
        Sptr < JSStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        return stateMachine->findState( "JSStateUnknown" );
    }

    Sptr < JSCall > call;
    call.dynamicCast(event->getCallInfo());
    assert( call != 0 );

    JSProvider* provider = call->getProvider();

    AckMsg ack( *msg );
    ack.setProxyAuth( provider->getConfig()->getProxyAuthKey() );
    cpLog(LOG_DEBUG, "JSOpTransfer: Send ACK for INVITE(hold)");
    cpLog(LOG_DEBUG, "***************************************");
    cpLog(LOG_DEBUG, "%s", ack.encode().logData());
    cpLog(LOG_DEBUG, "***************************************");

    sipEvent->getSipStack()->sendAsync( ack );

    SipVia sipVia;
    sipVia.setprotoVersion( "2.0" );
    sipVia.setHost( Data( theSystem.gethostAddress() ) );
    sipVia.setPort( provider->getConfig()->getJSDSipPort().convertInt() );
    sipVia.setTransport( "UDP" );

    Sptr <SipUrl> transferToUrl = new SipUrl;
    Data user( call->getRequest()->getCalleeAddr() );
    transferToUrl->setUserValue( user );
    transferToUrl->setHost( provider->getConfig()->getRedirectHost() );
    transferToUrl->setPort( provider->getConfig()->getRedirectPort() );
    transferToUrl->setUserParam( "phone" );

    TransferMsg transfer( call->getControl200(), transferToUrl );
    transfer.flushViaList();
    transfer.setVia( sipVia, 0 );

    // Add authorization
    //transfer.setAuthBasic( provider->getConfig()->getProxyAuthKey() );
    transfer.setProxyAuth( provider->getConfig()->getProxyAuthKey() );

    cpLog(LOG_DEBUG, "JSOpTransfer: Send TRANSFER to caller");
    cpLog(LOG_DEBUG, "***************************************");
    cpLog(LOG_DEBUG, "%s", transfer.encode().logData());
    cpLog(LOG_DEBUG, "***************************************");

    sipEvent->getSipStack()->sendAsync( transfer );

    return 0; 	// remain in the Transfer Caller state
}
