#ifndef JSPROXY_HXX
#define JSPROXY_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const JSProxy_hxx_Version =
    "$Id$";


#include <iostream.h>
#include <cassert>
#include "cpLog.h"
#include "Fifo.h"
#include "UdpStack.hxx"

#include "JSProvider.hxx"
#include "JSWorkerThread.hxx"
#include "SipThread.hxx"
#include "JSUserCallContainer.hxx"
#include "JSSipCallContainer.hxx"
#include "CommThread.hxx"
#include "HeartbeatTxThread.hxx"
#include "JSUserRegContainer.hxx"
#include "SipTransceiverFilter.hxx"


/** The only purpose of this class is to spawn all functional threads during
 * start up. This thread itself is instantiated in the main().
 */
class JSProxy
{
    public:
        ///
        JSProxy( const Sptr < JSProvider > provider, unsigned short JSNSipPort = 15060,
                 unsigned short JSDSipPort = 25060);
        ///
        virtual void run();
        ///
        virtual void join();
        ///
        JSProxy( const JSProxy& rhs);
        ///
        JSProxy& operator=( const JSProxy& rhs);
        ///
        virtual ~JSProxy();

    protected:
        ///
        Sptr < JSUserCallContainer > itsUserCallContainer;
        Sptr < JSSipCallContainer > itsSipCallContainer;
        Sptr < JSUserRegContainer > itsUserRegContainer;

        ///
        Sptr < JSProvider > itsProvider;
        ///
        Sptr < Fifo < Sptr < SipProxyEvent > > > itsCallProcessingQueue;
        ///
        ///
        Sptr < SipTransceiverFilter > itsJSNSipStack;
        Sptr < SipTransceiverFilter > itsJSDSipStack;
        ///
        Sptr < JSWorkerThread > itsWorkerThread;
        ///
        Sptr < SipThread > itsJSNSipThread;
        Sptr < SipThread > itsJSDSipThread;
        ///
        Sptr < UdpStack > itsClientUdpStack;
        Sptr < CommThread > itsCommThread;
        ///
        Sptr < UdpStack > itsJSNUdpStack;
        Sptr < HeartbeatTxThread > itsJSNHeartbeatTxThread;
        Sptr < UdpStack > itsJSDUdpStack;
        Sptr < HeartbeatTxThread > itsJSDHeartbeatTxThread;

    private:
        ///
        void JSProxy::copyRhsToThis( const JSProxy& rhs );
};


#endif // JSPROXY_HXX
