
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const js_cxx_Version =
    "$Id$";
#include "js.hxx"
#include "ProvisionInterface.hxx"
#include "BaseCommandLine.hxx"
#include "VNetworkException.hxx"

int
main( int argc, const char** argv )
{
    // JTAPI server needs two sip ports, JSCalling & JSCalled
    // Current commandline only supports one, so we'll intercept
    // the commandline first, then pass the rest to BaseCommandLine.
    // So the required command line for Jtapi server should be
    // $js pservername:pserverport callingPort calledPort

    const char * callingPort = argv[argc - 2];
    const char * calledPort = argv[argc - 1];

    // Usage Check: Make sure ports are specified
    unsigned int i;

    for (i = 0; i < sizeof(callingPort); i++)
    {
        if ( (callingPort[i] < 48) || (callingPort[i] > 57) )
        {
            cpLog(LOG_ERR,
                  "Usage (Requried): js pserverName:pserverPort callingPort calledPort");
            exit( 1 );
        }
    }

    for (i = 0; i < sizeof(calledPort); i++)
    {
        if ( (calledPort[i] < 48) || (calledPort[i] > 57) )
        {
            cpLog(LOG_ERR,
                  "Usage (Requried): js pserverName:pserverPort callingPort calledPort");
            exit( 1 );
        }
    }

    // Take out the extra port sepcified
    argc -= 1;

    BaseCommandData data;

    baseCommandLine(argc, argv, &data);

    Sptr < JSProvider > provider = new JSProvider;
    JSConfig* config = new JSConfig;

    cpLogSetLabel("JTAPI Server");
    int priority = cpLogStrToPriority( (data.debugLevel).c_str() );
    cpLogSetPriority ( priority );

    ProvisionInterface& pif = ProvisionInterface::instance(data.pServer.c_str(), data.pServerPort, data.backupPServer.c_str(), data.backupPServerPort);

    cout << "pServer: " << data.pServer.c_str() << endl;
    cout << "pServerPort: " << data.pServerPort << endl;
    cout << "backupPServer: " << data.backupPServer.c_str() << endl;
    cout << "backupPServerPort: " << data.backupPServerPort << endl;

    list < string > tmpList;
    VRedirectServerData rsData;
    pif.getRedirectServers( tmpList );

    // For now, we'll use the first redirect server on the list
    list < string > ::iterator iter = tmpList.begin();
    try
    {
        pif.getRedirectData( *iter, rsData);
    }
    catch ( VMissingDataException& missing )
    {
        cpLog (LOG_ERR, "*** Caught exception VMissingDataException while getting RS data! ***");
        cpLog (LOG_ERR, "*** JTAPI Server cannot continue ***");
        exit (1);
    }
    catch ( VBadDataException& bad )
    {
        cpLog (LOG_ERR, "*** Caught exception VBadDataException while getting RS data! ***");
        cpLog (LOG_ERR, "*** JTAPI Server cannot continue ***");
        exit (1);
    }

    config->setRedirectHost( rsData.getHost() );
    config->setRedirectPort( rsData.getPort() );

    // Get JTAPI server configuration data
    VJtapiServerData jtapiData;

    // Get the IP address of the server
    string jtapiName( theSystem.gethostAddress() );

    jtapiName += ":";
    jtapiName += callingPort;
    jtapiName += ":";
    jtapiName += calledPort;

    cpLog(LOG_DEBUG, "Jtapi name used to retrieve JS data from PS %s", jtapiName.c_str());

    try
    {
        pif.getJtapiData( jtapiName, jtapiData);
    }
    catch ( VMissingDataException& missing )
    {
        cpLog (LOG_ERR, "*** Caught exception VMissingDataException while pif.getJtapiData! ***");
        cpLog (LOG_ERR, "*** JTAPI Server cannot continue ***");
        exit (1);
    }
    catch ( VBadDataException& bad )
    {
        cpLog (LOG_ERR, "*** Caught exception VBadDataException while pif.getJtapiData! ***");
        cpLog (LOG_ERR, "*** JTAPI Server cannot continue ***");
        exit (1);
    }

    // Get the proxy authorizatino key from VGlobalConfigData
    VGlobalConfigData globalData;

    try
    {
        pif.getConfigData( globalData);
    }
    catch ( VNetworkException& netException )
    {
        cpLog (LOG_ERR, "*** Caught exception VNetworkException while pif.getConfigData! ***");
        cpLog (LOG_ERR, "*** JTAPI Server cannot continue ***");
        exit (1);
    }

    config->setJSNSipPort( jtapiData.getCallingPort() );
    config->setJSDSipPort( jtapiData.getCalledPort() );
    config->setUdpPort( jtapiData.getClientPort() );
    config->setUserName( "js" );
    //config->setBasicCookie( "VovidaClassXswitch" );
    config->setProxyAuthKey( globalData.getproxyAuthKey() );

    provider->setConfig(config);

    cpLog(LOG_DEBUG, "******** CONFIGURATION ********");
    cpLog(LOG_DEBUG, "Provisioning Server     : %s", data.pServer.c_str());
    cpLog(LOG_DEBUG, "Provisioning Server port: %d", data.pServerPort);
    cpLog(LOG_DEBUG, "Redirect Server         : %s", rsData.getHost().c_str());
    cpLog(LOG_DEBUG, "Redirect Server port    : %d", rsData.getPort());
    cpLog(LOG_DEBUG, "Jtapi Server callingport: %d", jtapiData.getCallingPort());
    cpLog(LOG_DEBUG, "Jtapi Server calledport : %d", jtapiData.getCalledPort());
    cpLog(LOG_DEBUG, "JTAPI client port       : %d", jtapiData.getClientPort());
    cpLog(LOG_DEBUG, "Debug level             : %s", data.debugLevel.c_str());
    cpLog(LOG_DEBUG, "******** CONFIGURATION ********");

    cpLog(LOG_DEBUG, "Starting JSProxy");

    JSProxy jsProxy( provider, config->getJSNSipPort().convertInt(), config->getJSDSipPort().convertInt() );
    jsProxy.run();
    {
        for ( ; sleep (0xFFFFFFU) ; );
    }

    return 0;
}

// End of File
