/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const VmBuilder_cxx_Version =
    "$Id: VmBuilder.cxx,v 1.15 2002/09/06 17:17:14 bko Exp $";

#include "VmBuilder.hxx"
#include "SipEvent.hxx"
#include "VmProvisioning.hxx"
#include "InviteMsg.hxx"
#include "FsStateMachnData.hxx"

using namespace Vocal;


VmBuilder::VmBuilder()
{
    VmProvisioning::Instance()->initialize();
    addFeature(VmProvisioning::Instance()->getVmFeature());
}


VmBuilder::~VmBuilder()
{}


/// This is identical to Builder::process() but removeCall() is not called at
/// the end as it is for other applications (the call Info is needed for the
/// life of the call as the fsvm is stateful)

void
VmBuilder::process( const Sptr < Vocal::SipProxyEvent > nextEvent )
{
    assert ( nextEvent != 0 );
    assert ( myCallContainer != 0 );

    cpLog( LOG_DEBUG, "Processing Event." );

    FeatureIter iter;
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( nextEvent );
    Sptr < CallInfo > callInfo;

    if ( sipEvent != 0 )
    {
        cpLog (LOG_DEBUG, "Event is a SIP event." );

        //First search to see if CallInfo exists
        callInfo = myCallContainer->findCall( sipEvent->getSipCallLeg() );
        if ( callInfo == 0)
        {
            cpLog (LOG_DEBUG, "No CallInfo found, so try to create one." );

            Sptr < InviteMsg > inviteMsg;
            inviteMsg.dynamicCast( sipEvent->getSipMsg() );
            if ( inviteMsg != 0 )
            {
                callInfo = myCallContainer->getCall(sipEvent->getSipCallLeg());
                Sptr < FsStateMachnData > callData = new FsStateMachnData;
                callInfo->setCallStateData( callData );
            }
            else
            {
                cpLog (LOG_DEBUG, "Event is not an INVITE, so discard." );
                return ;   //thereby discard handling this event
            }

        }  /*if*/

        assert( callInfo != 0 );
        sipEvent->setCallInfo ( callInfo, myCallContainer );
    }

    // Locking is done in the operators
    for ( iter = myFeatures.begin(); iter != myFeatures.end(); iter++)
    {
        (* iter)->process(nextEvent);
    }

    if ( sipEvent != 0 )
    {
        //Detach CallInfo from SipEvent, otherwise smart pointer for
        //CallInfo will never decrement to zero.
        sipEvent->removeCallInfo();
    }
}  //process

// End of File
