
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const MrshlConfMapEntry_cxx_Version =
    "$Id: MrshlConfMapEntry.cxx,v 1.16 2002/11/12 20:25:22 veer Exp $";

#include "Marshal.hxx"
#include "MrshlOpBye.hxx"
#include "MrshlConfMapEntry.hxx"
#include "ByeMsg.hxx"
#include "CdrInterface.hxx"
#include "MrshlOpStatus.hxx"
#include "VCdrException.hxx"
#include "SipRoute.hxx"
#include "ByeMsg.hxx"

using namespace Vocal;

MrshlConfMapEntry::MrshlConfMapEntry()
        :
        myConfCallParticipantList(),
        myConfBridgeAccessNumber(),
        myLastUserDisconnect( false )
{}



MrshlConfMapEntry::MrshlConfMapEntry( const MrshlConfMapEntry& rhs )
{
    copyRhsToThis( rhs );
}


const MrshlConfMapEntry&
MrshlConfMapEntry::operator=( const MrshlConfMapEntry& rhs )
{
    if (this != &rhs )
    {
        copyRhsToThis ( rhs );
    }

    return *this;
}


const vector < MrshlConfParticipantInfo >&
MrshlConfMapEntry::getConfCallParticipantList() const
{
    return ( myConfCallParticipantList );
}


void
MrshlConfMapEntry::addCallToConf( const MrshlConfParticipantInfo& callInfo )
{
    myConfCallParticipantList.push_back( callInfo );
}


bool
MrshlConfMapEntry::removeCallFromConf( const Data& user )
{
    for( vector<MrshlConfParticipantInfo>::iterator iter =
             myConfCallParticipantList.begin();
         iter != myConfCallParticipantList.end(); iter++ )
    {
        if ( ( *iter ).getUser() == user )
        {
            cpLog( LOG_DEBUG, "Found user %s, removing from participant list",
                   user.logData() );
            myConfCallParticipantList.erase( iter );
            return true;
        }
    }
    
    cpLog( LOG_ERR, "User %s not found in conference", user.logData() );
    return false;
}


void
MrshlConfMapEntry::setStatusMsg( const Data& user, const StatusMsg& statusMsg )
{
    for( vector<MrshlConfParticipantInfo>::iterator iter =
             myConfCallParticipantList.begin();
         iter != myConfCallParticipantList.end(); iter++ )
    {
        if ( ( *iter ).getUser() == user )
        {
            cpLog( LOG_DEBUG, "Setting statusMsg for user %s",
                   user.logData() );
            ( *iter ).setStatusMsg( statusMsg );
            return ;
        }
    }
}


void
MrshlConfMapEntry::copyRhsToThis( const MrshlConfMapEntry& rhs)
{
    myConfCallParticipantList = rhs.myConfCallParticipantList;
    myConfBridgeAccessNumber = rhs.myConfBridgeAccessNumber;
    myLastUserDisconnect = rhs.myLastUserDisconnect;
}


MrshlConfMapEntry::~MrshlConfMapEntry()
{
    myConfCallParticipantList.clear();
    myLastUserDisconnect = false;
    Marshal::instance().accessNumbers().push_back( myConfBridgeAccessNumber.convertString() );
}


void
MrshlConfMapEntry::disconnectLastCall( const Sptr < SipEvent > sipEvent )
{
    // Use the saved invite and status msgs to issue BYEs to both
    // ua and mcu
    MrshlConfParticipantInfo participantInfo =
                             getConfCallParticipantList().front();
    Marshal &mrshl = Marshal::instance();

    // bye to participant
    const InviteMsg& inviteMsg = participantInfo.getInviteMsg();
    if ( &inviteMsg != 0)
    {
        cpLog( LOG_DEBUG, "BYE to ua" );
        SipVia via;
        ByeMsg uaBye( inviteMsg, via, inviteMsg.getCSeq() );

        // flush the via list and add the host info only
        uaBye.flushViaList();
	mrshl.addViaWithBranch( uaBye, Data(MRSHL_INCOMING_CONTACT) );

        // add ProxyAuthorization key
        cpLog( LOG_DEBUG, "Add proxy authorization key" );
        uaBye.setAuthBasic( Marshal::instance().proxyAuthKey() );

        // set CSeq to 101 ( arbitrary, can be whatever we want )
        SipCSeq sipCSeq = uaBye.getCSeq();
        Data sipCSeqData( "101" );
        sipCSeq.setCSeqData( sipCSeqData );
        uaBye.setCSeq( sipCSeq );

        // get sipUrl from last item in route list
        SipRoute lastRoute( uaBye.getRoute( -1 ) );
        SipRequestLine byeRequestLine( SIP_BYE, lastRoute.getUrl() );
        uaBye.removeRoute( -1 );

        // set the request line and send msg
        uaBye.setRequestLine( byeRequestLine );
        sipEvent->getSipStack()->sendAsync( uaBye );
    }
    else
    {
        cpLog( LOG_ERR , "No INVITE message for participant" );
    }

    // bye to mcu
    const StatusMsg& statusMsg = participantInfo.getStatusMsg();
    if ( &statusMsg != 0 )
    {
        cpLog( LOG_DEBUG, "BYE to mcu" );
        ByeMsg mcuBye( statusMsg );

        // flush the via list and add the host info only
        mcuBye.flushViaList();
	mrshl.addViaWithBranch( mcuBye, Data(MRSHL_OUTGOING_CONTACT) ); 

        // flush the Record Route List (not needed)
        mcuBye.flushrouteList();

        // get sipUrl from contact in status msg
        Sptr<SipContact> contact;
        contact = MrshlOpStatus::getContactFromList( statusMsg.getContactList(),
                  true );
        SipRequestLine byeRequestLine( SIP_BYE, contact->getUrl() );

        // set the request line and send msg
        mcuBye.setRequestLine( byeRequestLine );
        sipEvent->getSipStack()->sendAsync( mcuBye );
    }
    else
    {
        cpLog( LOG_ERR , "No status message for BYE" );
    }

    // set flag to indicate termination process has started
    myLastUserDisconnect = true;

    // remove call from conference list
    removeCallFromConf( participantInfo.getUser() );
    
    // stop billing
    cpLog( LOG_INFO, "Stop billing From: %s, To: %s, CallId: %s",
           inviteMsg.getFrom().getUser().logData(),
           inviteMsg.getTo().getUser().logData(),
           inviteMsg.getCallId().getCallIdData().logData() );
    try
    {
        CdrInterface::instance().callEnded( inviteMsg.getFrom().getUser().convertString(),
                                 inviteMsg.getTo().getUser().convertString(),
                                 inviteMsg.getCallId().getCallIdData().convertString() );
    }
    catch (VCdrException&e)
    {
        cpLog( LOG_ERR, "WARNING : Cannot send record to CDR Server ");
    }
}

// End of File

