/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const MrshlOpRegister_cxx_Version =
    "$Id: MrshlOpRegister.cxx,v 1.59 2002/11/13 02:39:19 veer Exp $";


#include "Marshal.hxx"
#include "MrshlOpRegister.hxx"
#include "MrshlUserCache.hxx"
#include "RegisterMsg.hxx"
#include "RoundRobinServers.hxx"
#include "SipContact.hxx"
#include "SipEvent.hxx"
#include "StatusMsg.hxx"
#include "SystemInfo.hxx"

using namespace Vocal;

MrshlOpRegister::MrshlOpRegister()
    : Operator()
{
}


MrshlOpRegister::~MrshlOpRegister()
{
}


const char* const
MrshlOpRegister::name() const
{
    return "MrshlOpRegister";
}


const Sptr < State >
MrshlOpRegister::process(const Sptr < SipProxyEvent > event )
{
    Marshal &mrshl = Marshal::instance();

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );

    // return if not a valid event
    if ( sipEvent == 0 )
    {
        cpLog( LOG_ERR, "process: SipEvent = 0" );
        return PROXY_DONE_WITH_EVENT;
    }

    // return if not a Register message
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    Sptr < RegisterMsg > registerMsg;
    registerMsg.dynamicCast( sipMsg );
    if ( registerMsg == 0 )
    {
        return PROXY_CONTINUE;
    }

    // log reception of Register message
    doCpLog();
    cpLog( LOG_DEBUG_STACK, "SIP message received:\n%s\n%s,",
	   sipMsg->getReceivedIPName().logData(), sipMsg->encode().logData() );
    
    // verify the toUrl
    Sptr < SipUrl > toSipUrl;
    toSipUrl.dynamicCast( registerMsg->getTo().getUrl() );
    if ( toSipUrl == 0 )
    {
        cpLog( LOG_ERR, "process: toSipUrl = 0" );
	return PROXY_DONE_WITH_EVENT;
    }
        
    // marshal the message
    unsigned int mrshlStatus = mrshl.marshalIncomingCommand( registerMsg );
    if ( mrshlStatus == MRSHL_STATUS_INCOMING )
    {
    	// return 100 to sender
	StatusMsg statusMsg( *registerMsg, 100 );
	sipEvent->getSipStack()->sendReply( statusMsg );
    
        // add self to Via list
        cpLog( LOG_DEBUG, "add self to via ..." );
        Marshal::instance().addViaWithBranch( *registerMsg,
					      Data(MRSHL_INCOMING_REDIRECT) );
        // add self to Contact list
    	//If it is fetch Register message, just pass it along
        if(registerMsg->getNumContact())
	{
            cpLog( LOG_DEBUG, "add self to contact list ..." );
            Sptr < SipUrl > myUrl = new SipUrl;
            myUrl->setUser( toSipUrl->getUser() );
            myUrl->setHost( theSystem.gethostAddress() );
            myUrl->setPort( Data(mrshl.sipPort()) );
            SipContact myContact;
	    myContact.setUrl( myUrl );
            registerMsg->setContact( myContact, 0 );
	}

        // set RequestUri to redirect server and send thru SIP stack
        cpLog( LOG_DEBUG, "send to redirect server ..." );

	Sptr < SipUrl > requestSipUrl;
	requestSipUrl.dynamicCast( registerMsg->getRequestLine().getUrl() );
	if ( requestSipUrl == 0 )
	{
	    cpLog( LOG_ERR, "process: requestSipUrl = 0" );
	    return PROXY_DONE_WITH_EVENT;
	}

        RoundRobinServers& rServers = RoundRobinServers::instance( SERVER_RS );
        Data lHost;
        int  lPort;
        rServers.getHostNPort(sipMsg->getFrom().getUrl()->encode(), lHost, lPort);
        requestSipUrl->setHost( lHost );
        requestSipUrl->setPort( Data(lPort) );

        SipRequestLine requestLine( registerMsg->getRequestLine() );
        requestLine.setUrl( requestSipUrl );
        registerMsg->setRequestLine( requestLine );
	 cpLog( LOG_DEBUG_STACK, "SIP message sent:\n %s:%d\n%s,",
		lHost.logData(), lPort, registerMsg->encode().logData());
        sipEvent->getSipStack()->sendAsync( *registerMsg );
    }
    else // mrshlStatus == 401
    {
        cpLog( LOG_INFO, "Try to Send Right Status Msg 401,407, 403, 404");
	Sptr<StatusMsg> statusMsg;
	if (mrshlStatus == 401)
	{
	    statusMsg = 
		mrshl.generateChallenge(registerMsg,
					401,
					Marshal::CHALLENGE_WWW_AUTH);
	    sipEvent->getSipStack()-> sendReply( *statusMsg);
	    cpLog( LOG_DEBUG_STACK, "SIP message sent:\n %s\n%s,",
		  registerMsg->getFrom().getHost().logData(), statusMsg->encode().logData() );
	    cpLog(LOG_DEBUG, 
		 "\n\nMY RESPONSE to REGISTER Since Authentication turned On is =  [ 401 Unauthorized ]\n\n\n");
	    
	} else if(mrshlStatus == 407){
	    statusMsg = 
		mrshl.generateChallenge(registerMsg,
					407,
					Marshal::CHALLENGE_PROXY_AUTH);
	    cpLog( LOG_DEBUG_STACK, "SIP message sent:\n %s\n%s,",
		    registerMsg->getFrom().getHost().logData(), statusMsg->encode().logData() );
	    cpLog(LOG_DEBUG, 
		 "\n\nMY RESPONSE to REGISTER Since Authentication turned On is =  [ 407 Unauthorized ]\n\n\n");
	    sipEvent->getSipStack()-> sendReply( *statusMsg);
	    
	}else if(mrshlStatus == 403){
	    
	    StatusMsg statMsg(*registerMsg, 403);
	    cpLog( LOG_DEBUG_STACK, "SIP message sent:\n %s\n%s,",
		    registerMsg->getTo().getHost().logData(), statMsg.encode().logData() );
	    cpLog(LOG_DEBUG, 
		 "\n\nMY RESPONSE to REGISTER Since Authentication turned On is =  [ 403 Unauthorized ]\n\n\n");
	    sipEvent->getSipStack()-> sendReply( statMsg);
	
	}else {
	    StatusMsg statMsg(*registerMsg, 400);
	     cpLog( LOG_DEBUG_STACK, "SIP message sent:\n %s\n%s,",
		    registerMsg->getTo().getHost().logData(), statMsg.encode().logData() );
	    cpLog(LOG_DEBUG,
		 "\n\nMY RESPONSE to REGISTER Since SOMETHING BAD HAPPENED... :( =  [ 400 ERROR ]\n\n\n");
	    sipEvent->getSipStack()-> sendReply( statMsg);
	    
	}
     
    }

    cpLog( LOG_INFO, "Done with REGISTER." );
    return PROXY_DONE_WITH_EVENT;
}


// End of File
