
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const MrshlState_cxx_Version =
    "$Id: MrshlState.cxx,v 1.23 2002/11/09 02:08:07 derekm Exp $";

#include "MrshlState.hxx"
#include "MrshlOpAck.hxx"
#include "MrshlOpBye.hxx"
#include "MrshlOpCancel.hxx"
#include "MrshlOpInvite.hxx"
#include "MrshlOpOptions.hxx"
#include "MrshlOpRegister.hxx"
#include "MrshlOpStatus.hxx"
#include "MrshlOpProxy.hxx"
#include "MrshlOpSubNotify.hxx"

using namespace Vocal;

MrshlState::MrshlState()
{
    // add operators for each expected message type
    /* note, the ordering of the operators is important in that
     * MrshlOpProxy() must be the last operator such that it 
     * serves as a default operator to proxy all sip commands
     * which are not handled by any previous operators.
     */
    Sptr < MrshlOpAck > opAck           = new MrshlOpAck;
    addOperator( opAck );

    Sptr < MrshlOpBye > opBye           = new MrshlOpBye;
    addOperator( opBye );

    Sptr < MrshlOpCancel > opCancel     = new MrshlOpCancel;
    addOperator( opCancel );

    Sptr < MrshlOpInvite > opInvite     = new MrshlOpInvite;
    addOperator( opInvite );

    Sptr < MrshlOpSubNotify > opSubNotify     = new MrshlOpSubNotify;
    addOperator( opSubNotify );

    Sptr < MrshlOpOptions > opOptions     = new MrshlOpOptions;
    addOperator( opOptions );

    Sptr < MrshlOpRegister > opRegister = new MrshlOpRegister;
    addOperator( opRegister );

    Sptr < MrshlOpStatus > opStatus     = new MrshlOpStatus;
    addOperator( opStatus );

    Sptr < MrshlOpProxy > opProxy       = new MrshlOpProxy;
    addOperator( opProxy );    
}


MrshlState::~MrshlState()
{}



const char* const
MrshlState::name() const
{
    return "MrshlState";
}

// End of File
