
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const ms_cxx_Version =
    "$Id$";

#include <signal.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>

#include "Marshal.hxx"
#include "VException.hxx"
#include "BaseCommandLine.hxx"
#include "ProvisionInterface.hxx"
#include "cpLog.h"

using namespace Vocal;

/**
 * Handler for signal SIGABRT, this needs to be here
 * so that the abort signal is caught when a TCP
 * connection is terminated by the other end.
 * @param signo the SIGABRT number
 * @return void
 */
void
signalHandler( int signo )
{
    if ( signo == SIGABRT )
    {
        // nothing to be done
    }

}


#if 0
void segv(int a)
{
    struct rlimit rl;
    if(fork()==0) abort();
    rl.rlim_cur=0;
    rl.rlim_max=0;
    setrlimit(RLIMIT_CORE, &rl);
    getrlimit(RLIMIT_CORE, &rl);
    rl.rlim_cur=0;
    setrlimit(RLIMIT_CORE, &rl);
    chdir("/");
    chdir("/proc");
    abort();
}
#endif



/**
 * Marshal entry function; receives command line parameters,
 * calls parser, sets provisioning up, creates and runs marshal
 * instance
 * @param argc the number of command line arguments
 * @param argv pointer to the list of command line arguments
 * @return -1 on abort of program
 */
int
main( const int argc, const char** argv )
{
    cpLogSetLabel( "Mrshl" );
    // setup SIGABRT handler, for the tcp connection
    if ( signal( SIGABRT, signalHandler ) == SIG_ERR )
    {
	cpLog(LOG_ALERT, "could not set signal handler for SIGABRT, exiting");
        exit( -1 );
    }

#if 0
    if ( signal( SIGSEGV, segv ) == SIG_ERR )
    {
	cpLog(LOG_ALERT, "could not set signal handler for SIGSEGV, exiting");
        exit( -1 );
    }
#endif

    BaseCommandData data;
    baseCommandLine( argc, argv, &data );

    try
    {
        cpLog(LOG_DEBUG, "****** Pserver host: %s", data.pServer.c_str());
        // setup provisioning
        ProvisionInterface::instance( data.pServer.c_str(),
                                      data.pServerPort,
                                      data.backupPServer.c_str(),
                                      data.backupPServerPort,
				      data.readSecretFile.c_str(),
				      data.writeSecretFile.c_str()
				      );

        Marshal::instance( data.sipPort );
        Marshal::instance().showMrshlConfig();
        Marshal::instance().run();
    }
    catch ( VException&e )
    {
        cpLog( LOG_ALERT, "Application exiting, reason:%s", e.getDescription().c_str() );
    }
    catch ( ... )
    {
        cpLog( LOG_ALERT, "Application exiting, caught unknown exception" );
    }
}

// End of File

/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
