#ifndef ClientSession_H
#define ClientSession_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const ClientSession_hxx_Version =
    "$Id$";

#include "NetworkAddress.h"
#include "Tcp_ServerSocket.hxx"
#include "Tcp_ClientSocket.hxx"
#include "Connection.hxx"
#include "Http_def.h"
#include "cpLog.h"
#include "Sptr.hxx"
#include "ThreadIf.hxx"

class ClientSession: public ThreadIf
{
    public:
        /**
         * set the connection
         * @param conn connection
         */
        void setConnection(Connection& conn);
        ///
        ClientSession();
        ///
        bool toBeDeleted(){
            return mDelete;
        };
        virtual ~ClientSession();   // the destructor will "delete this"

    protected:

        /**
         * infinate loop that will handle in incomming requeses from the gui
         *  current requests are
         *  CONFIG_REQ,<num>
         *  STOP_REQ,<hostname:port>,<serverType>
         *  START_REQ,<hostname:port>,<serverType>
         *  
         *  CONFIG_REQ returns the latest system configureation in html
         *  format to the caller (gui).  If the sequence number submitted
         *  is the same as the current number, an ack is returned
         *  
         *  STOP_REQ will do a killall -9 on the hostname for the
         *  serverType identified.
         *
         *  START_REQ will start a server process on the 
         *  hostname and port identified by the caller.
         */
        virtual void thread();
    private:

        ///
        Connection myConnection;
        
    public:
        static void sigTerm(int signo);
        static void sigPipe(int signo);
        
    private:

        /**
         * process the stop request from the GUI
         * "snmpset %s private .1.3.6.1.4.1.5738.1.1.1.3.0 a %s"
         * "snmpset %s private .1.3.6.1.4.1.5738.1.1.1.4.0 i 1"
         * This function will set the localGrpStartProcessName to "vocalstart stop"
         * then set the localGrpStartProcess to 1 to execute the localGrpStartProcessName.
         * It will then reset the localGrpStartProcessName back to blank.
         * %s4 = "command" or 1  (thing to set it to)
         * @param hostnamePort the hostname and port. format hostname:port
         * @param serverType the type of server (fs/ms/etc..)
         * @param conn connection
         */


        /**
         * process the stop request from the GUI
         * "snmpset %s private .1.3.6.1.4.1.5738.1.1.1.1.0 a %s"
         * "snmpset %s private .1.3.6.1.4.1.5738.1.1.1.2.0 i 1"
         * This function will set the localGrpStartProcessName to "vocalstart start"
         * then set the localGrpStartProcess to 1 to execute the localGrpStartProcessName.
         * It will then reset the localGrpStartProcessName back to blank.
         * %s4 = "command" or 1  (thing to set it to)
         * @param hostnamePort the hostname and port. format hostname:port
         * @param serverType the type of server (fs/ms/etc..)
         * @param conn connection
         */
        void handleRequest(string& hostnamePort, string& serverType, Connection& conn,string& type);


        /**
         * handle the config_req from the gui.  send the response back to the gui
         * @param uiVerNum the configuration version the ui has
         * @param conn connection
         */
        void handleConfigReq(int uiVerNum, Connection& conn);

        /**
         * return success/failure status to the gui
         * @param success success/fail bool
         * @param conn connection
         */
        void returnStatus(bool success, Connection& conn);

        /**
         * creates a string with the xml header.  The remainder of the 
         * xml will be done elsewhere
         * @param success success/fail bool
         * @param contLength length of the entire message
         * @return string of the header.  It would be tagged onto the top
         * of the remainder of the message
         */
        string makeHTTPHeader(int success, int contLength);
        bool mDelete;
};

#endif //ClientSession_H
