
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const NetMgntAgent_cxx_Version =
    "$Id: NetMgntAgent.cxx,v 1.21 2001/07/20 23:14:38 eoin Exp $";
#include "NetMgntAgent.hxx"
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

ServerType setServerTypeEnum(string& serverTypeStr);

Sptr < NetMgntAgent > NetMgntAgent::theNetMgntAgent = 0;

/// Return the one NetMgntAgent or create it
/// if it doesn't already exist.
Sptr < NetMgntAgent >
NetMgntAgent::instance()
{
    if ( theNetMgntAgent == 0 )
    {
        theNetMgntAgent = new NetMgntAgent;
    }
    return theNetMgntAgent;
}

NetMgntAgent::NetMgntAgent() : AgentApi(SERVER_NetMgnt)
{
}


voReturnStatus
NetMgntAgent::processMessage(ipcMessage *message, NetworkAddress *sender)
{
    struct in_addr tmpIn_addr;
    struct hostent *tmpHostent = 0;
    uint pos1, pos2;
    string hostPort, portStr, serverTypeStr, ipStr, tmpStr;

    assert(message);
    assert(sender);

    switch (message->action)
    {
        case Trap :
        {
            switch (message->mibVariable)
            {
                case ServerUp :
                case ServerDown :
                {
                    ServerType serverTypeEnum;
                    // the format of this message is MS:192.168.05.140:5060
                    // or SERVERTYPE:IP:PORT
                    // need to change IP to Hostname

                    tmpStr = message->parm1;
                    pos1 = tmpStr.find(":");
                    pos2 = tmpStr.rfind(":");

                    if (pos1 <= 0 || pos2 == pos1)
                    {
                        cpLog (LOG_ERR, "received invalid trap string");
                        return voFailure;
                    }

                    serverTypeStr = tmpStr.substr(0, pos1);
                    ipStr = tmpStr.substr(pos1 + 1, pos2 - pos1 - 1);
                    portStr = tmpStr.substr(pos2 + 1, tmpStr.length() - pos2);

                    tmpIn_addr.s_addr = inet_addr(ipStr.c_str());
                    tmpHostent = gethostbyaddr((char *) & tmpIn_addr, sizeof(struct in_addr), AF_INET);
                    if (!tmpHostent)
                    {
                        cpLog (LOG_ERR, "received invalid trap IP");
                        return voFailure;
                    }

                    hostPort = tmpHostent->h_name;
                    hostPort.append(":");
                    hostPort.append(portStr);


                    serverTypeEnum = setServerTypeEnum(serverTypeStr);
                    if(serverTypeEnum == SERVER_Unknown)
                        return voFailure;
                    SystemConfig::instance()->ProcessTrap(hostPort, serverTypeEnum,
                                                          ((message->mibVariable == ServerUp) ? Active : Inactive));
                    return voSuccess;
                }
                default :
                {
                    return voUnknown;
                }
            }
        }
        case Get :
        case Set :
        case Response :
        case Register :
        default :
        {
            return voFailure;
        }
    }
}

ServerType setServerTypeEnum(string& serverTypeStr)
{
    if (serverTypeStr == "FS")
        return SERVER_FS;
    else
        if (serverTypeStr == "MS")
            return SERVER_MS;
        else
            if(serverTypeStr == "RS")
                return SERVER_RS;
            else
                if (serverTypeStr == "JS")
                    return SERVER_JS;
                else
                    if (serverTypeStr == "POS")
                        return SERVER_POS;
                    else
                        if (serverTypeStr == "CDR")
                            return SERVER_CDR;
                        else
                            return SERVER_Unknown;
}
