#ifndef STATUSPOLLER_H
#define STATUSPOLLER_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const StatusPoller_hxx_Version =
    "$Id: StatusPoller.hxx,v 1.10 2000/12/18 23:44:38 bko Exp $";

#include <vector>
#include "Common.h"
#include "Sptr.hxx"
#include "NetworkAddress.h"
#include "ThreadIf.hxx"
#include "SystemConfig.hxx"
#include "BaseCommandLine.hxx"

typedef vector < NetworkAddress > hsListT;
typedef hsListT::iterator hsListIter;

class StatusPoller: public ThreadIf
{
    public:
        ///
        StatusPoller(int inPollInt, BaseCommandData& inData);

        ///
        virtual ~StatusPoller();

        /**
         * change the netMgnt's polling interval
         * @param inPollInt new interval in seconds
         */
        void updatePollingInterval (int inPollInt);

    protected:
        /**
         * periodically poll the heartbeat server for the system status
         * get a list of the heartbeat servers from the ps and register to receive traps.
         * then, every polling period, clear the trap list, re-register to receive traps
         * and get the full list of servers and their status
         */
        virtual void thread();

    private:
        int pollingIntervalSec;
        hsListT hsList;
        BaseCommandData pServerData;

        /**
         * place holder function to free the system configuration list
         * @param inList the list to free
         * @return true on success false on failure
         */
        voReturnStatus FreeSystemConfigList( configListT& inList);


        /**
         * get the list of heartbeat servers from the provisioning server
         * @return a vector of the heartbear servers
         */
        hsListT getHsList();


        /**
         *  This function will poll the heartbeat server for the latest system configuration.
         *  it will use "snmp getnext 1,3,6,1,4,1,5738,1,1,3,1,1,1,0" through the serverTable
         *  this result will be compaired with the number of servers from 
         *  "snmp get 1,3,6,1,4,1,5738,1,1,5,1,0".  If the entire table is read then it will be
         *  processed.  if not, it will try to read again.
         * @param reqAddr the network address of the heartbeat server
         * @return a vector of server configurations
         */
        configListT pollSystemStatus(const NetworkAddress& reqAddr);
};

#endif // STATUSPOLLER_H
