#ifndef RSTIMEREVENT_HXX
#define RSTIMEREVENT_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "RSCommon.h"
#include "TimerEvent.hxx"

static const char* const RSTimerEventVersion = "$Id$";

/**
 * RSTimerEvent is a TimerEvent.  This event is posted when a timer goes off.
 */

class RSTimerEvent : public Vocal::TimerEvent
{
    public:

        /**
         * Constructor 
         * @param  outputFifo  Fifo for this event
         * @param  timerEventType  Type of timer event
         */
        RSTimerEvent( const Sptr < Vocal::Fifo < Sptr < Vocal::SipProxyEvent > > > outputFifo, 
                      const TimerEventType eventType );
      
        /// Destructor
        virtual ~RSTimerEvent();

        /// Obtain the name of this object
        virtual const char* const name() const
        {
            return "RSTimerEvent";
        }

        /**
         * Obtain the event type
         * @return  The timer event type
         */
        TimerEventType getTimerEventType() const;

        /**
         * Store the timer event type
         * @param  eventType  Timer event type 
         */
        void setTimerEventType( const TimerEventType eventType);

        /**
         * Obtain the data info
         * @return  Timer data
         */
        Data getData() const
        {
            return itsData;
        }

        /**
         * Store the data info
         * @param  data  Store the Timer object's data
         */
        void setData(const Data data)
        {
            itsData = data;
        }

        /**
         * Obtain the flag info
         * @return  Timer object's flag
         */
        bool getFlag() const 
        {
            return itsFlag;
        }

        /**
         * Store the flag info
         * @param  flag  Timer flag
         */
        void setFlag(const bool flag)
        {
            itsFlag = flag;
        }

        /**
         * Obtain the count info
         * @return  Timer count 
         */
        unsigned int getCount() const 
        {
            return itsCounter;
        }

        /**
         * Store the count info
         * @param  count  Timer's count info
         */
        void setCount(const bool count)
        {
            itsCounter = count;
        }

    private:
        /// Type of timer event
        TimerEventType itsTimerEventType;

        /// Timer flag
        bool itsFlag;

        /// Timer data
        Data itsData;

        /// Timer counter
        unsigned short itsCounter;
};

#endif //TIMEREVENT_HXX
