#ifndef SERVERWITHGROUP_HXX
#define SERVERWITHGROUP_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include <vector>
#include "Data.hxx"
#include "Sptr.hxx"
#include "Server.hxx"

static const char* const ServerWithGroupVersion = "$Id: ServerWithGroup.hxx,v 1.11 2001/02/08 01:16:35 marcl Exp $";

/**
 *  This class expands on the base class of Server by
 *  adding the concept of groups. Each Server can belong
 *  to a group or groups. These group names are stored 
 *  within this class as a list.
 */
class ServerWithGroup : public Server
{
    public:
        /**
         * Constructor
         * @param  serverType  Type of server
         */
        ServerWithGroup( const ServerType serverType);

        /**
         * Copy Constructor
         * @param  rhs  Object from which to copy
         */
        ServerWithGroup( const ServerWithGroup& rhs );

        /**
         * Operator overload for "="
         * @param  rhs  Object from which to copy
         * @return  Reference to this object with new data
         */
        ServerWithGroup& operator=( const ServerWithGroup& rhs );

        /// Destructor
        virtual ~ServerWithGroup();

        /**
         * Obtain the group data
         * @param  groups  Structure to hold the groups to be returned
         */
        void getGroups(vector < Data > & groups) const;

        /**
         * Store the group data
         * @param  groups  Groups to be stored
         */
        void setGroups( const Sptr < vector < Data > > & groups);

        /**
         * Process a heartbeat from a specific server
         * @param  key  Server who sent the heartbeat
         */
        void processHeartbeat( const Data& key);

        /**
         * Check to see if server is still active. Mark a server as no longer
         * active if it has reached it's max missed heartbeats.
         * @param  serverId  Server on which to perform housekeeping
         */
        void doHouseKeeping( const Data& serverId);

        /**
         * Perform actions to mark a server as no longer active.
         * @param  serverId  Server to be marked inactive
         */
        void processServerDown( const Data& serverId);

    protected:
        /**
         * Process the initial heartbeat from a server by adding it to the active list
         * @param  key  Server who sent the heartbeat
         */
        void gotFirstHeartbeat( const Data& key);

    private:
        /// list of groups this server belongs to
        Sptr < vector < Data > > itsGroups;

        /**
         * Copy member data from given object into this one
         * @param  rhs  Object from which to copy
         */
        void copyRhsToThis( const ServerWithGroup& rhs );
};

#endif //SERVERWITHGROUP_HXX
