
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const rs_cxx_Version =
    "$Id: rs.cxx,v 1.50.2.2 2003/01/23 02:51:46 sprajpat Exp $";

#include "BaseCommandLine.hxx"
#include "DialPlanContainer.hxx"
#include "IPPlanContainer.hxx"
#include "ProvisionInterface.hxx"
#include "RSBuilder.hxx"
#include "RSProxy.hxx"
#include "SubscriberContainer.hxx"

#include "Serialize.hxx"

time_t lastRegForUpdateTime=0;

extern int setUpRegistrationForUpdate(bool force);

int
main( int argc, const char** argv )
{
    /// Initialize the Sip Port

    BaseCommandData commandLine;

    baseCommandLine(argc, argv, &commandLine);

    unsigned short localSipPort = commandLine.sipPort;

    cpLog( LOG_DEBUG, "RS Start");

    /// Set up the Subscriber Container to store the local sip port
    SubscriberContainer::instance().setPort(localSipPort);

    ProvisionInterface::instance(
	commandLine.pServer.c_str(), 
	commandLine.pServerPort, 
	commandLine.backupPServer.c_str(), 
	commandLine.backupPServerPort,
	commandLine.readSecretFile.c_str(),
	commandLine.writeSecretFile.c_str()
	);

    /// Obtain various data from the Provisioning server
    setUpRegistrationForUpdate(true);
    lastRegForUpdateTime= time(0);

    cpLog (LOG_DEBUG, "Local Sip Port: %d", localSipPort);

    Sptr < RSBuilder > builder = new RSBuilder;
    RSProxy & rsProxy = RSProxy::instance( builder, localSipPort );
    SubscriberContainer::instance().setProxy(rsProxy);

    if ( strcmp( commandLine.debugLevel.c_str(), "LOG_DEBUG_HB" ) == 0 )
    {
        RSServerContainer::instance().turnHeartbeatDebugOn();
    }

    SubscriberContainer::instance().updateAllSubscribers();

    #ifdef SERIALIZE	 // defined in vocal/util/io/Serialize.hxx
       Vocal::IO::Serialize::singleton("/usr/local/vocal/bin/persist-register.py", "w");
    #endif

    rsProxy.run();
    rsProxy.join();

    return 0;
}

int setUpRegistrationForUpdate(bool force)
{
    time_t cTime = time(0);
    cpLog(LOG_DEBUG, "Updating registration entries");

    //If the registration request is coming earlier than 5 min
    //ignore
    if((lastRegForUpdateTime > 0 ) && !force && 
       ((cTime - lastRegForUpdateTime) < 300))
    {
        cpLog(LOG_DEBUG, "too soon, Ignoring setUpRegistrationForUpdate");
        return 0;
    }

    lastRegForUpdateTime = cTime;
    ProvisionInterface& pif = ProvisionInterface::instance();
    try
    {
        pif.registerForUpdate( updateExpireTime, "GlobalConfigData", ProvisionInterface::VCONFIG);
    }
    catch (VException& e)
    {
        cpLog(LOG_ALERT, "Failed to register for update with Provisioning Server for GlobalConfigData, reason %s", e.getDescription().c_str());
    }

    updateExpireTime("", "GlobalConfigData", false);

    try
    {
        pif.registerForUpdate( updateDialPlan, "defaultPhonePlan", ProvisionInterface::VCONFIG);
    }
    catch (VException& e)
    {
        cpLog(LOG_ALERT, "Failed to register for update with Provisioning Server for defaultPhonePlan, reason %s", e.getDescription().c_str());
    }

    updateDialPlan("", "defaultPhonePlan", false);

    try
    {
        sleep(1);
        pif.registerForUpdate( updateIPPlan, "defaultIpPlan", ProvisionInterface::VCONFIG);
    }
    catch (VException& e)
    {
        cpLog(LOG_ALERT, "Failed to register for update with Provisioning Server for defaultIpPlan, reason %s", e.getDescription().c_str());
    }

    updateIPPlan("", "defaultIpPlan", false);

    try
    {
        pif.registerDirForUpdate( updateAccounts, "Accounts");

    }
    catch (VException& e)
    {
        cpLog(LOG_ALERT, "Failed to register for update with Provisioning Server for Accounts directory, reason %s", e.getDescription().c_str());
    }

    try
    {
        pif.registerDirForUpdate( updateAliases, "Aliases");

    }
    catch (VException& e)
    {
        cpLog(LOG_ALERT, "Failed to register for update with Provisioning Server for Aliases directory, reason %s", e.getDescription().c_str());
    }

    try
    {
        pif.registerForUpdate( updateRegister, "REGISTERITEM", ProvisionInterface::VCONFIG);

    }
    catch (VException& e)
    {
        cpLog(LOG_ALERT, "Failed to register for update with Provisioning Server , reason %s", e.getDescription().c_str());
    }

    return 1;
}
