/* minimal include directives */
#include "xgcpMGCCfgTable.h"

/* 
 * xgcpMGCCfgTable_variables_oid:
 *   this is the top level oid that we want to register under.  This
 *   is essentially a prefix, with the suffix appearing in the
 *   variable below.
 */


oid xgcpMGCCfgTable_variables_oid[] = { 1,3,6,1,4,1,5738,3,90,1,5 };


/* 
 * variable2 xgcpMGCCfgTable_variables:
 *   this variable defines function callbacks and type return information 
 *   for the xgcpMGCCfgTable mib section 
 */

#define   xgcpMGCCfgAddress     3
#define   xgcpMGCCfgUDPPort     4
#define   xgcpMGCCfgConnStatus  5
#define   xgcpMGCCfgTimeStamp   6

struct variable2 xgcpMGCCfgTable_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
  { xgcpMGCCfgAddress   , ASN_OCTET_STR , RWRITE, var_xgcpMGCCfgTable, 1, { 2 } },
  { xgcpMGCCfgUDPPort   , ASN_INTEGER   , RWRITE, var_xgcpMGCCfgTable, 1, { 3 } },
  { xgcpMGCCfgConnStatus, ASN_INTEGER   , RONLY , var_xgcpMGCCfgTable, 1, { 4 } },
  { xgcpMGCCfgTimeStamp , ASN_TIMETICKS , RONLY , var_xgcpMGCCfgTable, 1, { 5 } },

};
/*    (L = length of the oidsuffix) */


/*
 * init_xgcpMGCCfgTable():
 *   Initialization routine.  This is called when the agent starts up.
 *   At a minimum, registration of your variables should take place here.
 */
void init_xgcpMGCCfgTable(void) {


  /* register ourselves with the agent to handle our mib tree */
  REGISTER_MIB("xgcpMGCCfgTable", xgcpMGCCfgTable_variables, variable2,
               xgcpMGCCfgTable_variables_oid);
}


/*
 * var_xgcpMGCCfgTable():
 *   This function is called every time the agent gets a request for
 *   a scalar variable that might be found within your mib section
 *   registered above.  It is up to you to do the right thing and
 *   return the correct value.
 *     You should also correct the value of "var_len" if necessary.
 *
 *   Please see the documentation for more information about writing
 *   module extensions, and check out the examples in the examples
 *   and mibII directories.
 */
unsigned char *
var_xgcpMGCCfgTable(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{


  /* variables we may use later */
  static long long_ret;
  static unsigned char string[SPRINT_MAX_LEN];
  static oid objid[MAX_OID_LEN];
  static struct counter64 c64;

  int index, mgcpPort;

  index = header_xgcp_table(vp,name,length,exact,var_len,write_method, 13);
  if( index == MATCH_FAILED )
    return NULL;

  mgcpPort = TrapAgent_GetStackPortFromIndex((void *)trapAgent, index);

  if (mgcpPort == 0)
     return NULL; 

/*
if (header_simple_table(vp,name,length,exact,var_len,write_method, TABLE_SIZE)
                                                == MATCH_FAILED )
    return NULL; 
*/
  /* 
   * this is where we do the value assignments for the mib results.
   */
  switch(vp->magic) {


    case xgcpMGCCfgAddress:
        *write_method = write_xgcpMGCCfgAddress;
        *string = 0;
        *var_len = strlen(string);
        return (unsigned char *) string;

    case xgcpMGCCfgUDPPort:
        *write_method = write_xgcpMGCCfgUDPPort;
        long_ret = 0;
        return (unsigned char *) &long_ret;

    case xgcpMGCCfgConnStatus:
/*
        if (requestXgcpCounter(vp->magic, &long_ret, mgcpPort) != voSuccess) {
            return NULL;
        }
*/
        long_ret = 0;
        return (unsigned char *) &long_ret;    

    case xgcpMGCCfgTimeStamp:

        /* Since this oid indicates the time stamp of the 
         * state transition of the previous oid.. This 
         * need to be processed further......
         */

        long_ret = 0;
        return (unsigned char *) &long_ret;


    default:
      ERROR_MSG("");
  }
  return NULL;
}


int
write_xgcpMGCCfgAddress(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
  static unsigned char string[SPRINT_MAX_LEN];
  int size;


  switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR){
              fprintf(stderr, "write to xgcpMGCCfgAddress not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)){
              fprintf(stderr,"write to xgcpMGCCfgAddress: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;


        case RESERVE2:
          size = var_val_len;
        //  string = (char *) var_val;


          break;


        case FREE:
             /* Release any resources that have been allocated */
          break;


        case ACTION:
             /* The variable has been stored in string for
             you to use, and you have just been asked to do something with
             it.  Note that anything done here must be reversable in the UNDO case */
          break;


        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;


        case COMMIT:
             /* Things are working well, so it's now safe to make the change
             permanently.  Make sure that anything done here can't fail! */
          break;
  }
  return SNMP_ERR_NOERROR;
}

int
write_xgcpMGCCfgUDPPort(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
  static long *long_ret;
  int size;


  switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER){
              fprintf(stderr, "write to xgcpMGCCfgUDPPort not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)){
              fprintf(stderr,"write to xgcpMGCCfgUDPPort: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;


        case RESERVE2:
          size = var_val_len;
          long_ret = (long *) var_val;


          break;


        case FREE:
             /* Release any resources that have been allocated */
          break;


        case ACTION:
             /* The variable has been stored in long_ret for
             you to use, and you have just been asked to do something with
             it.  Note that anything done here must be reversable in the UNDO case */
          break;


        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;


        case COMMIT:
             /* Things are working well, so it's now safe to make the change
             permanently.  Make sure that anything done here can't fail! */
          break;
  }
  return SNMP_ERR_NOERROR;
}
