/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const EntrySDP_cxx_Version = 
"$Id: EntrySDP.cxx,v 1.5 2001/05/23 20:50:47 vzubarev Exp $";

#include "EntrySDP.hxx"
#include "FeatureSDP.hxx"
#include "EvtCapMgrIntrsctChg.hxx"
#include "XlatorException.hxx"
#include "SM_CapManager.hxx"

using namespace Translator;

using namespace CapabilityTranslator;

using namespace CapabilityManager;


bool EntrySDP::makeIntersect(FeatureSDP &f_sdp) throw()
{    

    // TODO:
    // here should be the code responsible for making 
    // intersection between CapabilitySetSip and
    // CapabilitySetH323, updating media sessions

   const string fn("EntrySDP::makeIntersect");
   VLog     	log(fn);
   
   int descrVecSizeInt = 0, i = 0, j = 0, h = 0;
   int descrSizeInt = 0, descrSizeSip = 0;
   
   try
   {
     f_sdp.getSM()->capSetIntersect.makeIntersect();
   }
   
   catch (XlatorException &mS)
   {
     VERR(log) << fn << ": CapabilitySetIntersect intersection error... " << mS.getDescription() << VERR_END(log);
     throw XlatorException("Intersection error...", __FILE__, __LINE__);
   }
   
  // Finding the appropriate SessionDescriptors (RTP/RTCP pairs)
  // for the CapabilitySetIntersect CapabilityDescriptors
  // Looking for the maximum intersection of CapabilitySetIntersect CapDescriptors
  // with CapDescriptors from CapabilitySetSip
  
  CapabilitySet::CapabilityDescrVec::AltCapSet altCapSetIntersect;
  
  descrVecSizeInt = f_sdp.getSM()->capSetIntersect.capRecv.getCapDescrVecSize();
  
  int maxIntersectIndex = 0;
  unsigned int maxIntersectNumber = 0;
  //Address address;
  CapabilitySet::CapabilityDescrVec::AltCapSet altCapInt;
  CapabilitySet::CapabilityDescrVec::AltCapSet altCapSip;
  
  for(i = 0; i < descrVecSizeInt; i++)
  {
    descrSizeInt = f_sdp.getSM()->capSetIntersect.capRecv.getCapDescrSize(i);
    
    for(j = 0; j < descrSizeInt; j++)
    {
      // every altCapSet we intersect with each altCapSet from capSetSip
      // whichever gives us the maximum Intersection - it's SessionDescriptor
      // should be associated with given altCapSet of CapSetIntersect
   
      altCapInt = f_sdp.getSM()->capSetIntersect.capRecv.getAltCapSet(i, j);
      std::sort(altCapInt.begin(), altCapInt.end());
      
      descrSizeSip = f_sdp.getSM()->capSetSip.capRecv.getCapDescrSize(0);
      altCapSetIntersect.clear();
      maxIntersectNumber = 0;
      maxIntersectIndex = -1;
      
      for(h = 0; h < descrSizeSip; h++)
      {
         altCapSip = f_sdp.getSM()->capSetSip.capRecv.getAltCapSet(0, h);
	 std::sort(altCapSip.begin(), altCapSip.end());
         std::set_intersection(
	                       altCapSip.begin(), 
			       altCapSip.end(), 
			       altCapInt.begin(), 
			       altCapInt.end(), 
			       std::inserter(altCapSetIntersect, altCapSetIntersect.begin()));
         
	 if(maxIntersectNumber < altCapSetIntersect.size())
	 {
	   maxIntersectNumber = altCapSetIntersect.size();
	   maxIntersectIndex = h;
	 }
      }
      
      if(maxIntersectIndex == -1)
         VERR(log) << fn << ": Problems with finding SessionDescriptor..." << VERR_END(log);
      
      /*TODO:
        the following code should be migrated
	to media session manager and modified accordingly
      */
      
      /*try
      {
        address = sDescriptor.transportIDSip.getRTPAddr(0, maxIntersectIndex);
      }
      catch (XlatorException &mS)
      {
        VERR(log) << fn << ": Error with getting RTP addr from Sip : " << mS.getDescription() << VERR_END(log);
	assert(0);
      }
      
      try
      {
        sDescriptor.transportIDIntSip.setRTPAddr(address, i, j);
      }
      catch (XlatorException &mS)
      {
        VERR(log) << fn << ": Error with setting RTP addr to SipInt : " << mS.getDescription() << VERR_END(log);
	assert(0);
      }

      try
      {
        address = sDescriptor.transportIDSip.getRTCPAddr(0, maxIntersectIndex);
	IPAddress ipAddress;
	ipAddress = address;
	VDEBUG(log) << fn << ": RTCP port we retreive from trIDSip : " << ipAddress.getPort() << VDEBUG_END(log);
	VDEBUG(log) << fn << ": maxIntersectIndex : " << maxIntersectIndex << VDEBUG_END(log);
      }
      catch (XlatorException &mS)
      {
        VERR(log) << fn << ": Error with getting RTCP addr from Sip : " << mS.getDescription() << VERR_END(log);
	assert(0);
      }
      
      try
      {
        sDescriptor.transportIDIntSip.setRTCPAddr(address, i, j);
      }
      catch (XlatorException &mS)
      {
        VERR(log) << fn << ": Error with setting RTCP addr to SipInt : " << mS.getDescription() << VERR_END(log);
    	assert(0);
      }*/

    }
  }

return true;

}

void EntrySDP::inIntersect(FeatureSDP& f_sdp) throw()
{
    // here we make intersection
    
    if(makeIntersect(f_sdp))
    {
         // Intersection changed since the last one
	 
	 EvtCapMgrIntrsctChg xsctChg(f_sdp.getSM());
	 xsctChg.start();
    }


    // TODO
    // after making intersection we have to update
    // media session with up to date information


    // here we should wait for all info received from media
    // session before constructing and sending SDP
    //


    // sending out SDP
    // Note: generating SDP only after getting confirmation
    // from media session manager with valid information
    // needed to construct SDP (c= addresses and RTP ports)
    
    
  SDP msgSDP;
  
  const string	fn("EntrySDP::inIntersect");
  VLog	    	log(fn);
  
/*  SdpConnection connection;
  IPAddress ipAddress;
  std::string connAddr;
  Capability::intSDP packetSize;
  int capNumber;
  int i = 0, descrSize = 0;
  int pTypeCap = 0;
  int clockRate = 0;
  
  VDEBUG(log) << fn << ": BEGIN -------------------- " << VDEBUG_END(log);
  
  try
  {
    ipAddress = sDescriptor.transportIDH323.getRTPAddr(0, 0);
  }
  catch (XlatorException &mS)
  {
    VERR(log) << fn << ": Error with getting RTPAddr from IDH323 : " << mS.getDescription() << VERR_END(log);
    throw XlatorException("Error with getting RTPAddr from IDH323", __FILE__, __LINE__);
  }
  
  VDEBUG(log) << fn << ": Here is ipAddress : " << ipAddress.getIPAsString() << VDEBUG_END(log);
  
  connAddr = ipAddress.getIPAsString();
  connection.setAddressType(AddressTypeIPV4);
  connection.setUnicast(connAddr.c_str());
  msgSDP.setConnection(connection);

  extern string	    SipToH323CallSigGW_HostAddress;

  VDEBUG(log) << fn << ": Here is the IP address : " << SipToH323CallSigGW_HostAddress << VDEBUG_END(log);
  msgSDP.setAddress(SipToH323CallSigGW_HostAddress.c_str());
  
  msgSDP.setSessionName("VOVIDA session");
  
  // at this point we assume that we use only first (0) capability descriptor
  // from CapabilityDescrVec
  
  descrSize = capSetIntersect.capRecv.getCapDescrSize(0);

  for(i = 0; i < descrSize; i++)
  {
    SdpMedia *media = new SdpMedia;
    ipAddress = sDescriptor.transportIDH323.getRTPAddr(0, i);
    media->setPort(ipAddress.getPort());
  
    MediaAttributes *mediaAttrib = new MediaAttributes();
    ValueAttribute *attrib = new ValueAttribute();
  
    capNumber = capSetIntersect.capRecv.getCapability(0, i, 0);
  
    //H323_CapTable::CapTable_I pI = capSetIntersect.h323CapTable.getTableIter_pt(capNumber);
    H323_CapTable::CapTable_I pI = fromOLCtoSDPcapTable.getTableIter_pt(capNumber);
    
    if(pI == NULL)
    {
	pI = capSetIntersect.h323CapTable.getTableIter_pt(capNumber);
        
	if(pI == NULL)
        {
              VCRIT(log) << fn << ": Did not find the entry in CapTable by number gotten from capDescriptor..." << VCRIT_END(log);
              throw XlatorException("Did not find the entry in CapTable by number gotten from capDescriptor",
                             __FILE__, __LINE__);
	}
    }
    
    (*pI)->getParamSendMsg(packetSize);

    clockRate = (*pI)->getClockRate()/1000;
    packetSize = packetSize/clockRate;

    attrib->setAttribute("ptime");
    attrib->setValue(Data(packetSize));
  
    SdpRtpMapAttribute *rtpMapAttrib = new SdpRtpMapAttribute();
    pTypeCap = (*pI)->getCapabilityNumber();
    
    
    if(pTypeCap == 1)pTypeCap = 0;
    
    rtpMapAttrib->setPayloadType(pTypeCap);
  
    GlobalCapTable::GlobalCapTable_p pG;
    try
    {
      pG = globalCapTable.getTableIter_pt((*pI)->getCapabilityNumber());
    }
    catch (XlatorException)
    {
      VCRIT(log) << fn << ": Did not find the entry in GlobalCapTable..." << VCRIT_END(log);
      throw XlatorException("Did not find the entry in GlobalCapTable", __FILE__, __LINE__);
    }
  
    media->clearFormatList();
    media->addFormat(pTypeCap);

    rtpMapAttrib->setEncodingName(pG->nameCapIANA);
    
    rtpMapAttrib->setClockRate(pG->clockrateCap);
  
    mediaAttrib->addValueAttribute(attrib);
    mediaAttrib->addmap(rtpMapAttrib);
  
    media->setMediaAttributes(mediaAttrib);
  
    msgSDP.addMedia(media);
  }*/

  f_sdp.getSM()->getEM()->startSendSDP(msgSDP, true);

  VDEBUG(log) << fn << ": END -------------------- " << VDEBUG_END(log);

}

void EntrySDP::
inOther(FeatureSDP& f_sdp)
{
    // Ignore
}
