/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const EntryTCS_cxx_Version = 
"$Id: EntryTCS.cxx,v 1.5 2001/05/23 20:50:47 vzubarev Exp $";

#include "EntryTCS.hxx"
#include "FeatureTCS.hxx"
#include "SM_CapManager.hxx"

using namespace CapabilityTranslator;

using namespace CapabilityManager;


void EntryTCS::
inSent(FeatureTCS& f_tcs)
{
    //f_tcs.;
    // TODO:
    // here should be the code responsible for generating
    // appropriate TCS message according to the following
    // rule:
    // Upon entering the state:
    // generate outTCS = Intersect or outTCS = SDP
    // (if TCS not received)
    
    
    // send out TCS
    
  TCS msgTCS;
  
  const string	fn("EntryTCS::inSent");
  VLog	    	log(fn);
  
  int tableSize = 0, descrVecSize = 0, descrSize = 0, altSize = 0, i = 0, j = 0, h = 0;
  VDEBUG(log) << fn << ": BEGIN -----------------" << VDEBUG_END(log);
  
  msgTCS.IncludeOptionalField(H245_TerminalCapabilitySet::e_capabilityTable);
  
  const H323_CapTable::CapTable &capTable = f_tcs.getSM()->capSetSip.h323CapTable.getCapabilityTable();
  
  tableSize = capTable.size();
  msgTCS.m_capabilityTable.SetSize(tableSize);
  
  for(i = 0; i < tableSize; i++)
  {
    msgTCS.m_capabilityTable[i].m_capabilityTableEntryNumber = capTable[i]->getCapabilityNumber();
    msgTCS.m_capabilityTable[i].IncludeOptionalField(H245_CapabilityTableEntry::e_capability);

    VDEBUG(log) << fn << ": Capability " << capTable[i]->getCapabilityNumber()
    	    	<< " with packetSize " << capTable[i]->getPacketSizeFrames() 
		<< VDEBUG_END(log);

    capTable[i]->getParamSendMsg(msgTCS.m_capabilityTable[i].m_capability);
  }
  
  msgTCS.IncludeOptionalField(H245_TerminalCapabilitySet::e_capabilityDescriptors);
  
  descrVecSize = f_tcs.getSM()->capSetSip.capRecv.getCapDescrVecSize();
  msgTCS.m_capabilityDescriptors.SetSize(descrVecSize);
  
  for(i = 0; i < descrVecSize; i++)
  {
    H245_CapabilityDescriptor &descr = msgTCS.m_capabilityDescriptors[i];
    descr.m_capabilityDescriptorNumber = i + 1;
    descr.IncludeOptionalField(H245_CapabilityDescriptor::e_simultaneousCapabilities);
    
    descrSize = f_tcs.getSM()->capSetSip.capRecv.getCapDescrSize(i);
    descr.m_simultaneousCapabilities.SetSize(descrSize);
    
    for(j = 0; j < descrSize; j++)
    {
      H245_AlternativeCapabilitySet &alt = descr.m_simultaneousCapabilities[j];
      
      altSize = f_tcs.getSM()->capSetSip.capRecv.getAltCapSize(i, j);
      alt.SetSize(altSize);
      
      for(h = 0; h < altSize; h++)
      {
        try
	{
	  alt[h] = f_tcs.getSM()->capSetSip.capRecv.getCapability(i, j, h);
	}
	
	catch (XlatorException)
	{
	  VERR(log) << fn << ": Problems with altCapSet, weird, man...: " 
	    	    << i << ' ' << j << ' ' << h << VERR_END(log);
	  assert(0);
	}
      }
    }
  }
  
  msgTCS.IncludeOptionalField(H245_TerminalCapabilitySet::e_multiplexCapability);
  msgTCS.m_multiplexCapability.SetTag(H245_MultiplexCapability::e_h2250Capability);
  
  H245_H2250Capability &hcap = msgTCS.m_multiplexCapability;
  hcap.m_maximumAudioDelayJitter = 60;
  
  VDEBUG(log) << fn << ": END ----------------" << VDEBUG_END(log);
  
  f_tcs.getSM()->getEM()->startCEOTransferReq(msgTCS);
   
}

void EntryTCS::
inOther(FeatureTCS& f_tcs)
{
    // Ignore
}
