#ifndef __SMMSMANAGER_H_
#define __SMMSMANAGER_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const SM_MSManager_hxx_Version = 
"$Id: SM_MSManager.hxx,v 1.3 2001/05/26 01:32:37 vzubarev Exp $";

#include "../CapabilityTranslatorDefs.hxx"
#include "Address.hxx"
#include "FeatureMSDefs.hxx"
#include "FeatureMS.hxx"
#include "FeatureOutLC.hxx"
#include "FeatureOutLCDefs.hxx"
#include "FeatureInLCDefs.hxx"
#include "FeatureInLC.hxx"

using namespace CapabilityTranslator;

namespace MediaSessionManager
{

    /// MediaSessionManager State Machine
    class SM_MSManager
    {
    private:
	/// Transport ID: <IPaddress>:<port> for RTP & RTCP
	struct TransportID
	{
	    /** shows whether we have sent an OLC msg already
	        used for H323 side
	    */
	    bool isDone;
	    
	    ///
	    Address addressRTP;
	    
	    ///
	    Address addressRTCP;
	};

	int sessionID;
	
	int logChOutN;
	int logChInN;
	
	int capabilityN;
	
	TransportID trIDSIP;
	TransportID trIDH323;
	
	translator::CallTranslatorEM *evtMgr;

    	FeatureMS *f_ms;
	FeatureOutLC *f_outLC;
	FeatureInLC *f_inLC;

    public:
	
	///
	SM_MSManager(translator::CallTranslatorEM *em);
	
	///
	virtual ~SM_MSManager();

	///
	void init();
	
	///
	SM_MSManager* clone() const;
	
        /// set capability for which we are opening this MS
	void setCapability(int cap);
	
	int getCapability() const;
	
	void setLogChOutN(int lchN);
	
	void setLogChInN(int lchN);
	
	void getLogChOutN() const;
	
	void getLogChInN() const;
	
	/// set RTP port of a SIP side
	void setRTPAddrSIP(const Address &rtpAddr);
	
	/// set RTCP port of a SIP side
	void setRTCPAddrSIP(const Address &rtcpAddr);

	/// get RTP port of a SIP side
	Address getRTPAddrSIP() const;

	/// get RTCP port of a SIP side
	Address getRTCPAddrSIP() const;

        /// set RTP port of an H323 side
	void setRTPAddrH323(const Address &rtpAddr);
	
	/// set RTCP port of an H323 side
	void setRTCPAddrH323(const Address &rtcpAddr);

	/// get RTP port of an H323 side
	Address getRTPAddrH323() const;

	/// get RTCP port of an H323 side
	Address getRTCPAddrH323() const;

	/// Return pointer to MS Feature
	virtual FeatureMSBase *getFeatureMS() throw();
	virtual FeatureMS &getFeatureImp() throw();
	
	/// Return pointer to InLC Feature
	virtual FeatureInLCBase *getFeatureInLC() throw();
	virtual FeatureInLC &getFeatureInLCImp() throw();
	
	/// Return pointer to OutLC Feature
	virtual FeatureOutLCBase *getFeatureOutLC() throw();
	virtual FeatureOutLC &getFeatureOutLCImp() throw();

    };
}

#endif
