/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323RegistrationEvent_cxx_Version = 
    "$Id: H323RegistrationEvent.cxx,v 1.10 2002/07/17 00:21:09 icahoon Exp $";


#include "H323RegistrationEvent.hxx"


using Vocal::H323Events::H323RegistrationEvent;
using Vocal::H323Events::H323Event;
using Vocal::SipH323Info::EndpointInfo;
using Vocal::SipH323Info::GatekeeperInfo;
using Vocal::SipH323Info::EndpointInfo;
using Vocal::Transport::IPAddress;


H323RegistrationEvent::H323RegistrationEvent(
    Sptr<EndpointInfo>	    	    eptInfo,
    GatekeeperInfo	    	&   gkInfo,
    Sptr<H225_RasMessage>           raspdu,
    Sptr<IPAddress> 	    	    remoteAddr,
    bool    	    	    	    reg,
    FifoBase< Sptr<Event> > 	&   replyfifo
)
    :	H323Event(replyfifo),
	endpointInfo_(eptInfo),
	gatekeeperInfo_(gkInfo),
    	rasPDU_(raspdu),
    	remoteAddr_(remoteAddr),
	registration_(reg)
{
}
					

H323RegistrationEvent::~H323RegistrationEvent()
{
}


Sptr<EndpointInfo>  	    
H323RegistrationEvent::endpointInfo() const
{
    return ( endpointInfo_ );
}


GatekeeperInfo &
H323RegistrationEvent::gatekeeperInfo() const
{
    return ( gatekeeperInfo_ );
}


Sptr<H225_RasMessage>    	    
H323RegistrationEvent::rasPDU() const
{
    return ( rasPDU_ );
}


Sptr<IPAddress>     	    
H323RegistrationEvent::remoteAddress() const
{
    return ( remoteAddr_ );
}


bool
H323RegistrationEvent::registration() const
{
    return ( registration_ );
}


const char*
H323RegistrationEvent::name() const
{
    return ( "H323RegistrationEvent" );
}


ostream &           	    
H323RegistrationEvent::writeTo(ostream & out) const
{
    H323Event::writeTo(out);

    out << ", endpoint = { ";
    
    if ( endpointInfo_ != 0 )
    {
    	out << *endpointInfo_;
    }
    
    return ( out << " }" );
}
