#if !defined(VOCAL_DEFAULT_RESOURCES_AVAILABLE_POLICY_HXX)
#define VOCAL_DEFAULT_RESOURCES_AVAILABLE_POLICY_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const DefaultResourcesAvailablePolicy_hxx_Version = 
    "$Id: DefaultResourcesAvailablePolicy.hxx,v 1.3 2002/05/15 11:35:43 icahoon Exp $";


#include "ptlib.h"
#include "h323pdu.h"
#include <string>
#include "ResourcesAvailablePolicy.hxx"
#include "IPAddress.hxx"
#include "EndpointInfo.hxx"


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** H.323 Gatekeeper<br><br>
 */
namespace H323GK
{


using Vocal::SipH323Info::EndpointInfo;


/** Default policy for determining how to handle RAS resource available
 *  messages. (RAI, RAC)<br><br>
 *
 *  @see    Vocal::H323GK::ResourcesAvailablePolicy
 *  @see    Vocal::H323GK::RASPolicy
 *  @see    Vocal::H323GK::RASGKProtocol
 *  @see    Vocal::SipH323Info::GatekeeperInfo
 *  @see    Vocal::SipH323Info::EndpointInfoMap
 *  @see    Vocal::Services::Service
 *  @see    Vocal::Transport::IPAddress
 */
class DefaultResourcesAvailablePolicy : public Vocal::H323GK::ResourcesAvailablePolicy
{


    public:
    

    	/** Construct given the controling service, the associated ras gk
	 *  protocol, the configuration information for the gatekeeper and
	 *  the table of registered endpoints.
	 */
    	DefaultResourcesAvailablePolicy(
	    Service &,
	    RASGKProtocol &,
	    GatekeeperInfo &,
	    EndpointInfoMap &
	);


    	/** Virtual destructor
	 */
	virtual		    ~DefaultResourcesAvailablePolicy();
			


    	/** Called to process an incoming resource available indication.
    	 *
	 *  This will first validate the incoming rai. It will then give
	 *  the user an opportunity to process the incoming message by calling
	 *  the callback onResourcesAvailable. The response, rac, is then sent.
	 */
    	void 	    	    processResourcesAvailableIndication(
	    	    	    	Sptr<H225_RasMessage>   rasPDU, 
			    	Sptr<IPAddress>         remoteAddr);


    	/** Write the policy to an ostream. 
	 */
    	ostream &           writeTo(ostream & out) const;

				
    protected:


	/** Called on receipt of a resources available indication. 
	 *  
	 *  On wait, the processing stops. It is the responsibility of
	 *  the derived class to retain message state.
	 *
	 *  Otherwise, the processing continues processing, sending the 
         *  response.
	 *
	 */
    	virtual Proceed	    onResourcesAvailable(
				Sptr<H225_RasMessage>	    	    rasPDU, 
				Sptr<IPAddress> 	    	    remoteAddr,
				Sptr<EndpointInfo>  	    	    endpointInfo,
	    	    	    	H225_ResourcesAvailableIndicate &   rai);


    	/** Called before the rac is sent.
	 */
    	virtual void	    finalizeResourcesAvailableConfirm(
	    	    	    	H225_ResourcesAvailableIndicate &   rai,
	    	    	    	H225_ResourcesAvailableConfirm  &   rac);


    	/** Returns the address where the response will be sent.
	 */
    	Sptr<IPAddress>     responseAddress(
				Sptr<EndpointInfo>  	    	    endpointInfo,
			    	Sptr<IPAddress>     	    	    remoteAddr);


    	/** Ensures the multicast gatekeeper request is sane. If the request
	 *  is sane it returns true, otherwise it returns false. 
	 */	
    	virtual bool  	    validateResourcesAvailableIndicate(
	    	    	    	H225_ResourcesAvailableIndicate &   rai,
				Sptr<EndpointInfo>  	        &   eptInfo);


    	/** Builds and sends the response.
	 *  Will finalize before sending.
	 */
    	virtual void	    buildResourcesAvailableConfirm(
	    	    	    	H225_ResourcesAvailableIndicate  &  rai, 
			    	Sptr<IPAddress>     	            remoteAddr);
};


} // namespace H323GK
} // namespace Vocal


#endif // !defined(VOCAL_DEFAULT_RESOURCES_AVAILABLE_POLICY_HXX)
