#if !defined(VOCAL_KEEP_ALIVE_TIMER_EVENT_HXX)
#define VOCAL_KEEP_ALIVE_TIMER_EVENT_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const KeepAliveTimerEvent_hxx_Version = 
    "$Id: KeepAliveTimerEvent.hxx,v 1.10 2002/07/17 00:21:09 icahoon Exp $";


#include "Event.hxx"
#include <string>

/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** H.323 Gatekeeper<br><br>
 */
namespace H323GK
{


/** Event used to track keep alive events for each endpoint.
 */
class KeepAliveTimerEvent : public Vocal::Signaling::Event
{

    public:


    	/** Construct given the associated endpoint identifier.
	 */
    	KeepAliveTimerEvent(const string & endpointIdentifier);


    	/** Virtual destructor.
	 */
	virtual     	    	    ~KeepAliveTimerEvent();


    	/** Access the endpoint identifier that is associated with
	 *  this keepalive event.
	 */
    	const	string &    	    endpointIdentifier() const;
	

    	/** Name of this class of event, "KeepAliveTimerEvent".
	 */
    	virtual const char*         name() const;


    	/** Write the event to an ostream.
	 */
	virtual ostream &           writeTo(ostream &) const;

    private:

    	string	    	    	    myEndpointIdentifier;
};


} // namespace H323GK
} // namespace Vocal

#endif // !defined(VOCAL_KEEP_ALIVE_TIMER_EVENT_HXX)
