#if !defined(VOCAL_RAS_POLICY_HXX)
#define VOCAL_RAS_POLICY_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const RASPolicy_hxx_Version = 
    "$Id: RASPolicy.hxx,v 1.10 2002/05/15 11:35:44 icahoon Exp $";


#include "Writer.hxx"


// Forward declarations
//
namespace Vocal {
    namespace Services {
    	class Service;
    }
    namespace SipH323Info {
    	class GatekeeperInfo;
    	class EndpointInfoMap;
    }
}


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** H.323 Gatekeeper<br><br>
 */
namespace H323GK
{


using Vocal::Services::Service;
using Vocal::SipH323Info::GatekeeperInfo;
using Vocal::SipH323Info::EndpointInfoMap;


class RASGKProtocol;


/** The data shared between all RAS policy classes.<br><br>
 *
 *  @see    Vocal::H323GK::RASGKProtocol
 *  @see    Vocal::SipH323Info::GatekeeperInfo
 *  @see    Vocal::SipH323Info::EndpointInfoMap
 *  @see    Vocal::Services::Service
 */
class RASPolicy : public Vocal::IO::Writer
{

    protected:


    	/**
	 */
    	RASPolicy(
	    Service &,
	    RASGKProtocol &, 
	    GatekeeperInfo &,
	    EndpointInfoMap &
	);


    	/** Virtual destructor
	 */
	virtual		    	~RASPolicy();
			

    	/** Return code for callbacks.
    	 *  
    	 *  wait: The gatekeeper is probably waiting for an external 
    	 *  	event to drive the registration forward. The dervied 
    	 *  	class needs to save the rasPDU and the remoteAddr 
    	 *  	somewhere and will probably just call this function 
    	 *  	again when the external event has happened, making sure
    	 *  	that the function doesn't return wait.
	 *
    	 *  confirm: proceed processing the request
    	 *  
    	 *  reject: reject the request without further processing
	 */
	enum	Proceed
	{
	    wait,
	    confirm,
	    reject
	};


    	/** The service that controls this policy.
	 */
    	Service	    	    &	myService;


    	/** The protocol that provides incoming messages to the policy.
	 */
    	RASGKProtocol       &   myRasGkProtocol;


    	/** The gatekeeper's configuration information.
	 */
	GatekeeperInfo	    &	myGatekeeperInfo;


    	/** The table of registered endpoints.
	 */
	EndpointInfoMap     &	myEndpointInfoMap;
};


} // namespace H323GK
} // namespace Vocal


#endif // !defined(VOCAL_RAS_POLICY_HXX)
