#if !defined(CALL_IDENTIFIER_DOT_H)
#define CALL_IDENTIFIER_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const CallIdentifier_hxx_Version = 
    "$Id: CallIdentifier.hxx,v 1.9 2000/12/18 23:47:24 bko Exp $";


#include "ptlib.h"
#include "h323pdu.h"


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** SIP to H.323 Object Information<br><br>
 */
namespace SipH323Info
{


/** Key used to identify an H.323 call.<br><br>
 */
class CallIdentifier
{

    public:


    	/** Default constructor
    	 */
    	CallIdentifier();


    	/** Construct given the h.225 conference identifier and the h.225
	 *  call identifier.
    	 */
    	CallIdentifier(
    	    const H225_ConferenceIdentifier 	&,
	    const H225_CallIdentifier 	    	&
	);


    	/** Copy constructor
    	 */
    	CallIdentifier(const CallIdentifier &);


    	/** Virtual destructor
    	 */
	virtual     ~CallIdentifier();


    	/** Assignment operator
    	 */
    	CallIdentifier &    	    operator=(const CallIdentifier &);


    	/** Access the h.225 conference identifier.
    	 */
    	H225_ConferenceIdentifier   conferenceIdentifier() const;


    	/** Access the h.225 call identifier.
    	 */
	H225_CallIdentifier 	    callIdentifier() const;


    	/** Less than relational operator.
    	 */
    	bool	operator<(const CallIdentifier &) const;


    	/** Equality relational operator.
    	 */
    	bool	operator==(const CallIdentifier &) const;


    	/** Write call identifier to an ostream. 
	 */
	ostream &   writeTo(ostream &) const;

    private:

    	H225_ConferenceIdentifier   	confId_;
	H225_CallIdentifier 	    	callId_;
};


} // namespace SipH323Info
} // namespace Vocal


#endif // !defined(CALL_IDENTIFIER_DOT_H)
