/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const tAlias_cxx_Version = 
    "$Id: tAlias.cxx,v 1.8 2002/05/15 11:11:47 icahoon Exp $";


#include "Application.hxx"
#include "Alias.hxx"
#include "Verify.hxx"
#include <iostream>


using namespace Vocal;
using namespace Vocal::SipH323Info;
using namespace Vocal::Transport;


class tApplication : public Application
{
    public:
    	ReturnCode  	run();
};


class tApplicationCreator : public ApplicationCreator
{
    Application * create() { return ( new tApplication ); }
};

int main(int argc, char ** argv, char ** arge)
{
    tApplicationCreator creator;
    Application::creator(&creator);
    ReturnCode  count = Application::main(argc, argv, arge);
    return ( test_return_code(count) );
}


ReturnCode
tApplication::run()
{
    int count = 0;
    
    Alias   	alias0;
    
    cout << "alias0 = " << alias0 << endl;

    bool    has = alias0.hasUser();
    test_verify( !has ); ++count;
        
    alias0.user("Ian Cahoon");

    has = alias0.hasUser();
    test_verify( has ); ++count;

    cout << "alias0 = " << alias0 << endl;


    has = alias0.hasPhone();
    test_verify( !has ); ++count;

    alias0.phone("2222");

    has = alias0.hasPhone();
    test_verify( has ); ++count;

    cout << "alias0 = " << alias0 << endl;


    has = alias0.hasEmail();
    test_verify( !has ); ++count;

    alias0.email("icahoon@vovida.com");

    has = alias0.hasEmail();
    test_verify( has ); ++count;

    cout << "alias0 = " << alias0 << endl;


    has = alias0.hasUrl();
    test_verify( !has ); ++count;

    alias0.url("sip:icahoon@vovida.com");

    has = alias0.hasUrl();
    test_verify( has ); ++count;

    cout << "alias0 = " << alias0 << endl;


    has = alias0.hasTransport();
    test_verify( !has ); ++count;

    IPAddress	ip_alias("192.168.5.23", 1720);
    
    alias0.transport(ip_alias);

    has = alias0.hasTransport();
    test_verify( has ); ++count;

    cout << "alias0 = " << alias0 << endl;


    Alias alias1 = alias0;
    Alias alias2(alias0);

    bool result = alias0 == alias2;
    test_verify( result == true ); ++count;
    
    cout << "alias0 == alias2 ? " << ( result ? "true" : "false") << endl;
    
    alias0.clear();

    cout << "alias0 = " << alias0 << endl;

    result = alias0 == alias1;
    test_verify( result == false ); ++count;
    
    cout << "alias0 == alias1 ? " << (result ? "true" : "false") << endl;
    cout << "alias1 = " << alias1 << endl;
    cout << "alias2 = " << alias2 << endl;


    alias1.user("Danny");
    cout << "alias1 = " << alias1 << endl;

    alias1.phone("1111");
    cout << "alias1 = " << alias1 << endl;

    alias1.email("danny@vovida.com");
    cout << "alias1 = " << alias1 << endl;

    alias1.url("sip:danny@vovida.com");
    cout << "alias1 = " << alias1 << endl;

    IPAddress	local_alias("127.0.0.1", 7);
    alias1.transport(local_alias);

    cout << "alias1 = " << alias1 << endl;

    result = alias1 < alias2;
    test_verify( result == true ); ++count;
    
    cout << "alias1 < alias2 ? " << (result ? "true" : "false") << endl;

    result = alias2 < alias1;
    test_verify( result == false ); ++count;
    
    cout << "alias2 < alias1 ? " << (result ? "true" : "false") << endl;

    
    Alias   a0, a1, a2, a3;
    
    a1.phone("1111");
    a2.phone("2222");

    result = a0 < a0; test_verify( result == false ); ++count;
    cout << "a0 < a0 ? " << ( result ? "true" : "false") << endl;

    result = a0 < a1; test_verify( result == true ); ++count;
    cout << "a0 < a1 ? " << ( result ? "true" : "false") << endl;

    result = a0 < a2; test_verify( result == true ); ++count;
    cout << "a0 < a2 ? " << (result ? "true" : "false") << endl;


    result = a1 < a0; test_verify( result == false ); ++count;
    cout << "a1 < a0 ? " << (result ? "true" : "false") << endl;

    result = a1 < a1; test_verify( result == false ); ++count;
    cout << "a1 < a1 ? " << (result ? "true" : "false") << endl;

    result = a1 < a2; test_verify( result == true ); ++count;
    cout << "a1 < a2 ? " << (result ? "true" : "false") << endl;


    result = a2 < a0; test_verify( result == false ); ++count;
    cout << "a2 < a0 ? " << (result ? "true" : "false") << endl;

    result = a2 < a1; test_verify( result == false ); ++count;
    cout << "a2 < a1 ? " << (result ? "true" : "false") << endl;

    result = a2 < a2; test_verify( result == false ); ++count;
    cout << "a2 < a2 ? " << (result ? "true" : "false") << endl;
    

    result = a0 == a0; test_verify( result == true ); ++count;
    cout << "a0 = a0 ? " << (result ? "true" : "false") << endl;

    result = a0 == a1; test_verify( result == false ); ++count;
    cout << "a0 = a1 ? " << (result ? "true" : "false") << endl;

    result = a0 == a2; test_verify( result == false ); ++count;
    cout << "a0 = a2 ? " << (result ? "true" : "false") << endl;


    result = a1 == a0; test_verify( result == false ); ++count;
    cout << "a1 = a0 ? " << (result ? "true" : "false") << endl;

    result = a1 == a1; test_verify( result == true ); ++count;
    cout << "a1 = a1 ? " << (result ? "true" : "false") << endl;

    result = a1 == a2; test_verify( result == false ); ++count;
    cout << "a1 = a2 ? " << (result ? "true" : "false") << endl;


    result = a2 == a0; test_verify( result == false ); ++count;
    cout << "a2 = a0 ? " << (result ? "true" : "false") << endl;

    result = a2 == a1; test_verify( result == false ); ++count;
    cout << "a2 = a1 ? " << (result ? "true" : "false") << endl;

    result = a2 == a2; test_verify( result == true ); ++count;
    cout << "a2 = a2 ? " << (result ? "true" : "false") << endl;


    result = a0 == a3; test_verify( result == true ); ++count;
    cout << "a0 = a3 ? " << (result ? "true" : "false") << endl;
        
    return ( count );
}
