/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323SipOpOut_cxx_Version = 
    "$Id: H323SipOpOut.cxx,v 1.19 2002/12/10 04:15:00 icahoon Exp $";


#include "H323SipOpOut.hxx"
#include "SipProxyInfo.hxx"
#include "SipMsg.hxx"
#include "SipVia.hxx"
#include "Address.hxx"
#include "AddressToFrom.hxx"
#include "OutgoingSipEvent.hxx"
#include "VLog.hxx"
#include "Data.hxx"


using namespace Vocal;
using namespace Vocal::SipH323Proxy;
using namespace Vocal::SipH323Info;
using namespace Vocal::SipEvents;
using namespace Vocal::Logging;


H323SipOpOut::H323SipOpOut(
    SipProxyInfo    	    & 	sipProxyInfo
)
    :	sipProxyInfo_(sipProxyInfo),
    	sipStack_(0)
{
}


H323SipOpOut::~H323SipOpOut()
{
}



void	    	    	    	
H323SipOpOut::sipStack(
    Sptr<SipTransceiver>     	sipStack
)
{
    sipStack_ = sipStack;
}

void	    	    	    	
H323SipOpOut::updateRequestLine(
    SipCommand 	    &	sipCommand
)
{
    const string    fn("H323SipOpOut::requestLine");
    VLog    	    log(fn);

    SipRequestLine  	requestLine = sipCommand.getRequestLine();
    Sptr<BaseUrl>       url = requestLine.getUrl();

    Sptr<SipUrl>        sipUrl;
    if ( sipUrl.dynamicCast(url) == 0 )
    {
        assert(0);
        return;
    }

    sipUrl->setHost(sipProxyInfo_.sipRemoteAddr().getIPAsString().c_str());
    sipUrl->setPort(Data((int)sipProxyInfo_.sipRemoteAddr().getPort()));


    VDEBUG(log) << fn << ": requestLine url = " << url->encode().logData() << VDEBUG_END(log);

    requestLine.setUrl(url);
    sipCommand.setRequestLine(requestLine);
}


void
H323SipOpOut::updateVia(SipMsg & sipMsg)
{
    SipVia  	sipvia = sipMsg.getVia();
    sipvia.setHost(sipProxyInfo_.sipProxy().getIPAsString().c_str());
    sipvia.setPort(sipProxyInfo_.sipProxy().getPort());
    sipMsg.removeVia();
    sipMsg.setVia(sipvia);
}


void	    	    	    	
H323SipOpOut::updateTo(SipMsg & sipMsg)
{
    const string    fn("H323SipOpOut::updateTo");
    VLog    	    log(fn);

    Sptr<BaseUrl>   url = sipMsg.getTo().getUrl();
    Sptr<SipUrl>    sipUrl;
    if ( sipUrl.dynamicCast(url) == 0 )
    {
        assert(0);
        return;
    }

    sipUrl->setHost(sipProxyInfo_.sipRemoteAddr().getIPAsString().c_str());
    sipUrl->setPort(Data((int)sipProxyInfo_.sipRemoteAddr().getPort()));

    VDEBUG(log) << fn << ": url = " << url->encode().logData() << VDEBUG_END(log);

    SipTo   to(sipMsg.getTo());
    to.setUrl(url);

    sipMsg.setTo(to);
}


void	    	    	    	
H323SipOpOut::updateFrom(SipMsg & sipMsg)
{
    const string    fn("H323SipOpOut::updateFrom");
    VLog    	    log(fn);

    Sptr<BaseUrl>   url = sipMsg.getFrom().getUrl();
    Sptr<SipUrl>    sipUrl;
    if ( sipUrl.dynamicCast(url) == 0 )
    {
        assert(0);
        return;
    }

    sipUrl->setHost(sipProxyInfo_.sipRemoteAddr().getIPAsString().c_str());
    sipUrl->setPort(Data((int)sipProxyInfo_.sipRemoteAddr().getPort()));

    VDEBUG(log) << fn << ": url = " << url->encode().logData() << VDEBUG_END(log);

    SipFrom from(sipMsg.getFrom());
    from.setUrl(url);

    sipMsg.setFrom(from);
}


void	    	    	    	
H323SipOpOut::addContact(SipMsg & sipMsg, bool basedOnTo)
{
    const string    fn("H323SipOpOut::addContact");
    VLog    	    log(fn);

    // Work around strangeness in sip stack.
    //
    sipMsg.setNumContact(0);
    
    Sptr<BaseUrl>   url;
    
    if ( basedOnTo )
    {
    	url = sipMsg.getTo().getUrl();
    }
    else
    {
    	url = sipMsg.getFrom().getUrl();
    }

    Sptr<SipUrl>    sipUrl;
    if ( sipUrl.dynamicCast(url) == 0 )
    {
        assert(0);
        return;
    }

    sipUrl->setHost(sipProxyInfo_.sipProxy().getIPAsString().c_str());
    sipUrl->setPort(Data((int)sipProxyInfo_.sipProxy().getPort()));

    VDEBUG(log) << fn << ": url = " << url->encode().logData() << VDEBUG_END(log);

    SipContact	contact;
    contact.setUrl(url);
    
    sipMsg.setContact(contact, 0);
}


Sptr<SipEvent>  
H323SipOpOut::getSipEvent(Sptr<SipProxyEvent> event)
{
    Sptr<OutgoingSipProxyEvent>     sipProxyEvent;
    
    if ( sipProxyEvent.dynamicCast(event) == 0 )
    {
        return ( 0 );
    }

    return ( sipProxyEvent->sipEvent() );    
}
