
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const ConnectionCallCtl_cxx_Version = 
"$Id: ConnectionCallCtl.cxx,v 1.15 2001/05/09 22:57:14 vzubarev Exp $";

#include <vector>
#include <string>

#include "VLog.hxx"
#include "TPKTClientSocket.hxx"

#include "ConnectionCallCtl.hxx"
#include "CallTrace.hxx"
#include "IPAddress.hxx"


using namespace router;
using Vocal::Logging::VLog;

ConnectionCallCtl::
ConnectionCallCtl(SMCall* sm)
    : ConnCallCtl(sm)
      , check_(new EvtTunnelingCheck(sm))
{
    assert(sm);
}


ConnectionCallCtl::
~ConnectionCallCtl()
{
    VLog log;
    VDEBUG(log) << "~ConnectionCallCtl" << VDEBUG_END(log);

    getSM()->stopTimer(check_);
}



void ConnectionCallCtl::
send(H245Message& message)
{
    VLog log;
    VDEBUG(log) << "ConnectionCallCtl::send" << VDEBUG_END(log);

#ifndef DISABLE_WRITER
    string  msgName;
    
    switch ( message.GetTag() )
    {
    	case H245Message::e_request:
	{
	    H245_RequestMessage & req = message;
	    msgName = string(req.GetTagName());
	    break;
	}
    	case H245Message::e_response:
	{
	    H245_ResponseMessage & resp = message;
	    msgName = string(resp.GetTagName());
	    break;
	}
    	case H245Message::e_command:
	{
	    H245_CommandMessage & cmd = message;
	    msgName = string(cmd.GetTagName());
	    break;
	}
    	case H245Message::e_indication:
	{
	    H245_IndicationMessage & ind = message;
	    msgName = string(ind.GetTagName());
	    break;
	}
	default:
	{
	    break;
	}
    }

    if (getSM()->getH245Socket() != 0)
    {
        CALLTRACE(log, CT_OUT, getSM()->getH245Socket()->getRemoteAddress(), CT_H245, msgName) 
            << "{\n  " << setprecision(2) << message 
            << "\n}" << CALLTRACE_END(log);
    }
    else
    {
        CALLTRACE(log, CT_OUT, IPAddress(), CT_H245_TUNNEL, msgName) 
            << "{\n  " << setprecision(2) << message 
            << "\n}" << CALLTRACE_END(log);
    }
#endif

    // Encode and send message to TCP socket
    PPER_Stream strm;
    message.Encode(strm);
    strm.CompleteEncoding();

    // Send it to TCP socket
    if (getSM()->getH245Socket() != 0)
    {
        // TODO: is there a better way to get uint8*?
        PASN_OctetString pos; pos = strm;

        VDEBUG(log) << "Sending H245 PDU"
                    << VDEBUG_END(log);

        getSM()->getH245Socket()->send(pos.GetPointer(), strm.GetSize());
    }
    else //if (getSM()->getCallSigConnection()->h245Tunneling())
    {
        getSM()->startTimer(check_);
        queue_.push(message);
    }
    /* If tunneling is closed but separate channel not yet up, still use tunn.
    else
    {
        VWARN(log) << "No socket to send H.245 message" << VWARN_END(log);
    }
    */
}


void ConnectionCallCtl::
routeImpl(EvtRouterType& evtRouter, SrcType& evtSource)
{
    VLog log;
    VDEBUG(log) << "ConnectionCallCtl::routeImpl" << VDEBUG_END(log);

    evtRouter.process(*getSM());
}


#ifndef DISABLE_WRITER

ostream& ConnectionCallCtl::
writeTo(ostream& out) const
{
    out << "ConnectionCallCtl: " << this
        << ", sm=" << getSM();
    return out;
}

#endif
