#ifndef INCLUDED_ConnectionCallSig_hxx
#define INCLUDED_ConnectionCallSig_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const ConnectionCallSig_hxx_Version = 
"$Id: ConnectionCallSig.hxx,v 1.14 2001/03/09 06:54:42 icahoon Exp $";

#include <queue>

#include "Sptr.hxx"
#include "TPKTClientSocket.hxx"

#include "CallIdentifier.hxx"

#include "mte/Connection.hxx"

#include "CallData.hxx"
#include "CallSigID.hxx"
#include "SMCall.hxx"
#include "EvtQ931Msg.hxx"
#include "Q931Defs.hxx"
#include "H245Defs.hxx"

namespace router
{

    using Vocal::Transport::TPKTClientSocket;
    using Vocal::SipH323Info::CallIdentifier;
    using Vocal::Transport::IPAddress;
    
    typedef mte::Connection<CallSigID, SMCall, EvtQ931Msg>
        ConnCallSig;

    /// Call signalling connection
    class ConnectionCallSig: public ConnCallSig
    {
    public:
        /// Construct connection assigning pointer to call state machine
        ConnectionCallSig(SMCall* sm);

        /// Set pointer to Q.931 client socket
        void setSocket(TPKTClientSocket* q931Socket);

        void close();

        void disable();

        void connect();

        /// Send Q.931 message to the connection
        void send(Q931Base& message);

        /// Process tunneled i. e. attachments
        void processAttachments(Q931Base& message);

        /// Analyze incoming message received from the connection
        void onIncoming(Q931Base& message);

        /// Prepare Setup message to be sent to the connection
        void prepareSetup(const CallData& data);

        void setToDisplay(const std::string& display);

        /// Send Setup message to the connection
        void sendSetup();

        /// Send CallProceeding message to the connection
        void sendCallProceeding();

        /// Send Alerting message to the connection
        void sendAlerting(BH225Alerting& alerting);

        /// Send Connect message to the connection
        void sendConnect(BH225Connect& connect);

        /// Send Release Complete message to the connection
        void sendReleaseComplete(int reason);

        void sendEmptyFacility();

        CallIdentifier getCallID();

        void connected() throw();

        void attachTunneling(Q931Base& message);

        void setH245(IPAddress remoteH245);

        bool haveRemoteH245() const throw();

        IPAddress getRemoteH245() const;

        IPAddress getRemoteAddress() const;

        void setOverrideH245() throw();

        bool overrideH245() const throw();

        void clearH245Tunneling() throw();

        bool h245Tunneling() const throw();

        void setFastConnect() throw();

        bool fastConnect() const throw();

        void addFSOLC(BH245OLC& olc);

        void sendFastStart(H225_ArrayOf_PASN_OctetString* target);

    private:
        /// Route event to state machine (implementation)
        virtual void routeImpl(EvtRouterType& evtRouter, SrcType& evtSource);
        
#ifndef DISABLE_WRITER
        virtual ostream& writeTo(ostream& out) const;
#endif

        bool outgoingCall_;
        CallData data_;
        TPKTClientSocket* socket_;
        bool connected_;
        bool haveRemoteH245_;
        IPAddress remoteH245_;
        bool overrideH245_;
        bool h245tunneling_;
        bool fastConnect_;
        std::queue<BH245OLC> queue_;
    };

}

#endif
