/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtTCS_cxx_Version = 
    "$Id: EvtTCS.cxx,v 1.13 2001/10/24 21:26:39 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "EvtTCS.hxx"
#include "FeatureCEI.hxx"
#include "FeatureH245.hxx"
#include "SMCall.hxx"
#include "EvtMsdRequest.hxx"

using namespace router;
using Vocal::Logging::VLog;

EvtTCS::
EvtTCS(RouterCallCtl* sm)
    : EvtCallCtlCEIRequest2(sm)
{
}


void EvtTCS::
inIdle(FeatureCEI& cei)
{
    const string    fn("EvtTCS::inIdle");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    processNewCapSet(cei);
}


void EvtTCS::
inWait(FeatureCEI& cei)
{
    const string    fn("EvtTCS::inWait");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    cei.callbackReject_.start();
    processNewCapSet(cei);
}


void EvtTCS::
inOther(FeatureCEI& cei)
{
    const string    fn("EvtTCS::inOther(FeatureCEI&)");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    assert(false);
}


void EvtTCS::
inPaused(FeatureH245& h245)
{
    const string    fn("EvtTCS::inPaused");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    if (isNonZero())
    {
	h245.gotTCS = true;
    }
    else
    {
	h245.gotTCS = false;
	h245.msdCompleted = false;
    }
}


void EvtTCS::
inTPaused(FeatureH245& h245)
{
    const string    fn("EvtTCS::inTPaused");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    if (isNonZero())
    {
	h245.gotTCS = true;

	// Start MSD procedure
	if (!h245.msdStarted)
	{
	    h245.msdCompleted = false;
	    EvtMsdRequest msd(h245.getSM());
	    msd.start();
	}

	h245.setState(&PatternH245::inTCE, "inTCE");
    }
    else
    {
	h245.gotTCS = false;
	h245.msdCompleted = false;
    }
}


void EvtTCS::
inTCE(FeatureH245& h245)
{
    const string    fn("EvtTCS::inTCE");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    if (isNonZero())
    {
	h245.gotTCS = true;
    }
    else
    {
	h245.gotTCS = false;
	h245.msdCompleted = false;
	h245.setState(&PatternH245::inTPaused, "inTPaused");
    }
}


void EvtTCS::
inOpen(FeatureH245& h245)
{
    const string    fn("EvtTCS::inOpen");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    if (isNonZero())
    {
	h245.gotTCS = true;
    }
    else
    {
	h245.gotTCS = false;
	h245.msdCompleted = false;
	h245.setState(&PatternH245::inTPaused, "inTPaused");
    }
}


void EvtTCS::
inOther(FeatureH245& h245)
{
    const string    fn("EvtTCS::inOther(FeatureH245&)");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    VWARN(log) << fn << ": Unexpected EvtTCS event for call "
               << h245.getSM() << VWARN_END(log);
}


bool EvtTCS::
isNonZero()
{
    // (depends from H.245 parser)
    return
	getMessage().HasOptionalField(
	    H245_TerminalCapabilitySet::e_multiplexCapability) ||
	getMessage().HasOptionalField(
	    H245_TerminalCapabilitySet::e_capabilityTable) ||
	getMessage().HasOptionalField(
	    H245_TerminalCapabilitySet::e_capabilityDescriptors);
}


void EvtTCS::
processNewCapSet(FeatureCEI& cei)
{
    const string    fn("EvtTCS::processNewCapSet");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    // (depends from H.245 parser)

    cei.in_SQ = getMessage().m_sequenceNumber;
    cei.setState(&PatternCEI::inWait, "inWait");
    reply(cei);
}


void EvtTCS::
reply(FeatureCEI& cei)
{
    // To be implemented in derived class
}


EvtH245B* EvtTCS::
clone_impl()
{
    return new EvtTCS(*this);
}


H245RequestTag EvtTCS::
getTag() const throw()
{
    // (depends from H.245 parser)
    return H245Request::e_terminalCapabilitySet;
}


#ifndef DISABLE_WRITER

ostream& EvtTCS::
writeTo(ostream& out) const
{
    out << "EvtTCS";
    EvtCallCtlCEIRequest2::writeTo(out);
    return out;
}

#endif
