
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const FeatureCallCtlSession_cxx_Version = 
"$Id: FeatureCallCtlSession.cxx,v 1.9 2001/10/19 21:37:34 icahoon Exp $";

#include "VLog.hxx"

#include "FeatureCallCtlSession.hxx"
#include "SMCall.hxx"

using namespace router;
using Vocal::Logging::VLog;


FeatureCallCtlSession::
FeatureCallCtlSession(SMCall* sm)
    : sm_(sm)
      , timer_(new EvtCallCtlSessionTimer(sm))
{
    setEntry(&entry_);
    setExit(&exit_);
    reset();
}


FeatureCallCtlSession::
~FeatureCallCtlSession()
{
    VLog log;
    VDEBUG(log) << "~FeatureCallCtlSession" << VDEBUG_END(log);

    // Call exit methods for any state
    setState(&PatternCallCtlSession::inClosed, "inClosed");
}


void FeatureCallCtlSession::
reset()
{
    setStateStealth(&PatternCallCtlSession::inClosed, "inClosed");
}


void FeatureCallCtlSession::
process_impl(PatternCallCtlSession& pattern)
{
    (pattern.*(getState()))(*this);
}


void FeatureCallCtlSession::
startTimer()
{
    assert(this);

    getSM()->startTimer(timer_);
}


void FeatureCallCtlSession::
stopTimer()
{
    assert(this);

    getSM()->stopTimer(timer_);
}


#ifndef DISABLE_WRITER

ostream& FeatureCallCtlSession::
writeTo(ostream& out) const
{
    out << "FeatureCallCtlSession:"
        << " state=" << getStateName();
    return out;
}


char* FeatureCallCtlSession::
getStateName() const
{
    if (false)
        return "";
    else if (getState() == &PatternCallCtlSession::inClosed)
        return "Closed";
    else if (getState() == &PatternCallCtlSession::inOpened)
        return "Opened";
    else if (getState() == &PatternCallCtlSession::inWaitEndSession)
        return "WaitEndSession";
    else if (getState() == &PatternCallCtlSession::inOther)
        return "Other";
    else
        return "UNKNOWN";
}

#endif
