#ifndef INCLUDED_FeatureH245_hxx
#define INCLUDED_FeatureH245_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF 1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const FeatureH245_hxx_Version = 
"$Id: FeatureH245.hxx,v 1.1 2001/03/09 06:54:42 icahoon Exp $";

#include "Sptr.hxx"

#include "mte/EventPtr.hxx"
#include "mte/Feature.hxx"

#include "H323Consts.hxx"
#include "H323RouterTypes.hxx"
#include "FeatureH245Defs.hxx"
#include "PatternH245.hxx"
#include "EntryH245.hxx"
#include "ExitH245.hxx"

namespace router
{

    class SMCall;
    
    /// H245 state variables
    class H245Data
    {
    public:
	bool msdStarted;
	bool msdCompleted;
	bool gotTCS;
	bool gotTCSAck;
	int attempt;
	int tcsFailures;

    private:
        H245Data()
            {}

        friend class FeatureH245;
    };

    /** Feature for H245 initial communication
        Implements initial communication procedures through H.245 channel
	described in H.323 document, clause 8.2.
    */
    class FeatureH245:
        public FeatureH245Base,
        public H245Data
    {
    public:
        /// Construct feature and assign pointer to state machine
        FeatureH245(SMCall* sm);
        
        virtual ~FeatureH245();

        /// Return pointer to the state machine
        SMCall* getSM() const;

        /// Reset feature
        virtual void reset();

        // Callback events

        mte::EventPtr<CallbackEvent> phaseB_;
        mte::EventPtr<CallbackEvent> phaseC_;
        mte::EventPtr<CallbackEvent> restartTCS_[N_TCS];

#ifndef DISABLE_WRITER
        virtual ostream& writeTo(ostream& out) const;
#endif

    private:
        /// Process event (implementation)
        virtual void process_impl(PatternH245& pattern);

#ifndef DISABLE_WRITER
        char* getStateName() const;
#endif
        
        SMCall* const sm_;
	EntryH245 entry_;
	ExitH245 exit_;
    };
    
    //----------------------------
    // Inline  methods
    //----------------------------

    inline SMCall* FeatureH245::
    getSM() const
    {
        return sm_;
    }

}

#endif
