/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const FeatureMsd_cxx_Version = 
    "$Id: FeatureMsd.cxx,v 1.12 2001/10/19 21:37:34 icahoon Exp $";


#include <cstdlib>

#include "VLog.hxx"

#include "FeatureMsd.hxx"
#include "SMCall.hxx"

using namespace router;
using Vocal::Logging::VLog;


FeatureMsd::
FeatureMsd(SMCall* sm)
    : sm_(sm),
      timer_(new EvtMsdTimer(sm))
{
    setEntry(&entry_);
    setExit(&exit_);
    reset();
}


FeatureMsd::
~FeatureMsd()
{
    // Call exit methods for any state
    setState(&PatternMsd::inIdle, "inIdle");
}


void FeatureMsd::
reset()
{
    VLog log;
    VDEBUG(log) << "FeatureMsd::reset" << VDEBUG_END(log);

    setStateStealth(&PatternMsd::inIdle, "inIdle");

    sv_STATUS = SV_Indeterminate;
    sv_NCOUNT = 1;
    set_sv_SDNUM();
}


void FeatureMsd::
set_sv_SDNUM()
{
    VLog log;
    VDEBUG(log) << "FeatureMsd::set_sv_SDNUM" << VDEBUG_END(log);

    // Assume 32-bit rand()
    sv_SDNUM = std::rand() & 0xFFFFFF;
}


void FeatureMsd::
determineStatus(int peer_TT, long peer_SDNUM)
{
    VLog log;
    VDEBUG(log) << "FeatureMsd::determineStatus" << VDEBUG_END(log);

    if (peer_TT == getSM()->getTerminalType())
    {
        int difference = (peer_SDNUM - sv_SDNUM) & 0xFFFFFF;

        if (difference == 0 ||
            difference == 0x800000)
        {
            sv_STATUS = SV_Indeterminate;
        }
        else
        {
            if (difference < 0x800000) // peer_SDNUM > sv_SDNUM
            {
                sv_STATUS = SV_Master;
            }
            else // peer_SDNUM < sv_SDNUM
            {
                sv_STATUS = SV_Slave;
            }
        }
    }
    else if (peer_TT < getSM()->getTerminalType())
    {
        sv_STATUS = SV_Master;
    }
    else // if (peer_TT > getSM()->getTerminalType())
    {
        sv_STATUS = SV_Slave;
    }
}


void FeatureMsd::
process_impl(PatternMsd& pattern)
{
    (pattern.*(getState()))(*this);
}


void FeatureMsd::
startTimer()
{
    assert(this);

    getSM()->startTimer(timer_);
}


void FeatureMsd::
stopTimer()
{
    assert(this);

    getSM()->stopTimer(timer_);
}


#ifndef DISABLE_WRITER

ostream& FeatureMsd::
writeTo(ostream& out) const
{
    out << "FeatureMsd:"
        << " state=" << getStateName()
        << " sv_STATUS=" << static_cast<unsigned>(sv_STATUS)
        << " sv_SDNUM=" << sv_SDNUM
        << " sv_NCOUNT=" << sv_NCOUNT;
    return out;
}


char* FeatureMsd::
getStateName() const
{
    if (getState() == &PatternMsd::inIdle)
        return "Idle";
    else if (getState() == &PatternMsd::inWaitOutgoing)
        return "WaitOutgoing";
    else if (getState() == &PatternMsd::inWaitIncoming)
        return "WaitIncoming";
    else
        return "UNKNOWN";
}

#endif
