
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const FeatureOpenCC_cxx_Version = 
"$Id: FeatureOpenCC.cxx,v 1.7 2001/10/19 21:37:34 icahoon Exp $";
#include "FeatureOpenCC.hxx"
#include "Call.hxx"

using namespace router;


FeatureOpenCC::
FeatureOpenCC(
    SMCall* sm,
    H245Protocol** h245ActiveProtocol
    )
    : sm_(sm),
      h245ActiveProtocol_(h245ActiveProtocol)
{
    reset();
}


FeatureOpenCC::
~FeatureOpenCC()
{
    // Call exit methods for any state
    setState(&PatternOpenCC::inIdle, "inIdle");
}


void FeatureOpenCC::
reset()
{
    setStateStealth(&PatternOpenCC::inIdle, "inIdle");
}


H245Protocol* FeatureOpenCC::
getH245Protocol() throw()
{
    assert(this);
    assert(h245ActiveProtocol_);
    assert(*h245ActiveProtocol_);

    return *h245ActiveProtocol_;
}


void FeatureOpenCC::
process_impl(PatternOpenCC& pattern)
{
    (pattern.*(getState()))(*this);
}


#ifndef DISABLE_WRITER

ostream& FeatureOpenCC::
writeTo(ostream& out) const
{
    out << "FeatureOpenCC:"
        << " state=" << getStateName();
    return out;
}


char* FeatureOpenCC::
getStateName() const
{
    if (false)
        return "";
    else if (getState() == &PatternOpenCC::inIdle)
        return "Idle";
    else if (getState() == &PatternOpenCC::inConfigured)
        return "Configured";
    else if (getState() == &PatternOpenCC::inConnected)
        return "Connected";
    else if (getState() == &PatternOpenCC::inClosing)
        return "Closing";
    else if (getState() == &PatternOpenCC::inDisallowed)
        return "Disallowed";
    else if (getState() == &PatternOpenCC::inOther)
        return "Other";
    else
        return "UNKNOWN";
}

#endif
