#ifndef INCLUDED_Q931Defs_hxx
#define INCLUDED_Q931Defs_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const Q931Defs_hxx_Version = 
"$Id: Q931Defs.hxx,v 1.7 2001/03/09 06:54:42 icahoon Exp $";

#include "Q931Base.hxx"

namespace router
{
    // Definitions specific to OpenH323

    typedef H225_H323_UU_PDU_h323_message_body H225Tag;
    typedef H225_ArrayOf_AliasAddress Aliases;
    typedef H225_AliasAddress Alias;
    typedef H225_EndpointType EndpointType;

    // Q.931 messages creation
    
    template<class Base, class Msg, H225Tag::Choices ttag>
    class Q931Derive: public Base
    {
    public:
        typedef Msg MsgType;

        Q931Derive();
        Msg& m();
    };

    template<class Base, class Msg, H225Tag::Choices ttag>
    inline Q931Derive<Base, Msg, ttag>::
    Q931Derive()
    {
        m_h323_uu_pdu.m_h323_message_body.SetTag(ttag);
    }

    template<class Base, class Msg, H225Tag::Choices ttag>
    inline Msg& Q931Derive<Base, Msg, ttag>::
    m()
    {
        return m_h323_uu_pdu.m_h323_message_body;
    }

    typedef H225_Setup_UUIE H225Setup;
    typedef Q931Derive<Q931Base, H225Setup,
        H225Tag::e_setup> BH225Setup;

    typedef H225_CallProceeding_UUIE H225CallProceeding;
    typedef Q931Derive<Q931Base, H225CallProceeding,
        H225Tag::e_callProceeding>
        BH225CallProceeding;

    typedef H225_Connect_UUIE H225Connect;
    typedef Q931Derive<Q931Base, H225Connect,
        H225Tag::e_connect> BH225Connect;

    typedef H225_Alerting_UUIE H225Alerting;
    typedef Q931Derive<Q931Base, H225Alerting,
        H225Tag::e_alerting> BH225Alerting;

    typedef H225_Information_UUIE H225Information;
    typedef Q931Derive<Q931Base, H225Information,
        H225Tag::e_information> BH225Information;

    typedef H225_ReleaseComplete_UUIE H225ReleaseComplete;
    typedef Q931Derive<Q931Base, H225ReleaseComplete,
        H225Tag::e_releaseComplete>
        BH225ReleaseComplete;

    typedef H225_Facility_UUIE H225Facility;
    typedef Q931Derive<Q931Base, H225Facility,
        H225Tag::e_facility> BH225Facility;

    typedef Q931Derive<Q931Base, H225Facility,
        H225Tag::e_empty> BH225Empty;

    typedef H225_Progress_UUIE H225Progress;
    typedef Q931Derive<Q931Base, H225Progress,
        H225Tag::e_progress> BH225Progress;


}

#endif
