
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const RouterLCI_cxx_Version = 
"$Id: RouterLCI.cxx,v 1.8 2001/05/09 22:57:14 vzubarev Exp $";

#include <assert.h>

#include "VLog.hxx"

#include "RouterLCI.hxx"
#include "LCI.hxx"
#include "SMCall.hxx"
#include "EvtH245.hxx"
#include "EvtOLC.hxx"

using namespace router;
using Vocal::Logging::VLog;


RouterLCI::
RouterLCI(SMCall& smcall)
    : smcall_(smcall)
{
}


RouterLCI::
~RouterLCI()
{
    VLog log;
    VDEBUG(log) << "~RouterLCI" << VDEBUG_END(log);

    assert(this);

    // While there are connections
    while (!empty())
    {
        // Remove SM for connection
        delete begin()->second->getSM();
        erase(begin());
    }
}


bool RouterLCI::
route(EvtRouterType& evtRouter, SrcType& evtSource)
{
    assert(this);
    VLog log;

    if (RtLCI::route(evtRouter, evtSource))
    {
        return true;
    }
    else
    {
	if (dynamic_cast<EvtOLC*>(&evtSource))
	{
	    VINFO(log) << "Starting new LCI procedure" << VINFO_END(log);
	    LCI* lci = new LCI(&smcall_, evtSource.getLCN());

	    if (add(*(lci->getLCIConnection())))
	    {
		return RtLCI::route(evtRouter, evtSource);
	    }
	    else
	    {
		VWARN(log) << "Starting new LCI procedure failed"
			   << VWARN_END(log);
		delete lci;
		return false;
	    }
	}
	else
	{
	    VWARN(log) << "Received LCI message but didn't get OLC"
		       << VWARN_END(log);
	    return false;
	}
    }
}


#ifndef DISABLE_WRITER

ostream& RouterLCI::
writeTo(ostream& out) const
{
    assert(this);

    out << "RouterLCI: " << this;
    return out;
}

#endif
