/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const SipToH323RegistrationPolicy_cxx_Version = 
    "$Id: SipToH323RegistrationPolicy.cxx,v 1.5 2002/07/10 22:09:00 icahoon Exp $";


#include "SipToH323RegistrationPolicy.hxx"
#include "H323RegistrationEvent.hxx"
#include "Service.hxx"
#include "VLog.hxx"


using namespace Vocal::SipH323CSGW;
using Vocal::Logging::VLog;


SipToH323RegistrationPolicy::SipToH323RegistrationPolicy(
    Service 	    	    &	service,
    RASGKProtocol   	    &	rasGkProtocol,
    GatekeeperInfo  	    &	gatekeeperInfo,
    EndpointInfoMap 	    &	endpointInfoMap,
    FifoBase< Sptr<Event> > & 	sipProxyFifo
)
    :   DefaultRegistrationPolicy(service, rasGkProtocol, 
    	    	    	    	  gatekeeperInfo, endpointInfoMap),
	mySipProxyFifo(sipProxyFifo),
	myH323RegistrationEvent(0)
{
    const string    	fn("SipToH323RegistrationPolicy::SipToH323RegistrationPolicy");
    VLog    	    	log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);
}


SipToH323RegistrationPolicy::~SipToH323RegistrationPolicy()
{
    const string    	fn("~SipToH323RegistrationPolicy::~SipToH323RegistrationPolicy");
    VLog    	    	log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);
}


bool	    
SipToH323RegistrationPolicy::interestedIn(const Sptr<Event> & event)
{
    const string    	fn("SipToH323RegistrationPolicy::interestedIn");
    VLog    	    	log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    myH323RegistrationEvent.dynamicCast(event);
    
    if ( myH323RegistrationEvent != 0 )
    {
    	return ( true );
    }
        
    return ( DefaultRegistrationPolicy::interestedIn(event) );
}


void	    
SipToH323RegistrationPolicy::onEvent(Sptr<Event> event)
{
    const string    	fn("SipToH323RegistrationPolicy::onEvent");
    VLog    	    	log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    if ( myH323RegistrationEvent == 0 )
    {    
    	DefaultRegistrationPolicy::onEvent(event);
    	return;
    }

    if ( myH323RegistrationEvent->registration() )
    {    
	processRegistrationRequest(myH323RegistrationEvent->rasPDU(), 
				   myH323RegistrationEvent->remoteAddress());
    }
    else
    {
    	processUnregistrationRequest(myH323RegistrationEvent->rasPDU(), 
				     myH323RegistrationEvent->remoteAddress());
    }
    
    myH323RegistrationEvent = 0;    
}


ostream &           
SipToH323RegistrationPolicy::writeTo(ostream & out) const
{
    return ( out << "SipToH323RegistrationPolicy" );
}


Vocal::H323GK::RegistrationPolicy::Proceed	    
SipToH323RegistrationPolicy::onKeepAliveRegistration(
    Sptr<H225_RasMessage>	        rasPDU, 
    Sptr<IPAddress> 	    	        remoteAddr,
    Sptr<EndpointInfo>	    	    	endpointInfo,
    H225_RegistrationRequest        &   karrq,
    H225_RegistrationRejectReason   &   reason
)
{
    return ( onRegistration(rasPDU, remoteAddr, endpointInfo, karrq, reason) );
}


Vocal::H323GK::RegistrationPolicy::Proceed	    
SipToH323RegistrationPolicy::onRegistration(
    Sptr<H225_RasMessage>    	    	rasPDU, 
    Sptr<IPAddress> 	    	    	remoteAddr,
    Sptr<EndpointInfo>	    	    	endpointInfo,
    H225_RegistrationRequest        &   rrq,
    H225_RegistrationRejectReason   &   reason
)
{
    const string    	fn("SipToH323RegistrationPolicy::onRegistration");
    VLog     	    	log(fn);
   
    VDEBUG(log) << fn << VDEBUG_END(log);

    // If this is a registration from a gateway, or in stand alone mode,
    // don't send the SIP registration.
    //
    if  (   endpointInfo->endpointType() == EndpointInfo::GATEWAY_TYPE
        ||  myGatekeeperInfo.standAlone() 
        )
    {
    	return ( DefaultRegistrationPolicy::onRegistration(
    	    	    rasPDU, remoteAddr, endpointInfo, rrq, reason) );
    }
    
    // If this is a new registration.
    //   
    if ( myH323RegistrationEvent == 0 )
    {
    	Sptr<H323RegistrationEvent> h323RegistrationEvent 
	    = new H323RegistrationEvent(endpointInfo, myGatekeeperInfo, 
	    	    	    	    	rasPDU, remoteAddr,
					true, myService.getFifo());
	
	mySipProxyFifo.add(h323RegistrationEvent);
	
    	return ( wait );
    }
    
    // This is a registration response from the proxy.
    //
    if  (   myH323RegistrationEvent->returnCode() == SUCCESS
    	||  myH323RegistrationEvent->returnCode() == 200
    	)    
    {
    	return ( DefaultRegistrationPolicy::onRegistration(
    	    	    rasPDU, remoteAddr, endpointInfo, rrq, reason) );
    }

    return ( reject );
}


Vocal::H323GK::RegistrationPolicy::Proceed     
SipToH323RegistrationPolicy::onUnregistration(
    Sptr<H225_RasMessage>    	    	rasPDU, 
    Sptr<IPAddress> 	    	    	remoteAddr,
    Sptr<EndpointInfo>	    	    	endpointInfo,
    H225_UnregistrationRequest      &   urq,
    H225_UnregRejectReason          &   reason
)
{
    const string    	fn("SipToH323RegistrationPolicy::onUnregistration");
    VLog    	    	log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    if  (   endpointInfo->endpointType() == EndpointInfo::GATEWAY_TYPE
        ||  myGatekeeperInfo.standAlone()  
        )
    {
    	return ( DefaultRegistrationPolicy::onUnregistration(
    	    	    rasPDU, remoteAddr, endpointInfo, urq, reason) );
    }
    
    // If this is a new unregistration.
    //   
    if ( myH323RegistrationEvent == 0 )
    {
    	Sptr<H323RegistrationEvent> h323RegistrationEvent 
	    = new H323RegistrationEvent(endpointInfo, myGatekeeperInfo, 
	    	    	    	    	rasPDU, remoteAddr,
					false, myService.getFifo());
	
	mySipProxyFifo.add(h323RegistrationEvent);
	
    	return ( wait );
    }
    
    // This is a registration response from the proxy.
    //
    if  (   myH323RegistrationEvent->returnCode() == SUCCESS
    	||  myH323RegistrationEvent->returnCode() == 200
    	)    
    {
    	return ( DefaultRegistrationPolicy::onUnregistration(
    	    	    rasPDU, remoteAddr, endpointInfo, urq, reason) );
    }

    return ( reject );
}
