#!/bin/bash
#
# $Id: calltrace_generator.cgi,v 1.2 2002/05/15 07:33:55 icahoon Exp $
#
# ====================================================================
# The Vovida Software License, Version 1.0 
# 
# Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. The names "VOCAL", "Vovida Open Communication Application Library",
#    and "Vovida Open Communication Application Library (VOCAL)" must
#    not be used to endorse or promote products derived from this
#    software without prior written permission. For written
#    permission, please contact vocal\@vovida.org.
# 
# 4. Products derived from this software may not be called "VOCAL", nor
#    may "VOCAL" appear in their name, without prior written
#    permission of Vovida Networks, Inc.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
# NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
# NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
# IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES LOSS OF USE, DATA, OR
# PROFITS OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
# 
# ====================================================================
# 
# This software consists of voluntary contributions made by Vovida
# Networks, Inc. and many individuals on behalf of Vovida Networks,
# Inc.  For more information on Vovida Networks, Inc., please see
# <http://www.vovida.org/>.
#

version="\$Id: calltrace_generator.cgi,v 1.2 2002/05/15 07:33:55 icahoon Exp $"

######################################################################
#
#   Load libraries
#
######################################################################

lib_path="${0%/$(basename $0)}"

. $lib_path/vocal_html_lib

log_level=$log_warning

log $log_info "\nVersion: $version\nCommand: $0 $cmd_line"
log $log_info "log level:             $log_level"

######################################################################
#
#   Local functions
#
######################################################################

function    get_date_and_time()
{
    log $log_verbose "get_date_and_time: $*"

    local date=$($head -1 "$*")
    date="${date%% *}"

    log $log_verbose "$date"

    $print "$date"
}

function   get_gkid()
{
    log $log_verbose "get_gkid: $*"
    
    local   gkid=$($head -30 "$1" | $egrep "gatekeeperIdentifier:")
    gkid=${gkid#*: }
    
    $print $gkid
}

#######################################################################
#
#   Title / Branding
#
#######################################################################

title="Sip to H.323 Call Signaling Gateway: Call Trace Generator"

log $log_info "title:                 $title"

title_html="&nbsp;"

background_image="$image_dir/crumpled.jpg"
brand_image="$image_dir/call_trace.gif"
brand_image_span="8"
brand_image_align="center"
brand_name="Cisco"
brand_link="./calltrace_generator.cgi"

#######################################################################
#
#   Servers
#
#######################################################################

servers="lithium sodium calcium hydrogen krypton"
server_path_prefix="./h323testbed"
log_path_postfix="log/siph323csgw"

log $log_info "servers:               $servers"
log $log_info "server path prefix:    $server_path_prefix"
log $log_info "log path postfix:      $log_path_postfix"

num_servers="$($print $servers | $wc -w)"

log $log_info "number servers:        $num_servers"

#######################################################################
#
#   Find the number of rows
#
#######################################################################

num_rows=0

let "num_rows = 1 + 4 * num_servers + 1"

for server in $servers
do
    num_logs=$($ls -1 $server_path_prefix/$server/$log_path_postfix/*.[lt][ox][gt] 2>/dev/null | $wc -l)
        
    if [ $num_logs -gt 0 ]; then
	let "num_rows = num_rows + num_logs"
    fi
    
    log $log_info "server: $server,\tnumber logs: $num_logs,\tnumber rows: $num_rows"
done

#######################################################################
#
#   Server background color
#
#######################################################################

even_bg_color="#EEEEEE"
odd_bg_color="#DDDDDD"

bg_color=""

function get_next_bg_color()
{
    if [ "$bg_color" = "$even_bg_color" ]; then
    	$print "$odd_bg_color"
    else
    	$print "$even_bg_color"
    fi
}


#######################################################################
#
#   Common html: clear_image, table_outline, server_table_cap
#
#######################################################################


clear_image="<img width=\"1\" border=\"0\" src=\"images/clear.gif\">"

table_outline="
$row_begin
$(cell_begin 0 10 0 center center \#000000 5)${clear_image}$cell_end
$row_end
"

server_table_cap="
$row_begin
$(cell_begin 10 0 $num_rows) &nbsp; $cell_end
$(cell_begin)${clear_image}$cell_end
$(cell_begin 10) &nbsp; $cell_end
$(cell_begin 10) &nbsp; $cell_end
$(cell_begin 10) &nbsp; $cell_end
$(cell_begin 10) &nbsp; $cell_end
$(cell_begin 10) &nbsp; $cell_end
$(cell_begin 10) &nbsp; $cell_end
$(cell_begin 10) &nbsp; $cell_end
$(cell_begin 10) &nbsp; $cell_end
$(cell_begin)${clear_image}$cell_end
$(cell_begin 9 0 $num_rows) &nbsp; $cell_end
$row_end"

#######################################################################
#
#   Misc variables
#
#######################################################################

output_directory="siph323csgw"
green="#AAFFAA"
yellow="#FFFFAA"
red="#FFAAAA"
make_call_trace="./make_call_trace.cgi"

#######################################################################
#
#   Generate server table entry
#
#######################################################################

function    blank_server_entry_row()
{
    $print "
$row_begin
$(cell_begin 0 0 0 center center \#000000 5)${clear_image}$cell_end
$(cell_begin 0 8 0 center center $bg_color) &nbsp; $cell_end
$(cell_begin 0 0 0 center center \#000000 5)${clear_image}$cell_end
$row_end
"
}

function    server_table_entry()
{
    local   server="$*"

    local   gatekeeper_instance=""


    $print "
$table_outline    
$row_begin
$(cell_begin 0 0 0 center center \#000000 5)${clear_image}$cell_end
$(cell_begin 0 0 0 left center $bg_color)
${content_indent}<br>&nbsp; 
${content_indent_2}$(strong "Server:")
${cell_indent}$cell_end
$(cell_begin 0 7 0 left center $bg_color)
${content_indent}<br>$(strong "$server")
${cell_indent}$cell_end
$(cell_begin 0 0 0 center center \#000000 5)${clear_image}$cell_end
$row_end

$row_begin
$(cell_begin 0 0 0 center center \#000000 5)${clear_image}$cell_end
$(cell_begin 0 8 0 left center $bg_color)
${content_indent}<br>&nbsp; 
${content_indent}$(strong "Logs:")
${cell_indent}$cell_end
$(cell_begin 0 0 0 center center \#000000 5)${clear_image}$cell_end
$row_end"

    for in_file in $($ls -1 $server_path_prefix/$server/$log_path_postfix/*.[lt][ox][gt] 2>/dev/null)
    do
        gatekeeper_instance="$(get_gkid $in_file).$(get_date_and_time $in_file)"
	
    	log $log_info "log file: $in_file, gatekeeper instance: $gatekeeper_instance"
	
    	$print "
$row_begin
$(cell_begin 0 0 0 center center \#000000 5)${clear_image}$cell_end
$(cell_begin 0 0 0 left center $bg_color)
${content_indent}&nbsp; 
${cell_indent}$cell_end
$(cell_begin 0 0 0 left center $bg_color)
${content_indent}<a href=\"$in_file\">
${content_indent_2}$($basename $in_file)</a>
${cell_indent}$cell_end"

    	if [ -r ./$output_directory/$gatekeeper_instance/index.html ]; then
    	    $print "	    
$(cell_begin 0 2 0 left center $green)
${content_indent}&nbsp;
${content_indent}<a href=\"./$output_directory/$gatekeeper_instance/index.html\">
${content_indent_2}View call trace.</a>
${cell_indent}$cell_end
$(cell_begin 0 2 0 left center $yellow)
${content_indent}&nbsp;
${content_indent}<a href=\"./$make_call_trace?server=$server&log=$($basename $in_file)\">
${content_indent_2}Update call trace.</a>
${cell_indent}$cell_end
$(cell_begin 0 2 0 left center $bg_color)
${content_indent}&nbsp; 
${cell_indent}$cell_end"
	else
    	    $print "	    
$(cell_begin 0 2 0 left center $bg_color)
${content_indent}&nbsp; 
${cell_indent}$cell_end
$(cell_begin 0 2 0 left center $bg_color)
${content_indent}&nbsp; 
${cell_indent}$cell_end
$(cell_begin 0 2 0 left center $red)
${content_indent}&nbsp;
${content_indent}<a href=\"./$make_call_trace?server=$server&log=$($basename $in_file)\">
${content_indent_2}Create call trace.</a>"
	fi
	
    	$print "
$(cell_begin 0 0 0 center center \#000000 5)${clear_image}$cell_end
$row_end"

    done

$print "$(blank_server_entry_row)"

}

#######################################################################
#
#   Generate html
#
#######################################################################

$print "$(content_type)

$(html_start) $(brand_information)
$table_begin
$server_table_cap"

for server in $servers
do
    bg_color=$(get_next_bg_color)
    $print "$(server_table_entry $server)"
done

$print "$table_outline
$(blank_row 12 \#FFFFFF "&nbsp;<br><br><br>")\n
$table_end
$html_end"
