#if !defined(RAS_CLIENT_HXX)
#define RAS_CLIENT_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2001 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const RasClient_hxx_Version = 
    "$Id: RasClient.hxx,v 1.2 2002/05/15 07:33:56 icahoon Exp $";


#include "ptlib.h"
#include "h323pdu.h"
#include "VocalCommon.hxx"
#include "NonCopyable.hxx"
#include "UDPSocket.hxx"
#include "IPAddress.hxx"


namespace Vocal
{


using namespace Transport;


class RasConfig;


class RasClient : public NonCopyable
{
    public:

    	RasClient(const RasConfig &);
	virtual ~RasClient();


        virtual ReturnCode          discoverGatekeeper();
        virtual ReturnCode          registerEndpoint();
        virtual ReturnCode          unregisterEndpoint();
        virtual ReturnCode          admitCall();
        virtual ReturnCode          disengageCall();
        virtual ReturnCode          locateEndpoint();
        virtual ReturnCode          reportResources();

    protected:

        virtual ReturnCode          discoverGatekeeperResponse();
        virtual ReturnCode          registerEndpointResponse();
        virtual ReturnCode          unregisterEndpointResponse();
        virtual ReturnCode          admitCallResponse();
        virtual ReturnCode          disengageCallResponse();
        virtual ReturnCode          locateEndpointResponse();
        virtual ReturnCode          reportResourcesResponse();

        UDPSocket &                 socket();
    
        H225_TransportAddress       getIPAddress(
            const IPAddress                 &   ipAddr
        ) const;
        IPAddress                   getTransportAddress(
            const H225_TransportAddress     &   transAddr
        ) const;
        IPAddress                   getIPAddress(
            const H225_ArrayOf_TransportAddress & 
        ) const;
        
        H225_VendorIdentifier       buildVendorIdentifier() const;
        H225_EndpointType           buildEndpointType() const;
        H225_ArrayOf_AliasAddress   buildAliases() const;
        H225_ArrayOf_AliasAddress   buildAlias() const;

        void        appendAddress(H225_ArrayOf_TransportAddress &, IPAddress) const;
        IPAddress   getIPAdderss(const H225_TransportAddress &) const;
        
        void                        encode();
        ReturnCode                  decode();
        ReturnCode                  send();
        ReturnCode                  receive();
        bool                        sequenceNumberMatch() const;
    
	const RasConfig     &   myConfig;
        
    private:
    
        UDPSocket           *   mySocket;
        IPAddress               myGkAddr;   

        bool                    myEncoded;
        H225_RasMessage	        outPdu;
        H225_RasMessage	    	inPdu;

        PPER_Stream             myOut;
        PPER_Stream             myIn;
        size_t	    	    	myInSize;
};


} // namespace Vocal


#endif // !defined(RAS_CLIENT_HXX)
