/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2001 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const RasConfig_cxx_Version = 
    "$Id: RasConfig.cxx,v 1.2 2002/05/15 07:33:56 icahoon Exp $";


#include "RasCommon.hxx"
#include "RasConfig.hxx"
#include "NameValue.hxx"
#include "vstring.hxx"
#include <iostream>


using namespace Vocal;
using namespace Vocal::Configuration;
using namespace Vocal::Transport;
using namespace std;



RasConfig::RasConfig()
    :   GRQStr("grq"),
        RRQStr("rrq"),
        URQStr("urq"),
        ARQStr("arq"),
        DRQStr("drq"),
        LRQStr("lrq"),
        RAIStr("rai"),
        RasAddrStr("ras-address"),
        CallSigAddrStr("call-sig-address"),
        EndpointIdStr("endpoint-id"),
        SequenceNumberStr("sequence-number"),
        TerminalStr("terminal"),
        GatewayStr("gateway"),
        MCUStr("mcu"),
        PhoneStr("phone"),
        UserStr("user"),
        ProductIdStr("product"),
        VersionIdStr("version"),
        H221Str("h221"),
        DiscCompleteStr("discovery-complete"),
        TimeToLiveStr("time-to-live"),
        DirectStr("direct"),
        GkRoutedStr("gatekeeper-routed"),
        DestPhoneStr("dest-phone"),
        DestUserStr("dest-user"),
        DestCallSigAddrStr("dest-call-sig-address"),
        CRVStr("crv"),
        BandwidthStr("bandwidth"),
        ConfIdStr("conf-id"),
        CallIdStr("call-id"),
        AnswerStr("answer"),
        MCStr("mc"),
        MulticastStr("multicast"),
        AddressStr("address"),
        OutOfResourcesStr("out-of-resources"),
        RetryCountStr("retry-count"),
        RetryDelayStr("retry-delay"),
        KeepAliveStr("keepalive"),
        GkIdStr("gk-id"),
        DebugStr("debug"),
        
        myType(Unknown_MessageType),
        myRasAddr(),
        myCallSigAddr(),
        myEndpointId(""),
        mySeqNum(0),
        myEndpointType(Unknown_EndpointType),
        myPhone(""),
        myUser(""),
        myProductId(""),
        myVersionId(""),
        myCountryCode(0),
        myExtension(0),
        myManufacturerCode(0),
        myDiscoveryComplete(false),
        myTimeToLive(0),
        myCallType(PointToPoint),
        myCallModel(Unknown_CallModel),
        myDestPhone(""),
        myDestCallSigAddr(),
        myCRV(0),
        myBandwidth(0),
        myConfId(""),
        myCallId(""),
        myAnswerCall(false),
        myMC(false),
        myMulticast(false),
        myAddress(),
        myOutOfResources(false),
        myRetryCount(0),
        myRetryDelay(0),
        myKeepAlive(false),
        myGkId(""),
        myGkAddr(),
        myDebug(false)
{
}


RasConfig::~RasConfig()
{
}


void        
RasConfig::usage() const
{
    string IN("  ");
    
    cout << "usage: " << myApplicationName << " [options] gk_address" << NL

        << IN << "--" << GRQStr 
        << TAB3 << "Send a GatekeeperRequest     (GRQ)" << NL

        << IN << "--" << RRQStr 
        << TAB3 << "Send a RegistrationRequest   (RRQ)" << NL

        << IN << "--" << URQStr 
        << TAB3 << "Send a UnregistrationRequest (URQ)" << NL

        << IN << "--" << ARQStr 
        << TAB3 << "Send a AdmissionRequest      (ARQ)" << NL

        << IN << "--" << DRQStr 
        << TAB3 << "Send a DisengageRequest      (DRQ)" << NL

        << IN << "--" << LRQStr 
        << TAB3 << "Send a LocationRequest       (LRQ)" << NL

        << IN << "--" << RAIStr 
        << TAB3 << "Send a Resources Available Indication (RAI)" << NL << NL


        << IN << "--" << RasAddrStr 
        << " ip" << TAB2 << "RAS address of this endpoint." << NL

        << IN << "--" << CallSigAddrStr 
        << " ip" << TAB1 << "Call signaling address of this endpoint." << NL 

        << IN << "--" << DestCallSigAddrStr
        << " ip" << TAB1 << "Call signaling address of destination endpoint." << NL << NL


        << IN << "--" << EndpointIdStr
        << " string" << TAB2 << "String to identify the endpoint" << NL << NL

        << IN << "--" << SequenceNumberStr
        << " n" << TAB2 << "Sequence number of this message" << NL << NL

        << IN << "--" << TerminalStr 
        << TAB2 << "Message is coming from a terminal" << NL

        << IN << "--" << GatewayStr 
        << TAB2 << "Message is coming from a gateway" << NL

        << IN << "--" << MCUStr
        << TAB2 << "Message is coming from an MCU" << NL 
        
        << IN << "--" << MCStr
        << TAB3 << "Message is coming from an MC. Assumed set for MCU" << NL << NL


        << IN << "--" << PhoneStr 
        << " phone" << TAB1 << "Phone number of this endpoint" << NL

        << IN << "--" << UserStr
        << " string" << TAB1 << "User associated with this endpoint" << NL

        << IN << "--" << DestPhoneStr
        << " phone " << TAB1 << "Phone number of the destination endpoint" << NL

        << IN << "--" << DestUserStr
        << " string" << TAB1 << "User associated with the destination endpoint" << NL << NL

        << IN << "--" << ProductIdStr
        << " string" << TAB1 << "Product identifier for this endpoint" << NL

        << IN << "--" << VersionIdStr
        << " string" << TAB1 << "Version identifier for this endpoint" << NL

        << IN << "--" << H221Str
        << " x,y,z" << TAB2 << "H.221 Non standard identifier" << NL << NL

        
        << IN << "--" << DiscCompleteStr
        << TAB1 << "Indicates whether discovery has taken place." << NL 
        
        << IN << "--" << TimeToLiveStr
        << " sec" << TAB1 << "Incidate the desired time to live." << NL << NL
        

        << IN << "--" << DirectStr
        << TAB2 << "Call will be place directly to the destination" << NL

        << IN << "--" << GkRoutedStr
        << TAB1 << "Call will be place indirectly thru gatekeeper" << NL << NL
        
        << IN << "--" << CRVStr
        << " number" << TAB2 << "Call reference value" << NL

        << IN << "--" << ConfIdStr
        << " string" << TAB1 << "Conference identifier" << NL

        << IN << "--" << CallIdStr
        << " string" << TAB1 << "Call identifier" << NL << NL

        << IN << "--" << BandwidthStr
        << TAB2 << "Bandwidth to allocate for this call" << NL

        << IN << "--" << AnswerStr
        << TAB2 << "Answer the call" << NL << NL
        
        << IN << "--" << MulticastStr
        << TAB2 << "Transmit message via well known multicast address" << NL << NL
        
        << IN << "--" << AddressStr
        << TAB2 << "Use this address to transmit message. If different from" << NL
        << TAB3 << "ras_address, ras_address will be the advertised address" << NL
        << TAB3 << "in the message, and address will be used to send the" << NL
        << TAB3 << "message." << NL << NL
        
        << IN << "--" << OutOfResourcesStr
        << TAB1 << "For an RAI, indicates endpoint is swamped" << NL << NL
        
        << IN << "--" << RetryCountStr
        << " count" << TAB1 << "Number of times to retry the message" << NL 
        
        << IN << "--" << RetryDelayStr
        << " seconds" << " Time between retransmissions of the message" << NL << NL
        
        << IN << "--" << KeepAliveStr
        << TAB2 << "The RRQ is a keep alive" << NL << NL
        
        << IN << "--" << GkIdStr
        << " ip" << TAB2 << "Identifier of the gatekeeper" << NL << NL
        
        << IN << "--" << DebugStr
        << TAB2 << "Turns on debugging information" << NL;
}


ReturnCode  
RasConfig::preParse()
{
    // Don't call config's preParse.
    //
    ReturnCode  rc = SUCCESS;

    if  (   (rc = myOptions.add(GRQStr)) != SUCCESS
        ||  (rc = myOptions.add(RRQStr)) != SUCCESS
        ||  (rc = myOptions.add(URQStr)) != SUCCESS
        ||  (rc = myOptions.add(ARQStr)) != SUCCESS
        ||  (rc = myOptions.add(DRQStr)) != SUCCESS
        ||  (rc = myOptions.add(LRQStr)) != SUCCESS
        ||  (rc = myOptions.add(RAIStr)) != SUCCESS
        ||  (rc = myOptions.add(RasAddrStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(CallSigAddrStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(EndpointIdStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(SequenceNumberStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(TerminalStr)) != SUCCESS
        ||  (rc = myOptions.add(GatewayStr)) != SUCCESS
        ||  (rc = myOptions.add(MCUStr)) != SUCCESS
        ||  (rc = myOptions.add(PhoneStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(UserStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(ProductIdStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(VersionIdStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(H221Str, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(DiscCompleteStr)) != SUCCESS
        ||  (rc = myOptions.add(TimeToLiveStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(DirectStr)) != SUCCESS
        ||  (rc = myOptions.add(GkRoutedStr)) != SUCCESS
        ||  (rc = myOptions.add(DestPhoneStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(DestUserStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(DestCallSigAddrStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(CRVStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(BandwidthStr)) != SUCCESS
        ||  (rc = myOptions.add(ConfIdStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(CallIdStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(AnswerStr)) != SUCCESS
        ||  (rc = myOptions.add(MCStr)) != SUCCESS
        ||  (rc = myOptions.add(MulticastStr)) != SUCCESS
        ||  (rc = myOptions.add(AddressStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(OutOfResourcesStr)) != SUCCESS
        ||  (rc = myOptions.add(RetryCountStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(RetryDelayStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(KeepAliveStr)) != SUCCESS
        ||  (rc = myOptions.add(GkIdStr, 0, VALUE_PRESENT)) != SUCCESS
        ||  (rc = myOptions.add(DebugStr)) != SUCCESS
        )
    {
        // Do nothing since rc is already set on failure.
    }

    return ( rc );
}


ReturnCode  
RasConfig::postParse()
{
    size_t i = 0;

    if ( myOptions.get(GRQStr) )
    {
        myType = GRQ;
        ++i;
    }
    if ( myOptions.get(RRQStr) )
    {
        myType = RRQ;
        ++i;
    }
    if ( myOptions.get(URQStr) )
    {
        myType = URQ;
        ++i;
    }
    if ( myOptions.get(ARQStr) )
    {
        myType = ARQ;
        ++i;
    }
    if ( myOptions.get(DRQStr) )
    {
        myType = DRQ;
        ++i;
    }
    if ( myOptions.get(LRQStr) )
    {
        myType = LRQ;
        ++i;
    }
    if ( myOptions.get(RAIStr) )
    {
        myType = RAI;
        ++i;
    }
    if ( i != 1 )
    {
        if ( i == 0 )
        {
            cerr << "Error: must specify a message type." << endl;
        }
        else
        {
            cerr << "Error: May only specify one of " 
                 << GRQStr << ", " << RRQStr << ", " << URQStr << ", "
                 << ARQStr << ", " << DRQStr << ", " << LRQStr << " or "
                 << RAIStr << NL << endl;
        }
        return ( CONFIG_ERROR );
    }
    
    string  ras_addr;
    if ( myOptions.get(RasAddrStr, ras_addr) )
    {
        myRasAddr.setIP(ras_addr);
    }
    if ( ( myType == RRQ || myType == GRQ ) && myRasAddr.isAny() )
    {
        cerr << "Error: must specify a ras address." << endl;
        return ( CONFIG_ERROR );
    }
    
    string cs_addr;
    if ( myOptions.get(CallSigAddrStr, cs_addr) )
    {
        myCallSigAddr.setIP(cs_addr);
    }
    if ( ( myType == RRQ || myType == URQ ) && myCallSigAddr.isAny() )
    {
        cerr << "Error: must specify a call signal address." << endl;
        return ( CONFIG_ERROR );
    }
    
    
    myOptions.get(EndpointIdStr, myEndpointId);
    if ( ( myType == URQ || myType == ARQ || myType == LRQ || myType == DRQ || 
           myType == RAI ) && myEndpointId.size() == 0
       )
    {
        cerr << "Error: must specify an endpoint id." << endl;
        return ( CONFIG_ERROR );
    }
    
    int sn = 0;
    myOptions.get(SequenceNumberStr, sn);
    mySeqNum = sn;
    if ( mySeqNum == 0 )
    {
        cerr << "Error: sequence number is mandatory and cannot be 0" << endl;
        return ( CONFIG_ERROR );
    }

    i = 0;
    if ( myOptions.get(TerminalStr) )
    {
        myEndpointType = Terminal;
        ++i;
    }
    if ( myOptions.get(GatewayStr) )
    {
        myEndpointType = Gateway;
        ++i;
    }
    if ( myOptions.get(MCUStr) )
    {
        myEndpointType = MCU;
        myMC = true;
        ++i;
    }
    if ( ( myType == GRQ || myType == RRQ ) && i != 1 )
    {
        if ( i == 0 )
        {
            cerr << "Error: must specify an endpoint type." << endl;
        }
        else
        {
            cerr << "Error: May only specify one of "
                 << TerminalStr << ", " << GatewayStr << " or "
                 << MCUStr << NL << endl;
        }
        return ( CONFIG_ERROR );
    }
    
    myOptions.get(PhoneStr, myPhone);
    myOptions.get(UserStr, myUser);

    myOptions.get(ProductIdStr, myProductId);
    myOptions.get(VersionIdStr, myVersionId);
    
    string h221;
    if ( myOptions.get(H221Str, h221) )
    {
        string  c, e, m;
        
        c = getnext(h221, ",");
        e = getnext(h221, ",");
        m = getnext(h221, ",");

        myCountryCode = atoi(c.c_str());
        myExtension = atoi(e.c_str());
        myManufacturerCode = atoi(m.c_str());
    }

    myDiscoveryComplete = myOptions.get(DiscCompleteStr);

    int ttl = myTimeToLive;
    myOptions.get(TimeToLiveStr, ttl);
    myTimeToLive = ttl;
    
    i = 0;
    if ( myOptions.get(DirectStr) )
    {
        myCallModel = Direct;
        ++i;
    }
    if ( myOptions.get(GkRoutedStr) )
    {
        myCallModel = GatekeeperRouted;
        ++i;
    }
    if ( myType == ARQ && i != 1 )
    {
        if ( i == 0 )
        {
            cerr << "Error: Must specify a call model." << endl;
        }
        else
        {
            cerr << "Error: May only specify one of "
                 << DirectStr << " or " << GkRoutedStr << NL << endl;
        }
        return ( CONFIG_ERROR );
    }
    
    myOptions.get(DestPhoneStr, myDestPhone);
    myOptions.get(DestUserStr, myDestUser);
    if ( myType == LRQ && myDestPhone.size() == 0 && myDestUser.size() == 0 )
    {
        cerr << "Error: must specify either a destination user or phone number."
             << endl;
        return ( CONFIG_ERROR );
    }
    
    string  dcs_addr;
    if ( myOptions.get(DestCallSigAddrStr, dcs_addr) )
    {
        myDestCallSigAddr.setIP(dcs_addr);
    }

    int crv = 0;
    myOptions.get(CRVStr, crv);
    myCRV = crv;
    if ( ( myType == ARQ || myType == DRQ ) && myCRV == 0 )
    {
        cerr << "Error: CRV must to set to something other than 0." << endl;
        return ( CONFIG_ERROR );
    }
    
    int bandwidth = 0;
    myOptions.get(BandwidthStr, bandwidth);
    myBandwidth = bandwidth;
    
    myOptions.get(ConfIdStr, myConfId);
    if ( ( myType == ARQ || myType == DRQ ) && myConfId.size() == 0 )
    {
        cerr << "Error: must specify a conference id." << endl;
        return ( CONFIG_ERROR );
    }
    
    myOptions.get(CallIdStr, myCallId);
    if ( ( myType == ARQ || myType == DRQ ) && myCallId.size() == 0 )
    {
        cerr << "Error: must specify a call id." << endl;
        return ( CONFIG_ERROR );
    }
    
    myAnswerCall = myOptions.get(AnswerStr);
    myMC         = myMC || myOptions.get(MCStr);
    myMulticast  = myOptions.get(MulticastStr);

    string  addr;
    if ( myOptions.get(AddressStr, addr) )
    {
        myAddress.setIP(addr);
    }
    
    myOutOfResources = myOptions.get(OutOfResourcesStr);
    
    int retry_count = 0;
    myOptions.get(RetryCountStr, retry_count);
    myRetryCount = retry_count;
    if ( myRetryCount == 0  )
    {
        cerr << "Error: retry count is mandatory and cannot be 0" << endl;
        return ( CONFIG_ERROR );
    }
    
    int retry_delay = 0;
    myOptions.get(RetryDelayStr, retry_delay);
    myRetryDelay = retry_delay;
    if ( myRetryDelay == 0 )
    {
        cerr << "Error: retry delay is mandatory and cannot be 0" << endl;
        return ( CONFIG_ERROR );
    }

    myKeepAlive = myOptions.get(KeepAliveStr);
    if ( myKeepAlive && myType == RRQ )
    { 
        if ( myEndpointId.size() == 0 )
        {
            cerr << "Error: must set endpoint id for a keepalive RRQ" << endl;
            return ( CONFIG_ERROR );
        }
        if ( myTimeToLive == 0 )
        {
            cerr << "Error: must set time to live for a keepalive RRQ" << endl;
            return ( CONFIG_ERROR );
        }
    }

    myOptions.get(GkIdStr, myGkId);
    
    if ( myMulticast == false )
    {
        if ( myOptions.remaining().size() != 1 )
        {
            cerr << "Error: must specify the ip address of the gatekeeper." << endl;
            return ( CONFIG_ERROR );
        }
    
        myGkAddr.setIP(myOptions.remaining().front());
    }

    myDebug = myOptions.get(DebugStr);
    
    return ( SUCCESS );
}


ostream &   
RasConfig::writeTo(ostream & out) const
{
    out << zMessage_Type << ":                       " << MessageTypeName(myType) << NL
        << zRas_Address << ":                        " << myRasAddr << NL
        << zCall_Signaling_Address << ":             " << myCallSigAddr << NL
        << zDestination_Call_Signaling_Address << ": " << myDestCallSigAddr << NL
        << zEndpoint_Id << ":                        " << myEndpointId << NL
        << zSequence_Number << ":                    " << mySeqNum << NL
        << zEndpoint_Type << ":                      " << EndpointTypeName(myEndpointType) << NL
        << zMC << ":                                 " << ( myMC ? "Yes" : "No" ) << NL
        << zPhone_Number << ":                       " << myPhone << NL
        << zUser << ":                               " << myUser << NL
        << zDestination_Phone_Number << ":           " << myDestPhone << NL
        << zDestination_User << ":                   " << myDestUser << NL
        << zProduct_Id << ":                         " << myProductId << NL
        << zVersion_Id << ":                         " << myVersionId << NL
        << zDiscovery_Complete << ":                 " << ( myDiscoveryComplete ? "Yes" : "No" ) << NL
        << zTime_To_Live << ":                       " << myTimeToLive << NL
        << zCounty_Code << ":                        " << static_cast<unsigned>(myCountryCode) << NL
        << zExtension << ":                          " << static_cast<unsigned>(myExtension) << NL
        << zManufacturer_Code << ":                  " << myManufacturerCode << NL
        << zCall_Model << ":                         " << CallModelName(myCallModel) << NL
        << zCRV << ":                                " << myCRV << NL
        << zConference_Id << ":                      " << myConfId << NL
        << zCall_Id << ":                            " << myCallId << NL
        << zBandwidth << ":                          " << myBandwidth << NL
        << zAnswer_Call << ":                        " << ( myAnswerCall ? "Yes" : "No" ) << NL
        << zMulticast << ":                          " << ( myMulticast ? "Yes" : "No" ) << NL
        << zAddress << ":                            " << myAddress << NL
        << zOut_of_Resources << ":                   " << ( myOutOfResources ? "Yes" : "No" ) << NL
        << zRetry_Count << ":                        " << myRetryCount << NL
        << zRetry_Delay << ":                        " << myRetryDelay << NL
        << zKeep_Alive << ":                         " << ( myKeepAlive ? "Yes" : "No" ) << NL
        << zGatekeeper_Id << ":                      " << myGkId << NL
        << zGatekeeper_Address << ":                 " << myGkAddr;

    return ( out );
}


const char * 
RasConfig::MessageTypeName(MessageType type)
{
    static const char *     name[Number_MessageType] =
    {
        "Unknown Message Type",
        "GRQ",
        "RRQ",
        "URQ",
        "ARQ",
        "DRQ",
        "LRQ",
        "RAI"
    };
    
    size_t  i = static_cast<size_t>(type);
    
    return ( name[ i >= Number_MessageType ? 0 : i ] );
}


const char * 
RasConfig::EndpointTypeName(EndpointType type)
{
    static const char *     name[Number_EndpointType] =
    {
        "Unknown Message Type",
        "Terminal",
        "Gateway",
        "MCU"
    };
    
    size_t  i = static_cast<size_t>(type);
    
    return ( name[ i >= Number_EndpointType ? 0 : i ] );
}


const char * 
RasConfig::CallTypeName(CallType type)
{
    static const char *     name[Number_CallType] =
    {
        "Unknown Call Type",
        "PointToPoint",
        "OneToN",
        "NToOne",
        "NToN"
    };
    
    size_t  i = static_cast<size_t>(type);
    
    return ( name[ i >= Number_CallType ? 0 : i ] );
}


const char * 
RasConfig::CallModelName(CallModel type)
{
    static const char *     name[Number_CallModel] =
    {
        "Unknown Call Model",
        "Direct",
        "Gatekeeper Routed"
    };
    
    size_t  i = static_cast<size_t>(type);
    
    return ( name[ i >= Number_CallModel ? 0 : i ] );
}

        
RasConfig::MessageType     
RasConfig::type() const
{
    return ( myType );
}


IPAddress       
RasConfig::rasAddr() const
{
    return ( myRasAddr );
}


IPAddress       
RasConfig::callSigAddr() const
{
    return ( myCallSigAddr );
}


string          
RasConfig::endpointId() const
{
    return ( myEndpointId );
}


u_int16_t       
RasConfig::seqNum() const
{
    return ( mySeqNum );
}


RasConfig::EndpointType    
RasConfig::endpointType() const
{
    return ( myEndpointType );
}


string          
RasConfig::phone() const
{
    return ( myPhone );
}


string          
RasConfig::user() const
{
    return ( myUser );
}


string          
RasConfig::productId() const
{
    return ( myProductId );
}


string          
RasConfig::versionId() const
{
    return ( myVersionId );
}


u_int8_t        
RasConfig::countryCode() const
{
    return ( myCountryCode );
}


u_int8_t        
RasConfig::extension() const
{
    return ( myExtension );
}


u_int16_t        
RasConfig::manufacturerCode() const
{
    return ( myManufacturerCode );
}


bool
RasConfig::discoveryComplete() const
{
    return ( myDiscoveryComplete );
}


u_int32_t
RasConfig::timeToLive() const
{
    return ( myTimeToLive );
}


RasConfig::CallType        
RasConfig::callType() const
{
    return ( myCallType );
}


RasConfig::CallModel       
RasConfig::callModel() const
{
    return ( myCallModel );
}


string          
RasConfig::destPhone() const
{
    return ( myDestPhone );
}


string          
RasConfig::destUser() const
{
    return ( myDestUser );
}


IPAddress       
RasConfig::destCallSigAddr() const
{
    return ( myDestCallSigAddr );
}


u_int16_t       
RasConfig::crv() const
{
    return ( myCRV );
}


u_int32_t       
RasConfig::bandwidth() const
{
    return ( myBandwidth );
}


string          
RasConfig::confId() const
{
    return ( myConfId );
}


string          
RasConfig::callId() const
{
    return ( myCallId );
}


bool            
RasConfig::answerCall() const
{
    return ( myAnswerCall );
}


bool            
RasConfig::mc() const
{
    return ( myMC );
}

bool            
RasConfig::multicast() const
{
    return ( myMulticast );
}

IPAddress
RasConfig::address() const
{
    return ( myAddress );
}


bool
RasConfig::outOfResources() const
{
    return ( myOutOfResources );
}

u_int16_t
RasConfig::retryCount() const
{
    return ( myRetryCount );
}


u_int16_t
RasConfig::retryDelay() const
{
    return ( myRetryDelay );
}


bool
RasConfig::keepAlive() const
{
    return ( myKeepAlive );
}


string
RasConfig::gkId() const
{
    return ( myGkId );
}


IPAddress
RasConfig::gkAddr() const
{
    return ( myGkAddr );
}


bool
RasConfig::debug() const
{
    return ( myDebug );
}
