#if !defined(RAS_CONFIG_HXX)
#define RAS_CONFIG_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2001 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const RasConfig_hxx_Version = 
    "$Id: RasConfig.hxx,v 1.2 2002/05/15 07:33:56 icahoon Exp $";


#include "Config.hxx"
#include "IPAddress.hxx"


namespace Vocal
{


using Vocal::Transport::IPAddress;


class RasConfig : public Vocal::Configuration::Config
{

    public:

    	RasConfig();
	virtual ~RasConfig();

        virtual void        usage() const;
        virtual ReturnCode  preParse();
        virtual ReturnCode  postParse();
        virtual ostream &   writeTo(ostream &) const;

        enum MessageType
        {
            Unknown_MessageType,
            GRQ,
            RRQ,
            URQ,
            ARQ,
            DRQ,
            LRQ,
            RAI,
            Number_MessageType
        };
        
        static const char * MessageTypeName(MessageType);
        
        enum EndpointType
        {
            Unknown_EndpointType,
            Terminal,
            Gateway,
            MCU,
            Number_EndpointType
        };

        static const char * EndpointTypeName(EndpointType);
        
        enum CallType
        {
            Unknown_CallType,
	    PointToPoint,
	    OneToN,
	    NToOne,
	    NToN,
            Number_CallType
        };
        
        static const char * CallTypeName(CallType);
        
        enum CallModel
        {
            Unknown_CallModel,
            Direct,
            GatekeeperRouted,
            Number_CallModel
        };
        
        static const char * CallModelName(CallModel);
        
        MessageType     type() const;
        IPAddress       rasAddr() const;
        IPAddress       callSigAddr() const;
        string          endpointId() const;
        u_int16_t       seqNum() const;
        EndpointType    endpointType() const;
        string          phone() const;
        string          user() const;
        string          productId() const;
        string          versionId() const;
        u_int8_t        countryCode() const;
        u_int8_t        extension() const;
        u_int16_t       manufacturerCode() const;
        bool            discoveryComplete() const;
        u_int32_t       timeToLive() const;
        CallType        callType() const;
        CallModel       callModel() const;
        string          destPhone() const;
        IPAddress       destCallSigAddr() const;
        string          destUser() const;
        u_int16_t       crv() const;
        u_int32_t       bandwidth() const;
        string          confId() const;
        string          callId() const;
        bool            answerCall() const;
        bool            mc() const;
        bool            multicast() const;
        IPAddress       address() const;
        bool            outOfResources() const;
        u_int16_t       retryCount() const;
        u_int16_t       retryDelay() const;
        bool            keepAlive() const;
        string          gkId() const;
        IPAddress       gkAddr() const;
        bool            debug() const;

    private:

        string          GRQStr;
        string          RRQStr;
        string          URQStr;
        string          ARQStr;
        string          DRQStr;
        string          LRQStr;
        string          RAIStr;
        string          RasAddrStr;
        string          CallSigAddrStr;
        string          EndpointIdStr;
        string          SequenceNumberStr;
        string          TerminalStr;
        string          GatewayStr;
        string          MCUStr;
        string          PhoneStr;
        string          UserStr;
        string          ProductIdStr;
        string          VersionIdStr;
        string          H221Str;
        string          DiscCompleteStr;
        string          TimeToLiveStr;
        string          DirectStr;
        string          GkRoutedStr;
        string          DestPhoneStr;
        string          DestUserStr;
        string          DestCallSigAddrStr;
        string          CRVStr;
        string          BandwidthStr;
        string          ConfIdStr;
        string          CallIdStr;
        string          AnswerStr;
        string          MCStr;
        string          MulticastStr;
        string          AddressStr;
        string          OutOfResourcesStr;
        string          RetryCountStr;
        string          RetryDelayStr;
        string          KeepAliveStr;
        string          GkIdStr;
        string          DebugStr;
        
        MessageType     myType;
        IPAddress       myRasAddr;
        IPAddress       myCallSigAddr;
        string          myEndpointId;
        u_int16_t       mySeqNum;
        EndpointType    myEndpointType;
        string          myPhone;
        string          myUser;
        string          myProductId;
        string          myVersionId;
        u_int8_t        myCountryCode;
        u_int8_t        myExtension;
        u_int16_t       myManufacturerCode;
        bool            myDiscoveryComplete;
        u_int32_t       myTimeToLive;
        CallType        myCallType;
        CallModel       myCallModel;
        string          myDestPhone;
        IPAddress       myDestCallSigAddr;
        string          myDestUser;
        u_int16_t       myCRV;
        u_int32_t       myBandwidth;
        string          myConfId;
        string          myCallId;
        bool            myAnswerCall;
        bool            myMC;
        bool            myMulticast;
        IPAddress       myAddress;
        bool            myOutOfResources;
        u_int16_t       myRetryCount;
        u_int16_t       myRetryDelay;
        bool            myKeepAlive;
        string          myGkId;
        IPAddress       myGkAddr;
        bool            myDebug;
};


} // namespace Vocal


#endif // !defined(RAS_CONFIG_HXX)
