/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const AddressToFrom_cxx_Version = 
    "$Id: AddressToFrom.cxx,v 1.13 2001/05/11 02:24:34 vzubarev Exp $";


#include <string>
#include "Data.hxx"
#include "AddressToFrom.hxx"

using namespace Translator;

std::string AddressToFrom::getDisplayName()
{
    int i = 0, a_size = h_address.GetSize();
    bool a_found = false;
    std::string ret_str;
    
    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (h_address[i++].GetTag() == H225_AliasAddress::e_h323_ID);
    
    if(a_found)
    {
      PASN_BMPString &bmph323 = (PASN_BMPString &)h_address[i-1];
            
      ret_str = std::string((const char*)((PString)bmph323));
    }
    else ret_str = "";
    
    return ret_str;
}

SipTo AddressToFrom::getTo() throw (XlatorException)
{
   Data displayName;
   
   displayName = std::string(getDisplayName());
   
   if(displayName.length() != 0)
   {
      sipTo.setDisplayName(displayName);
   }
   
   try
   {
     sipUrl = address;
   }
   catch (XlatorException)
   {
     throw XlatorException("Not valid sipUrl", __FILE__, __LINE__);
   }
   
   Sptr<BaseUrl> baseUrl = new SipUrl(sipUrl);
   
   sipTo.setUrl(baseUrl);
   
   return sipTo;
}

SipFrom AddressToFrom::getFrom() throw (XlatorException)
{
   Data displayName;
   
   displayName = std::string(getDisplayName());
   
   if(displayName.length() != 0)
   {
      sipFrom.setDisplayName(displayName);
   }
   
   try
   {
     sipUrl = address;
   }
   catch (XlatorException)
   { 
     throw XlatorException("Not valid sipUrl", __FILE__, __LINE__);
   }
   
   Sptr<BaseUrl> baseUrl = new SipUrl(sipUrl);
   
   sipFrom.setUrl(baseUrl);
   
   return sipFrom;
}
