#ifndef __AUDIOCAPABILITY_H_
#define __AUDIOCAPABILITY_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const AudioCapability_hxx_Version = 
"$Id: AudioCapability.hxx,v 1.9 2000/12/20 01:47:43 vzubarev Exp $";

#include "Capability.hxx"

namespace Translator
{

///
class AudioCapability : public Capability
{
  protected:
    ///
    unsigned int samplesPerFrame;

    /**
       Packet size (common parameter for most of audio caps)
       in Frames; packetSize in samples would be
       samplesPerFrame*packetSize(in Frames)
       packetSize in milliseconds would be:
       packetSize(in Samples)/(clockrate(in Hz)/1000)
    */
    unsigned int packetSize;
    
    ///   
    unsigned int clockRate;

  public:
  
  /**
     sPF constant - samples per Frame
     pSize - Packet size in Frames
     clock rate in Hz
  */
    AudioCapability(
      unsigned int sPF,
      unsigned int pSize,
      unsigned int clockRate
    );
    
    virtual ~AudioCapability() {}

    /**
       Get the main type of the capability.
       Always returns e_Audio.
     */
    virtual MainTypes getMainType() const;

    /// set packet size in samples
    virtual void setPacketSizeFrames(unsigned int pSize);
    
    /// set packet size in milliseconds
    virtual void setPacketSizeMilSec(float pSmS);

    /// get packet size in milliseconds
    virtual float getPacketSizeMilSec() const;
    
    /// get packet size in samples
    virtual unsigned int getPacketSizeSamples() const;
    
    /// get packet size in frames
    virtual int getPacketSizeFrames() const;
    
    ///
    virtual int getClockRate() const;

    /// get samples per Frame parameter
    unsigned int getSamplesPerFrame() const;
    
    /**
       is called on construction of TerminalCapability
       message to send.
       Sets message fields from information in members specific
       to the class.
     */
    virtual bool getParamSendMsg(
      H245_Capability & pdu
    ) const;

    /**
       is called on construction of OpenLogicalChannel message to send. 
       Sets message fields from information in members specific
       to the class.
     */
    virtual bool getParamSendMsg(
      H245_DataType & pdu
    ) const;

    /**
       is called on construction of OpenLogicalChannel or TerminalCapabilitySet
       message to send. 
       Sets message fields from information in members specific
       to the class.
       The default behaviour is pure - to be overloaded by descendents.
     */
    virtual bool getParamSendMsg(
      H245_AudioCapability & pdu,  /// PDU to set information on
      unsigned packetSize          /// Packet size to use in capability
    ) const = 0;

    /**
       is called on receiving of TerminalCapabilitySet message.
       Sets the classes members from the corresponding message fields.
     */
    virtual bool setParamRecvMsg(
      const H245_Capability & pdu  /// PDU to get information from
    );

    /**
       is called on receiving of OpenLogicalChannel message.
       Sets the classes members from the corresponding message fields.
     */
    virtual bool setParamRecvMsg(
      const H245_DataType & pdu  /// PDU to get information from
    );

    /**
       is called on receiving of OpenLogicalChannel or TerminalCapabilitySet
       messages.
       Sets the classes members from the corresponding message fields.
       The default behaviour is pure - to be overloaded by descendents.
     */
    virtual bool setParamRecvMsg(
      const H245_AudioCapability & pdu,  /// PDU to get information from
      unsigned & packetSize              /// Packet size to use in capability
    ) = 0;
    
    /// set packet size of the particular capability received in SDP message
    virtual void setParamRecvMsg(intSDP pSize);
    
    /// get packet size of the particular capability to send in SDP message
    virtual void getParamSendMsg(intSDP &pSize);
};

}
#endif
