/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const CallTranslatorEM_cxx_Version = 
"$Id: CallTranslatorEM.cxx,v 1.2 2002/03/01 22:51:07 bko Exp $";

#include "CallTranslatorEM.hxx"

using Vocal::SDP::SdpSession;
using Vocal::SDP::SdpConnection;
using Vocal::SDP::SdpMedia;

using namespace translator;

CallTranslatorEM::
CallTranslatorEM()
{
}


CallTranslatorEM::
~CallTranslatorEM()
{
}


void CallTranslatorEM::
add(EvtResolve* evtResolve)
{
    evtResolve_ = evtResolve;
}


void CallTranslatorEM::
add(router::EvtSendFSOLC* evtSendFSOLC)
{
    evtSendFSOLC_ = evtSendFSOLC;
}


void CallTranslatorEM::
add(EvtZeroTCS* evtZeroTCS)
{
    evtZeroTCS_ = evtZeroTCS;
}


void CallTranslatorEM::
add(EvtZeroSDP* evtZeroSDP)
{
    evtZeroSDP_ = evtZeroSDP;
}


void CallTranslatorEM::
add(EvtSendSDP* evtSendSDP)
{
    evtSendSDP_ = evtSendSDP;
}


void CallTranslatorEM::
add(router::EvtCEOTransferReq* evtCEOTransferReq)
{
    evtCEOTransferReq_ = evtCEOTransferReq;
}


void CallTranslatorEM::
add(router::EvtLCOEstablishReq* evtLCOEstablishReq)
{
    evtLCOEstablishReq_ = evtLCOEstablishReq;
}


void CallTranslatorEM::
add(router::EvtLCOReleaseReq* evtLCOReleaseReq)
{
    evtLCOReleaseReq_ = evtLCOReleaseReq;
}


void CallTranslatorEM::
add(router::EvtLCIEstablishResp* evtLCIEstablishResp)
{
    evtLCIEstablishResp_ = evtLCIEstablishResp;
}


void CallTranslatorEM::
add(router::EvtLCIReleaseReq* evtLCIReleaseReq)
{
    evtLCIReleaseReq_ = evtLCIReleaseReq;
}


void CallTranslatorEM::
add(router::EvtRCORequest* evtRCORequest)
{
    evtRCORequest_ = evtRCORequest;
}


void CallTranslatorEM::
add(router::EvtCreateInLC* evtCreateInLC)
{
    evtCreateInLC_ = evtCreateInLC;
}


void CallTranslatorEM::
add(router::EvtCreateOutLC* evtCreateOutLC)
{
    evtCreateOutLC_ = evtCreateOutLC;
}


void CallTranslatorEM::
add(EvtSDPParseDone* evtSDPParseDone)
{
    evtSDPParseDone_ = evtSDPParseDone;
}


void CallTranslatorEM::
add(router::EvtReceivedCCAddress* evtForceCCOpen)
{
    evtForceCCOpen_ = evtForceCCOpen;
}


void CallTranslatorEM::
startResolve(std::string hostName)
{
    if (evtResolve_)
    {
        evtResolve_->hostName_ = hostName;
        mte::EventDynamic<mte::BEvent>* per = evtResolve_->clone();
        per->start();
    }
}


void CallTranslatorEM::
startSendFSOLC(const router::H245OLC& olc)
{
    router::EvtSendFSOLC* peSFS = evtSendFSOLC_;
    peSFS->olc_.m() = olc;
    peSFS->start();
}


void CallTranslatorEM::
startZeroTCS()
{
    if (evtZeroTCS_)
    {
        EvtZeroTCS* peZTCS = evtZeroTCS_;
        peZTCS->start();
    }
}


void CallTranslatorEM::
startZeroSDP()
{
    if (evtZeroSDP_)
    {
        EvtZeroSDP* peZSDP = evtZeroSDP_;
        peZSDP->start();
    }
}


void CallTranslatorEM::
startSendSDP(const SdpSession& sdp, bool change)
{
    if (evtSendSDP_)
    {
        EvtSendSDP* peSSDP = evtSendSDP_;
        peSSDP->sdp_ = sdp;
        peSSDP->change_ = change;
        peSSDP->start();
    }
}


void CallTranslatorEM::
startCEOTransferReq(const router::H245TCS& tcs)
{
    if (evtCEOTransferReq_)
    {
        router::EvtCEOTransferReq* peCEOTR = evtCEOTransferReq_;
        peCEOTR->tcs_.m() = tcs;
        peCEOTR->start();
    }
}


void CallTranslatorEM::
startLCOEstablishReq(const router::H245OLC& olc)
{
    if (evtLCOEstablishReq_)
    {
        router::EvtLCOEstablishReq* peLCOER = evtLCOEstablishReq_;
        peLCOER->olc_.m() = olc;
        peLCOER->setLCN(olc.m_forwardLogicalChannelNumber);
        peLCOER->start();
    }
}


void CallTranslatorEM::
startLCOReleaseReq(const router::H245CLC& clc)
{
    if (evtLCOReleaseReq_)
    {
        router::EvtLCOReleaseReq* peLCORR = evtLCOReleaseReq_;
        peLCORR->clc_.m() = clc;
        peLCORR->setLCN(clc.m_forwardLogicalChannelNumber);
        peLCORR->start();
    }
}


void CallTranslatorEM::
startLCIEstablishResp(const router::H245OLCAck& olcAck)
{
    if (evtLCIEstablishResp_)
    {
        router::EvtLCIEstablishResp* peLCIER = evtLCIEstablishResp_;
        peLCIER->olcAck_.m() = olcAck;
        peLCIER->setLCN(olcAck.m_forwardLogicalChannelNumber);
        peLCIER->start();
    }
}


void CallTranslatorEM::
startLCIReleaseReq(const router::H245OLCReject& olcReject)
{
    if (evtLCIReleaseReq_)
    {
        router::EvtLCIReleaseReq* peLCIRR = evtLCIReleaseReq_;
        peLCIRR->olcReject_.m() = olcReject;
        peLCIRR->setLCN(olcReject.m_forwardLogicalChannelNumber);
        peLCIRR->start();
    }
}


void CallTranslatorEM::
startRCORequest(const router::H245RCC& rcc)
{
    if (evtRCORequest_)
    {
        router::EvtRCORequest* peRCC = evtRCORequest_;
        peRCC->rcc_.m() = rcc;
        peRCC->setLCN(rcc.m_forwardLogicalChannelNumber);
        peRCC->start();
    }
}


void CallTranslatorEM::
startCreateInLC(int lcNumber)
{
    if (evtCreateInLC_)
    {
        evtCreateInLC_->number_ = lcNumber;
        evtCreateInLC_->start();
    }
}


void CallTranslatorEM::
startCreateOutLC(int lcNumber)
{
    if (evtCreateOutLC_)
    {
        evtCreateOutLC_->number_ = lcNumber;
        evtCreateOutLC_->start();
    }
}


void CallTranslatorEM::
startSDPParseDone()
{
    if (evtSDPParseDone_)
    {
        EvtSDPParseDone* pePD = evtSDPParseDone_;
        pePD->start();
    }
}


void CallTranslatorEM::
startForceCCOpen()
{
    if (evtForceCCOpen_)
    {
        router::EvtReceivedCCAddress* peFCO = evtForceCCOpen_;
        peFCO->start();
    }
}
