/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const CapabilityMap_cxx_Version = 
    "$Id: CapabilityMap.cxx,v 1.29 2002/12/10 04:15:01 icahoon Exp $";


#include <cstdlib>
#include <netdb.h>
#include <sys/socket.h>
#include <unistd.h>
#include <iterator>
#include <string>
#include <algorithm>
#include <map>
#include "Data.hxx"
#include "VLog.hxx"
#include "Sdp2Connection.hxx"
#include "Sdp2Attributes.hxx"
#include "support.hxx"
#include "Sdp2Media.hxx"
#include "IPAddress.hxx"
#include "CapabilityMap.hxx"
#include "Assert.hxx"
#include "G711Capability.hxx"
#include "GSMCapability.hxx"
#include "G7231Capability.hxx"
#include "G722Capability.hxx"
#include "G728Capability.hxx"
#include "G729Capability.hxx"


#ifndef DEBUG
#define DEBUG 1
#endif

using namespace Translator;
using Vocal::Transport::IPAddress;
using Vocal::Logging::VLog;

using Vocal::SDP::SdpSession;
using Vocal::SDP::SdpConnection;
using Vocal::SDP::SdpMedia;
using Vocal::SDP::SdpAttributes;
using Vocal::SDP::SdpRtpMapAttribute;
using Vocal::SDP::ValueAttribute;
using Vocal::SDP::MediaAttributes;


// only for debugging purposes
//
void CapabilityMap::fillDebug()
{
  const string	fn("CapabilityMap::fillDebug");
  VLog	    	log(fn);

  int pTypeCap = 1;

    Capability *capability = capabilityMap[pTypeCap];
       
    Capability *copy = capability->clone();
  
    copy->assignCapabilityNumber(pTypeCap);
    capSetSip.h323CapTable.addCapTableEntry(copy);
    
    int sTableSize = capSetSip.h323CapTable.getCapabilityTable().size();
    
    VDEBUG(log) << fn << ": Now the size of Sip capTable is : "
    	    	<< sTableSize << VDEBUG_END(log);

  capSetSip.capRecv.addCapability(1);
  
  IPAddress ipAddress1("192.168.22.39", 2222);
  
  IPAddress ipAddress2("192.168.22.39", 2223);
  
  IPAddress ipAddress3("192.168.22.39", 3333);

  Address address;
  address = ipAddress1;
  sDescriptor.transportIDSip.setRTPAddr(address, 0, 0);

  address = ipAddress2;
  sDescriptor.transportIDSip.setRTCPAddr(address, 0, 0);
  
}

// only for debugging purposes

void capabilitySetGetTestM(CapabilitySet &csSip)
{
  const string  fn("capabilitySetGetTestM");
  VLog	    	log(fn);
  
  int i = 0, j = 0, h = 0, vecSize = 0, descrSize = 0, altSize = 0;
  
  vecSize = csSip.capRecv.getCapDescrVecSize();
  if(!vecSize) { VDEBUG(log) << fn << ": Seems like this guy is empty..." << VDEBUG_END(log); }
  
  VDEBUG(log) << fn << ": Here is the descriptors... " << VDEBUG_END(log);
  
  for(i = 0; i < csSip.capRecv.getCapDescrVecSize(); i++)
  {
    descrSize = csSip.capRecv.getCapDescrSize(i);

    for(j = 0; j < descrSize; j++)
    {
      altSize = csSip.capRecv.getAltCapSize(i, j);

      VDEBUG(log) << fn << ": " << i << ' ' << j << VDEBUG_END(log);

      for(h = 0; h < altSize; h++)
      {
        VDEBUG(log) << fn << ":            "<< csSip.capRecv.getCapability(i, j, h) << VDEBUG_END(log);
      }
    }
  }
  
  VDEBUG(log) << fn << ":           " << VDEBUG_END(log);
}

void CapabilityMap::fillCapMap()
{
    const string    fn("CapabilityMap::fillCapMap");
    VLog	    log(fn);
  
    capabilityMap[1] = new G711Capability(G711Capability::muLaw, G711Capability::At64k);
    
    VDEBUG(log) << fn << ":G711 Cap was intialized with packet size : " 
    	    	<< capabilityMap[1]->getPacketSizeFrames()  << VDEBUG_END(log);
    capabilityMap[3] = new GSMCapability(GSMCapability::FullRate);
    capabilityMap[4] = new G7231Capability;
    capabilityMap[8] = new G711Capability(G711Capability::ALaw, G711Capability::At64k);
    capabilityMap[9] = new G722Capability(G722Capability::At64k);
    capabilityMap[15] = new G728Capability;
    capabilityMap[18] = new G729Capability;
    capabilityMap[96] = new GSMCapability(GSMCapability::HalfRate);
    capabilityMap[97] = new GSMCapability(GSMCapability::EnhancedFullRate);
}

CapabilityMap::CapabilityMap(bool mode) : strictModePackSize(mode),
					  capSetIntersect(capSetSip, capSetH323)
					  
{
    master = false;
    curDescrN = 0;
    maxSesID = 0;
    maxLogChN = 0;
    receivedSDP = 0;
    sentOLC = 0;
    receivedOLCAck = 0;
    receivedOLC = 0;
    fillCapMap();
    
    // for debugging
    //fillDebug();
}

CapabilityMap::~CapabilityMap()
{
  const string  fn("CapabilityMap::~CapabilityMap");
  VLog	    	log(fn);
    CapMap::const_iterator p = capabilityMap.begin();
    
    while(p != capabilityMap.end())
    {
      delete p->second;
      p++;
    }
  const string  fn2("exit CapabilityMap::~CapabilityMap");
  VLog	    	log2(fn2);
}

void CapabilityMap::setMaster(bool mI)
{
  master = mI;
}

void CapabilityMap::handleTCS(const TCS &msgTCS) throw (XlatorException)
{
  const string  fn("CapabilityMap::handleTCS");
  VLog	    	log(fn);
  
  int i = 0, j = 0, h = 0, vecSize = 0, descrSize = 0;
  int altSize = 0, tableSize = 0;
  
  VDEBUG(log) << fn << ": BEGIN -------------- " << VDEBUG_END(log);
  
  if(!msgTCS.HasOptionalField(H245_TerminalCapabilitySet::e_capabilityTable))
    throw XlatorException("no capabilityTable in TCS message", __FILE__, __LINE__);

  if(!msgTCS.HasOptionalField(H245_TerminalCapabilitySet::e_capabilityDescriptors))
    throw XlatorException("no capabilityDescriptors in TCS message", __FILE__, __LINE__);

  tableSize = msgTCS.m_capabilityTable.GetSize();
  if(!tableSize) { VWARN(log) << fn << ": Seems like capTable is empty..."  << VDEBUG_END(log); }
  
  int pTypeCap, tableN;
  
  CapMap::const_iterator cM_I;

  for(i = 0; i < tableSize; i++)
  {
    if(msgTCS.m_capabilityTable[i].HasOptionalField(H245_CapabilityTableEntry::e_capability))
    {
       try
       {	
	 pTypeCap = globalCapTable.findCapTableEntry(msgTCS.m_capabilityTable[i].m_capability);
       }
       catch (XlatorException &mS)
       {
         VDEBUG(log) << fn << ": Do not support this capability " 
	    	     <<  mS.getDescription() << " continue..." << VDEBUG_END(log);
	 continue;
       }
      
       cM_I = capabilityMap.find(pTypeCap);
       
       if(cM_I == capabilityMap.end())
       {
         VERR(log) << fn << ": Capability number initialized in capabilityMap & in GlobalCapTable..." << VERR_END(log);
	 VERR(log) << fn << ": ...pTypeCap : "<< pTypeCap << " ..." << VERR_END(log);
	 VERR(log) << fn << ": ... do not match - check CapabilityMap constructor & GlobalCapTable" << VERR_END(log);
	 throw XlatorException("Capability number initialized in capabilityMap & in GCTable do not match",
	                       __FILE__, __LINE__);
       }
       
       Capability *capability = capabilityMap[pTypeCap];
       
       Capability *copy = capability->clone();

       if(copy->setParamRecvMsg(msgTCS.m_capabilityTable[i].m_capability))
       {
	  copy->assignCapabilityNumber(pTypeCap);
          VDEBUG(log) << fn << ": Capability " << pTypeCap << " with packetSize " << copy->getPacketSizeFrames() << VDEBUG_END(log);

	  tableN = (int)msgTCS.m_capabilityTable[i].m_capabilityTableEntryNumber;
	  
	  mapTable[tableN] = copy->getCapabilityNumber();
	  capSetH323.h323CapTable.addCapTableEntry(copy);
	    
       } else delete copy;
    }
  }
	
  vecSize = msgTCS.m_capabilityDescriptors.GetSize();
  
  int y = 0, z = 0;
  
  VDEBUG(log) << fn << ": Filled out the capabilityTable..." << VDEBUG_END(log);

  int hTableSize = capSetH323.h323CapTable.getCapabilityTable().size();
    
  VDEBUG(log) << fn << ": Now the size of H323 capTable is : "<< hTableSize << VDEBUG_END(log);

  if(!vecSize)
  {
      VWARN(log) << fn << ": Seems like capDescriptors is empty..." << VWARN_END(log);
  }
  
  for(i = 0; i < vecSize; i++)
  {
    H245_CapabilityDescriptor &descr = msgTCS.m_capabilityDescriptors[i];
    if(descr.HasOptionalField(H245_CapabilityDescriptor::e_simultaneousCapabilities))
    {
    descrSize = descr.m_simultaneousCapabilities.GetSize();

     z = 0;
     for(j = 0; j < descrSize; j++)
     {
      H245_AlternativeCapabilitySet &alt = descr.m_simultaneousCapabilities[j];
      altSize = alt.GetSize();

      for(h = 0; h < altSize; h++)
      {
	if(mapTable.find(alt[h]) != mapTable.end())
	{
	  try
	  {
	    capSetH323.capRecv.addCapability(mapTable[alt[h]], y, z);
	  }
	  catch (XlatorException &mS)
	  {
	    VERR(log) << fn << ": Error adding to capability descriptor : " << mS.getDescription() << VERR_END(log);
	    VERR(log) << fn << ":  y  -  z : " << y << " - " << z << VERR_END(log);
	  }
	}
	else
	{
	  VDEBUG(log) << fn << ": The capability " << (int)alt[h] << " from cap descriptor is not in capTable..." << VDEBUG_END(log);
	}
      }
      
      if(capSetH323.capRecv.getAltCapSize(y, z) != 0)z++;
        
     }
     
     if(capSetH323.capRecv.getCapDescrSize(y) != 0)y++;
    }
  }

  VDEBUG(log) << fn << ": Filled out the descriptors..." << VDEBUG_END(log);
  capabilitySetGetTestM(capSetH323);
  
  VDEBUG(log) << fn << ": END ----------------" << VDEBUG_END(log);
}


void CapabilityMap::makeTCS(TCS &msgTCS) const
{
  const string	fn("CapabilityMap::makeTCS");
  VLog	    	log(fn);
  
  int tableSize = 0, descrVecSize = 0, descrSize = 0, altSize = 0, i = 0, j = 0, h = 0;
  VDEBUG(log) << fn << ": BEGIN -----------------" << VDEBUG_END(log);
  
  msgTCS.IncludeOptionalField(H245_TerminalCapabilitySet::e_capabilityTable);
  
  const H323_CapTable::CapTable &capTable = capSetSip.h323CapTable.getCapabilityTable();
  
  tableSize = capTable.size();
  msgTCS.m_capabilityTable.SetSize(tableSize);
  
  for(i = 0; i < tableSize; i++)
  {
    msgTCS.m_capabilityTable[i].m_capabilityTableEntryNumber = capTable[i]->getCapabilityNumber();
    msgTCS.m_capabilityTable[i].IncludeOptionalField(H245_CapabilityTableEntry::e_capability);

    VDEBUG(log) << fn << ": Capability " << capTable[i]->getCapabilityNumber()
    	    	<< " with packetSize " << capTable[i]->getPacketSizeFrames() 
		<< VDEBUG_END(log);

    capTable[i]->getParamSendMsg(msgTCS.m_capabilityTable[i].m_capability);
  }
  
  msgTCS.IncludeOptionalField(H245_TerminalCapabilitySet::e_capabilityDescriptors);
  
  descrVecSize = capSetSip.capRecv.getCapDescrVecSize();
  msgTCS.m_capabilityDescriptors.SetSize(descrVecSize);
  
  for(i = 0; i < descrVecSize; i++)
  {
    H245_CapabilityDescriptor &descr = msgTCS.m_capabilityDescriptors[i];
    descr.m_capabilityDescriptorNumber = i + 1;
    descr.IncludeOptionalField(H245_CapabilityDescriptor::e_simultaneousCapabilities);
    
    descrSize = capSetSip.capRecv.getCapDescrSize(i);
    descr.m_simultaneousCapabilities.SetSize(descrSize);
    
    for(j = 0; j < descrSize; j++)
    {
      H245_AlternativeCapabilitySet &alt = descr.m_simultaneousCapabilities[j];
      
      altSize = capSetSip.capRecv.getAltCapSize(i, j);
      alt.SetSize(altSize);
      
      for(h = 0; h < altSize; h++)
      {
        try
	{
	  alt[h] = capSetSip.capRecv.getCapability(i, j, h);
	}
	
	catch (XlatorException)
	{
	  VERR(log) << fn << ": Problems with altCapSet, weird, man...: " 
	    	    << i << ' ' << j << ' ' << h << VERR_END(log);
	  assert(0);
	}
      }
    }
  }
  
  msgTCS.IncludeOptionalField(H245_TerminalCapabilitySet::e_multiplexCapability);
  msgTCS.m_multiplexCapability.SetTag(H245_MultiplexCapability::e_h2250Capability);
  
  H245_H2250Capability &hcap = msgTCS.m_multiplexCapability;
  hcap.m_maximumAudioDelayJitter = 60;
  
  VDEBUG(log) << fn << ": END ----------------" << VDEBUG_END(log);
}

void CapabilityMap::makeIntersect() throw (XlatorException)
{
   const string fn("CapabilityMap::makeIntersect");
   VLog     	log(fn);
   
   int descrVecSizeInt = 0, i = 0, j = 0, h = 0;
   int descrSizeInt = 0, descrSizeSip = 0;
   
   try
   {
     capSetIntersect.makeIntersect();
   }
   
   catch (XlatorException &mS)
   {
     VERR(log) << fn << ": CapabilitySetIntersect intersection error... " << mS.getDescription() << VERR_END(log);
     throw XlatorException("Intersection error...", __FILE__, __LINE__);
   }
   
  // Finding the appropriate SessionDescriptors (RTP/RTCP pairs)
  // for the CapabilitySetIntersect CapabilityDescriptors
  // Looking for the maximum intersection of CapabilitySetIntersect CapDescriptors
  // with CapDescriptors from CapabilitySetSip
  
  CapabilitySet::CapabilityDescrVec::AltCapSet altCapSetIntersect;
  
  descrVecSizeInt = capSetIntersect.capRecv.getCapDescrVecSize();
  
  int maxIntersectIndex = 0;
  unsigned int maxIntersectNumber = 0;
  Address address;
  CapabilitySet::CapabilityDescrVec::AltCapSet altCapInt;
  CapabilitySet::CapabilityDescrVec::AltCapSet altCapSip;
  
  for(i = 0; i < descrVecSizeInt; i++)
  {
    descrSizeInt = capSetIntersect.capRecv.getCapDescrSize(i);
    
    for(j = 0; j < descrSizeInt; j++)
    {
      // every altCapSet we intersect with each altCapSet from capSetSip
      // whichever gives us the maximum Intersection - it's SessionDescriptor
      // should be associated with given altCapSet of CapSetIntersect
   
      altCapInt = capSetIntersect.capRecv.getAltCapSet(i, j);
      std::sort(altCapInt.begin(), altCapInt.end());
      
      descrSizeSip = capSetSip.capRecv.getCapDescrSize(0);
      altCapSetIntersect.clear();
      maxIntersectNumber = 0;
      maxIntersectIndex = -1;
      
      for(h = 0; h < descrSizeSip; h++)
      {
         altCapSip = capSetSip.capRecv.getAltCapSet(0, h);
	 std::sort(altCapSip.begin(), altCapSip.end());
         std::set_intersection(
	                       altCapSip.begin(), 
			       altCapSip.end(), 
			       altCapInt.begin(), 
			       altCapInt.end(), 
			       std::inserter(altCapSetIntersect, altCapSetIntersect.begin()));
         
	 if(maxIntersectNumber < altCapSetIntersect.size())
	 {
	   maxIntersectNumber = altCapSetIntersect.size();
	   maxIntersectIndex = h;
	 }
      }
      
      if(maxIntersectIndex == -1)
         VERR(log) << fn << ": Problems with finding SessionDescriptor..." << VERR_END(log);
      
      try
      {
        address = sDescriptor.transportIDSip.getRTPAddr(0, maxIntersectIndex);
      }
      catch (XlatorException &mS)
      {
        VERR(log) << fn << ": Error with getting RTP addr from Sip : " << mS.getDescription() << VERR_END(log);
	assert(0);
      }
      
      try
      {
        sDescriptor.transportIDIntSip.setRTPAddr(address, i, j);
      }
      catch (XlatorException &mS)
      {
        VERR(log) << fn << ": Error with setting RTP addr to SipInt : " << mS.getDescription() << VERR_END(log);
	assert(0);
      }

      try
      {
        address = sDescriptor.transportIDSip.getRTCPAddr(0, maxIntersectIndex);
	IPAddress ipAddress;
	ipAddress = address;
	VDEBUG(log) << fn << ": RTCP port we retreive from trIDSip : " << ipAddress.getPort() << VDEBUG_END(log);
	VDEBUG(log) << fn << ": maxIntersectIndex : " << maxIntersectIndex << VDEBUG_END(log);
      }
      catch (XlatorException &mS)
      {
        VERR(log) << fn << ": Error with getting RTCP addr from Sip : " << mS.getDescription() << VERR_END(log);
	assert(0);
      }
      
      try
      {
        sDescriptor.transportIDIntSip.setRTCPAddr(address, i, j);
      }
      catch (XlatorException &mS)
      {
        VERR(log) << fn << ": Error with setting RTCP addr to SipInt : " << mS.getDescription() << VERR_END(log);
    	assert(0);
      }

    }
  }

}


void CapabilityMap::handleSDP(SDP &msgSDP) throw (XlatorException)
{
   const string     fn("CapabilityMap::handleSDP");
   VLog     	    log(fn);
   
   bool found = false, gFound = false, haveSesCon;
   int rtpPacketSize = 0, j = 0;
   int clockRate, pTypeCap = 0;
   std::string sdpAddress;
   Address address;
   GlobalCapTable::GlobalCapTable_p p;
   Capability::intSDP packetSize = 0;
   std::vector<int> pTypeCapList;
   std::vector<int> pTypeCapListBad;
   
   IPAddress ipAddress;
   
   VDEBUG(log) << fn << ": BEGIN ------------------ " << VDEBUG_END(log);
   
   list<SdpMedia*> mediaList;
   mediaList = msgSDP.getMediaList();
   list<SdpMedia*>::const_iterator mediaIterator = mediaList.begin();
   
   SdpConnection *sessionConnection = msgSDP.getConnection();
   haveSesCon = (sessionConnection != NULL);
   
   while(mediaIterator != mediaList.end())
   {
    vector<int> *formatList = (*mediaIterator)->getFormatList();
    vector<int>::const_iterator fmtListIterator = formatList->begin();
   
    MediaAttributes *mediaAttrib;
    mediaAttrib = (*mediaIterator)->getMediaAttributes();
   
    rtpPacketSize = 0; 
    packetSize = 0;

    if(mediaAttrib != 0)
    {
      vector<ValueAttribute*> *valueAttribList = mediaAttrib->getValueAttributes();
      vector<ValueAttribute*>::const_iterator attribIterator = valueAttribList->begin();
     
      while(attribIterator != valueAttribList->end())
      {
        char *attribName = (*attribIterator)->getAttribute();
       
        if(strcmp(attribName, "ptime") == 0)
        {
          rtpPacketSize = Data((*attribIterator)->getValue()).convertInt();
	  break;
        }
        attribIterator++;
      }

      vector<SdpRtpMapAttribute*> *rtpMapAttribList = mediaAttrib->getmap();
      vector<SdpRtpMapAttribute*>::const_iterator rtpMapIterator = rtpMapAttribList->begin();
     
      found = false;

      if((rtpMapAttribList == 0)&&(formatList == 0))
      {
         VERR(log) << fn << ": No capabilities given in SDP message... " << VERR_END(log);
	 throw XlatorException("No capabilities given in SDP message...", __FILE__, __LINE__);
      }

      while(rtpMapIterator != rtpMapAttribList->end())
      {
       char *capName = (*rtpMapIterator)->getEncodingName();

       try
       {	
	 p = globalCapTable.getTableIter_ni(capName);
	 pTypeCap = p->ptypeCap;
       }
       catch (XlatorException)
       {
	 pTypeCapListBad.push_back((*rtpMapIterator)->getPayloadType());
         rtpMapIterator++;
	 continue;
       }
      
       if((*rtpMapIterator)->getClockRate()!=8000)
       {
         rtpMapIterator++;
	 VWARN(log) << fn << ": Clockrate for this capability is not 8000Hz, just skip it" << VWARN_END(log);
	 continue;
       }
       
       found = true;
       if(!gFound)gFound = true;
       
       Capability *capability = capabilityMap[pTypeCap];
       
       Capability *copy = capability->clone();
       clockRate = (*rtpMapIterator)->getClockRate()/1000;
       packetSize = rtpPacketSize*clockRate;

       if(rtpPacketSize != 0)
       {
         if(packetSize >= copy->getSamplesPerFrame())
	 {
	   copy->setParamRecvMsg(packetSize);
	 }
	 else 
	 {
	   VERR(log) << fn << "Packet Size in SDP is less than ms per Frame parameter "
	                   << "leaving this capability out..." << VERR_END(log);
	   fmtListIterator++;
	   continue;
	 }
       }
       
       VDEBUG(log) << fn << ": packetSize : " << packetSize << VDEBUG_END(log);
       VDEBUG(log) << fn << ": default pSize of copy : " << copy->getPacketSizeMilSec() << VDEBUG_END(log);
       VDEBUG(log) << fn << ": Here is the cap from rtpmap- to add : " << pTypeCap << VDEBUG_END(log);
       copy->assignCapabilityNumber(pTypeCap);
       capSetSip.h323CapTable.addCapTableEntry(copy);
       capSetSip.capRecv.addCapability(pTypeCap, 0, j);
       
       pTypeCapList.push_back(pTypeCap);
	    
       rtpMapIterator++;
      }
    } // if(mediaAttrib)
     
     VDEBUG(log) << fn << ": Out of rtpMapAtrribList... " << VDEBUG_END(log);
     
     while(fmtListIterator != formatList->end())
     {
        pTypeCap = *fmtListIterator;
	
	VDEBUG(log) << fn << ": pTypeCap from formatList : " << pTypeCap << VDEBUG_END(log);

	if(pTypeCap == 0)pTypeCap = 1;
	   
	try
        {	
	  p = globalCapTable.getTableIter_pt(pTypeCap);
        }
        catch (XlatorException)
        {
	  fmtListIterator++;
	  p++;
	  continue;
        }
        
	if((std::find(pTypeCapList.begin(), 
	        pTypeCapList.end(), pTypeCap) == pTypeCapList.end())&&
          (std::find(pTypeCapListBad.begin(), 
	        pTypeCapListBad.end(), pTypeCap) == pTypeCapListBad.end())&&
	  (p->nameCapH323!= -1))
	{
           found = true;

           if(!gFound)gFound = true;
       
           Capability *capability = capabilityMap[pTypeCap];
       
           Capability *copy = capability->clone();
           clockRate = 8;

           VDEBUG(log) << fn << ": Here is the packetSize : " << rtpPacketSize << VDEBUG_END(log);
           VDEBUG(log) << fn << ": default pSize of copy : " << copy->getPacketSizeMilSec() << VDEBUG_END(log);
	   
	   packetSize = rtpPacketSize*clockRate;
	   
	   if(rtpPacketSize != 0)
	   {
	     if(packetSize >= copy->getSamplesPerFrame())
	     {
	       copy->setParamRecvMsg(packetSize);
	     }
	     else 
	     {
	       VERR(log) << fn << "Packet Size in SDP is less than ms per Frame parameter "
	                   << "leaving this capability out..." << VERR_END(log);
	       fmtListIterator++;
	       continue;
	     }
	   }
	     
	     VDEBUG(log) << fn << ": Here is the cap from fmtList to add : " << pTypeCap << VDEBUG_END(log);
	   
	   copy->assignCapabilityNumber(pTypeCap);
	   capSetSip.h323CapTable.addCapTableEntry(copy);
           capSetSip.capRecv.addCapability(pTypeCap, 0, j);
	}

        fmtListIterator++;
     }

     VDEBUG(log) << fn << ": Out of formatList... " << VDEBUG_END(log);

    if(found)
    {

     SdpConnection *mediaConnection = (*mediaIterator)->getConnection();

     if(haveSesCon == false && mediaConnection == NULL)
       throw XlatorException("No connection parameters...", __FILE__, __LINE__);
     else
      if(mediaConnection == NULL)mediaConnection = sessionConnection;
      
     if(mediaConnection->getAddressType() != Vocal::SDP::AddressTypeIPV4)
       throw XlatorException("Not a IPV4 address...", __FILE__, __LINE__);
       
     sdpAddress = mediaConnection->getUnicast().convertString();
     
     if(sdpAddress == "")
       throw XlatorException("No Unicast address...", __FILE__, __LINE__);
     
     VDEBUG(log) << fn << ": IP Address from SDP : " << sdpAddress << VDEBUG_END(log);
     
     ipAddress.setIP(sdpAddress.c_str());
     ipAddress.setPort((*mediaIterator)->getPort());
     
     address = ipAddress;
     
     try
     {
       // Setting RTP host/port
       sDescriptor.transportIDSip.setRTPAddr(address, 0, j);
     }
     catch (XlatorException)
     {
       VERR(log) << fn << ": Something wrong with setting RTP address..." << VERR_END(log);
     }
     
     ipAddress.setPort((*mediaIterator)->getPort()+1);
     
     address = ipAddress;
     
     try
     {
       // Setting RTCP host/port
       sDescriptor.transportIDSip.setRTCPAddr(address, 0, j);
     }
     catch (XlatorException)
     {
       VERR(log) << fn << ": Something wrong with setting RTCP address..." << VERR_END(log);
     }

     j++;
    } // if(found)
   
     VDEBUG(log) << fn << ": While mediaIterator... " << VDEBUG_END(log);
     
     mediaIterator++;
   } // while mediaIterator

   if(!gFound)throw XlatorException("No capability supported...", __FILE__, __LINE__);

   int sTableSize = capSetSip.h323CapTable.getCapabilityTable().size();
   VDEBUG(log) << fn << ": Now the size of Sip capTable is : " << sTableSize << VDEBUG_END(log);
   
   receivedSDP = true;

   capabilitySetGetTestM(capSetSip);

   VDEBUG(log) << fn << ": END ------------------ " << VDEBUG_END(log);

}

void CapabilityMap::makeSDP(SDP &msgSDP) throw (XlatorException)
{
  const string	fn("CapabilityMap::makeSDP");
  VLog	    	log(fn);
  
  SdpConnection connection;
  IPAddress ipAddress;
  std::string connAddr;
  Capability::intSDP packetSize;
  int capNumber;
  int i = 0, descrSize = 0;
  int pTypeCap = 0;
  int clockRate = 0;
  
  VDEBUG(log) << fn << ": BEGIN -------------------- " << VDEBUG_END(log);
  
  try
  {
    ipAddress = sDescriptor.transportIDH323.getRTPAddr(0, 0);
  }
  catch (XlatorException &mS)
  {
    VERR(log) << fn << ": Error with getting RTPAddr from IDH323 : " << mS.getDescription() << VERR_END(log);
    throw XlatorException("Error with getting RTPAddr from IDH323", __FILE__, __LINE__);
  }
  
  VDEBUG(log) << fn << ": Here is ipAddress : " << ipAddress.getIPAsString() << VDEBUG_END(log);
  
  connAddr = ipAddress.getIPAsString();
  connection.setAddressType(Vocal::SDP::AddressTypeIPV4);
  connection.setUnicast(connAddr.c_str());
  msgSDP.setUserName("-");
  msgSDP.setConnection(connection);

  extern string	    SipToH323CallSigGW_HostAddress;

  VDEBUG(log) << fn << ": Here is the IP address : " << SipToH323CallSigGW_HostAddress << VDEBUG_END(log);
  msgSDP.setAddress(SipToH323CallSigGW_HostAddress.c_str());
  
  msgSDP.setSessionName("VOVIDA session");
  
  // at this point we assume that we use only first (0) capability descriptor
  // from CapabilityDescrVec
  
  descrSize = capSetIntersect.capRecv.getCapDescrSize(0);

  for(i = 0; i < descrSize; i++)
  {
    SdpMedia *media = new SdpMedia;
    ipAddress = sDescriptor.transportIDH323.getRTPAddr(0, i);
    media->setPort(ipAddress.getPort());
  
    MediaAttributes *mediaAttrib = new MediaAttributes();
    ValueAttribute *attrib = new ValueAttribute();
  
    capNumber = capSetIntersect.capRecv.getCapability(0, i, 0);
  
    //H323_CapTable::CapTable_I pI = capSetIntersect.h323CapTable.getTableIter_pt(capNumber);
    H323_CapTable::CapTable_I pI = fromOLCtoSDPcapTable.getTableIter_pt(capNumber);
    
    if(pI == NULL)
    {
	pI = capSetIntersect.h323CapTable.getTableIter_pt(capNumber);
        
	if(pI == NULL)
        {
              VCRIT(log) << fn << ": Did not find the entry in CapTable by number gotten from capDescriptor..." << VCRIT_END(log);
              throw XlatorException("Did not find the entry in CapTable by number gotten from capDescriptor",
                             __FILE__, __LINE__);
	}
    }
    
    (*pI)->getParamSendMsg(packetSize);

    clockRate = (*pI)->getClockRate()/1000;
    packetSize = packetSize/clockRate;

    attrib->setAttribute("ptime");
    LocalScopeAllocator lo;
    attrib->setValue(Data((unsigned long)packetSize).getData(lo));
  
    SdpRtpMapAttribute *rtpMapAttrib = new SdpRtpMapAttribute();
    pTypeCap = (*pI)->getCapabilityNumber();
    
    
    if(pTypeCap == 1)pTypeCap = 0;
    
    rtpMapAttrib->setPayloadType(pTypeCap);
  
    GlobalCapTable::GlobalCapTable_p pG;
    try
    {
      pG = globalCapTable.getTableIter_pt((*pI)->getCapabilityNumber());
    }
    catch (XlatorException)
    {
      VCRIT(log) << fn << ": Did not find the entry in GlobalCapTable..." << VCRIT_END(log);
      throw XlatorException("Did not find the entry in GlobalCapTable", __FILE__, __LINE__);
    }
  
    media->clearFormatList();
    media->addFormat(pTypeCap);

    rtpMapAttrib->setEncodingName(pG->nameCapIANA);
    
    rtpMapAttrib->setClockRate(pG->clockrateCap);
  
    mediaAttrib->addValueAttribute(attrib);
    mediaAttrib->addmap(rtpMapAttrib);
  
    media->setMediaAttributes(mediaAttrib);
  
    msgSDP.addMedia(media);
  }

  VDEBUG(log) << fn << ": END -------------------- " << VDEBUG_END(log);
}

int CapabilityMap::findAltCapSetN(int pTypeCap)
{
  const string	fn("CapabilityMap::findAltCapSetN");
  VLog	    	log(fn);

  int index = 0, index_pr = 0;
  mIter = beginTableIter();
  bool iDone = false;
  
  VDEBUG(log) << fn << ": BEGIN ----------------- " << VDEBUG_END(log);
  
  // trying to find the altCapSetNumber for the cap with pTypeCap, which is not in
  // mediaDescrTable yet, that is, no logical channels were opened for it yet
  
  while(mIter != mediaDescrTable.end())
  {
    try
    {
	// get the altCapNumber for a cap with pTypeCap
	index = capSetIntersect.capRecv.getCapabilityIndex(pTypeCap, curDescrN, index_pr);
	
	// get the iterator for this altCapNumber in mediaDescrTable
	// to find out whether this altCapNumber is already in use
	// or not; if it IS in use start looking in altCapSets starting from the
	// next to the one with altCapNumber = index
	
	VDEBUG(log) << fn << ": After getCapIndex: index : " << index << ", index_pr : " << index_pr << VDEBUG_END(log);
	
	mIter = std::find(mediaDescrTable.begin(), mediaDescrTable.end(), index);

	// besides just finding the index we should check whether the session is already
	// done
	
        try
        {
          iDone = sDescriptor.transportIDH323.isDone(curDescrN, index);
        }
        catch (XlatorException &mS)
        {
          VDEBUG(log) << fn << ": isDone error: " << mS.getDescription().c_str() << VDEBUG_END(log);
        }
	
	if(iDone)index_pr = index + 1;
	else
	{
	  VDEBUG(log) << fn << ": Current altCapN is in mediaDescrTable, but session is not Done yet..." << VDEBUG_END(log);
	  VDEBUG(log) << fn << ": ... which probably means we have already sent an OLC, but havent got any response" << VDEBUG_END(log);
	  VDEBUG(log) << fn << ": END ------------ " << VDEBUG_END(log);
	  
	  return index;
	}
    }
    catch (XlatorException)
    {
	VDEBUG(log) << fn << ": END - did not find the index" << VDEBUG_END(log);
	return -1;
    }
  }

  VDEBUG(log) << fn << ": END ----------------- " << VDEBUG_END(log);

  return index;
}


void CapabilityMap::generateSID(int &sID, int altCapN)
{
  const string	fn("CapabilityMap::generateSID");
  VLog	    	log(fn);

  VDEBUG(log) << fn << ": BEGIN ----------- " << VDEBUG_END(log);
  
  if(master)
  {
    sID = ++maxSesID;
    // do not support data currently
    if(sID == 3)sID = ++maxSesID;
  }
  else
  {
    if(maxSesID < sID)maxSesID = sID;  
  }
  
  sessionAltCapMap[sID] = altCapN;

  VDEBUG(log) << fn << ": END ----------- " << VDEBUG_END(log);
}

bool CapabilityMap::handleOLC_makeOLCAck(const OLC &msgOLC, 
                                         OLCAck &msgOLCAck,
				         H245_OpenLogicalChannelReject_cause::Choices *RejCause)
{
  const string	fn("CapabilityMap::handleOLC_makeOLCAck");
  VLog	    	log(fn);
  
  int pTypeCap = 0;
  unsigned int lChannelNumber = 0;
  int sID = 0, altCapN = 0, index;
  
  VDEBUG(log) << fn << ": BEGIN ----------------- " << VDEBUG_END(log);
  
  lChannelNumber = (int)msgOLC.m_forwardLogicalChannelNumber;
  VDEBUG(log) << fn << ": LC # : " <<  lChannelNumber << VDEBUG_END(log);
  
  const H245_DataType &dataType = msgOLC.m_forwardLogicalChannelParameters.m_dataType;

  // I'm afraid we should increment maxLogChN only when we form OLC
  // maxLogChN = lChannelNumber;
  
  if(msgOLC.m_forwardLogicalChannelParameters.m_multiplexParameters.GetTag() !=
     H245_OpenLogicalChannel_forwardLogicalChannelParameters_multiplexParameters
     ::e_h2250LogicalChannelParameters)
  {
    VERR(log) << fn << ": No H2250LogicalChannelParameters field in OLC..." << VERR_END(log);
    if ( RejCause != 0 )
    {
        *RejCause = H245_OpenLogicalChannelReject_cause::e_unspecified;
    }
    return false;
  }

  try
  {	
    pTypeCap = globalCapTable.findCapTableEntry(dataType);
  }
  catch (XlatorException)
  {
    VERR(log) << fn << ": Capability received in OLC not supported - weird..." << VERR_END(log);
    if ( RejCause != 0 )
    {
        *RejCause = H245_OpenLogicalChannelReject_cause::e_dataTypeNotSupported;
    }
    return false;
  }
      
  Capability *capability = capabilityMap[pTypeCap];
       
  Capability *copy = capability->clone();

  if(copy->setParamRecvMsg(dataType))
  {
    copy->assignCapabilityNumber(pTypeCap);
    
    H323_CapTable::CapTable_I pCap = capSetSip.h323CapTable.getTableIter_pt(pTypeCap);
    if(pCap == NULL)
    {
      VERR(log) << fn << ": Capability received in OLC - not in Sip CapTable" << VERR_END(log);
    if ( RejCause != 0 )
    {
      *RejCause = H245_OpenLogicalChannelReject_cause::e_unspecified;
    }
      return false;
    }
    
    VDEBUG(log) << fn << ": packetSize in OLC : " << copy->getPacketSizeMilSec() << VDEBUG_END(log);
    VDEBUG(log) << fn << ": packetSize in SDP : " << (*pCap)->getPacketSizeMilSec() << VDEBUG_END(log);
    
    if(copy->getPacketSizeSamples() > (*pCap)->getPacketSizeSamples())
    {
      if(strictModePackSize)
      {
        VERR(log) << fn << ": Packet size of Cap received in OLC > than in SDP" << VERR_END(log);
        if ( RejCause != 0 )
        {
            *RejCause = H245_OpenLogicalChannelReject_cause::e_unspecified;
        }
        return false;
      }
      else
      {
        VWARN(log) << fn << ": Packet size of Cap received in OLC > than in SDP" << VWARN_END(log);
      }
    }
      // add the capability to a table which then will be used to form
      // SDP message
      fromOLCtoSDPcapTable.addCapTableEntry(copy);
	    
  }
  else
  {
    delete copy;
    VERR(log) << fn << ": Could not set parameters for a cap received in OLC" << VERR_END(log);
    // should never happen
    if ( RejCause != 0 )
    {
        *RejCause = H245_OpenLogicalChannelReject_cause::e_unspecified;
    }
    return false;
  }

  const H245_H2250LogicalChannelParameters &params = 
        msgOLC.m_forwardLogicalChannelParameters.m_multiplexParameters;

   sID = params.m_sessionID;
	  
   if(sID == 3)
   {
	VERR(log) << fn << ": Do not support data for now..." << VERR_END(log);
        if ( RejCause != 0 )
        {
            *RejCause = H245_OpenLogicalChannelReject_cause::e_dataTypeNotSupported;
        }
	return false;
   }

   if((master)&&(sID > 3))
   {
      sA_I = sessionAltCapMap.find(sID);
      if(sA_I == sessionAltCapMap.end())
      {
	// we did not assign this sessionID
	VERR(log) << fn << ": Only master can assign the sessionID number > 3" << VERR_END(log);
        if ( RejCause != 0 )
        {
            *RejCause = H245_OpenLogicalChannelReject_cause::e_invalidSessionID;
        }
        return false;
      }
      else
      {
	  // this OLC message is the last in a given session
	  // we have already assigned the sessionID
	  // and sent the first OLC message with this sessionID
	      
	  index = sessionAltCapMap[sID];
	  try
	  {
	    sDescriptor.transportIDH323.setDone(curDescrN, altCapN);
	  }
	  catch (XlatorException &mS)
	  {
	    VERR(log) << fn << ": setDone error : " << mS.getDescription() << VERR_END(log);
	    VDEBUG(log) << fn << ": We are master, sID : " << sID << ", pTypeCap : " <<  pTypeCap << VDEBUG_END(log);
	    VDEBUG(log) << fn << ": curDescrN : " << curDescrN << ", altCapN : " << altCapN << VDEBUG_END(log);
            if ( RejCause != 0 )
            {
                *RejCause = H245_OpenLogicalChannelReject_cause::e_unspecified;
            }
	    return false;
	  }
      }
   }
   else if((master))
   {
        // here we should find out whether we already have
	// sent the OLC message for this particular pTypeCap
	
	// if altCapN for this pTypeCap is in mediaDescrTable
	// already but this particular session is not done yet
	// then obviously we have already sent an OLC for this
	// capability from this particular AltCapSet with this
	// altCapN, so this OLC we receive is the last in the
	// session
	
	// if we find altCapN for this particular capability,
	// for which the session is not done yet, then we
	// interpret the message we have just received
	// as the first OLC in a new session
        // going from slave endpoint, so we should assign
        // a new sessionID for this session as well as find
	// altCapN of the particular pTypeCap we received
	    
	VDEBUG(log) << fn << ": We are master and sID : " << sID << ", pTypeCap : " <<  pTypeCap << VDEBUG_END(log);
	
	altCapN = findAltCapSetN(pTypeCap);
	
	if((sID < 0)||(sID > 3))
	{
	    VERR(log) << fn << ": Invalid sessionID received from slave endpoint : " << sID << VERR_END(log);
            if ( RejCause != 0 )
            {
                *RejCause = H245_OpenLogicalChannelReject_cause::e_dataTypeNotSupported;
            }
	    return false;
	}
	
	
	if(altCapN == -1)
	{
	    VERR(log) << fn << ": Did not find the altCapSet for a cap received in OLC" << VERR_END(log);
            if ( RejCause != 0 )
            {
                *RejCause = H245_OpenLogicalChannelReject_cause::e_unspecified;
            }
	    return false;
	}

	VDEBUG(log) << fn << ": handle OLC: master now sID : " << sID << VDEBUG_END(log);
	
	MediaDescrTable_I mI = std::find(mediaDescrTable.begin(), mediaDescrTable.end(), altCapN);
	
	if(mI == mediaDescrTable.end())
	{
            // this is the first OLC message in a session, so we should do
            // all appropriate stuff

            generateSID(sID, altCapN);

            mediaDescrTable.push_back(altCapN);

            // we should assign the capability in the appropriate
            // sessionDescriptor

            try
            {
                sDescriptor.setCapability(pTypeCap, curDescrN, altCapN);
            }
	    catch (XlatorException)
	    {
	        VERR(log) << fn << ": Could not set cap for a given sessionDescriptor" << VERR_END(log);
                if ( RejCause != 0 )
                {
                    *RejCause = H245_OpenLogicalChannelReject_cause::e_unspecified;
                }
	        return false;
	    }
	}
	else
	{
	   // this is the last message in a session, just find out the 
	   // sessionID for this particular altCapN
	   
	   std::map<int, int>::const_iterator s_I = sessionAltCapMap.begin();
	   
	   while(s_I != sessionAltCapMap.end())
	   {
	     if(s_I->second == altCapN)
	     {
	       sID = s_I->first;
	       break;
	     }
	   }
	   
	   if(sID == 0)
	   {
	     VERR(log) << fn << ": Could not find appropriate sID, weird..." << VERR_END(log);
	     // basically should never happen
             if ( RejCause != 0 )
             {
                *RejCause = H245_OpenLogicalChannelReject_cause::e_invalidSessionID;
             }
	     return false;
	   }
	}
   }
   else if((!master)&&(sID == 0))
   {
        VERR(log) << fn << ": Being a slave and receiving sessionID - 0, weird..." << VERR_END(log);
        if ( RejCause != 0 )
        {
            *RejCause = H245_OpenLogicalChannelReject_cause::e_invalidSessionID;
        }
        return false;
   }
   else if((!master)&&(sID!=0))
   {
	// if the given sID is already associated with particular
	// altCapN, than this message
	// is the last one in a given session
	// otherwise (if this sID is not associated yet) it is
	// the first OLC message for a new session
	// so we should then send an OLC from
	// our side
	      
	if(sID == 3)
	{
	  VERR(log) << fn << ": Do not support data for now..." << VERR_END(log);
          if ( RejCause != 0 )
          {
            *RejCause = H245_OpenLogicalChannelReject_cause::e_invalidSessionID;
          }
	  return false;
	}
	
	sA_I = sessionAltCapMap.find(sID);

	if(sA_I != sessionAltCapMap.end())
	{
	   // this message is the last OLC for this session
	   
	   altCapN = sessionAltCapMap[sID];

	   MediaDescrTable_I mI = std::find(mediaDescrTable.begin(), mediaDescrTable.end(), altCapN);
	
	   if(mI == mediaDescrTable.end())
	   {
	     // this is the first OLC message in a session, so we should do
	     // all appropriate stuff
	  
	     generateSID(sID, altCapN);

	     mediaDescrTable.push_back(altCapN);
	   
	     // we should assign the capability in the appropriate
	     // sessionDescriptor
	  
	     try
	     {
	        sDescriptor.setCapability(pTypeCap, curDescrN, altCapN);
	     }
	      catch (XlatorException)
	     {
	        VERR(log) << fn << ": Could not set cap for a given sessionDescriptor" << VERR_END(log);
                if ( RejCause != 0 )
                {
                    *RejCause = H245_OpenLogicalChannelReject_cause::e_unspecified;
                }
	        return false;
	     }
	}
	else
	{
	   // this is the last message in a session, just find out the 
	   // sessionID for this particular altCapN
	   
	   std::map<int, int>::const_iterator s_I = sessionAltCapMap.begin();
	   
	   while(s_I != sessionAltCapMap.end())
	   {
	     if(s_I->second == altCapN)
	     {
	       sID = s_I->first;
	       break;
	     }
	   }
	   
	   if(sID == 0)
	   {
	     VERR(log) << fn << ": Could not find appropriate sID, weird..." << VERR_END(log);
	     // basically should never happen
             if ( RejCause != 0 )
             {
                *RejCause = H245_OpenLogicalChannelReject_cause::e_invalidSessionID;
             }
	     return false;
	   }
	}

	   try
	   {
	     sDescriptor.transportIDH323.setDone(curDescrN, altCapN);
	   }
	   catch (XlatorException &mS)
	   {
	     VDEBUG(log) << fn << ": Last OLC message" << VDEBUG_END(log);
	     VDEBUG(log) << fn << ": slave sID : " << sID << ", altCapN : " << altCapN << VDEBUG_END(log);
	     VERR(log) << fn << ": setDone error" << VERR_END(log);
             if ( RejCause != 0 )
             {
                *RejCause = H245_OpenLogicalChannelReject_cause::e_unspecified;
             }
	     return false;
	   }
	}
	else
	{
	  // this message is the first one for a new session
	  // should find the altCapN for this particular pTypeCap
	  
	  VDEBUG(log) << fn << ": First OLC message" << VDEBUG_END(log);
	  VDEBUG(log) << fn << ": We are slave; sID : " << sID << ", pTypeCap : " << pTypeCap << VDEBUG_END(log);
	  
	  altCapN = findAltCapSetN(pTypeCap);
	  if(altCapN == -1)
	  {
	     VERR(log) << fn << ": Did not find the altCapSet for a cap received in OLC" << VERR_END(log);
	     VERR(log) << fn << ": Probably LC with invalid capability in it" << VERR_END(log);
             if ( RejCause != 0 )
             {
                *RejCause = H245_OpenLogicalChannelReject_cause::e_unspecified;
             }
	     return false;
	  }
	  
	  generateSID(sID, altCapN);
	  mediaDescrTable.push_back(altCapN);
	  
	  VDEBUG(log) << fn << ": We are slave; pTypeCap : " << pTypeCap
	    	      << ", atlCapN : " << altCapN
		      << ", sID : " << sID << VDEBUG_END(log);
	  
	  // this message is the first OLC in a new session so
	  // we should assign the capability in the appropriate
	  // sessionDescriptor
	  try
	  {
	     sDescriptor.setCapability(pTypeCap, curDescrN, altCapN);
	  }
	  catch (XlatorException)
	  {
	     VERR(log) << fn << ": Could not set cap for a given sessionDescriptor" << VERR_END(log);
	     // should never happen
             if ( RejCause != 0 )
             {
                *RejCause = H245_OpenLogicalChannelReject_cause::e_unspecified;
             }
	     return false;
	  }
          
	  VDEBUG(log) << fn << ": We are slave, after setCapability" << VDEBUG_END(log);
	}
   }

  if(params.HasOptionalField(H245_H2250LogicalChannelParameters::e_mediaControlChannel))
  {
    const H245_TransportAddress &trAddress = params.m_mediaControlChannel;
    if(trAddress.GetTag() == H245_TransportAddress::e_unicastAddress)
    {
      const H245_UnicastAddress &uAddress = trAddress;
      if(uAddress.GetTag() == H245_UnicastAddress::e_iPAddress)
      {
        const H245_UnicastAddress_iPAddress &ipAddress = uAddress;
        
        Address address(ipAddress);   

	  try
          {
            // Setting RTCP host/port, currently use only first (# 0) capability descriptor
            sDescriptor.transportIDH323.setRTCPAddr(address, curDescrN, altCapN);
	    
	    // Setting Capability for a given RTP Session
	    sDescriptor.setCapability(pTypeCap, curDescrN, altCapN);
          }
          catch (XlatorException)
          {
            VERR(log) << fn << ": Something wrong with setting RTCP address..." << VERR_END(log);
          }
      }
      else VWARN(log) << fn << ": H245_UnicastAddress is not ipAddress" << VWARN_END(log);
    }
    else VWARN(log) << fn << ": H245_TransportAddress is not Unicast" << VWARN_END(log);
  }
  else VWARN(log) << fn << ": msgOLC does not have mediaControl channel" << VWARN_END(log);
  
  receivedOLC++;
  
  VDEBUG(log) << fn << ": Logical channel # : " << lChannelNumber << VDEBUG_END(log);
  
  msgOLCAck.m_forwardLogicalChannelNumber = lChannelNumber;
  msgOLCAck.IncludeOptionalField(H245_OpenLogicalChannelAck::e_forwardMultiplexAckParameters);
  msgOLCAck.m_forwardMultiplexAckParameters.SetTag(
    H245_OpenLogicalChannelAck_forwardMultiplexAckParameters::e_h2250LogicalChannelAckParameters);
  
  H245_H2250LogicalChannelAckParameters &paramAck = msgOLCAck.m_forwardMultiplexAckParameters;
  
  paramAck.IncludeOptionalField(
    H245_H2250LogicalChannelAckParameters::e_sessionID);  

  paramAck.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaChannel);  

  paramAck.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaControlChannel);  

  paramAck.m_sessionID = sID;

  H245_UnicastAddress_iPAddress mediaChannel;

  try
  {
    mediaChannel = sDescriptor.transportIDIntSip.getRTPAddr(curDescrN, altCapN);
  }
  catch (XlatorException)
  {
    VERR(log) << fn << ": Error with setting mediaChannel" << VERR_END(log);
    if ( RejCause != 0 )
    {
        *RejCause = H245_OpenLogicalChannelReject_cause::e_unspecified;
    }
    return false;
  }

  H245_TransportAddress &trAddress = paramAck.m_mediaChannel;
  trAddress.SetTag(H245_TransportAddress::e_unicastAddress);
  
  H245_UnicastAddress &uAddress = trAddress;
  uAddress.SetTag(H245_UnicastAddress::e_iPAddress);
  H245_UnicastAddress_iPAddress &ipAddress = uAddress;
        
  ipAddress = mediaChannel;

  H245_UnicastAddress_iPAddress mediaControlChannel;
  try
  {
    mediaControlChannel = sDescriptor.transportIDIntSip.getRTCPAddr(curDescrN, altCapN);
  }
  catch (XlatorException)
  {
    VERR(log) << fn << ": Error with setting mediaControlChannel" << VERR_END(log);
    if ( RejCause != 0 )
    {
        *RejCause = H245_OpenLogicalChannelReject_cause::e_unspecified;
    }
    return false;
  }

  H245_TransportAddress &trcAddress = paramAck.m_mediaControlChannel;
  trcAddress.SetTag(H245_TransportAddress::e_unicastAddress);
  
  H245_UnicastAddress &ucAddress = trcAddress;
  ucAddress.SetTag(H245_UnicastAddress::e_iPAddress);
  H245_UnicastAddress_iPAddress &ipcAddress = ucAddress;
        
  ipcAddress = mediaControlChannel;

  VDEBUG(log) << fn << ": END ----------------- " << VDEBUG_END(log);
  
  return true;
}

int CapabilityMap::findFirst(char cap, int &index) throw (XlatorException)
{
  const string	fn("CapabilityMap::findFirst");
  VLog	    	log(fn);

  int i = 0; 
  unsigned int j = 0;
  GlobalCapTable::GlobalCapTable_p p;
  
  VDEBUG(log) << fn << ": BEGIN ----------------" << VDEBUG_END(log);
  
  for(i = 0; i < capSetIntersect.capRecv.getCapDescrSize(curDescrN); i++)
  {
    const CapabilitySet::CapabilityDescrVec::AltCapSet &altCapSet = 
        capSetIntersect.capRecv.getAltCapSet(curDescrN, i);
    
    for(j = 0; j < altCapSet.size(); j++)
      VDEBUG(log) << fn << ": altCapSet[" << j << "] : " <<  altCapSet[j] << VDEBUG_END(log);
    
    for(j = 0; j < altCapSet.size(); j++)
    {
      try
      {
        p = globalCapTable.getTableIter_pt(altCapSet[j]);
      }
      catch (XlatorException)
      {
        throw XlatorException("Cannot find the capability from altCapSet in GlobalCapTable", __FILE__, __LINE__);
      }
      
      if(p->Audio_Video == cap)
      {
        index = i; // return the number of a given altCapSet
        VDEBUG(log) << fn << ": pTypeCap we found : " << p->ptypeCap << VDEBUG_END(log);
	VDEBUG(log) << fn << ": END ----------------" << VDEBUG_END(log);
	return p->ptypeCap;
      }
    }
  }

  VDEBUG(log) << fn << ": END ----------------" << VDEBUG_END(log);

  return -1;
}

int CapabilityMap::findNext(int &index) throw (XlatorException)
{
  int i = 0;
  MediaDescrTable_I mI;
  
  for(i = 0; i < capSetIntersect.capRecv.getCapDescrSize(curDescrN); i++)
  {
      mI = find(mediaDescrTable.begin(), mediaDescrTable.end(), i);
      if(mI == mediaDescrTable.end())
      {
        // we have found the next available altCapSet

        const CapabilitySet::CapabilityDescrVec::AltCapSet &altCapSet = 
              capSetIntersect.capRecv.getAltCapSet(curDescrN, i);
	
	index = i;
	// so return the first capability from it
	return altCapSet[0];
      }
  }
  throw XlatorException("Have reached the end of current CapDescriptor", __FILE__, __LINE__);
  return -1;
}

void CapabilityMap::makeOLC(OLC &msgOLC) throw (XlatorException)
{
   const string     fn("CapabilityMap::makeOLC");
   VLog     	    log(fn);
   
   int logChNumber = ++maxLogChN;
   int pTypeCap = 0, sID = maxSesID;
   int altCapN = 0;
   bool iDone = false;
   
   VDEBUG(log) << fn << ": BEGIN --------------" << VDEBUG_END(log);
   
   sA_I = sessionAltCapMap.find(sID);
   if(sA_I == sessionAltCapMap.end())
   {
     VDEBUG(log) << fn << ": Did not find an altCapNumber assosiated with current sID" << VDEBUG_END(log);
   }
   else
   {
     altCapN = sessionAltCapMap[sID];
   }

   try
   {
     iDone = sDescriptor.transportIDH323.isDone(curDescrN, altCapN);
   }
   catch (XlatorException &mS)
   {
     VDEBUG(log) << fn << ": isDone error: " << mS.getDescription() << VDEBUG_END(log);
   }
   
   if(iDone || (sID == 0))
   {
     // for the maxSesID we are done, so this is the first
     // message in a new session
     // so we start with new sessionID if we are master
     // or just assign 0 if we are slave
     
     if(master)
     {
       // if we are master, we create new sessionID
       sID = ++maxSesID;
       
       VDEBUG(log) << fn << ": We master, so sID : " << sID << VDEBUG_END(log);

       if(maxSesID == 1)
       {
         try
	 {
	   pTypeCap = findFirst('A', altCapN);
	   VDEBUG(log) << fn << ": Here is the pTypeCap we found first : " << pTypeCap << VDEBUG_END(log);
	 }
	 catch (XlatorException)
	 {
	   VCRIT(log) << fn << ": Cannot find the capability from altCapSet in GlobalCapTable" << VCRIT_END(log);
	   throw XlatorException("Cannot find the capability from altCapSet in GlobalCapTable",
	                         __FILE__, __LINE__);
	 }
       }
       else if(maxSesID == 2)
       {
         try
	 {
	   pTypeCap = findFirst('V', altCapN);
	 }
	 catch (XlatorException)
	 {
	   VCRIT(log) << fn << ": Cannot find the capability from altCapSet in GlobalCapTable" << VCRIT_END(log);
	   throw XlatorException("Cannot find the capability from altCapSet in GlobalCapTable",
	                         __FILE__, __LINE__);
	 }
       }
       // currently do not support data
       else
       {
         // here we should find the next altcapset for which we
	 // have not opened the logical channel yet
	 
	 try
	 {
	   pTypeCap = findNext(altCapN);
	 }
	 catch (XlatorException)
	 {
	   VCRIT(log) << fn << ": Trying to open logical channel, whereas we ran out of caps" << VCRIT_END(log);
	   throw XlatorException("Trying to open logical channel, whereas we ran out of caps",
	                         __FILE__, __LINE__);
	 }
       }
       
       // here we should add the association between
       // new assigned sessionID and altcapNumber for
       // which we issue OLC
       
       mediaDescrTable.push_back(altCapN);
       sessionAltCapMap[sID]=altCapN;
       VDEBUG(log) << fn << ": Orig iDone, sID 0; however now - sID : "
            	   << sID
		   << ", altCapN : "
		   << altCapN << VDEBUG_END(log);
     }
     else
     {
       // being a slave we just indicate 0
       sID = 0;
       
       // however, if primary sessions with sessionIDs 1 through 3 are not opened yet
       // we should assign appropriate sessionID
      
       if(maxSesID == 0)
       {
         // have not opened any session yet, so we are going to make OLC
	 // for to open first audio session
	 try
	 {
	   pTypeCap = findFirst('A', altCapN);
	   sID = 1;
	   VDEBUG(log) << fn << ": Here is the pTypeCap we found first : " << pTypeCap << VDEBUG_END(log);
	 }
	 catch (XlatorException)
	 {
	   VCRIT(log) << fn << ": Cannot find the capability from altCapSet in GlobalCapTable" << VCRIT_END(log);
	   throw XlatorException("Cannot find the capability from altCapSet in GlobalCapTable",
	                          __FILE__, __LINE__);
	 }
       }
       else if(maxSesID == 1)
       {
         // we have opened only audio session with sessionID = 1, so we are
	 // going to open video session
	 try
	 {
	   pTypeCap = findFirst('V', altCapN);
	   sID = 2;
	 }
	 catch (XlatorException)
	 {
	   VCRIT(log) << fn << ": Cannot find the capability from altCapSet in GlobalCapTable" << VCRIT_END(log);
	   throw XlatorException("Cannot find the capability from altCapSet in GlobalCapTable",
	                          __FILE__, __LINE__);
	 }
       }
       // currently do not support data
       else
       {
         // here we should find the next altcapset for which we
	 // have not opened the logical channel yet
	 
	 try
	 {
	   pTypeCap = findNext(altCapN);
	 }
	 catch (XlatorException)
	 {
	   VCRIT(log) << fn << ": Trying to open logical channel, whereas we ran out of caps" << VCRIT_END(log);
	   throw XlatorException("Trying to open logical channel, whereas we ran out of caps",
	                          __FILE__, __LINE__);
	 }
       }
       // add altCapN for which we open new OLC to mediaDescrTable
       // we will make the association between this altCapNumber
       // and sID which we will receive from master side in OLCAck
       // message
       
       VDEBUG(log) << fn << ": sID, altCapN, logChNumber : " << sID << ", " << altCapN << ", " << logChNumber << VDEBUG_END(log);
       
       mediaDescrTable.push_back(altCapN);
     }
       // plus we have to make an association between logChNumber
       // we are opening and altCapN
       
       altCapLogChMap[logChNumber] = altCapN;
   }
   else
   {
      // this OLC is for existing session, the last one
      // so we should use current sessionID
      
      
   // NB> should we undertake different actions in the case
   // if we are slave and master
   
      VDEBUG(log) << fn << ": sID : " << sID << VDEBUG_END(log);
      sID = maxSesID;
      
      altCapN = sessionAltCapMap[sID];
      
      try
      {
        // get the cap from a particular sessionDescriptor
        pTypeCap = sDescriptor.getCapability(curDescrN, altCapN);
      }
      catch (XlatorException)
      {
        VERR(log) << fn << ": Did not get the capability from sDescriptor right" << VERR_END(log);
	throw XlatorException("Did not get the capability from sDescriptor right",
	                       __FILE__, __LINE__);
      }

      // should set isDone flag for a given altCapSet
      sDescriptor.transportIDH323.setDone(curDescrN, altCapN);

   // NB> should we associate logChNumber with current altCapN
   // create the map between given logical channel number
   // and the altcapNumber for which we issue OLC
       
   // altCapLogChMap[logChNumber] = altCapN;
   }
     
   msgOLC.m_forwardLogicalChannelNumber = logChNumber;
   
   VDEBUG(log) << fn << ": sID, logChNumber, pTypeCap : " 
    	       << sID << ", " << logChNumber << ", " << pTypeCap << VDEBUG_END(log);
   H323_CapTable::CapTable_I cI = capSetIntersect.h323CapTable.getTableIter_pt(pTypeCap);
   
   if(cI == NULL)
   {
     VERR(log) << fn << ": Did not find the cap with pTypeCap in h323CapTable" << VERR_END(log);
     throw XlatorException("Did not find the cap with pTypeCap in h323CapTable",
                            __FILE__, __LINE__);
   }

   H245_DataType &dataType = msgOLC.m_forwardLogicalChannelParameters.m_dataType;

   Capability *capability = *cI;
   
   if(!capability->getParamSendMsg(dataType))
   {
     VERR(log) << fn << ": Did not get the Parameters correctly to form OLC msg" << VERR_END(log);
     throw XlatorException("Did not get the Parameters correctly to form OLC msg",
                            __FILE__, __LINE__);
   }

   msgOLC.m_forwardLogicalChannelParameters.m_multiplexParameters.
    SetTag(H245_OpenLogicalChannel_forwardLogicalChannelParameters_multiplexParameters::e_h2250LogicalChannelParameters);
  
   H245_H2250LogicalChannelParameters &params =
     msgOLC.m_forwardLogicalChannelParameters.m_multiplexParameters;
   
   params.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_mediaControlChannel);

   params.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_mediaGuaranteedDelivery);
   params.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_mediaControlGuaranteedDelivery);
   params.IncludeOptionalField(H245_H2250LogicalChannelParameters::e_silenceSuppression);
   
   params.m_mediaGuaranteedDelivery = FALSE;
   params.m_mediaControlGuaranteedDelivery = FALSE;
   params.m_silenceSuppression = FALSE;
   params.m_sessionID = sID;
   
   VDEBUG(log) << fn << ": Here is the sessionID we assigned : " << (int)params.m_sessionID << VDEBUG_END(log);

   H245_UnicastAddress_iPAddress mediaControlChannel;

   try
   {
     VDEBUG(log) << fn << ": altCapN, curDescrN for mediaControlChannel : " << altCapN << ' ' << curDescrN << VDEBUG_END(log);
     mediaControlChannel = sDescriptor.transportIDIntSip.getRTCPAddr(curDescrN, altCapN);
   }
   catch (XlatorException)
   {
     VERR(log) << fn << ": Error with setting mediaControlChannel" << VERR_END(log);
     // should never happen
     throw XlatorException("Error with setting mediaControlChannel", __FILE__, __LINE__);
   }

   H245_TransportAddress &trcAddress = params.m_mediaControlChannel;
   trcAddress.SetTag(H245_TransportAddress::e_unicastAddress);
  
   H245_UnicastAddress &ucAddress = trcAddress;
   ucAddress.SetTag(H245_UnicastAddress::e_iPAddress);
   H245_UnicastAddress_iPAddress &ipcAddress = ucAddress;
        
   ipcAddress = mediaControlChannel;

   sentOLC++;
   
   VDEBUG(log) << fn << ": END --------------" << VDEBUG_END(log);

}


bool CapabilityMap::handleOLCAck(const OLCAck &msgOLCAck)
{
   const string     fn("CapabilityMap::handleOLCAck");
   VLog     	    log(fn);
   
   int sID = 0;
   int logChNumber = 0;
   int altCapN = 0;
   std::map<int, int>::const_iterator aI, sI;
   
   VDEBUG(log) << fn << ": BEGIN ------------- " << VDEBUG_END(log);
   
   logChNumber = msgOLCAck.m_forwardLogicalChannelNumber;
   
   if(!msgOLCAck.HasOptionalField(
      H245_OpenLogicalChannelAck::e_forwardMultiplexAckParameters))
   {
     VERR(log) << fn << ": OLCAck without forwardMultiplexParameters" << VERR_END(log);
     return false;
   }
   
   if(msgOLCAck.m_forwardMultiplexAckParameters.GetTag() !=
     H245_OpenLogicalChannelAck_forwardMultiplexAckParameters::e_h2250LogicalChannelAckParameters)
   {
     VERR(log) << fn << ": no h2250LogicalChannelAckParameters" << VERR_END(log);
     return false;
   }
   
   const H245_H2250LogicalChannelAckParameters &params =
     msgOLCAck.m_forwardMultiplexAckParameters;
   
   if(!params.HasOptionalField(
     H245_H2250LogicalChannelAckParameters::e_sessionID))sID = 0;
   else
   {
     sID = params.m_sessionID;
   }
   

   if(master)
   {
     aI = altCapLogChMap.find(logChNumber);
     
     if(aI == altCapLogChMap.end())
     {
       VERR(log) << fn << ": Did not open LC with this logChNumber - invalid OLCAck" << VERR_END(log);
       return false;
     }
     
     // find association between logChNumber and altCapN
     altCapN = altCapLogChMap[logChNumber];
   }
   else
   { // we are slave - hate when people manipulate me :.(
     if(sID == 0)
     {
       // did not receive sessionID in OLCAck, therefore this OLCAck
       // is the last in a session
       
       aI = altCapLogChMap.find(logChNumber);
       if(aI == altCapLogChMap.end())
       {
         VERR(log) << fn << ": Did not open LC with this logChNumber - invalid OLCAck" << VERR_END(log);
	 return false;
       }
       
       // find association between logChNumber and altCapN
       altCapN = altCapLogChMap[logChNumber];
     }
     else
     {
       // did receive sessionID in OLCAck
       sI = sessionAltCapMap.find(sID);
     
       if(sI == sessionAltCapMap.end())
       {
         // given sID is not associated with any particular altCapN
	 // therefore we tend to interprete this situation as
	 // we sent OLC first with sID set to 0 (we are slave)
	 // therefore this OLCAck is the first in a session
	 // with valid sID
	 
	 aI = altCapLogChMap.find(logChNumber);
	 if(aI == altCapLogChMap.end())
	 {
	   VERR(log) << fn << ": Did not open LC with this logChNumber" << VERR_END(log);
	   return false;
	 }
	 
	 // find association between logChNumber and altCapN
	 altCapN = altCapLogChMap[logChNumber];
	 
	 // associate just received valid sessionID with a given altCapN
	 sessionAltCapMap[altCapN] = sID;
	 
	 // we need to adjust maxSesID since we received valid sessionID
	 // from master
	 if(maxSesID < sID)maxSesID = sID;
       }
       else
       {
         // sID is already associated with some particular altCapN
	 // this is the last OLCAck message in a current session
	 
	 altCapN = altCapLogChMap[logChNumber];
       }
       
       if(find(mediaDescrTable.begin(), mediaDescrTable.end(), altCapN) ==
          mediaDescrTable.end())
       {
         VERR(log) << fn << ": Hey, we forgot to add altCapN to mediaDescrTable. " 
	           << "Probably, on the stage of makeOLC" << VERR_END(log);
       }
     }
   }

   if(!params.HasOptionalField(
     H245_H2250LogicalChannelAckParameters::e_mediaChannel))
   {
     VERR(log) << fn << ": no mediaChannel - no RTP address" << VERR_END(log);
     return false;
   }

    const H245_TransportAddress &trAddress = 
      params.m_mediaChannel;

    if(trAddress.GetTag() == H245_TransportAddress::e_unicastAddress)
    {
      const H245_UnicastAddress &uAddress = trAddress;
      if(uAddress.GetTag() == H245_UnicastAddress::e_iPAddress)
      {
        const H245_UnicastAddress_iPAddress &ipAddress = uAddress;
        
        Address address(ipAddress);   
   
 	  try
          {
            // Setting RTP host/port, currently use only first (# 0) capability descriptor
            sDescriptor.transportIDH323.setRTPAddr(address, curDescrN, altCapN);
          }
          catch (XlatorException)
          {
            VERR(log) << fn << ": Something wrong with setting RTP address..." << VERR_END(log);
	    // basically should never happen
	    return false;
          }
      }
    }
    else
    {
      VERR(log) << fn << ": Address is not supported" << VERR_END(log);
      return false;
    }
    

   if(!params.HasOptionalField(
     H245_H2250LogicalChannelAckParameters::e_mediaControlChannel))
   {
    const H245_TransportAddress &trAddress = 
      params.m_mediaControlChannel;

    if(trAddress.GetTag() == H245_TransportAddress::e_unicastAddress)
    {
      const H245_UnicastAddress &uAddress = trAddress;
      if(uAddress.GetTag() == H245_UnicastAddress::e_iPAddress)
      {
        const H245_UnicastAddress_iPAddress &ipAddress = uAddress;
        
        Address address(ipAddress);   
   
 	try
        {
            // Setting RTCP host/port, currently use only first (# 0) capability descriptor
            sDescriptor.transportIDH323.setRTCPAddr(address, curDescrN, altCapN);
        }
        catch (XlatorException)
        {
            VERR(log) << fn << ": Something wrong with setting RTCP address..." << VERR_END(log);
	    // basically should never happen
	    return false;
        }
      }
    }
   }

   receivedOLCAck++;
   VDEBUG(log) << fn << ": END ------------- " << VDEBUG_END(log);
   return true;
}

bool CapabilityMap::isSDPreceived() const
{
    return receivedSDP;
}

bool CapabilityMap::areAllOLCsent() const
{
    int descrSize = capSetIntersect.capRecv.getCapDescrSize(curDescrN);
    
    return (descrSize == sentOLC);

}

bool CapabilityMap::areAllOLCAckreceived() const
{
    int descrSize = capSetIntersect.capRecv.getCapDescrSize(curDescrN);
    
    return (descrSize == receivedOLCAck);
}

bool CapabilityMap::areAllOLCreceived() const
{
    int descrSize = capSetIntersect.capRecv.getCapDescrSize(curDescrN);
    
    return (descrSize == receivedOLC);
}
