/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const CapabilityTranslator_cxx_Version = 
"$Id: CapabilityTranslator.cxx,v 1.4 2001/05/25 07:42:07 icahoon Exp $";

#include "CapabilityTranslator.hxx"
#include "VLog.hxx"
#include <string>

using Vocal::Logging::VLog;
using std::string;


CapabilityTranslator::CapabilityTranslator::CapabilityTranslator()
    : em_(0)
    , gotOlc_(false)
    , gotAck_(false)
    , gotSDP_(false)
    , gotCC_(false)
{
}


CapabilityTranslator::CapabilityTranslator::~CapabilityTranslator()
{
}


void CapabilityTranslator::CapabilityTranslator::setMaster(bool mI)
{
    map_.setMaster(mI);
}
	

void CapabilityTranslator::CapabilityTranslator::setControlChannel(bool ccI)
{
    const string    fn("CapabilityTranslator::setControlChannel");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);
    
    gotCC_ = ccI;

    if (gotCC_ && gotSDP_)
    {
        VDEBUG(log) << fn << ": sending TCS." << VDEBUG_END(log);
        sendTCS(0);
    }
}
	

void CapabilityTranslator::CapabilityTranslator::setPhaseC(bool pcI)
{
    if (pcI)
    {
        map_.makeIntersect();

        OLC olc;
        map_.makeOLC(olc);
        em_->startLCOEstablishReq(olc);
    }
    else
    {
        // ignore
    }
}
	

void CapabilityTranslator::CapabilityTranslator::resolved(std::string hostName, Transport::IPAddress address)
{
    // ignore
}
	

void CapabilityTranslator::CapabilityTranslator::enterFS()
{
    // ignore
}
	

void CapabilityTranslator::CapabilityTranslator::handleFSOLC(const OLC&)
{
    // ignore
}
	

void CapabilityTranslator::CapabilityTranslator::exitFS()
{
    // ignore
}
	

void CapabilityTranslator::CapabilityTranslator::handleSDP(SDPS &msgSDP)
{
    const string    fn("CapabilityTranslator::handleSDP");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);
    
    gotSDP_ = true;
    map_.handleSDP(msgSDP);

    if (gotCC_)
    {
        VDEBUG(log) << fn << ": sending TCS." << VDEBUG_END(log);

        sendTCS(0);
    }

    em_->startSDPParseDone();
}
	

void CapabilityTranslator::CapabilityTranslator::handleEmptySDP()
{
    em_->startSDPParseDone();
}


void CapabilityTranslator::CapabilityTranslator::handleTCS(const TCS &msgTCS)
{
    map_.handleTCS(msgTCS);
}
	

void CapabilityTranslator::CapabilityTranslator::handleOLC(const OLC& olc)
{
    OLCAck response;

    map_.handleOLC_makeOLCAck(olc, response);
    em_->startLCIEstablishResp(response);
    gotOlc_ = true;

    if (gotAck_)
    {
        sendSDP();
    }
}
	

void CapabilityTranslator::CapabilityTranslator::handleOLCAck(const OLCAck& ack)
{
    map_.handleOLCAck(ack);
    gotAck_ = true;

    if (gotOlc_)
    {
        sendSDP();
    }
}
	

void CapabilityTranslator::CapabilityTranslator::handleCLC(const CLC&)
{
    // ignore
}


void CapabilityTranslator::CapabilityTranslator::handleRCC(const RCC&)
{
    // ignore
}


void CapabilityTranslator::CapabilityTranslator::handleOLCReject(const OLCReject&)
{
    // ignore
}
	

void CapabilityTranslator::CapabilityTranslator::closeInLC(unsigned)
{
    // ignore
}
	

void CapabilityTranslator::CapabilityTranslator::closeOutLC(unsigned)
{
    // ignore
}
	

void CapabilityTranslator::CapabilityTranslator::cancelSDP()
{
    // ignore
}
	

void CapabilityTranslator::CapabilityTranslator::confirmSDP()
{
    // ignore
}
	

void CapabilityTranslator::CapabilityTranslator::sendSDP()
{
    SDPS msg;
    map_.makeSDP(msg);
    em_->startSendSDP(msg, true);
}
	

void CapabilityTranslator::CapabilityTranslator::sendTCS(int)
{
    TCS msg;
    map_.makeTCS(msg);
    em_->startCEOTransferReq(msg);
}
	

void CapabilityTranslator::CapabilityTranslator::add(translator::CallTranslatorEM* em)
{
    em_ = em;
}
