/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtMediaReady_cxx_Version = 
    "$Id: EvtMediaReady.cxx,v 1.10 2002/03/01 22:51:07 bko Exp $";


#include <cassert>

#include "VLog.hxx"

#include "AckMsg.hxx"
#include "StatusMsg.hxx"
#include "SipSdp.hxx"

#include "EvtMediaReady.hxx"
#include "FeatureTrCallSig.hxx"
#include "TrConnectionSIP.hxx"
#include "TrConnectionSDP.hxx"
#include "CapabilityMap.hxx"

using namespace translator;
using Vocal::Logging::VLog;
using Vocal::SDP::SdpSession;
using Vocal::SDP::SdpConnection;
using Vocal::SDP::SdpMedia;


EvtMediaReady::
EvtMediaReady(TrSMCall* sm)
    : EvtUserTrCallSig(sm)
{
}


void EvtMediaReady::
inWaitS2HMedia(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtMediaReady::inWaitS2HMedia");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    ++trcallsig.mediaReady_;

    // If both media SM are ready
    if (trcallsig.mediaReady_ >= 2)
    {
        // (depends from SIP parser)

        trcallsig.setState(&PatternTrCallSig::inWaitAck, "inWaitAck");

        // Fill SDP
        SdpSession sdp;
        trcallsig.getSM()->getCapabilityMap().makeSDP(sdp);
        trcallsig.getSM()->getSDPConnection()->send(sdp);

        Sptr<StatusMsg> sip200ok = new StatusMsg(*(trcallsig.invite_), 200);
        trcallsig.getSM()->getSIPConnection()->send(sip200ok);
        trcallsig.status_ = sip200ok;
    }

}


void EvtMediaReady::
inWaitH2SMedia(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtMediaReady::inWaitH2SMedia");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    ++trcallsig.mediaReady_;

    // If both media SM are ready
    if (trcallsig.mediaReady_ >= 2)
    {
        // (depends from H.323/SIP parser)

        trcallsig.setState(&PatternTrCallSig::inConnected, "inConnected");

        // Fill SDP
        SdpSession sdp;
        trcallsig.getSM()->getCapabilityMap().makeSDP(sdp);
        trcallsig.getSM()->getSDPConnection()->send(sdp);

        Sptr<AckMsg> sipAck = new AckMsg(*(trcallsig.status_));
        trcallsig.getSM()->getSIPConnection()->send(sipAck);
    }
}


void EvtMediaReady::
inOther(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtMediaReady::inOther");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    ++trcallsig.mediaReady_;
}


#ifndef DISABLE_WRITER

ostream& EvtMediaReady::
writeTo(ostream& out) const
{
    out << "EvtMediaReady";
    return out;
}

#endif
