/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtTrCalligTimer_cxx_Version = 
    "$Id: EvtTrCallSigTimer.cxx,v 1.3 2001/10/24 21:26:43 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "EvtTrCallSigTimer.hxx"
#include "Consts.hxx"
#include "FeatureTrCallSig.hxx"
#include "CapabilityTranslator.hxx"
#include "TrConnectionSIP.hxx"

using namespace translator;
using Vocal::Logging::VLog;

EvtTrCallSigTimer::
EvtTrCallSigTimer(TrSMCall* sm)
    : TrEvtTimerCallSig(sm),
      timeout_(0)
{
}


void EvtTrCallSigTimer::
inSDPH2S(FeatureTrCallSig& callsig)
{
    const string    fn("EvtTrCallSigTimer::inIdleRAS");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Force SDP sending
    callsig.getSM()->getCapabilityTranslator().sendSDP();
}


void EvtTrCallSigTimer::
inSDPInvite(FeatureTrCallSig& callsig)
{
    const string    fn("EvtTrCallSigTimer::inIdleNoRAS");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Force SDP sending
    callsig.getSM()->getCapabilityTranslator().sendSDP();
}


void EvtTrCallSigTimer::
inCollision(FeatureTrCallSig& callsig)
{
    const string    fn("EvtTrCallSigTimer::inAdmitted");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    if (callsig.collisionCnt_ < 3)
    {
        // Continue sending invite
        callsig.getSM()->getSIPConnection()->send(callsig.invite_);
        callsig.setState(&PatternTrCallSig::inWait200, "inWait200");
        ++ callsig.collisionCnt_;
    }
    else
    {
        callsig.getSM()->getSIPConnection()->disable();
        callsig.getSM()->complete();
    }
}


void EvtTrCallSigTimer::
inOther(FeatureTrCallSig& callsig)
{
    const string    fn("EvtTrCallSigTimer::inOther");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    assert(false);
}


mte::TimeInterval EvtTrCallSigTimer::
getTimerValue() const throw()
{
    return timeout_;
}


void EvtTrCallSigTimer::
setTimeout(mte::TimeInterval timeout) throw()
{
    const string    fn("EvtTrCallSigTimer::setTimeout(");
    VLog            log(fn);
    
    VDEBUG(log) << fn << timeout << ")" << VDEBUG_END(log);

    timeout_ = timeout;
}


#ifndef DISABLE_WRITER

ostream& EvtTrCallSigTimer::
writeTo(ostream& out) const
{
    out << "EvtTrCallSigTimer";
    return out;
}

#endif
