/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtTrTCSAck_cxx_Version = 
    "$Id: EvtTrTCSAck.cxx,v 1.11 2001/10/24 21:26:43 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "TSMMediator.hxx"
#include "EvtTrTCSAck.hxx"
#include "FeatureTrMediaS2H.hxx"
#include "TrConnectionSDP.hxx"
#include "CapabilityTranslator.hxx"
#include "EvtMediaReady.hxx"

using namespace translator;
using Vocal::Logging::VLog;


EvtTrTCSAck::
EvtTrTCSAck(router::RouterCallCtl* tr, TrSMCall* sm)
    : mte::EvtBind11<router::EvtTCSAck, EvtDirectTrMediaS2H>(tr, sm)
{
}


void EvtTrTCSAck::
inCapSent(FeatureTrMediaS2H& trmedias2h)
{
    const string    fn("EvtTrTCSAck::inCapSent");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    trmedias2h.setState(&PatternTrMediaS2H::inCapAck, "inCapAck");

    EvtMediaReady mr(trmedias2h.getSM());
    mr.start();
}


void EvtTrTCSAck::
inOther(FeatureTrMediaS2H& trmedias2h)
{
    const string    fn("EvtTrTCSAck::inOther");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Ignore
    VWARN(log) << fn << ": Got unexpected TCS Ack message for call "
               << trmedias2h.getSM() << VWARN_END(log);
}


router::EvtH245B* EvtTrTCSAck::
clone_impl()
{
    return new EvtTrTCSAck(*this);
}


void EvtTrTCSAck::
updateRoute()
{
    assert(getConnection());
    assert(getConnection()->getSM());

    TSMMediator& m = dynamic_cast<TSMMediator&>(
        getConnection()->getSM()->getMaster()
    );

    setDirectTarget(&(m.getSM()));
}


#ifndef DISABLE_WRITER

ostream& EvtTrTCSAck::
writeTo(ostream& out) const
{
    out << "EvtTrTCSAck";
    mte::EvtBind11<router::EvtTCSAck, EvtDirectTrMediaS2H>::writeTo(out);
    return out;
}

#endif
