#ifndef INCLUDED_FeatureTrCallSig_hxx
#define INCLUDED_FeatureTrCallSig_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const FeatureTrCallSig_hxx_Version = 
"$Id: FeatureTrCallSig.hxx,v 1.9 2001/05/09 23:15:43 vzubarev Exp $";

#include "Sptr.hxx"

#include "mte/EventPtr.hxx"
#include "mte/Feature.hxx"

#include "InviteMsg.hxx"
#include "StatusMsg.hxx"
#include "AckMsg.hxx"

#include "Address.hxx"
#include "H323RouterTypes.hxx"
#include "PatternTrCallSig.hxx"
#include "FeatureTrCallSigDefs.hxx"
#include "Contacts.hxx"
#include "EvtTrCallSigTimer.hxx"
#include "EntryTrCallSig.hxx"
#include "ExitTrCallSig.hxx"

namespace translator
{

    class TrSMCall;
    
    /// Call Signalling state variables
    class TrCallSigData
    {
    private:
        TrCallSigData()
            : alerted_(false)
            , statusCode_(0)
            , waitTime_(0)
            , collisionCnt_(0)
            {}

    public:
        Sptr<InviteMsg> invite_;
        Sptr<StatusMsg> status_;
        Sptr<AckMsg> ack_;
        bool alerted_;
        int statusCode_;
        mte::TimeInterval waitTime_;
        int collisionCnt_;

	Contacts& getContacts()
	    { return contacts_; }

private:
        Contacts contacts_;

        // These classes are allowed to access constructor:
        friend class FeatureTrCallSig;
    };

    /// Feature for Call Signalling
    class FeatureTrCallSig:
        public FeatureTrCallSigBase,
        public TrCallSigData
    {
    public:
        /// Construct feature and assign pointer to state machine
        FeatureTrCallSig(TrSMCall* sm);
        
        virtual ~FeatureTrCallSig();

        /// Return pointer to the state machine
        TrSMCall* getSM() const;

        /// Reset feature
        virtual void reset();

        int getReason();

        void setReason(int reason);

        /// Start timer
        void startTimer(mte::TimeInterval timeout);

        /// Stop timer
        void stopTimer();

#ifndef DISABLE_WRITER
        virtual ostream& writeTo(ostream& out) const;
#endif

    private:
        /// Process event (implementation)
        virtual void process_impl(PatternTrCallSig& pattern);

#ifndef DISABLE_WRITER
        char* getStateName() const;
#endif
        
        TrSMCall* const sm_;
        Sptr<EvtTrCallSigTimer> timer_;
        EntryTrCallSig entry_;
        ExitTrCallSig exit_;
    };
    
    //----------------------------
    // Inline  methods
    //----------------------------

    inline TrSMCall* FeatureTrCallSig::
    getSM() const
    {
        return sm_;
    }

}

#endif
