#ifndef __GCAPTABLE_H_
#define __GCAPTABLE_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const GlobalCapTable_hxx_Version = 
"$Id: GlobalCapTable.hxx,v 1.12 2001/05/13 09:06:38 icahoon Exp $";

#include <vector>
#include <algorithm>
#include <iterator>
#include <functional>
#include <string>
#include "XlatorException.hxx"

namespace Translator
{
    
    ///
    class GlobalCapTable
    {
    private:
	///
	struct GlobalCapTableEntry
	{
	    /// Payload type # as assigned by IANA
	    int ptypeCap;
	    
	    /// name of the capability per H323 standard
	    int nameCapH323;
	    
	    /// name of the capability per IANA
	    char* nameCapIANA;
	    
	    ///
	    char Audio_Video;
	    
	    ///
	    int clockrateCap;
	    
	    ///
	    int channels;
	    
	    ///
	    char* numberRFC;
	    
	    ///
	    char* atrAdditional;
	};
    
	/// const int size for globalCapTable
	enum { size = 35 };
	
	///
	static const GlobalCapTableEntry globalCapTable[size];
        
	/// Predicate for getTableIter_pt
	class GlCapTab_pt : public std::unary_function<GlobalCapTableEntry, bool>
        {
        private:
            int ptype;
    
        public:
            explicit GlCapTab_pt(int pt) : ptype(pt) {}
	
	    bool operator() (const GlobalCapTableEntry &gcTable) const
	        { return gcTable.ptypeCap==ptype; }
        };

        /// Predicate for getTableIter_ni
        class GlCapTab_ni : public std::unary_function<GlobalCapTableEntry, bool>
        {
        private:
            std::string nameI;
	
        public:
            explicit GlCapTab_ni(std::string ss) : nameI(ss) {}

	    bool operator() (const GlobalCapTableEntry &gcTable) const
	        { return gcTable.nameCapIANA==nameI; }
        };

        /// Predicate for getTableIter_nh
        class GlCapTab_nh : public std::unary_function<GlobalCapTableEntry, bool>
        {
        private:
            int nameH;
	
        public:
            explicit GlCapTab_nh(int ss) : nameH(ss) {}

	    bool operator() (const GlobalCapTableEntry &gcTable) const
	        { return gcTable.nameCapH323==nameH; }
        };

    public:
	///
	typedef const GlobalCapTableEntry* GlobalCapTable_p;
	
    private:
	///
	GlobalCapTable_p pGlCapTable;

    public:
        ///
	GlobalCapTable() {};
	
	///
	~GlobalCapTable() {};
	
	/// To access the table entry by payload type
	GlobalCapTable_p getTableIter_pt(int ptype) throw (XlatorException);

        /// To access the table entry by nameIANA
	GlobalCapTable_p getTableIter_ni(std::string nameI) throw (XlatorException);

        /// To access the table entry by nameH323
	GlobalCapTable_p getTableIter_nh(int nameH) throw (XlatorException);
	
	///
	int findCapTableEntry(const H245_Capability &) throw (XlatorException);
	
	///
	int findCapTableEntry(const H245_DataType &) throw (XlatorException);
    };

}

#endif
