#ifndef INCLUDED_SIPID_hxx
#define INCLUDED_SIPID_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const SIPID_hxx_Version = 
"$Id: SIPID.hxx,v 1.14 2002/07/10 07:10:23 icahoon Exp $";

#include "VLog.hxx"

#include "IncomingSipEvent.hxx"
#include "SipCallId.hxx"

namespace translator
{

    using Vocal::Services::Event;
    
    class TrConnectionSIP;

    /// SIP ID
    class SIPID
    {
    public:
        SIPID();
        void setProxy(Vocal::FifoBase< Sptr<Event> >* proxy);
        Vocal::FifoBase< Sptr<Event> >* getProxy() throw()
            { return proxy_; }
        bool operator < (const SIPID& other) const;

        Vocal::SipCallId id_;

    private:
        Vocal::FifoBase< Sptr<Event> >* proxy_;
    };

    //----------------------------
    // Inline and template methods
    //----------------------------

    inline SIPID::
    SIPID()
    {
    }
    
    inline void SIPID::
    setProxy(Vocal::FifoBase< Sptr<Event> >* proxy)
    {
        proxy_ = proxy;
    }

    inline bool SIPID::
    operator < (const SIPID& other) const
    {
        Vocal::Logging::VLog log;
            VDEBUG(log) << "this'=" << this
                        << " pr'=" << &proxy_
                        << VDEBUG_END(log);
	VDEBUG(log) << "Comparing p=" << proxy_ << " with p=" << other.proxy_ << VDEBUG_END(log);

        if (proxy_ == other.proxy_)
	{
	    VDEBUG(log) << "Proxies are equal, comparing ID..." << VDEBUG_END(log);
            return id_ < other.id_;
	}
        else
        {
	    VDEBUG(log) << "Proxies are different" << VDEBUG_END(log);
	    return proxy_ < other.proxy_;
	}
    }

}

#endif
