
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const TrCall_cxx_Version = 
"$Id: TrCall.cxx,v 1.14 2001/05/09 23:15:43 vzubarev Exp $";
#include "VLog.hxx"

#include "FeatureCallSig.hxx"

#include "TrCall.hxx"
#include "TrRouterSIP.hxx"
#include "TrRouterRas.hxx"

using namespace translator;

TrCall::
TrCall()
    : TrSMCall(),
      mediator_(*this),
      h323Call_(mediator_),
      sipConnection_(this),
      sdpConnection_(this),
      rasConnection_(this),
      callsig_(this),
      mediaSDP_(this),
      evtTrCCOpen_(this),
      close_(this),
      terminate_(&h323Call_, this),
      complete_(&h323Call_),
      routerSip_(0),
      routerRas_(0),
      evtTrMsdConfirm_(&h323Call_, this),
      h245Patch_(false),
      threadPool_(0),
      ePhaseB_(this),
      ePhaseC_(this),
      eEnterFS_(this),
      eExitFS_(this),
      eFSOLC_(this),
      evtResolve_(this),
      evtSendFSOLC_(&h323Call_),
      evtZeroTCS_(this),
      evtZeroSDP_(this),
      evtSendSDP_(this),
      evtCEOTransferReq_(&h323Call_),
      evtLCOEstablishReq_(h323Call_.getRtLCO()),
      evtLCOReleaseReq_(h323Call_.getRtLCO()),
      evtLCIEstablishResp_(h323Call_.getRtLCI()),
      evtLCIReleaseReq_(h323Call_.getRtLCI()),
      evtRCORequest_(h323Call_.getRtLCI()),
      evtCreateInLC_(&h323Call_),
      evtCreateOutLC_(&h323Call_),
      evtSDPParseDone_(this),
      evtForceCCOpen_(&h323Call_),
      em_()
{
    assert(h323Call_.getCallCtlSession());
    h323Call_.getCallCtlSessionImp().ccConnected_ = &evtTrCCOpen_;
    h323Call_.getMsdImp().callbackConfirm_ = &evtTrMsdConfirm_;
    h323Call_.callbackComplete_ = &complete_;
    h323Call_.callbackTerminate_ = &terminate_;
    h323Call_.callbackClose_ = &close_;
    h323Call_.getCallSigImp().callCleanupInd_ = &terminate_;
    h323Call_.getH245Imp().phaseB_ = &ePhaseB_;
    h323Call_.getH245Imp().phaseC_ = &ePhaseC_;
    h323Call_.getCallSigImp().fsEnterInd_ = &eEnterFS_;
    h323Call_.getCallSigImp().fsExitInd_ = &eExitFS_;
    h323Call_.getCallSigImp().fsOLCInd_ = &eFSOLC_;

    em_.add(&evtResolve_);
    em_.add(&evtSendFSOLC_);
    em_.add(&evtZeroTCS_);
    em_.add(&evtZeroSDP_);
    em_.add(&evtSendSDP_);
    em_.add(&evtCEOTransferReq_);
    em_.add(&evtLCOEstablishReq_);
    em_.add(&evtLCOReleaseReq_);
    em_.add(&evtLCIEstablishResp_);
    em_.add(&evtLCIReleaseReq_);
    em_.add(&evtRCORequest_);
    em_.add(&evtCreateInLC_);
    em_.add(&evtCreateOutLC_);
    em_.add(&evtSDPParseDone_);
    em_.add(&evtForceCCOpen_);

    capabilityTranslator_.add(&em_);
}


TrCall::
~TrCall()
{
    const string    	fn("TrCall::~TrCall");
    VLog    	    	log(fn);

    // Turn off all callbacks to prevent race conditions
    h323Call_.getCallCtlSessionImp().ccConnected_ = 0;
    h323Call_.getMsdImp().callbackConfirm_ = 0;
    h323Call_.callbackComplete_ = 0;
    h323Call_.callbackTerminate_ = 0;
    h323Call_.callbackClose_ = 0;
    h323Call_.getCallSigImp().callCleanupInd_ = 0;
    h323Call_.getH245Imp().phaseB_ = 0;
    h323Call_.getH245Imp().phaseC_ = 0;
    h323Call_.getCallSigImp().fsEnterInd_ = 0;
    h323Call_.getCallSigImp().fsExitInd_ = 0;
    h323Call_.getCallSigImp().fsOLCInd_ = 0;

    routerSip_->remove(sipConnection_);

    if (routerRas_)
    {
        routerRas_->remove(rasConnection_);
    }
}


FeatureTrCallSigBase* TrCall::
getTrCallSig() throw()
{
    assert(this);

    return &callsig_;
}


FeatureTrSDPBase* TrCall::
getTrSDP() throw()
{
    assert(this);

    return &mediaSDP_;
}


FeatureTrSDP* TrCall::getTrSDPImpl() throw()
{
    assert(this);

    return &mediaSDP_;
}


TrRouterSIP* TrCall::
getSipRouter() throw()
{
    assert(this);

    return routerSip_;
}


TrRouterRas* TrCall::
getRasRouter() throw()
{
    assert(this);

    return routerRas_;
}


TrConnectionSIP* TrCall::
getSIPConnection() throw()
{
    assert(this);

    return &sipConnection_;
}


TrConnectionSDP* TrCall::
getSDPConnection() throw()
{
    assert(this);

    return &sdpConnection_;
}


TrConnectionRas* TrCall::
getRasConnection() throw()
{
    assert(this);

    return &rasConnection_;
}


router::Call* TrCall::
getH323Call() throw()
{
    assert(this);

    return &h323Call_;
}


void TrCall::
setGatekeeperFifo(FifoBase< Sptr<Event> >* gatekeeperFifo)
    throw()
{
    assert(this);

    gatekeeperFifo_ = gatekeeperFifo;
}


void TrCall::
setProxyFifo(FifoBase< Sptr<Event> >* sipProxyFifo) throw()
{
    assert(this);

    sipProxyFifo_ = sipProxyFifo;
}


FifoBase< Sptr<Event> >* TrCall::
getGatekeeperFifo() throw()
{
    assert(this);

    return gatekeeperFifo_;
}


FifoBase< Sptr<Event> >* TrCall::
getProxyFifo() throw()
{
    assert(this);

    return sipProxyFifo_;
}


void TrCall::
setSipRouter(TrRouterSIP* sipRouter) throw()
{
    assert(this);
    assert(sipRouter);

    routerSip_ = sipRouter;
}


void TrCall::
setRasRouter(TrRouterRas* rasRouter) throw()
{
    assert(this);
    assert(rasRouter);

    routerRas_ = rasRouter;
}


void TrCall::
setThreadPool(VThreadPool* threadPool) throw()
{
    assert(this);
    assert(threadPool);

    threadPool_ = threadPool;
}


VThreadPool* TrCall::
getThreadPool() throw()
{
    assert(this);

    return threadPool_;
}


CapabilityTranslator::CapabilityTranslator& TrCall::
getCapabilityTranslator() throw()
{
    assert(this);

    return capabilityTranslator_;
}


void TrCall::
complete()
{
    assert(this);
    const string    	fn("TrCall::complete");
    VLog    	    	log(fn);

    getH323Call()->complete();
}


void TrCall::
terminate(
    int reason = H225_ReleaseCompleteReason::e_undefinedReason)
{
    assert(this);
    const string    	fn("TrCall::terminate");
    VLog    	    	log(fn);

    getH323Call()->terminate(reason);
}


void TrCall::
close()
{
    assert(this);
    const string    	fn("TrCall::close");
    VLog    	    	log(fn);

    getH323Call()->close();
}


void TrCall::
setH245Patch() throw()
{
    h245Patch_ = true;
}


bool TrCall::
needH245Patch() const throw()
{
    return h245Patch_;
}


void TrCall::
setReason(int reason) throw()
{
    callsig_.setReason(reason);
}


void TrCall::
startTimer(Sptr<mte::EvtTimer<router::TimerEvent> > timer)
{
    h323Call_.startTimer(timer);
}


void TrCall::
stopTimer(Sptr<mte::EvtTimer<router::TimerEvent> > timer)
{
    h323Call_.stopTimer(timer);
}


#ifndef DISABLE_WRITER

ostream& TrCall::
writeTo(ostream& out) const
{
    out << "TrCall: " << this << "\n"
        << "{\n"
        << "    " << sipConnection_ << "\n"
        << "    " << sdpConnection_ << "\n"
        << "    " << rasConnection_ << "\n"
        << "    " << callsig_ << "\n"
        << "}" << endl
        << h323Call_;
    return out;
}

#endif
