/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const tTrMediaS2H_cxx_Version = 
    "$Id: tTrMediaS2H.cxx,v 1.5 2000/12/18 23:48:26 bko Exp $";


#include <assert.h>
#include <iostream>

#include "VLog.hxx"

#include "TrCall.hxx"
#include "TrConnectionSDP.hxx"
#include "RouterCallCtl.hxx"
#include "LCI.hxx"
#include "EvtGotSDP.hxx"
#include "EvtTrCCOpen.hxx"
#include "EvtTrTCSAck.hxx"
#include "EvtTrOLC.hxx"

using namespace translator;
using Vocal::Logging::VLog;


// TEMP CODE: 	This is a temporary work around to avoid calling
//          	gethostbyname in CapabilityMap.
//
string	    SipToH323CallSigGW_HostAddress;
//
// TEMP CODE END

int main()
{
    VLog::init(LOG_DEBUG);
    VLog::on(LOG_DEBUG);
    VLog::on(LOG_WARNING);
    
    router::RouterCallCtl rRouter;
    TrCall call;
    TrConnectionSDP connection(&call);
    router::LCI lci(call.getH323Call(), 1);
    call.getH323Call()->getRtLCI()->add(*(lci.getLCIConnection()));

    EvtGotSDP gotsdp(&connection);
    EvtTrTCSAck tcsack(&rRouter, &call);
    EvtTrOLC olc(&rRouter, &call);
    EvtTrCCOpen cco(&call);

    router::BH245TCSAck msg;
    msg.m().m_sequenceNumber = 1;
    tcsack.setMessage(msg);

    router::BH245OLC msg2;
    msg2.m().m_forwardLogicalChannelNumber = 1;
    olc.setMessage(msg2);
    
    tcsack.setConnection(call.getH323Call()->getCallCtlConnection());
    olc.setConnection(call.getH323Call()->getCallCtlConnection());

    cerr << "Initial state:" << endl;
    cerr << call;
    
    gotsdp.start();
    cerr << "State after SDP:" << endl;
    cerr << call;

    cco.start();
    cerr << "State after CC open:" << endl;
    cerr << call;

    tcsack.start();
    cerr << "State after TCS Ack:" << endl;
    cerr << call;

    olc.start();
    cerr << "State after OLC:" << endl;
    cerr << call;

    return 0;
}
