/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const CallTrace_cxx_Version = 
    "$Id: CallTrace.cxx,v 1.5 2001/03/09 06:55:29 icahoon Exp $";


#include "CallTrace.hxx"
#include "TransportAddress.hxx"


using namespace Vocal::H323Util;


calltrace_dir::calltrace_dir(calltrace_dir::dir d)
    :	dir_(d)
{
}


ostream & 
calltrace_dir::writeTo(ostream & out) const
{
    out << "CALLTRACE, Direction: ";
    switch ( dir_ )
    {
    	case IN:
	{
	    out << "Inbound,";
	    break;
	}
	case OUT:
	{
	    out << "Outbound,";
	    break;
	}
	default:
	{
	    out << "UNKNOWN,";
	}
    }
    return ( out );
}


calltrace_addr::calltrace_addr(const TransportAddress & addr)
    :	addr_(&addr)
{
}


calltrace_addr::calltrace_addr(const TransportAddress * addr)
    :	addr_(addr)
{
}


calltrace_addr::calltrace_addr(Sptr<TransportAddress> addr)
    :	addr_(addr.operator->())
{
}

ostream & 
calltrace_addr::writeTo(ostream & out) const
{
    out << " Address: ";
    
    if ( addr_ )
    {
    	out << *addr_;
    }
    else
    { 
    	out << "UNKNOWN";
    }
    return ( out << ',' );
}
	

calltrace_prot::calltrace_prot(prot p)
    :	prot_(p)
{
}


ostream & 
calltrace_prot::writeTo(ostream & out) const
{
    out << " Protocol: ";
    switch ( prot_ )
    {
    	case PROT_SIP:
	{
	    out << "SIP,";
	    break;
	}
	case PROT_RAS:
	{
	    out << "RAS,";
	    break;
	}
    	case PROT_Q931:
	{
	    out << "Q931,";
	    break;
	}
	case PROT_H245:
	{
	    out << "H245,";
	    break;
	}
	case PROT_H245_TUNNEL:
	{
	    out << "H245_Tunnel,";
	    break;
	}
    	case PROT_TCP:
	{
	    out << "TCP, ";
	    break;
	}
	case PROT_RAS_TIMER_EXPIRY:
	{
	    out << "RAS Timer Expiry, ";
	    break;
	}
	case PROT_Q931_TIMER_EXPIRY:
	{
	    out << "Q931 Timer Expiry, ";
	    break;
	}
	case PROT_H245_TIMER_EXPIRY:
	{
	    out << "H245 Timer Expiry, ";
	    break;
	}
	default:
	{
	    out << "UNKNOWN,";
	}
    }
    return ( out );
}	

calltrace_msgtype::calltrace_msgtype(const string & type)
    :	msgtype_(type)
{
}

ostream & 
calltrace_msgtype::writeTo(ostream & out) const
{
    return ( out << " Message: " << msgtype_ );
}
	
