#ifndef INCLUDED_Router_hxx
#define INCLUDED_Router_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const Router_hxx_Version = 
"$Id: Router.hxx,v 1.9 2001/10/18 04:33:54 icahoon Exp $";

#include <map>

#ifndef DISABLE_WRITER
#include "Writer.hxx"
#endif

#include "Connection.hxx"
#include "VLog.hxx"

using Vocal::Logging::VLog;

namespace mte
{

    /// Router template
    template<class ID, class SM, class Src>
    class Router:protected std::map<ID, Connection<ID, SM, Src>*>
#ifndef DISABLE_WRITER
        , public Vocal::IO::Writer
#endif
    {
    public:
        typedef Connection<ID, SM, Src> Conn;
        typedef Conn::EvtRouterType EvtRouterType;
        typedef ID IDType;
        typedef SM SMType;
        typedef Src SrcType;
        
        Router();
        
        virtual ~Router() {};

        /// Route event to connection
        virtual bool route(EvtRouterType& evtRouter, Src& evtSource);

        /// Add connection
        virtual bool add(Conn& conn);

        /// Remove connection
        virtual bool remove(const ID& id);

        /// Search connection
        virtual Conn* search(const ID& id);
    };
    
    //----------------------------
    // Inline and template methods
    //----------------------------

    template<class ID, class SM, class Src>
    Router<ID, SM, Src>::
    Router()
    {
    }

    template<class ID, class SM, class Src>
    bool Router<ID, SM, Src>::
    route(EvtRouterType& evtRouter, Src& evtSource)
    {
        const string    fn("Router::route");
        VLog            log(fn);

        bool rc = false;
                
        // Src& will be implicitly converted to ID&
        Conn* conn = search(evtSource);

        if (conn)
        {
            VVERBOSE(log)   << fn << ": routing to connection id (" << conn 
                            << ")" << VVERBOSE_END(log);
                            
            conn->route(evtRouter, evtSource);
            rc = true;
        }
        else
        {
            rc = false;

            VVERBOSE(log)   << fn << ": no connection to route to" 
                            << VVERBOSE_END(log);
        }
        
                        
        return ( rc );
    }

    template<class ID, class SM, class Src>
    bool Router<ID, SM, Src>::
    add(Conn& conn)
    {
        const string    fn("Router::add");
        VLog            log(fn);
        
        bool    rc = false;
        
        if (!search(conn) && insert(pair<ID, Conn*>(conn, &conn)).second)
        {
            rc = true;
        }
        else
        {
            rc = false;
        }
        
        VVERBOSE(log)   << fn << ": adding connection id (" << &conn 
                        << "), " << ( rc == true ? "success" : "failed" )
                        << VVERBOSE_END(log);
                        
        return ( rc );
    }

    template<class ID, class SM, class Src>
    bool Router<ID, SM, Src>::
    remove(const ID& id)
    {
        const string    fn("Router::remove");
        VLog            log(fn);
        
        bool    rc = false;
        
        if ( erase(id) )
        {
            rc = true;
        }
        else
        {
            rc = false;
        }
        
        VVERBOSE(log)   << fn << ": removing connection id (" << &id << "), "
                        << ( rc == true ? "success" : "failed" )
                        << VVERBOSE_END(log);
                        
        return ( rc );
    }

    template<class ID, class SM, class Src>
    Router<ID, SM, Src>::Conn* Router<ID, SM, Src>::
    search(const ID& id)
    {
        const string    fn("Router::search");
        VLog            log(fn);

        std::map<ID, Conn*>::iterator it = find(id);
        
        if (it == end())
        {
            VVERBOSE(log)   << fn << ": searching for connection id (" << &id 
                            << "), failed" << VVERBOSE_END(log);
            return 0;
        }
        else
        {
            VVERBOSE(log)   << fn << ": searching for connection id (" << &id 
                            << "), success" << VVERBOSE_END(log);

            return it->second;
        }
    }

}

#endif
