
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CAServer_cxx_Version = "$Id: CAServer.cxx,v 1.18 2002/11/12 20:25:29 veer Exp $";

#include "cpLog.h"
#include "CAServer.hxx"
#include "Data.hxx"
#include "SnmpCommon.h"
#include "endpoint.hxx"
#include "potStateEngine.hxx"
#include "callProcessing.hxx"
#include "sipProxy.hxx"
#include "HeartbeatUtil.h"
#include "StartupSequence.hxx"
#include "CAServerContainer.hxx"

/*
 * Default Constructor
 * @param id callAgent ip address
 */
CAServer::CAServer(Data id)
        : Server(SERVER_CA),
          selfId(id)
{}

/*
 * Copy Constructor
 * @param rhs Element to be copied
 */
CAServer::CAServer(const CAServer& rhs)
        : Server(rhs)
{
    if (this != &rhs)
        copyRhsToThis(rhs);
}

/*
 * Assignment Operator
 * @param rhs Element contains data to copy
 * @return This element
 */
CAServer&
CAServer::operator=(const CAServer& rhs)
{
    if (this != &rhs)
    {
        Server::operator=(rhs);
        copyRhsToThis(rhs);
    }
    return *this;
}

/*
 * Copy data from an element into self
 * @param rhs Element contains data to copy
 */
void
CAServer::copyRhsToThis(const CAServer& rhs)
{
    selfId = rhs.selfId; 
}

/*
 * Destructor
 */
CAServer::~CAServer()
{}

/*
 * Increment heartbeat count and change the status accordingly
 * @param key ip address
 */
void
CAServer::processHeartbeat(const Data& key)
{
    incrHeartbeatCountByOne();

    if (itsStatus == Inactive)
        gotFirstHeartbeat(key);
    else
        if (itsStatus == ActivePending )
            gotHiccup();

}

/*
 * Set active status and detect if it is the primary callAgent to
 * start the startup sequence.
 * @param key ip address
 */
void
CAServer::gotFirstHeartbeat(const Data& key)
{
    setStatus(Active);

    cpLog( LOG_ALERT, "\n");
    cpLog( LOG_ALERT, "** CallAgent Up: %s **\n", key.logData() );

    Data caPrime = ca.sipMyHost;
    caPrime += ":";
    caPrime += Data(ca.sipMyPort);
    if (caPrime == key)
    {
        cpLog (LOG_DEBUG, "Receive self 1st heartbeat");
        // tell the container that it receive 1st hb.
        CAServerContainer::instance().startStartupSequence(true);
    }

}

/*
 * Detect other callAgent heartbeat up or down.
 * @param agentId the ip address
 */
void
CAServer::doHouseKeeping(const Data& agentId)
{
    Data caPrime = ca.sipMyHost;
    caPrime += ":";
    caPrime += Data(ca.sipMyPort);

    // self (prime) doesn't need to do housekeeping
    if (caPrime == agentId)
        return ;

    if ( itsStatus != Inactive )
        if ( heartbeatCount == 0 )
        {
            incrNumOfHeartbeatMissed();

            // *** keep this code around in case we decide
            // *** to reinstate the ActivePending state.
            //if ( itsStatus == Active)
            //   setStatus(ActivePending);

            if ( serverIsDead() )
            {
                setStatus(Inactive);

                cpLog (LOG_ALERT, "\n");
                cpLog (LOG_ALERT, "** CallAgent Down: %s **\n", agentId.logData() );

                // need to call startup sequence since the other is just going down...
                CAServerContainer::instance().startStartupSequence(false);

                // clear HB counters
                clearHeartbeatCount();
                clearNumOfHeartbeatMissed();
            }

        }
        else
        {
            // clear HB counters
            clearHeartbeatCount();
            clearNumOfHeartbeatMissed();
        }

}
