
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const callProcessing_cxx_Version = "$Id: callProcessing.cxx,v 1.50 2002/02/25 05:57:18 sprajpat Exp $";



#include "global.h"

#include <stdio.h>
#include "callProcessing.hxx"
#include "cpLog.h"
#include "MgcpResponseCode.hxx"
#include "potStateEngine.hxx"
#include "sipProxy.hxx"


///
int
newRequestIdentifier (void)
{
    static pthread_mutex_t requestIdMutex = PTHREAD_MUTEX_INITIALIZER;
    static unsigned long currentNumber = 1;

    int error = pthread_mutex_lock( &requestIdMutex );
    unsigned long x = ++currentNumber;
    error = pthread_mutex_unlock( &requestIdMutex );
    return x;
}    /* newRequestIdentifier */


int
newCallIdentifier (void)
{
    static pthread_mutex_t callIdMutex = PTHREAD_MUTEX_INITIALIZER;
    static unsigned long currentNumber = 1;

    int error = pthread_mutex_lock( &callIdMutex );
    unsigned long x = ++currentNumber;
    error = pthread_mutex_unlock( &callIdMutex );
    return x;
}    /* newCallIdentifier */


///
void
cpStateChange (Endpoint* ep, int newState)
{
    if ( ep == 0 )
    {
        cpLog( LOG_DEBUG, "The ep is a NULL pointer !!");
        assert( ep );
        return ;
    }
    cpLog (LOG_DEBUG,
           "%s cpStateChange: %s -> %s",
           ep->epId.logData(),
           CallStateString[ep->state],
           CallStateString[newState]);

    ep->state = newState;
}    /* cpStateChange */


///Call the state Engine Function
void
cpProcessEvent (Endpoint* ep, int event)
{
    if (!ep)
    {
        cpLog (LOG_ERR, "cpProcessEvent(): ep is NULL");
        return ;
    }

    cpLog (LOG_DEBUG, "%s process event %s in state %s",
           ep->epId.logData(),
           CallEventString[event],
           CallStateString[ep->state]);

    void* ptr = (*((*(ep->stateEngine)) [ep->state] [event])) (static_cast < void* > (ep));
    if (ptr != 0)
    {
        cpLog (LOG_ERR, "cpProcessEvent: Endpoint(%s) State(%s) Event(%s)",
               (static_cast < Endpoint* > (ptr))->epId.logData(),
               CallStateString[ep->state],
               CallEventString[event]);
        cpLog (LOG_ERR, "              : requestIdentifier(%s) callDigits(%s) dialedDigits(%s)",
               ep->requestIdentifier,
               ((ep->call) ? ep->call->callDigits : ""),
               ((ep->call) ? ep->call->dialedDigits : ""));
    }
}    /* cpProcessEvent */


///Call the State Engine Function
void
cpProcessEvent (CpEventData* ed, int event)
{
    if (!ed)
    {
        cpLog (LOG_ERR, "cpProcessEvent(): ed is NULL");
        return ;
    }

    if (!ed->ep)
    {
        cpLog (LOG_ERR, "cpProcessEvent(): ed->ep is NULL");
        return ;
    }

    cpLog (LOG_DEBUG, "%s process event %s in state %s",
           ed->ep->epId.logData(),
           CallEventString[event],
           CallStateString[ed->ep->state]);

    void* ptr = (*((*(ed->ep->stateEngine)) [ed->ep->state] [event])) (static_cast < void* > (ed));
    if (ptr)
    {
        cpLog (LOG_DEBUG, "CP Error: Endpoint(%s) State(%s) Event(%s)",
               ed->ep->epId.logData(),
               CallEventString[event],
               CallStateString[ed->ep->state]);
        cpLog (LOG_ERR, "              : requestIdentifier(%s) callDigits(%s) dialedDigits(%s)",
               ed->ep->requestIdentifier,
               ((ed->ep->call) ? ed->ep->call->callDigits : ""),
               ((ed->ep->call) ? ed->ep->call->dialedDigits : ""));
    }
}    /* cpProcessEvent */


///Initialize the CallData
void
callDataInit (CallData* cd)
{
    if ( cd == 0 )
    {
        cpLog( LOG_DEBUG, "The cd is a NULL pointer !!");
        assert( cd != 0 );
        return ;
    }

    for (int j = 0; j < ENDPOINTS_PER_CALL_MAX; j++)
    {
        cd->endpoint[j].id = -1;
        cd->endpoint[j].connectionId[0] = 0;
        // odd way to initialize Sdp field
        SdpSession sdp;
        cd->endpoint[j].localConnection = sdp;
        /** re-initialize localConnection */
    }
    cd->dialedDigits[0] = 0;
    cd->callDigits[0] = 0;
    cd->dtmfDigits[0] = 0;
    cd->playDtmfDigit = 0;
    cd->forwardNoAnswerId = 0;
    cd->callIdentifier[0] = 0;
    cd->blockCallWaiting = false;
    // "usually" resides in the heldCall only
    cd->callWaiting = false;  // true only while the "call waiting" call is ringing
    cd->callWaited = false;   // true even after the "call waiting" call is answered & swapped
    cd->ackReceived = false;
    cd->lastInfoCseq = 0;

    int error = pthread_mutex_lock( &callDataMutex );
    if (error)
    {
        cpLog (LOG_ERR, "Cannot lock call data list");
    }
    else
    {
        callDataList.push_back (cd);

        error = pthread_mutex_unlock( &callDataMutex );
        if (error)
        {
            cpLog (LOG_ERR, "Cannot unlock call data list");
        }
    }
}    /* callDataInit */


///
CallData*
newCallData (Endpoint* ep)
{
    cpLog (LOG_DEBUG, "newCallData");

    CallData* rc = 0;

    int error = pthread_mutex_lock( &callDataMutex );
    if (error)
    {
        cpLog (LOG_ERR, "Cannot lock call data list");
    }
    else
    {
        list < CallData* > ::iterator p = callDataList.begin();
        if (p != callDataList.end())
        {
            if ( (*p) == 0 )
            {
                cpLog( LOG_DEBUG, "The iterator p points to a NULL pointer !!");
                assert( (*p) != 0 );
                return ( 0 );
            }

            if ( ep == 0 )
            {
                cpLog( LOG_DEBUG, "The ep is a NULL pointer !!");
                assert( ep != 0 );
                return ( 0 );
            }
            
            (*p)->endpoint[0].id = ep->id;
            rc = *p;
            callDataList.pop_front();
        }
        else
        {
            cpLog (LOG_ERR, "No call data!");
        }
        error = pthread_mutex_unlock( &callDataMutex );
        if (error)
        {
            cpLog (LOG_ERR, "Cannot unlock call data list");
        }
    }
    return rc;
}    /* newCallData */


///
int
fakeAddressTranslation (const char* digits)
{

    map < string, Endpoint* > ::iterator p = dnMap.find(digits);
    if (p == dnMap.end())
    {
        cpLog (LOG_ERR, "Invalid DN %s", digits);
        return ENDPOINT_MAX;
    }

    if ( p->second == 0 )
    {
        cpLog( LOG_DEBUG, "The iterator p points to a NULL pointer !!");
        assert( p->second != 0 );
        return ( ENDPOINT_MAX );
    }
    return p->second->id;
}    // fakeAddressTranslation


///
void
giveBusyTone (Endpoint* ep)
{
    if ( ep == 0 )
    {
        cpLog( LOG_DEBUG, "The ep is a NULL pointer !!");
        assert( ep != 0 );
        return ;
    }
    cpLog (LOG_INFO, "Give Busy Tone to %s (%s)",
           ep->epId.logData(),
           CallStateString [ep->state]);

    sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
    MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
    cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());

    /** Code corresponding requested events */
    addRequestOrSignalEvent( (MgcCommand*)&rqnt, LinePackageRequestOnHookTransition );
    addRequestOrSignalEvent( (MgcCommand*)&rqnt, LinePackageRequestFlashHook );

    /** Code corresponding signal */
    addRequestOrSignalEvent( &rqnt, LinePackageSignalBusyToneOn );

    if ( ep->gw == 0 )
    {
        cpLog( LOG_DEBUG, "The ep->gw is a NULL pointer !!");
        assert( ep->gw != 0 );
        return ;
    }

    if ( ep->gw->asyncStack == 0 )
    {
        cpLog( LOG_DEBUG, "The ep->gw->asyncStack is a NULL pointer !!");
        assert( ep->gw->asyncStack != 0 );
        return ;
    }

    ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);
}    // giveBusyTone


/// This is the main procedure for making a all, similar to DIGPROC
/// It needs to work with all applicable endpoint types
void
makeCall (Endpoint* ep)
{
    cpStateChange (ep, CallStateDigitsCollected);

    // - the rest of the code below can be moved to where makecall
    // is being called from

    if ( ep == 0 )
    {
        cpLog( LOG_DEBUG, "The ep is a NULL pointer !!");
        assert( ep != 0 );
        return ;
    }

    if (ep->heldCall != 0)
    {
        // this will call connectionReady for further processing
        if ( !ep->call )
        {
            cpLog( LOG_ERR, "The endpoint does not have an active call.");
            return ;
        }
        ep->call->endpoint[0].id = ep->heldCall->endpoint[0].id;
        strcpy( ep->call->callIdentifier, ep->heldCall->callIdentifier );
        ep->call->callIdentifier[CALL_ID_LEN_MAX - 1] = 0;
        strcpy( ep->call->endpoint[0].connectionId, ep->heldCall->endpoint[0].connectionId );
        ep->call->endpoint[0].connectionId[CONNECTION_ID_LEN_MAX - 1] = 0;

        ep->call->endpoint[0].localConnection = ep->heldCall->endpoint[0].localConnection;

        cpProcessEvent(ep, CallEventResponse);
        return ;
    }

    MgcCreateConnection crcx (ep->epId.convertString(), ep->call->callIdentifier, "sendrecv");

    MgcpConnectionOptions co;
    if (ep->echoCancellation == false)
    {
        co.setEchoCancellation (false);
    }
    cpLog (LOG_DEBUG, "Echo Cancellation %s",
           co.getEchoCancellation() ? "true" : "false");

    crcx.setLocalConnectionOptions (co);

    if ( ep->gw == 0 )
    {
        cpLog( LOG_DEBUG, "The ep->gw is a NULL pointer !");
        assert( ep->gw != 0 );
        return ;
    }

    if ( ep->gw->asyncStack == 0 )
    {
        cpLog( LOG_DEBUG, "The ep->gw->asyncStack is a NULL pointer !");
        assert( ep->gw->asyncStack != 0 );
        return ;
    }
    ep->gw->asyncStack->asyncSend (crcx, ep->gw->handle);

    cpLog(LOG_DEBUG, "CRCX FOR ENDPOINT %s\n", ep->epId.logData());
} // makeCall


// If we want to not allow our call agent to do CFNA locally ...
// - remove from here ...
// This needs to work with all applicable endpoint types
int
startForwardNoAnswerTimer (CallData* call)
{
    assert(0);
    cpLog (LOG_DEBUG, "startForwardNoAnswerTimer");
    // 22 second delay before CFNA, to match 5 hardware ringing time out
    // 13.2 second delay before CFNA, to match 3 hardware ringing time out
    QueueEventData* ed = newQueueEventData ();
    assert( ed );
    ed->type = TimeoutEvent;
    ed->callData = call;
    assert( call );
    ed->ep = &eps[call->endpoint[0].id];
    assert( ed->ep );
    assert( ed->ep->iq );
    call->forwardNoAnswerId = ed->ep->iq->addDelayMs(ed, 13200);
    cpLog (LOG_DEBUG, "TIMER started at endpoint id %d for endpoint id = %d",
           call->endpoint[0].id, call->endpoint[1].id);
    return 1;
}    /* startForwardNoAnswerTimer */
// - up to here.


///
void
cpInternalEvent (Endpoint* ep,
                 CallEvent event,
                 char* callId,
                 int featureId)
{
    if ( ep == 0 )
    {
        cpLog( LOG_DEBUG, "The ep is a NULL pointer !!");
        assert( ep != 0 );
        return ;
    }

    cpLog (LOG_DEBUG, "endpoint %s cpInternalEvent %s",
           ep->epId.logData(), CallEventString [event]);

    if (featureId == -1 && callId[0] == 0)
    {
        cpProcessEvent (ep, event);
    }
    else
    {
        CpEventData* ed = newCpEventData (ep->gw->edPool);
        if ( ed == 0 )
        {
            cpLog( LOG_DEBUG, "Ran out of CpEventData in the pool !!");
            assert( ed != 0 );
            return ;
        }

        ed->ep = ep;
        strcpy (ed->callIdentifier, callId);
        ed->featureId = featureId;
        cpProcessEvent (ed, event);
    }
}    // cpInternalEvent


///
void
cpTimeoutEvent (QueueEventData* event)
{
    cpLog (LOG_DEBUG, "cpTimeoutEvent");

    if ( event == 0 )
    {
        cpLog( LOG_DEBUG, "The event is a NULL pointer !!");
        assert( event != 0 );
        return ;
    }

    if (event->featureId == FeatureEventSipRegister)
        sipEventSend (event->ep, SipEventSendRegister);
}   // cpTimeoutEvent



/// Call Processing event data pool
list < CpEventData* > eds [CP_THREAD_MAX];


///
void
cpEventDataInit (int cpThread)
{
    for (int i = 0; i < CP_EVENT_DATA_MAX; i++)
    {
        CpEventData * cpEvtData = new CpEventData;
        if ( cpEvtData == 0 )
        {
            cpLog( LOG_DEBUG, "Ran out of CpEventData in the pool !!");
            assert( cpEvtData != 0 );
            return ;
        }

        eds [cpThread].push_back( cpEvtData );
        //  eds [cpThread].push_back (new CpEventData);
    }

    cpLog (LOG_DEBUG, "CP thread %d allocated %d cp event data",
           cpThread,
           CP_EVENT_DATA_MAX);
}    // cpEventDataInit


///
CpEventData* newCpEventData (list < CpEventData* > * edPool)
{
    CpEventData* rc = 0;

    if ( edPool == 0 )
    {
        cpLog( LOG_DEBUG, "The edPool is a NULL pointer !!");
        assert( edPool != 0 );
        return ( 0 );
    }
    list < CpEventData* > ::iterator p = edPool->begin();
    if (p == edPool->end())
    {
        cpLog (LOG_ERR, "Cannot allocate cp event data");
    }
    else
    {
        if ( (*p) == 0 )
        {
            cpLog( LOG_DEBUG, "The iterator p points to a NULL pointer !!");
            assert( (*p) != 0 );
            return ( 0 );
        }

        cpLog (LOG_DEBUG, "Allocate cp event data");
        (*p)->callIdentifier[0] = 0;
        (*p)->featureId = -1;
        (*p)->ep = 0;
        rc = *p;
        edPool->pop_front ();
    }
    return rc;
}    // newCpEventData


///
void freeCpEventData (list < CpEventData* > * edPool, CpEventData* ed)
{
    if ( edPool == 0 )
    {
        cpLog( LOG_DEBUG, "The edPool is a NULL pointer !!");
        assert( edPool != 0 );
        return ;
    }

    cpLog (LOG_DEBUG, "Free cp event data");
    edPool->push_back (ed);
}    // freeCpEventData


///
void*
processEvent (void* ptr)
{

    Fifo < QueueEventData* > * iq = static_cast < Fifo < QueueEventData* > * > (ptr);
    if ( iq == 0 )
    {
        cpLog( LOG_DEBUG, "The iq is a NULL pointer !!");
        assert( iq != 0 );
        return ( 0 );
    }

    int i = 0;
    while ((i < INPUT_Q_MAX) && (iq != &iqs[i++]));
    cpLog (LOG_INFO, "CP thread %d started (pid %d)",
           i - 1,
           getpid());

    cpEventDataInit (i - 1);

    QueueEventData* event;

    while ((event = iq->getNext ()) != 0)
    {
        assert(!((event < &qEventData[0]) || (event > &qEventData[Q_EVENT_DATA_MAX - 1])));

        switch (event->type)
        {
            case StackEvent:
            {
                cpStackEvent (event->ep, event->cmd);
            }
            break;
            case InternalEvent:
            {
                cpInternalEvent (event->ep,
                                 event->internalEvent,
                                 event->callIdentifier,
                                 event->featureId);
            }
            break;
            case TimeoutEvent:
            {
                cpTimeoutEvent (event);
            }
            break;
            case SipStackEvent:
            {
                sipStackEvent (event->SIPcmd);
            }
            break;
            default:
            {
                cpLog (LOG_ERR, "Unknown event: type = %d, address = %d",
                       event->type,
                       (int)event);
            }
        }    /* switch */
        freeQueueEventData (event);
    }
    cpLog (LOG_ERR, "Call Processing thread terminated");
    return 0;
}  // processEvent


///
void
callProcessingStart (void)
{
    int errNum;
    pthread_t tid;

    int i;
    // start call processing thread(s) according to configuration
    for (i = 0; i < ca.threadCount; i++)
    {
        if ((errNum = pthread_create (&tid, NULL, processEvent, &iqs[i])) > 0)
        {
            cpLog (LOG_ERR, "Cannot create thread: %s", strerror (errNum));
        }
    }
    sleep (1);     // Make sure the call processing threads
    // are started and waiting for messages
}    // callProcessingStart



///
void
sendInternalEvent (Endpoint* ep,
                   CallEvent event,
                   char* callId,
                   int featureId)
{
    if ( ep == 0 )
    {
        cpLog( LOG_DEBUG, "The ep is a NULL pointer !!");
        assert( ep != 0 );
        return ;
    }
    cpLog (LOG_DEBUG, "send internal event to %s", ep->epId.logData());
    QueueEventData* ed = newQueueEventData ();
    if ( ed == 0 )
    {
        cpLog( LOG_DEBUG, "Ran out of QueueEventData in the pool !!");
        assert( ed != 0 );
        return ;
    }
    ed->type = InternalEvent;
    ed->ep = ep;
    ed->internalEvent = event;
    if (callId != 0)
    {
        strcpy (ed->callIdentifier, callId);
    }
    ed->featureId = featureId;

    if ( ep->iq == 0 )
    {
        cpLog( LOG_DEBUG, "The ep->iq is a NULL pointer !!");
        assert( ep->iq != 0 );
        return ;
    }
    (void) ep->iq->add (ed);
}    // sendInternalEvent


#define BUF_SIZE 31
