/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const LSRouteInterface_cxx_Version = 
"$Id: LSRouteInterface.cxx,v 1.6 2001/06/29 21:02:44 bko Exp $";


#include <unistd.h>
#include <stdlib.h> 
#include <strstream.h>
#include "LSRouteInterface.hxx"
#include "Tcp_ClientSocket.hxx"
#include "VCondition.h"
#include "NetworkAddress.h"
#include "VNetworkException.hxx"
#include "TRIPOpenMsg.hxx"
#include "TRIPUpdateMsg.hxx"
#include "TRIPNotificationMsg.hxx"
#include "TRIPTransceiver.hxx"
#include "Data.hxx"
#include "cpLog.h"

LSRouteInterface* LSRouteInterface::_instance = 0;

LSRouteInterface&
LSRouteInterface::instance()
{
    if (!_instance)
    {
        _instance = new LSRouteInterface();
        cpLog(LOG_INFO, "Started LSRouteInterface");
    }
    return (*_instance);
}

void 
LSRouteInterface::destroy(void)
{
    delete LSRouteInterface::_instance;
    LSRouteInterface::_instance = 0;
}

LSRouteInterface::LSRouteInterface()
{
    sessionInitiated = false;
    ///Get the list of LSs in the system
    _listOfLS.push_back("kevinmc-lnx:8000");
    _listOfLS.push_back("vvs-sailing:8000"); 
#ifdef __linux__
    if (atexit(LSRouteInterface::destroy) < 0)
    {
        cpLog(LOG_ALERT, "Failed to register with atexit()");
    };
#endif
    _connection = new Connection();
    _thread = new VThread();
    _thread->spawn(LSRouteInterface::run, this);
}

LSRouteInterface::~LSRouteInterface()
{
    
}

void*
LSRouteInterface::run(void* args)
{
    LSRouteInterface* self = static_cast<LSRouteInterface*>(args);
    self->processMsg();
    return(0);
}

void
LSRouteInterface::processMsg()
{
    //Monitor the connection for incoming msgs from the
    //LSApiServer - usually Update msgs indicating changes
    //in the Routing data
    for (; ; )
    {
        try
        {
            if (!sessionInitiated)
            {
                vusleep(100000);
                continue;
            }
            if((_connection->isLive()) && (_connection->isReadReady(1)))
            {
                TRIPTransceiver trans;

                Sptr<TRIPMsg> msg = trans.receiveMsg(_connection->getConnId());
                assert(msg != 0);
                if (msg->getType() == TRIP_OP_UPDATE)
                {
                    processUpdateMsg(msg);
                }
                if (msg->getType() == TRIP_OP_NOTIF)
                {
                    _connection->close();
                }
            }
            else if(!(_connection->isLive()))
            {
                if (isConnected())
                {
                    sendOpenMsg();
                }
            }

        }
        catch(VNetworkException& e)
        {
            cpLog(LOG_ALERT, "Network error, reason:%s",
                                     e.getDescription().c_str());
            if(_connection->isLive())
            {
                _connection->close();
            }
        }
    }
}

void
LSRouteInterface::processUpdateMsg(Sptr<TRIPMsg> msg)
{
    cpLog(LOG_DEBUG, "LSRouteInterface::processUpdateMsg");
    if(msg == 0)
    {
        assert(0);
        return;
    }
    Sptr<TRIPUpdateMsg> update;
    update.dynamicCast(msg);
    assert(update != 0);
    if (update.getPtr())
    {
        //First take care of withdrawn routes
        const Sptr<TRIPWithdrawnRoute>& wRoute = update->getWithdrawnRoute();
        const TRIPRouteContainer::RouteList& rList = wRoute->getRouteList();
        for(TRIPRouteContainer::RouteList::const_iterator itr = rList.begin();
                                                    itr != rList.end(); itr++)
        {
            //removeDest((*itr));
        }
        const Sptr<TRIPReachableRoute>& rRoute = update->getReachableRoute();
        Sptr<TRIPRouteAttr> rAttr = update->getAttribute(TRIP_NEXT_HOP_SERVER);
        Sptr<TRIPNextHopServer> nHopAttr;
        nHopAttr.dynamicCast(rAttr);
        assert(nHopAttr != 0);
        Data nHop = nHopAttr->getHopAddress().str();
        const TRIPRouteContainer::RouteList& rList2 = rRoute->getRouteList();
        for(TRIPRouteContainer::RouteList::const_iterator itr = rList2.begin();
                                                    itr != rList2.end(); itr++)
        {
            //insertDest((*itr));
            //comment
        }
    }
}

bool
LSRouteInterface::isConnected()
{
    bool connected = false;
    if(_connection->isLive())
    {
        connected =true;
    }
    else
    {
        for(ListOfLS::iterator itr = _listOfLS.begin();
                  itr != _listOfLS.end(); itr++)
        {
            try
            {
                NetworkAddress na((*itr));
                TcpClientSocket cl(na, false);
                Connection conn;
                cl.connect();
                _connection = new Connection(cl.getConn());
                cpLog(LOG_INFO, "LSRouteInterface connected to (%s)", (*itr).c_str());
                connected = true;
                break;
            }
            catch(VNetworkException& e)
            {
                cpLog(LOG_INFO, "Failed to connect to server (%s), trying next..", (*itr).c_str());
            }
        }
    }
    return connected;
}

void
LSRouteInterface::initiateSession()
{
    _connection = new Connection();
    createOpenMsg();
    sessionInitiated = true;
}

void
LSRouteInterface::sendOpenMsg()
{
    if (_connection->isLive())
    {
        assert(itsOpenMsg != 0);
        cpLog(LOG_DEBUG, "Sending OPEN Msg ");
        itsOpenMsg->writeData(_connection->getConnId());
    }
}

void
LSRouteInterface::endSession()
{
    assert(_connection->isLive());
    TRIPNotificationMsg msg(TRIP_ERR_CEASE, 0, TRIPData("Stop Updates"));
    msg.writeData(_connection->getConnId());
}

void
LSRouteInterface::createOpenMsg()
{
    itsOpenMsg = new TRIPOpenMsg(1234, 0, 4*60);
    Sptr<TRIPCapabilityInfo>  cInfo = new TRIPCapabilityInfo();
    Sptr<TRIPRouteCapability> rCap = new TRIPRouteCapability();
    Sptr<TRIPRouteHdr> rHdr1 = new TRIPRouteHdr(TRIP_ADDR_POTS, TRIP_APP_PROTO_SIP);
    Sptr<TRIPRouteHdr> rHdr2 = new TRIPRouteHdr(TRIP_ADDR_POTS, TRIP_APP_PROTO_H323_Q921);
    Sptr<TRIPRouteHdr> rHdr3 = new TRIPRouteHdr(TRIP_ADDR_POTS, TRIP_APP_PROTO_H323_RAS);
    Sptr<TRIPRouteHdr> rHdr4 = new TRIPRouteHdr(TRIP_ADDR_POTS, TRIP_APP_PROTO_H323_ANNEX_G);
    rCap->addRouteCapability(rHdr1);
    rCap->addRouteCapability(rHdr2);
//    rCap->addRouteCapability(rHdr3);
//    rCap->addRouteCapability(rHdr4);
    cInfo->addCapability(rCap);
    itsOpenMsg->addParameter(cInfo);
}

